









with Ada.Containers;                  use Ada.Containers;
with Ada.Containers.Hashed_Maps;
with Ada.Containers.Vectors;
with Ada.Directories;
with Ada.Exceptions;
with Ada.Finalization;
with Ada.Strings.Unbounded;           use Ada.Strings.Unbounded;
with Ada.Strings.Wide_Wide_Unbounded; use Ada.Strings.Wide_Wide_Unbounded;

pragma Warnings (Off, "internal");
with Ada.Strings.Wide_Wide_Unbounded.Aux;
use Ada.Strings.Wide_Wide_Unbounded.Aux;
pragma Warnings (On, "internal");

with Ada.Text_IO;                     use Ada.Text_IO;
with Ada.Unchecked_Conversion;
with Ada.Unchecked_Deallocation;
with System;

with GNAT.Task_Lock;
with GNAT.Traceback.Symbolic;

with GNATCOLL.Traces;

with Langkit_Support.Adalog.Debug;
with Langkit_Support.Generic_API.Analysis;
with Langkit_Support.Generic_API.Introspection;
with Langkit_Support.Hashes; use Langkit_Support.Hashes;
with Langkit_Support.Images; use Langkit_Support.Images;
with Langkit_Support.Names;  use Langkit_Support.Names;
with Langkit_Support.Relative_Get;

with Liblktlang.Private_Converters;
use Liblktlang.Private_Converters;

pragma Warnings (Off, "referenced");

          with Liblktlang.Default_Provider;
          with Liblktlang.Implementation.Extensions;
            use Liblktlang.Implementation.Extensions;

pragma Warnings (On, "referenced");



package body Liblktlang.Implementation is

   use Precomputed_Symbols;

   pragma Warnings (Off, "has no effect");
   use Solver;
   pragma Warnings (On, "has no effect");

   package Context_Vectors is new Ada.Containers.Vectors
     (Index_Type   => Positive,
      Element_Type => Internal_Context);

   type Contexts_Destructor is limited
      new Ada.Finalization.Limited_Controlled with null record;
   overriding procedure Finalize (CD : in out Contexts_Destructor);
   --  Helper to destroy all contexts when terminating the process

   package Context_Pool is

      procedure Acquire (Context : out Internal_Context)
         with Post => Context /= null;
      --  If a context is free for reuse, increment its serial number and
      --  return it. Otherwise, allocate a new one. In any case, this does not
      --  initialize it, except for the Serial_Number field.

      procedure Release (Context : in out Internal_Context)
         with Pre  => Context /= null,
              Post => Context = null;
      --  Tag Context as free for reuse and set it to null

      procedure Free;
      --  Free all contexts in this pool. Intended to be called only when the
      --  process is terminating, to avoid reported memory leaks.

   private

      Available : Context_Vectors.Vector;
      --  List of allocated contexts that can be re-used right now

      CD : Contexts_Destructor with Unreferenced;
      --  Singleton whose only purpose is to free all contexts in Available
      --  when finalized.

   end Context_Pool;

   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure);
   --  Common underlying implementation for Register_Destroyable_Gen

   pragma Warnings (Off, "referenced");
   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access;
   pragma Warnings (On, "referenced");

   procedure Reset_Envs_Caches (Unit : Internal_Unit);
   --  Reset the env caches of all lexical environments created for ``Unit``

   procedure Destroy (Env : in out Lexical_Env_Access);

   function Snaps_At_Start (Self : Bare_Lkt_Node) return Boolean;
   function Snaps_At_End (Self : Bare_Lkt_Node) return Boolean;

   --  Those maps are used to give unique ids to lexical envs while pretty
   --  printing them.

   package Address_To_Id_Maps is new Ada.Containers.Hashed_Maps
     (Lexical_Env, Integer, Hash, "=");

   type Dump_Lexical_Env_State is record
      Env_Ids : Address_To_Id_Maps.Map;
      --  Mapping: Lexical_Env -> Integer, used to remember which unique Ids we
      --  assigned to the lexical environments we found.

      Next_Id : Positive := 1;
      --  Id to assign to the next unknown lexical environment

      Root_Env : Lexical_Env;
      --  Lexical environment we consider a root (this is the Root_Scope from
      --  the current analysis context), or null if unknown.
   end record;
   --  Holder for the state of lexical environment dumpers

   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String;
   --  If E is known, return its unique Id from State. Otherwise, assign it a
   --  new unique Id and return it.

   procedure Print
     (Node        : Langkit_Support.Generic_API.Analysis.Lk_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "");
   --  Helper for the public overload, but working on the generic API node type

   ------------------------
   -- Precomputed_Symbol --
   ------------------------

   pragma Warnings (Off, "referenced");
   function Precomputed_Symbol
     (Index : Precomputed_Symbol_Index) return Text_Type is
   pragma Warnings (On, "referenced");
   begin
         declare
            Raw_Text : constant Text_Type := (case Index is
            
               when Precomputed_Sym_Abstract => "abstract",
               when Precomputed_Sym_Analysisunit => "AnalysisUnit",
               when Precomputed_Sym_Array => "Array",
               when Precomputed_Sym_As => "as",
               when Precomputed_Sym_Astlist => "ASTList",
               when Precomputed_Sym_Bigint => "BigInt",
               when Precomputed_Sym_Bool => "Bool",
               when Precomputed_Sym_Builtin => "builtin",
               when Precomputed_Sym_Call => "__call__",
               when Precomputed_Sym_Char => "Char",
               when Precomputed_Sym_Dedent => "dedent",
               when Precomputed_Sym_Dont_Skip => "dont_skip",
               when Precomputed_Sym_Env_Spec => "env_spec",
               when Precomputed_Sym_Errornode => "ErrorNode",
               when Precomputed_Sym_External => "external",
               when Precomputed_Sym_Family => "family",
               when Precomputed_Sym_Indent => "indent",
               when Precomputed_Sym_Int => "Int",
               when Precomputed_Sym_Invalid => "invalid",
               when Precomputed_Sym_Iterator => "Iterator",
               when Precomputed_Sym_Keep => "keep",
               when Precomputed_Sym_List => "list",
               when Precomputed_Sym_Main_Rule => "main_rule",
               when Precomputed_Sym_Newline => "newline",
               when Precomputed_Sym_No_Case => "no_case",
               when Precomputed_Sym_Node => "Node",
               when Precomputed_Sym_Node_31 => "node",
               when Precomputed_Sym_Open => "open",
               when Precomputed_Sym_Out => "out",
               when Precomputed_Sym_Pick => "pick",
               when Precomputed_Sym_Previous_Token => "previous_token",
               when Precomputed_Sym_Property => "property",
               when Precomputed_Sym_Propertyerror => "PropertyError",
               when Precomputed_Sym_Regexp => "Regexp",
               when Precomputed_Sym_Self => "self",
               when Precomputed_Sym_Send => "send",
               when Precomputed_Sym_Skip => "skip",
               when Precomputed_Sym_Stop_Cut => "stop_cut",
               when Precomputed_Sym_String => "String",
               when Precomputed_Sym_Symbol => "Symbol",
               when Precomputed_Sym_Tokennode => "TokenNode",
               when Precomputed_Sym_When => "when",
               when Precomputed_Sym_With_Lexer => "with_lexer"
            );

            Symbol : constant Symbolization_Result :=
                  Create_Symbol (Raw_Text)
            ;
         begin
            if Symbol.Success then
               return Symbol.Symbol;
            else
               raise Program_Error with
                 "Cannot canonicalize symbol literal: " & Image (Raw_Text);
            end if;
         end;
   end Precomputed_Symbol;

   ----------------------------
   -- Construct_Entity_Array --
   ----------------------------

   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access
   is
      Ret : Internal_Entity_Array_Access :=
        Create_Internal_Entity_Array (V.Length);
   begin
      for J in V.First_Index .. V.Last_Index loop
         Ret.Items (J) := V.Get (J);
      end loop;

      declare
         Tmp : AST_Envs.Entity_Vectors.Vector := V;
      begin
         Tmp.Destroy;
      end;

      return Ret;
   end Construct_Entity_Array;

   -----------
   -- Image --
   -----------

   function Image (Self : Symbol_Type) return String_Type is
   begin
      return Create_String (Image (Self));
   end Image;

   ------------------
   -- Context_Pool --
   ------------------

   package body Context_Pool is

      -------------
      -- Acquire --
      -------------

      procedure Acquire (Context : out Internal_Context) is
      begin
         GNAT.Task_Lock.Lock;

         if Available.Is_Empty then
            Context := new Analysis_Context_Type;
            Context.Serial_Number := 1;
         else
            Context := Available.Last_Element;
            Available.Delete_Last;
         end if;

         GNAT.Task_Lock.Unlock;

         Context.Initialized := False;
         Context.Ref_Count := 1;

      exception
         when others =>
            GNAT.Task_Lock.Unlock;
            raise;
      end Acquire;

      -------------
      -- Release --
      -------------

      procedure Release (Context : in out Internal_Context) is
      begin
         GNAT.Task_Lock.Lock;

         Available.Append (Context);
         Context.Serial_Number := Context.Serial_Number + 1;
         Context := null;

         GNAT.Task_Lock.Unlock;

      exception
         when others =>
            GNAT.Task_Lock.Unlock;
            raise;
      end Release;

      ----------
      -- Free --
      ----------

      procedure Free is
      begin
         GNAT.Task_Lock.Lock;

         for C of Available loop
            Free (C);
         end loop;

         GNAT.Task_Lock.Unlock;

      exception
         when others =>
            GNAT.Task_Lock.Unlock;
            raise;
      end Free;

   end Context_Pool;

   --------------
   -- Finalize --
   --------------

   overriding procedure Finalize (CD : in out Contexts_Destructor) is
      pragma Unreferenced (CD);
   begin
      Context_Pool.Free;
   end Finalize;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (File_Reader : in out Internal_File_Reader_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_File_Reader'Class, Internal_File_Reader_Access);
   begin
      if File_Reader /= null and then File_Reader.all.Dec_Ref then
         Destroy (File_Reader);
      end if;
   end Dec_Ref;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (Self : in out Internal_Event_Handler_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_Event_Handler'Class, Internal_Event_Handler_Access);
   begin
      if Self /= null and then Self.all.Dec_Ref then
         Destroy (Self);
      end if;
   end Dec_Ref;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (Provider : in out Internal_Unit_Provider_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_Unit_Provider'Class, Internal_Unit_Provider_Access);
   begin
      if Provider /= null and then Provider.all.Dec_Ref then
         Destroy (Provider);
      end if;
   end Dec_Ref;

   ----------------
   -- Get_Env_Id --
   ----------------

   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String
   is
      C        : Address_To_Id_Maps.Cursor;
      Inserted : Boolean;
   begin
      if E = Null_Lexical_Env then
         return "$null";

      elsif E = State.Root_Env then
         --  Insert root env with a special Id so that we only print it once
         State.Env_Ids.Insert (E, -1, C, Inserted);
         return "$root";
      end if;

      State.Env_Ids.Insert (E, State.Next_Id, C, Inserted);
      if Inserted then
         State.Next_Id := State.Next_Id + 1;
      end if;

      return '@' & Stripped_Image (Address_To_Id_Maps.Element (C));
   end Get_Env_Id;

   pragma Warnings (Off, "referenced");
   function To_Lookup_Kind_Type (K : Lookup_Kind) return Lookup_Kind_Type
   is
     (Lookup_Kind_Type'Val (Lookup_Kind'Pos (K)));
   pragma Warnings (On, "referenced");

   ----------------------
   -- Allocate_Context --
   ----------------------

   function Allocate_Context return Internal_Context is
   begin
      return Context : Internal_Context do
         Context_Pool.Acquire (Context);
      end return;
   end Allocate_Context;

   ------------------------
   -- Initialize_Context --
   ------------------------

   procedure Initialize_Context
     (Context        : Internal_Context;
      Charset        : String;
      File_Reader    : Internal_File_Reader_Access;
      Unit_Provider  : Internal_Unit_Provider_Access;
      Event_Handler  : Internal_Event_Handler_Access;
      With_Trivia    : Boolean;
      Tab_Stop       : Positive)
   is
      Actual_Charset : constant String :=
        (if Charset = "" then Default_Charset else Charset);
      Symbols        : constant Precomputed_Symbol_Table
        := Create_Symbol_Table;
   begin
      Context.Initialized := True;
      Context.Symbols := Symbol_Table (Symbols);
      Context.Charset := To_Unbounded_String (Actual_Charset);
      Context.Tab_Stop := Tab_Stop;
      Context.With_Trivia := With_Trivia;
      Context.Root_Scope := Create_Static_Lexical_Env
        (Parent    => Null_Lexical_Env,
         Node      => null,
         Sym_Table => Context.Symbols);

      --  Create a new ownership share for Event_Handler so that it lives at
      --  least as long as this analysis context.
      Context.Event_Handler := Event_Handler;
      if Context.Event_Handler /= null then
         Context.Event_Handler.Inc_Ref;
      end if;

      --  Create a new ownership share for File_Reader so that it lives at
      --  least as long as this analysis context.
      Context.File_Reader := File_Reader;
      if Context.File_Reader /= null then
         Context.File_Reader.Inc_Ref;
      end if;

      --  Create a new ownership share for Unit_Provider so that it lives at
      --  least as long as this analysis context.
      Context.Unit_Provider := Unit_Provider;
      if Context.Unit_Provider /= null then
         Context.Unit_Provider.Inc_Ref;
      end if;

         if Context.Unit_Provider = null then
            Context.Unit_Provider := Liblktlang.Default_Provider.Create;
         end if;

      Initialize (Context.Parser);

      Context.Discard_Errors_In_Populate_Lexical_Env := True;
      Context.Logic_Resolution_Timeout :=
        Langkit_Support.Adalog.Default_Timeout_Ticks_Number;
      Context.In_Populate_Lexical_Env := False;
      Context.Cache_Version := 0;
      Context.Reparse_Cache_Version := 0;

      Context.Rewriting_Handle := No_Rewriting_Handle_Pointer;
      Context.Templates_Unit := No_Analysis_Unit;

      Context.Available_Rebindings := Env_Rebindings_Vectors.Empty_Vector;

      

   end Initialize_Context;

   -----------------
   -- Create_Unit --
   -----------------

   function Create_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;

      Unit : Internal_Unit;
   begin
      Unit := Create_Special_Unit
        (Context, Normalized_Filename, Charset, Rule);
      Context.Units.Insert (Normalized_Filename, Unit);
      return Unit;
   end Create_Unit;

   --------------
   -- Get_Unit --
   --------------

   function Get_Unit
     (Context           : Internal_Context;
      Filename, Charset : String;
      Reparse           : Boolean;
      Input             : Internal_Lexer_Input;
      Rule              : Grammar_Rule;
      Is_Internal       : Boolean := False) return Internal_Unit
   is
      use Units_Maps;

      Normalized_Filename : constant GNATCOLL.VFS.Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);

      Cur     : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
      Created : constant Boolean := Cur = No_Element;
      Unit    : Internal_Unit;

      Actual_Charset : Unbounded_String;
      Refined_Input  : Internal_Lexer_Input := Input;

   begin
      --  Determine which encoding to use. Use the Charset parameter (if
      --  provided), otherwise use the context-wide default.

      Actual_Charset := (if Charset'Length /= 0
                         then To_Unbounded_String (Charset)
                         else Context.Charset);

      if Refined_Input.Kind = File then
         Refined_Input.Filename := Normalized_Filename;
      end if;

      if Refined_Input.Kind in File | Bytes_Buffer then
         Refined_Input.Charset := Actual_Charset;

         --  Unless the caller requested a specific charset for this unit,
         --  allow the lexer to automatically discover the source file encoding
         --  before defaulting to the context-specific one. We do this trying
         --  to match a byte order mark.

         Refined_Input.Read_BOM := Charset'Length = 0;
      end if;

      --  Create the Internal_Unit if needed

      Unit :=
        (if Created
         then Create_Unit (Context, Normalized_Filename,
                           To_String (Actual_Charset), Rule)
         else Element (Cur));

      --  If an internal unit is requested, set the corresponding flag.
      --  Otherwise, make sure that the unit we return isn't internal.

      if Is_Internal then
         Unit.Is_Internal := True;
      end if;

      --  (Re)parse it if needed

      if Created or else Reparse then

         --  It is illegal to reparse an internal unit for public API users.
         --  Since public APIs do not allow to pass True to Is_Internal, we can
         --  check here that only the implementation can ask to reparse an
         --  internal unit.

         if Unit.Is_Internal and then not Is_Internal then
            raise Precondition_Failure with "cannot reparse an internal unit";
         end if;

         declare
            Reparsed : Reparsed_Unit;
         begin
            Do_Parsing (Unit, Refined_Input, Reparsed);
            Update_After_Reparse (Unit, Reparsed);
         end;

         --  Now that we have removed reparsed the unit, update its current
         --  charset.

         Unit.Charset := Actual_Charset;
      end if;

      if Context.Event_Handler /= null then
         Context.Event_Handler.Unit_Parsed_Callback
           (Context  => Context,
            Unit     => Unit,
            Reparsed => not Created and then Reparse);
      end if;

      return Unit;
   end Get_Unit;

   --------------
   -- Has_Unit --
   --------------

   function Has_Unit
     (Context : Internal_Context; Unit_Filename : String) return Boolean is
   begin
      return Context.Units.Contains
        (Normalized_Unit_Filename (Context, Unit_Filename));
   end Has_Unit;

   -------------------
   -- Get_From_File --
   -------------------

   function Get_From_File
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Reparse  : Boolean;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind     => File,
         Charset  => <>,
         Read_BOM => False,
         Filename => <>);
   begin
      if Reparse and then Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot reparse during tree rewriting";
      end if;

      return Get_Unit (Context, Filename, Charset, Reparse, Input, Rule);
   end Get_From_File;

   ---------------------
   -- Get_From_Buffer --
   ---------------------

   function Get_From_Buffer
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Buffer   : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind        => Bytes_Buffer,
         Charset     => <>,
         Read_BOM    => False,
         Bytes       => Buffer'Address,
         Bytes_Count => Buffer'Length);
   begin
      if Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot parse from buffer during tree rewriting";

      elsif Context.File_Reader /= null then
         raise Precondition_Failure with
            "cannot parse from buffer with a file reader";
      end if;

      return Get_Unit (Context, Filename, Charset, True, Input, Rule);
   end Get_From_Buffer;

   --------------------
   -- Get_With_Error --
   --------------------

   function Get_With_Error
     (Context  : Internal_Context;
      Filename : String;
      Error    : Text_Type;
      Charset  : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;

      Normalized_Filename : constant Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);
      Cur                 : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
   begin
      if Cur = No_Element then
         declare
            Unit : constant Internal_Unit := Create_Unit
              (Context, Normalized_Filename, Charset, Rule);
         begin
            Append (Unit.Diagnostics, No_Source_Location_Range, Error);
            return Unit;
         end;
      else
         return Element (Cur);
      end if;
   end Get_With_Error;


   -----------------------
   -- Get_From_Provider --
   -----------------------

   function Get_From_Provider
     (Context : Internal_Context;
      Name    : Text_Type;
      Kind    : Analysis_Unit_Kind;
      Charset : String;
      Reparse : Boolean) return Internal_Unit is
   begin
      if Reparse and then Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot reparse during tree rewriting";
      end if;

      declare
         Result      : Internal_Unit;
         Dummy_Index : Positive;
      begin
         Context.Unit_Provider.Get_Unit_And_PLE_Root
           (Context, Name, Kind, Charset, Reparse, Result, Dummy_Index);
         return Result;
      exception
         when Property_Error =>
            raise Invalid_Unit_Name_Error with
               "Invalid unit name: " & Image (Name, With_Quotes => True)
               & " (" & Analysis_Unit_Kind'Image (Kind) & ")";
      end;
   end Get_From_Provider;


   -------------------
   -- Unit_Provider --
   -------------------

   function Unit_Provider
     (Context : Internal_Context) return Internal_Unit_Provider_Access
   is (Context.Unit_Provider);

   ------------------
   -- Resolve_Unit --
   ------------------

   procedure Resolve_Unit
     (Context : Internal_Context;
      Name    : Text_Type;
      Kind    : Analysis_Unit_Kind;
      Unit    : out Resolved_Unit)
   is
      --  Look for the cache entry corresponding to Unit; create one if needed

      Dummy    : Resolved_Unit_Array;
      Key      : constant Symbol_Type := Find (Context.Symbols, Name);
      Pos      : Unit_Provider_Cache_Maps.Cursor;
      Inserted : Boolean;
   begin
      Context.Unit_Provider_Cache.Insert (Key, Dummy, Pos, Inserted);
      declare
         Units : Resolved_Unit_Array renames
           Context.Unit_Provider_Cache.Reference (Pos);
         U     : Resolved_Unit renames Units (Kind);
      begin
         --  If the cache entry is not populated for the requested kind, run
         --  the query and save the result for later requests.

         if U.Filename = null then
            declare
               Provider : Internal_Unit_Provider'Class renames
                 Context.Unit_Provider.all;
               Filename : Unbounded_String;
            begin
               Provider.Get_Unit_Location
                 (Name           => Name,
                  Kind           => Kind,
                  Filename       => Filename,
                  PLE_Root_Index => U.PLE_Root_Index);
               Provider.Get_Unit_And_PLE_Root
                 (Context        => Context,
                  Name           => Name,
                  Kind           => Kind,
                  Unit           => U.Unit,
                  PLE_Root_Index => U.PLE_Root_Index);
               U.Filename := new String'(To_String (Filename));
            end;
         end if;

         Unit := U;
      end;
   end Resolve_Unit;

   -----------------------
   -- Get_Unit_Location --
   -----------------------

   procedure Get_Unit_Location
     (Context        : Internal_Context;
      Name           : Text_Type;
      Kind           : Analysis_Unit_Kind;
      Filename       : out String_Access;
      PLE_Root_Index : out Positive)
   is
      U : Resolved_Unit;
   begin
      Resolve_Unit (Context, Name, Kind, U);
      Filename := U.Filename;
      PLE_Root_Index := U.PLE_Root_Index;
   end Get_Unit_Location;

   ---------------------------
   -- Get_Unit_And_PLE_Root --
   ---------------------------

   procedure Get_Unit_And_PLE_Root
     (Context        : Internal_Context;
      Name           : Text_Type;
      Kind           : Analysis_Unit_Kind;
      Unit           : out Internal_Unit;
      PLE_Root_Index : out Positive)
   is
      U : Resolved_Unit;
   begin
      Resolve_Unit (Context, Name, Kind, U);
      Unit := U.Unit;
      PLE_Root_Index := U.PLE_Root_Index;
   end Get_Unit_And_PLE_Root;

   ----------
   -- Hash --
   ----------

   function Hash (Context : Internal_Context) return Hash_Type is
      function H is new Hash_Access (Analysis_Context_Type, Internal_Context);
   begin
      return H (Context);
   end Hash;

   ---------------------
   -- Has_With_Trivia --
   ---------------------

   function Has_With_Trivia (Context : Internal_Context) return Boolean is
   begin
      return Context.With_Trivia;
   end Has_With_Trivia;

   --------------------------------------------
   -- Discard_Errors_In_Populate_Lexical_Env --
   --------------------------------------------

   procedure Discard_Errors_In_Populate_Lexical_Env
     (Context : Internal_Context; Discard : Boolean) is
   begin
      Context.Discard_Errors_In_Populate_Lexical_Env := Discard;
   end Discard_Errors_In_Populate_Lexical_Env;

   ----------------------------------
   -- Set_Logic_Resolution_Timeout --
   ----------------------------------

   procedure Set_Logic_Resolution_Timeout
     (Context : Internal_Context; Timeout : Natural) is
   begin
      Context.Logic_Resolution_Timeout := Timeout;
   end Set_Logic_Resolution_Timeout;

   --------------------------
   -- Has_Rewriting_Handle --
   --------------------------

   function Has_Rewriting_Handle (Context : Internal_Context) return Boolean is
   begin
      return Context.Rewriting_Handle /= No_Rewriting_Handle_Pointer;
   end Has_Rewriting_Handle;

   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (Context : Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count + 1;
      end if;
   end Inc_Ref;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (Context : in out Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count - 1;
         if Context.Ref_Count = 0 then

            --  If this context was not completely initialized, just release
            --  the allocated object. Do the full destruction otherwise.
            if Context.Initialized then
               Destroy (Context);
            end if;
            Context_Pool.Release (Context);
         end if;
      end if;
   end Dec_Ref;

   -------------
   -- Destroy --
   -------------

   procedure Destroy (Context : Internal_Context) is
   begin
      --  Destroy all named environment data structures
      for Desc of Context.Named_Envs loop
         for V of Desc.Foreign_Nodes loop
            V.Destroy;
         end loop;
         Destroy (Desc);
      end loop;
      Context.Named_Envs.Clear;

      --  If we are asked to free this context, it means that no one else have
      --  references to its analysis units, so it's safe to destroy these.
      for Unit of Context.Units loop
         Destroy (Unit);
      end loop;
      Context.Units := Units_Maps.Empty_Map;
      Context.Filenames := Virtual_File_Maps.Empty_Map;

      declare
         procedure Destroy is new Ada.Unchecked_Deallocation
           (Env_Rebindings_Type, Env_Rebindings);

         AR : Env_Rebindings_Vectors.Vector renames
            Context.Available_Rebindings;
         R  : Env_Rebindings;
      begin
         for I in AR.First_Index .. AR.Last_Index loop
            R := AR.Get (I);
            Destroy (R);
         end loop;
         AR.Destroy;
      end;

      for Pos in Context.Unit_Provider_Cache.Iterate loop
         declare
            Units : Resolved_Unit_Array renames
              Context.Unit_Provider_Cache.Reference (Pos);
         begin
            for U of Units loop
               Free (U.Filename);
            end loop;
         end;
      end loop;
      Context.Unit_Provider_Cache.Clear;

      Destroy (Context.Templates_Unit);
      AST_Envs.Destroy (Context.Root_Scope);
      Destroy (Context.Symbols);
      Destroy (Context.Parser);
      Dec_Ref (Context.File_Reader);
      Dec_Ref (Context.Unit_Provider);
      Dec_Ref (Context.Event_Handler);
   end Destroy;

   -------------
   -- Context --
   -------------

   function Context (Unit : Internal_Unit) return Internal_Context is
   begin
      return Unit.Context;
   end Context;

   ----------
   -- Hash --
   ----------

   function Hash (Unit : Internal_Unit) return Hash_Type is
      function H is new Hash_Access (Analysis_Unit_Type, Internal_Unit);
   begin
      return H (Unit);
   end Hash;

   -------------
   -- Reparse --
   -------------

   procedure Reparse (Unit : Internal_Unit; Charset : String) is
      Dummy : constant Internal_Unit := Get_From_File
        (Unit.Context, +Unit.Filename.Full_Name, Charset,
         Reparse => True,
         Rule    => Unit.Rule);
   begin
      null;
   end Reparse;

   -------------
   -- Reparse --
   -------------

   procedure Reparse (Unit : Internal_Unit; Charset : String; Buffer : String)
   is
      Dummy : constant Internal_Unit := Get_From_Buffer
        (Unit.Context, +Unit.Filename.Full_Name, Charset, Buffer, Unit.Rule);
   begin
      null;
   end Reparse;

   -----------------------
   -- Reset_Envs_Caches --
   -----------------------

   procedure Reset_Envs_Caches (Unit : Internal_Unit) is
      procedure Internal (Node : Bare_Lkt_Node);
      --  Reset env caches in ``Node`` and then in its children recursively

      Generic_Unit : constant Generic_Unit_Ptr := Convert_Unit (Unit);

      --------------
      -- Internal --
      --------------

      procedure Internal (Node : Bare_Lkt_Node) is
      begin
         if Node = null then
            return;
         end if;
         --  Make sure to only reset caches of envs belonging to this unit
         if Node.Self_Env.Owner = Generic_Unit then
            Reset_Caches (Node.Self_Env);
         end if;
         for I in 1 .. Children_Count (Node) loop
            Internal (Child (Node, I));
         end loop;
      end Internal;
   begin
      Internal (Unit.Ast_Root);
   end Reset_Envs_Caches;

   --------------------------
   -- Populate_Lexical_Env --
   --------------------------

   procedure Populate_Lexical_Env
     (Unit           : Internal_Unit;
      PLE_Root_Index : Positive
         := 1
   ) is
      Context : constant Internal_Context := Unit.Context;

      Saved_In_Populate_Lexical_Env : constant Boolean :=
        Context.In_Populate_Lexical_Env;

      Has_Errors : Boolean := False;
      --  Whether at least one Property_Error occurred during this PLE pass

   begin
      --  TODO??? Handle env invalidation when reparsing a unit and when a
      --  previous call raised a Property_Error.

      --  If we have already run PLE on this root, there is nothing to do.
      --  Otherwise, keep track of the fact that PLE was requested for it,
      --  possibly extending the vector if needed.

      if Unit.Env_Populated_Roots.Last_Index >= PLE_Root_Index
         and then Unit.Env_Populated_Roots.Get (PLE_Root_Index)
      then
         return;
      end if;
      for Dummy in Unit.Env_Populated_Roots.Last_Index + 1 .. PLE_Root_Index
      loop
         Unit.Env_Populated_Roots.Append (False);
      end loop;
      Unit.Env_Populated_Roots.Set (PLE_Root_Index, True);

      --  Create context for the PLE run: all exit points must call the Cleanup
      --  procedure above first to clean this context.

      Context.In_Populate_Lexical_Env := True;
      if Main_Trace.Active then
         Main_Trace.Trace
           ("Populating lexical envs for"
            & " unit: " & Basename (Unit));
         Main_Trace.Increase_Indent;
      end if;

      --  Fetch the node on which to run PLE: it's the unit root node, or one
      --  of its children if PLE roots are enabled and the unit has a list of
      --  PLE roots. Then run PLE itself.

      declare
         PLE_Root : Bare_Lkt_Node := Unit.Ast_Root;
      begin

         if PLE_Root /= null then
            Has_Errors := Populate_Lexical_Env (PLE_Root);
         end if;
      end;

      --  Restore the context for PLE run (undo what was done above)

      Context.In_Populate_Lexical_Env := Saved_In_Populate_Lexical_Env;
      if Main_Trace.Active then
         Main_Trace.Decrease_Indent;
         Main_Trace.Trace
           ("Finished populating lexical envs for"
            & " unit: " & Basename (Unit));
      end if;

      Reset_Envs_Caches (Unit);

      if Has_Errors and then not Context.Discard_Errors_In_Populate_Lexical_Env
      then
         raise Property_Error with
            "errors occurred in Populate_Lexical_Env";
      end if;
   end Populate_Lexical_Env;

   -----------------------------------
   -- Populate_Lexical_Env_For_Unit --
   -----------------------------------

   procedure Populate_Lexical_Env_For_Unit (Node : Bare_Lkt_Node) is
      Root  : Bare_Lkt_Node;
      Index : Natural;
   begin
      Lookup_PLE_Root (Node, Root, Index);
      if Index = 0 then
         Index := 1;
      end if;
      Populate_Lexical_Env (Node.Unit, Index);
   end Populate_Lexical_Env_For_Unit;

   ------------------
   -- Get_Filename --
   ------------------

   function Get_Filename (Unit : Internal_Unit) return String is
     (+Unit.Filename.Full_Name);

   -----------------
   -- Get_Charset --
   -----------------

   function Get_Charset (Unit : Internal_Unit) return String is
   begin
      return To_String (Unit.Charset);
   end Get_Charset;

   ---------------------
   -- Has_Diagnostics --
   ---------------------

   function Has_Diagnostics (Unit : Internal_Unit) return Boolean is
   begin
      return not Unit.Diagnostics.Is_Empty;
   end Has_Diagnostics;

   -----------------
   -- Diagnostics --
   -----------------

   function Diagnostics (Unit : Internal_Unit) return Diagnostics_Array is
      Result : Diagnostics_Array (1 .. Natural (Unit.Diagnostics.Length));
      I      : Natural := 1;
   begin
      for D of Unit.Diagnostics loop
         Result (I) := D;
         I := I + 1;
      end loop;
      return Result;
   end Diagnostics;

   ---------------------------
   -- Format_GNU_Diagnostic --
   ---------------------------

   function Format_GNU_Diagnostic
     (Unit : Internal_Unit; D : Diagnostic) return String
   is
      Filename : constant String := Basename (Unit);
      Sloc     : constant Source_Location := Start_Sloc (D.Sloc_Range);
      Msg      : constant String :=
         Image
           (Ada.Strings.Wide_Wide_Unbounded.To_Wide_Wide_String (D.Message));
   begin
      return (Filename
              & (if Sloc = No_Source_Location then "" else ":" & Image (Sloc))
              & ": " & Msg);
   end Format_GNU_Diagnostic;

   ----------
   -- Root --
   ----------

   function Root (Unit : Internal_Unit) return Bare_Lkt_Node is
     (Unit.Ast_Root);

   -----------------
   -- First_Token --
   -----------------

   function First_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.Context,
                            Unit.TDH'Access,
                            First_Token_Or_Trivia (Unit.TDH)));

   ----------------
   -- Last_Token --
   ----------------

   function Last_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.Context,
                            Unit.TDH'Access,
                            Last_Token_Or_Trivia (Unit.TDH)));

   -----------------
   -- Token_Count --
   -----------------

   function Token_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Tokens.Length);

   ------------------
   -- Trivia_Count --
   ------------------

   function Trivia_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Trivias.Length);

   ----------
   -- Text --
   ----------

   function Text (Unit : Internal_Unit) return Text_Type is
   begin
      return Text (First_Token (Unit), Last_Token (Unit));
   end Text;

   ------------------
   -- Lookup_Token --
   ------------------

   function Lookup_Token
     (Unit : Internal_Unit; Sloc : Source_Location) return Token_Reference
   is
      Result : constant Token_Or_Trivia_Index := Lookup_Token (Unit.TDH, Sloc);
   begin
      return Wrap_Token_Reference (Unit.Context, Unit.TDH'Access, Result);
   end Lookup_Token;

   ---------------------
   -- Lookup_PLE_Root --
   ---------------------

   procedure Lookup_PLE_Root
     (Node  : Bare_Lkt_Node;
      Root  : out Bare_Lkt_Node;
      Index : out Natural)
   is
      Unit : constant Internal_Unit := Node.Unit;
   begin
      --  If this unit does not contain a list of PLE roots, just return the
      --  unit root node.

      if Unit.PLE_Roots_Starting_Token.Is_Empty then
         Root := Unit.Ast_Root;
         Index := 0;
         return;
      end if;

      --  Otherwise, look for the last PLE root whose first token (in
      --  Unit.PLE_Roots_Starting_Token) appears before Node's (T). This vector
      --  is sorted by construction, so we can perform a binary search.

      declare
         T      : constant Token_Index := Node.Token_Start_Index;
         Tokens : Token_Index_Vectors.Vector renames
           Unit.PLE_Roots_Starting_Token;

         First : Positive := Tokens.First_Index;
         Last  : Positive := Tokens.Last_Index;
         I     : Positive;
      begin
         while First < Last loop

            --  Because we look for the "floor" (last element that is <= T), we
            --  need to look at the value in Last when there are only two
            --  elements left to look at. If we did not do that, then we would
            --  go into an infinite loop when Tokens[First] < T.

            I := (if First + 1 = Last
                  then Last
                  else (First + Last) / 2);
            declare
               I_T : constant Token_Index := Tokens.Get (I);
            begin
               if I_T <= T then
                  First := I;
               else
                  Last := I - 1;
               end if;
            end;
         end loop;

         Root := Child (Unit.Ast_Root, First);
         Index := First;
      end;
   end Lookup_PLE_Root;

   --------------
   -- Ple_Root --
   --------------

   function Ple_Root
     (Node : Bare_Lkt_Node) return Bare_Lkt_Node
   is
      Root        : Bare_Lkt_Node;
      Dummy_Index : Natural;
   begin
      if Node = null then
         raise Property_Error with "null node dereference";
      end if;
      Lookup_PLE_Root (Node, Root, Dummy_Index);
      return Root;
   end Ple_Root;

   ----------------------
   -- Dump_Lexical_Env --
   ----------------------

   procedure Dump_Lexical_Env (Unit : Internal_Unit) is
      Node     : constant Bare_Lkt_Node := Unit.Ast_Root;
      Root_Env : constant Lexical_Env := Unit.Context.Root_Scope;
      State    : Dump_Lexical_Env_State := (Root_Env => Root_Env, others => <>);

      function Get_Parent (Env : Lexical_Env) return Lexical_Env
      is (Unwrap (Env).Parent);

      --------------------------
      -- Explore_Parent_Chain --
      --------------------------

      procedure Explore_Parent_Chain (Env : Lexical_Env) is
         P : Lexical_Env;
      begin
         if Env /= Null_Lexical_Env then
            P := Get_Parent (Env);
            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State), Get_Env_Id (P, State));
            Explore_Parent_Chain (P);
         end if;
      end Explore_Parent_Chain;

      --------------
      -- Internal --
      --------------

      procedure Internal (Current : Bare_Lkt_Node) is
         Explore_Parent : Boolean := False;
         Env, Parent    : Lexical_Env;
      begin
         if Current = null then
            return;
         end if;

         --  We only dump environments that we haven't dumped before. This way
         --  we'll only dump environments at the site of their creation, and
         --  not in any subsequent link. We use the Env_Ids map to check which
         --  envs we have already seen or not.
         if not State.Env_Ids.Contains (Current.Self_Env) then
            Env := Current.Self_Env;
            Parent := Get_Parent (Env);
            Explore_Parent := not State.Env_Ids.Contains (Parent);

            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State), Get_Env_Id (Parent, State));

            if Explore_Parent then
               Explore_Parent_Chain (Parent);
            end if;
         end if;

         for Child of Internal_Bare_Lkt_Node_Array'(Children (Current))
         loop
            Internal (Child);
         end loop;
      end Internal;
      --  This procedure implements the main recursive logic of dumping the
      --  environments.
   begin
      Internal (Bare_Lkt_Node (Node));
   end Dump_Lexical_Env;

   --------------
   -- Get_Line --
   --------------

   function Get_Line
     (Unit : Internal_Unit; Line_Number : Positive) return Text_Type
   is
   begin
      return Get_Line (Unit.TDH, Line_Number);
   end Get_Line;

   -----------
   -- Print --
   -----------

   procedure Print (Unit : Internal_Unit; Show_Slocs : Boolean) is
   begin
      if Unit.Ast_Root = null then
         Put_Line ("<empty analysis unit>");
      else
         Print (Unit.Ast_Root, Show_Slocs);
      end if;
   end Print;

   ---------------
   -- PP_Trivia --
   ---------------

   procedure PP_Trivia (Unit : Internal_Unit) is

      procedure Process (Trivia : Token_Index) is
         Data : constant Stored_Token_Data :=
            Unit.TDH.Trivias.Get (Natural (Trivia)).T;
      begin
         Put_Line (Image (Text (Unit.TDH, Data)));
      end Process;

      Last_Token : constant Token_Index :=
         Token_Index (Token_Vectors.Last_Index (Unit.TDH.Tokens) - 1);
      --  Index for the last token in Unit excluding the Termination token
      --  (hence the -1).
   begin
      for Tok of Get_Leading_Trivias (Unit.TDH) loop
         Process (Tok);
      end loop;

      PP_Trivia (Unit.Ast_Root);

      for Tok of Get_Trivias (Unit.TDH, Last_Token) loop
         Process (Tok);
      end loop;
   end PP_Trivia;

   -------------
   -- Destroy --
   -------------

   procedure Destroy (Unit : in out Internal_Unit) is
   begin
      if Unit = No_Analysis_Unit then
         return;
      end if;

      --  Clear the env caches while the unit is still fully alive to make sure
      --  that what is accessed in ``Lexical_Env_Cache_Updated`` is still
      --  valid, as it will be called back by lexical envs that are being
      --  destroyed.
      Reset_Envs_Caches (Unit);

      Unit.PLE_Roots_Starting_Token.Destroy;
      Unit.Env_Populated_Roots.Destroy;

      Unit.Exiled_Entries.Destroy;
      Unit.Foreign_Nodes.Destroy;
      Unit.Exiled_Entries_In_NED.Destroy;
      Unit.Exiled_Envs.Destroy;
      Unit.Named_Envs.Destroy;
      Analysis_Unit_Sets.Destroy (Unit.Referenced_Units);

         Destroy (Unit.Memoization_Map);

      Destroy_Rebindings (Unit.Rebindings'Access);
      Unit.Rebindings.Destroy;

      if Unit.Ast_Root /= null then
         Destroy (Unit.Ast_Root);
      end if;

      Free (Unit.TDH);
      Free (Unit.Ast_Mem_Pool);
      Destroy_Unit_Destroyables (Unit);
      Destroyable_Vectors.Destroy (Unit.Destroyables);
      

      Free (Unit);
   end Destroy;

   -------------------
   -- Is_Token_Node --
   -------------------

   function Is_Token_Node (Node : Bare_Lkt_Node) return Boolean is
   begin
      return Is_Token_Node (Node.Kind);
   end Is_Token_Node;

   ------------------
   -- Is_Synthetic --
   ------------------

   function Is_Synthetic (Node : Bare_Lkt_Node) return Boolean is
   begin
      return Node.Kind in Synthetic_Nodes;
   end Is_Synthetic;

   ------------------------------
   -- Raise_Property_Exception --
   ------------------------------

   procedure Raise_Property_Exception
     (Node    : Bare_Lkt_Node;
      Exc     : Ada.Exceptions.Exception_Id;
      Message : String)
   is
      Sloc_Prefix : constant String :=
        (if Node = null
         then ""
         else Ada.Directories.Simple_Name (Get_Filename (Unit (Node)))
              & ":" & Image (Sloc_Range (Node)) & ": ");
   begin
      Ada.Exceptions.Raise_Exception (Exc, Sloc_Prefix & Message);
   end Raise_Property_Exception;

   ------------------------------
   -- Register_Destroyable_Gen --
   ------------------------------

   procedure Register_Destroyable_Gen
     (Unit : Internal_Unit; Object : T_Access)
   is
      function Convert is new Ada.Unchecked_Conversion
        (System.Address, Destroy_Procedure);
      procedure Destroy_Procedure (Object : in out T_Access) renames Destroy;
   begin
      Register_Destroyable_Helper
        (Unit,
         Object.all'Address,
         Convert (Destroy_Procedure'Address));
   end Register_Destroyable_Gen;

      

   


   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Bare_Lkt_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Lkt_Node
   is
      function Absolute_Get
        (T : Bare_Lkt_Node_Array_Access; Index : Integer)
         return Bare_Lkt_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Lkt_Node,
         Sequence_Type => Bare_Lkt_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Bare_Lkt_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Lkt_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Bare_Lkt_Node_Array_Access) return Bare_Lkt_Node_Array_Access is
      Ret : Bare_Lkt_Node_Array_Access := Create_Bare_Lkt_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Bare_Lkt_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Bare_Lkt_Node_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Bare_Lkt_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Bare_Lkt_Node_Array (Items_Count : Natural) return Bare_Lkt_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Lkt_Node_Array_Type
       else new Bare_Lkt_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Bare_Lkt_Node_Array
     (Items : Internal_Bare_Lkt_Node_Array) return Bare_Lkt_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Lkt_Node_Array_Type;
      end if;

      return new Bare_Lkt_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Bare_Lkt_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Bare_Lkt_Node_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Bare_Type_Decl_Vectors is new Langkit_Support.Vectors (Bare_Type_Decl);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Bare_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Type_Decl
   is
      function Absolute_Get
        (T : Bare_Type_Decl_Array_Access; Index : Integer)
         return Bare_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Type_Decl,
         Sequence_Type => Bare_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Bare_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Lkt_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Bare_Type_Decl_Array_Access) return Bare_Type_Decl_Array_Access is
      Ret : Bare_Type_Decl_Array_Access := Create_Bare_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Bare_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Bare_Type_Decl_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Bare_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Bare_Type_Decl_Array (Items_Count : Natural) return Bare_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Bare_Type_Decl_Array_Type
       else new Bare_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Bare_Type_Decl_Array
     (Items : Internal_Bare_Type_Decl_Array) return Bare_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Type_Decl_Array_Type;
      end if;

      return new Bare_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Bare_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Bare_Type_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;


      ----------
      -- Hash --
      ----------

      function Hash (R : Bare_Type_Decl_Array_Access) return Hash_Type is
         Result : Hash_Type := Initial_Hash;
      begin
         for I in R.Items'Range loop
            Result := Combine (Result, Hash (R.Items (I)));
         end loop;
         return Result;
      end Hash;


      

   


   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Boolean_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Boolean
   is
      function Absolute_Get
        (T : Boolean_Array_Access; Index : Integer)
         return Boolean
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Boolean,
         Sequence_Type => Boolean_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Boolean;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return False;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Boolean_Array_Access) return Boolean_Array_Access is
      Ret : Boolean_Array_Access := Create_Boolean_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Boolean_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Boolean_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Boolean_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Boolean_Array (Items_Count : Natural) return Boolean_Array_Access
   is (if Items_Count = 0
       then No_Boolean_Array_Type
       else new Boolean_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Boolean_Array
     (Items : Internal_Boolean_Array) return Boolean_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Boolean_Array_Type;
      end if;

      return new Boolean_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Boolean_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Boolean_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Integer_Vectors is new Langkit_Support.Vectors (Integer);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Integer_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Integer
   is
      function Absolute_Get
        (T : Integer_Array_Access; Index : Integer)
         return Integer
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Integer,
         Sequence_Type => Integer_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Integer;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return 0;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Integer_Array_Access) return Integer_Array_Access is
      Ret : Integer_Array_Access := Create_Integer_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Integer_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Integer_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Integer_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Integer_Array (Items_Count : Natural) return Integer_Array_Access
   is (if Items_Count = 0
       then No_Integer_Array_Type
       else new Integer_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Integer_Array
     (Items : Internal_Integer_Array) return Integer_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Integer_Array_Type;
      end if;

      return new Integer_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Integer_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Integer_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   


   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Entity_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity
   is
      function Absolute_Get
        (T : Internal_Entity_Array_Access; Index : Integer)
         return Internal_Entity
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity,
         Sequence_Type => Internal_Entity_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Entity;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Entity_Array_Access) return Internal_Entity_Array_Access is
      Ret : Internal_Entity_Array_Access := Create_Internal_Entity_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Entity_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Entity_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Entity_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Entity_Array (Items_Count : Natural) return Internal_Entity_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));

   function Create_Internal_Entity_Array
     (Items : AST_Envs.Entity_Array) return Internal_Entity_Array_Access
   is (if Items'Length = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'
         (N         => Items'Length,
          Items     => Implementation.Internal_Internal_Entity_Array (Items),
          Ref_Count => 1));

   function Create_Internal_Entity_Array
     (Items : Internal_Internal_Entity_Array) return Internal_Entity_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Array_Type;
      end if;

      return new Internal_Entity_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Entity_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Entity_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Entity_Component_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Component_Decl);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Entity_Component_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Component_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Component_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Component_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Component_Decl,
         Sequence_Type => Internal_Entity_Component_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Entity_Component_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Component_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Entity_Component_Decl_Array_Access) return Internal_Entity_Component_Decl_Array_Access is
      Ret : Internal_Entity_Component_Decl_Array_Access := Create_Internal_Entity_Component_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Entity_Component_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Entity_Component_Decl_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Entity_Component_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Entity_Component_Decl_Array (Items_Count : Natural) return Internal_Entity_Component_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Component_Decl_Array_Type
       else new Internal_Entity_Component_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Entity_Component_Decl_Array
     (Items : Internal_Internal_Entity_Component_Decl_Array) return Internal_Entity_Component_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Component_Decl_Array_Type;
      end if;

      return new Internal_Entity_Component_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Entity_Component_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Entity_Component_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Entity_Enum_Class_Alt_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Enum_Class_Alt_Decl);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Entity_Enum_Class_Alt_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Enum_Class_Alt_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Enum_Class_Alt_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Enum_Class_Alt_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Enum_Class_Alt_Decl,
         Sequence_Type => Internal_Entity_Enum_Class_Alt_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Entity_Enum_Class_Alt_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Enum_Class_Alt_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Entity_Enum_Class_Alt_Decl_Array_Access) return Internal_Entity_Enum_Class_Alt_Decl_Array_Access is
      Ret : Internal_Entity_Enum_Class_Alt_Decl_Array_Access := Create_Internal_Entity_Enum_Class_Alt_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Entity_Enum_Class_Alt_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Entity_Enum_Class_Alt_Decl_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Entity_Enum_Class_Alt_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Entity_Enum_Class_Alt_Decl_Array (Items_Count : Natural) return Internal_Entity_Enum_Class_Alt_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Enum_Class_Alt_Decl_Array_Type
       else new Internal_Entity_Enum_Class_Alt_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Entity_Enum_Class_Alt_Decl_Array
     (Items : Internal_Internal_Entity_Enum_Class_Alt_Decl_Array) return Internal_Entity_Enum_Class_Alt_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Enum_Class_Alt_Decl_Array_Type;
      end if;

      return new Internal_Entity_Enum_Class_Alt_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Entity_Enum_Class_Alt_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Entity_Enum_Class_Alt_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Entity_Expr_Vectors is new Langkit_Support.Vectors (Internal_Entity_Expr);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Entity_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Expr
   is
      function Absolute_Get
        (T : Internal_Entity_Expr_Array_Access; Index : Integer)
         return Internal_Entity_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Expr,
         Sequence_Type => Internal_Entity_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Entity_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Expr;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Entity_Expr_Array_Access) return Internal_Entity_Expr_Array_Access is
      Ret : Internal_Entity_Expr_Array_Access := Create_Internal_Entity_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Entity_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Entity_Expr_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Entity_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Entity_Expr_Array (Items_Count : Natural) return Internal_Entity_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Expr_Array_Type
       else new Internal_Entity_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Entity_Expr_Array
     (Items : Internal_Internal_Entity_Expr_Array) return Internal_Entity_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Expr_Array_Type;
      end if;

      return new Internal_Entity_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Entity_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Entity_Expr_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Entity_Full_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Full_Decl);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Entity_Full_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Full_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Full_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Full_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Full_Decl,
         Sequence_Type => Internal_Entity_Full_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Entity_Full_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Full_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Entity_Full_Decl_Array_Access) return Internal_Entity_Full_Decl_Array_Access is
      Ret : Internal_Entity_Full_Decl_Array_Access := Create_Internal_Entity_Full_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Entity_Full_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Entity_Full_Decl_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Entity_Full_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Entity_Full_Decl_Array (Items_Count : Natural) return Internal_Entity_Full_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Full_Decl_Array_Type
       else new Internal_Entity_Full_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Entity_Full_Decl_Array
     (Items : Internal_Internal_Entity_Full_Decl_Array) return Internal_Entity_Full_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Full_Decl_Array_Type;
      end if;

      return new Internal_Entity_Full_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Entity_Full_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Entity_Full_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Entity_Generic_Formal_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Generic_Formal_Type_Decl);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Entity_Generic_Formal_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Generic_Formal_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Generic_Formal_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Generic_Formal_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Generic_Formal_Type_Decl,
         Sequence_Type => Internal_Entity_Generic_Formal_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Entity_Generic_Formal_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Generic_Formal_Type_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Entity_Generic_Formal_Type_Decl_Array_Access) return Internal_Entity_Generic_Formal_Type_Decl_Array_Access is
      Ret : Internal_Entity_Generic_Formal_Type_Decl_Array_Access := Create_Internal_Entity_Generic_Formal_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Entity_Generic_Formal_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Entity_Generic_Formal_Type_Decl_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Entity_Generic_Formal_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Entity_Generic_Formal_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Generic_Formal_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Generic_Formal_Type_Decl_Array_Type
       else new Internal_Entity_Generic_Formal_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Entity_Generic_Formal_Type_Decl_Array
     (Items : Internal_Internal_Entity_Generic_Formal_Type_Decl_Array) return Internal_Entity_Generic_Formal_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Generic_Formal_Type_Decl_Array_Type;
      end if;

      return new Internal_Entity_Generic_Formal_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Entity_Generic_Formal_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Entity_Generic_Formal_Type_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Entity_Param_Vectors is new Langkit_Support.Vectors (Internal_Entity_Param);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Entity_Param_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Param
   is
      function Absolute_Get
        (T : Internal_Entity_Param_Array_Access; Index : Integer)
         return Internal_Entity_Param
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Param,
         Sequence_Type => Internal_Entity_Param_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Entity_Param;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Param;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Entity_Param_Array_Access) return Internal_Entity_Param_Array_Access is
      Ret : Internal_Entity_Param_Array_Access := Create_Internal_Entity_Param_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Entity_Param_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Entity_Param_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Entity_Param_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Entity_Param_Array (Items_Count : Natural) return Internal_Entity_Param_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Param_Array_Type
       else new Internal_Entity_Param_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Entity_Param_Array
     (Items : Internal_Internal_Entity_Param_Array) return Internal_Entity_Param_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Param_Array_Type;
      end if;

      return new Internal_Entity_Param_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Entity_Param_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Entity_Param_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Entity_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Type_Decl);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Entity_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Type_Decl,
         Sequence_Type => Internal_Entity_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Entity_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Type_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Entity_Type_Decl_Array_Access) return Internal_Entity_Type_Decl_Array_Access is
      Ret : Internal_Entity_Type_Decl_Array_Access := Create_Internal_Entity_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Entity_Type_Decl_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Entity_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Type_Decl_Array_Type
       else new Internal_Entity_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Entity_Type_Decl_Array
     (Items : Internal_Internal_Entity_Type_Decl_Array) return Internal_Entity_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Type_Decl_Array_Type;
      end if;

      return new Internal_Entity_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Entity_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Entity_Type_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Env_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Env_Assoc);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Env_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Env_Assoc
   is
      function Absolute_Get
        (T : Internal_Env_Assoc_Array_Access; Index : Integer)
         return Internal_Env_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Env_Assoc,
         Sequence_Type => Internal_Env_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Env_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Env_Assoc;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Env_Assoc_Array_Access) return Internal_Env_Assoc_Array_Access is
      Ret : Internal_Env_Assoc_Array_Access := Create_Internal_Env_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Env_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Env_Assoc_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Env_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Env_Assoc_Array (Items_Count : Natural) return Internal_Env_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Env_Assoc_Array_Type
       else new Internal_Env_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Env_Assoc_Array
     (Items : Internal_Internal_Env_Assoc_Array) return Internal_Env_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Env_Assoc_Array_Type;
      end if;

         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Env_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Env_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Env_Assoc_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Env_Kv_Vectors is new Langkit_Support.Vectors (Internal_Env_Kv);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Env_Kv_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Env_Kv
   is
      function Absolute_Get
        (T : Internal_Env_Kv_Array_Access; Index : Integer)
         return Internal_Env_Kv
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Env_Kv,
         Sequence_Type => Internal_Env_Kv_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Env_Kv;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Env_Kv;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Env_Kv_Array_Access) return Internal_Env_Kv_Array_Access is
      Ret : Internal_Env_Kv_Array_Access := Create_Internal_Env_Kv_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Env_Kv_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Env_Kv_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Env_Kv_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Env_Kv_Array (Items_Count : Natural) return Internal_Env_Kv_Array_Access
   is (if Items_Count = 0
       then No_Internal_Env_Kv_Array_Type
       else new Internal_Env_Kv_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Env_Kv_Array
     (Items : Internal_Internal_Env_Kv_Array) return Internal_Env_Kv_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Env_Kv_Array_Type;
      end if;

      return new Internal_Env_Kv_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Env_Kv_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Env_Kv_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Formal_Param_Vectors is new Langkit_Support.Vectors (Internal_Formal_Param);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Formal_Param_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Formal_Param
   is
      function Absolute_Get
        (T : Internal_Formal_Param_Array_Access; Index : Integer)
         return Internal_Formal_Param
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Formal_Param,
         Sequence_Type => Internal_Formal_Param_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Formal_Param;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Formal_Param;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Formal_Param_Array_Access) return Internal_Formal_Param_Array_Access is
      Ret : Internal_Formal_Param_Array_Access := Create_Internal_Formal_Param_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Formal_Param_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Formal_Param_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Formal_Param_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Formal_Param_Array (Items_Count : Natural) return Internal_Formal_Param_Array_Access
   is (if Items_Count = 0
       then No_Internal_Formal_Param_Array_Type
       else new Internal_Formal_Param_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Formal_Param_Array
     (Items : Internal_Internal_Formal_Param_Array) return Internal_Formal_Param_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Formal_Param_Array_Type;
      end if;

      return new Internal_Formal_Param_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Formal_Param_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Formal_Param_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Generic_Param_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Generic_Param_Assoc);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Generic_Param_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Generic_Param_Assoc
   is
      function Absolute_Get
        (T : Internal_Generic_Param_Assoc_Array_Access; Index : Integer)
         return Internal_Generic_Param_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Generic_Param_Assoc,
         Sequence_Type => Internal_Generic_Param_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Generic_Param_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Generic_Param_Assoc;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Generic_Param_Assoc_Array_Access) return Internal_Generic_Param_Assoc_Array_Access is
      Ret : Internal_Generic_Param_Assoc_Array_Access := Create_Internal_Generic_Param_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Generic_Param_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Generic_Param_Assoc_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Generic_Param_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Generic_Param_Assoc_Array (Items_Count : Natural) return Internal_Generic_Param_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Generic_Param_Assoc_Array_Type
       else new Internal_Generic_Param_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Generic_Param_Assoc_Array
     (Items : Internal_Internal_Generic_Param_Assoc_Array) return Internal_Generic_Param_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Generic_Param_Assoc_Array_Type;
      end if;

      return new Internal_Generic_Param_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Generic_Param_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Generic_Param_Assoc_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Infer_Instantiation_Vectors is new Langkit_Support.Vectors (Internal_Infer_Instantiation);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Infer_Instantiation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Infer_Instantiation
   is
      function Absolute_Get
        (T : Internal_Infer_Instantiation_Array_Access; Index : Integer)
         return Internal_Infer_Instantiation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Infer_Instantiation,
         Sequence_Type => Internal_Infer_Instantiation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Infer_Instantiation;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Infer_Instantiation;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Infer_Instantiation_Array_Access) return Internal_Infer_Instantiation_Array_Access is
      Ret : Internal_Infer_Instantiation_Array_Access := Create_Internal_Infer_Instantiation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Infer_Instantiation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Infer_Instantiation_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Infer_Instantiation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Infer_Instantiation_Array (Items_Count : Natural) return Internal_Infer_Instantiation_Array_Access
   is (if Items_Count = 0
       then No_Internal_Infer_Instantiation_Array_Type
       else new Internal_Infer_Instantiation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Infer_Instantiation_Array
     (Items : Internal_Internal_Infer_Instantiation_Array) return Internal_Infer_Instantiation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Infer_Instantiation_Array_Type;
      end if;

         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Infer_Instantiation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Infer_Instantiation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Infer_Instantiation_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   


   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Inner_Env_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Inner_Env_Assoc
   is
      function Absolute_Get
        (T : Internal_Inner_Env_Assoc_Array_Access; Index : Integer)
         return Internal_Inner_Env_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Inner_Env_Assoc,
         Sequence_Type => Internal_Inner_Env_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Inner_Env_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Inner_Env_Assoc;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Inner_Env_Assoc_Array_Access) return Internal_Inner_Env_Assoc_Array_Access is
      Ret : Internal_Inner_Env_Assoc_Array_Access := Create_Internal_Inner_Env_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Inner_Env_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Inner_Env_Assoc_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Inner_Env_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Inner_Env_Assoc_Array (Items_Count : Natural) return Internal_Inner_Env_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Inner_Env_Assoc_Array_Type
       else new Internal_Inner_Env_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Inner_Env_Assoc_Array
     (Items : Internal_Internal_Inner_Env_Assoc_Array) return Internal_Inner_Env_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Inner_Env_Assoc_Array_Type;
      end if;

      return new Internal_Inner_Env_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Inner_Env_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Inner_Env_Assoc_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   


   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Logic_Context_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Logic_Context
   is
      function Absolute_Get
        (T : Internal_Logic_Context_Array_Access; Index : Integer)
         return Internal_Logic_Context
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Logic_Context,
         Sequence_Type => Internal_Logic_Context_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Logic_Context;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Logic_Context;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Logic_Context_Array_Access) return Internal_Logic_Context_Array_Access is
      Ret : Internal_Logic_Context_Array_Access := Create_Internal_Logic_Context_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Logic_Context_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Logic_Context_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Logic_Context_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Logic_Context_Array (Items_Count : Natural) return Internal_Logic_Context_Array_Access
   is (if Items_Count = 0
       then No_Internal_Logic_Context_Array_Type
       else new Internal_Logic_Context_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Logic_Context_Array
     (Items : Internal_Internal_Logic_Context_Array) return Internal_Logic_Context_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Logic_Context_Array_Type;
      end if;

      return new Internal_Logic_Context_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Logic_Context_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Logic_Context_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Param_Match_Vectors is new Langkit_Support.Vectors (Internal_Param_Match);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Param_Match_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Param_Match
   is
      function Absolute_Get
        (T : Internal_Param_Match_Array_Access; Index : Integer)
         return Internal_Param_Match
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Param_Match,
         Sequence_Type => Internal_Param_Match_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Param_Match;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Param_Match;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Param_Match_Array_Access) return Internal_Param_Match_Array_Access is
      Ret : Internal_Param_Match_Array_Access := Create_Internal_Param_Match_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Param_Match_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Param_Match_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Param_Match_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Param_Match_Array (Items_Count : Natural) return Internal_Param_Match_Array_Access
   is (if Items_Count = 0
       then No_Internal_Param_Match_Array_Type
       else new Internal_Param_Match_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Param_Match_Array
     (Items : Internal_Internal_Param_Match_Array) return Internal_Param_Match_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Param_Match_Array_Type;
      end if;

      return new Internal_Param_Match_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Param_Match_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Param_Match_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Semantic_Result_Vectors is new Langkit_Support.Vectors (Internal_Semantic_Result);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Semantic_Result_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Semantic_Result
   is
      function Absolute_Get
        (T : Internal_Semantic_Result_Array_Access; Index : Integer)
         return Internal_Semantic_Result
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Semantic_Result,
         Sequence_Type => Internal_Semantic_Result_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Semantic_Result;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Semantic_Result;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Semantic_Result_Array_Access) return Internal_Semantic_Result_Array_Access is
      Ret : Internal_Semantic_Result_Array_Access := Create_Internal_Semantic_Result_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Semantic_Result_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Semantic_Result_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Semantic_Result_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Semantic_Result_Array (Items_Count : Natural) return Internal_Semantic_Result_Array_Access
   is (if Items_Count = 0
       then No_Internal_Semantic_Result_Array_Type
       else new Internal_Semantic_Result_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Semantic_Result_Array
     (Items : Internal_Internal_Semantic_Result_Array) return Internal_Semantic_Result_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Semantic_Result_Array_Type;
      end if;

         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Semantic_Result_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Semantic_Result_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Semantic_Result_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   


   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Solver_Diagnostic_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Solver_Diagnostic
   is
      function Absolute_Get
        (T : Internal_Solver_Diagnostic_Array_Access; Index : Integer)
         return Internal_Solver_Diagnostic
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Solver_Diagnostic,
         Sequence_Type => Internal_Solver_Diagnostic_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Solver_Diagnostic;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Solver_Diagnostic;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Solver_Diagnostic_Array_Access) return Internal_Solver_Diagnostic_Array_Access is
      Ret : Internal_Solver_Diagnostic_Array_Access := Create_Internal_Solver_Diagnostic_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Solver_Diagnostic_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Solver_Diagnostic_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Solver_Diagnostic_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Solver_Diagnostic_Array (Items_Count : Natural) return Internal_Solver_Diagnostic_Array_Access
   is (if Items_Count = 0
       then No_Internal_Solver_Diagnostic_Array_Type
       else new Internal_Solver_Diagnostic_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Solver_Diagnostic_Array
     (Items : Internal_Internal_Solver_Diagnostic_Array) return Internal_Solver_Diagnostic_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Solver_Diagnostic_Array_Type;
      end if;

         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Solver_Diagnostic_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Solver_Diagnostic_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Solver_Diagnostic_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Internal_Tree_Semantic_Result_Vectors is new Langkit_Support.Vectors (Internal_Tree_Semantic_Result);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Internal_Tree_Semantic_Result_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Tree_Semantic_Result
   is
      function Absolute_Get
        (T : Internal_Tree_Semantic_Result_Array_Access; Index : Integer)
         return Internal_Tree_Semantic_Result
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Tree_Semantic_Result,
         Sequence_Type => Internal_Tree_Semantic_Result_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Internal_Tree_Semantic_Result;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Tree_Semantic_Result;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Internal_Tree_Semantic_Result_Array_Access) return Internal_Tree_Semantic_Result_Array_Access is
      Ret : Internal_Tree_Semantic_Result_Array_Access := Create_Internal_Tree_Semantic_Result_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Tree_Semantic_Result_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Internal_Tree_Semantic_Result_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Tree_Semantic_Result_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Internal_Tree_Semantic_Result_Array (Items_Count : Natural) return Internal_Tree_Semantic_Result_Array_Access
   is (if Items_Count = 0
       then No_Internal_Tree_Semantic_Result_Array_Type
       else new Internal_Tree_Semantic_Result_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Internal_Tree_Semantic_Result_Array
     (Items : Internal_Internal_Tree_Semantic_Result_Array) return Internal_Tree_Semantic_Result_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Tree_Semantic_Result_Array_Type;
      end if;

         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Tree_Semantic_Result_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Internal_Tree_Semantic_Result_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Tree_Semantic_Result_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package Lexical_Env_Vectors is new Langkit_Support.Vectors (Lexical_Env);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Lexical_Env_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Lexical_Env
   is
      function Absolute_Get
        (T : Lexical_Env_Array_Access; Index : Integer)
         return Lexical_Env
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Lexical_Env,
         Sequence_Type => Lexical_Env_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Lexical_Env;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Empty_Env;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Lexical_Env_Array_Access) return Lexical_Env_Array_Access is
      Ret : Lexical_Env_Array_Access := Create_Lexical_Env_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Lexical_Env_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Lexical_Env_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Lexical_Env_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Lexical_Env_Array (Items_Count : Natural) return Lexical_Env_Array_Access
   is (if Items_Count = 0
       then No_Lexical_Env_Array_Type
       else new Lexical_Env_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Lexical_Env_Array
     (Items : Internal_Lexical_Env_Array) return Lexical_Env_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Lexical_Env_Array_Type;
      end if;

         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Lexical_Env_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Lexical_Env_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Lexical_Env_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   

      package String_Type_Vectors is new Langkit_Support.Vectors (String_Type);

   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : String_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return String_Type
   is
      function Absolute_Get
        (T : String_Type_Array_Access; Index : Integer)
         return String_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => String_Type,
         Sequence_Type => String_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : String_Type;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Empty_String;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : String_Type_Array_Access) return String_Type_Array_Access is
      Ret : String_Type_Array_Access := Create_String_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;

      ------------------
      -- Join_Strings --
      ------------------

      function Join_Strings
        (Separator : String_Type;
         Strings   : String_Type_Array_Access) return String_Type
      is
         Separator_Length : constant Natural := Separator.Length;
         Length           : Natural := 0;
         First            : Boolean;
      begin
         --  First, compute the length of the result: the sum of all string
         --  lengths in Strings.
         First := True;
         for S of Strings.Items loop
            if First then
               First := False;
            else
               Length := Length + Separator_Length;
            end if;
            Length := Length + S.Length;
         end loop;

         --  Create the result string with the correct length. Do not use our
         --  constructor and initialize the content in-place, to avoid extra
         --  copies.
         return Result : constant String_Type :=
            new String_Record (Length)
         do
            Result.Ref_Count := 1;

            --  Now copy the content of all strings into the result
            declare
               Last : Natural := 0;
            begin
               First := True;
               for S of Strings.Items loop
                  if First then
                     First := False;
                  else
                     Result.Content (Last + 1 .. Last + Separator_Length) :=
                        Separator.Content;
                     Last := Last + Separator_Length;
                  end if;
                  Result.Content (Last + 1 .. Last + S.Length) := S.Content;
                  Last := Last + S.Length;
               end loop;
            end;
         end return;
      end Join_Strings;

   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : String_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : String_Type_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out String_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_String_Type_Array (Items_Count : Natural) return String_Type_Array_Access
   is (if Items_Count = 0
       then No_String_Type_Array_Type
       else new String_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_String_Type_Array
     (Items : Internal_String_Type_Array) return String_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_String_Type_Array_Type;
      end if;

         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new String_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : String_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : String_Type_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;




      

   


   ---------
   -- Get --
   ---------

   function Get
     (Node    : Bare_Lkt_Node;
      T       : Symbol_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Symbol_Type
   is
      function Absolute_Get
        (T : Symbol_Type_Array_Access; Index : Integer)
         return Symbol_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Symbol_Type,
         Sequence_Type => Symbol_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Symbol_Type;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Symbol;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;

   ------------
   -- Concat --
   ------------

   function Concat (L, R : Symbol_Type_Array_Access) return Symbol_Type_Array_Access is
      Ret : Symbol_Type_Array_Access := Create_Symbol_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;


   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Symbol_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   ------------
   -- Length --
   ------------

   function Length (T : Symbol_Type_Array_Access) return Natural is (T.N);

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Symbol_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

   function Create_Symbol_Type_Array (Items_Count : Natural) return Symbol_Type_Array_Access
   is (if Items_Count = 0
       then No_Symbol_Type_Array_Type
       else new Symbol_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));


   function Create_Symbol_Type_Array
     (Items : Internal_Symbol_Type_Array) return Symbol_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Symbol_Type_Array_Type;
      end if;

      return new Symbol_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (L, R : Symbol_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;

      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;

      return True;
   end Equivalent;


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Symbol_Type_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;





         

   

   ----------
   -- Next --
   ----------

   function Next
     (T       : Bare_Lkt_Node_Iterator_Access;
      Element : out Bare_Lkt_Node) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);

      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;

   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Bare_Lkt_Node_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Bare_Lkt_Node_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Bare_Lkt_Node_Iterator_Access) return String is
      begin
         return "<Iterator of LktNode, index="
                & A.Index'Image & ">";
      end Trace_Image;


         

   

   ----------
   -- Next --
   ----------

   function Next
     (T       : Internal_Entity_Iterator_Access;
      Element : out Internal_Entity) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);

      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;

   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Entity_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Entity_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Entity_Iterator_Access) return String is
      begin
         return "<Iterator of LktNode.entity, index="
                & A.Index'Image & ">";
      end Trace_Image;


         

   

   ----------
   -- Next --
   ----------

   function Next
     (T       : Internal_Inner_Env_Assoc_Iterator_Access;
      Element : out Internal_Inner_Env_Assoc) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);

      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;

   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (T : Internal_Inner_Env_Assoc_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (T : in out Internal_Inner_Env_Assoc_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;

      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (A : Internal_Inner_Env_Assoc_Iterator_Access) return String is
      begin
         return "<Iterator of InnerEnvAssoc, index="
                & A.Index'Image & ">";
      end Trace_Image;



   ---------
   -- "<" --
   ---------

   function "<" (Left, Right : Internal_Unit) return Boolean is
   begin
      return Left.Filename < Right.Filename;
   end "<";

      



function Hash (Key : Mmz_Key_Item) return Hash_Type;
function Equivalent (L, R : Mmz_Key_Item) return Boolean;
procedure Destroy (Key : in out Mmz_Key_Array_Access);

----------------
-- Equivalent --
----------------

function Equivalent (L, R : Mmz_Key_Item) return Boolean is
begin
   if L.Kind /= R.Kind then
      return False;
   end if;

   case L.Kind is
         when Mmz_Bare_Call_Expr =>
            
               return L.As_Bare_Call_Expr = R.As_Bare_Call_Expr;
         when Mmz_Bare_Generic_Decl =>
            
               return L.As_Bare_Generic_Decl = R.As_Bare_Generic_Decl;
         when Mmz_Bare_Instantiated_Generic_Type =>
            
               return L.As_Bare_Instantiated_Generic_Type = R.As_Bare_Instantiated_Generic_Type;
         when Mmz_Bare_Lambda_Arg_Decl =>
            
               return L.As_Bare_Lambda_Arg_Decl = R.As_Bare_Lambda_Arg_Decl;
         when Mmz_Bare_Langkit_Root =>
            
               return L.As_Bare_Langkit_Root = R.As_Bare_Langkit_Root;
         when Mmz_Bare_Lexer_Decl =>
            
               return L.As_Bare_Lexer_Decl = R.As_Bare_Lexer_Decl;
         when Mmz_Bare_Lkt_Node =>
            
               return L.As_Bare_Lkt_Node = R.As_Bare_Lkt_Node;
         when Mmz_Bare_Type_Decl =>
            
               return L.As_Bare_Type_Decl = R.As_Bare_Type_Decl;
         when Mmz_Bare_Type_Decl_Array_Access =>
            
               return Equivalent (L.As_Bare_Type_Decl_Array_Access, R.As_Bare_Type_Decl_Array_Access);
         when Mmz_Boolean =>
            
               return L.As_Boolean = R.As_Boolean;
         when Mmz_Env_Rebindings =>
            
               return L.As_Env_Rebindings = R.As_Env_Rebindings;
         when Mmz_Internal_Entity_Info =>
            
               return L.As_Internal_Entity_Info = R.As_Internal_Entity_Info;
         when Mmz_Internal_Metadata =>
            
               return L.As_Internal_Metadata = R.As_Internal_Metadata;
         when Mmz_Symbol_Type =>
            
               return L.As_Symbol_Type = R.As_Symbol_Type;
   end case;
end Equivalent;

----------
-- Hash --
----------

function Hash (Key : Mmz_Key_Item) return Hash_Type is
begin
   case Key.Kind is
         when Mmz_Bare_Call_Expr =>
            return Hash (Key.As_Bare_Call_Expr);
         when Mmz_Bare_Generic_Decl =>
            return Hash (Key.As_Bare_Generic_Decl);
         when Mmz_Bare_Instantiated_Generic_Type =>
            return Hash (Key.As_Bare_Instantiated_Generic_Type);
         when Mmz_Bare_Lambda_Arg_Decl =>
            return Hash (Key.As_Bare_Lambda_Arg_Decl);
         when Mmz_Bare_Langkit_Root =>
            return Hash (Key.As_Bare_Langkit_Root);
         when Mmz_Bare_Lexer_Decl =>
            return Hash (Key.As_Bare_Lexer_Decl);
         when Mmz_Bare_Lkt_Node =>
            return Hash (Key.As_Bare_Lkt_Node);
         when Mmz_Bare_Type_Decl =>
            return Hash (Key.As_Bare_Type_Decl);
         when Mmz_Bare_Type_Decl_Array_Access =>
            return Hash (Key.As_Bare_Type_Decl_Array_Access);
         when Mmz_Boolean =>
            return Hash (Key.As_Boolean);
         when Mmz_Env_Rebindings =>
            return Hash (Key.As_Env_Rebindings);
         when Mmz_Internal_Entity_Info =>
            return Hash (Key.As_Internal_Entity_Info);
         when Mmz_Internal_Metadata =>
            return Hash (Key.As_Internal_Metadata);
         when Mmz_Symbol_Type =>
            return Hash (Key.As_Symbol_Type);
   end case;
end Hash;

----------
-- Hash --
----------

function Hash (Key : Mmz_Key) return Hash_Type is
   Result : Hash_Type := Mmz_Property'Pos (Key.Property);
begin
   for K of Key.Items.all loop
      Result := Combine (Result, Hash (K));
   end loop;
   return Result;
end Hash;

----------------
-- Equivalent --
----------------

function Equivalent (L, R : Mmz_Key) return Boolean is
   L_Items : Mmz_Key_Array renames L.Items.all;
   R_Items : Mmz_Key_Array renames R.Items.all;
begin
   if L.Property /= R.Property or else L_Items'Length /= R_Items'Length then
      return False;
   end if;

   for I in L_Items'Range loop
      if not Equivalent (L_Items (I), R_Items (I)) then
         return False;
      end if;
   end loop;

   return True;
end Equivalent;

-------------
-- Destroy --
-------------

procedure Destroy (Map : in out Memoization_Maps.Map) is
   use Memoization_Maps;

   --  We need keys and values to be valid when clearing the memoization map,
   --  but on the other hand we need to free keys and values as well. To
   --  achieve both goals, we first copy key and values into arrays, then we
   --  clear the map, and then we free keys/values in the arrays. Allocate both
   --  arrays on the heap to avoid stack overflow, as they can be quite big.

   Length : constant Natural := Natural (Map.Length);

   type Key_Array is array (1 .. Length) of Mmz_Key_Array_Access;
   type Key_Array_Access is access Key_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Key_Array, Key_Array_Access);

   type Value_Array is array (1 .. Length) of Mmz_Value;
   type Value_Array_Access is access Value_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Value_Array, Value_Array_Access);

   Keys   : Key_Array_Access := new Key_Array;
   Values : Value_Array_Access := new Value_Array;
   I      : Positive := 1;
begin
   for Cur in Map.Iterate loop
      Keys (I) := Key (Cur).Items;
      Values (I) := Element (Cur);
      I := I + 1;
   end loop;

   Map.Clear;

   for K_Array of Keys.all loop
      Destroy (K_Array);
   end loop;

   
   for V of Values.all loop
      case V.Kind is
         when Mmz_Error =>
            Free_Memoized_Error (V.Exc_Id, V.Exc_Msg);

            when Mmz_Internal_Entity_Type_Decl_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Type_Decl_Array_Access);
            when Mmz_Internal_Env_Assoc_Array_Access =>
               Dec_Ref (V.As_Internal_Env_Assoc_Array_Access);
            when Mmz_Internal_Generic_Param_Assoc_Array_Access =>
               Dec_Ref (V.As_Internal_Generic_Param_Assoc_Array_Access);
            when Mmz_Internal_Param_Match_Array_Access =>
               Dec_Ref (V.As_Internal_Param_Match_Array_Access);
            when Mmz_Internal_Semantic_Result_Array_Access =>
               Dec_Ref (V.As_Internal_Semantic_Result_Array_Access);
            when Mmz_Lexical_Env =>
               Dec_Ref (V.As_Lexical_Env);

         when others => null;
      end case;
   end loop;

   Free (Keys);
   Free (Values);
end Destroy;

-------------
-- Destroy --
-------------

procedure Destroy (Key : in out Mmz_Key_Array_Access) is
   procedure Free is new Ada.Unchecked_Deallocation
     (Mmz_Key_Array, Mmz_Key_Array_Access);
begin
   

      for K of Key.all loop
         case K.Kind is
               when Mmz_Bare_Type_Decl_Array_Access =>
                  Dec_Ref (K.As_Bare_Type_Decl_Array_Access);

            when others => null;
         end case;
      end loop;
   Free (Key);
end Destroy;

-------------------------
-- Find_Memoized_Value --
-------------------------

function Find_Memoized_Value
  (Unit       : Internal_Unit;
   Handle     : out Memoization_Handle;
   Value      : out Mmz_Value;
   Create_Key : access function return Mmz_Key) return Boolean
is
   Inserted : Boolean;
begin
   --  Make sure that we don't lookup stale caches
   Reset_Caches (Unit);

   --  Initialize handle: create the key and create a cursor pointing to an
   --  existing entry.
   Handle.Key := Create_Key.all;
   Handle.Cache_Version := Unit.Cache_Version;
   Value := (Kind => Mmz_Evaluating);
   Unit.Memoization_Map.Insert (Handle.Key, Value, Handle.Cur, Inserted);

   --  No existing entry yet? The above just created one. Otherwise, destroy
   --  our key and reuse the existing entry's.
   if not Inserted then
      Destroy (Handle.Key.Items);
      Handle.Key := Memoization_Maps.Key (Handle.Cur);
      Value := Memoization_Maps.Element (Handle.Cur);
   end if;

   return not Inserted;
end Find_Memoized_Value;

------------------------
-- Add_Memoized_Value --
------------------------

procedure Add_Memoized_Value
  (Unit   : Internal_Unit;
   Handle : in out Memoization_Handle;
   Value  : Mmz_Value;
   Stored : out Boolean) is
begin
   --  If Handle was created using a memoization map that has been since then
   --  reset, do nothing: the result can be partly stale due to the event that
   --  triggered the memoization tables reset.

   Stored := Unit.Cache_Version <= Handle.Cache_Version;
   if Stored then
      Unit.Memoization_Map.Replace_Element (Handle.Cur, Value);
   end if;
end Add_Memoized_Value;

------------------------
-- Add_Memoized_Error --
------------------------

procedure Add_Memoized_Error
  (Unit   : Internal_Unit;
   Handle : in out Memoization_Handle;
   Exc    : Ada.Exceptions.Exception_Occurrence;
   Stored : out Boolean)
is
   Value : Mmz_Value (Kind => Mmz_Error);
begin
   Store_Memoized_Error (Exc, Value.Exc_Id, Value.Exc_Msg);
   Add_Memoized_Value (Unit, Handle, Value, Stored);
   if not Stored then
      Free_Memoized_Error (Value.Exc_Id, Value.Exc_Msg);
   end if;
end Add_Memoized_Error;

----------------------------
-- Reraise_Memoized_Error --
----------------------------

procedure Reraise_Memoized_Error (Value : Mmz_Value) is
begin
   Reraise_Memoized_Error (Value.Exc_Id, Value.Exc_Msg);
end Reraise_Memoized_Error;

--------------------------
-- Store_Memoized_Error --
--------------------------

procedure Store_Memoized_Error
  (Exc     : Ada.Exceptions.Exception_Occurrence;
   Exc_Id  : out Ada.Exceptions.Exception_Id;
   Exc_Msg : out String_Access) is
begin
   Exc_Id := Ada.Exceptions.Exception_Identity (Exc);
   Exc_Msg := new String'(Ada.Exceptions.Exception_Message (Exc));
end Store_Memoized_Error;

-------------------------
-- Free_Memoized_Error --
-------------------------

procedure Free_Memoized_Error
  (Exc_Id  : in out Ada.Exceptions.Exception_Id;
   Exc_Msg : in out String_Access)
is
   pragma Unreferenced (Exc_Id);
begin
   Free (Exc_Msg);
end Free_Memoized_Error;

----------------------------
-- Reraise_Memoized_Error --
----------------------------

procedure Reraise_Memoized_Error
  (Exc_Id  : Ada.Exceptions.Exception_Id;
   Exc_Msg : String_Access) is
begin
   Ada.Exceptions.Raise_Exception (Exc_Id, Exc_Msg.all & " (memoized)");
end Reraise_Memoized_Error;



   package Solver_Diagnostic_Vectors is new Langkit_Support.Vectors
     (Internal_Solver_Diagnostic);

   ----------------------------
   -- Allocate_Logic_Context --
   ----------------------------

   function Allocate_Logic_Context
     (Ctx : Internal_Logic_Context) return Internal_Logic_Context_Access
   is (new Internal_Logic_Context'(Ctx));

   -------------------------
   -- Trace_Logic_Context --
   -------------------------

   function Trace_Logic_Context
     (Ctx : Internal_Logic_Context_Access) return String
   is (Trace_Image (Ctx.all));

   -----------------
   -- Deep_Equals --
   -----------------

   function Deep_Equals
     (X, Y : Internal_Logic_Context_Access) return Boolean
   is (X.all = Y.all);

   ------------------------
   -- Free_Logic_Context --
   ------------------------

   procedure Free_Logic_Context
     (Ctx : in out Internal_Logic_Context_Access)
   is
      procedure Free is new Ada.Unchecked_Deallocation
        (Internal_Logic_Context, Internal_Logic_Context_Access);
   begin
      Free (Ctx);
   end Free_Logic_Context;

   -------------------
   -- Solve_Wrapper --
   -------------------

   function Solve_Wrapper
     (R            : Solver.Relation;
      Context_Node : Bare_Lkt_Node) return Boolean is
   begin
      if Context_Node /= null and then Langkit_Support.Adalog.Debug.Debug then
         Assign_Names_To_Logic_Vars (Context_Node);
      end if;

      begin
         return Solver.Solve_First
           (R, Timeout => Context_Node.Unit.Context.Logic_Resolution_Timeout);
      exception
         when Langkit_Support.Adalog.Early_Binding_Error =>
            Raise_Property_Exception
              (Context_Node,
               Property_Error'Identity,
               "invalid equation for logic resolution");
         when Langkit_Support.Adalog.Timeout_Error =>
            Raise_Property_Exception
              (Context_Node,
               Property_Error'Identity,
               "logic resolution timed out");
      end;
   end Solve_Wrapper;

   ----------------------------
   -- Solve_With_Diagnostics --
   ----------------------------

   function Solve_With_Diagnostics
     (R            : Solver.Relation;
      Context_Node : Bare_Lkt_Node) return Internal_Solver_Result
   is
      Ret : Internal_Solver_Result :=
        (True, No_Internal_Solver_Diagnostic_Array_Type);

      Acc : Solver_Diagnostic_Vectors.Vector;
      --  Vectors that will accumulate diagnostic emitted during resolution

      procedure Emit_Diagnostic (Diag : Internal_Solver_Diagnostic) is
      begin
         Acc.Append (Diag);
      end Emit_Diagnostic;
   begin
      Ret.Success := Solve_Wrapper (R, Context_Node);

      if not Ret.Success then
         Ret.Success := Solver.Solve_First
           (R,
            Solve_Options => (Report_Errors => True),
            Diag_Emitter => Emit_Diagnostic'Unrestricted_Access,
            Timeout => Context_Node.Unit.Context.Logic_Resolution_Timeout);
         Ret.Diagnostics := Create_Internal_Solver_Diagnostic_Array
           (Acc.Length);
         for I in 1 .. Acc.Length loop
            Ret.Diagnostics.Items (I) := Acc.Get (I);
         end loop;
         Acc.Destroy;
      end if;
      return Ret;
   end Solve_With_Diagnostics;

   -------------
   -- Destroy --
   -------------

   procedure Destroy (Env : in out Lexical_Env_Access) is
      Mutable_Env : Lexical_Env :=
        (Wrap (Env), 0, Env.Kind, No_Generic_Unit, 0);
   begin
      Destroy (Mutable_Env);
      Env := null;
   end Destroy;

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize
     (Self              : Bare_Lkt_Node;
      Kind              : Lkt_Node_Kind_Type;
      Unit              : Internal_Unit;
      Token_Start_Index : Token_Index;
      Token_End_Index   : Token_Index;
      Parent            : Bare_Lkt_Node := null;
      Self_Env          : Lexical_Env := AST_Envs.Empty_Env) is
   begin
      pragma Unreferenced (Kind);
      Self.Parent := Parent;
      Self.Unit := Unit;

      Self.Token_Start_Index := Token_Start_Index;
      Self.Token_End_Index := Token_End_Index;

      Self.Self_Env := Self_Env;
      Self.Last_Attempted_Child := -1;

      

   end Initialize;

   --------------------
   -- Use_Direct_Env --
   --------------------

   procedure Use_Direct_Env (State : in out PLE_Node_State; Env : Lexical_Env)
   is
   begin
      State.Current_Env := Env;
      State.Current_NED := null;
   end Use_Direct_Env;

   -------------------
   -- Use_Named_Env --
   -------------------

   procedure Use_Named_Env
     (State   : in out PLE_Node_State;
      Context : Internal_Context;
      Name    : Symbol_Type) is
   begin
      State.Current_NED := Get_Named_Env_Descriptor (Context, Name);
      State.Current_Env := State.Current_NED.Env_With_Precedence;
   end Use_Named_Env;

   ---------------------
   -- Set_Initial_Env --
   ---------------------

   procedure Set_Initial_Env
     (Self         : Bare_Lkt_Node;
      State        : in out PLE_Node_State;
      Env          : Internal_Designated_Env;
      DSL_Location : String) is
   begin
      case Env.Kind is
         when None =>
            Use_Direct_Env (State, Empty_Env);

         when Current_Env =>
            null;

         when Named_Env =>
            Use_Named_Env (State, Self.Unit.Context, Env.Env_Name);

         when Direct_Env =>

            --  Sanitize this environment value: make sure it's a non-foreign
            --  and primary environment.

            if Env.Direct_Env.Kind /= Static_Primary then
               Raise_Property_Exception
                 (Self,
                  Property_Error'Identity,
                  "Cannot set an env that is not static-primary as the"
                  & " initial env");

            elsif Is_Foreign_Strict (Env.Direct_Env, Self) then
               Raise_Property_Exception
                 (Self,
                  Property_Error'Identity,
                  "unsound foreign environment in SetInitialEnv ("
                  & DSL_Location & ")");
            end if;
            Use_Direct_Env (State, Env.Direct_Env);
      end case;
   end Set_Initial_Env;

   ----------------
   -- Add_To_Env --
   ----------------

   procedure Add_To_Env
     (Self         : Bare_Lkt_Node;
      State        : PLE_Node_State;
      Key          : Symbol_Type;
      Value        : Bare_Lkt_Node;
      Md           : Internal_Metadata;
      Resolver     : Entity_Resolver;
      Dest_Env     : Internal_Designated_Env;
      DSL_Location : String)
   is
      Context    : constant Internal_Context := Self.Unit.Context;
      Root_Scope : Lexical_Env renames Context.Root_Scope;
      --  Shortcuts

      Actual_Dest_Env : Lexical_Env;
      Dest_NED        : Named_Env_Descriptor_Access;
      --  Description for the destination environment
   begin
      --  Skip the env addition if explicitly requested

      if Key = No_Symbol
         or else Value = null
         or else (case Dest_Env.Kind is
                  when None        => True,
                  when Current_Env => False,
                  when Named_Env   => Dest_Env.Env_Name = No_Symbol,
                  when Direct_Env  => Dest_Env.Direct_Env = Empty_Env)
      then
         return;
      end if;

      if Value.Unit /= Self.Unit then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "Cannot add_to_env an AST node that comes from another analysis"
            & " unit");
      end if;

      

      --  Then determine the destination environment

      case Dest_Env.Kind is
         when None =>
            raise Program_Error with "unreachable code";

         when Current_Env =>
            --  Just use the current environment
            Dest_NED := State.Current_NED;
            Actual_Dest_Env := State.Current_Env;

         when Named_Env =>
            --  There is an environment name: just lookup the corresponding
            --  NED/env.
            Dest_NED := Get_Named_Env_Descriptor (Context, Dest_Env.Env_Name);
            Actual_Dest_Env := Dest_NED.Env_With_Precedence;

         when Direct_Env =>
            --  There is an explicit destination environment
            Dest_NED := null;
            Actual_Dest_Env := Dest_Env.Direct_Env;
      end case;

      --  Sanitize it

      if Actual_Dest_Env.Kind /= Static_Primary then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "Cannot add elements to a lexical env that is not static-primary");

      elsif
         --  Since lexical envs need to sort the foreign nodes they contain,
         --  and that the total order on nodes is not defined for synthetic
         --  nodes, it is not possible to add a synthetic node to a foreign
         --  lexical environment.
         --
         --  This reasoning applies to environments that belong to foreign
         --  units, but also to the root environment.
         Is_Foreign (Actual_Dest_Env, Self) and then Is_Synthetic (Value)
      then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "Cannot add a synthetic node to a lexical env from another"
            & " analysis unit");

      elsif
         --  Reject direct references to foreign destination environments.
         --
         --  This is an attempt at identifying uses of the unsound relocation
         --  mechanism (as opposed to named environments), so this applies to
         --  all foreign environments (root scope included).
         DSL_Location'Length > 0
         and then Dest_Env.Kind = Direct_Env
         and then Is_Foreign_Strict (Actual_Dest_Env, Self)
      then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "unsound foreign environment in AddToEnv (" & DSL_Location & ")");
      end if;

      --  Now that everything is sanitized, we can proceed with the actual
      --  key/value pair addition. Note that this does nothing if
      --  Actual_Dest_Env ended up empty.
      Add (Actual_Dest_Env, Thin (Key), Value, Md, Resolver);

      --  If we're adding the element to an environment by env name, we must
      --  register this association in two places: in the target named env
      --  entry, and in Value's unit.
      if Dest_NED /= null then
         declare
            use NED_Assoc_Maps;

            FN    : Map renames Dest_NED.Foreign_Nodes;
            Dummy : Boolean;
            Cur   : Cursor;
         begin
            FN.Insert (Key      => Key,
                       New_Item => Internal_Map_Node_Vectors.Empty_Vector,
                       Position => Cur,
                       Inserted => Dummy);
            declare
               V : Internal_Map_Node_Vectors.Vector renames
                  FN.Reference (Cur);
            begin
               V.Append ((Value, null, Md, Resolver));
               --  Note that the rebindings field is unused by the relocation
               --  mechanism (since we don't even allow adding env entries
               --  with custom rebindings), hence we simply leave it to null.
            end;
         end;
         Value.Unit.Exiled_Entries_In_NED.Append ((Dest_NED, Key, Value));

      --  Otherwise, if we're adding the element to an environment that belongs
      --  to a different unit, or to the root scope, then...
      elsif Is_Foreign_Not_Empty (Actual_Dest_Env, Self) then
         --  Add the Key/Value association to the list of entries contained in
         --  other units, so we can remove them when reparsing Value's unit.
         Value.Unit.Exiled_Entries.Append ((Actual_Dest_Env, Key, Value));

         if Actual_Dest_Env /= Root_Scope then
            --  Add Val to the list of foreign nodes that Actual_Dest_Env's
            --  unit contains, so that when that unit is reparsed, we can call
            --  Add_To_Env again on those nodes.
            Convert_Unit (Actual_Dest_Env.Owner).Foreign_Nodes.Append
              ((Value, Self.Unit));
         end if;
      end if;
   end Add_To_Env;

   -------------
   -- Ref_Env --
   -------------

   procedure Ref_Env
     (Self                : Bare_Lkt_Node;
      Dest_Env            : Lexical_Env;
      Ref_Env_Nodes       : in out Bare_Lkt_Node_Array_Access;
      Resolver            : Lexical_Env_Resolver;
      Kind                : Ref_Kind;
      Cats                : Ref_Categories;
      Shed_Rebindings     : Boolean) is
   begin
      for N of Ref_Env_Nodes.Items loop
         if N /= null then
            if N.Unit /= Self.Unit then
               Raise_Property_Exception
                 (Self,
                  Property_Error'Identity,
                  "attempt to add a referenced environment to a foreign unit");
            end if;
            Reference (Dest_Env, N, Resolver, Kind, Cats, Shed_Rebindings);
         end if;
      end loop;
      Dec_Ref (Ref_Env_Nodes);
   end Ref_Env;

   -------------
   -- Add_Env --
   -------------

   procedure Add_Env
     (Self              : Bare_Lkt_Node;
      State             : in out PLE_Node_State;
      No_Parent         : Boolean;
      Transitive_Parent : Boolean;
      Names             : in out Symbol_Type_Array_Access)
   is
      Parent_From_Name : constant Boolean := State.Current_NED /= null;
      --  Does the parent environment comes from a named environment lookup?

      --  Determine the parent of this new environment:
      --
      --  (1) no parent if requested;
      --  (2) the current environment as the static parent if it comes from a
      --      named env lookup or if it is not foreign (or is the empty/root
      --      environment).
      Parent : constant Lexical_Env :=
        (if No_Parent
         then Null_Lexical_Env
         else State.Current_Env);
   begin
      --  Create the environment itself
      Self.Self_Env := Create_Static_Lexical_Env
        (Parent            => Parent,
         Node              => Self,
         Transitive_Parent => Transitive_Parent,
         Sym_Table         => Self.Unit.Context.Symbols);

      --  If the parent of this new environment comes from a named environment
      --  lookup, register this new environment so that its parent is updated
      --  when the precence for this named environment changes.
      if Parent_From_Name then
         declare
            NED : constant Named_Env_Descriptor_Access := State.Current_NED;
         begin
            Self.Unit.Exiled_Envs.Append ((NED, Self.Self_Env));
            NED.Foreign_Envs.Insert (Self, Self.Self_Env);
         end;
      end if;

      --  From now on, the current environment is Self.Self_Env, with a direct
      --  access to it. It does not go through the env naming scheme, since
      --  only this node and its children (i.e. non-foreign nodes) will access
      --  it as a "current" environment during PLE.
      Use_Direct_Env (State, Self.Self_Env);

      --  Register the environment we just created on all the requested names
      if Names /= null then
         declare
            Context   : constant Internal_Context := Self.Unit.Context;
            Env       : constant Lexical_Env := Self.Self_Env;
            NENU      : NED_Maps.Map renames
               State.Unit_State.Named_Envs_Needing_Update;
         begin
            for N of Names.Items loop
               Register_Named_Env (Context, N, Env, NENU);
            end loop;
            Dec_Ref (Names);
         end;
      end if;
   end Add_Env;

   ---------------------
   -- Pre_Env_Actions --
   ---------------------

   procedure Pre_Env_Actions
     (Self            : Bare_Lkt_Node;
      State           : in out PLE_Node_State;
      Add_To_Env_Only : Boolean := False) is
   begin

      
   

   case Self.Kind is
            when Lkt_Match_Branch =>
            
            Match_Branch_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Decl_Block =>
            
            Decl_Block_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Langkit_Root =>
            
            Langkit_Root_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Import =>
            
            Import_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Lambda_Expr =>
            
            Lambda_Expr_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Block_Expr =>
            
            Block_Expr_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Enum_Type_Decl =>
            
            Enum_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Enum_Class_Decl =>
            
            Enum_Class_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Enum_Class_Alt_Decl =>
            
            null;
      
            when Lkt_Lexer_Decl =>
            
            Lexer_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Grammar_Decl =>
            
            Grammar_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Generic_Decl =>
            
            Generic_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Fun_Decl =>
            
            Fun_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
            when Lkt_Enum_Lit_Decl =>
            
            null;
      
            when Lkt_Grammar_Rule_Decl .. Lkt_Self_Decl | Lkt_Field_Decl .. Lkt_Val_Decl | Lkt_Env_Spec_Decl | Lkt_Lexer_Family_Decl | Lkt_Function_Type .. Lkt_Class_Decl | Lkt_Struct_Decl .. Lkt_Trait_Decl =>
            
            Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      
      when others =>  null; 
   end case;


   end Pre_Env_Actions;

   ----------------------
   -- Post_Env_Actions --
   ----------------------

   procedure Post_Env_Actions
     (Self : Bare_Lkt_Node; State : in out PLE_Node_State) is
   begin
      
   

   case Self.Kind is
            when Lkt_Match_Branch =>
            
            null;
      
            when Lkt_Decl_Block =>
            
            null;
      
            when Lkt_Langkit_Root =>
            
            null;
      
            when Lkt_Import =>
            
            null;
      
            when Lkt_Lambda_Expr =>
            
            null;
      
            when Lkt_Block_Expr =>
            
            null;
      
            when Lkt_Enum_Type_Decl =>
            
            Enum_Type_Decl_Post_Env_Actions (Self, State);
      
            when Lkt_Enum_Class_Decl =>
            
            Enum_Class_Decl_Post_Env_Actions (Self, State);
      
            when Lkt_Enum_Class_Alt_Decl =>
            
            null;
      
            when Lkt_Lexer_Decl =>
            
            null;
      
            when Lkt_Grammar_Decl =>
            
            null;
      
            when Lkt_Generic_Decl =>
            
            null;
      
            when Lkt_Fun_Decl =>
            
            null;
      
            when Lkt_Enum_Lit_Decl =>
            
            null;
      
            when Lkt_Grammar_Rule_Decl .. Lkt_Self_Decl | Lkt_Field_Decl .. Lkt_Val_Decl | Lkt_Env_Spec_Decl | Lkt_Lexer_Family_Decl | Lkt_Function_Type .. Lkt_Class_Decl | Lkt_Struct_Decl .. Lkt_Trait_Decl =>
            
            null;
      
      when others =>  null; 
   end case;


   end Post_Env_Actions;

   ----------------
   -- Get_Symbol --
   ----------------

   function Get_Symbol
     (Node : Bare_Lkt_Node) return Symbol_Type is
   begin
      if Node = null then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "cannot get the symbol of a null node");
      end if;
      return Get_Symbol (Token (Node, Node.Token_Start_Index));
   end Get_Symbol;

   ----------
   -- Text --
   ----------

   function Text
     (Node : Bare_Lkt_Node) return Text_Type
   is
   begin
      if Node = null then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "cannot get the text of a null node");
      end if;

      declare
         Start_T : constant Token_Reference :=
            Token (Node, Node.Token_Start_Index);
         End_T   : constant Token_Reference :=
            Token (Node, Node.Token_End_Index);
      begin
         --  No text is associated to synthetic and ghost nodes

         if Is_Synthetic (Node) then
            return "";
         end if;

         if Is_Ghost (Node) then
            return "";
         end if;

         return Text (Start_T, End_T);
      end;
   end Text;

   ---------------------
   -- Is_Visible_From --
   ---------------------

   function Is_Visible_From
     (Self                     : Bare_Lkt_Node;
      Referenced_Env, Base_Env : Lexical_Env) return Boolean
   is
      Referenced_Unit : constant Internal_Unit :=
         Convert_Unit (Referenced_Env.Owner);
      Base_Unit       : constant Internal_Unit :=
         Convert_Unit (Base_Env.Owner);
   begin
      if Referenced_Unit = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "referenced environment does not belong to any analysis unit");
      elsif Base_Unit = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "base environment does not belong to any analysis unit");
      end if;
      return Is_Referenced_From (Referenced_Unit, Base_Unit);
   end Is_Visible_From;

   ----------
   -- Unit --
   ----------

   function Unit (Node : Bare_Lkt_Node) return Internal_Unit is
   begin
      return Node.Unit;
   end Unit;

   function Lookup_Internal
     (Node : Bare_Lkt_Node;
      Sloc : Source_Location) return Bare_Lkt_Node;
   procedure Lookup_Relative
     (Node       : Bare_Lkt_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Lkt_Node);
   --  Implementation helpers for the looking up process

   -----------------
   -- Set_Parents --
   -----------------

   procedure Set_Parents
     (Node, Parent : Bare_Lkt_Node)
   is
   begin
      if Node = null then
         return;
      end if;

      Node.Parent := Bare_Lkt_Node (Parent);

      for I in 1 .. Children_Count (Node) loop
         Set_Parents (Child (Node, I), Node);
      end loop;
   end Set_Parents;

   -------------
   -- Destroy --
   -------------

   procedure Destroy (Node : Bare_Lkt_Node) is
   begin
      if Node = null then
         return;
      end if;

      Free_User_Fields (Node);
      for I in 1 .. Children_Count (Node) loop
         Destroy (Child (Node, I));
      end loop;
   end Destroy;

   -----------
   -- Child --
   -----------

   function Child (Node  : Bare_Lkt_Node;
                   Index : Positive) return Bare_Lkt_Node
   is
      Result          : Bare_Lkt_Node;
      Index_In_Bounds : Boolean;
   begin
      Get_Child (Node, Index, Index_In_Bounds, Result);
      return Result;
   end Child;

   --------------
   -- Traverse --
   --------------

   function Traverse
     (Node  : Bare_Lkt_Node;
      Visit : access function (Node : Bare_Lkt_Node)
              return Visit_Status)
     return Visit_Status
   is
      Status : Visit_Status := Into;

   begin
      if Node /= null then
         Status := Visit (Node);

         --  Skip processing the child nodes if the returned status is Over
         --  or Stop. In the former case the previous call to Visit has taken
         --  care of processing the needed childs, and in the latter case we
         --  must immediately stop processing the tree.

         if Status = Into then
            for I in 1 .. Children_Count (Node) loop
               declare
                  Cur_Child : constant Bare_Lkt_Node :=
                     Child (Node, I);

               begin
                  if Cur_Child /= null then
                     Status := Traverse (Cur_Child, Visit);
                     exit when Status /= Into;
                  end if;
               end;
            end loop;
         end if;
      end if;

      if Status = Stop then
         return Stop;

      --  At this stage the Over status has no sense and we just continue
      --  processing the tree.

      else
         return Into;
      end if;
   end Traverse;

   --------------
   -- Traverse --
   --------------

   procedure Traverse
     (Node  : Bare_Lkt_Node;
      Visit : access function (Node : Bare_Lkt_Node)
                               return Visit_Status)
   is
      Result_Status : Visit_Status;
      pragma Unreferenced (Result_Status);
   begin
      Result_Status := Traverse (Node, Visit);
   end Traverse;

   ------------------------
   -- Traverse_With_Data --
   ------------------------

   function Traverse_With_Data
     (Node  : Bare_Lkt_Node;
      Visit : access function (Node : Bare_Lkt_Node;
                               Data : in out Data_Type)
                               return Visit_Status;
      Data  : in out Data_Type)
      return Visit_Status
   is
      function Helper (Node : Bare_Lkt_Node) return Visit_Status;

      ------------
      -- Helper --
      ------------

      function Helper (Node : Bare_Lkt_Node) return Visit_Status is
      begin
         return Visit (Node, Data);
      end Helper;

      Saved_Data : Data_Type;
      Result     : Visit_Status;

   begin
      if Reset_After_Traversal then
         Saved_Data := Data;
      end if;
      Result := Traverse (Node, Helper'Access);
      if Reset_After_Traversal then
         Data := Saved_Data;
      end if;
      return Result;
   end Traverse_With_Data;

   ----------------
   -- Sloc_Range --
   ----------------

   function Sloc_Range
     (Node : Bare_Lkt_Node) return Source_Location_Range
   is
      type Token_Anchor is (T_Start, T_End);
      type Token_Pos is record
         Pos    : Token_Index;
         Anchor : Token_Anchor;
      end record;

      TDH                    : Token_Data_Handler renames Node.Unit.TDH;
      Token_Start, Token_End : Token_Pos;

      function Get (Index : Token_Index) return Stored_Token_Data is
        (Get_Token (TDH, Index));

      function Sloc (T : Token_Pos) return Source_Location is
        (if T.Anchor = T_Start
         then Sloc_Start (TDH, Get (T.Pos))
         else Sloc_End (TDH, Get (T.Pos)));

   begin
      if Is_Synthetic (Node) then
         return (if Node.Parent = null
                 then No_Source_Location_Range
                 else Sloc_Range (Node.Parent));
      end if;

      if Is_Ghost (Node) then
         Token_Start := (if Node.Token_Start_Index = 1
                         then (1, T_Start)
                         else (Node.Token_Start_Index - 1, T_End));
         Token_End := Token_Start;
      else
         Token_Start := (Node.Token_Start_Index, T_Start);
         Token_End := (Node.Token_End_Index, T_End);
      end if;

      if Snaps_At_Start (Node)
         and then not Is_Ghost (Node)
         and then Token_Start.Pos /= 1
      then
         Token_Start := (Token_Start.Pos - 1, T_End);
      end if;

      if Snaps_At_End (Node) and then Token_End.Pos /= Last_Token (TDH) then
         Token_End := (Token_End.Pos + 1, T_Start);
      end if;

      return Make_Range (Sloc (Token_Start), Sloc (Token_End));
   end Sloc_Range;

   ------------
   -- Lookup --
   ------------

   function Lookup
     (Node : Bare_Lkt_Node;
      Sloc : Source_Location) return Bare_Lkt_Node
   is
      Position : Relative_Position;
      Result   : Bare_Lkt_Node;
   begin
      if Sloc = No_Source_Location then
         return null;
      end if;

      Lookup_Relative
        (Bare_Lkt_Node (Node), Sloc, Position, Result);
      return Result;
   end Lookup;

   ---------------------
   -- Lookup_Internal --
   ---------------------

   function Lookup_Internal
     (Node : Bare_Lkt_Node;
      Sloc : Source_Location) return Bare_Lkt_Node
   is
      --  For this implementation helper (i.e. internal primitive), we can
      --  assume that all lookups fall into this node's sloc range.
      pragma Assert (Compare (Sloc_Range (Node), Sloc) = Inside);

      Children : constant Internal_Bare_Lkt_Node_Array :=
         Implementation.Children (Node);
      Pos      : Relative_Position;
      Result   : Bare_Lkt_Node;
   begin
      --  Look for a child node that contains Sloc (i.e. return the most
      --  precise result).

      for Child of Children loop
         --  Note that we assume here that child nodes are ordered so that the
         --  first one has a sloc range that is before the sloc range of the
         --  second child node, etc.

         if Child /= null then
            Lookup_Relative (Child, Sloc, Pos, Result);
            case Pos is
               when Before =>
                   --  If this is the first node, Sloc is before it, so we can
                   --  stop here.  Otherwise, Sloc is between the previous
                   --  child node and the next one...  so we can stop here,
                   --  too.
                   return Node;

               when Inside =>
                   return Result;

               when After =>
                   --  Sloc is after the current child node, so see with the
                   --  next one.
                   null;
            end case;
         end if;
      end loop;

      --  If we reach this point, we found no children that covers Sloc, but
      --  Node still covers it (see the assertion).
      return Node;
   end Lookup_Internal;

   -------------
   -- Compare --
   -------------

   function Compare
     (Node : Bare_Lkt_Node;
      Sloc : Source_Location) return Relative_Position is
   begin
      return Compare (Sloc_Range (Node), Sloc);
   end Compare;

   ---------------------
   -- Lookup_Relative --
   ---------------------

   procedure Lookup_Relative
     (Node       : Bare_Lkt_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Lkt_Node)
   is
      Result : constant Relative_Position :=
        Compare (Node, Sloc);
   begin
      Position := Result;
      Node_Found := (if Result = Inside
                     then Lookup_Internal (Node, Sloc)
                     else null);
   end Lookup_Relative;

   -------------
   -- Compare --
   -------------

   function Compare
     (Self, Left, Right : Bare_Lkt_Node;
      Relation          : Comparison_Relation) return Boolean
   is
      LS, RS : Source_Location;
   begin
      if Left = null or else Right = null or else Left.Unit /= Right.Unit then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "invalid node comparison");
      end if;

      LS := Start_Sloc (Sloc_Range (Left));
      RS := Start_Sloc (Sloc_Range (Right));
      return (case Relation is
              when Langkit_Support.Types.Less_Than        => LS < RS,
              when Langkit_Support.Types.Less_Or_Equal    => LS <= RS,
              when Langkit_Support.Types.Greater_Than     => LS > RS,
              when Langkit_Support.Types.Greater_Or_Equal => LS >= RS);
   end Compare;

   --------------
   -- Children --
   --------------

   function Children
     (Node : Bare_Lkt_Node) return Internal_Bare_Lkt_Node_Array
   is
      First : constant Integer := Bare_Lkt_Node_Vectors.Index_Type'First;
      Last  : constant Integer := First + Children_Count (Node) - 1;
   begin
      return A : Internal_Bare_Lkt_Node_Array (First .. Last)
      do
         for I in First .. Last loop
            A (I) := Child (Node, I);
         end loop;
      end return;
   end Children;

   function Children
     (Node : Bare_Lkt_Node) return Bare_Lkt_Node_Array_Access
   is
      C : Internal_Bare_Lkt_Node_Array := Children (Node);
   begin
      return Ret : Bare_Lkt_Node_Array_Access :=
         Create_Bare_Lkt_Node_Array (C'Length)
      do
         Ret.Items := C;
      end return;
   end Children;

   ---------
   -- Get --
   ---------

   function Get
     (Self    : Bare_Lkt_Node;
      Node    : Bare_Lkt_Node_Base_List;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Lkt_Node
   is
      function Length (Node : Bare_Lkt_Node_Base_List) return Natural
      is (Node.Count);
      --  Wrapper around the Length primitive to get the compiler happy for the
      --  the package instantiation below.

      function Absolute_Get
        (L     : Bare_Lkt_Node_Base_List;
         Index : Integer) return Bare_Lkt_Node
      is (L.Nodes.all (Index + 1));
      --  L.Nodes is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Lkt_Node,
         Sequence_Type => Bare_Lkt_Node_Base_List,
         Length        => Length,
         Get           => Absolute_Get);

      Result : Bare_Lkt_Node;
   begin
      if Node = null and then Or_Null then
         return null;
      elsif Relative_Get (Node, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "out-of-bounds AST list access");
      end if;
   end Get;

   ---------------
   -- PP_Trivia --
   ---------------

   procedure PP_Trivia
     (Node        : Bare_Lkt_Node;
      Line_Prefix : String := "")
   is
      Children_Prefix : constant String := Line_Prefix & "|  ";
   begin
      if Node = null then
         Put_Line (Line_Prefix & "None");
         return;
      end if;
      Put_Line (Line_Prefix & Kind_Name (Node));
      for C of Children_And_Trivia (Node) loop
         case C.Kind is
            when Trivia =>
               Put_Line (Children_Prefix & Image (Text (C.Trivia)));
            when Child =>
               PP_Trivia (C.Node, Children_Prefix);
         end case;
      end loop;
   end PP_Trivia;

   --------------------------
   -- Populate_Lexical_Env --
   --------------------------

   function Populate_Lexical_Env (Node : Bare_Lkt_Node) return Boolean is

      Context    : constant Internal_Context := Node.Unit.Context;
      Unit_State : aliased PLE_Unit_State := (Named_Envs_Needing_Update => <>);
      Root_State : constant PLE_Node_State :=
        (Unit_State  => Unit_State'Unchecked_Access,
         Current_Env => Context.Root_Scope,
         Current_NED => null);

      function Populate_Internal
        (Node         : Bare_Lkt_Node;
         Parent_State : PLE_Node_State) return Boolean;
      --  Do the lexical env population on Node and recurse on its children

      procedure Register_Foreign_Env
        (Node : Bare_Lkt_Node; State : PLE_Node_State);
      --  Given a node and its PLE state, register Node.Self_Env as being
      --  initialized through the named environment mechanism, if that's indeed
      --  the case. Do nothing otherwise.

      -----------------------
      -- Populate_Internal --
      -----------------------

      function Populate_Internal
        (Node         : Bare_Lkt_Node;
         Parent_State : PLE_Node_State) return Boolean
      is
         Result : Boolean := False;
         State  : PLE_Node_State := Parent_State;
      begin
         if Node = null then
            return Result;
         end if;

         --  By default (i.e. unless env actions add a new env), the
         --  environment we store in Node is the current one.
         Node.Self_Env := State.Current_Env;

         --  Run pre/post actions, and run PLE on children in between. Make
         --  sure we register the potential foreign Node.Self_Env environment
         --  at the end, even when an exception interrupts PLE to keep the
         --  state consistent.
         begin
            Pre_Env_Actions (Node, State);
            if State.Current_Env /= Null_Lexical_Env then
               Node.Self_Env := State.Current_Env;
               Register_Foreign_Env (Node, State);
            end if;

            --  Call recursively on children
            for I in First_Child_Index (Node) .. Last_Child_Index (Node) loop
               Result := Populate_Internal
                 (Child (Node, I), State) or else Result;
            end loop;

            Post_Env_Actions (Node, State);
         exception
            when Exc : Property_Error =>
               if PLE_Errors_Trace.Is_Active then
                   GNATCOLL.Traces.Trace
                     (PLE_Errors_Trace,
                      "Exception raised during PLE "
                      & Ada.Exceptions.Exception_Name (Exc) & " : "
                      & Ada.Exceptions.Exception_Message (Exc));
                   GNATCOLL.Traces.Trace
                     (PLE_Errors_Trace,
                      GNAT.Traceback.Symbolic.Symbolic_Traceback (Exc));
               end if;
               Register_Foreign_Env (Node, State);
               return True;
         end;

         return Result;
      end Populate_Internal;

      --------------------------
      -- Register_Foreign_Env --
      --------------------------

      procedure Register_Foreign_Env
        (Node : Bare_Lkt_Node; State : PLE_Node_State) is
      begin
         if State.Current_NED /= null then
            State.Current_NED.Nodes_With_Foreign_Env.Insert (Node);
            Node.Unit.Nodes_With_Foreign_Env.Insert (Node, State.Current_NED);
         end if;
      end Register_Foreign_Env;

   begin
      --  This is intended to be called on the root node only (when there is no
      --  PLE root) or on a PLE root (child of the root node with a specific
      --  kind).
      if
         Node.Parent /= null
      then
         raise Program_Error;
      end if;

      return Result : constant Boolean :=
         Populate_Internal (Node, Root_State)
      do
         Update_Named_Envs (Context, Unit_State.Named_Envs_Needing_Update);
      end return;
   end Populate_Lexical_Env;

   ------------------------------
   -- AST_Envs_Node_Text_Image --
   ------------------------------

   function AST_Envs_Node_Text_Image
     (Node  : Bare_Lkt_Node;
      Short : Boolean := True) return Text_Type is
   begin
      if Short then
         if Node = null then
            return "null";
         end if;
         return To_Text (Basename (Node.Unit))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node))));
      else
         return Short_Text_Image (Node);
      end if;
   end AST_Envs_Node_Text_Image;

   -------------------
   -- Is_Rebindable --
   -------------------

   function Is_Rebindable (Node : Bare_Lkt_Node) return Boolean is
   begin
      
         return Node.Kind in Lkt_Generic_Decl;
   end Is_Rebindable;

   -----------------------
   -- Acquire_Rebinding --
   -----------------------

   function Acquire_Rebinding
     (Node             : Bare_Lkt_Node;
      Parent           : Env_Rebindings;
      Old_Env, New_Env : Lexical_Env) return Env_Rebindings
   is
      Result    : Env_Rebindings;
      Available : Env_Rebindings_Vectors.Vector renames
         Node.Unit.Context.Available_Rebindings;
   begin
      --  Use an existing and available Env_Rebindings_Type record for Node's
      --  Context, otherwise allocate a new rebinding.
      Result := (if Available.Is_Empty
                 then new Env_Rebindings_Type'(Version => 0, others => <>)
                 else Available.Pop);

      Result.Parent := Parent;
      Result.Old_Env := Old_Env;
      Result.New_Env := New_Env;
      Result.Children := Env_Rebindings_Vectors.Empty_Vector;
      return Result;
   end Acquire_Rebinding;

   -----------------------
   -- Release_Rebinding --
   -----------------------

   procedure Release_Rebinding (Self : in out Env_Rebindings) is
      Available : Env_Rebindings_Vectors.Vector renames
         Unwrap (Self.Old_Env).Node.Unit.Context.Available_Rebindings;
   begin
      --  Bumping the version number, to invalidate existing references to
      --  Self.
      Self.Version := Self.Version + 1;

      Self.Children.Destroy;
      Available.Append (Self);
      Self := null;
   end Release_Rebinding;

   ------------------------
   -- Register_Rebinding --
   ------------------------

   procedure Register_Rebinding
     (Node : Bare_Lkt_Node; Rebinding : Env_Rebindings) is
   begin
      Node.Unit.Rebindings.Append (Rebinding);
   end Register_Rebinding;


   --------------------
   -- Element_Parent --
   --------------------

   function Element_Parent
     (Node : Bare_Lkt_Node) return Bare_Lkt_Node
   is (Node.Parent);

   ---------------
   -- Node_Unit --
   ---------------

   function Node_Unit (Node : Bare_Lkt_Node) return Generic_Unit_Ptr is
   begin
      return Convert_Unit (Node.Unit);
   end Node_Unit;

   ----------
   -- Hash --
   ----------

   function Hash (Node : Bare_Lkt_Node) return Hash_Type
   is
      function H is new Hash_Access
        (Root_Node_Record, Bare_Lkt_Node);
   begin
      return H (Node);
   end Hash;

      function Hash (B : Boolean) return Hash_Type is (Boolean'Pos (B));





   ------------------------
   -- Named environments --
   ------------------------

   ---------
   -- Add --
   ---------

   procedure Add
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : AST_Envs.Internal_Map_Node)
   is
      use NED_Assoc_Maps;

      Pos   : Cursor;
      Dummy : Boolean;
   begin
      --  Make sure there is a vector entry for Key
      Self.Insert (Key, Internal_Map_Node_Vectors.Empty_Vector, Pos, Dummy);

      --  Append Node to that vector
      declare
         V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Pos);
      begin
         V.Append (Node);
      end;
   end Add;

   ------------
   -- Remove --
   ------------

   procedure Remove
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : Bare_Lkt_Node)
   is
      use NED_Assoc_Maps;

      V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Key);
   begin
      --  Remove the (assumed unique) entry in V whose node is Node. The order
      --  of items in V is not significant, so we can use Pop for efficient
      --  removal. Do the traversal in reverse order for correctness.
      for I in reverse 1 .. V.Length loop
         if V.Get_Access (I).Node = Node then
            V.Pop (I);
            exit;
         end if;
      end loop;
   end Remove;

   ------------------------------
   -- Get_Named_Env_Descriptor --
   ------------------------------

   function Get_Named_Env_Descriptor
     (Context : Internal_Context;
      Name    : Symbol_Type) return Named_Env_Descriptor_Access
   is
      use NED_Maps;

      --  Look for an existing entry for Name
      Pos : constant Cursor := Context.Named_Envs.Find (Name);
   begin
      if Has_Element (Pos) then
         return Element (Pos);
      end if;

      --  There is no such entry: create one
      return Result : constant Named_Env_Descriptor_Access :=
         new Named_Env_Descriptor'
           (Name                   => Name,
            Envs                   => <>,
            Env_With_Precedence    => Empty_Env,
            Foreign_Nodes          => <>,
            Foreign_Envs           => <>,
            Nodes_With_Foreign_Env => <>)
      do
         Context.Named_Envs.Insert (Name, Result);
      end return;
   end Get_Named_Env_Descriptor;

   ------------------------
   -- Register_Named_Env --
   ------------------------

   procedure Register_Named_Env
     (Context                   : Internal_Context;
      Name                      : Symbol_Type;
      Env                       : Lexical_Env;
      Named_Envs_Needing_Update : in out NED_Maps.Map)
   is
      NED_Access : constant Named_Env_Descriptor_Access :=
         Get_Named_Env_Descriptor (Context, Name);
      NED        : Named_Env_Descriptor renames NED_Access.all;
      Node       : constant Bare_Lkt_Node := Env_Node (Env);
   begin
      NED.Envs.Insert (Node, Env);
      Node.Unit.Named_Envs.Append ((Name, Env));

      --  If that insertion must change the env that has precedence, signal
      --  that NED requires an update.

      if NED.Envs.First_Element /= NED.Env_With_Precedence then
         Named_Envs_Needing_Update.Include (Name, NED_Access);
      end if;
   end Register_Named_Env;

   ----------------------
   -- Update_Named_Env --
   ----------------------

   procedure Update_Named_Envs
     (Context : Internal_Context; Named_Envs : NED_Maps.Map)
   is
      Require_Cache_Reset : Boolean := False;
   begin
      for Cur in Named_Envs.Iterate loop
         declare
            NE      : Named_Env_Descriptor renames NED_Maps.Element (Cur).all;
            New_Env : constant Lexical_Env :=
              (if NE.Envs.Is_Empty
               then Empty_Env
               else NE.Envs.First_Element);
         begin
            --  If there was an environment with precedence, remove its foreign
            --  nodes.
            if NE.Env_With_Precedence /= Empty_Env then
               for Cur in NE.Foreign_Nodes.Iterate loop
                  declare
                     Key   : constant Symbol_Type :=
                        NED_Assoc_Maps.Key (Cur);
                     Nodes : Internal_Map_Node_Vectors.Vector renames
                        NE.Foreign_Nodes.Reference (Cur);
                  begin
                     for N of Nodes loop
                        Remove (NE.Env_With_Precedence, Thin (Key), N.Node);
                     end loop;
                  end;
               end loop;
            end if;

            --  Now, set the new environment that has precedence
            NE.Env_With_Precedence := New_Env;

            --  Add the foreign nodes to the new environment with precedence,
            --  if any.
            for Cur in NE.Foreign_Nodes.Iterate loop
               declare
                  Key   : constant Symbol_Type :=
                     NED_Assoc_Maps.Key (Cur);
                  Nodes : Internal_Map_Node_Vectors.Vector renames
                     NE.Foreign_Nodes.Reference (Cur);
               begin
                  for N of Nodes loop
                     Add (New_Env, Thin (Key), N.Node, N.Md, N.Resolver);
                  end loop;
               end;
            end loop;

            --  Set the parent environment of all foreign environments
            for Cur in NE.Foreign_Envs.Iterate loop
               declare
                  Env : Lexical_Env_Record renames
                     Unwrap (Sorted_Env_Maps.Element (Cur)).all;
               begin
                  Env.Parent := New_Env;

                  --  We have updated the lexical env hierarchy (e.g. an env
                  --  which had no parent may have one now), so the cached
                  --  entries for queries that traveresed the old env hierarchy
                  --  need to be invalidated.
                  Require_Cache_Reset := True;
               end;
            end loop;

            --  Update nodes whose environment was the old env with precedence
            for N of NE.Nodes_With_Foreign_Env loop
               N.Self_Env := New_Env;
            end loop;
         end;
      end loop;
      if Require_Cache_Reset then
         Invalidate_Caches (Context, Invalidate_Envs => True);
      end if;
   end Update_Named_Envs;

   --------------------------
   -- Big integers wrapper --
   --------------------------

   ------------------------
   -- Create_Big_Integer --
   ------------------------

   function Create_Big_Integer
     (Image : String; Base : Integer := 10) return Big_Integer_Type
   is
      use GNATCOLL.GMP;
      use GNATCOLL.GMP.Integers;
   begin
      return new Big_Integer_Record'(Value     => Make (Image, Int (Base)),
                                     Ref_Count => 1);
   end Create_Big_Integer;

   ------------------------
   -- Create_Big_Integer --
   ------------------------

   function Create_Big_Integer
     (Big_Int : GNATCOLL.GMP.Integers.Big_Integer) return Big_Integer_Type
   is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (Big_Int);
      return Result;
   end Create_Big_Integer;

   ------------------------
   -- Create_Big_Integer --
   ------------------------

   function Create_Big_Integer (Int : Integer) return Big_Integer_Type is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (GNATCOLL.GMP.Long (Int));
      return Result;
   end Create_Big_Integer;

   -------------------------------
   -- Create_Public_Big_Integer --
   -------------------------------

   function Create_Public_Big_Integer
     (Big_Int : Big_Integer_Type) return GNATCOLL.GMP.Integers.Big_Integer is
   begin
      return Result : GNATCOLL.GMP.Integers.Big_Integer do
         Result.Set (Big_Int.Value);
      end return;
   end Create_Public_Big_Integer;

   -----------------
   -- Trace_Image --
   -----------------

   function Trace_Image (I : Big_Integer_Type) return String is
   begin
      return GNATCOLL.GMP.Integers.Image (I.Value);
   end Trace_Image;

   ----------------
   -- To_Integer --
   ----------------

   function To_Integer
     (Self    : Bare_Lkt_Node;
      Big_Int : Big_Integer_Type) return Integer
   is
      Image : constant String := Big_Int.Value.Image;
   begin
      return Integer'Value (Image);
   exception
      when Constraint_Error =>
         Raise_Property_Exception
           (Self, Property_Error'Identity, "out of range big integer");
   end To_Integer;

   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (Big_Int : Big_Integer_Type) is
   begin
      if Big_Int.Ref_Count /= -1 then
         Big_Int.Ref_Count := Big_Int.Ref_Count + 1;
      end if;
   end Inc_Ref;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (Big_Int : in out Big_Integer_Type) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Big_Integer_Record, Big_Integer_Type);
   begin
      if Big_Int = null or else Big_Int.Ref_Count = -1 then
         return;
      end if;

      Big_Int.Ref_Count := Big_Int.Ref_Count - 1;
      if Big_Int.Ref_Count = 0 then
         Destroy (Big_Int);
      end if;
   end Dec_Ref;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value = Right.Value;
   end Equivalent;

   ---------
   -- "<" --
   ---------

   function "<" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value < Right.Value;
   end "<";

   ----------
   -- "<=" --
   ----------

   function "<=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value <= Right.Value;
   end "<=";

   ---------
   -- ">" --
   ---------

   function ">" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value > Right.Value;
   end ">";

   ----------
   -- ">=" --
   ----------

   function ">=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value >= Right.Value;
   end ">=";

   ---------
   -- "+" --
   ---------

   function "+" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value + Right.Value);
   end "+";

   ---------
   -- "-" --
   ---------

   function "-" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value - Right.Value);
   end "-";

   ---------
   -- "-" --
   ---------

   function "-" (Value : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (-Value.Value);
   end "-";

   ------------------
   -- Unit_Version --
   ------------------

   function Unit_Version (Unit : Generic_Unit_Ptr) return Version_Number is
   begin
      return Convert_Unit (Unit).Unit_Version;
   end Unit_Version;

   -------------------------
   -- Get_Context_Version --
   -------------------------

   function Get_Context_Version
     (Node : Bare_Lkt_Node) return Version_Number is
   begin
      return Node.Unit.Context.Cache_Version;
   end Get_Context_Version;

   ---------------
   --  Self_Env --
   ---------------

   function Self_Env (Node : Bare_Lkt_Node) return Lexical_Env is
   begin
      return Node.Self_Env;
   end Self_Env;

   --------------------------
   -- Properties_May_Raise --
   --------------------------

   function Properties_May_Raise
     (Exc : Ada.Exceptions.Exception_Occurrence) return Boolean is
   begin
      return Ada.Exceptions.Exception_Identity (Exc) in
            Property_Error'Identity
      ;
   end Properties_May_Raise;

   ----------------------
   -- Short_Text_Image --
   ----------------------

   function Short_Text_Image (Self : Bare_Lkt_Node) return Text_Type
   is
   begin
      if Self = null then
         return "None";
      end if;

      
   

   case Self.Kind is
            when Lkt_Ref_Id =>
            
         return Ref_Id_Short_Image (Self);
      
            when Lkt_Grammar_Rule_Decl .. Lkt_Trait_Decl =>
            
         return Decl_Short_Image (Self);
      
      when others => 
         return "<" & To_Text (Kind_Name (Self))
                & " "
                & To_Text
                  (Ada.Directories.Simple_Name
                     (Get_Filename (Unit (Self))))
                & ":" & To_Text (Image (Sloc_Range (Self))) & ">";
      
   end case;

   end Short_Text_Image;

   --------------------
   -- Snaps_At_Start --
   --------------------

   function Snaps_At_Start (Self : Bare_Lkt_Node) return Boolean is
   begin
      
   

   case Self.Kind is
      when others => 
         return False;
      
   end case;

   end Snaps_At_Start;

   ------------------
   -- Snaps_At_End --
   ------------------

   function Snaps_At_End (Self : Bare_Lkt_Node) return Boolean is
   begin
      
   

   case Self.Kind is
      when others => 
         return Is_Incomplete (Self);
      
   end case;

   end Snaps_At_End;

   -------------
   -- Parents --
   -------------

   function Parents
     (Node      : Bare_Lkt_Node;
      With_Self : Boolean := True)
      return Bare_Lkt_Node_Array_Access
   is
      Count : Natural := 0;
      Start : Bare_Lkt_Node :=
        (if With_Self then Node else Node.Parent);
      Cur   : Bare_Lkt_Node := Start;
   begin
      while Cur /= null loop
         Count := Count + 1;
         Cur := Cur.Parent;
      end loop;

      declare
         Result : constant Bare_Lkt_Node_Array_Access :=
            Create_Bare_Lkt_Node_Array (Count);
      begin
         Cur := Start;
         for I in Result.Items'Range loop
            Result.Items (I) := Cur;
            Cur := Cur.Parent;
         end loop;
         return Result;
      end;
   end Parents;

   -----------------------
   -- First_Child_Index --
   -----------------------

   function First_Child_Index (Node : Bare_Lkt_Node) return Natural
   is (1);

   ----------------------
   -- Last_Child_Index --
   ----------------------

   function Last_Child_Index (Node : Bare_Lkt_Node) return Natural
   is (Children_Count (Node));

   ---------------
   -- Get_Child --
   ---------------

   procedure Get_Child
     (Node            : Bare_Lkt_Node;
      Index           : Positive;
      Index_In_Bounds : out Boolean;
      Result          : out Bare_Lkt_Node)
   is
      K : constant Lkt_Node_Kind_Type := Node.Kind;
   begin
      

      Index_In_Bounds := True;
      Result := null;
      case Lkt_Lkt_Node (K) is
when Lkt_Lexer_Case_Rule_Cond_Alt_Range =>
declare
N_Bare_Lexer_Case_Rule_Cond_Alt : constant Bare_Lexer_Case_Rule_Cond_Alt := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Lexer_Case_Rule_Cond_Alt.Lexer_Case_Rule_Cond_Alt_F_Cond_Exprs;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Lexer_Case_Rule_Cond_Alt.Lexer_Case_Rule_Cond_Alt_F_Send;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Lexer_Case_Rule_Default_Alt_Range =>
declare
N_Bare_Lexer_Case_Rule_Default_Alt : constant Bare_Lexer_Case_Rule_Default_Alt := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Lexer_Case_Rule_Default_Alt.Lexer_Case_Rule_Default_Alt_F_Send;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Rule_Decl_Range =>
declare
N_Bare_Grammar_Rule_Decl : constant Bare_Grammar_Rule_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Rule_Decl.Grammar_Rule_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Grammar_Rule_Decl.Grammar_Rule_Decl_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Enum_Lit_Decl_Range =>
declare
N_Bare_Enum_Lit_Decl : constant Bare_Enum_Lit_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Enum_Lit_Decl.Enum_Lit_Decl_F_Syn_Name;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Field_Decl_Range =>
declare
N_Bare_Field_Decl : constant Bare_Field_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Field_Decl.Field_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Field_Decl.Field_Decl_F_Decl_Type;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Field_Decl.Field_Decl_F_Default_Val;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Fun_Arg_Decl_Range =>
declare
N_Bare_Fun_Arg_Decl : constant Bare_Fun_Arg_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Fun_Arg_Decl.Fun_Arg_Decl_F_Decl_Annotations;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Fun_Arg_Decl.Fun_Arg_Decl_F_Syn_Name;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Fun_Arg_Decl.Fun_Arg_Decl_F_Decl_Type;
                            return;
                    

                        when 4 =>
                            Result := N_Bare_Fun_Arg_Decl.Fun_Arg_Decl_F_Default_Val;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Lambda_Arg_Decl_Range =>
declare
N_Bare_Lambda_Arg_Decl : constant Bare_Lambda_Arg_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Lambda_Arg_Decl.Lambda_Arg_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Lambda_Arg_Decl.Lambda_Arg_Decl_F_Decl_Type;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Lambda_Arg_Decl.Lambda_Arg_Decl_F_Default_Val;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Dyn_Var_Decl_Range =>
declare
N_Bare_Dyn_Var_Decl : constant Bare_Dyn_Var_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Dyn_Var_Decl.Dyn_Var_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Dyn_Var_Decl.Dyn_Var_Decl_F_Decl_Type;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Match_Val_Decl_Range =>
declare
N_Bare_Match_Val_Decl : constant Bare_Match_Val_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Match_Val_Decl.Match_Val_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Match_Val_Decl.Match_Val_Decl_F_Decl_Type;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Val_Decl_Range =>
declare
N_Bare_Val_Decl : constant Bare_Val_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Val_Decl.Val_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Val_Decl.Val_Decl_F_Decl_Type;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Val_Decl.Val_Decl_F_Val;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Fun_Decl_Range =>
declare
N_Bare_Fun_Decl : constant Bare_Fun_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Fun_Decl.Fun_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Fun_Decl.Fun_Decl_F_Args;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Fun_Decl.Fun_Decl_F_Return_Type;
                            return;
                    

                        when 4 =>
                            Result := N_Bare_Fun_Decl.Fun_Decl_F_Body;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Env_Spec_Decl_Range =>
declare
N_Bare_Env_Spec_Decl : constant Bare_Env_Spec_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Env_Spec_Decl.Env_Spec_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Env_Spec_Decl.Env_Spec_Decl_F_Actions;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Generic_Decl_Range =>
declare
N_Bare_Generic_Decl : constant Bare_Generic_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Generic_Decl.Generic_Decl_F_Generic_Formal_Decls;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Generic_Decl.Generic_Decl_F_Decl;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Decl_Range =>
declare
N_Bare_Grammar_Decl : constant Bare_Grammar_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Decl.Grammar_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Grammar_Decl.Grammar_Decl_F_Rules;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Lexer_Decl_Range =>
declare
N_Bare_Lexer_Decl : constant Bare_Lexer_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Lexer_Decl.Lexer_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Lexer_Decl.Lexer_Decl_F_Rules;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Lexer_Family_Decl_Range =>
declare
N_Bare_Lexer_Family_Decl : constant Bare_Lexer_Family_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Lexer_Family_Decl.Lexer_Family_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Lexer_Family_Decl.Lexer_Family_Decl_F_Rules;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Enum_Class_Alt_Decl_Range =>
declare
N_Bare_Enum_Class_Alt_Decl : constant Bare_Enum_Class_Alt_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Enum_Class_Alt_Decl.Enum_Class_Alt_Decl_F_Syn_Name;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Generic_Formal_Type_Decl_Range =>
declare
N_Bare_Generic_Formal_Type_Decl : constant Bare_Generic_Formal_Type_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Generic_Formal_Type_Decl.Generic_Formal_Type_Decl_F_Has_Class;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Generic_Formal_Type_Decl.Generic_Formal_Type_Decl_F_Syn_Name;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Basic_Class_Decl =>
declare
N_Bare_Basic_Class_Decl : constant Bare_Basic_Class_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Basic_Class_Decl.Basic_Class_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Basic_Class_Decl.Basic_Class_Decl_F_Syn_Base_Type;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Basic_Class_Decl.Basic_Class_Decl_F_Traits;
                            return;
                    

                        when others => null;
                    end case;
                
case Lkt_Basic_Class_Decl (K) is
when Lkt_Class_Decl_Range =>
declare
N_Bare_Class_Decl : constant Bare_Class_Decl := N_Bare_Basic_Class_Decl;
begin
case Index is

                        when 4 =>
                            Result := N_Bare_Class_Decl.Class_Decl_F_Decls;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Enum_Class_Decl_Range =>
declare
N_Bare_Enum_Class_Decl : constant Bare_Enum_Class_Decl := N_Bare_Basic_Class_Decl;
begin
case Index is

                        when 4 =>
                            Result := N_Bare_Enum_Class_Decl.Enum_Class_Decl_F_Branches;
                            return;
                    

                        when 5 =>
                            Result := N_Bare_Enum_Class_Decl.Enum_Class_Decl_F_Decls;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when others => null;
end case;
end;
when Lkt_Enum_Type_Decl_Range =>
declare
N_Bare_Enum_Type_Decl : constant Bare_Enum_Type_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Enum_Type_Decl.Enum_Type_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Enum_Type_Decl.Enum_Type_Decl_F_Traits;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Enum_Type_Decl.Enum_Type_Decl_F_Literals;
                            return;
                    

                        when 4 =>
                            Result := N_Bare_Enum_Type_Decl.Enum_Type_Decl_F_Decls;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Struct_Decl_Range =>
declare
N_Bare_Struct_Decl : constant Bare_Struct_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Struct_Decl.Struct_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Struct_Decl.Struct_Decl_F_Traits;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Struct_Decl.Struct_Decl_F_Decls;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Trait_Decl_Range =>
declare
N_Bare_Trait_Decl : constant Bare_Trait_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Trait_Decl.Trait_Decl_F_Syn_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Trait_Decl.Trait_Decl_F_Decls;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Decl_Annotation_Range =>
declare
N_Bare_Decl_Annotation : constant Bare_Decl_Annotation := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Decl_Annotation.Decl_Annotation_F_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Decl_Annotation.Decl_Annotation_F_Params;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Decl_Annotation_Params_Range =>
declare
N_Bare_Decl_Annotation_Params : constant Bare_Decl_Annotation_Params := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Decl_Annotation_Params.Decl_Annotation_Params_F_Params;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Elsif_Branch_Range =>
declare
N_Bare_Elsif_Branch : constant Bare_Elsif_Branch := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Elsif_Branch.Elsif_Branch_F_Cond_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Elsif_Branch.Elsif_Branch_F_Then_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Enum_Class_Case_Range =>
declare
N_Bare_Enum_Class_Case : constant Bare_Enum_Class_Case := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Enum_Class_Case.Enum_Class_Case_F_Decls;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Any_Of_Range =>
declare
N_Bare_Any_Of : constant Bare_Any_Of := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Any_Of.Any_Of_F_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Any_Of.Any_Of_F_Values;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Array_Literal_Range =>
declare
N_Bare_Array_Literal : constant Bare_Array_Literal := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Array_Literal.Array_Literal_F_Exprs;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Array_Literal.Array_Literal_F_Element_Type;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Base_Dot_Expr =>
declare
N_Bare_Base_Dot_Expr : constant Bare_Base_Dot_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Base_Dot_Expr.Base_Dot_Expr_F_Prefix;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Base_Dot_Expr.Base_Dot_Expr_F_Suffix;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Bin_Op_Range =>
declare
N_Bare_Bin_Op : constant Bare_Bin_Op := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Left;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Op;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Right;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Block_Expr_Range =>
declare
N_Bare_Block_Expr : constant Bare_Block_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Block_Expr.Block_Expr_F_Val_Defs;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Block_Expr.Block_Expr_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Args;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Cast_Expr_Range =>
declare
N_Bare_Cast_Expr : constant Bare_Cast_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Cast_Expr.Cast_Expr_F_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Cast_Expr.Cast_Expr_F_Excludes_Null;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Cast_Expr.Cast_Expr_F_Dest_Type;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Error_On_Null_Range =>
declare
N_Bare_Error_On_Null : constant Bare_Error_On_Null := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Error_On_Null.Error_On_Null_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Generic_Instantiation_Range =>
declare
N_Bare_Generic_Instantiation : constant Bare_Generic_Instantiation := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Generic_Instantiation.Generic_Instantiation_F_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Generic_Instantiation.Generic_Instantiation_F_Args;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Discard_Range =>
declare
N_Bare_Grammar_Discard : constant Bare_Grammar_Discard := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Discard.Grammar_Discard_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Dont_Skip_Range =>
declare
N_Bare_Grammar_Dont_Skip : constant Bare_Grammar_Dont_Skip := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Dont_Skip.Grammar_Dont_Skip_F_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Grammar_Dont_Skip.Grammar_Dont_Skip_F_Dont_Skip;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_List_Range =>
declare
N_Bare_Grammar_List : constant Bare_Grammar_List := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_List.Grammar_List_F_List_Type;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Grammar_List.Grammar_List_F_Kind;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Grammar_List.Grammar_List_F_Expr;
                            return;
                    

                        when 4 =>
                            Result := N_Bare_Grammar_List.Grammar_List_F_Sep;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Null_Range =>
declare
N_Bare_Grammar_Null : constant Bare_Grammar_Null := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Null.Grammar_Null_F_Name;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Opt_Range =>
declare
N_Bare_Grammar_Opt : constant Bare_Grammar_Opt := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Opt.Grammar_Opt_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Opt_Error_Range =>
declare
N_Bare_Grammar_Opt_Error : constant Bare_Grammar_Opt_Error := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Opt_Error.Grammar_Opt_Error_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Opt_Error_Group_Range =>
declare
N_Bare_Grammar_Opt_Error_Group : constant Bare_Grammar_Opt_Error_Group := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Opt_Error_Group.Grammar_Opt_Error_Group_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Opt_Group_Range =>
declare
N_Bare_Grammar_Opt_Group : constant Bare_Grammar_Opt_Group := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Opt_Group.Grammar_Opt_Group_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Or_Expr_Range =>
declare
N_Bare_Grammar_Or_Expr : constant Bare_Grammar_Or_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Or_Expr.Grammar_Or_Expr_F_Sub_Exprs;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Pick_Range =>
declare
N_Bare_Grammar_Pick : constant Bare_Grammar_Pick := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Pick.Grammar_Pick_F_Exprs;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Predicate_Range =>
declare
N_Bare_Grammar_Predicate : constant Bare_Grammar_Predicate := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Predicate.Grammar_Predicate_F_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Grammar_Predicate.Grammar_Predicate_F_Prop_Ref;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Rule_Ref_Range =>
declare
N_Bare_Grammar_Rule_Ref : constant Bare_Grammar_Rule_Ref := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Rule_Ref.Grammar_Rule_Ref_F_Node_Name;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Skip_Range =>
declare
N_Bare_Grammar_Skip : constant Bare_Grammar_Skip := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Skip.Grammar_Skip_F_Name;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_Stop_Cut_Range =>
declare
N_Bare_Grammar_Stop_Cut : constant Bare_Grammar_Stop_Cut := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_Stop_Cut.Grammar_Stop_Cut_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Parse_Node_Expr_Range =>
declare
N_Bare_Parse_Node_Expr : constant Bare_Parse_Node_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Parse_Node_Expr.Parse_Node_Expr_F_Node_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Parse_Node_Expr.Parse_Node_Expr_F_Sub_Exprs;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Token_No_Case_Lit_Range =>
declare
N_Bare_Token_No_Case_Lit : constant Bare_Token_No_Case_Lit := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Token_No_Case_Lit.Token_No_Case_Lit_F_Lit;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Token_Ref_Range =>
declare
N_Bare_Token_Ref : constant Bare_Token_Ref := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Token_Ref.Token_Ref_F_Token_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Token_Ref.Token_Ref_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_If_Expr_Range =>
declare
N_Bare_If_Expr : constant Bare_If_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Cond_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Then_Expr;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Alternatives;
                            return;
                    

                        when 4 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Else_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Isa_Range =>
declare
N_Bare_Isa : constant Bare_Isa := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Isa.Isa_F_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Isa.Isa_F_Dest_Type;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Keep_Expr_Range =>
declare
N_Bare_Keep_Expr : constant Bare_Keep_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Keep_Expr.Keep_Expr_F_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Keep_Expr.Keep_Expr_F_Keep_Type;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Lambda_Expr_Range =>
declare
N_Bare_Lambda_Expr : constant Bare_Lambda_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Lambda_Expr.Lambda_Expr_F_Params;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Lambda_Expr.Lambda_Expr_F_Return_Type;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Lambda_Expr.Lambda_Expr_F_Body;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Null_Lit_Range =>
declare
N_Bare_Null_Lit : constant Bare_Null_Lit := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Null_Lit.Null_Lit_F_Dest_Type;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Block_String_Lit_Range =>
declare
N_Bare_Block_String_Lit : constant Bare_Block_String_Lit := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Block_String_Lit.Block_String_Lit_F_Lines;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Logic_Expr_Range =>
declare
N_Bare_Logic_Expr : constant Bare_Logic_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Logic_Expr.Logic_Expr_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Match_Expr_Range =>
declare
N_Bare_Match_Expr : constant Bare_Match_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Match_Expr.Match_Expr_F_Match_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Match_Expr.Match_Expr_F_Branches;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Not_Expr_Range =>
declare
N_Bare_Not_Expr : constant Bare_Not_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Not_Expr.Not_Expr_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Paren_Expr_Range =>
declare
N_Bare_Paren_Expr : constant Bare_Paren_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Paren_Expr.Paren_Expr_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Raise_Expr_Range =>
declare
N_Bare_Raise_Expr : constant Bare_Raise_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Dest_Type;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Except_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Subscript_Expr_Range =>
declare
N_Bare_Subscript_Expr : constant Bare_Subscript_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Subscript_Expr.Subscript_Expr_F_Prefix;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Subscript_Expr.Subscript_Expr_F_Index;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Try_Expr_Range =>
declare
N_Bare_Try_Expr : constant Bare_Try_Expr := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Try_Expr.Try_Expr_F_Try_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Try_Expr.Try_Expr_F_Or_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Un_Op_Range =>
declare
N_Bare_Un_Op : constant Bare_Un_Op := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Op;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Full_Decl_Range =>
declare
N_Bare_Full_Decl : constant Bare_Full_Decl := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Full_Decl.Full_Decl_F_Doc;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Full_Decl.Full_Decl_F_Decl_Annotations;
                            return;
                    

                        when 3 =>
                            Result := N_Bare_Full_Decl.Full_Decl_F_Decl;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Grammar_List_Sep_Range =>
declare
N_Bare_Grammar_List_Sep : constant Bare_Grammar_List_Sep := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Grammar_List_Sep.Grammar_List_Sep_F_Token;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Grammar_List_Sep.Grammar_List_Sep_F_Extra;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Import_Range =>
declare
N_Bare_Import : constant Bare_Import := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Import.Import_F_Name;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Langkit_Root_Range =>
declare
N_Bare_Langkit_Root : constant Bare_Langkit_Root := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Langkit_Root.Langkit_Root_F_Imports;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Langkit_Root.Langkit_Root_F_Decls;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Lexer_Case_Rule_Range =>
declare
N_Bare_Lexer_Case_Rule : constant Bare_Lexer_Case_Rule := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Lexer_Case_Rule.Lexer_Case_Rule_F_Expr;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Lexer_Case_Rule.Lexer_Case_Rule_F_Alts;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Lexer_Case_Rule_Send_Range =>
declare
N_Bare_Lexer_Case_Rule_Send : constant Bare_Lexer_Case_Rule_Send := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Lexer_Case_Rule_Send.Lexer_Case_Rule_Send_F_Sent;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Lexer_Case_Rule_Send.Lexer_Case_Rule_Send_F_Match_Size;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Lkt_Node_Base_List =>
declare
N_Bare_Lkt_Node_Base_List : constant Bare_Lkt_Node_Base_List := Node;
begin

                    if Index > N_Bare_Lkt_Node_Base_List.Count then
                        Index_In_Bounds := False;
                    else
                        Result := N_Bare_Lkt_Node_Base_List.Nodes (Index);
                    end if;
                    return;
                
end;
when Lkt_Match_Branch_Range =>
declare
N_Bare_Match_Branch : constant Bare_Match_Branch := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Match_Branch.Match_Branch_F_Decl;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Match_Branch.Match_Branch_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Param_Range =>
declare
N_Bare_Param : constant Bare_Param := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Param.Param_F_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Param.Param_F_Value;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Function_Type_Ref_Range =>
declare
N_Bare_Function_Type_Ref : constant Bare_Function_Type_Ref := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Function_Type_Ref.Function_Type_Ref_F_Args_Types;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Function_Type_Ref.Function_Type_Ref_F_Return_Type;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Generic_Type_Ref_Range =>
declare
N_Bare_Generic_Type_Ref : constant Bare_Generic_Type_Ref := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Generic_Type_Ref.Generic_Type_Ref_F_Type_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Generic_Type_Ref.Generic_Type_Ref_F_Params;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Simple_Type_Ref_Range =>
declare
N_Bare_Simple_Type_Ref : constant Bare_Simple_Type_Ref := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Simple_Type_Ref.Simple_Type_Ref_F_Type_Name;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when Lkt_Var_Bind_Range =>
declare
N_Bare_Var_Bind : constant Bare_Var_Bind := Node;
begin
case Index is

                        when 1 =>
                            Result := N_Bare_Var_Bind.Var_Bind_F_Name;
                            return;
                    

                        when 2 =>
                            Result := N_Bare_Var_Bind.Var_Bind_F_Expr;
                            return;
                    

                        when others => null;
                    end case;
                
end;
when others => null;
end case;

      --  Execution should reach this point iff nothing matched this index, so
      --  we must be out of bounds.
      Index_In_Bounds := False;
   end Get_Child;

   -----------
   -- Print --
   -----------

   procedure Print
     (Node        : Langkit_Support.Generic_API.Analysis.Lk_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "")
   is
      use Langkit_Support.Generic_API.Analysis;
      use Langkit_Support.Generic_API.Introspection;

      T : Type_Ref;
   begin
      if Node.Is_Null then
         Put_Line ("None");
         return;
      end if;

      T := Type_Of (Node);
      Put (Line_Prefix & Image (Node_Type_Repr_Name (T)));
      if Show_Slocs then
         Put ("[" & Image (Node.Sloc_Range) & "]");
      end if;

      if Node.Is_Incomplete then
         Put (" <<INCOMPLETE>>");
      end if;

      if Node.Is_Token_Node then
         Put_Line (": " & Image (Node.Text));

      elsif Is_List_Node (Node) then

         --  List nodes are displayed in a special way (they have no field)

         declare
            Count : constant Natural := Node.Children_Count;
            Child : Lk_Node;
         begin
            if Count = 0 then
               Put_Line (": <empty list>");
               return;
            end if;
            New_Line;

            for I in 1 .. Count loop
               Child := Node.Child (I);
               if not Child.Is_Null then
                  Print (Child, Show_Slocs, Line_Prefix & "|  ");
               end if;
            end loop;
         end;

      else
         --  This is for regular nodes: display each syntax field (i.e.
         --  non-property member).

         declare
            Attr_Prefix     : constant String := Line_Prefix & "|";
            Children_Prefix : constant String := Line_Prefix & "|  ";
            M_List          : constant Struct_Member_Ref_Array := Members (T);
            Child           : Lk_Node;
         begin
            New_Line;
            for M of M_List loop
               if not Is_Property (M) and then not Is_Null_For (M, T) then
                  Child := As_Node (Eval_Node_Member (Node, M));
                  Put (Attr_Prefix
                       & Image (Format_Name (Member_Name (M), Lower)) & ":");
                  if Child.Is_Null then
                     Put_Line (" <null>");
                  else
                     New_Line;
                     Print (Child, Show_Slocs, Children_Prefix);
                  end if;
               end if;
            end loop;
         end;
      end if;
   end Print;

   -----------
   -- Print --
   -----------

   procedure Print
     (Node        : Bare_Lkt_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "")
   is
      Entity : constant Internal_Entity := (Node, No_Entity_Info);
   begin
      Print (To_Generic_Node (Entity), Show_Slocs, Line_Prefix);
   end Print;

   ------------
   -- Parent --
   ------------

   function Parent (Node : Bare_Lkt_Node) return Bare_Lkt_Node is
   begin
      return Node.Parent;
   end Parent;

   ------------------
   -- Stored_Token --
   ------------------

   function Stored_Token
     (Node  : Bare_Lkt_Node;
      Token : Token_Reference) return Token_Index
   is
      Index : constant Token_Or_Trivia_Index := Get_Token_Index (Token);
   begin
      if Node.Unit.TDH'Access /= Get_Token_TDH (Token) then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "Cannot associate a token and a node from different analysis"
            & " units");
      elsif Index.Trivia /= No_Token_Index then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "A node cannot hold trivia");
      end if;

      return Index.Token;
   end Stored_Token;

   -------------------------
   -- Children_And_Trivia --
   -------------------------

   function Children_And_Trivia
     (Node : Bare_Lkt_Node) return Bare_Children_Vector
   is
      Ret_Vec : Bare_Children_Vector;
      Ctx     : Internal_Context renames Node.Unit.Context;
      TDH     : Token_Data_Handler renames Node.Unit.TDH;

      procedure Append_Trivias (First, Last : Token_Index);
      --  Append all the trivias of tokens between indices First and Last to
      --  the returned vector.

      function Filter_Children
        (Parent : Bare_Lkt_Node)
         return Internal_Bare_Lkt_Node_Array;
      --  Return an array for all children in Parent that are not null

      --------------------
      -- Append_Trivias --
      --------------------

      procedure Append_Trivias (First, Last : Token_Index) is
      begin
         for I in First .. Last loop
            for D of Get_Trivias (TDH, I) loop
               Ret_Vec.Append
                 (Bare_Child_Record'
                    (Kind   => Trivia,
                     Trivia => Wrap_Token_Reference
                                 (Ctx, TDH'Access, (I, D))));
            end loop;
         end loop;
      end Append_Trivias;

      ---------------------
      -- Filter_Children --
      ---------------------

      function Filter_Children
        (Parent : Bare_Lkt_Node)
         return Internal_Bare_Lkt_Node_Array
      is
         Children : constant Internal_Bare_Lkt_Node_Array :=
            Implementation.Children (Parent);
         Result   : Internal_Bare_Lkt_Node_Array (Children'Range);
         Next     : Integer := Result'First;
      begin
         for I in Children'Range loop
            if Children (I) /= null then
               Result (Next) := Children (I);
               Next := Next + 1;
            end if;
         end loop;
         return Result (Result'First .. Next - 1);
      end Filter_Children;

      First_Child : constant Positive := 1;
      N_Children  : constant Internal_Bare_Lkt_Node_Array :=
         Filter_Children (Node);
   begin
      if N_Children'Length > 0
        and then (Node.Token_Start_Index
                    /= N_Children (First_Child).Token_Start_Index)
      then
         Append_Trivias (Node.Token_Start_Index,
                         N_Children (First_Child).Token_Start_Index - 1);
      end if;

      --  Append each node to Ret_Vec, and append trivia that follow after each
      --  non-ghost nodes.
      for I in N_Children'Range loop
         Ret_Vec.Append (Bare_Child_Record'(Child, N_Children (I)));
         if not Is_Ghost (N_Children (I)) then
            Append_Trivias (N_Children (I).Token_End_Index,
                            (if I = N_Children'Last
                             then Node.Token_End_Index - 1
                             else N_Children (I + 1).Token_Start_Index - 1));
         end if;
      end loop;

      return Ret_Vec;
   end Children_And_Trivia;

   --------------
   -- Is_Ghost --
   --------------

   function Is_Ghost (Node : Bare_Lkt_Node) return Boolean
   is (Node.Token_End_Index = No_Token_Index);

   -------------------
   -- Is_Incomplete --
   -------------------

   function Is_Incomplete (Node : Bare_Lkt_Node) return Boolean
   is
      LGC : Bare_Lkt_Node;
   begin
     if Is_List_Node (Node.Kind) then
        LGC := (if Last_Child_Index (Node) /= 0
                then Child (Node, Last_Child_Index (Node))
                else null);
        return LGC /= null and then Is_Incomplete (LGC);
      else
         return Node.Last_Attempted_Child > -1;
      end if;
   end;

   -----------------
   -- Token_Start --
   -----------------

   function Token_Start (Node : Bare_Lkt_Node) return Token_Reference
   is (Token (Node, Node.Token_Start_Index));

   ---------------
   -- Token_End --
   ---------------

   function Token_End (Node : Bare_Lkt_Node) return Token_Reference
   is
     (if Node.Token_End_Index = No_Token_Index
      then Token_Start (Node)
      else Token (Node, Node.Token_End_Index));

   -----------
   -- Token --
   -----------

   function Token
     (Node  : Bare_Lkt_Node;
      Index : Token_Index) return Token_Reference
   is
      Unit    : constant Internal_Unit := Node.Unit;
      Context : constant Internal_Context := Unit.Context;
   begin
      return Wrap_Token_Reference
        (Context, Token_Data (Unit), (Index, No_Token_Index));
   end Token;

   ---------
   -- "<" --
   ---------

   function "<" (Left, Right : Bare_Lkt_Node) return Boolean is
   begin
      --  Reject invalid inputs
      if Left /= null and Is_Synthetic (Left) then
         raise Property_Error with "left node is synthetic";
      elsif Right /= null and Is_Synthetic (Right) then
         raise Property_Error with "right node is synthetic";
      end if;

      --  Null nodes come first
      if Left = null then
         return Right /= null;
      elsif Right = null then
         return False;
      end if;

      --  So we have two non-null nodes. Sort by unit filename
      if Left.Unit < Right.Unit then
         return True;
      elsif Left.Unit /= Right.Unit then
         return False;
      end if;

      --  Both nodes come from the same unit: compare their token indexes
      if Left.Token_Start_Index < Right.Token_Start_Index then
         return True;
      elsif Left.Token_Start_Index > Right.Token_Start_Index then
         return False;
      else
         return Left.Token_End_Index < Right.Token_End_Index;
      end if;
   end "<";

   -------------
   -- Is_Null --
   -------------

   function Is_Null (Node : Bare_Lkt_Node) return Boolean
   is (Node = null);

   ----------
   -- Kind --
   ----------

   function Kind (Node : Bare_Lkt_Node) return Lkt_Node_Kind_Type
   is (Node.Kind);

   -----------------
   -- Child_Index --
   -----------------

   function Child_Index (Node : Bare_Lkt_Node) return Integer
   is
      N : Bare_Lkt_Node := null;
   begin
      if Node.Parent = null then
         raise Property_Error with
            "Trying to get the child index of a root node";
      end if;

      for I in First_Child_Index (Node.Parent)
            .. Last_Child_Index (Node.Parent)
      loop
         N := Child (Node.Parent, I);
         if N = Node then
            return I - 1;
         end if;
      end loop;

      --  If we reach this point, then Node isn't a Child of Node.Parent. This
      --  is not supposed to happen.
      raise Program_Error;
   end Child_Index;

   -------------------
   -- Fetch_Sibling --
   -------------------

   function Fetch_Sibling
     (Node   : Bare_Lkt_Node;
      Offset : Integer) return Bare_Lkt_Node is
   begin
      --  Root nodes have no sibling: handle them now to avoid invalid requests
      --  in the code below.
      if Node.Parent = null then
         return null;
      end if;

      declare
         Node_Index : constant Positive := Child_Index (Node) + 1;
         --  Child_Index is 0-based, but the Child primitive expects a 1-based
         --  index.

         Sibling_Index : constant Integer := Node_Index + Offset;
      begin
         --  Child returns null for out-of-bound indexes

         return (if Sibling_Index >= 1
                 then Child (Node.Parent, Sibling_Index)
                 else null);
      end;
   end Fetch_Sibling;

   -------------------
   -- Fetch_Sibling --
   -------------------

   function Fetch_Sibling
     (Node   : Bare_Lkt_Node;
      E_Info : Internal_Entity_Info;
      Offset : Integer) return Internal_Entity
   is
      Sibling : constant Bare_Lkt_Node := Fetch_Sibling (Node, Offset);
   begin
      --  Don't forget to clear entity info if the result is nul

      return (if Sibling = null
              then No_Entity
              else (Sibling, E_Info));
   end Fetch_Sibling;

   ----------------------
   -- Previous_Sibling --
   ----------------------

   function Previous_Sibling
     (Node   : Bare_Lkt_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, -1);
   end Previous_Sibling;

   ------------------
   -- Next_Sibling --
   ------------------

   function Next_Sibling
     (Node   : Bare_Lkt_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, 1);
   end Next_Sibling;

   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (Self : Node_Builder_Type) is
   begin
      if Self.Ref_Count > 0 then
         Self.Ref_Count := Self.Ref_Count + 1;
      end if;
   end Inc_Ref;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (Self : in out Node_Builder_Type) is
   begin
      if Self = null or else Self.Ref_Count < 0 then
         return;
      elsif Self.Ref_Count = 1 then
         Self.Release;
         Free (Self);
      else
         Self.Ref_Count := Self.Ref_Count - 1;
      end if;
   end Dec_Ref;

   ------------------------------
   -- Create_Copy_Node_Builder --
   ------------------------------

   function Create_Copy_Node_Builder
     (Value : Bare_Lkt_Node) return Node_Builder_Type is
   begin
      --  No need to allocate a new builder if in practice it cannot be
      --  distinguished from the "null" builder.

      if Value = null then
         return Null_Node_Builder;
      else
         return new Copy_Node_Builder_Record'(Ref_Count => 1, Value => Value);
      end if;
   end Create_Copy_Node_Builder;



   ----------------------
   -- Compare_Metadata --
   ----------------------

   --  Deactivate "not referenced" warnings because if the metadata struct has
   --  no fields, formals and temporaries won't be referenced in the two
   --  following functions.
   pragma Warnings (Off, "referenced");
   function Compare_Metadata (L, R : Internal_Metadata) return Boolean is
   begin
      return True;
   end Compare_Metadata;

   ----------
   -- Hash --
   ----------

   function Hash (Self : Internal_Metadata) return Hash_Type is
      Ret : Hash_Type := Langkit_Support.Hashes.Initial_Hash;
   begin
      return Ret;
   end Hash;
   pragma Warnings (On, "referenced");

   -------------
   -- Combine --
   -------------

   function Combine
     (L, R : Internal_Metadata) return Internal_Metadata
   is
      pragma Unreferenced (L, R);
      Ret : Internal_Metadata := No_Metadata;
   begin
      return Ret;
   end Combine;

   -------------------------------
   -- Create_Static_Lexical_Env --
   -------------------------------

   function Create_Static_Lexical_Env
     (Parent            : Lexical_Env;
      Node              : Bare_Lkt_Node;
      Sym_Table         : Symbol_Table;
      Transitive_Parent : Boolean := False) return Lexical_Env
   is
      Unit : constant Internal_Unit :=
        (if Node = null then null else Node.Unit);
   begin
      return Result : Lexical_Env := Create_Lexical_Env
        (Parent, Node, Transitive_Parent, Sym_Table, Convert_Unit (Unit))
      do
         if Unit /= null then
            Register_Destroyable (Unit, Unwrap (Result.Env));
         end if;
      end return;
   end Create_Static_Lexical_Env;

   ---------
   -- Get --
   ---------

   function Get
     (Self  : Bare_Lkt_Node;
      A     : AST_Envs.Entity_Array;
      Index : Integer) return Internal_Entity
   is
      function Length (A : AST_Envs.Entity_Array) return Natural
      is (A'Length);

      function Get
        (A     : AST_Envs.Entity_Array;
         Index : Integer) return Internal_Entity
      is (A (Index + 1)); --  A is 1-based but Index is 0-based

      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Entity,
         Sequence_Type => AST_Envs.Entity_Array,
         Length        => Length,
         Get           => Get);
      Result : Internal_Entity;
   begin
      if Relative_Get (A, Index, Result) then
         return Result;
      else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "out-of-bounds array access");
      end if;
   end Get;

   -----------
   -- Group --
   -----------

   function Group
     (Envs   : Lexical_Env_Array_Access;
      Env_Md : Internal_Metadata := No_Metadata) return Lexical_Env
   is (Group (Lexical_Env_Array (Envs.Items), Env_Md));

       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   

      --
      --  Untyped wrappers for Bare_Type_Decl
      --

         
function Type_Decl_P_Type_Base_Scope
  (E : Entity
  ) return Lexical_Env;



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   



       

   




   ------------------
   -- Children_Env --
   ------------------

   function Children_Env
     (Node   : Bare_Lkt_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is (Rebind_Env (Node.Self_Env, E_Info));

   --------------
   -- Node_Env --
   --------------

   function Node_Env
     (Node   : Bare_Lkt_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is
      function Get_Base_Env return Lexical_Env;
      --  Return the environment that we need to rebind before returning

      ------------------
      -- Get_Base_Env --
      ------------------

      function Get_Base_Env return Lexical_Env is
         pragma Warnings (Off, "referenced");
         function Get_Parent_Env return Lexical_Env;
         pragma Warnings (On, "referenced");

         --------------------
         -- Get_Parent_Env --
         --------------------

         function Get_Parent_Env return Lexical_Env is
            Parent : constant Lexical_Env := AST_Envs.Parent (Node.Self_Env);
         begin
            --  If Node is the root scope or the empty environment, Parent can
            --  be a wrapper around the null node. Turn this into the
            --  Empty_Env, as null envs are erroneous values in properties.
            return (if Unwrap (Parent) = null
                    then Empty_Env
                    else Parent);
         end Get_Parent_Env;

      begin
         
         return
           (if Node.Kind in Lkt_Fun_Decl | Lkt_Generic_Decl | Lkt_Grammar_Decl | Lkt_Lexer_Decl | Lkt_Block_Expr | Lkt_Lambda_Expr | Lkt_Decl_Block | Lkt_Match_Branch
            then Get_Parent_Env
            else Node.Self_Env);
      end Get_Base_Env;

      Base_Env : Lexical_Env := Get_Base_Env;
      Result   : constant Lexical_Env := Rebind_Env (Base_Env, E_Info);
   begin
      Dec_Ref (Base_Env);
      return Result;
   end Node_Env;

   ------------
   -- Parent --
   ------------

   function Parent
     (Node   : Bare_Lkt_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      --  TODO: shed entity information as appropriate
      return (Node.Parent, E_Info);
   end Parent;

   -------------
   -- Parents --
   -------------

   function Parents
     (Node      : Bare_Lkt_Node;
      With_Self : Boolean := True;
      E_Info    : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Parents : Bare_Lkt_Node_Array_Access := Parents (Node, With_Self);
      Result       : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Parents.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Parents.Items'Range loop
         Result.Items (I) := (Bare_Parents.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Parents);
      return Result;
   end Parents;

   --------------
   -- Children --
   --------------

   function Children
     (Node   : Bare_Lkt_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Children : Bare_Lkt_Node_Array_Access := Children (Node);
      Result        : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Children.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Children.Items'Range loop
         Result.Items (I) := (Bare_Children.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Children);
      return Result;
   end Children;

   ---------------------
   -- New_Unit_String --
   ---------------------

   function New_Unit_String
     (Unit : Internal_Unit; Str : String) return String_Access
   is
      procedure Register_Destroyable_String is new Register_Destroyable_Gen
        (String, String_Access, Free);
   begin
      return Ret : String_Access := new String'(Str) do
         Register_Destroyable_String (Unit, Ret);
      end return;
   end New_Unit_String;

   --------------------------------
   -- Assign_Names_To_Logic_Vars --
   --------------------------------

   procedure Assign_Names_To_Logic_Vars (Node : Bare_Lkt_Node) is

      pragma Warnings (Off, "referenced");

      procedure Assign
        (Node  : Bare_Lkt_Node;
         LV    : in out Logic_Var_Record;
         Field : String);
      --  Assign a name to the LV logic variable. Node must be the node that
      --  owns LV, and Field must be the name of the field in Node that holds
      --  LV.

      ------------
      -- Assign --
      ------------

      procedure Assign
        (Node  : Bare_Lkt_Node;
         LV    : in out Logic_Var_Record;
         Field : String) is
      begin
         LV.Dbg_Name :=
           new String'(Image (Short_Text_Image (Node)) & "." & Field);
      end Assign;

      K : constant Lkt_Node_Kind_Type := Node.Kind;

      pragma Warnings (On, "referenced");

   begin
      
      pragma Unreferenced (K);
      for Child of Internal_Bare_Lkt_Node_Array'(Children (Node)) loop
         if Child /= null then
            Assign_Names_To_Logic_Vars (Child);
         end if;
      end loop;
   end Assign_Names_To_Logic_Vars;

   --------------------------
   -- Initialization_Error --
   --------------------------

   function Initialization_Error
     (Exc : Ada.Exceptions.Exception_Occurrence)
      return Error_Initialization_State
   is
      use Ada.Exceptions;
   begin
      if
         
            Exception_Identity (Exc) = Property_Error'Identity
         then
            return Raised_Property_Error;
      else
         raise Program_Error;
      end if;
   end Initialization_Error;

   ----------------------------------
   -- Reraise_Initialization_Error --
   ----------------------------------

   procedure Reraise_Initialization_Error
     (Node    : Bare_Lkt_Node;
      State   : Error_Initialization_State;
      Message : String)
   is
      Exc : Ada.Exceptions.Exception_Id;
   begin
      case State is
            when Raised_Property_Error =>
               Exc := Property_Error'Identity;
      end case;
      Raise_Property_Exception (Node, Exc, Message);
   end Reraise_Initialization_Error;

   ----------------
   -- Text_Image --
   ----------------

   function Text_Image (Ent : Internal_Entity) return Text_Type is
   begin
      if Ent.Node /= null then
         declare
            Node_Image : constant Text_Type := Short_Text_Image (Ent.Node);
         begin
            return
            (if Ent.Info.Rebindings /= null
             then "<| "
             & Node_Image (Node_Image'First + 1 .. Node_Image'Last - 1) & " "
             & AST_Envs.Text_Image (Ent.Info.Rebindings) & " |>"
             else Node_Image);
         end;
      else
         return "None";
      end if;
   end Text_Image;

   ---------------------
   -- Full_Sloc_Image --
   ---------------------

   function Full_Sloc_Image (Node : Bare_Lkt_Node) return String_Type
   is
      Res      : constant Text_Type :=
        To_Text
          (Ada.Directories.Simple_Name
             (Get_Filename (Unit (Node))))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node)))) & ": ";
   begin
      return Create_String (Res);
   end Full_Sloc_Image;

   -----------
   -- Image --
   -----------

   function Image (Ent : Internal_Entity) return String is
      Result : constant Text_Type := Text_Image (Ent);
   begin
      return Image (Result);
   end Image;

   ---------------
   -- Can_Reach --
   ---------------

   function Can_Reach (El, From : Bare_Lkt_Node) return Boolean is
   begin
      return Lkt_Node_P_Can_Reach (El, From);
   end Can_Reach;

   -----------------
   -- Hash_Entity --
   -----------------

   function Hash_Entity (Self : Internal_Entity) return Hash_Type is
   begin
      return Combine
        ((Hash (Self.Node), Hash (Self.Info.Rebindings), Hash (Self.Info.Md)));
   end Hash_Entity;

   --------------------
   -- Compare_Entity --
   --------------------

   function Compare_Entity (Left, Right : Internal_Entity) return Boolean
   is
   begin
      return Left.Node = Right.Node
             and then Left.Info.Rebindings = Right.Info.Rebindings
             and then Compare_Metadata (Left.Info.Md, Right.Info.Md);
   end Compare_Entity;

   --------------------------------
   -- Create_Dynamic_Lexical_Env --
   --------------------------------

   function Create_Dynamic_Lexical_Env
     (Self              : Bare_Lkt_Node;
      Assocs_Getter     : Inner_Env_Assocs_Resolver;
      Assoc_Resolver    : Entity_Resolver;
      Transitive_Parent : Boolean;
      Sym_Table         : Symbol_Table) return Lexical_Env
   is
      Unit : constant Internal_Unit := Self.Unit;
   begin
      --  This restriction is necessary to avoid relocation issues when
      --  Self.Self_Env is terminated.
      if Is_Foreign_Strict (Self.Self_Env, Self) then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "cannot create a dynamic lexical env when Self.Self_Env is"
            & " foreign");
      end if;

      return Result : constant Lexical_Env := Create_Dynamic_Lexical_Env
        (Parent            => Null_Lexical_Env,
         Node              => Self,
         Transitive_Parent => Transitive_Parent,
         Owner             => Convert_Unit (Unit),
         Assocs_Getter     => Assocs_Getter,
         Assoc_Resolver    => Assoc_Resolver,
         Sym_Table         => Sym_Table)
      do
         --  Since dynamic lexical environments can only be created in lazy
         --  field initializers, it is fine to tie Result's lifetime to the
         --  its owning unit's lifetime.
         Register_Destroyable (Unit, Unwrap (Result));
      end return;
   end Create_Dynamic_Lexical_Env;

   procedure Destroy_Synthetic_Node (Node : in out Bare_Lkt_Node);
   --  Helper for the Register_Destroyable above

   ------------
   -- Length --
   ------------

   function Length (Node : Bare_Lkt_Node_Base_List) return Natural
   is (if Node = null then 0 else Children_Count (Node));


      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (B : Boolean) return String is
      begin
         return (if B then "True" else "False");
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (I : Integer) return String is
      begin
         return Integer'Image (I);
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (S : Symbol_Type) return String is
      begin
         return (if S = No_Symbol
                 then "None"
                 else Image (S, With_Quotes => True));
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (C : Character_Type) return String is
         C_Str : constant Text_Type := (1 => C);
      begin
         return "'" & Image (C_Str) & "'";
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (S : String_Type) return String is
      begin
         return Image (S.Content, With_Quotes => True);
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (Env : Lexical_Env) return String is
      begin
         case Env.Kind is
         when Static_Primary =>
            return "<LexicalEnv static-primary for "
                   & Trace_Image (Env_Node (Env)) & ">";
         when others =>
            return "<LexicalEnv synthetic>";
         end case;
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (R : Env_Rebindings) return String is
      begin
         return Image (Text_Image (R));
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (Unit : Internal_Unit) return String is
      begin
         return "Internal_Unit (""" & Basename (Unit) & """)";
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (Eq : Logic_Equation) return String is
         pragma Unreferenced (Eq);
      begin
         return "<LogicEquation>";
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (Var : Logic_Var) return String is
         pragma Unreferenced (Var);
      begin
         return "<LogicVariable>";
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (K : Analysis_Unit_Kind) return String is
      begin
         return Analysis_Unit_Kind'Image (K);
      end Trace_Image;

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image (Self : Ref_Categories) return String is
         Result : Unbounded_String;
         First  : Boolean := True;
      begin
         Append (Result, "RefCategories(");
         for C in Ref_Category loop
            if Self (C) then
               if First then
                  First := False;
               else
                  Append (Result, ", ");
               end if;
               Append (Result, C'Image);
            end if;
         end loop;
         Append (Result, ")");
         return To_String (Result);
      end Trace_Image;


   

   


      -------------
      -- Inc_Ref --
      -------------

      procedure Inc_Ref (R : Internal_Decoded_Char_Value) is
      begin
               Inc_Ref (R.Error_Message);
      end Inc_Ref;

      -------------
      -- Dec_Ref --
      -------------

      procedure Dec_Ref (R : in out Internal_Decoded_Char_Value) is
      begin
               Dec_Ref (R.Error_Message);
      end Dec_Ref;




      ----------------
      -- Equivalent --
      ----------------

      function Equivalent (L, R : Internal_Decoded_Char_Value) return Boolean is
      begin
         return L.Value = R.Value and then L.Has_Error = R.Has_Error and then L.Error_Sloc = R.Error_Sloc and then Equivalent (L.Error_Message, R.Error_Message);
      end Equivalent;


   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Decoded_Char_Value) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Has_Error => " & Trace_Image (R.Has_Error)
                        & ", "
                     & "Error_Sloc => " & Trace_Image (R.Error_Sloc)
                        & ", "
                     & "Error_Message => " & Trace_Image (R.Error_Message)
               & ")");
      end Trace_Image;


   

   


      -------------
      -- Inc_Ref --
      -------------

      procedure Inc_Ref (R : Internal_Decoded_String_Value) is
      begin
               Inc_Ref (R.Value);
               Inc_Ref (R.Error_Message);
      end Inc_Ref;

      -------------
      -- Dec_Ref --
      -------------

      procedure Dec_Ref (R : in out Internal_Decoded_String_Value) is
      begin
               Dec_Ref (R.Value);
               Dec_Ref (R.Error_Message);
      end Dec_Ref;




      ----------------
      -- Equivalent --
      ----------------

      function Equivalent (L, R : Internal_Decoded_String_Value) return Boolean is
      begin
         return Equivalent (L.Value, R.Value) and then L.Has_Error = R.Has_Error and then L.Error_Sloc = R.Error_Sloc and then Equivalent (L.Error_Message, R.Error_Message);
      end Equivalent;


   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Decoded_String_Value) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Has_Error => " & Trace_Image (R.Has_Error)
                        & ", "
                     & "Error_Sloc => " & Trace_Image (R.Error_Sloc)
                        & ", "
                     & "Error_Message => " & Trace_Image (R.Error_Message)
               & ")");
      end Trace_Image;


   

   


      -------------
      -- Inc_Ref --
      -------------

      procedure Inc_Ref (R : Internal_Designated_Env) is
      begin
               Inc_Ref (R.Direct_Env);
      end Inc_Ref;

      -------------
      -- Dec_Ref --
      -------------

      procedure Dec_Ref (R : in out Internal_Designated_Env) is
      begin
               Dec_Ref (R.Direct_Env);
      end Dec_Ref;




      ----------------
      -- Equivalent --
      ----------------

      function Equivalent (L, R : Internal_Designated_Env) return Boolean is
      begin
         return L.Kind = R.Kind and then L.Env_Name = R.Env_Name and then Equivalent (L.Direct_Env, R.Direct_Env);
      end Equivalent;


   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Designated_Env) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Kind => " & Trace_Image (R.Kind)
                        & ", "
                     & "Env_Name => " & Trace_Image (R.Env_Name)
                        & ", "
                     & "Direct_Env => " & Trace_Image (R.Direct_Env)
               & ")");
      end Trace_Image;


   

   




   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Metadata) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                  & "null record"
               & ")");
      end Trace_Image;


   

   




   

      ----------
      -- Hash --
      ----------

      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Info) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
         
            return Combine ((Hash (R.Md), Hash (R.Rebindings), Hash (R.From_Rebound)));
      end Hash;


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Info) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Md => " & Trace_Image (R.Md)
                        & ", "
                     & "Rebindings => " & Trace_Image (R.Rebindings)
                        & ", "
                     & "From_Rebound => " & Trace_Image (R.From_Rebound)
               & ")");
      end Trace_Image;


   

   



      function Create_Internal_Entity
        (Node : Bare_Lkt_Node; Info : Internal_Entity_Info)
         return Internal_Entity is
      begin
         if Node = null then
            return No_Entity;
         end if;
         return (Node => Node, Info => Info);
      end;



   

      ----------
      -- Hash --
      ----------

      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
         
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Expr
        (Node : Bare_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Expr is
      begin
         if Node = null then
            return No_Entity_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Any_Of
        (Node : Bare_Any_Of; Info : Internal_Entity_Info)
         return Internal_Entity_Any_Of is
      begin
         if Node = null then
            return No_Entity_Any_Of;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Any_Of) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lkt_Node_Base_List
        (Node : Bare_Lkt_Node_Base_List; Info : Internal_Entity_Info)
         return Internal_Entity_Lkt_Node_Base_List is
      begin
         if Node = null then
            return No_Entity_Lkt_Node_Base_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lkt_Node_Base_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Expr_List
        (Node : Bare_Expr_List; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_List is
      begin
         if Node = null then
            return No_Entity_Expr_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Any_Of_List
        (Node : Bare_Any_Of_List; Info : Internal_Entity_Info)
         return Internal_Entity_Any_Of_List is
      begin
         if Node = null then
            return No_Entity_Any_Of_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Any_Of_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Array_Literal
        (Node : Bare_Array_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Literal is
      begin
         if Node = null then
            return No_Entity_Array_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Array_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Base_Dot_Expr
        (Node : Bare_Base_Dot_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Dot_Expr is
      begin
         if Node = null then
            return No_Entity_Base_Dot_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Dot_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Decl
        (Node : Bare_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Decl is
      begin
         if Node = null then
            return No_Entity_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Base_Grammar_Rule_Decl
        (Node : Bare_Base_Grammar_Rule_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Grammar_Rule_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Grammar_Rule_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Grammar_Rule_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Base_Lexer_Case_Rule_Alt
        (Node : Bare_Base_Lexer_Case_Rule_Alt; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Lexer_Case_Rule_Alt is
      begin
         if Node = null then
            return No_Entity_Base_Lexer_Case_Rule_Alt;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Lexer_Case_Rule_Alt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Base_Lexer_Case_Rule_Alt_List
        (Node : Bare_Base_Lexer_Case_Rule_Alt_List; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Lexer_Case_Rule_Alt_List is
      begin
         if Node = null then
            return No_Entity_Base_Lexer_Case_Rule_Alt_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Lexer_Case_Rule_Alt_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Base_Val_Decl
        (Node : Bare_Base_Val_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Val_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Val_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Val_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Type_Decl
        (Node : Bare_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Named_Type_Decl
        (Node : Bare_Named_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Named_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Named_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Named_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Basic_Class_Decl
        (Node : Bare_Basic_Class_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Class_Decl is
      begin
         if Node = null then
            return No_Entity_Basic_Class_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Class_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lit
        (Node : Bare_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Lit is
      begin
         if Node = null then
            return No_Entity_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Big_Num_Lit
        (Node : Bare_Big_Num_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Big_Num_Lit is
      begin
         if Node = null then
            return No_Entity_Big_Num_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Big_Num_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Bin_Op
        (Node : Bare_Bin_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Bin_Op is
      begin
         if Node = null then
            return No_Entity_Bin_Op;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bin_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lkt_Node_List
        (Node : Bare_Lkt_Node_List; Info : Internal_Entity_Info)
         return Internal_Entity_Lkt_Node_List is
      begin
         if Node = null then
            return No_Entity_Lkt_Node_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lkt_Node_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Block_Decl_List
        (Node : Bare_Block_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Block_Decl_List is
      begin
         if Node = null then
            return No_Entity_Block_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Block_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Block_Expr
        (Node : Bare_Block_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Block_Expr is
      begin
         if Node = null then
            return No_Entity_Block_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Block_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Block_String_Line
        (Node : Bare_Block_String_Line; Info : Internal_Entity_Info)
         return Internal_Entity_Block_String_Line is
      begin
         if Node = null then
            return No_Entity_Block_String_Line;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Block_String_Line) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Block_String_Line_List
        (Node : Bare_Block_String_Line_List; Info : Internal_Entity_Info)
         return Internal_Entity_Block_String_Line_List is
      begin
         if Node = null then
            return No_Entity_Block_String_Line_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Block_String_Line_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_String_Lit
        (Node : Bare_String_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_String_Lit is
      begin
         if Node = null then
            return No_Entity_String_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_String_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Block_String_Lit
        (Node : Bare_Block_String_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Block_String_Lit is
      begin
         if Node = null then
            return No_Entity_Block_String_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Block_String_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Call_Expr
        (Node : Bare_Call_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Expr is
      begin
         if Node = null then
            return No_Entity_Call_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Call_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Call_Expr_List
        (Node : Bare_Call_Expr_List; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Expr_List is
      begin
         if Node = null then
            return No_Entity_Call_Expr_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Call_Expr_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Cast_Expr
        (Node : Bare_Cast_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Cast_Expr is
      begin
         if Node = null then
            return No_Entity_Cast_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Cast_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Char_Lit
        (Node : Bare_Char_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Char_Lit is
      begin
         if Node = null then
            return No_Entity_Char_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Char_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Class_Decl
        (Node : Bare_Class_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Class_Decl is
      begin
         if Node = null then
            return No_Entity_Class_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Class_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Class_Qualifier
        (Node : Bare_Class_Qualifier; Info : Internal_Entity_Info)
         return Internal_Entity_Class_Qualifier is
      begin
         if Node = null then
            return No_Entity_Class_Qualifier;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Class_Qualifier) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Class_Qualifier_Absent
        (Node : Bare_Class_Qualifier_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Class_Qualifier_Absent is
      begin
         if Node = null then
            return No_Entity_Class_Qualifier_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Class_Qualifier_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Class_Qualifier_Present
        (Node : Bare_Class_Qualifier_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Class_Qualifier_Present is
      begin
         if Node = null then
            return No_Entity_Class_Qualifier_Present;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Class_Qualifier_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_User_Val_Decl
        (Node : Bare_User_Val_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_User_Val_Decl is
      begin
         if Node = null then
            return No_Entity_User_Val_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_User_Val_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Explicitly_Typed_Decl
        (Node : Bare_Explicitly_Typed_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Explicitly_Typed_Decl is
      begin
         if Node = null then
            return No_Entity_Explicitly_Typed_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Explicitly_Typed_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Component_Decl
        (Node : Bare_Component_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Decl is
      begin
         if Node = null then
            return No_Entity_Component_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Decl_Annotation
        (Node : Bare_Decl_Annotation; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Annotation is
      begin
         if Node = null then
            return No_Entity_Decl_Annotation;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Annotation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Decl_Annotation_List
        (Node : Bare_Decl_Annotation_List; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Annotation_List is
      begin
         if Node = null then
            return No_Entity_Decl_Annotation_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Annotation_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Decl_Annotation_Params
        (Node : Bare_Decl_Annotation_Params; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Annotation_Params is
      begin
         if Node = null then
            return No_Entity_Decl_Annotation_Params;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Annotation_Params) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Full_Decl_List
        (Node : Bare_Full_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Full_Decl_List is
      begin
         if Node = null then
            return No_Entity_Full_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Full_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Decl_Block
        (Node : Bare_Decl_Block; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Block is
      begin
         if Node = null then
            return No_Entity_Decl_Block;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Block) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Id
        (Node : Bare_Id; Info : Internal_Entity_Info)
         return Internal_Entity_Id is
      begin
         if Node = null then
            return No_Entity_Id;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Id) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Def_Id
        (Node : Bare_Def_Id; Info : Internal_Entity_Info)
         return Internal_Entity_Def_Id is
      begin
         if Node = null then
            return No_Entity_Def_Id;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Def_Id) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Type_Ref
        (Node : Bare_Type_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Ref is
      begin
         if Node = null then
            return No_Entity_Type_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Default_List_Type_Ref
        (Node : Bare_Default_List_Type_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Default_List_Type_Ref is
      begin
         if Node = null then
            return No_Entity_Default_List_Type_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Default_List_Type_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Dot_Expr
        (Node : Bare_Dot_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Dot_Expr is
      begin
         if Node = null then
            return No_Entity_Dot_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Dot_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Dyn_Var_Decl
        (Node : Bare_Dyn_Var_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Dyn_Var_Decl is
      begin
         if Node = null then
            return No_Entity_Dyn_Var_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Dyn_Var_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Elsif_Branch
        (Node : Bare_Elsif_Branch; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Branch is
      begin
         if Node = null then
            return No_Entity_Elsif_Branch;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Branch) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Elsif_Branch_List
        (Node : Bare_Elsif_Branch_List; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Branch_List is
      begin
         if Node = null then
            return No_Entity_Elsif_Branch_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Branch_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Enum_Class_Alt_Decl
        (Node : Bare_Enum_Class_Alt_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Class_Alt_Decl is
      begin
         if Node = null then
            return No_Entity_Enum_Class_Alt_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Class_Alt_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Enum_Class_Alt_Decl_List
        (Node : Bare_Enum_Class_Alt_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Class_Alt_Decl_List is
      begin
         if Node = null then
            return No_Entity_Enum_Class_Alt_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Class_Alt_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Enum_Class_Case
        (Node : Bare_Enum_Class_Case; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Class_Case is
      begin
         if Node = null then
            return No_Entity_Enum_Class_Case;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Class_Case) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Enum_Class_Case_List
        (Node : Bare_Enum_Class_Case_List; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Class_Case_List is
      begin
         if Node = null then
            return No_Entity_Enum_Class_Case_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Class_Case_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Enum_Class_Decl
        (Node : Bare_Enum_Class_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Class_Decl is
      begin
         if Node = null then
            return No_Entity_Enum_Class_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Class_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Enum_Lit_Decl
        (Node : Bare_Enum_Lit_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Lit_Decl is
      begin
         if Node = null then
            return No_Entity_Enum_Lit_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Lit_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Enum_Lit_Decl_List
        (Node : Bare_Enum_Lit_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Lit_Decl_List is
      begin
         if Node = null then
            return No_Entity_Enum_Lit_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Lit_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Enum_Type_Decl
        (Node : Bare_Enum_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Enum_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Env_Spec_Decl
        (Node : Bare_Env_Spec_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Env_Spec_Decl is
      begin
         if Node = null then
            return No_Entity_Env_Spec_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Env_Spec_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Error_On_Null
        (Node : Bare_Error_On_Null; Info : Internal_Entity_Info)
         return Internal_Entity_Error_On_Null is
      begin
         if Node = null then
            return No_Entity_Error_On_Null;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Error_On_Null) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Excludes_Null
        (Node : Bare_Excludes_Null; Info : Internal_Entity_Info)
         return Internal_Entity_Excludes_Null is
      begin
         if Node = null then
            return No_Entity_Excludes_Null;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Excludes_Null) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Excludes_Null_Absent
        (Node : Bare_Excludes_Null_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Excludes_Null_Absent is
      begin
         if Node = null then
            return No_Entity_Excludes_Null_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Excludes_Null_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Excludes_Null_Present
        (Node : Bare_Excludes_Null_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Excludes_Null_Present is
      begin
         if Node = null then
            return No_Entity_Excludes_Null_Present;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Excludes_Null_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Field_Decl
        (Node : Bare_Field_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Field_Decl is
      begin
         if Node = null then
            return No_Entity_Field_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Field_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Full_Decl
        (Node : Bare_Full_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Full_Decl is
      begin
         if Node = null then
            return No_Entity_Full_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Full_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Fun_Arg_Decl
        (Node : Bare_Fun_Arg_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Fun_Arg_Decl is
      begin
         if Node = null then
            return No_Entity_Fun_Arg_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Fun_Arg_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Fun_Arg_Decl_List
        (Node : Bare_Fun_Arg_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Fun_Arg_Decl_List is
      begin
         if Node = null then
            return No_Entity_Fun_Arg_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Fun_Arg_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Fun_Decl
        (Node : Bare_Fun_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Fun_Decl is
      begin
         if Node = null then
            return No_Entity_Fun_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Fun_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Function_Type
        (Node : Bare_Function_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Function_Type is
      begin
         if Node = null then
            return No_Entity_Function_Type;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Function_Type) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Function_Type_Ref
        (Node : Bare_Function_Type_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Function_Type_Ref is
      begin
         if Node = null then
            return No_Entity_Function_Type_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Function_Type_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Generic_Decl
        (Node : Bare_Generic_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Generic_Formal_Decl_List
        (Node : Bare_Generic_Formal_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Decl_List is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node : Bare_Generic_Formal_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Generic_Instantiation
        (Node : Bare_Generic_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Generic_Type_Ref
        (Node : Bare_Generic_Type_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Type_Ref is
      begin
         if Node = null then
            return No_Entity_Generic_Type_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Type_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Expr
        (Node : Bare_Grammar_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Expr is
      begin
         if Node = null then
            return No_Entity_Grammar_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Cut
        (Node : Bare_Grammar_Cut; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Cut is
      begin
         if Node = null then
            return No_Entity_Grammar_Cut;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Cut) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Decl
        (Node : Bare_Grammar_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Decl is
      begin
         if Node = null then
            return No_Entity_Grammar_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Discard
        (Node : Bare_Grammar_Discard; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Discard is
      begin
         if Node = null then
            return No_Entity_Grammar_Discard;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Discard) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Dont_Skip
        (Node : Bare_Grammar_Dont_Skip; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Dont_Skip is
      begin
         if Node = null then
            return No_Entity_Grammar_Dont_Skip;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Dont_Skip) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Expr_List
        (Node : Bare_Grammar_Expr_List; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Expr_List is
      begin
         if Node = null then
            return No_Entity_Grammar_Expr_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Expr_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Expr_List_List
        (Node : Bare_Grammar_Expr_List_List; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Expr_List_List is
      begin
         if Node = null then
            return No_Entity_Grammar_Expr_List_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Expr_List_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Pick
        (Node : Bare_Grammar_Pick; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Pick is
      begin
         if Node = null then
            return No_Entity_Grammar_Pick;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Pick) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Implicit_Pick
        (Node : Bare_Grammar_Implicit_Pick; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Implicit_Pick is
      begin
         if Node = null then
            return No_Entity_Grammar_Implicit_Pick;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Implicit_Pick) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_List
        (Node : Bare_Grammar_List; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_List is
      begin
         if Node = null then
            return No_Entity_Grammar_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_List_Sep
        (Node : Bare_Grammar_List_Sep; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_List_Sep is
      begin
         if Node = null then
            return No_Entity_Grammar_List_Sep;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_List_Sep) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Null
        (Node : Bare_Grammar_Null; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Null is
      begin
         if Node = null then
            return No_Entity_Grammar_Null;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Null) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Opt
        (Node : Bare_Grammar_Opt; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Opt is
      begin
         if Node = null then
            return No_Entity_Grammar_Opt;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Opt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Opt_Error
        (Node : Bare_Grammar_Opt_Error; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Opt_Error is
      begin
         if Node = null then
            return No_Entity_Grammar_Opt_Error;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Opt_Error) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Opt_Error_Group
        (Node : Bare_Grammar_Opt_Error_Group; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Opt_Error_Group is
      begin
         if Node = null then
            return No_Entity_Grammar_Opt_Error_Group;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Opt_Error_Group) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Opt_Group
        (Node : Bare_Grammar_Opt_Group; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Opt_Group is
      begin
         if Node = null then
            return No_Entity_Grammar_Opt_Group;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Opt_Group) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Or_Expr
        (Node : Bare_Grammar_Or_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Or_Expr is
      begin
         if Node = null then
            return No_Entity_Grammar_Or_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Or_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Predicate
        (Node : Bare_Grammar_Predicate; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Predicate is
      begin
         if Node = null then
            return No_Entity_Grammar_Predicate;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Predicate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Rule_Decl
        (Node : Bare_Grammar_Rule_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Rule_Decl is
      begin
         if Node = null then
            return No_Entity_Grammar_Rule_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Rule_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Rule_Ref
        (Node : Bare_Grammar_Rule_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Rule_Ref is
      begin
         if Node = null then
            return No_Entity_Grammar_Rule_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Rule_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Skip
        (Node : Bare_Grammar_Skip; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Skip is
      begin
         if Node = null then
            return No_Entity_Grammar_Skip;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Skip) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Grammar_Stop_Cut
        (Node : Bare_Grammar_Stop_Cut; Info : Internal_Entity_Info)
         return Internal_Entity_Grammar_Stop_Cut is
      begin
         if Node = null then
            return No_Entity_Grammar_Stop_Cut;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Grammar_Stop_Cut) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_If_Expr
        (Node : Bare_If_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_If_Expr is
      begin
         if Node = null then
            return No_Entity_If_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_If_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Import
        (Node : Bare_Import; Info : Internal_Entity_Info)
         return Internal_Entity_Import is
      begin
         if Node = null then
            return No_Entity_Import;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Import) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Import_List
        (Node : Bare_Import_List; Info : Internal_Entity_Info)
         return Internal_Entity_Import_List is
      begin
         if Node = null then
            return No_Entity_Import_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Import_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Instantiated_Generic_Type
        (Node : Bare_Instantiated_Generic_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Instantiated_Generic_Type is
      begin
         if Node = null then
            return No_Entity_Instantiated_Generic_Type;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Instantiated_Generic_Type) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Isa
        (Node : Bare_Isa; Info : Internal_Entity_Info)
         return Internal_Entity_Isa is
      begin
         if Node = null then
            return No_Entity_Isa;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Isa) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Type_Ref_List
        (Node : Bare_Type_Ref_List; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Ref_List is
      begin
         if Node = null then
            return No_Entity_Type_Ref_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Ref_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Isa_List
        (Node : Bare_Isa_List; Info : Internal_Entity_Info)
         return Internal_Entity_Isa_List is
      begin
         if Node = null then
            return No_Entity_Isa_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Isa_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Keep_Expr
        (Node : Bare_Keep_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Keep_Expr is
      begin
         if Node = null then
            return No_Entity_Keep_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Keep_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lambda_Arg_Decl
        (Node : Bare_Lambda_Arg_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Lambda_Arg_Decl is
      begin
         if Node = null then
            return No_Entity_Lambda_Arg_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lambda_Arg_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lambda_Arg_Decl_List
        (Node : Bare_Lambda_Arg_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Lambda_Arg_Decl_List is
      begin
         if Node = null then
            return No_Entity_Lambda_Arg_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lambda_Arg_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lambda_Expr
        (Node : Bare_Lambda_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Lambda_Expr is
      begin
         if Node = null then
            return No_Entity_Lambda_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lambda_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Langkit_Root
        (Node : Bare_Langkit_Root; Info : Internal_Entity_Info)
         return Internal_Entity_Langkit_Root is
      begin
         if Node = null then
            return No_Entity_Langkit_Root;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Langkit_Root) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lexer_Case_Rule
        (Node : Bare_Lexer_Case_Rule; Info : Internal_Entity_Info)
         return Internal_Entity_Lexer_Case_Rule is
      begin
         if Node = null then
            return No_Entity_Lexer_Case_Rule;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lexer_Case_Rule) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lexer_Case_Rule_Cond_Alt
        (Node : Bare_Lexer_Case_Rule_Cond_Alt; Info : Internal_Entity_Info)
         return Internal_Entity_Lexer_Case_Rule_Cond_Alt is
      begin
         if Node = null then
            return No_Entity_Lexer_Case_Rule_Cond_Alt;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lexer_Case_Rule_Cond_Alt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lexer_Case_Rule_Default_Alt
        (Node : Bare_Lexer_Case_Rule_Default_Alt; Info : Internal_Entity_Info)
         return Internal_Entity_Lexer_Case_Rule_Default_Alt is
      begin
         if Node = null then
            return No_Entity_Lexer_Case_Rule_Default_Alt;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lexer_Case_Rule_Default_Alt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lexer_Case_Rule_Send
        (Node : Bare_Lexer_Case_Rule_Send; Info : Internal_Entity_Info)
         return Internal_Entity_Lexer_Case_Rule_Send is
      begin
         if Node = null then
            return No_Entity_Lexer_Case_Rule_Send;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lexer_Case_Rule_Send) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lexer_Decl
        (Node : Bare_Lexer_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Lexer_Decl is
      begin
         if Node = null then
            return No_Entity_Lexer_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lexer_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Lexer_Family_Decl
        (Node : Bare_Lexer_Family_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Lexer_Family_Decl is
      begin
         if Node = null then
            return No_Entity_Lexer_Family_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Lexer_Family_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_List_Kind
        (Node : Bare_List_Kind; Info : Internal_Entity_Info)
         return Internal_Entity_List_Kind is
      begin
         if Node = null then
            return No_Entity_List_Kind;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_List_Kind) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_List_Kind_One
        (Node : Bare_List_Kind_One; Info : Internal_Entity_Info)
         return Internal_Entity_List_Kind_One is
      begin
         if Node = null then
            return No_Entity_List_Kind_One;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_List_Kind_One) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_List_Kind_Zero
        (Node : Bare_List_Kind_Zero; Info : Internal_Entity_Info)
         return Internal_Entity_List_Kind_Zero is
      begin
         if Node = null then
            return No_Entity_List_Kind_Zero;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_List_Kind_Zero) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Logic_Expr
        (Node : Bare_Logic_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Logic_Expr is
      begin
         if Node = null then
            return No_Entity_Logic_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Logic_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Match_Branch
        (Node : Bare_Match_Branch; Info : Internal_Entity_Info)
         return Internal_Entity_Match_Branch is
      begin
         if Node = null then
            return No_Entity_Match_Branch;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Match_Branch) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Match_Branch_List
        (Node : Bare_Match_Branch_List; Info : Internal_Entity_Info)
         return Internal_Entity_Match_Branch_List is
      begin
         if Node = null then
            return No_Entity_Match_Branch_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Match_Branch_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Match_Expr
        (Node : Bare_Match_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Match_Expr is
      begin
         if Node = null then
            return No_Entity_Match_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Match_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Match_Val_Decl
        (Node : Bare_Match_Val_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Match_Val_Decl is
      begin
         if Node = null then
            return No_Entity_Match_Val_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Match_Val_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Module_Ref_Id
        (Node : Bare_Module_Ref_Id; Info : Internal_Entity_Info)
         return Internal_Entity_Module_Ref_Id is
      begin
         if Node = null then
            return No_Entity_Module_Ref_Id;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Module_Ref_Id) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Node_Decl
        (Node : Bare_Node_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Node_Decl is
      begin
         if Node = null then
            return No_Entity_Node_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Node_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Not_Expr
        (Node : Bare_Not_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Expr is
      begin
         if Node = null then
            return No_Entity_Not_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Null_Cond_Dotted_Name
        (Node : Bare_Null_Cond_Dotted_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Cond_Dotted_Name is
      begin
         if Node = null then
            return No_Entity_Null_Cond_Dotted_Name;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Cond_Dotted_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Subscript_Expr
        (Node : Bare_Subscript_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Subscript_Expr is
      begin
         if Node = null then
            return No_Entity_Subscript_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subscript_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Null_Cond_Subscript_Expr
        (Node : Bare_Null_Cond_Subscript_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Cond_Subscript_Expr is
      begin
         if Node = null then
            return No_Entity_Null_Cond_Subscript_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Cond_Subscript_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Null_Lit
        (Node : Bare_Null_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Lit is
      begin
         if Node = null then
            return No_Entity_Null_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Num_Lit
        (Node : Bare_Num_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Num_Lit is
      begin
         if Node = null then
            return No_Entity_Num_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Num_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op
        (Node : Bare_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Op is
      begin
         if Node = null then
            return No_Entity_Op;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Amp
        (Node : Bare_Op_Amp; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Amp is
      begin
         if Node = null then
            return No_Entity_Op_Amp;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Amp) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_And
        (Node : Bare_Op_And; Info : Internal_Entity_Info)
         return Internal_Entity_Op_And is
      begin
         if Node = null then
            return No_Entity_Op_And;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_And) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Div
        (Node : Bare_Op_Div; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Div is
      begin
         if Node = null then
            return No_Entity_Op_Div;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Div) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Eq
        (Node : Bare_Op_Eq; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Eq is
      begin
         if Node = null then
            return No_Entity_Op_Eq;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Eq) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Gt
        (Node : Bare_Op_Gt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gt is
      begin
         if Node = null then
            return No_Entity_Op_Gt;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Gt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Gte
        (Node : Bare_Op_Gte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gte is
      begin
         if Node = null then
            return No_Entity_Op_Gte;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Gte) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Lt
        (Node : Bare_Op_Lt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lt is
      begin
         if Node = null then
            return No_Entity_Op_Lt;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Lt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Lte
        (Node : Bare_Op_Lte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lte is
      begin
         if Node = null then
            return No_Entity_Op_Lte;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Lte) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Minus
        (Node : Bare_Op_Minus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Minus is
      begin
         if Node = null then
            return No_Entity_Op_Minus;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Minus) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Mult
        (Node : Bare_Op_Mult; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Mult is
      begin
         if Node = null then
            return No_Entity_Op_Mult;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Mult) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Ne
        (Node : Bare_Op_Ne; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Ne is
      begin
         if Node = null then
            return No_Entity_Op_Ne;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Ne) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Or
        (Node : Bare_Op_Or; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or is
      begin
         if Node = null then
            return No_Entity_Op_Or;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Or) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Or_Int
        (Node : Bare_Op_Or_Int; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or_Int is
      begin
         if Node = null then
            return No_Entity_Op_Or_Int;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Or_Int) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Op_Plus
        (Node : Bare_Op_Plus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Plus is
      begin
         if Node = null then
            return No_Entity_Op_Plus;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Plus) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Param
        (Node : Bare_Param; Info : Internal_Entity_Info)
         return Internal_Entity_Param is
      begin
         if Node = null then
            return No_Entity_Param;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Param_List
        (Node : Bare_Param_List; Info : Internal_Entity_Info)
         return Internal_Entity_Param_List is
      begin
         if Node = null then
            return No_Entity_Param_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Paren_Expr
        (Node : Bare_Paren_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Paren_Expr is
      begin
         if Node = null then
            return No_Entity_Paren_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Paren_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Parse_Node_Expr
        (Node : Bare_Parse_Node_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Parse_Node_Expr is
      begin
         if Node = null then
            return No_Entity_Parse_Node_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Parse_Node_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Single_Line_String_Lit
        (Node : Bare_Single_Line_String_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Line_String_Lit is
      begin
         if Node = null then
            return No_Entity_Single_Line_String_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Line_String_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Pattern_Single_Line_String_Lit
        (Node : Bare_Pattern_Single_Line_String_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Pattern_Single_Line_String_Lit is
      begin
         if Node = null then
            return No_Entity_Pattern_Single_Line_String_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pattern_Single_Line_String_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Raise_Expr
        (Node : Bare_Raise_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Raise_Expr is
      begin
         if Node = null then
            return No_Entity_Raise_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Raise_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Ref_Id
        (Node : Bare_Ref_Id; Info : Internal_Entity_Info)
         return Internal_Entity_Ref_Id is
      begin
         if Node = null then
            return No_Entity_Ref_Id;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ref_Id) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Ref_Id_List
        (Node : Bare_Ref_Id_List; Info : Internal_Entity_Info)
         return Internal_Entity_Ref_Id_List is
      begin
         if Node = null then
            return No_Entity_Ref_Id_List;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ref_Id_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Self_Decl
        (Node : Bare_Self_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Self_Decl is
      begin
         if Node = null then
            return No_Entity_Self_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Self_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Simple_Type_Ref
        (Node : Bare_Simple_Type_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Simple_Type_Ref is
      begin
         if Node = null then
            return No_Entity_Simple_Type_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Simple_Type_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Struct_Decl
        (Node : Bare_Struct_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Struct_Decl is
      begin
         if Node = null then
            return No_Entity_Struct_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Struct_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Synthetic_Lexer_Decl
        (Node : Bare_Synthetic_Lexer_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Lexer_Decl is
      begin
         if Node = null then
            return No_Entity_Synthetic_Lexer_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Lexer_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Token_Lit
        (Node : Bare_Token_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Token_Lit is
      begin
         if Node = null then
            return No_Entity_Token_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Token_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Token_No_Case_Lit
        (Node : Bare_Token_No_Case_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Token_No_Case_Lit is
      begin
         if Node = null then
            return No_Entity_Token_No_Case_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Token_No_Case_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Token_Pattern_Lit
        (Node : Bare_Token_Pattern_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Token_Pattern_Lit is
      begin
         if Node = null then
            return No_Entity_Token_Pattern_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Token_Pattern_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Token_Ref
        (Node : Bare_Token_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Token_Ref is
      begin
         if Node = null then
            return No_Entity_Token_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Token_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Trait_Decl
        (Node : Bare_Trait_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Trait_Decl is
      begin
         if Node = null then
            return No_Entity_Trait_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Trait_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Try_Expr
        (Node : Bare_Try_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Try_Expr is
      begin
         if Node = null then
            return No_Entity_Try_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Try_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Un_Op
        (Node : Bare_Un_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Un_Op is
      begin
         if Node = null then
            return No_Entity_Un_Op;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Un_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Val_Decl
        (Node : Bare_Val_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Val_Decl is
      begin
         if Node = null then
            return No_Entity_Val_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Val_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   



      function Create_Internal_Entity_Var_Bind
        (Node : Bare_Var_Bind; Info : Internal_Entity_Info)
         return Internal_Entity_Var_Bind is
      begin
         if Node = null then
            return No_Entity_Var_Bind;
         end if;
         return (Node => Node, Info => Info);
      end;



   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Var_Bind) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;


   

   


      -------------
      -- Inc_Ref --
      -------------

      procedure Inc_Ref (R : Internal_Env_Assoc) is
      begin
               Inc_Ref (R.Dest_Env);
      end Inc_Ref;

      -------------
      -- Dec_Ref --
      -------------

      procedure Dec_Ref (R : in out Internal_Env_Assoc) is
      begin
               Dec_Ref (R.Dest_Env);
      end Dec_Ref;




      ----------------
      -- Equivalent --
      ----------------

      function Equivalent (L, R : Internal_Env_Assoc) return Boolean is
      begin
         return L.Key = R.Key and then L.Value = R.Value and then Equivalent (L.Dest_Env, R.Dest_Env) and then L.Metadata = R.Metadata;
      end Equivalent;


   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Env_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Dest_Env => " & Trace_Image (R.Dest_Env)
                        & ", "
                     & "Metadata => " & Trace_Image (R.Metadata)
               & ")");
      end Trace_Image;


   

   




   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Env_Kv) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;


   

   




   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Formal_Param) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Formal_Name => " & Trace_Image (R.Formal_Name)
                        & ", "
                     & "Formal_Type => " & Trace_Image (R.Formal_Type)
                        & ", "
                     & "Default_Value => " & Trace_Image (R.Default_Value)
               & ")");
      end Trace_Image;


   

   




   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Generic_Param_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Formal => " & Trace_Image (R.Formal)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
               & ")");
      end Trace_Image;


   

   


      -------------
      -- Inc_Ref --
      -------------

      procedure Inc_Ref (R : Internal_Semantic_Result) is
      begin
               Inc_Ref (R.Error_Message);
      end Inc_Ref;

      -------------
      -- Dec_Ref --
      -------------

      procedure Dec_Ref (R : in out Internal_Semantic_Result) is
      begin
               Dec_Ref (R.Error_Message);
      end Dec_Ref;




      ----------------
      -- Equivalent --
      ----------------

      function Equivalent (L, R : Internal_Semantic_Result) return Boolean is
      begin
         return L.Node = R.Node and then L.Result_Type = R.Result_Type and then L.Result_Decl = R.Result_Decl and then L.Has_Error = R.Has_Error and then Equivalent (L.Error_Message, R.Error_Message) and then L.Exempt_Analysis = R.Exempt_Analysis;
      end Equivalent;


   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Semantic_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Node => " & Trace_Image (R.Node)
                        & ", "
                     & "Result_Type => " & Trace_Image (R.Result_Type)
                        & ", "
                     & "Result_Decl => " & Trace_Image (R.Result_Decl)
                        & ", "
                     & "Has_Error => " & Trace_Image (R.Has_Error)
                        & ", "
                     & "Error_Message => " & Trace_Image (R.Error_Message)
                        & ", "
                     & "Exempt_Analysis => " & Trace_Image (R.Exempt_Analysis)
               & ")");
      end Trace_Image;


   

   


      -------------
      -- Inc_Ref --
      -------------

      procedure Inc_Ref (R : Internal_Infer_Instantiation) is
      begin
               Inc_Ref (R.Result);
               Inc_Ref (R.Error);
      end Inc_Ref;

      -------------
      -- Dec_Ref --
      -------------

      procedure Dec_Ref (R : in out Internal_Infer_Instantiation) is
      begin
               Dec_Ref (R.Result);
               Dec_Ref (R.Error);
      end Dec_Ref;




      ----------------
      -- Equivalent --
      ----------------

      function Equivalent (L, R : Internal_Infer_Instantiation) return Boolean is
      begin
         return Equivalent (L.Result, R.Result) and then Equivalent (L.Error, R.Error);
      end Equivalent;


   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Infer_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Result => " & Trace_Image (R.Result)
                        & ", "
                     & "Error => " & Trace_Image (R.Error)
               & ")");
      end Trace_Image;


   

   




   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Inner_Env_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Rebindings => " & Trace_Image (R.Rebindings)
                        & ", "
                     & "Metadata => " & Trace_Image (R.Metadata)
               & ")");
      end Trace_Image;


   

   




   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Logic_Context) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Ref_Node => " & Trace_Image (R.Ref_Node)
                        & ", "
                     & "Decl_Node => " & Trace_Image (R.Decl_Node)
               & ")");
      end Trace_Image;


   

   




   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Param_Match) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Has_Matched => " & Trace_Image (R.Has_Matched)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
                        & ", "
                     & "Formal => " & Trace_Image (R.Formal)
               & ")");
      end Trace_Image;


   

   


      -------------
      -- Inc_Ref --
      -------------

      procedure Inc_Ref (R : Internal_Solver_Diagnostic) is
      begin
               Inc_Ref (R.Message_Template);
               Inc_Ref (R.Args);
               Inc_Ref (R.Contexts);
      end Inc_Ref;

      -------------
      -- Dec_Ref --
      -------------

      procedure Dec_Ref (R : in out Internal_Solver_Diagnostic) is
      begin
               Dec_Ref (R.Message_Template);
               Dec_Ref (R.Args);
               Dec_Ref (R.Contexts);
      end Dec_Ref;




      ----------------
      -- Equivalent --
      ----------------

      function Equivalent (L, R : Internal_Solver_Diagnostic) return Boolean is
      begin
         return Equivalent (L.Message_Template, R.Message_Template) and then Equivalent (L.Args, R.Args) and then L.Location = R.Location and then Equivalent (L.Contexts, R.Contexts) and then L.Round = R.Round;
      end Equivalent;


   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Solver_Diagnostic) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Message_Template => " & Trace_Image (R.Message_Template)
                        & ", "
                     & "Args => " & Trace_Image (R.Args)
                        & ", "
                     & "Location => " & Trace_Image (R.Location)
                        & ", "
                     & "Contexts => " & Trace_Image (R.Contexts)
                        & ", "
                     & "Round => " & Trace_Image (R.Round)
               & ")");
      end Trace_Image;


   

   


      -------------
      -- Inc_Ref --
      -------------

      procedure Inc_Ref (R : Internal_Solver_Result) is
      begin
               Inc_Ref (R.Diagnostics);
      end Inc_Ref;

      -------------
      -- Dec_Ref --
      -------------

      procedure Dec_Ref (R : in out Internal_Solver_Result) is
      begin
               Dec_Ref (R.Diagnostics);
      end Dec_Ref;




      ----------------
      -- Equivalent --
      ----------------

      function Equivalent (L, R : Internal_Solver_Result) return Boolean is
      begin
         return L.Success = R.Success and then Equivalent (L.Diagnostics, R.Diagnostics);
      end Equivalent;


   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Solver_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Success => " & Trace_Image (R.Success)
                        & ", "
                     & "Diagnostics => " & Trace_Image (R.Diagnostics)
               & ")");
      end Trace_Image;


   

   


      -------------
      -- Inc_Ref --
      -------------

      procedure Inc_Ref (R : Internal_Tree_Semantic_Result) is
      begin
               Inc_Ref (R.Results);
      end Inc_Ref;

      -------------
      -- Dec_Ref --
      -------------

      procedure Dec_Ref (R : in out Internal_Tree_Semantic_Result) is
      begin
               Dec_Ref (R.Results);
      end Dec_Ref;




      ----------------
      -- Equivalent --
      ----------------

      function Equivalent (L, R : Internal_Tree_Semantic_Result) return Boolean is
      begin
         return Equivalent (L.Results, R.Results) and then L.Has_Error = R.Has_Error;
      end Equivalent;


   


      -----------------
      -- Trace_Image --
      -----------------

      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Tree_Semantic_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Results => " & Trace_Image (R.Results)
                        & ", "
                     & "Has_Error => " & Trace_Image (R.Has_Error)
               & ")");
      end Trace_Image;



   
         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         

         



      

   --
   --  Primitives for Bare_Lkt_Node
   --

   





   







--# property-start LktNode.concat_tree_semantic_results /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:139
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Concat_Tree_Semantic_Results
  
  (Node : Bare_Lkt_Node
      ; Res : Internal_Tree_Semantic_Result_Array_Access
  )

   return Internal_Tree_Semantic_Result
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind res Res

   Property_Result : Internal_Tree_Semantic_Result;

      
            procedure Finalizer_Scope_6 with Inline_Always;
            procedure Finalizer_Scope_311 with Inline_Always;

      Fld : Internal_Semantic_Result_Array_Access;
Map_Result : Internal_Semantic_Result_Array_Access;
Fld_1 : Boolean;
Quantifier_Result : Boolean;
New_Struct : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Let_Result : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;

            procedure Finalizer_Scope_6 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (New_Struct);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_6;
            procedure Finalizer_Scope_311 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_311;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 6 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5 '<New TreeSemanticResult at parser.py:145>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:145
--# expr-start 2 '<AnyQuantifier at parser.py:147>' Quantifier_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:147








   

   

   
      Quantifier_Result := False;
   

   

      

      declare
         
         Collection : constant Internal_Tree_Semantic_Result_Array_Access := Res;
      begin
         for Item_1 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind r Item_1

            
      --# expr-start 1 '<FieldAccess for has_error at parser.py:147>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:147






   




Fld_1 := Item_1.Has_Error;
--# expr-done 1
      Quantifier_Result := Fld_1;
   

            
   --# end


               exit when Quantifier_Result;

         end loop;
      end;
   

   
   



--# expr-done 2
--# expr-start 4 '<Mapcat at parser.py:146>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:146








   

   

   
      declare
         Map_Result_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Tree_Semantic_Result_Array_Access := Res;
      begin
         for Item_0 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind r Item_0

            
         
         
      --# expr-start 3 '<FieldAccess for results at parser.py:146>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:146






   




Fld := Item_0.Results;
Inc_Ref (Fld);
--# expr-done 3
      

         for Item_To_Append of
               Fld.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;

   
   
   

            
   --# end
      Finalizer_Scope_311;



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 4
Inc_Ref (Map_Result);
New_Struct := (Has_Error => Quantifier_Result, Results => Map_Result); 
--# expr-done 5
Let_Result := New_Struct; Inc_Ref (Let_Result);
--# end
--# expr-done 6

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_6;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_6;
                     Finalizer_Scope_311;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Concat_Tree_Semantic_Results;
--# end

   







--# property-start LktNode.new_tree_semantic_result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:150
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_New_Tree_Semantic_Result
  
  (Node : Bare_Lkt_Node
      ; Elements : Internal_Semantic_Result_Array_Access
  )

   return Internal_Tree_Semantic_Result
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind elements Elements

   Property_Result : Internal_Tree_Semantic_Result;

      
            procedure Finalizer_Scope_7 with Inline_Always;

      Fld : Boolean;
Quantifier_Result : Boolean;
New_Struct : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Let_Result : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;

            procedure Finalizer_Scope_7 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_7;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 10 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9 '<New TreeSemanticResult at parser.py:152>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:152
--# expr-start 8 '<AnyQuantifier at parser.py:154>' Quantifier_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:154








   

   

   
      Quantifier_Result := False;
   

   

      

      declare
         
         Collection : constant Internal_Semantic_Result_Array_Access := Elements;
      begin
         for Item_2 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind e Item_2

            
      --# expr-start 7 '<FieldAccess for has_error at parser.py:154>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:154






   




Fld := Item_2.Has_Error;
--# expr-done 7
      Quantifier_Result := Fld;
   

            
   --# end


               exit when Quantifier_Result;

         end loop;
      end;
   

   
   



--# expr-done 8

Inc_Ref (Elements);
New_Struct := (Has_Error => Quantifier_Result, Results => Elements); 
--# expr-done 9
Let_Result := New_Struct; Inc_Ref (Let_Result);
--# end
--# expr-done 10

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_7;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_7;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_New_Tree_Semantic_Result;
--# end

   







--# property-start LktNode.root_get /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:157
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Root_Get
  
  (Node : Bare_Lkt_Node
      ; Entity_Name : Symbol_Type
  )

   return Internal_Entity_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind entity_name Entity_Name

   Property_Result : Internal_Entity_Decl;

      
            procedure Finalizer_Scope_8 with Inline_Always;

      Fld : Internal_Unit;
Fld_1 : Bare_Lkt_Node;
Fld_2 : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Decl;
Let_Result : Internal_Entity_Decl;

            procedure Finalizer_Scope_8 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_8;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 16 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 15 '<Cast to Decl at parser.py:160>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:160





--# expr-start 14 '<EnvGet at parser.py:159>' Env_Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:159
--# expr-start 13 '<FieldAccess for node_env at parser.py:159>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:159



--# expr-start 12 '<FieldAccess for root at parser.py:159>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:159



--# expr-start 11 '<FieldAccess for unit at parser.py:159>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:159






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Unit (Node => Self);
--# expr-done 11


   

      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Fld.Ast_Root;
--# expr-done 12


   

      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Liblktlang.Implementation.Node_Env (Node => Fld_1);
--# expr-done 13



Env_Get_Result := AST_Envs.Get_First (Self => Fld_2, Key => Thin (Entity_Name), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Nocat => True)); 
--# expr-done 14



   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Lkt_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 15
Let_Result := Cast_Result; 
--# end
--# expr-done 16

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_8;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_8;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Root_Get;
--# end

   







--# property-start LktNode.get_builtin_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:162
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Get_Builtin_Type
  
  (Node : Bare_Lkt_Node
      ; Entity_Name : Symbol_Type
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind entity_name Entity_Name

   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Fld : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Named_Type_Decl;
Let_Result : Internal_Entity_Named_Type_Decl;


      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Lkt_Node_Lkt_Node_P_Get_Builtin_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Lkt_Node,
                                As_Bare_Lkt_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Entity_Name);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Named_Type_Decl;

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 19 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 18 '<Cast to NamedTypeDecl at parser.py:164>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:164





--# expr-start 17 '<FieldAccess for root_get at parser.py:164>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:164






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start LktNode.root_get
Fld := Liblktlang.Implementation.Lkt_Node_P_Root_Get (Node => Self, Entity_Name => Entity_Name);
--# end
--# expr-done 17



   if Fld.Node = null
      or else Fld.Node.Kind in Lkt_Named_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Named_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 18
Let_Result := Cast_Result; 
--# end
--# expr-done 19

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Internal_Entity_Named_Type_Decl,
                        As_Internal_Entity_Named_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);

         end if;
      end if;



   return Property_Result;
end Lkt_Node_P_Get_Builtin_Type;
--# end

   







--# property-start LktNode.get_builtin_gen_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:166
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Get_Builtin_Gen_Decl
  
  (Node : Bare_Lkt_Node
      ; Entity_Name : Symbol_Type
  )

   return Internal_Entity_Generic_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind entity_name Entity_Name

   Property_Result : Internal_Entity_Generic_Decl;

      

      Fld : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Generic_Decl;
Let_Result : Internal_Entity_Generic_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 22 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 21 '<Cast to GenericDecl at parser.py:168>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:168





--# expr-start 20 '<FieldAccess for root_get at parser.py:168>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:168






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start LktNode.root_get
Fld := Liblktlang.Implementation.Lkt_Node_P_Root_Get (Node => Self, Entity_Name => Entity_Name);
--# end
--# expr-done 20



   if Fld.Node = null
      or else Fld.Node.Kind in Lkt_Generic_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 21
Let_Result := Cast_Result; 
--# end
--# expr-done 22

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Get_Builtin_Gen_Decl;
--# end

   







--# property-start LktNode.node_gen_trait /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:170
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Node_Gen_Trait
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Generic_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Generic_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Generic_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 24 '<FieldAccess for get_builtin_gen_decl at parser.py:171>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:171






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 23 '<Symbol Node at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Node); 
--# expr-done 23
--# property-call-start LktNode.get_builtin_gen_decl
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Gen_Decl (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 24

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Node_Gen_Trait;
--# end

   







--# property-start LktNode.node_trait /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:175
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Node_Trait
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Trait_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Trait_Decl;

      

      Fld : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Trait_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 27 '<Cast to TraitDecl at parser.py:176>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:176





--# expr-start 26 '<FieldAccess for decl at parser.py:176>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:176



--# expr-start 25 '<FieldAccess for node_gen_trait at parser.py:176>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:176






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.node_gen_trait
Fld := Liblktlang.Implementation.Lkt_Node_P_Node_Gen_Trait (Node => Self);
--# end
--# expr-done 25


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Decl (Node => Fld.Node.Generic_Decl_F_Decl, Info => Fld.Info);
--# expr-done 26



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Trait_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Trait_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Cast_Result := No_Entity_Trait_Decl;
   end if;


--# expr-done 27

         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Node_Trait;
--# end

   







--# property-start LktNode.token_node_trait /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:180
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Token_Node_Trait
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 29 '<FieldAccess for get_builtin_type at parser.py:181>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:181






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 28 '<Symbol TokenNode at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Tokennode); 
--# expr-done 28
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 29

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Token_Node_Trait;
--# end

   







--# property-start LktNode.error_node_trait /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:185
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Error_Node_Trait
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 31 '<FieldAccess for get_builtin_type at parser.py:186>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:186






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 30 '<Symbol ErrorNode at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Errornode); 
--# expr-done 30
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 31

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Error_Node_Trait;
--# end

   







--# property-start LktNode.char_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:190
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Char_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 33 '<FieldAccess for get_builtin_type at parser.py:191>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:191






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 32 '<Symbol Char at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Char); 
--# expr-done 32
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 33

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Char_Type;
--# end

   







--# property-start LktNode.int_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:195
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Int_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 35 '<FieldAccess for get_builtin_type at parser.py:196>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:196






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 34 '<Symbol Int at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Int); 
--# expr-done 34
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 35

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Int_Type;
--# end

   







--# property-start LktNode.bool_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:200
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Bool_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 37 '<FieldAccess for get_builtin_type at parser.py:201>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:201






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 36 '<Symbol Bool at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Bool); 
--# expr-done 36
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 37

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Bool_Type;
--# end

   







--# property-start LktNode.bigint_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:205
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Bigint_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 39 '<FieldAccess for get_builtin_type at parser.py:206>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:206






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 38 '<Symbol BigInt at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Bigint); 
--# expr-done 38
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 39

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Bigint_Type;
--# end

   







--# property-start LktNode.string_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:210
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_String_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 41 '<FieldAccess for get_builtin_type at parser.py:211>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:211






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 40 '<Symbol String at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 40
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 41

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_String_Type;
--# end

   







--# property-start LktNode.symbol_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:215
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Symbol_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 43 '<FieldAccess for get_builtin_type at parser.py:216>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:216






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 42 '<Symbol Symbol at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Symbol); 
--# expr-done 42
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 43

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Symbol_Type;
--# end

   







--# property-start LktNode.property_error_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:220
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Property_Error_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 45 '<FieldAccess for get_builtin_type at parser.py:221>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:221






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 44 '<Symbol PropertyError at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Propertyerror); 
--# expr-done 44
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 45

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Property_Error_Type;
--# end

   







--# property-start LktNode.regexp_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:225
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Regexp_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 47 '<FieldAccess for get_builtin_type at parser.py:226>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:226






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 46 '<Symbol Regexp at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Regexp); 
--# expr-done 46
--# property-call-start LktNode.get_builtin_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Type (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 47

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Regexp_Type;
--# end

   







--# property-start LktNode.array_gen_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:230
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Array_Gen_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Generic_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Generic_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Generic_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 49 '<FieldAccess for get_builtin_gen_decl at parser.py:231>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:231






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 48 '<Symbol Array at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Array); 
--# expr-done 48
--# property-call-start LktNode.get_builtin_gen_decl
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Gen_Decl (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 49

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Array_Gen_Type;
--# end

   







--# property-start LktNode.array_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:235
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Array_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Fld : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 52 '<Cast to NamedTypeDecl at parser.py:236>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:236





--# expr-start 51 '<FieldAccess for decl at parser.py:236>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:236



--# expr-start 50 '<FieldAccess for array_gen_type at parser.py:236>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:236






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.array_gen_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Array_Gen_Type (Node => Self);
--# end
--# expr-done 50


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Decl (Node => Fld.Node.Generic_Decl_F_Decl, Info => Fld.Info);
--# expr-done 51



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Named_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Named_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Cast_Result := No_Entity_Named_Type_Decl;
   end if;


--# expr-done 52

         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Array_Type;
--# end

   







--# property-start LktNode.astlist_gen_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:240
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Astlist_Gen_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Generic_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Generic_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Generic_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 54 '<FieldAccess for get_builtin_gen_decl at parser.py:241>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:241






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 53 '<Symbol ASTList at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Astlist); 
--# expr-done 53
--# property-call-start LktNode.get_builtin_gen_decl
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Gen_Decl (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 54

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Astlist_Gen_Type;
--# end

   







--# property-start LktNode.astlist_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:246
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Astlist_Type
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Named_Type_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Named_Type_Decl;

      

      Fld : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Named_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 57 '<Cast to NamedTypeDecl at parser.py:247>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:247





--# expr-start 56 '<FieldAccess for decl at parser.py:247>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:247



--# expr-start 55 '<FieldAccess for astlist_gen_type at parser.py:247>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:247






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.astlist_gen_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Astlist_Gen_Type (Node => Self);
--# end
--# expr-done 55


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Decl (Node => Fld.Node.Generic_Decl_F_Decl, Info => Fld.Info);
--# expr-done 56



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Named_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Named_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Cast_Result := No_Entity_Named_Type_Decl;
   end if;


--# expr-done 57

         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Astlist_Type;
--# end

   







--# property-start LktNode.iterator_gen_trait /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:252
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Iterator_Gen_Trait
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Generic_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Generic_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Generic_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 59 '<FieldAccess for get_builtin_gen_decl at parser.py:253>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:253






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 58 '<Symbol Iterator at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterator); 
--# expr-done 58
--# property-call-start LktNode.get_builtin_gen_decl
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Gen_Decl (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 59

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Iterator_Gen_Trait;
--# end

   







--# property-start LktNode.iterator_trait /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:258
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Iterator_Trait
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Trait_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Trait_Decl;

      

      Fld : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Trait_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 62 '<Cast to TraitDecl at parser.py:259>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:259





--# expr-start 61 '<FieldAccess for decl at parser.py:259>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:259



--# expr-start 60 '<FieldAccess for iterator_gen_trait at parser.py:259>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:259






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.iterator_gen_trait
Fld := Liblktlang.Implementation.Lkt_Node_P_Iterator_Gen_Trait (Node => Self);
--# end
--# expr-done 60


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Decl (Node => Fld.Node.Generic_Decl_F_Decl, Info => Fld.Info);
--# expr-done 61



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Trait_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Trait_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Cast_Result := No_Entity_Trait_Decl;
   end if;


--# expr-done 62

         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Iterator_Trait;
--# end

   







--# property-start LktNode.analysis_unit_gen_trait /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:264
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Analysis_Unit_Gen_Trait
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Generic_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Generic_Decl;

      

      Sym : Symbol_Type;
Fld : Internal_Entity_Generic_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 64 '<FieldAccess for get_builtin_gen_decl at parser.py:265>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:265






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 63 '<Symbol AnalysisUnit at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Analysisunit); 
--# expr-done 63
--# property-call-start LktNode.get_builtin_gen_decl
Fld := Liblktlang.Implementation.Lkt_Node_P_Get_Builtin_Gen_Decl (Node => Self, Entity_Name => Sym);
--# end
--# expr-done 64

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Analysis_Unit_Gen_Trait;
--# end

   







--# property-start LktNode.analysis_unit_trait /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:270
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Analysis_Unit_Trait
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Entity_Trait_Decl
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Trait_Decl;

      

      Fld : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Trait_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 67 '<Cast to TraitDecl at parser.py:271>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:271





--# expr-start 66 '<FieldAccess for decl at parser.py:271>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:271



--# expr-start 65 '<FieldAccess for analysis_unit_gen_trait at parser.py:271>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:271






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.analysis_unit_gen_trait
Fld := Liblktlang.Implementation.Lkt_Node_P_Analysis_Unit_Gen_Trait (Node => Self);
--# end
--# expr-done 65


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Decl (Node => Fld.Node.Generic_Decl_F_Decl, Info => Fld.Info);
--# expr-done 66



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Trait_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Trait_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Cast_Result := No_Entity_Trait_Decl;
   end if;


--# expr-done 67

         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Analysis_Unit_Trait;
--# end

   







--# property-start LktNode.function_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:297
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Function_Type
  
  (Node : Bare_Lkt_Node
      ; Args_Types : Bare_Type_Decl_Array_Access
      ; Return_Type : Bare_Type_Decl
  )

   return Bare_Function_Type
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind args_types Args_Types
   --# bind return_type Return_Type

   Property_Result : Bare_Function_Type;

      

      Fld : Internal_Unit;
Fld_1 : Bare_Lkt_Node;
Cast_Result : Bare_Langkit_Root;
Fld_2 : Bare_Function_Type;
Let_Result : Bare_Function_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 72 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 71 '<FieldAccess for function_type_helper at parser.py:310>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:310



--# expr-start 70 '<Cast to LangkitRoot at parser.py:310>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:310





--# expr-start 69 '<FieldAccess for root at parser.py:310>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:310



--# expr-start 68 '<FieldAccess for unit at parser.py:310>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:310






   

      if Return_Type = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Unit (Node => Return_Type);
--# expr-done 68


   

      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Fld.Ast_Root;
--# expr-done 69



   if Fld_1 = null
      or else Fld_1.Kind in Lkt_Langkit_Root_Range
   then
      
      Cast_Result := Fld_1;

   else
         Cast_Result := No_Bare_Lkt_Node;
   end if;


--# expr-done 70


   

      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;





--# property-call-start LangkitRoot.function_type_helper
Fld_2 := Liblktlang.Implementation.Langkit_Root_P_Function_Type_Helper (Node => Cast_Result, Args_Types => Args_Types, Return_Type => Return_Type);
--# end
--# expr-done 71
Let_Result := Fld_2; 
--# end
--# expr-done 72

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Function_Type;
--# end

   







--# property-start LktNode.exempt /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:314
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Exempt
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Semantic_Result
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_34 with Inline_Always;

      New_Struct : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_34 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_34;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 74 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 73 '<New SemanticResult at parser.py:319>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:319






Inc_Ref (Empty_String);
New_Struct := (Error_Message => Empty_String, Exempt_Analysis => True, Has_Error => True, Node => Self, Result_Decl => No_Entity_Decl, Result_Type => No_Entity_Type_Decl); 
--# expr-done 73
Let_Result := New_Struct; Inc_Ref (Let_Result);
--# end
--# expr-done 74

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_34;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_34;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Exempt;
--# end

   







--# property-start LktNode.error /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:323
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Error
  
  (Node : Bare_Lkt_Node
      ; Message : String_Type
  )

   return Internal_Semantic_Result
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind message Message

   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_35 with Inline_Always;

      New_Struct : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_35 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_35;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 76 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 75 '<New SemanticResult at parser.py:328>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:328






Inc_Ref (Message);
New_Struct := (Error_Message => Message, Exempt_Analysis => False, Has_Error => True, Node => Self, Result_Decl => No_Entity_Decl, Result_Type => No_Entity_Type_Decl); 
--# expr-done 75
Let_Result := New_Struct; Inc_Ref (Let_Result);
--# end
--# expr-done 76

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_35;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_35;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Error;
--# end

   







--# property-start LktNode.internal_error /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:332
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Internal_Error
  
  (Node : Bare_Lkt_Node
  )

   return Internal_Semantic_Result
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_36 with Inline_Always;

      Str : String_Type;
Fld : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_36 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_36;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 79 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 78 '<FieldAccess for error at parser.py:337>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:337






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 77 '<String at parser.py:337>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:337
Str := Create_String (""); 
--# expr-done 77
--# property-call-start LktNode.error
Fld := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Str);
--# end
--# expr-done 78
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 79

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_36;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_36;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Internal_Error;
--# end

   







--# property-start LktNode.expected_type_error /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:339
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Expected_Type_Error
  
  (Node : Bare_Lkt_Node
      ; Expected : Internal_Entity_Type_Decl
      ; Got : String_Type
  )

   return Internal_Semantic_Result
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind expected Expected
   --# bind got Got

   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_37 with Inline_Always;

      Str : String_Type;
Fld : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;
Concat_Result_2 : String_Type;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_37 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_37;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 84 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 83 '<FieldAccess for error at parser.py:348>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:348






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 80 '<String at parser.py:349>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:349
Str := Create_String ("Mismatched types: expected `"); 
--# expr-done 80
--# expr-start 81 '<FieldAccess for full_name at parser.py:350>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:350






   

      if Expected.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Expected.Node);
--# end
--# expr-done 81
Concat_Result := Concat_String (Str, Fld); 
--# expr-start 82 '<String at parser.py:351>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:351
Str_1 := Create_String ("`, got "); 
--# expr-done 82

Concat_Result_1 := Concat_String (Str_1, Got); 
Concat_Result_2 := Concat_String (Concat_Result, Concat_Result_1); 
--# property-call-start LktNode.error
Fld_1 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Concat_Result_2);
--# end
--# expr-done 83
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 84

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_37;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_37;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Expected_Type_Error;
--# end

   







--# property-start LktNode.type_mismatch_error /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:354
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Type_Mismatch_Error
  
  (Node : Bare_Lkt_Node
      ; Expected : Internal_Entity_Type_Decl
      ; Got : Internal_Entity_Type_Decl
  )

   return Internal_Semantic_Result
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind expected Expected
   --# bind got Got

   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_38 with Inline_Always;

      Str : String_Type;
Fld : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_38 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_38;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 89 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 88 '<FieldAccess for expected_type_error at parser.py:364>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:364






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# expr-start 85 '<String at parser.py:366>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:366
Str := Create_String ("`"); 
--# expr-done 85
--# expr-start 86 '<FieldAccess for full_name at parser.py:366>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:366






   

      if Got.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Got.Node);
--# end
--# expr-done 86
Concat_Result := Concat_String (Str, Fld); 
--# expr-start 87 '<String at parser.py:366>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:366
Str_1 := Create_String ("`"); 
--# expr-done 87
Concat_Result_1 := Concat_String (Concat_Result, Str_1); 
--# property-call-start LktNode.expected_type_error
Fld_1 := Liblktlang.Implementation.Lkt_Node_P_Expected_Type_Error (Node => Self, Expected => Expected, Got => Concat_Result_1);
--# end
--# expr-done 88
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 89

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_38;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_38;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Type_Mismatch_Error;
--# end

   







--# property-start LktNode.check_legality_visitor /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:369
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Check_Legality_Visitor
  
  (Node : Bare_Lkt_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_39 with Inline_Always;
            procedure Finalizer_Scope_315 with Inline_Always;
            procedure Finalizer_Scope_316 with Inline_Always;

      Block_Var : Internal_Semantic_Result_Array_Access;
Var_Expr_1 : Internal_Entity;
Fld : Internal_Semantic_Result_Array_Access;
Fld_1 : Internal_Entity_Array_Access;
Fld_2 : Internal_Semantic_Result_Array_Access;
Scope_Result : Internal_Semantic_Result_Array_Access;
Result_Var : Internal_Semantic_Result_Array_Access;
Map_Result : Internal_Semantic_Result_Array_Access;
Concat_Result : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_39 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_39;
            procedure Finalizer_Scope_315 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_315;
            procedure Finalizer_Scope_316 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_316;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 94 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 90 '<FieldAccess for check_legality at parser.py:375>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:375






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]LktNode.check_legality'
Fld := Liblktlang.Implementation.Dispatcher_Lkt_Node_P_Check_Legality (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 90
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind own_results Block_Var

--# expr-start 93 '<Mapcat at parser.py:377>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:377








   

   --# expr-start 91 '<children at parser.py:377>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:377






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Liblktlang.Implementation.Children (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 91

   
      declare
         Map_Result_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_3 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind c Item_3

            
         
         
      



Var_Expr_1 := Item_3;





if Var_Expr_1 /= No_Entity then
   



   --# scope-start


   

--# expr-start 92 '<FieldAccess for check_legality_visitor at parser.py:377>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:377






   

      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.check_legality_visitor
Fld_2 := Liblktlang.Implementation.Lkt_Node_P_Check_Legality_Visitor (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 92
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_316;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var);


      

         for Item_To_Append of
               Result_Var.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;

   
   
   

            
   --# end
      Finalizer_Scope_315;



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 93
Concat_Result := Concat (Block_Var, Map_Result); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 94

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_39;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_39;
                     Finalizer_Scope_315;
                     Finalizer_Scope_316;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Check_Legality_Visitor;
--# end

   







--# property-start LktNode.check_sem_recursive /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:380
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Check_Sem_Recursive
  
  (Node : Bare_Lkt_Node
      ; Exempted_Nodes : Bare_Lkt_Node_Array_Access
         := No_Bare_Lkt_Node_Array_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Tree_Semantic_Result
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind exempted_nodes Exempted_Nodes

   Property_Result : Internal_Tree_Semantic_Result;

      
            procedure Finalizer_Scope_40 with Inline_Always;
            procedure Finalizer_Scope_318 with Inline_Always;
            procedure Finalizer_Scope_319 with Inline_Always;
            procedure Finalizer_Scope_320 with Inline_Always;
            procedure Finalizer_Scope_322 with Inline_Always;

      Block_Var : Internal_Semantic_Result_Array_Access;
Block_Var_1 : Bare_Lkt_Node_Array_Access;
Block_Var_2 : Internal_Semantic_Result_Array_Access;
Block_Var_3 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Var_Expr : Internal_Entity_Array_Access;
Var_Expr_2 : Internal_Entity;
Fld : Internal_Semantic_Result_Array_Access;
Fld_1 : Bare_Lkt_Node;
Fld_2 : Boolean;
Map_Result : Bare_Lkt_Node_Array_Access;
Concat_Result : Bare_Lkt_Node_Array_Access;
Fld_3 : Internal_Entity_Array_Access;
Fld_4 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Scope_Result : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Result_Var : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Fld_5 : Bare_Lkt_Node;
Cast_Expr : Bare_Lkt_Node;
Cast_Result : Bare_Lkt_Node;
Is_Equal : Boolean;
Find_Result : Bare_Lkt_Node;
Is_Null : Boolean;
If_Result : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Map_Result_1 : Internal_Tree_Semantic_Result_Array_Access;
Scope_Result_1 : Internal_Tree_Semantic_Result_Array_Access;
Result_Var_1 : Internal_Tree_Semantic_Result_Array_Access;
Fld_6 : Internal_Semantic_Result_Array_Access;
Map_Result_2 : Internal_Semantic_Result_Array_Access;
Fld_7 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Concat_Result_1 : Internal_Semantic_Result_Array_Access;
Fld_8 : Internal_Semantic_Result_Array_Access;
Concat_Result_2 : Internal_Semantic_Result_Array_Access;
Fld_9 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Let_Result : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;

            procedure Finalizer_Scope_40 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_40;
            procedure Finalizer_Scope_318 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_318;
            procedure Finalizer_Scope_319 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_319;
            procedure Finalizer_Scope_320 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_320;
            procedure Finalizer_Scope_322 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_322;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 112 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 95 '<FieldAccess for check_correctness_pre at parser.py:391>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:391






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]LktNode.check_correctness_pre'
Fld := Liblktlang.Implementation.Dispatcher_Lkt_Node_P_Check_Correctness_Pre (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 95
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind pre_diagnostics Block_Var

--# expr-start 98 '<FilterMap at parser.py:392>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:392








   

   

   
      declare
         Map_Result_Vec : Bare_Lkt_Node_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Semantic_Result_Array_Access := Block_Var;
      begin
         for Item_4 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind d Item_4

            
         --# expr-start 96 '<FieldAccess for exempt_analysis at parser.py:393>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:393






   




Fld_2 := Item_4.Exempt_Analysis;
--# expr-done 96
         if Fld_2 then
            
         
      --# expr-start 97 '<FieldAccess for node at parser.py:393>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:393






   




Fld_1 := Item_4.Node;
--# expr-done 97
      

         declare
            Item_To_Append : constant Bare_Lkt_Node := Fld_1;
         begin
            Bare_Lkt_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result := Create_Bare_Lkt_Node_Array
           (Items_Count => Natural (Bare_Lkt_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Lkt_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Lkt_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Lkt_Node_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 98
Concat_Result := Concat (Exempted_Nodes, Map_Result); 
Block_Var_1 := Concat_Result; Inc_Ref (Block_Var_1);
--# bind exemptions Block_Var_1
--# expr-start 108 '<Mapcat at parser.py:410>' Map_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:410








   

   


--# expr-start 99 '<children at parser.py:397>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:397






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Liblktlang.Implementation.Children (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 99
Var_Expr := Fld_3;

      Inc_Ref (Var_Expr);




if Var_Expr /= No_Internal_Entity_Array_Type then
   



   --# scope-start


   --# bind children Var_Expr

--# expr-start 106 '<Map at parser.py:398>' Map_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:398








   

   

   
      declare
         Map_Result_1_Vec : Internal_Tree_Semantic_Result_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Entity_Array_Access := Var_Expr;
      begin
         for Item_6 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind child Item_6

            
         
         
      --# expr-start 105 '<If at parser.py:398>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:398



--# expr-start 103 '<IsNull at parser.py:406>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:406
--# expr-start 102 '<Find at parser.py:406>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:406








   

   

   
      Find_Result := No_Bare_Lkt_Node;
   

   

      

      declare
         
         Collection : constant Bare_Lkt_Node_Array_Access := Block_Var_1;
      begin
         for Item_7 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind n Item_7

            
      --# expr-start 101 '<Eq at parser.py:406>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:406






Cast_Expr := Item_7; 



   
      Cast_Result := Cast_Expr;


--# expr-start 100 '<FieldAccess for node at parser.py:406>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:406






   




Fld_5 := Item_6.Node;
--# expr-done 100
Is_Equal := Cast_Result = Fld_5; 
--# expr-done 101
      if Is_Equal then
         Find_Result := Item_7; 
      end if;
   

            
   --# end


               exit when Is_Equal;

         end loop;
      end;
   

   
   



--# expr-done 102
Is_Null := Find_Result = null; 
--# expr-done 103
if Is_Null then
   



Var_Expr_2 := Item_6;





if Var_Expr_2 /= No_Entity then
   



   --# scope-start


   

--# expr-start 104 '<FieldAccess for check_sem_recursive at parser.py:407>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:407






   

      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start LktNode.check_sem_recursive
Fld_4 := Liblktlang.Implementation.Lkt_Node_P_Check_Sem_Recursive (Node => Var_Expr_2.Node, Exempted_Nodes => Block_Var_1, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 104
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_320;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Tree_Semantic_Result;
end if;

      Inc_Ref (Result_Var);


   If_Result := Result_Var;
else
   
   If_Result := No_Tree_Semantic_Result;
end if;

      Inc_Ref (If_Result);


--# expr-done 105
      

         declare
            Item_To_Append : constant Internal_Tree_Semantic_Result := If_Result;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Tree_Semantic_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_319;



         end loop;
      end;
   

   

         Map_Result_1 := Create_Internal_Tree_Semantic_Result_Array
           (Items_Count => Natural (Internal_Tree_Semantic_Result_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Tree_Semantic_Result_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Tree_Semantic_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Tree_Semantic_Result_Vectors.Destroy (Map_Result_1_Vec);

      end;
   



--# expr-done 106
Scope_Result_1 := Map_Result_1;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_318;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Internal_Tree_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_1);



   
      declare
         Map_Result_2_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Tree_Semantic_Result_Array_Access := Result_Var_1;
      begin
         for Item_5 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind res Item_5

            
         
         
      --# expr-start 107 '<FieldAccess for results at parser.py:410>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:410






   




Fld_6 := Item_5.Results;
Inc_Ref (Fld_6);
--# expr-done 107
      

         for Item_To_Append of
               Fld_6.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;

   
   
   

            
   --# end
      Finalizer_Scope_322;



         end loop;
      end;
   

   

         Map_Result_2 := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_2_Vec);

      end;
   



--# expr-done 108
Block_Var_2 := Map_Result_2; Inc_Ref (Block_Var_2);
--# bind children_results Block_Var_2
--# expr-start 109 '<FieldAccess for check_semantic_impl at parser.py:412>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:412






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.check_semantic_impl
Fld_7 := Liblktlang.Implementation.Lkt_Node_P_Check_Semantic_Impl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 109
Block_Var_3 := Fld_7; Inc_Ref (Block_Var_3);
--# bind own_results Block_Var_3
--# expr-start 111 '<FieldAccess for new_tree_semantic_result at parser.py:420>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:420






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;





Concat_Result_1 := Concat (Block_Var, Block_Var_2); 
--# expr-start 110 '<FieldAccess for results at parser.py:417>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:417






   




Fld_8 := Block_Var_3.Results;
Inc_Ref (Fld_8);
--# expr-done 110
Concat_Result_2 := Concat (Concat_Result_1, Fld_8); 
--# property-call-start LktNode.new_tree_semantic_result
Fld_9 := Liblktlang.Implementation.Lkt_Node_P_New_Tree_Semantic_Result (Node => Ent.Node, Elements => Concat_Result_2);
--# end
--# expr-done 111
Let_Result := Fld_9; Inc_Ref (Let_Result);
--# end
--# expr-done 112

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_40;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_40;
                     Finalizer_Scope_318;
                     Finalizer_Scope_319;
                     Finalizer_Scope_320;
                     Finalizer_Scope_322;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Check_Sem_Recursive;
--# end

   







--# property-start '[dispatcher]LktNode.check_legality' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Lkt_Node_P_Check_Legality
  
  (Node : Bare_Lkt_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result_Array_Access;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Lkt_Node (Self.Kind) is
               when Lkt_Lexer_Case_Rule_Cond_Alt .. Lkt_Val_Decl | Lkt_Env_Spec_Decl .. Lkt_Generic_Decl | Lkt_Lexer_Decl .. Lkt_Instantiated_Generic_Type | Lkt_Decl_Annotation .. Lkt_Call_Expr | Lkt_Error_On_Null .. Lkt_Module_Ref_Id | Lkt_If_Expr .. Lkt_Paren_Expr | Lkt_Subscript_Expr .. Lkt_Var_Bind =>
                  --# property-call-start LktNode.check_legality
                  Property_Result := Lkt_Node_P_Check_Legality
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Fun_Decl =>
                  --# property-call-start FunDecl.check_legality
                  Property_Result := Fun_Decl_P_Check_Legality
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Grammar_Decl =>
                  --# property-call-start GrammarDecl.check_legality
                  Property_Result := Grammar_Decl_P_Check_Legality
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Enum_Type_Decl .. Lkt_Trait_Decl =>
                  --# property-call-start NamedTypeDecl.check_legality
                  Property_Result := Named_Type_Decl_P_Check_Legality
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Class_Decl .. Lkt_Enum_Class_Decl =>
                  --# property-call-start BasicClassDecl.check_legality
                  Property_Result := Basic_Class_Decl_P_Check_Legality
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Cast_Expr =>
                  --# property-call-start CastExpr.check_legality
                  Property_Result := Cast_Expr_P_Check_Legality
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Ref_Id =>
                  --# property-call-start RefId.check_legality
                  Property_Result := Ref_Id_P_Check_Legality
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Raise_Expr =>
                  --# property-call-start RaiseExpr.check_legality
                  Property_Result := Raise_Expr_P_Check_Legality
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Lkt_Node_P_Check_Legality;
--# end

   







--# property-start '[dispatcher]LktNode.check_correctness_pre' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Lkt_Node_P_Check_Correctness_Pre
  
  (Node : Bare_Lkt_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result_Array_Access;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Lkt_Node (Self.Kind) is
               when Lkt_Lexer_Case_Rule_Cond_Alt .. Lkt_Generic_Decl | Lkt_Lexer_Decl .. Lkt_Block_Expr | Lkt_Cast_Expr .. Lkt_Error_On_Null | Lkt_Grammar_Cut .. Lkt_Keep_Expr | Lkt_Big_Num_Lit .. Lkt_Langkit_Root | Lkt_Lexer_Case_Rule_Send .. Lkt_Function_Type_Ref | Lkt_Var_Bind =>
                  --# property-call-start LktNode.check_correctness_pre
                  Property_Result := Lkt_Node_P_Check_Correctness_Pre
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Grammar_Decl =>
                  --# property-call-start GrammarDecl.check_correctness_pre
                  Property_Result := Grammar_Decl_P_Check_Correctness_Pre
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Call_Expr =>
                  --# property-call-start CallExpr.check_correctness_pre
                  Property_Result := Call_Expr_P_Check_Correctness_Pre
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Generic_Instantiation =>
                  --# property-call-start GenericInstantiation.check_correctness_pre
                  Property_Result := Generic_Instantiation_P_Check_Correctness_Pre
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Lambda_Expr =>
                  --# property-call-start LambdaExpr.check_correctness_pre
                  Property_Result := Lambda_Expr_P_Check_Correctness_Pre
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Lexer_Case_Rule =>
                  --# property-call-start LexerCaseRule.check_correctness_pre
                  Property_Result := Lexer_Case_Rule_P_Check_Correctness_Pre
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Generic_Type_Ref =>
                  --# property-call-start GenericTypeRef.check_correctness_pre
                  Property_Result := Generic_Type_Ref_P_Check_Correctness_Pre
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Simple_Type_Ref =>
                  --# property-call-start SimpleTypeRef.check_correctness_pre
                  Property_Result := Simple_Type_Ref_P_Check_Correctness_Pre
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Lkt_Node_P_Check_Correctness_Pre;
--# end

   







--# property-start LktNode.check_semantic_impl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:452
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Check_Semantic_Impl
  
  (Node : Bare_Lkt_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Tree_Semantic_Result
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Tree_Semantic_Result;

      
            procedure Finalizer_Scope_43 with Inline_Always;
            procedure Finalizer_Scope_323 with Inline_Always;
            procedure Finalizer_Scope_324 with Inline_Always;
            procedure Finalizer_Scope_325 with Inline_Always;
            procedure Finalizer_Scope_326 with Inline_Always;

      Block_Var : Internal_Semantic_Result := No_Semantic_Result;
Block_Var_1 : Internal_Semantic_Result_Array_Access;
Match_0 : Internal_Entity_Ref_Id;
Match_1 : Internal_Entity_Expr;
Match_2 : Internal_Entity;
Rd : Internal_Semantic_Result := No_Semantic_Result;
Var_Expr_3 : Internal_Semantic_Result := No_Semantic_Result;
Var_Expr_4 : Internal_Semantic_Result := No_Semantic_Result;
Fld : Internal_Semantic_Result := No_Semantic_Result;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;
Try_Result : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
Fld_2 : Internal_Semantic_Result := No_Semantic_Result;
Try_Result_1 : Internal_Semantic_Result := No_Semantic_Result;
Singleton_1 : Internal_Semantic_Result_Array_Access;
Scope_Result : Internal_Semantic_Result_Array_Access;
Result_Var : Internal_Semantic_Result_Array_Access;
Fld_3 : Boolean;
Not_Val : Boolean;
If_Result : Internal_Semantic_Result_Array_Access;
Concat_Result : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;
Fld_4 : Internal_Semantic_Result := No_Semantic_Result;
Try_Result_2 : Internal_Semantic_Result := No_Semantic_Result;
Singleton_2 : Internal_Semantic_Result_Array_Access;
Scope_Result_1 : Internal_Semantic_Result_Array_Access;
Result_Var_1 : Internal_Semantic_Result_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Ref_Id;
Let_Result_1 : Internal_Semantic_Result_Array_Access;
Scope_Result_2 : Internal_Semantic_Result_Array_Access;
Cast_Result_1 : Internal_Entity_Expr;
Let_Result_2 : Internal_Semantic_Result_Array_Access;
Scope_Result_3 : Internal_Semantic_Result_Array_Access;
Cast_Result_2 : Internal_Entity;
Let_Result_3 : Internal_Semantic_Result_Array_Access;
Scope_Result_4 : Internal_Semantic_Result_Array_Access;
Match_Result : Internal_Semantic_Result_Array_Access;
Fld_5 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Let_Result_4 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;

            procedure Finalizer_Scope_43 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_43;
            procedure Finalizer_Scope_323 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Try_Result);
                     Dec_Ref (Rd);
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Try_Result_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_323;
            procedure Finalizer_Scope_324 is
            begin
                     Dec_Ref (Var_Expr_3);
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_324;
            procedure Finalizer_Scope_325 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Try_Result_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_325;
            procedure Finalizer_Scope_326 is
            begin
                     Dec_Ref (Var_Expr_4);
                     Dec_Ref (Singleton_2);
            end Finalizer_Scope_326;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 126 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 113 '<FieldAccess for internal_error at parser.py:476>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:476






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.internal_error
Fld := Liblktlang.Implementation.Lkt_Node_P_Internal_Error (Node => Self);
--# end
--# expr-done 113
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind err Block_Var
--# expr-start 124 '<Match at parser.py:479>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:479









   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Ent; 

case Lkt_Lkt_Node (Match_Prefix.Node.Kind) is
         when Lkt_Ref_Id =>
            



   --# scope-start



--# scope-start









   
      Cast_Result := Create_Internal_Entity_Ref_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result; 
--# bind r Match_0
--# expr-start 121 '<Let at parser.py:480>' Let_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:480
--# scope-start
--# expr-start 115 '<Try at parser.py:69>' Try_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:69



begin
   --# expr-start 114 '<FieldAccess for referenced_decl at parser.py:481>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:481






   

      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 114
   Try_Result := Fld_1;
exception
   when Property_Error =>
      
      Try_Result := Block_Var;
end;

      Inc_Ref (Try_Result);


--# expr-done 115
Rd := Try_Result; Inc_Ref (Rd);
--# bind rd Rd

                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Rd;
                Inc_Ref (Singleton.Items (1));
            
--# expr-start 120 '<If at parser.py:483>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:483



--# expr-start 117 '<Not at parser.py:483>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:483
--# expr-start 116 '<FieldAccess for has_error at parser.py:483>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:483






   




Fld_3 := Rd.Has_Error;
--# expr-done 116
Not_Val := not (Fld_3); 
--# expr-done 117
if Not_Val then
   


--# expr-start 119 '<Try at parser.py:69>' Try_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:69



begin
   --# expr-start 118 '<FieldAccess for expr_type at parser.py:484>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:484






   

      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expr_type
Fld_2 := Liblktlang.Implementation.Expr_P_Expr_Type (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 118
   Try_Result_1 := Fld_2;
exception
   when Property_Error =>
      
      Try_Result_1 := Block_Var;
end;

      Inc_Ref (Try_Result_1);


--# expr-done 119
Var_Expr_3 := Try_Result_1;

      Inc_Ref (Var_Expr_3);




if Var_Expr_3 /= No_Semantic_Result then
   



   --# scope-start


   


                Singleton_1 := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr_3;
                Inc_Ref (Singleton_1.Items (1));
            
Scope_Result := Singleton_1;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_324;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var);


   If_Result := Result_Var;
else
   
   If_Result := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (If_Result);


--# expr-done 120
Concat_Result := Concat (Singleton, If_Result); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 121
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);


   --# end
      Finalizer_Scope_323;


            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Lkt_Any_Of .. Lkt_Module_Ref_Id | Lkt_If_Expr .. Lkt_Un_Op =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_1; 
--# bind e Match_1



--# expr-start 123 '<Try at parser.py:69>' Try_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:69



begin
   --# expr-start 122 '<FieldAccess for expr_type at parser.py:488>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:488






   

      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expr_type
Fld_4 := Liblktlang.Implementation.Expr_P_Expr_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 122
   Try_Result_2 := Fld_4;
exception
   when Property_Error =>
      
      Try_Result_2 := Block_Var;
end;

      Inc_Ref (Try_Result_2);


--# expr-done 123
Var_Expr_4 := Try_Result_2;

      Inc_Ref (Var_Expr_4);




if Var_Expr_4 /= No_Semantic_Result then
   



   --# scope-start


   


                Singleton_2 := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton_2.Items (1) := Var_Expr_4;
                Inc_Ref (Singleton_2.Items (1));
            
Scope_Result_1 := Singleton_2;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_326;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_1);


Let_Result_2 := Result_Var_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);


   --# end
      Finalizer_Scope_325;


            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Lkt_Lexer_Case_Rule_Cond_Alt .. Lkt_Excludes_Null_Present | Lkt_Full_Decl .. Lkt_Var_Bind =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_2 := Cast_Result_2; 
--# bind _ Match_2

Let_Result_3 := No_Internal_Semantic_Result_Array_Type; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);


   --# end


            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
end case;

--# expr-done 124
Block_Var_1 := Match_Result; Inc_Ref (Block_Var_1);
--# bind results Block_Var_1
--# expr-start 125 '<FieldAccess for new_tree_semantic_result at parser.py:492>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:492






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start LktNode.new_tree_semantic_result
Fld_5 := Liblktlang.Implementation.Lkt_Node_P_New_Tree_Semantic_Result (Node => Ent.Node, Elements => Block_Var_1);
--# end
--# expr-done 125
Let_Result_4 := Fld_5; Inc_Ref (Let_Result_4);
--# end
--# expr-done 126

         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_43;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_43;
                     Finalizer_Scope_323;
                     Finalizer_Scope_324;
                     Finalizer_Scope_325;
                     Finalizer_Scope_326;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Check_Semantic_Impl;
--# end

   







--# property-start LktNode.topmost_invalid_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:494
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Topmost_Invalid_Decl
  
  (Node : Bare_Lkt_Node
  )

   return Bare_Lkt_Node
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Bare_Lkt_Node;

      
            procedure Finalizer_Scope_44 with Inline_Always;

      Var_Expr : Bare_Full_Decl;
Node_Parents : Bare_Lkt_Node_Array_Access;
Cast_Expr : Bare_Lkt_Node;
Cast_Result : Bare_Full_Decl;
Sym : Symbol_Type;
Fld : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Bare_Lkt_Node_Array_Access;
Get_Result : Bare_Lkt_Node;
Let_Result : Bare_Lkt_Node;

            procedure Finalizer_Scope_44 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_44;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 133 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 132 '<at at parser.py:503>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:503

--# expr-start 131 '<Filter at parser.py:500>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:500








   

   --# expr-start 127 '<FieldAccess for parents at parser.py:500>' Node_Parents /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:500






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Node_Parents := Parents (Self, True); 
--# expr-done 127

   
      declare
         Map_Result_Vec : Bare_Lkt_Node_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Bare_Lkt_Node_Array_Access := Node_Parents;
      begin
         for Item_8 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind p Item_8

            
         


--# expr-start 128 '<Cast to FullDecl at parser.py:502>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:502






Cast_Expr := Item_8; 



   if Cast_Expr = null
      or else Cast_Expr.Kind in Lkt_Full_Decl_Range
   then
      
      Cast_Result := Cast_Expr;

   else
         Cast_Result := No_Bare_Lkt_Node;
   end if;


--# expr-done 128
Var_Expr := Cast_Result;





if Var_Expr /= No_Bare_Lkt_Node then
   



   --# scope-start


   --# bind fd Var_Expr

--# expr-start 130 '<FieldAccess for has_annotation at parser.py:502>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:502






   

      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 129 '<Symbol invalid at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Invalid); 
--# expr-done 129
--# property-call-start FullDecl.has_annotation
Fld := Liblktlang.Implementation.Full_Decl_P_Has_Annotation (Node => Var_Expr, Name => Sym);
--# end
--# expr-done 130
Scope_Result := Fld;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := False;
end if;



         if Result_Var then
            
         
      
      

         declare
            Item_To_Append : constant Bare_Lkt_Node := Item_8;
         begin
            Bare_Lkt_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result := Create_Bare_Lkt_Node_Array
           (Items_Count => Natural (Bare_Lkt_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Lkt_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Lkt_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Lkt_Node_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 131


Get_Result := Get (Self, Map_Result, -1, True); 
--# expr-done 132
Let_Result := Get_Result; 
--# end
--# expr-done 133

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_44;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_44;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Topmost_Invalid_Decl;
--# end

   







--# property-start LktNode.check_generic_inst_correctness_pre /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:505
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Check_Generic_Inst_Correctness_Pre
  
  (Node : Bare_Lkt_Node
      ; Prefix : Internal_Entity_Expr
      ; Params : Internal_Entity_Type_Ref_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind prefix Prefix
   --# bind params Params

   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_45 with Inline_Always;

      Block_Var : Internal_Entity_Decl;
Block_Var_1 : Internal_Entity_Generic_Decl;
Block_Var_2 : Internal_Semantic_Result_Array_Access;
Block_Var_3 : Internal_Semantic_Result_Array_Access;
Fld : Internal_Semantic_Result := No_Semantic_Result;
Fld_1 : Internal_Entity_Decl;
Cast_Expr : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Generic_Decl;
Str : String_Type;
Fld_2 : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
Is_Null : Boolean;
Is_Null_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Semantic_Result_Array_Access;
Str_1 : String_Type;
Fld_3 : Internal_Semantic_Result := No_Semantic_Result;
Singleton_1 : Internal_Semantic_Result_Array_Access;
Is_Null_2 : Boolean;
Not_Val_1 : Boolean;
Fld_4 : Bare_Type_Ref_List;
Len : Integer;
Fld_5 : Internal_Entity_Generic_Formal_Decl_List;
Fld_6 : Bare_Generic_Formal_Decl_List;
Len_1 : Integer;
Is_Equal : Boolean;
Not_Val_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Semantic_Result_Array_Access;
Concat_Result : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_45 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_45;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 153 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 135 '<FieldAccess for result_decl at parser.py:516>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:516



--# expr-start 134 '<FieldAccess for referenced_decl at parser.py:516>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:516






   

      if Prefix.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Prefix.Node, E_Info => Prefix.Info);
--# end
--# expr-done 134


   




Fld_1 := Fld.Result_Decl;
--# expr-done 135
Block_Var := Fld_1; 
--# bind prefix_decl Block_Var
--# expr-start 136 '<Cast to GenericDecl at parser.py:517>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:517






Cast_Expr := Block_Var; 



   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Lkt_Generic_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);

   else
         Cast_Result := No_Entity_Generic_Decl;
   end if;


--# expr-done 136
Block_Var_1 := Cast_Result; 
--# bind gen_decl Block_Var_1
--# expr-start 142 '<If at parser.py:519>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:519






--# expr-start 137 '<IsNull at parser.py:520>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:520

Is_Null := Block_Var.Node = null; 
--# expr-done 137
if Is_Null then
   
   If_Result := True;
else
   --# expr-start 139 '<Not at parser.py:520>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:520
--# expr-start 138 '<IsNull at parser.py:520>' Is_Null_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:520

Is_Null_1 := Block_Var_1.Node = null; 
--# expr-done 138
Not_Val := not (Is_Null_1); 
--# expr-done 139
   If_Result := Not_Val;
end if;



if If_Result then
   
   If_Result_1 := No_Internal_Semantic_Result_Array_Type;
else
   --# expr-start 141 '<FieldAccess for error at parser.py:522>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:522






   

      if Prefix.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 140 '<String at parser.py:522>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:522
Str := Create_String ("invalid reference to a generic"); 
--# expr-done 140
--# property-call-start LktNode.error
Fld_2 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Prefix.Node, Message => Str);
--# end
--# expr-done 141
                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_2;
                Inc_Ref (Singleton.Items (1));
            
   If_Result_1 := Singleton;
end if;

      Inc_Ref (If_Result_1);


--# expr-done 142
Block_Var_2 := If_Result_1; Inc_Ref (Block_Var_2);
--# bind invalid_decl Block_Var_2
--# expr-start 152 '<If at parser.py:530>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:530






--# expr-start 144 '<Not at parser.py:531>' Not_Val_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:531
--# expr-start 143 '<IsNull at parser.py:531>' Is_Null_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:531

Is_Null_2 := Block_Var_1.Node = null; 
--# expr-done 143
Not_Val_1 := not (Is_Null_2); 
--# expr-done 144
if Not_Val_1 then
   --# expr-start 149 '<Not at parser.py:532>' Not_Val_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:532
--# expr-start 148 '<Eq at parser.py:532>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:532
--# expr-start 145 '<length at parser.py:532>' Len /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:532






   




Fld_4 := Params.Node; 
Len := Length (Fld_4); 
--# expr-done 145
--# expr-start 147 '<length at parser.py:532>' Len_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:532



--# expr-start 146 '<FieldAccess for generic_formal_decls at parser.py:532>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:532






   

      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Create_Internal_Entity_Generic_Formal_Decl_List (Node => Block_Var_1.Node.Generic_Decl_F_Generic_Formal_Decls, Info => Block_Var_1.Info);
--# expr-done 146


   




Fld_6 := Fld_5.Node; 
Len_1 := Length (Fld_6); 
--# expr-done 147
Is_Equal := Len = Len_1; 
--# expr-done 148
Not_Val_2 := not (Is_Equal); 
--# expr-done 149
   If_Result_2 := Not_Val_2;
else
   
   If_Result_2 := False;
end if;



if If_Result_2 then
   --# expr-start 151 '<FieldAccess for error at parser.py:534>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:534






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 150 '<String at parser.py:534>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:534
Str_1 := Create_String ("invalid number of generic parameters"); 
--# expr-done 150
--# property-call-start LktNode.error
Fld_3 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Ent.Node, Message => Str_1);
--# end
--# expr-done 151
                Singleton_1 := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton_1.Items (1) := Fld_3;
                Inc_Ref (Singleton_1.Items (1));
            
   If_Result_3 := Singleton_1;
else
   
   If_Result_3 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (If_Result_3);


--# expr-done 152
Block_Var_3 := If_Result_3; Inc_Ref (Block_Var_3);
--# bind invalid_params Block_Var_3


Concat_Result := Concat (Block_Var_2, Block_Var_3); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 153

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_45;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_45;




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Check_Generic_Inst_Correctness_Pre;
--# end

   







--# property-start LktNode.can_reach /usr/lib/python3.13/site-packages/langkit/compiled_types.py:0
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Can_Reach
  
  (Node : Bare_Lkt_Node
      ; From_Node : Bare_Lkt_Node
  )

   return Boolean
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   

   --# bind from_node From_Node

   Property_Result : Boolean;

      

      Cast_Expr : Bare_Lkt_Node;
Cast_Result : Bare_Lkt_Node;
Is_Equal : Boolean;
Fld : Internal_Unit;
Fld_1 : Internal_Unit;
Is_Equal_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Node_Comp : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 160 '<Block at ???>' Let_Result None
--# scope-start






--# expr-start 154 '<Eq at ???>' Is_Equal None






Cast_Expr := From_Node; 



   
      Cast_Result := Cast_Expr;



Is_Equal := Cast_Result = No_Bare_Lkt_Node; 
--# expr-done 154
if Is_Equal then
   
   If_Result := True;
else
   --# expr-start 158 '<Not at ???>' Not_Val None
--# expr-start 157 '<Eq at ???>' Is_Equal_1 None
--# expr-start 155 '<FieldAccess for unit at ???>' Fld None






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Unit (Node => Self);
--# expr-done 155
--# expr-start 156 '<FieldAccess for unit at ???>' Fld_1 None






   

      if From_Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Liblktlang.Implementation.Unit (Node => From_Node);
--# expr-done 156
Is_Equal_1 := Fld = Fld_1; 
--# expr-done 157
Not_Val := not (Is_Equal_1); 
--# expr-done 158
   If_Result := Not_Val;
end if;



if If_Result then
   
   If_Result_1 := True;
else
   --# expr-start 159 '<OrderingTest '"'"'lt'"'"' at ???>' Node_Comp None



Node_Comp := Compare (Self, Self, From_Node, Less_Than); 
--# expr-done 159
   If_Result_1 := Node_Comp;
end if;



Let_Result := If_Result_1; 
--# end
--# expr-done 160

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lkt_Node_P_Can_Reach;
--# end

   







--# property-start LktNode.check_legality /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:422
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Check_Legality
  
  (Node : Bare_Lkt_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_41 with Inline_Always;

      Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_41 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_41;

      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Lkt_Node_Lkt_Node_P_Check_Legality,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Lkt_Node,
                                As_Bare_Lkt_Node => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);




      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Internal_Semantic_Result_Array_Access;
                     Inc_Ref (Property_Result);

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 161 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := No_Internal_Semantic_Result_Array_Type; Inc_Ref (Let_Result);
--# end
--# expr-done 161

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_41;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_41;


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Internal_Semantic_Result_Array_Access,
                        As_Internal_Semantic_Result_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;

         end if;
      end if;



   return Property_Result;
end Lkt_Node_P_Check_Legality;
--# end

   







--# property-start LktNode.check_correctness_pre /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:437
pragma Warnings (Off, "is not referenced");
 function Lkt_Node_P_Check_Correctness_Pre
  
  (Node : Bare_Lkt_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Lkt_Node  := Bare_Lkt_Node (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_42 with Inline_Always;

      Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_42 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_42;

      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Lkt_Node_Lkt_Node_P_Check_Correctness_Pre,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Lkt_Node,
                                As_Bare_Lkt_Node => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);




      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Internal_Semantic_Result_Array_Access;
                     Inc_Ref (Property_Result);

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 162 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := No_Internal_Semantic_Result_Array_Type; Inc_Ref (Let_Result);
--# end
--# expr-done 162

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_42;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_42;


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Internal_Semantic_Result_Array_Access,
                        As_Internal_Semantic_Result_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;

         end if;
      end if;



   return Property_Result;
end Lkt_Node_P_Check_Correctness_Pre;
--# end



   


      

   --
   --  Primitives for Bare_Base_Lexer_Case_Rule_Alt
   --

   




      
   function Base_Lexer_Case_Rule_Alt_F_Send
     (Node : Bare_Base_Lexer_Case_Rule_Alt) return Bare_Lexer_Case_Rule_Send
   is
      

         Kind : constant Lkt_Base_Lexer_Case_Rule_Alt := Node.Kind;
   begin
         case Kind is
               when Lkt_Lexer_Case_Rule_Cond_Alt =>
                     
         return Node.Lexer_Case_Rule_Cond_Alt_F_Send;
      
               when Lkt_Lexer_Case_Rule_Default_Alt =>
                     
         return Node.Lexer_Case_Rule_Default_Alt_F_Send;
      
         end case;
   end;




   


      

   --
   --  Primitives for Bare_Lexer_Case_Rule_Cond_Alt
   --

   



      
      procedure Initialize_Fields_For_Lexer_Case_Rule_Cond_Alt
        (Self : Bare_Lexer_Case_Rule_Cond_Alt
         ; Lexer_Case_Rule_Cond_Alt_F_Cond_Exprs : Bare_Ref_Id_List
         ; Lexer_Case_Rule_Cond_Alt_F_Send : Bare_Lexer_Case_Rule_Send
        ) is
      begin

            Self.Lexer_Case_Rule_Cond_Alt_F_Cond_Exprs := Lexer_Case_Rule_Cond_Alt_F_Cond_Exprs;
            Self.Lexer_Case_Rule_Cond_Alt_F_Send := Lexer_Case_Rule_Cond_Alt_F_Send;
         

      end Initialize_Fields_For_Lexer_Case_Rule_Cond_Alt;

      
   function Lexer_Case_Rule_Cond_Alt_F_Cond_Exprs
     (Node : Bare_Lexer_Case_Rule_Cond_Alt) return Bare_Ref_Id_List
   is
      

   begin
         
         return Node.Lexer_Case_Rule_Cond_Alt_F_Cond_Exprs;
      
   end;




   


      

   --
   --  Primitives for Bare_Lexer_Case_Rule_Default_Alt
   --

   



      
      procedure Initialize_Fields_For_Lexer_Case_Rule_Default_Alt
        (Self : Bare_Lexer_Case_Rule_Default_Alt
         ; Lexer_Case_Rule_Default_Alt_F_Send : Bare_Lexer_Case_Rule_Send
        ) is
      begin

            Self.Lexer_Case_Rule_Default_Alt_F_Send := Lexer_Case_Rule_Default_Alt_F_Send;
         

      end Initialize_Fields_For_Lexer_Case_Rule_Default_Alt;




   


      

   --
   --  Primitives for Bare_Block_String_Line
   --

   







   


      

   --
   --  Primitives for Bare_Class_Qualifier
   --

   





   







--# property-start '[dispatcher]ClassQualifier.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Class_Qualifier_P_As_Bool
  
  (Node : Bare_Class_Qualifier
  )

   return Boolean
is
   Self : Bare_Class_Qualifier  := Bare_Class_Qualifier (Node);
      --# bind self Self

   


   Property_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Class_Qualifier (Self.Kind) is
               when Lkt_Class_Qualifier_Absent =>
                  --# property-call-start ClassQualifier.Absent.as_bool
                  Property_Result := Class_Qualifier_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Lkt_Class_Qualifier_Present =>
                  --# property-call-start ClassQualifier.Present.as_bool
                  Property_Result := Class_Qualifier_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Class_Qualifier_P_As_Bool;
--# end



   


      

   --
   --  Primitives for Bare_Class_Qualifier_Absent
   --

   





   







--# property-start ClassQualifier.Absent.as_bool /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1953
pragma Warnings (Off, "is not referenced");
 function Class_Qualifier_Absent_P_As_Bool
  
  (Node : Bare_Class_Qualifier_Absent
  )

   return Boolean
is
   Self : Bare_Class_Qualifier_Absent  := Bare_Class_Qualifier_Absent (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Class_Qualifier_Absent_P_As_Bool;
--# end



   


      

   --
   --  Primitives for Bare_Class_Qualifier_Present
   --

   





   







--# property-start ClassQualifier.Present.as_bool /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1953
pragma Warnings (Off, "is not referenced");
 function Class_Qualifier_Present_P_As_Bool
  
  (Node : Bare_Class_Qualifier_Present
  )

   return Boolean
is
   Self : Bare_Class_Qualifier_Present  := Bare_Class_Qualifier_Present (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := True;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Class_Qualifier_Present_P_As_Bool;
--# end



   


      

   --
   --  Primitives for Bare_Decl
   --

   



   

   

   

   

   

   

   


      procedure Decl_Pre_Env_Actions
        (Self            : Bare_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mapping : Internal_Env_Assoc :=
            Liblktlang.Implementation.Env_Mappings_2 (Node => 
Self);
      begin

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:739");
         Dec_Ref (Mapping.Dest_Env);

      end;
   
   
      end;




      
   function Decl_F_Syn_Name
     (Node : Bare_Decl) return Bare_Def_Id
   is
      

         Kind : constant Lkt_Decl := Node.Kind;
   begin
         case Kind is
               when Lkt_Lexer_Decl =>
                     
         return Node.Lexer_Decl_F_Syn_Name;
      
               when Lkt_Lexer_Family_Decl =>
                     
         return Node.Lexer_Family_Decl_F_Syn_Name;
      
               when Lkt_Grammar_Decl =>
                     
         return Node.Grammar_Decl_F_Syn_Name;
      
               when Lkt_Grammar_Rule_Decl =>
                     
         return Node.Grammar_Rule_Decl_F_Syn_Name;
      
               when Lkt_Synthetic_Lexer_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Function_Type =>
                     return No_Bare_Lkt_Node;
               when Lkt_Generic_Formal_Type_Decl =>
                     
         return Node.Generic_Formal_Type_Decl_F_Syn_Name;
      
               when Lkt_Generic_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Instantiated_Generic_Type =>
                     return No_Bare_Lkt_Node;
               when Lkt_Trait_Decl =>
                     
         return Node.Trait_Decl_F_Syn_Name;
      
               when Lkt_Enum_Type_Decl =>
                     
         return Node.Enum_Type_Decl_F_Syn_Name;
      
               when Lkt_Struct_Decl =>
                     
         return Node.Struct_Decl_F_Syn_Name;
      
               when Lkt_Class_Decl | Lkt_Enum_Class_Decl =>
                     
         return Node.Basic_Class_Decl_F_Syn_Name;
      
               when Lkt_Env_Spec_Decl =>
                     
         return Node.Env_Spec_Decl_F_Syn_Name;
      
               when Lkt_Enum_Class_Alt_Decl =>
                     
         return Node.Enum_Class_Alt_Decl_F_Syn_Name;
      
               when Lkt_Self_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Node_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Fun_Decl =>
                     
         return Node.Fun_Decl_F_Syn_Name;
      
               when Lkt_Enum_Lit_Decl =>
                     
         return Node.Enum_Lit_Decl_F_Syn_Name;
      
               when Lkt_Fun_Arg_Decl =>
                     
         return Node.Fun_Arg_Decl_F_Syn_Name;
      
               when Lkt_Lambda_Arg_Decl =>
                     
         return Node.Lambda_Arg_Decl_F_Syn_Name;
      
               when Lkt_Field_Decl =>
                     
         return Node.Field_Decl_F_Syn_Name;
      
               when Lkt_Match_Val_Decl =>
                     
         return Node.Match_Val_Decl_F_Syn_Name;
      
               when Lkt_Dyn_Var_Decl =>
                     
         return Node.Dyn_Var_Decl_F_Syn_Name;
      
               when Lkt_Val_Decl =>
                     
         return Node.Val_Decl_F_Syn_Name;
      
         end case;
   end;


   







--# property-start '[dispatcher]Decl.name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Decl_P_Name
  
  (Node : Bare_Decl
  )

   return Symbol_Type
is
   Self : Bare_Decl  := Bare_Decl (Node);
      --# bind self Self

   


   Property_Result : Symbol_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Decl (Self.Kind) is
               when Lkt_Grammar_Rule_Decl | Lkt_Enum_Lit_Decl .. Lkt_Env_Spec_Decl | Lkt_Grammar_Decl .. Lkt_Generic_Formal_Type_Decl | Lkt_Class_Decl .. Lkt_Trait_Decl =>
                  --# property-call-start Decl.name
                  Property_Result := Decl_P_Name
                    (Self
                    );
                  --# end
               when Lkt_Synthetic_Lexer_Decl =>
                  --# property-call-start SyntheticLexerDecl.name
                  Property_Result := Synthetic_Lexer_Decl_P_Name
                    (Self
                    );
                  --# end
               when Lkt_Node_Decl =>
                  --# property-call-start NodeDecl.name
                  Property_Result := Node_Decl_P_Name
                    (Self
                    );
                  --# end
               when Lkt_Self_Decl =>
                  --# property-call-start SelfDecl.name
                  Property_Result := Self_Decl_P_Name
                    (Self
                    );
                  --# end
               when Lkt_Generic_Decl =>
                  --# property-call-start GenericDecl.name
                  Property_Result := Generic_Decl_P_Name
                    (Self
                    );
                  --# end
               when Lkt_Instantiated_Generic_Type =>
                  --# property-call-start InstantiatedGenericType.name
                  Property_Result := Instantiated_Generic_Type_P_Name
                    (Self
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Decl_P_Name;
--# end

   







--# property-start '[dispatcher]Decl.full_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Decl_P_Full_Name
  
  (Node : Bare_Decl
  )

   return String_Type
is
   Self : Bare_Decl  := Bare_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Decl (Self.Kind) is
               when Lkt_Grammar_Rule_Decl .. Lkt_Enum_Class_Alt_Decl | Lkt_Generic_Formal_Type_Decl | Lkt_Class_Decl .. Lkt_Trait_Decl =>
                  --# property-call-start Decl.full_name
                  Property_Result := Decl_P_Full_Name
                    (Self
                    );
                  --# end
               when Lkt_Function_Type =>
                  --# property-call-start FunctionType.full_name
                  Property_Result := Function_Type_P_Full_Name
                    (Self
                    );
                  --# end
               when Lkt_Instantiated_Generic_Type =>
                  --# property-call-start InstantiatedGenericType.full_name
                  Property_Result := Instantiated_Generic_Type_P_Full_Name
                    (Self
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Decl_P_Full_Name;
--# end

   







--# property-start '[dispatcher]Decl.decl_type_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Decl_P_Decl_Type_Name
  
  (Node : Bare_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Decl  := Bare_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Decl (Self.Kind) is
               when Lkt_Grammar_Rule_Decl =>
                  --# property-call-start GrammarRuleDecl.decl_type_name
                  Property_Result := Grammar_Rule_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Synthetic_Lexer_Decl =>
                  --# property-call-start SyntheticLexerDecl.decl_type_name
                  Property_Result := Synthetic_Lexer_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Node_Decl =>
                  --# property-call-start NodeDecl.decl_type_name
                  Property_Result := Node_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Self_Decl =>
                  --# property-call-start SelfDecl.decl_type_name
                  Property_Result := Self_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Enum_Lit_Decl =>
                  --# property-call-start EnumLitDecl.decl_type_name
                  Property_Result := Enum_Lit_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Field_Decl =>
                  --# property-call-start FieldDecl.decl_type_name
                  Property_Result := Field_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Fun_Arg_Decl =>
                  --# property-call-start FunArgDecl.decl_type_name
                  Property_Result := Fun_Arg_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Lambda_Arg_Decl =>
                  --# property-call-start LambdaArgDecl.decl_type_name
                  Property_Result := Lambda_Arg_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Dyn_Var_Decl =>
                  --# property-call-start DynVarDecl.decl_type_name
                  Property_Result := Dyn_Var_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Match_Val_Decl =>
                  --# property-call-start MatchValDecl.decl_type_name
                  Property_Result := Match_Val_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Val_Decl =>
                  --# property-call-start ValDecl.decl_type_name
                  Property_Result := Val_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Fun_Decl =>
                  --# property-call-start FunDecl.decl_type_name
                  Property_Result := Fun_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Env_Spec_Decl =>
                  --# property-call-start EnvSpecDecl.decl_type_name
                  Property_Result := Env_Spec_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Generic_Decl =>
                  --# property-call-start GenericDecl.decl_type_name
                  Property_Result := Generic_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Grammar_Decl =>
                  --# property-call-start GrammarDecl.decl_type_name
                  Property_Result := Grammar_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Lexer_Decl =>
                  --# property-call-start LexerDecl.decl_type_name
                  Property_Result := Lexer_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Lexer_Family_Decl =>
                  --# property-call-start LexerFamilyDecl.decl_type_name
                  Property_Result := Lexer_Family_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Enum_Class_Alt_Decl =>
                  --# property-call-start EnumClassAltDecl.decl_type_name
                  Property_Result := Enum_Class_Alt_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Function_Type =>
                  --# property-call-start FunctionType.decl_type_name
                  Property_Result := Function_Type_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormalTypeDecl.decl_type_name
                  Property_Result := Generic_Formal_Type_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Instantiated_Generic_Type =>
                  --# property-call-start InstantiatedGenericType.decl_type_name
                  Property_Result := Instantiated_Generic_Type_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Class_Decl =>
                  --# property-call-start ClassDecl.decl_type_name
                  Property_Result := Class_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Enum_Class_Decl =>
                  --# property-call-start EnumClassDecl.decl_type_name
                  Property_Result := Enum_Class_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Enum_Type_Decl =>
                  --# property-call-start EnumTypeDecl.decl_type_name
                  Property_Result := Enum_Type_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Struct_Decl =>
                  --# property-call-start StructDecl.decl_type_name
                  Property_Result := Struct_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Trait_Decl =>
                  --# property-call-start TraitDecl.decl_type_name
                  Property_Result := Trait_Decl_P_Decl_Type_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Decl_P_Decl_Type_Name;
--# end

   







--# property-start Decl.full_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:728
pragma Warnings (Off, "is not referenced");
 function Decl_P_Full_Decl
  
  (Node : Bare_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Full_Decl
is
   Self : Bare_Decl  := Bare_Decl (Node);
     Ent : Internal_Entity_Decl :=
       Internal_Entity_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Full_Decl;

      

      Match_0 : Internal_Entity_Full_Decl;
Match_1 : Internal_Entity_Generic_Decl;
Match_2 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Full_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Full_Decl;
Let_Result : Internal_Entity_Full_Decl;
Scope_Result : Internal_Entity_Full_Decl;
Cast_Result_1 : Internal_Entity_Generic_Decl;
Let_Result_1 : Internal_Entity_Full_Decl;
Scope_Result_1 : Internal_Entity_Full_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Full_Decl;
Scope_Result_2 : Internal_Entity_Full_Decl;
Match_Result : Internal_Entity_Full_Decl;
Let_Result_3 : Internal_Entity_Full_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 166 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 165 '<Match at parser.py:730>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:730






--# expr-start 163 '<parent at parser.py:730>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:730






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 163


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Fld; 

case Lkt_Lkt_Node (Match_Prefix.Node.Kind) is
         when Lkt_Full_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result := Create_Internal_Entity_Full_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result; 
--# bind fd Match_0

Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;


   --# end


            Match_Result := Scope_Result; 
         when Lkt_Generic_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_1 := Create_Internal_Entity_Generic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_1; 
--# bind gd Match_1
--# expr-start 164 '<FieldAccess for full_decl at parser.py:732>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:732






   

      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.full_decl
Fld_1 := Liblktlang.Implementation.Decl_P_Full_Decl (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 164
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;


   --# end


            Match_Result := Scope_Result_1; 
         when Lkt_Lexer_Case_Rule_Cond_Alt .. Lkt_Env_Spec_Decl | Lkt_Grammar_Decl .. Lkt_Un_Op | Lkt_Grammar_List_Sep .. Lkt_Var_Bind =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_2 := Cast_Result_2; 
--# bind _ Match_2

Let_Result_2 := raise Property_Error with "should not happen"; 
--# end
Scope_Result_2 := Let_Result_2;


   --# end


            Match_Result := Scope_Result_2; 
end case;

--# expr-done 165
Let_Result_3 := Match_Result; 
--# end
--# expr-done 166

         Property_Result := Let_Result_3;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Decl_P_Full_Decl;
--# end

   







--# property-start Decl.quoted_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:737
pragma Warnings (Off, "is not referenced");
 function Decl_P_Quoted_Name
  
  (Node : Bare_Decl
  )

   return String_Type
is
   Self : Bare_Decl  := Bare_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_75 with Inline_Always;

      Str : String_Type;
Fld : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;

            procedure Finalizer_Scope_75 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_75;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 167 '<String at parser.py:737>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:737
Str := Create_String ("`"); 
--# expr-done 167
--# expr-start 168 '<FieldAccess for full_name at parser.py:737>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:737






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Self);
--# end
--# expr-done 168
Concat_Result := Concat_String (Str, Fld); 
--# expr-start 169 '<String at parser.py:737>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:737
Str_1 := Create_String ("`"); 
--# expr-done 169
Concat_Result_1 := Concat_String (Concat_Result, Str_1); 

         Property_Result := Concat_Result_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_75;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_75;




            raise;
      end;



   return Property_Result;
end Decl_P_Quoted_Name;
--# end

   







--# property-start Decl.assert_bare /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:744
pragma Warnings (Off, "is not referenced");
 function Decl_P_Assert_Bare
  
  (Node : Bare_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Bare_Decl
is
   Self : Bare_Decl  := Bare_Decl (Node);
     Ent : Internal_Entity_Decl :=
       Internal_Entity_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Bare_Decl;

      

      Fld : Bare_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Is_Equal : Boolean;
If_Result : Bare_Decl;
Let_Result : Bare_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 175 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 174 '<If at parser.py:746>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:746



--# expr-start 172 '<Eq at parser.py:747>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:747
--# expr-start 171 '<FieldAccess for rebindings at parser.py:747>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:747



--# expr-start 170 '<FieldAccess for info at parser.py:747>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:747






   




Fld_1 := Ent.Info;
--# expr-done 170


   




Fld_2 := Fld_1.Rebindings;
--# expr-done 171

Is_Equal := Fld_2 = null; 
--# expr-done 172
if Is_Equal then
   --# expr-start 173 '<FieldAccess for node at parser.py:748>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:748






   




Fld := Ent.Node;
--# expr-done 173
   If_Result := Fld;
else
   
   If_Result := raise Property_Error with "Type decl has rebindings but shouldn't";
end if;



--# expr-done 174
Let_Result := If_Result; 
--# end
--# expr-done 175

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Decl_P_Assert_Bare;
--# end

   







--# property-start Decl.as_bare_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:752
pragma Warnings (Off, "is not referenced");
 function Decl_P_As_Bare_Decl
  
  (Node : Bare_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Decl
is
   Self : Bare_Decl  := Bare_Decl (Node);
     Ent : Internal_Entity_Decl :=
       Internal_Entity_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Decl;

      

      Fld : Bare_Decl;
As_Entity : Internal_Entity_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Decl;
Let_Result : Internal_Entity_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 178 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 177 '<as_bare_entity at parser.py:761>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:761



--# expr-start 176 '<FieldAccess for node at parser.py:761>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:761






   




Fld := Ent.Node;
--# expr-done 176
Is_Null := Fld = null; 
if Is_Null then
   
   If_Result := No_Entity_Decl;
else
   

As_Entity := (Info => No_Entity_Info, Node => Fld); 
   If_Result := As_Entity;
end if;



--# expr-done 177
Let_Result := If_Result; 
--# end
--# expr-done 178

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Decl_P_As_Bare_Decl;
--# end

   







--# property-start '[dispatcher]Decl.call_scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Decl_P_Call_Scope
  
  (Node : Bare_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Decl  := Bare_Decl (Node);
      --# bind self Self

   


   Property_Result : Lexical_Env;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Decl (Self.Kind) is
               when Lkt_Grammar_Rule_Decl .. Lkt_Val_Decl | Lkt_Env_Spec_Decl .. Lkt_Lexer_Family_Decl =>
                  --# property-call-start Decl.call_scope
                  Property_Result := Decl_P_Call_Scope
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Fun_Decl =>
                  --# property-call-start FunDecl.call_scope
                  Property_Result := Fun_Decl_P_Call_Scope
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Enum_Class_Alt_Decl .. Lkt_Trait_Decl =>
                  --# property-call-start TypeDecl.call_scope
                  Property_Result := Type_Decl_P_Call_Scope
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Decl_P_Call_Scope;
--# end

   







--# property-start Decl.called_decl_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:772
pragma Warnings (Off, "is not referenced");
 function Decl_P_Called_Decl_Type
  
  (Node : Bare_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Decl  := Bare_Decl (Node);
     Ent : Internal_Entity_Decl :=
       Internal_Entity_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Fun_Decl;
Match_2 : Internal_Entity_Decl;
Fld : Internal_Entity_Type_Ref;
Fld_1 : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Fun_Decl;
Let_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity_Decl;
Let_Result_2 : Internal_Entity_Type_Decl;
Scope_Result_2 : Internal_Entity_Type_Decl;
Match_Result : Internal_Entity_Type_Decl;
Let_Result_3 : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 182 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 181 '<Match at parser.py:779>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:779









   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Ent; 

case Lkt_Decl (Match_Prefix.Node.Kind) is
         when Lkt_Enum_Class_Alt_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result; 
--# bind t Match_0

Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;


   --# end


            Match_Result := Scope_Result; 
         when Lkt_Fun_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_1 := Create_Internal_Entity_Fun_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_1; 
--# bind f Match_1
--# expr-start 180 '<FieldAccess for designated_type at parser.py:781>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:781



--# expr-start 179 '<FieldAccess for return_type at parser.py:781>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:781






   

      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref (Node => Match_1.Node.Fun_Decl_F_Return_Type, Info => Match_1.Info);
--# expr-done 179


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 180
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;


   --# end


            Match_Result := Scope_Result_1; 
         when Lkt_Grammar_Rule_Decl .. Lkt_Val_Decl | Lkt_Env_Spec_Decl .. Lkt_Lexer_Family_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_2 := Create_Internal_Entity_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_2 := Cast_Result_2; 
--# bind _ Match_2

Let_Result_2 := raise Property_Error with "Should not happen"; 
--# end
Scope_Result_2 := Let_Result_2;


   --# end


            Match_Result := Scope_Result_2; 
end case;

--# expr-done 181
Let_Result_3 := Match_Result; 
--# end
--# expr-done 182

         Property_Result := Let_Result_3;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Decl_P_Called_Decl_Type;
--# end

   







--# property-start Decl.is_generic_fun_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:785
pragma Warnings (Off, "is not referenced");
 function Decl_P_Is_Generic_Fun_Decl
  
  (Node : Bare_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Decl  := Bare_Decl (Node);
     Ent : Internal_Entity_Decl :=
       Internal_Entity_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Boolean;

      

      Var_Expr_6 : Internal_Entity_Generic_Decl;
Cast_Expr : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Decl;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 186 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 183 '<Cast to GenericDecl at parser.py:790>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:790






Cast_Expr := Ent; 



   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Lkt_Generic_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);

   else
         Cast_Result := No_Entity_Generic_Decl;
   end if;


--# expr-done 183
Var_Expr_6 := Cast_Result;





if Var_Expr_6 /= No_Entity_Generic_Decl then
   



   --# scope-start


   

--# expr-start 185 '<IsA FunDecl at parser.py:790>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:790
--# expr-start 184 '<FieldAccess for decl at parser.py:790>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:790






   

      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Decl (Node => Var_Expr_6.Node.Generic_Decl_F_Decl, Info => Var_Expr_6.Info);
--# expr-done 184
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Lkt_Fun_Decl_Range; 
--# expr-done 185
Scope_Result := Is_A;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := False;
end if;



Let_Result := Result_Var; 
--# end
--# expr-done 186

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Decl_P_Is_Generic_Fun_Decl;
--# end

   







--# property-start Decl._env_mappings_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:739
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_2
  
  (Node : Bare_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc
is
   Self : Bare_Decl  := Bare_Decl (Node);
     Ent : Internal_Entity_Decl :=
       Internal_Entity_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc;

      
            procedure Finalizer_Scope_280 with Inline_Always;

      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Decl;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;

            procedure Finalizer_Scope_280 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_280;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 189 '<New EnvAssoc at parser.py:739>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:739
--# expr-start 187 '<New DesignatedEnv at parser.py:739>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:739



Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 187
--# expr-start 188 '<FieldAccess for name at parser.py:740>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:740






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Ent.Node);
--# end
--# expr-done 188







Cast_Expr := Self; 



   
      Cast_Result := Cast_Expr;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 189

         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_280;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_280;




            raise;
      end;



   return Property_Result;
end Env_Mappings_2;
--# end

   







--# property-start Decl.name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:706
pragma Warnings (Off, "is not referenced");
 function Decl_P_Name
  
  (Node : Bare_Decl
  )

   return Symbol_Type
is
   Self : Bare_Decl  := Bare_Decl (Node);
      --# bind self Self

   


   Property_Result : Symbol_Type;

      

      Var_Expr_5 : Bare_Def_Id;
Fld : Bare_Def_Id;
Sym : Symbol_Type;
Scope_Result : Symbol_Type;
Result_Var : Symbol_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         


--# expr-start 190 '<FieldAccess for syn_name at parser.py:706>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:706






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Implementation.Decl_F_Syn_Name (Self);
--# expr-done 190
Var_Expr_5 := Fld;





if Var_Expr_5 /= No_Bare_Lkt_Node then
   



   --# scope-start


   

--# expr-start 191 '<GetSymbol at parser.py:706>' Sym /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:706

Sym := Get_Symbol (Var_Expr_5); 
--# expr-done 191
Scope_Result := Sym;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Symbol;
end if;




         Property_Result := Result_Var;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Decl_P_Name;
--# end

   







--# property-start Decl.full_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:710
pragma Warnings (Off, "is not referenced");
 function Decl_P_Full_Name
  
  (Node : Bare_Decl
  )

   return String_Type
is
   Self : Bare_Decl  := Bare_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_72 with Inline_Always;

      Fld : Symbol_Type;
Fld_1 : String_Type;

            procedure Finalizer_Scope_72 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_72;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 193 '<FieldAccess for image at parser.py:711>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:711



--# expr-start 192 '<FieldAccess for name at parser.py:711>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:711






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Self);
--# end
--# expr-done 192


   




Fld_1 := Liblktlang.Implementation.Image (Self => Fld);
--# expr-done 193

         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_72;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_72;




            raise;
      end;



   return Property_Result;
end Decl_P_Full_Name;
--# end

   







--# property-start Decl.call_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:763
pragma Warnings (Off, "is not referenced");
 function Decl_P_Call_Scope
  
  (Node : Bare_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Decl  := Bare_Decl (Node);
      --# bind self Self

   


   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_78 with Inline_Always;

      Let_Result : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_78 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_78;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 194 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := Empty_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 194

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_78;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_78;




            raise;
      end;



   return Property_Result;
end Decl_P_Call_Scope;
--# end



   


      

   --
   --  Primitives for Bare_Base_Grammar_Rule_Decl
   --

   




      
   function Base_Grammar_Rule_Decl_F_Expr
     (Node : Bare_Base_Grammar_Rule_Decl) return Bare_Grammar_Expr
   is
      

         Kind : constant Lkt_Base_Grammar_Rule_Decl := Node.Kind;
   begin
         case Kind is
               when Lkt_Grammar_Rule_Decl =>
                     
         return Node.Grammar_Rule_Decl_F_Expr;
      
               when Lkt_Synthetic_Lexer_Decl =>
                     return No_Bare_Lkt_Node;
         end case;
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Rule_Decl
   --

   



      
      procedure Initialize_Fields_For_Grammar_Rule_Decl
        (Self : Bare_Grammar_Rule_Decl
         ; Grammar_Rule_Decl_F_Syn_Name : Bare_Def_Id
         ; Grammar_Rule_Decl_F_Expr : Bare_Grammar_Expr
        ) is
      begin

            Self.Grammar_Rule_Decl_F_Syn_Name := Grammar_Rule_Decl_F_Syn_Name;
            Self.Grammar_Rule_Decl_F_Expr := Grammar_Rule_Decl_F_Expr;
         

      end Initialize_Fields_For_Grammar_Rule_Decl;


   







--# property-start GrammarRuleDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1314
pragma Warnings (Off, "is not referenced");
 function Grammar_Rule_Decl_P_Decl_Type_Name
  
  (Node : Bare_Grammar_Rule_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Grammar_Rule_Decl  := Bare_Grammar_Rule_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_105 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_105 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_105;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 195 '<String at parser.py:1314>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1314
Str := Create_String ("grammar rule declaration"); 
--# expr-done 195

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_105;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_105;




            raise;
      end;



   return Property_Result;
end Grammar_Rule_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Synthetic_Lexer_Decl
   --

   



      
      procedure Initialize_Fields_For_Synthetic_Lexer_Decl
        (Self : Bare_Synthetic_Lexer_Decl
        ) is
      begin

         
      Self.Synthetic_Lexer_Decl_Sym := No_Symbol;

      end Initialize_Fields_For_Synthetic_Lexer_Decl;


   







--# property-start SyntheticLexerDecl.name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1322
pragma Warnings (Off, "is not referenced");
 function Synthetic_Lexer_Decl_P_Name
  
  (Node : Bare_Synthetic_Lexer_Decl
  )

   return Symbol_Type
is
   Self : Bare_Synthetic_Lexer_Decl  := Bare_Synthetic_Lexer_Decl (Node);
      --# bind self Self

   


   Property_Result : Symbol_Type;

      

      Fld : Symbol_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 196 '<FieldAccess for sym at parser.py:1322>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1322






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Synthetic_Lexer_Decl_Sym;
--# expr-done 196

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Synthetic_Lexer_Decl_P_Name;
--# end

   







--# property-start SyntheticLexerDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1325
pragma Warnings (Off, "is not referenced");
 function Synthetic_Lexer_Decl_P_Decl_Type_Name
  
  (Node : Bare_Synthetic_Lexer_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Synthetic_Lexer_Decl  := Bare_Synthetic_Lexer_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_107 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_107 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_107;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 197 '<String at parser.py:1325>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1325
Str := Create_String ("synthetic lexer declaration"); 
--# expr-done 197

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_107;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_107;




            raise;
      end;



   return Property_Result;
end Synthetic_Lexer_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Base_Val_Decl
   --

   





   







--# property-start '[dispatcher]BaseValDecl.get_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Val_Decl_P_Get_Type
  
  (Node : Bare_Base_Val_Decl
      ; No_Inference : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Base_Val_Decl  := Bare_Base_Val_Decl (Node);
      --# bind self Self

   

   --# bind no_inference No_Inference

   Property_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Base_Val_Decl (Self.Kind) is
               when Lkt_Node_Decl =>
                  --# property-call-start NodeDecl.get_type
                  Property_Result := Node_Decl_P_Get_Type
                    (Self
                        , No_Inference
                        , E_Info
                    );
                  --# end
               when Lkt_Self_Decl =>
                  --# property-call-start SelfDecl.get_type
                  Property_Result := Self_Decl_P_Get_Type
                    (Self
                        , No_Inference
                        , E_Info
                    );
                  --# end
               when Lkt_Enum_Lit_Decl =>
                  --# property-call-start EnumLitDecl.get_type
                  Property_Result := Enum_Lit_Decl_P_Get_Type
                    (Self
                        , No_Inference
                        , E_Info
                    );
                  --# end
               when Lkt_Field_Decl .. Lkt_Fun_Arg_Decl | Lkt_Dyn_Var_Decl .. Lkt_Match_Val_Decl =>
                  --# property-call-start ExplicitlyTypedDecl.get_type
                  Property_Result := Explicitly_Typed_Decl_P_Get_Type
                    (Self
                        , No_Inference
                        , E_Info
                    );
                  --# end
               when Lkt_Lambda_Arg_Decl =>
                  --# property-call-start LambdaArgDecl.get_type
                  Property_Result := Lambda_Arg_Decl_P_Get_Type
                    (Self
                        , No_Inference
                        , E_Info
                    );
                  --# end
               when Lkt_Val_Decl =>
                  --# property-call-start ValDecl.get_type
                  Property_Result := Val_Decl_P_Get_Type
                    (Self
                        , No_Inference
                        , E_Info
                    );
                  --# end
               when Lkt_Fun_Decl =>
                  --# property-call-start FunDecl.get_type
                  Property_Result := Fun_Decl_P_Get_Type
                    (Self
                        , No_Inference
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Base_Val_Decl_P_Get_Type;
--# end



   


      

   --
   --  Primitives for Bare_Node_Decl
   --

   





   







--# property-start NodeDecl.name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2461
pragma Warnings (Off, "is not referenced");
 function Node_Decl_P_Name
  
  (Node : Bare_Node_Decl
  )

   return Symbol_Type
is
   Self : Bare_Node_Decl  := Bare_Node_Decl (Node);
      --# bind self Self

   


   Property_Result : Symbol_Type;

      

      Sym : Symbol_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 198 '<Symbol node at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Node_31); 
--# expr-done 198

         Property_Result := Sym;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Node_Decl_P_Name;
--# end

   







--# property-start NodeDecl.get_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2463
pragma Warnings (Off, "is not referenced");
 function Node_Decl_P_Get_Type
  
  (Node : Bare_Node_Decl
      ; No_Inference : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Node_Decl  := Bare_Node_Decl (Node);
     Ent : Internal_Entity_Node_Decl :=
       Internal_Entity_Node_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind no_inference No_Inference

   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 201 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 200 '<Cast to TypeDecl at parser.py:2466>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2466





--# expr-start 199 '<parent at parser.py:2466>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2466






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 199



   if Fld.Node = null
      or else Fld.Node.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 200
Let_Result := Cast_Result; 
--# end
--# expr-done 201

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Node_Decl_P_Get_Type;
--# end

   







--# property-start NodeDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2468
pragma Warnings (Off, "is not referenced");
 function Node_Decl_P_Decl_Type_Name
  
  (Node : Bare_Node_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Node_Decl  := Bare_Node_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_193 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_193 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_193;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 202 '<String at parser.py:2468>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2468
Str := Create_String ("node declaration"); 
--# expr-done 202

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_193;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_193;




            raise;
      end;



   return Property_Result;
end Node_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Self_Decl
   --

   





   







--# property-start SelfDecl.name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2443
pragma Warnings (Off, "is not referenced");
 function Self_Decl_P_Name
  
  (Node : Bare_Self_Decl
  )

   return Symbol_Type
is
   Self : Bare_Self_Decl  := Bare_Self_Decl (Node);
      --# bind self Self

   


   Property_Result : Symbol_Type;

      

      Sym : Symbol_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 203 '<Symbol self at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Self); 
--# expr-done 203

         Property_Result := Sym;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Self_Decl_P_Name;
--# end

   







--# property-start SelfDecl.get_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2445
pragma Warnings (Off, "is not referenced");
 function Self_Decl_P_Get_Type
  
  (Node : Bare_Self_Decl
      ; No_Inference : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Self_Decl  := Bare_Self_Decl (Node);
     Ent : Internal_Entity_Self_Decl :=
       Internal_Entity_Self_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind no_inference No_Inference

   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 206 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 205 '<Cast to TypeDecl at parser.py:2448>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2448





--# expr-start 204 '<parent at parser.py:2448>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2448






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 204



   if Fld.Node = null
      or else Fld.Node.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 205
Let_Result := Cast_Result; 
--# end
--# expr-done 206

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Self_Decl_P_Get_Type;
--# end

   







--# property-start SelfDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2450
pragma Warnings (Off, "is not referenced");
 function Self_Decl_P_Decl_Type_Name
  
  (Node : Bare_Self_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Self_Decl  := Bare_Self_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_190 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_190 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_190;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 207 '<String at parser.py:2450>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2450
Str := Create_String ("self declaration"); 
--# expr-done 207

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_190;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_190;




            raise;
      end;



   return Property_Result;
end Self_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_User_Val_Decl
   --

   







   


      

   --
   --  Primitives for Bare_Enum_Lit_Decl
   --

   



      
      procedure Initialize_Fields_For_Enum_Lit_Decl
        (Self : Bare_Enum_Lit_Decl
         ; Enum_Lit_Decl_F_Syn_Name : Bare_Def_Id
        ) is
      begin

            Self.Enum_Lit_Decl_F_Syn_Name := Enum_Lit_Decl_F_Syn_Name;
         

      end Initialize_Fields_For_Enum_Lit_Decl;


   







--# property-start EnumLitDecl.get_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2582
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Decl_P_Get_Type
  
  (Node : Bare_Enum_Lit_Decl
      ; No_Inference : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Enum_Lit_Decl  := Bare_Enum_Lit_Decl (Node);
     Ent : Internal_Entity_Enum_Lit_Decl :=
       Internal_Entity_Enum_Lit_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind no_inference No_Inference

   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_200 with Inline_Always;

      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_200 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_200;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 212 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 211 '<Cast to TypeDecl at parser.py:2587>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2587





--# expr-start 210 '<Find at parser.py:2586>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2586








   

   --# expr-start 208 '<FieldAccess for parents at parser.py:2586>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2586






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Fld := Liblktlang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 208

   
      Find_Result := No_Entity;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_9 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind t Item_9

            
      --# expr-start 209 '<IsA EnumTypeDecl at parser.py:2586>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2586

Is_A := Item_9.Node /= null 
and then Item_9.Node.Kind in Lkt_Enum_Type_Decl_Range; 
--# expr-done 209
      if Is_A then
         Find_Result := Item_9; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 210



   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 211
Let_Result := Cast_Result; 
--# end
--# expr-done 212

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_200;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_200;




            raise;
      end;



   return Property_Result;
end Enum_Lit_Decl_P_Get_Type;
--# end

   







--# property-start EnumLitDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2593
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Decl_P_Decl_Type_Name
  
  (Node : Bare_Enum_Lit_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Enum_Lit_Decl  := Bare_Enum_Lit_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_201 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_201 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_201;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 213 '<String at parser.py:2593>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2593
Str := Create_String ("enum literal declaration"); 
--# expr-done 213

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_201;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_201;




            raise;
      end;



   return Property_Result;
end Enum_Lit_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Explicitly_Typed_Decl
   --

   




      
   function Explicitly_Typed_Decl_F_Decl_Type
     (Node : Bare_Explicitly_Typed_Decl) return Bare_Type_Ref
   is
      

         Kind : constant Lkt_Explicitly_Typed_Decl := Node.Kind;
   begin
         case Kind is
               when Lkt_Fun_Arg_Decl =>
                     
         return Node.Fun_Arg_Decl_F_Decl_Type;
      
               when Lkt_Lambda_Arg_Decl =>
                     
         return Node.Lambda_Arg_Decl_F_Decl_Type;
      
               when Lkt_Field_Decl =>
                     
         return Node.Field_Decl_F_Decl_Type;
      
               when Lkt_Match_Val_Decl =>
                     
         return Node.Match_Val_Decl_F_Decl_Type;
      
               when Lkt_Dyn_Var_Decl =>
                     
         return Node.Dyn_Var_Decl_F_Decl_Type;
      
               when Lkt_Val_Decl =>
                     
         return Node.Val_Decl_F_Decl_Type;
      
         end case;
   end;


   







--# property-start ExplicitlyTypedDecl.get_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2604
pragma Warnings (Off, "is not referenced");
 function Explicitly_Typed_Decl_P_Get_Type
  
  (Node : Bare_Explicitly_Typed_Decl
      ; No_Inference : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Explicitly_Typed_Decl  := Bare_Explicitly_Typed_Decl (Node);
     Ent : Internal_Entity_Explicitly_Typed_Decl :=
       Internal_Entity_Explicitly_Typed_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind no_inference No_Inference

   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity_Type_Ref;
Fld_1 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 216 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 215 '<FieldAccess for designated_type at parser.py:2607>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2607



--# expr-start 214 '<FieldAccess for decl_type at parser.py:2607>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2607






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref (Node => Implementation.Explicitly_Typed_Decl_F_Decl_Type (Ent.Node), Info => Ent.Info);
--# expr-done 214


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 215
Let_Result := Fld_1; 
--# end
--# expr-done 216

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Explicitly_Typed_Decl_P_Get_Type;
--# end



   


      

   --
   --  Primitives for Bare_Component_Decl
   --

   




      
   function Component_Decl_F_Default_Val
     (Node : Bare_Component_Decl) return Bare_Expr
   is
      

         Kind : constant Lkt_Component_Decl := Node.Kind;
   begin
         case Kind is
               when Lkt_Fun_Arg_Decl =>
                     
         return Node.Fun_Arg_Decl_F_Default_Val;
      
               when Lkt_Lambda_Arg_Decl =>
                     
         return Node.Lambda_Arg_Decl_F_Default_Val;
      
               when Lkt_Field_Decl =>
                     
         return Node.Field_Decl_F_Default_Val;
      
         end case;
   end;


   







--# property-start ComponentDecl.to_formal_param /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2621
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_To_Formal_Param
  
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Formal_Param
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
     Ent : Internal_Entity_Component_Decl :=
       Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Formal_Param;

      

      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Internal_Entity_Expr;
New_Struct : Internal_Formal_Param;
Let_Result : Internal_Formal_Param;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 221 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 220 '<New FormalParam at parser.py:2626>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2626
--# expr-start 217 '<FieldAccess for default_val at parser.py:2628>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2628






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Create_Internal_Entity_Expr (Node => Implementation.Component_Decl_F_Default_Val (Ent.Node), Info => Ent.Info);
--# expr-done 217
--# expr-start 218 '<FieldAccess for name at parser.py:2626>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2626






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Self);
--# end
--# expr-done 218
--# expr-start 219 '<FieldAccess for get_type at parser.py:2627>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2627






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]BaseValDecl.get_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Base_Val_Decl_P_Get_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 219
New_Struct := (Default_Value => Fld_2, Formal_Name => Fld, Formal_Type => Fld_1); 
--# expr-done 220
Let_Result := New_Struct; 
--# end
--# expr-done 221

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Component_Decl_P_To_Formal_Param;
--# end



   


      

   --
   --  Primitives for Bare_Field_Decl
   --

   



      
      procedure Initialize_Fields_For_Field_Decl
        (Self : Bare_Field_Decl
         ; Field_Decl_F_Syn_Name : Bare_Def_Id
         ; Field_Decl_F_Decl_Type : Bare_Type_Ref
         ; Field_Decl_F_Default_Val : Bare_Expr
        ) is
      begin

            Self.Field_Decl_F_Syn_Name := Field_Decl_F_Syn_Name;
            Self.Field_Decl_F_Decl_Type := Field_Decl_F_Decl_Type;
            Self.Field_Decl_F_Default_Val := Field_Decl_F_Default_Val;
         

      end Initialize_Fields_For_Field_Decl;


   







--# property-start FieldDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2693
pragma Warnings (Off, "is not referenced");
 function Field_Decl_P_Decl_Type_Name
  
  (Node : Bare_Field_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Field_Decl  := Bare_Field_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_209 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_209 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_209;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 222 '<String at parser.py:2693>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2693
Str := Create_String ("field declaration"); 
--# expr-done 222

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_209;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_209;




            raise;
      end;



   return Property_Result;
end Field_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Fun_Arg_Decl
   --

   



      
      procedure Initialize_Fields_For_Fun_Arg_Decl
        (Self : Bare_Fun_Arg_Decl
         ; Fun_Arg_Decl_F_Decl_Annotations : Bare_Decl_Annotation_List
         ; Fun_Arg_Decl_F_Syn_Name : Bare_Def_Id
         ; Fun_Arg_Decl_F_Decl_Type : Bare_Type_Ref
         ; Fun_Arg_Decl_F_Default_Val : Bare_Expr
        ) is
      begin

            Self.Fun_Arg_Decl_F_Decl_Annotations := Fun_Arg_Decl_F_Decl_Annotations;
            Self.Fun_Arg_Decl_F_Syn_Name := Fun_Arg_Decl_F_Syn_Name;
            Self.Fun_Arg_Decl_F_Decl_Type := Fun_Arg_Decl_F_Decl_Type;
            Self.Fun_Arg_Decl_F_Default_Val := Fun_Arg_Decl_F_Default_Val;
         

      end Initialize_Fields_For_Fun_Arg_Decl;

      
   function Fun_Arg_Decl_F_Decl_Annotations
     (Node : Bare_Fun_Arg_Decl) return Bare_Decl_Annotation_List
   is
      

   begin
         
         return Node.Fun_Arg_Decl_F_Decl_Annotations;
      
   end;


   







--# property-start FunArgDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2639
pragma Warnings (Off, "is not referenced");
 function Fun_Arg_Decl_P_Decl_Type_Name
  
  (Node : Bare_Fun_Arg_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Fun_Arg_Decl  := Bare_Fun_Arg_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_204 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_204 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_204;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 223 '<String at parser.py:2639>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2639
Str := Create_String ("fun arg declaration"); 
--# expr-done 223

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_204;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_204;




            raise;
      end;



   return Property_Result;
end Fun_Arg_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Lambda_Arg_Decl
   --

   



      
      procedure Initialize_Fields_For_Lambda_Arg_Decl
        (Self : Bare_Lambda_Arg_Decl
         ; Lambda_Arg_Decl_F_Syn_Name : Bare_Def_Id
         ; Lambda_Arg_Decl_F_Decl_Type : Bare_Type_Ref
         ; Lambda_Arg_Decl_F_Default_Val : Bare_Expr
        ) is
      begin

            Self.Lambda_Arg_Decl_F_Syn_Name := Lambda_Arg_Decl_F_Syn_Name;
            Self.Lambda_Arg_Decl_F_Decl_Type := Lambda_Arg_Decl_F_Decl_Type;
            Self.Lambda_Arg_Decl_F_Default_Val := Lambda_Arg_Decl_F_Default_Val;
         

      end Initialize_Fields_For_Lambda_Arg_Decl;


   







--# property-start LambdaArgDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2650
pragma Warnings (Off, "is not referenced");
 function Lambda_Arg_Decl_P_Decl_Type_Name
  
  (Node : Bare_Lambda_Arg_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Lambda_Arg_Decl  := Bare_Lambda_Arg_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_205 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_205 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_205;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 224 '<String at parser.py:2650>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2650
Str := Create_String ("lambda arg declaration"); 
--# expr-done 224

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_205;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_205;




            raise;
      end;



   return Property_Result;
end Lambda_Arg_Decl_P_Decl_Type_Name;
--# end

   







--# property-start LambdaArgDecl.owning_lambda /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2652
pragma Warnings (Off, "is not referenced");
 function Lambda_Arg_Decl_P_Owning_Lambda
  
  (Node : Bare_Lambda_Arg_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Lambda_Expr
is
   Self : Bare_Lambda_Arg_Decl  := Bare_Lambda_Arg_Decl (Node);
     Ent : Internal_Entity_Lambda_Arg_Decl :=
       Internal_Entity_Lambda_Arg_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Lambda_Expr;

      
            procedure Finalizer_Scope_206 with Inline_Always;

      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Lambda_Expr;

            procedure Finalizer_Scope_206 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_206;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 228 '<Cast to LambdaExpr at parser.py:2653>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2653





--# expr-start 227 '<Find at parser.py:2653>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2653








   

   --# expr-start 225 '<FieldAccess for parents at parser.py:2653>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2653






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Fld := Liblktlang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 225

   
      Find_Result := No_Entity;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_10 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind p Item_10

            
      --# expr-start 226 '<IsA LambdaExpr at parser.py:2653>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2653

Is_A := Item_10.Node /= null 
and then Item_10.Node.Kind in Lkt_Lambda_Expr_Range; 
--# expr-done 226
      if Is_A then
         Find_Result := Item_10; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 227



   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Lkt_Lambda_Expr_Range
   then
      
      Cast_Result := Create_Internal_Entity_Lambda_Expr
        (Node => Find_Result.Node,
         Info => Find_Result.Info);

   else
         Cast_Result := No_Entity_Lambda_Expr;
   end if;


--# expr-done 228

         Property_Result := Cast_Result;
         
   --# end
      Finalizer_Scope_206;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_206;




            raise;
      end;



   return Property_Result;
end Lambda_Arg_Decl_P_Owning_Lambda;
--# end

   







--# property-start LambdaArgDecl.index /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2656
pragma Warnings (Off, "is not referenced");
 function Lambda_Arg_Decl_P_Index
  
  (Node : Bare_Lambda_Arg_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Integer
is
   Self : Bare_Lambda_Arg_Decl  := Bare_Lambda_Arg_Decl (Node);
     Ent : Internal_Entity_Lambda_Arg_Decl :=
       Internal_Entity_Lambda_Arg_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Integer;

      
            procedure Finalizer_Scope_207 with Inline_Always;

      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Lambda_Arg_Decl_List;
Fld_1 : Bare_Lambda_Arg_Decl_List;
Fld_2 : Internal_Entity_Info;
Item_11 : Internal_Entity_Lambda_Arg_Decl;
Bare_Item_11 : Bare_Lambda_Arg_Decl;
Index_12 : Integer;
As_Entity : Internal_Entity_Lambda_Arg_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Lambda_Arg_Decl;
Cast_Expr : Internal_Entity_Lambda_Arg_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Lambda_Arg_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Integer_Array_Access;
Get_Result : Integer;
Let_Result : Integer;

            procedure Finalizer_Scope_207 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_207;

      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Lambda_Arg_Decl_Lambda_Arg_Decl_P_Index,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Lambda_Arg_Decl,
                                As_Bare_Lambda_Arg_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);




      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Integer;

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 234 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 233 '<at at parser.py:2664>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2664

--# expr-start 232 '<FilterMap at parser.py:2661>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2661








   

   --# expr-start 230 '<Cast to LambdaArgDecl.list at parser.py:2661>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2661





--# expr-start 229 '<parent at parser.py:2661>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2661






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 229



   if Fld.Node = null
      or else Fld.Node.Kind in Lkt_Lambda_Arg_Decl_List_Range
   then
      
      Cast_Result := Create_Internal_Entity_Lambda_Arg_Decl_List
        (Node => Fld.Node,
         Info => Fld.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 230






   




Fld_1 := Cast_Result.Node;

   
      declare
         Map_Result_Vec : Integer_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Integer_Array (0);
   
      else
         
         Index_12 := 0;

      declare
         
         Collection : constant Bare_Lambda_Arg_Decl_List := Fld_1;
      begin
         for Untyped_Item_11 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_11 := Untyped_Item_11; 
                  



Is_Null := Bare_Item_11 = null; 
if Is_Null then
   
   If_Result := No_Entity_Lambda_Arg_Decl;
else
   





   




Fld_2 := Cast_Result.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_11); 
   If_Result := As_Entity;
end if;



                  Item_11 := If_Result; 

            
   --# scope-start


               --# bind arg Item_11
               --# bind i Index_12

            
         --# expr-start 231 '<Eq at parser.py:2663>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2663






Cast_Expr := Ent; 



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);








Cast_Expr_1 := Item_11; 



   
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);


Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 231
         if Is_Equiv then
            
         
      
      

         declare
            Item_To_Append : constant Integer := Index_12;
         begin
            Integer_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end



               Index_12 := Index_12 + 1;
         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Integer_Array
           (Items_Count => Natural (Integer_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Integer_Vectors.Get
              (Map_Result_Vec,
               I + Integer_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Integer_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 232


Get_Result := Get (Self, Map_Result, 0, True); 
--# expr-done 233
Let_Result := Get_Result; 
--# end
--# expr-done 234

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_207;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_207;


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Integer,
                        As_Integer => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);

         end if;
      end if;



   return Property_Result;
end Lambda_Arg_Decl_P_Index;
--# end

   







--# property-start LambdaArgDecl.get_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2666
pragma Warnings (Off, "is not referenced");
 function Lambda_Arg_Decl_P_Get_Type
  
  (Node : Bare_Lambda_Arg_Decl
      ; No_Inference : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Lambda_Arg_Decl  := Bare_Lambda_Arg_Decl (Node);
     Ent : Internal_Entity_Lambda_Arg_Decl :=
       Internal_Entity_Lambda_Arg_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind no_inference No_Inference

   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_342 with Inline_Always;

      Var_Expr : Internal_Entity_Function_Type;
Fld : Internal_Entity_Type_Ref;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Internal_Entity_Lambda_Expr;
Fld_3 : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Function_Type;
Fld_4 : Integer;
Fld_5 : Bare_Type_Decl_Array_Access;
Get_Result : Bare_Type_Decl;
As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
If_Result_1 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_342 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_342;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 245 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 244 '<If at parser.py:2668>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2668




if No_Inference then
   --# expr-start 236 '<FieldAccess for designated_type at parser.py:2672>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2672



--# expr-start 235 '<FieldAccess for decl_type at parser.py:2672>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2672






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref (Node => Ent.Node.Lambda_Arg_Decl_F_Decl_Type, Info => Ent.Info);
--# expr-done 235


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 236
   If_Result_1 := Fld_1;
else
   


--# expr-start 239 '<Cast to FunctionType at parser.py:2679>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2679





--# expr-start 238 '<FieldAccess for expected_type at parser.py:2678>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2678



--# expr-start 237 '<FieldAccess for owning_lambda at parser.py:2678>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2678






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LambdaArgDecl.owning_lambda
Fld_2 := Liblktlang.Implementation.Lambda_Arg_Decl_P_Owning_Lambda (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 237


   

      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_3 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 238



   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Lkt_Function_Type_Range
   then
      
      Cast_Result := Create_Internal_Entity_Function_Type
        (Node => Fld_3.Node,
         Info => Fld_3.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 239
Var_Expr := Cast_Result;





if Var_Expr /= No_Entity_Function_Type then
   



   --# scope-start


   --# bind ft Var_Expr

--# expr-start 243 '<as_entity at parser.py:2680>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2680



--# expr-start 242 '<at at parser.py:2680>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2680

--# expr-start 240 '<FieldAccess for args at parser.py:2680>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2680






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Var_Expr.Node.Function_Type_Args;
Inc_Ref (Fld_5);
--# expr-done 240
--# expr-start 241 '<FieldAccess for index at parser.py:2680>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2680






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LambdaArgDecl.index
Fld_4 := Liblktlang.Implementation.Lambda_Arg_Decl_P_Index (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 241

Get_Result := Get (Self, Fld_5, Fld_4, True); 
--# expr-done 242
Is_Null := Get_Result = null; 
if Is_Null then
   
   If_Result := No_Entity_Type_Decl;
else
   

As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;



--# expr-done 243
Scope_Result := If_Result;


   --# end
      Finalizer_Scope_342;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Type_Decl;
end if;



   If_Result_1 := Result_Var;
end if;



--# expr-done 244
Let_Result := If_Result_1; 
--# end
--# expr-done 245

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_342;




            raise;
      end;



   return Property_Result;
end Lambda_Arg_Decl_P_Get_Type;
--# end



   


      

   --
   --  Primitives for Bare_Dyn_Var_Decl
   --

   



      
      procedure Initialize_Fields_For_Dyn_Var_Decl
        (Self : Bare_Dyn_Var_Decl
         ; Dyn_Var_Decl_F_Syn_Name : Bare_Def_Id
         ; Dyn_Var_Decl_F_Decl_Type : Bare_Type_Ref
        ) is
      begin

            Self.Dyn_Var_Decl_F_Syn_Name := Dyn_Var_Decl_F_Syn_Name;
            Self.Dyn_Var_Decl_F_Decl_Type := Dyn_Var_Decl_F_Decl_Type;
         

      end Initialize_Fields_For_Dyn_Var_Decl;


   







--# property-start DynVarDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3733
pragma Warnings (Off, "is not referenced");
 function Dyn_Var_Decl_P_Decl_Type_Name
  
  (Node : Bare_Dyn_Var_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Dyn_Var_Decl  := Bare_Dyn_Var_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_255 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_255 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_255;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 246 '<String at parser.py:3733>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3733
Str := Create_String ("dynamic variable declaration"); 
--# expr-done 246

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_255;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_255;




            raise;
      end;



   return Property_Result;
end Dyn_Var_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Match_Val_Decl
   --

   



      
      procedure Initialize_Fields_For_Match_Val_Decl
        (Self : Bare_Match_Val_Decl
         ; Match_Val_Decl_F_Syn_Name : Bare_Def_Id
         ; Match_Val_Decl_F_Decl_Type : Bare_Type_Ref
        ) is
      begin

            Self.Match_Val_Decl_F_Syn_Name := Match_Val_Decl_F_Syn_Name;
            Self.Match_Val_Decl_F_Decl_Type := Match_Val_Decl_F_Decl_Type;
         

      end Initialize_Fields_For_Match_Val_Decl;


   







--# property-start MatchValDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3686
pragma Warnings (Off, "is not referenced");
 function Match_Val_Decl_P_Decl_Type_Name
  
  (Node : Bare_Match_Val_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Match_Val_Decl  := Bare_Match_Val_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_253 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_253 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_253;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 247 '<String at parser.py:3686>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3686
Str := Create_String ("match value declaration"); 
--# expr-done 247

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_253;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_253;




            raise;
      end;



   return Property_Result;
end Match_Val_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Val_Decl
   --

   



      
      procedure Initialize_Fields_For_Val_Decl
        (Self : Bare_Val_Decl
         ; Val_Decl_F_Syn_Name : Bare_Def_Id
         ; Val_Decl_F_Decl_Type : Bare_Type_Ref
         ; Val_Decl_F_Val : Bare_Expr
        ) is
      begin

            Self.Val_Decl_F_Syn_Name := Val_Decl_F_Syn_Name;
            Self.Val_Decl_F_Decl_Type := Val_Decl_F_Decl_Type;
            Self.Val_Decl_F_Val := Val_Decl_F_Val;
         

      end Initialize_Fields_For_Val_Decl;

      
   function Val_Decl_F_Val
     (Node : Bare_Val_Decl) return Bare_Expr
   is
      

   begin
         
         return Node.Val_Decl_F_Val;
      
   end;


   







--# property-start ValDecl.get_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3744
pragma Warnings (Off, "is not referenced");
 function Val_Decl_P_Get_Type
  
  (Node : Bare_Val_Decl
      ; No_Inference : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Val_Decl  := Bare_Val_Decl (Node);
     Ent : Internal_Entity_Val_Decl :=
       Internal_Entity_Val_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind no_inference No_Inference

   Property_Result : Internal_Entity_Type_Decl;

      

      Var_Expr : Internal_Entity_Type_Ref;
Fld : Internal_Entity_Type_Ref;
Fld_1 : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Type_Decl;
If_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 253 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 248 '<FieldAccess for decl_type at parser.py:3748>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3748






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref (Node => Ent.Node.Val_Decl_F_Decl_Type, Info => Ent.Info);
--# expr-done 248
Var_Expr := Fld;





if Var_Expr /= No_Entity_Type_Ref then
   



   --# scope-start


   --# bind tpe Var_Expr

--# expr-start 249 '<FieldAccess for designated_type at parser.py:3749>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3749






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 249
Scope_Result := Fld_1;


   --# end


   Result_Var := Scope_Result;
else
   --# expr-start 252 '<If at parser.py:3750>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3750




if No_Inference then
   
   If_Result := No_Entity_Type_Decl;
else
   --# expr-start 251 '<FieldAccess for check_expr_type at parser.py:3753>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3753



--# expr-start 250 '<FieldAccess for val at parser.py:3753>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3753






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Val_Decl_F_Val, Info => Ent.Info);
--# expr-done 250


   

      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_expr_type
Fld_3 := Liblktlang.Implementation.Expr_P_Check_Expr_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 251
   If_Result := Fld_3;
end if;



--# expr-done 252
   Result_Var := If_Result;
end if;



Let_Result := Result_Var; 
--# end
--# expr-done 253

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Val_Decl_P_Get_Type;
--# end

   







--# property-start ValDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3757
pragma Warnings (Off, "is not referenced");
 function Val_Decl_P_Decl_Type_Name
  
  (Node : Bare_Val_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Val_Decl  := Bare_Val_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_257 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_257 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_257;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 254 '<String at parser.py:3757>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3757
Str := Create_String ("value declaration"); 
--# expr-done 254

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_257;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_257;




            raise;
      end;



   return Property_Result;
end Val_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Fun_Decl
   --

   



   

   

   

   

   

   

   


      procedure Fun_Decl_Pre_Env_Actions
        (Self            : Bare_Fun_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mapping : Internal_Env_Assoc :=
            Liblktlang.Implementation.Env_Mappings_17 (Node => 
Self);
      begin

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:2495");
         Dec_Ref (Mapping.Dest_Env);

      end;
   
   
            


   
      if Add_To_Env_Only then
         return;
      end if;

      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Liblktlang.Implementation.Env_Trans_Parent_18 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
   
   
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mappings : Internal_Env_Assoc_Array_Access :=
            Liblktlang.Implementation.Env_Mappings_19 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:2498");

         end loop;
         Dec_Ref (Mappings);
      end;
   
   
      end;



      
      procedure Initialize_Fields_For_Fun_Decl
        (Self : Bare_Fun_Decl
         ; Fun_Decl_F_Syn_Name : Bare_Def_Id
         ; Fun_Decl_F_Args : Bare_Fun_Arg_Decl_List
         ; Fun_Decl_F_Return_Type : Bare_Type_Ref
         ; Fun_Decl_F_Body : Bare_Expr
        ) is
      begin

            Self.Fun_Decl_F_Syn_Name := Fun_Decl_F_Syn_Name;
            Self.Fun_Decl_F_Args := Fun_Decl_F_Args;
            Self.Fun_Decl_F_Return_Type := Fun_Decl_F_Return_Type;
            Self.Fun_Decl_F_Body := Fun_Decl_F_Body;
         

      end Initialize_Fields_For_Fun_Decl;

      
   function Fun_Decl_F_Args
     (Node : Bare_Fun_Decl) return Bare_Fun_Arg_Decl_List
   is
      

   begin
         
         return Node.Fun_Decl_F_Args;
      
   end;

      
   function Fun_Decl_F_Return_Type
     (Node : Bare_Fun_Decl) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Fun_Decl_F_Return_Type;
      
   end;

      
   function Fun_Decl_F_Body
     (Node : Bare_Fun_Decl) return Bare_Expr
   is
      

   begin
         
         return Node.Fun_Decl_F_Body;
      
   end;


   







--# property-start FunDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2488
pragma Warnings (Off, "is not referenced");
 function Fun_Decl_P_Decl_Type_Name
  
  (Node : Bare_Fun_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Fun_Decl  := Bare_Fun_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_194 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_194 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_194;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 255 '<String at parser.py:2488>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2488
Str := Create_String ("function declaration"); 
--# expr-done 255

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_194;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_194;




            raise;
      end;



   return Property_Result;
end Fun_Decl_P_Decl_Type_Name;
--# end

   







--# property-start FunDecl.owning_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2490
pragma Warnings (Off, "is not referenced");
 function Fun_Decl_P_Owning_Type
  
  (Node : Bare_Fun_Decl
  )

   return Bare_Type_Decl
is
   Self : Bare_Fun_Decl  := Bare_Fun_Decl (Node);
      --# bind self Self

   


   Property_Result : Bare_Type_Decl;

      
            procedure Finalizer_Scope_195 with Inline_Always;

      Node_Parents : Bare_Lkt_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Lkt_Node;
Cast_Result : Bare_Type_Decl;

            procedure Finalizer_Scope_195 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_195;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 259 '<Cast to TypeDecl at parser.py:2491>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2491





--# expr-start 258 '<Find at parser.py:2491>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2491








   

   --# expr-start 256 '<FieldAccess for parents at parser.py:2491>' Node_Parents /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2491






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Node_Parents := Parents (Self, True); 
--# expr-done 256

   
      Find_Result := No_Bare_Lkt_Node;
   

   

      

      declare
         
         Collection : constant Bare_Lkt_Node_Array_Access := Node_Parents;
      begin
         for Item_13 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind t Item_13

            
      --# expr-start 257 '<IsA TypeDecl at parser.py:2491>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2491

Is_A := Item_13 /= null 
and then Item_13.Kind in Lkt_Type_Decl; 
--# expr-done 257
      if Is_A then
         Find_Result := Item_13; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 258



   if Find_Result = null
      or else Find_Result.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Find_Result;

   else
         Cast_Result := No_Bare_Lkt_Node;
   end if;


--# expr-done 259

         Property_Result := Cast_Result;
         
   --# end
      Finalizer_Scope_195;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_195;




            raise;
      end;



   return Property_Result;
end Fun_Decl_P_Owning_Type;
--# end

   







--# property-start FunDecl.call_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2504
pragma Warnings (Off, "is not referenced");
 function Fun_Decl_P_Call_Scope
  
  (Node : Bare_Fun_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Fun_Decl  := Bare_Fun_Decl (Node);
     Ent : Internal_Entity_Fun_Decl :=
       Internal_Entity_Fun_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_196 with Inline_Always;

      Fld : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_196 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_196;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 260 '<FieldAccess for children_env at parser.py:2504>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2504






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 260

         Property_Result := Fld;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_196;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_196;




            raise;
      end;



   return Property_Result;
end Fun_Decl_P_Call_Scope;
--# end

   







--# property-start FunDecl.is_property /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2506
pragma Warnings (Off, "is not referenced");
 function Fun_Decl_P_Is_Property
  
  (Node : Bare_Fun_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Fun_Decl  := Bare_Fun_Decl (Node);
     Ent : Internal_Entity_Fun_Decl :=
       Internal_Entity_Fun_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Boolean;

      

      Fld : Internal_Entity_Full_Decl;
Sym : Symbol_Type;
Fld_1 : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 264 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 263 '<FieldAccess for has_annotation at parser.py:2515>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2515



--# expr-start 261 '<FieldAccess for full_decl at parser.py:2515>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2515






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.full_decl
Fld := Liblktlang.Implementation.Decl_P_Full_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 261


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 262 '<Symbol property at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Property); 
--# expr-done 262
--# property-call-start FullDecl.has_annotation
Fld_1 := Liblktlang.Implementation.Full_Decl_P_Has_Annotation (Node => Fld.Node, Name => Sym);
--# end
--# expr-done 263
Let_Result := Fld_1; 
--# end
--# expr-done 264

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Fun_Decl_P_Is_Property;
--# end

   







--# property-start FunDecl.get_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2517
pragma Warnings (Off, "is not referenced");
 function Fun_Decl_P_Get_Type
  
  (Node : Bare_Fun_Decl
      ; No_Inference : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Fun_Decl  := Bare_Fun_Decl (Node);
     Ent : Internal_Entity_Fun_Decl :=
       Internal_Entity_Fun_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind no_inference No_Inference

   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_198 with Inline_Always;

      Fld : Internal_Entity_Type_Ref;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Internal_Entity_Fun_Arg_Decl_List;
Fld_3 : Bare_Fun_Arg_Decl_List;
Fld_4 : Internal_Entity_Info;
Item_14 : Internal_Entity_Fun_Arg_Decl;
Bare_Item_14 : Bare_Fun_Arg_Decl;
Fld_5 : Internal_Entity_Type_Ref;
Fld_6 : Internal_Entity_Type_Decl;
Fld_7 : Bare_Decl;
Cast_Result : Bare_Type_Decl;
As_Entity : Internal_Entity_Fun_Arg_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Fun_Arg_Decl;
Map_Result : Bare_Type_Decl_Array_Access;
Fld_8 : Internal_Entity_Type_Ref;
Fld_9 : Internal_Entity_Type_Decl;
Fld_10 : Bare_Decl;
Cast_Result_1 : Bare_Type_Decl;
Fld_11 : Bare_Function_Type;
As_Entity_1 : Internal_Entity_Function_Type;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Function_Type;
Cast_Result_2 : Internal_Entity_Type_Decl;
Fld_12 : Boolean;
If_Result_2 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_198 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_198;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 281 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 280 '<If at parser.py:2520>' If_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2520



--# expr-start 265 '<FieldAccess for is_property at parser.py:2521>' Fld_12 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2521






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start FunDecl.is_property
Fld_12 := Liblktlang.Implementation.Fun_Decl_P_Is_Property (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 265
if Fld_12 then
   --# expr-start 267 '<FieldAccess for designated_type at parser.py:2523>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2523



--# expr-start 266 '<FieldAccess for return_type at parser.py:2523>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2523






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref (Node => Ent.Node.Fun_Decl_F_Return_Type, Info => Ent.Info);
--# expr-done 266


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 267
   If_Result_2 := Fld_1;
else
   




--# expr-start 279 '<as_bare_entity at parser.py:2539>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2539



--# expr-start 278 '<FieldAccess for function_type at parser.py:2525>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2525






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 273 '<Map at parser.py:2534>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2534








   

   --# expr-start 268 '<FieldAccess for args at parser.py:2534>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2534






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Create_Internal_Entity_Fun_Arg_Decl_List (Node => Ent.Node.Fun_Decl_F_Args, Info => Ent.Info);
--# expr-done 268






   




Fld_3 := Fld_2.Node;

   
      declare
         Map_Result_Vec : Bare_Type_Decl_Vectors.Vector;
      begin
   

   

      if Fld_3 = null then
         
      Map_Result := Create_Bare_Type_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Fun_Arg_Decl_List := Fld_3;
      begin
         for Untyped_Item_14 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_14 := Untyped_Item_14; 
                  



Is_Null := Bare_Item_14 = null; 
if Is_Null then
   
   If_Result := No_Entity_Fun_Arg_Decl;
else
   





   




Fld_4 := Fld_2.Info;

As_Entity := (Info => Fld_4, Node => Bare_Item_14); 
   If_Result := As_Entity;
end if;



                  Item_14 := If_Result; 

            
   --# scope-start


               --# bind a Item_14

            
         
         
      --# expr-start 272 '<Cast to TypeDecl at parser.py:2536>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2536





--# expr-start 271 '<FieldAccess for assert_bare at parser.py:2536>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2536



--# expr-start 270 '<FieldAccess for designated_type at parser.py:2536>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2536



--# expr-start 269 '<FieldAccess for decl_type at parser.py:2536>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2536






   

      if Item_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Create_Internal_Entity_Type_Ref (Node => Item_14.Node.Fun_Arg_Decl_F_Decl_Type, Info => Item_14.Info);
--# expr-done 269


   

      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_6 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 270


   

      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.assert_bare
Fld_7 := Liblktlang.Implementation.Decl_P_Assert_Bare (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 271



   if Fld_7 = null
      or else Fld_7.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Fld_7;

   else
         Cast_Result := No_Bare_Lkt_Node;
   end if;


--# expr-done 272
      

         declare
            Item_To_Append : constant Bare_Type_Decl := Cast_Result;
         begin
            Bare_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Bare_Type_Decl_Array
           (Items_Count => Natural (Bare_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 273
--# expr-start 277 '<Cast to TypeDecl at parser.py:2538>' Cast_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2538





--# expr-start 276 '<FieldAccess for assert_bare at parser.py:2538>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2538



--# expr-start 275 '<FieldAccess for designated_type at parser.py:2538>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2538



--# expr-start 274 '<FieldAccess for return_type at parser.py:2538>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2538






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_8 := Create_Internal_Entity_Type_Ref (Node => Ent.Node.Fun_Decl_F_Return_Type, Info => Ent.Info);
--# expr-done 274


   

      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_9 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 275


   

      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.assert_bare
Fld_10 := Liblktlang.Implementation.Decl_P_Assert_Bare (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 276



   if Fld_10 = null
      or else Fld_10.Kind in Lkt_Type_Decl
   then
      
      Cast_Result_1 := Fld_10;

   else
         Cast_Result_1 := No_Bare_Lkt_Node;
   end if;


--# expr-done 277
--# property-call-start LktNode.function_type
Fld_11 := Liblktlang.Implementation.Lkt_Node_P_Function_Type (Node => Self, Args_Types => Map_Result, Return_Type => Cast_Result_1);
--# end
--# expr-done 278
Is_Null_1 := Fld_11 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Function_Type;
else
   

As_Entity_1 := (Info => No_Entity_Info, Node => Fld_11); 
   If_Result_1 := As_Entity_1;
end if;



--# expr-done 279



   
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);


   If_Result_2 := Cast_Result_2;
end if;



--# expr-done 280
Let_Result := If_Result_2; 
--# end
--# expr-done 281

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_198;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_198;




            raise;
      end;



   return Property_Result;
end Fun_Decl_P_Get_Type;
--# end

   







--# property-start FunDecl.check_legality /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2542
pragma Warnings (Off, "is not referenced");
 function Fun_Decl_P_Check_Legality
  
  (Node : Bare_Fun_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Fun_Decl  := Bare_Fun_Decl (Node);
     Ent : Internal_Entity_Fun_Decl :=
       Internal_Entity_Fun_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_199 with Inline_Always;
            procedure Finalizer_Scope_346 with Inline_Always;
            procedure Finalizer_Scope_347 with Inline_Always;
            procedure Finalizer_Scope_348 with Inline_Always;
            procedure Finalizer_Scope_349 with Inline_Always;

      Block_Var : Boolean;
Block_Var_1 : Boolean;
Block_Var_2 : Boolean;
Block_Var_3 : Boolean;
Var_Expr : Boolean;
Var_Expr_1 : Boolean;
Var_Expr_2 : Boolean;
Var_Expr_7 : Internal_Semantic_Result := No_Semantic_Result;
Fld : Internal_Entity_Full_Decl;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Full_Decl;
Sym_1 : Symbol_Type;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Full_Decl;
Sym_2 : Symbol_Type;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Str : String_Type;
Fld_7 : Internal_Semantic_Result := No_Semantic_Result;
Scope_Result : Internal_Semantic_Result := No_Semantic_Result;
Result_Var : Internal_Semantic_Result := No_Semantic_Result;
Str_1 : String_Type;
Fld_8 : Internal_Semantic_Result := No_Semantic_Result;
Scope_Result_1 : Internal_Semantic_Result := No_Semantic_Result;
Result_Var_1 : Internal_Semantic_Result := No_Semantic_Result;
Str_2 : String_Type;
Fld_9 : Internal_Semantic_Result := No_Semantic_Result;
Scope_Result_2 : Internal_Semantic_Result := No_Semantic_Result;
Result_Var_2 : Internal_Semantic_Result := No_Semantic_Result;
Not_Val_1 : Boolean;
Str_3 : String_Type;
Fld_10 : Internal_Semantic_Result := No_Semantic_Result;
If_Result : Internal_Semantic_Result := No_Semantic_Result;
If_Result_1 : Internal_Semantic_Result := No_Semantic_Result;
If_Result_2 : Internal_Semantic_Result := No_Semantic_Result;
If_Result_3 : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
Scope_Result_3 : Internal_Semantic_Result_Array_Access;
Result_Var_3 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_199 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Str_3);
                     Dec_Ref (Fld_10);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_199;
            procedure Finalizer_Scope_346 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_346;
            procedure Finalizer_Scope_347 is
            begin
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_347;
            procedure Finalizer_Scope_348 is
            begin
                     Dec_Ref (Str_2);
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_348;
            procedure Finalizer_Scope_349 is
            begin
                     Dec_Ref (Var_Expr_7);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_349;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 304 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 284 '<FieldAccess for has_annotation at parser.py:2546>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2546



--# expr-start 282 '<FieldAccess for full_decl at parser.py:2546>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2546






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.full_decl
Fld := Liblktlang.Implementation.Decl_P_Full_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 282


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 283 '<Symbol abstract at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abstract); 
--# expr-done 283
--# property-call-start FullDecl.has_annotation
Fld_1 := Liblktlang.Implementation.Full_Decl_P_Has_Annotation (Node => Fld.Node, Name => Sym);
--# end
--# expr-done 284
Block_Var := Fld_1; 
--# bind has_abstract Block_Var
--# expr-start 287 '<FieldAccess for has_annotation at parser.py:2547>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2547



--# expr-start 285 '<FieldAccess for full_decl at parser.py:2547>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2547






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.full_decl
Fld_2 := Liblktlang.Implementation.Decl_P_Full_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 285


   

      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 286 '<Symbol builtin at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Builtin); 
--# expr-done 286
--# property-call-start FullDecl.has_annotation
Fld_3 := Liblktlang.Implementation.Full_Decl_P_Has_Annotation (Node => Fld_2.Node, Name => Sym_1);
--# end
--# expr-done 287
Block_Var_1 := Fld_3; 
--# bind has_builtin Block_Var_1
--# expr-start 290 '<FieldAccess for has_annotation at parser.py:2548>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2548



--# expr-start 288 '<FieldAccess for full_decl at parser.py:2548>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2548






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.full_decl
Fld_4 := Liblktlang.Implementation.Decl_P_Full_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 288


   

      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 289 '<Symbol external at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_External); 
--# expr-done 289
--# property-call-start FullDecl.has_annotation
Fld_5 := Liblktlang.Implementation.Full_Decl_P_Has_Annotation (Node => Fld_4.Node, Name => Sym_2);
--# end
--# expr-done 290
Block_Var_2 := Fld_5; 
--# bind has_external Block_Var_2
--# expr-start 293 '<Not at parser.py:2549>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2549
--# expr-start 292 '<IsNull at parser.py:2549>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2549
--# expr-start 291 '<FieldAccess for body at parser.py:2549>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2549






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Fun_Decl_F_Body, Info => Ent.Info);
--# expr-done 291
Is_Null := Fld_6.Node = null; 
--# expr-done 292
Not_Val := not (Is_Null); 
--# expr-done 293
Block_Var_3 := Not_Val; 
--# bind has_body Block_Var_3



--# expr-start 303 '<Cond at parser.py:2550>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2550




if Block_Var then
   



Var_Expr := Block_Var_3;





if Var_Expr /= False then
   



   --# scope-start


   --# bind _ Var_Expr

--# expr-start 295 '<FieldAccess for error at parser.py:2554>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2554






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 294 '<String at parser.py:2554>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2554
Str := Create_String ("abstract functions cannot have a body"); 
--# expr-done 294
--# property-call-start LktNode.error
Fld_7 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Str);
--# end
--# expr-done 295
Scope_Result := Fld_7;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_346;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Semantic_Result;
end if;

      Inc_Ref (Result_Var);


   If_Result_3 := Result_Var;
else
   



if Block_Var_1 then
   



Var_Expr_1 := Block_Var_3;





if Var_Expr_1 /= False then
   



   --# scope-start


   --# bind _ Var_Expr_1

--# expr-start 297 '<FieldAccess for error at parser.py:2560>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2560






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 296 '<String at parser.py:2560>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2560
Str_1 := Create_String ("builtin functions cannot have a body"); 
--# expr-done 296
--# property-call-start LktNode.error
Fld_8 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Str_1);
--# end
--# expr-done 297
Scope_Result_1 := Fld_8;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_347;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Semantic_Result;
end if;

      Inc_Ref (Result_Var_1);


   If_Result_2 := Result_Var_1;
else
   



if Block_Var_2 then
   



Var_Expr_2 := Block_Var_3;





if Var_Expr_2 /= False then
   



   --# scope-start


   --# bind _ Var_Expr_2

--# expr-start 299 '<FieldAccess for error at parser.py:2566>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2566






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 298 '<String at parser.py:2566>' Str_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2566
Str_2 := Create_String ("external functions cannot have a body"); 
--# expr-done 298
--# property-call-start LktNode.error
Fld_9 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Str_2);
--# end
--# expr-done 299
Scope_Result_2 := Fld_9;
   Inc_Ref (Scope_Result_2);


   --# end
      Finalizer_Scope_348;


   Result_Var_2 := Scope_Result_2;
else
   
   Result_Var_2 := No_Semantic_Result;
end if;

      Inc_Ref (Result_Var_2);


   If_Result_1 := Result_Var_2;
else
   


--# expr-start 300 '<Not at parser.py:2569>' Not_Val_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2569

Not_Val_1 := not (Block_Var_3); 
--# expr-done 300
if Not_Val_1 then
   --# expr-start 302 '<FieldAccess for error at parser.py:2570>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2570






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 301 '<String at parser.py:2570>' Str_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2570
Str_3 := Create_String ("regular functions must have a body"); 
--# expr-done 301
--# property-call-start LktNode.error
Fld_10 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Str_3);
--# end
--# expr-done 302
   If_Result := Fld_10;
else
   
   If_Result := No_Semantic_Result;
end if;

      Inc_Ref (If_Result);


   If_Result_1 := If_Result;
end if;

      Inc_Ref (If_Result_1);


   If_Result_2 := If_Result_1;
end if;

      Inc_Ref (If_Result_2);


   If_Result_3 := If_Result_2;
end if;

      Inc_Ref (If_Result_3);


--# expr-done 303
Var_Expr_7 := If_Result_3;

      Inc_Ref (Var_Expr_7);




if Var_Expr_7 /= No_Semantic_Result then
   



   --# scope-start


   


                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr_7;
                Inc_Ref (Singleton.Items (1));
            
Scope_Result_3 := Singleton;
   Inc_Ref (Scope_Result_3);


   --# end
      Finalizer_Scope_349;


   Result_Var_3 := Scope_Result_3;
else
   
   Result_Var_3 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_3);


Let_Result := Result_Var_3; Inc_Ref (Let_Result);
--# end
--# expr-done 304

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_199;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_199;
                     Finalizer_Scope_346;
                     Finalizer_Scope_347;
                     Finalizer_Scope_348;
                     Finalizer_Scope_349;




            raise;
      end;



   return Property_Result;
end Fun_Decl_P_Check_Legality;
--# end

   







--# property-start FunDecl._env_mappings_17 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2495
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_17
  
  (Node : Bare_Fun_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc
is
   Self : Bare_Fun_Decl  := Bare_Fun_Decl (Node);
     Ent : Internal_Entity_Fun_Decl :=
       Internal_Entity_Fun_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc;

      
            procedure Finalizer_Scope_297 with Inline_Always;

      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Fun_Decl;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;

            procedure Finalizer_Scope_297 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_297;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 307 '<New EnvAssoc at parser.py:2495>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2495
--# expr-start 305 '<New DesignatedEnv at parser.py:2495>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2495



Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 305
--# expr-start 306 '<FieldAccess for name at parser.py:2495>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2495






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Ent.Node);
--# end
--# expr-done 306







Cast_Expr := Self; 



   
      Cast_Result := Cast_Expr;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 307

         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_297;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_297;




            raise;
      end;



   return Property_Result;
end Env_Mappings_17;
--# end

   







--# property-start FunDecl._env_trans_parent_18 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2496
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_18
  
  (Node : Bare_Fun_Decl
  )

   return Boolean
is
   Self : Bare_Fun_Decl  := Bare_Fun_Decl (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Trans_Parent_18;
--# end

   







--# property-start FunDecl._env_mappings_19 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2498
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_19
  
  (Node : Bare_Fun_Decl
  )

   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Fun_Decl  := Bare_Fun_Decl (Node);
      --# bind self Self

   


   Property_Result : Internal_Env_Assoc_Array_Access;

      
            procedure Finalizer_Scope_299 with Inline_Always;
            procedure Finalizer_Scope_350 with Inline_Always;

      Var_Expr : Bare_Type_Decl;
Fld : Bare_Type_Decl;
Sym : Symbol_Type;
Fld_1 : Bare_Self_Decl;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Fld_2 : Bare_Node_Decl;
New_Struct_2 : Internal_Designated_Env := No_Designated_Env;
Cast_Result_1 : Bare_Lkt_Node;
New_Struct_3 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Scope_Result : Internal_Env_Assoc_Array_Access;
Result_Var : Internal_Env_Assoc_Array_Access;

            procedure Finalizer_Scope_299 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_299;
            procedure Finalizer_Scope_350 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (New_Struct_3);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_350;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         


--# expr-start 308 '<FieldAccess for owning_type at parser.py:2498>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2498






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start FunDecl.owning_type
Fld := Liblktlang.Implementation.Fun_Decl_P_Owning_Type (Node => Self);
--# end
--# expr-done 308
Var_Expr := Fld;





if Var_Expr /= No_Bare_Lkt_Node then
   



   --# scope-start


   --# bind ot Var_Expr

--# expr-start 317 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 312 '<New EnvAssoc at parser.py:2499>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2499
--# expr-start 309 '<New DesignatedEnv at parser.py:2499>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2499



Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 309
--# expr-start 310 '<Symbol self at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Self); 
--# expr-done 310






--# expr-start 311 '<FieldAccess for self_decl at parser.py:2499>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2499






   

      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.self_decl
Fld_1 := Liblktlang.Implementation.Type_Decl_P_Self_Decl (Node => Var_Expr);
--# end
--# expr-done 311



   
      Cast_Result := Fld_1;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Sym, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 312
--# expr-start 316 '<New EnvAssoc at parser.py:2500>' New_Struct_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2500
--# expr-start 313 '<New DesignatedEnv at parser.py:2500>' New_Struct_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2500



Inc_Ref (Empty_Env);
New_Struct_2 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 313
--# expr-start 314 '<Symbol node at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Node_31); 
--# expr-done 314






--# expr-start 315 '<FieldAccess for node_decl at parser.py:2500>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2500






   

      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.node_decl
Fld_2 := Liblktlang.Implementation.Type_Decl_P_Node_Decl (Node => Var_Expr);
--# end
--# expr-done 315



   
      Cast_Result_1 := Fld_2;


Inc_Ref (New_Struct_2);
New_Struct_3 := (Dest_Env => New_Struct_2, Key => Sym_1, Metadata => No_Metadata, Value => Cast_Result_1); 
--# expr-done 316
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => New_Struct_1, 2 => New_Struct_3)); 
--# expr-done 317
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_350;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Internal_Env_Assoc_Array_Type;
end if;

      Inc_Ref (Result_Var);



         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_299;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_299;
                     Finalizer_Scope_350;




            raise;
      end;



   return Property_Result;
end Env_Mappings_19;
--# end



   


      

   --
   --  Primitives for Bare_Env_Spec_Decl
   --

   



      
      procedure Initialize_Fields_For_Env_Spec_Decl
        (Self : Bare_Env_Spec_Decl
         ; Env_Spec_Decl_F_Syn_Name : Bare_Def_Id
         ; Env_Spec_Decl_F_Actions : Bare_Call_Expr_List
        ) is
      begin

            Self.Env_Spec_Decl_F_Syn_Name := Env_Spec_Decl_F_Syn_Name;
            Self.Env_Spec_Decl_F_Actions := Env_Spec_Decl_F_Actions;
         

      end Initialize_Fields_For_Env_Spec_Decl;

      
   function Env_Spec_Decl_F_Actions
     (Node : Bare_Env_Spec_Decl) return Bare_Call_Expr_List
   is
      

   begin
         
         return Node.Env_Spec_Decl_F_Actions;
      
   end;


   







--# property-start EnvSpecDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2360
pragma Warnings (Off, "is not referenced");
 function Env_Spec_Decl_P_Decl_Type_Name
  
  (Node : Bare_Env_Spec_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Env_Spec_Decl  := Bare_Env_Spec_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_180 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_180 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_180;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 318 '<String at parser.py:2360>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2360
Str := Create_String ("env spec declaration"); 
--# expr-done 318

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_180;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_180;




            raise;
      end;



   return Property_Result;
end Env_Spec_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Generic_Decl
   --

   



   

   

   

   

   

   

   


      procedure Generic_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mapping : Internal_Env_Assoc :=
            Liblktlang.Implementation.Env_Mappings_10 (Node => 
Self);
      begin

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:2070");
         Dec_Ref (Mapping.Dest_Env);

      end;
   
   
            


   
      if Add_To_Env_Only then
         return;
      end if;

      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Liblktlang.Implementation.Env_Trans_Parent_11 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
   
   
      end;



      
      procedure Initialize_Fields_For_Generic_Decl
        (Self : Bare_Generic_Decl
         ; Generic_Decl_F_Generic_Formal_Decls : Bare_Generic_Formal_Decl_List
         ; Generic_Decl_F_Decl : Bare_Decl
        ) is
      begin

            Self.Generic_Decl_F_Generic_Formal_Decls := Generic_Decl_F_Generic_Formal_Decls;
            Self.Generic_Decl_F_Decl := Generic_Decl_F_Decl;
         

      end Initialize_Fields_For_Generic_Decl;

      
   function Generic_Decl_F_Generic_Formal_Decls
     (Node : Bare_Generic_Decl) return Bare_Generic_Formal_Decl_List
   is
      

   begin
         
         return Node.Generic_Decl_F_Generic_Formal_Decls;
      
   end;

      
   function Generic_Decl_F_Decl
     (Node : Bare_Generic_Decl) return Bare_Decl
   is
      

   begin
         
         return Node.Generic_Decl_F_Decl;
      
   end;


   







--# property-start GenericDecl.name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2066
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Name
  
  (Node : Bare_Generic_Decl
  )

   return Symbol_Type
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
      --# bind self Self

   


   Property_Result : Symbol_Type;

      

      Fld : Bare_Decl;
Fld_1 : Symbol_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 320 '<FieldAccess for name at parser.py:2066>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2066



--# expr-start 319 '<FieldAccess for decl at parser.py:2066>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2066






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Generic_Decl_F_Decl;
--# expr-done 319


   

      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld_1 := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Fld);
--# end
--# expr-done 320

         Property_Result := Fld_1;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Generic_Decl_P_Name;
--# end

   







--# property-start GenericDecl.generic_formals /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2074
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Generic_Formals
  
  (Node : Bare_Generic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Generic_Formal_Type_Decl_Array_Access
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Generic_Formal_Type_Decl_Array_Access;

      
            procedure Finalizer_Scope_154 with Inline_Always;

      Fld : Internal_Entity_Generic_Formal_Decl_List;
Fld_1 : Bare_Generic_Formal_Decl_List;
Fld_2 : Internal_Entity_Info;
Item_15 : Internal_Entity_Full_Decl;
Bare_Item_15 : Bare_Full_Decl;
Fld_3 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Generic_Formal_Type_Decl;
As_Entity : Internal_Entity_Full_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Full_Decl;
Map_Result : Internal_Entity_Generic_Formal_Type_Decl_Array_Access;

            procedure Finalizer_Scope_154 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_154;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 324 '<Map at parser.py:2074>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2074








   

   --# expr-start 321 '<FieldAccess for generic_formal_decls at parser.py:2074>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2074






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Generic_Formal_Decl_List (Node => Ent.Node.Generic_Decl_F_Generic_Formal_Decls, Info => Ent.Info);
--# expr-done 321






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Internal_Entity_Generic_Formal_Type_Decl_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Internal_Entity_Generic_Formal_Type_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Generic_Formal_Decl_List := Fld_1;
      begin
         for Untyped_Item_15 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_15 := Untyped_Item_15; 
                  



Is_Null := Bare_Item_15 = null; 
if Is_Null then
   
   If_Result := No_Entity_Full_Decl;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_15); 
   If_Result := As_Entity;
end if;



                  Item_15 := If_Result; 

            
   --# scope-start


               --# bind gfd Item_15

            
         
         
      --# expr-start 323 '<Cast to GenericFormalTypeDecl at parser.py:2075>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2075





--# expr-start 322 '<FieldAccess for decl at parser.py:2075>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2075






   

      if Item_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Decl (Node => Item_15.Node.Full_Decl_F_Decl, Info => Item_15.Info);
--# expr-done 322



   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Lkt_Generic_Formal_Type_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);

   else
         Cast_Result := No_Entity_Generic_Formal_Type_Decl;
   end if;


--# expr-done 323
      

         declare
            Item_To_Append : constant Internal_Entity_Generic_Formal_Type_Decl := Cast_Result;
         begin
            Internal_Entity_Generic_Formal_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Entity_Generic_Formal_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Generic_Formal_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Generic_Formal_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Generic_Formal_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Generic_Formal_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 324

         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_154;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_154;




            raise;
      end;



   return Property_Result;
end Generic_Decl_P_Generic_Formals;
--# end

   







--# property-start GenericDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2078
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Decl_Type_Name
  
  (Node : Bare_Generic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_155 with Inline_Always;

      Str : String_Type;
Fld : Internal_Entity_Decl;
Fld_1 : String_Type;
Concat_Result : String_Type;

            procedure Finalizer_Scope_155 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_155;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 325 '<String at parser.py:2079>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2079
Str := Create_String ("generic "); 
--# expr-done 325
--# expr-start 327 '<FieldAccess for decl_type_name at parser.py:2079>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2079



--# expr-start 326 '<FieldAccess for decl at parser.py:2079>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2079






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Decl (Node => Ent.Node.Generic_Decl_F_Decl, Info => Ent.Info);
--# expr-done 326


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.decl_type_name'
Fld_1 := Liblktlang.Implementation.Dispatcher_Decl_P_Decl_Type_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 327
Concat_Result := Concat_String (Str, Fld_1); 

         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_155;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_155;




            raise;
      end;



   return Property_Result;
end Generic_Decl_P_Decl_Type_Name;
--# end

   







--# property-start GenericDecl.instantiate /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2082
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Instantiate
  
  (Node : Bare_Generic_Decl
      ; Actuals : Bare_Type_Decl_Array_Access
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Decl
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind actuals Actuals

   Property_Result : Internal_Entity_Decl;

      
            procedure Finalizer_Scope_156 with Inline_Always;

      Block_Var : Bare_Fun_Decl;
Fld : Bare_Decl;
Cast_Result : Bare_Fun_Decl;
Fld_1 : Internal_Entity_Decl;
Is_A : Boolean;
Fld_2 : Bare_Instantiated_Generic_Type;
Cast_Result_1 : Bare_Decl;
As_Entity : Internal_Entity_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Decl;
Fld_3 : Internal_Entity_Decl;
Is_A_1 : Boolean;
Fld_4 : Internal_Entity_Info;
Fld_5 : Env_Rebindings;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : Lexical_Env := Empty_Env;
Rebinding : Env_Rebindings;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Fun_Decl;
Cast_Result_2 : Internal_Entity_Decl;
If_Result_1 : Internal_Entity_Decl;
If_Result_2 : Internal_Entity_Decl;
Let_Result : Internal_Entity_Decl;

            procedure Finalizer_Scope_156 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_156;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 346 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 329 '<Cast to FunDecl at parser.py:2087>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2087





--# expr-start 328 '<FieldAccess for decl at parser.py:2087>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2087






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Generic_Decl_F_Decl;
--# expr-done 328



   if Fld = null
      or else Fld.Kind in Lkt_Fun_Decl_Range
   then
      
      Cast_Result := Fld;

   else
         Cast_Result := No_Bare_Lkt_Node;
   end if;


--# expr-done 329
Block_Var := Cast_Result; 
--# bind fd Block_Var
--# expr-start 345 '<Cond at parser.py:2088>' If_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2088



--# expr-start 331 '<IsA TypeDecl at parser.py:2089>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2089
--# expr-start 330 '<FieldAccess for decl at parser.py:2089>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2089






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Decl (Node => Ent.Node.Generic_Decl_F_Decl, Info => Ent.Info);
--# expr-done 330
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Lkt_Type_Decl; 
--# expr-done 331
if Is_A then
   --# expr-start 334 '<as_bare_entity at parser.py:2090>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2090



--# expr-start 333 '<Cast to Decl at parser.py:2090>' Cast_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2090





--# expr-start 332 '<FieldAccess for get_instantiated_type at parser.py:2090>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2090






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start GenericDecl.get_instantiated_type
Fld_2 := Liblktlang.Implementation.Generic_Decl_P_Get_Instantiated_Type (Node => Ent.Node, Actuals => Actuals);
--# end
--# expr-done 332



   
      Cast_Result_1 := Fld_2;


--# expr-done 333
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   
   If_Result := No_Entity_Decl;
else
   

As_Entity := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;



--# expr-done 334
   If_Result_2 := If_Result;
else
   


--# expr-start 336 '<IsA FunDecl at parser.py:2092>' Is_A_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2092
--# expr-start 335 '<FieldAccess for decl at parser.py:2092>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2092






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Decl (Node => Ent.Node.Generic_Decl_F_Decl, Info => Ent.Info);
--# expr-done 335
Is_A_1 := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Lkt_Fun_Decl_Range; 
--# expr-done 336
if Is_A_1 then
   --# expr-start 344 '<Cast to Decl.entity at ???>' Cast_Result_2 None





--# expr-start 343 '<New FunDecl.entity at ???>' New_Struct_1 None
--# expr-start 342 '<New EntityInfo at parser.py:2095>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2095


--# expr-start 341 '<append_rebinding at parser.py:2098>' Rebinding /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2098
--# expr-start 338 '<FieldAccess for rebindings at parser.py:2098>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2098



--# expr-start 337 '<FieldAccess for info at parser.py:2098>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2098






   




Fld_4 := Ent.Info;
--# expr-done 337


   




Fld_5 := Fld_4.Rebindings;
--# expr-done 338
--# expr-start 339 '<FieldAccess for children_env at parser.py:2099>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2099






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_6 := Liblktlang.Implementation.Children_Env (Node => Self);
--# expr-done 339
--# expr-start 340 '<FieldAccess for get_inst_env at parser.py:2100>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2100






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start GenericDecl.get_inst_env
Fld_7 := Liblktlang.Implementation.Generic_Decl_P_Get_Inst_Env (Node => Ent.Node, Actuals => Actuals, E_Info => Ent.Info);
--# end
--# expr-done 340
Rebinding := AST_Envs.Append_Rebinding (Fld_5, Fld_6, Fld_7); 
--# expr-done 341
New_Struct := (From_Rebound => False, Md => No_Metadata, Rebindings => Rebinding); 
--# expr-done 342

New_Struct_1 := (Info => New_Struct, Node => Block_Var); 
--# expr-done 343



   
      Cast_Result_2 := Create_Internal_Entity_Decl
        (Node => New_Struct_1.Node,
         Info => New_Struct_1.Info);


--# expr-done 344
   If_Result_1 := Cast_Result_2;
else
   
   If_Result_1 := raise Property_Error with "should not happen";
end if;



   If_Result_2 := If_Result_1;
end if;



--# expr-done 345
Let_Result := If_Result_2; 
--# end
--# expr-done 346

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_156;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_156;




            raise;
      end;



   return Property_Result;
end Generic_Decl_P_Instantiate;
--# end

   







--# property-start GenericDecl.get_instantiated_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2108
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Get_Instantiated_Type
  
  (Node : Bare_Generic_Decl
      ; Actuals : Bare_Type_Decl_Array_Access
  )

   return Bare_Instantiated_Generic_Type
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
      --# bind self Self

   

   --# bind actuals Actuals

   Property_Result : Bare_Instantiated_Generic_Type;

      

      Fld : Bare_Decl;
Cast_Result : Bare_Type_Decl;
New_Node : Bare_Instantiated_Generic_Type;
Let_Result : Bare_Instantiated_Generic_Type;


      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Generic_Decl_Generic_Decl_P_Get_Instantiated_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Generic_Decl,
                                As_Bare_Generic_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Type_Decl_Array_Access,
                                      As_Bare_Type_Decl_Array_Access => Actuals);
                  Inc_Ref (Mmz_K.Items (2).As_Bare_Type_Decl_Array_Access);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Bare_Instantiated_Generic_Type;

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 350 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 349 '<New InstantiatedGenericType at parser.py:2110>' New_Node /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2110

--# expr-start 348 '<Cast to TypeDecl at parser.py:2111>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2111





--# expr-start 347 '<FieldAccess for decl at parser.py:2111>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2111






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Generic_Decl_F_Decl;
--# expr-done 347



   if Fld = null
      or else Fld.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Fld;

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 348
Inc_Ref (Actuals);


if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;


New_Node := new Root_Node_Record
  (Lkt_Instantiated_Generic_Type);
Initialize
  (Self => New_Node,
   Kind => Lkt_Instantiated_Generic_Type,
   Unit => Self.Unit,

   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,

   Parent => Self,

   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);


      New_Node.Instantiated_Generic_Type_Actuals :=
         Actuals;
      New_Node.Instantiated_Generic_Type_Inner_Type_Decl :=
         Cast_Result;

--# expr-done 349
Let_Result := New_Node; 
--# end
--# expr-done 350

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Bare_Instantiated_Generic_Type,
                        As_Bare_Instantiated_Generic_Type => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);

         end if;
      end if;



   return Property_Result;
end Generic_Decl_P_Get_Instantiated_Type;
--# end

   







--# property-start GenericDecl.get_assocs /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2115
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Get_Assocs
  
  (Node : Bare_Generic_Decl
      ; Actuals : Bare_Type_Decl_Array_Access
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Generic_Param_Assoc_Array_Access
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind actuals Actuals

   Property_Result : Internal_Generic_Param_Assoc_Array_Access;

      
            procedure Finalizer_Scope_158 with Inline_Always;

      Fld : Internal_Entity_Generic_Formal_Type_Decl_Array_Access;
Index_17 : Integer;
Fld_1 : Bare_Generic_Formal_Type_Decl;
Get_Result : Bare_Type_Decl;
New_Struct : Internal_Generic_Param_Assoc;
Map_Result : Internal_Generic_Param_Assoc_Array_Access;
Fld_2 : Internal_Entity_Generic_Formal_Type_Decl_Array_Access;
Len : Integer;
Len_1 : Integer;
Is_Equal : Boolean;
If_Result : Internal_Generic_Param_Assoc_Array_Access;
Let_Result : Internal_Generic_Param_Assoc_Array_Access;

            procedure Finalizer_Scope_158 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_158;

      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Generic_Decl_Generic_Decl_P_Get_Assocs,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Generic_Decl,
                                As_Bare_Generic_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Type_Decl_Array_Access,
                                      As_Bare_Type_Decl_Array_Access => Actuals);
                  Inc_Ref (Mmz_K.Items (2).As_Bare_Type_Decl_Array_Access);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);




      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Internal_Generic_Param_Assoc_Array_Access;
                     Inc_Ref (Property_Result);

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 361 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 360 '<If at parser.py:2121>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2121



--# expr-start 354 '<Eq at parser.py:2122>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2122
--# expr-start 352 '<length at parser.py:2122>' Len /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2122
--# expr-start 351 '<FieldAccess for generic_formals at parser.py:2122>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2122






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start GenericDecl.generic_formals
Fld_2 := Liblktlang.Implementation.Generic_Decl_P_Generic_Formals (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 351
Len := Length (Fld_2); 
--# expr-done 352
--# expr-start 353 '<length at parser.py:2122>' Len_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2122

Len_1 := Length (Actuals); 
--# expr-done 353
Is_Equal := Len = Len_1; 
--# expr-done 354
if Is_Equal then
   --# expr-start 359 '<Map at parser.py:2123>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2123








   

   --# expr-start 355 '<FieldAccess for generic_formals at parser.py:2123>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2123






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start GenericDecl.generic_formals
Fld := Liblktlang.Implementation.Generic_Decl_P_Generic_Formals (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 355

   
      declare
         Map_Result_Vec : Internal_Generic_Param_Assoc_Vectors.Vector;
      begin
   

   

      
         Index_17 := 0;

      declare
         
         Collection : constant Internal_Entity_Generic_Formal_Type_Decl_Array_Access := Fld;
      begin
         for Item_16 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind f Item_16
               --# bind idx Index_17

            
         
         
      --# expr-start 358 '<New GenericParamAssoc at parser.py:2124>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2124
--# expr-start 356 '<at at parser.py:2125>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2125




Get_Result := Get (Self, Actuals, Index_17, True); 
--# expr-done 356
--# expr-start 357 '<FieldAccess for node at parser.py:2125>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2125






   




Fld_1 := Item_16.Node;
--# expr-done 357
New_Struct := (Actual => Get_Result, Formal => Fld_1); 
--# expr-done 358
      

         declare
            Item_To_Append : constant Internal_Generic_Param_Assoc := New_Struct;
         begin
            Internal_Generic_Param_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



               Index_17 := Index_17 + 1;
         end loop;
      end;
   

   

         Map_Result := Create_Internal_Generic_Param_Assoc_Array
           (Items_Count => Natural (Internal_Generic_Param_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Generic_Param_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Generic_Param_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Generic_Param_Assoc_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 359
   If_Result := Map_Result;
else
   
   If_Result := raise Property_Error with "Actuals and formals don't match";
end if;

      Inc_Ref (If_Result);


--# expr-done 360
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 361

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_158;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_158;


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Internal_Generic_Param_Assoc_Array_Access,
                        As_Internal_Generic_Param_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;

         end if;
      end if;



   return Property_Result;
end Generic_Decl_P_Get_Assocs;
--# end

   







--# property-start GenericDecl.get_inst_env /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2132
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Get_Inst_Env
  
  (Node : Bare_Generic_Decl
      ; Actuals : Bare_Type_Decl_Array_Access
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind actuals Actuals

   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_159 with Inline_Always;

      Fld : Internal_Generic_Param_Assoc_Array_Access;
Fld_1 : Bare_Generic_Formal_Type_Decl;
Fld_2 : Symbol_Type;
Fld_3 : Bare_Type_Decl;
Cast_Result : Bare_Lkt_Node;
New_Struct : Internal_Env_Kv;
Map_Result : Internal_Env_Kv_Array_Access;
Fld_4 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_159 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_159;

      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Generic_Decl_Generic_Decl_P_Get_Inst_Env,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Generic_Decl,
                                As_Bare_Generic_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Type_Decl_Array_Access,
                                      As_Bare_Type_Decl_Array_Access => Actuals);
                  Inc_Ref (Mmz_K.Items (2).As_Bare_Type_Decl_Array_Access);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 369 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 368 '<FieldAccess for env_from_vals_internal at parser.py:2139>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2139






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 367 '<Map at parser.py:2140>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2140








   

   --# expr-start 362 '<FieldAccess for get_assocs at parser.py:2140>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2140






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start GenericDecl.get_assocs
Fld := Liblktlang.Implementation.Generic_Decl_P_Get_Assocs (Node => Ent.Node, Actuals => Actuals, E_Info => Ent.Info);
--# end
--# expr-done 362

   
      declare
         Map_Result_Vec : Internal_Env_Kv_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Generic_Param_Assoc_Array_Access := Fld;
      begin
         for Item_18 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind assoc Item_18

            
         
         
      --# expr-start 366 '<New EnvKv at parser.py:2141>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2141
--# expr-start 364 '<FieldAccess for name at parser.py:2142>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2142



--# expr-start 363 '<FieldAccess for formal at parser.py:2142>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2142






   




Fld_1 := Item_18.Formal;
--# expr-done 363


   

      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld_2 := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Fld_1);
--# end
--# expr-done 364





--# expr-start 365 '<FieldAccess for actual at parser.py:2142>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2142






   




Fld_3 := Item_18.Actual;
--# expr-done 365



   
      Cast_Result := Fld_3;


New_Struct := (Key => Fld_2, Value => Cast_Result); 
--# expr-done 366
      

         declare
            Item_To_Append : constant Internal_Env_Kv := New_Struct;
         begin
            Internal_Env_Kv_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Env_Kv_Array
           (Items_Count => Natural (Internal_Env_Kv_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Env_Kv_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Env_Kv_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Env_Kv_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 367
Fld_4 := Liblktlang.Implementation.Extensions.Lkt_Node_P_Env_From_Vals_Internal (Node => Ent.Node, Vals => Map_Result);
--# expr-done 368
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
--# expr-done 369

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_159;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_159;


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;

         end if;
      end if;



   return Property_Result;
end Generic_Decl_P_Get_Inst_Env;
--# end

   







--# property-start GenericDecl._env_mappings_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2070
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_10
  
  (Node : Bare_Generic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc;

      
            procedure Finalizer_Scope_290 with Inline_Always;

      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Generic_Decl;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;

            procedure Finalizer_Scope_290 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_290;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 372 '<New EnvAssoc at parser.py:2070>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2070
--# expr-start 370 '<New DesignatedEnv at parser.py:2070>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2070



Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 370
--# expr-start 371 '<FieldAccess for name at parser.py:2070>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2070






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Ent.Node);
--# end
--# expr-done 371







Cast_Expr := Self; 



   
      Cast_Result := Cast_Expr;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 372

         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_290;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_290;




            raise;
      end;



   return Property_Result;
end Env_Mappings_10;
--# end

   







--# property-start GenericDecl._env_trans_parent_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2071
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_11
  
  (Node : Bare_Generic_Decl
  )

   return Boolean
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Trans_Parent_11;
--# end



   


      

   --
   --  Primitives for Bare_Grammar_Decl
   --

   



   

   

   

   

   

   

   


      procedure Grammar_Decl_Pre_Env_Actions
        (Self            : Bare_Grammar_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mapping : Internal_Env_Assoc :=
            Liblktlang.Implementation.Env_Mappings_6 (Node => 
Self);
      begin

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:1222");
         Dec_Ref (Mapping.Dest_Env);

      end;
   
   
            


   
      if Add_To_Env_Only then
         return;
      end if;

      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Liblktlang.Implementation.Env_Trans_Parent_7 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
   
   
      end;



      
      procedure Initialize_Fields_For_Grammar_Decl
        (Self : Bare_Grammar_Decl
         ; Grammar_Decl_F_Syn_Name : Bare_Def_Id
         ; Grammar_Decl_F_Rules : Bare_Full_Decl_List
        ) is
      begin

            Self.Grammar_Decl_F_Syn_Name := Grammar_Decl_F_Syn_Name;
            Self.Grammar_Decl_F_Rules := Grammar_Decl_F_Rules;
         

      end Initialize_Fields_For_Grammar_Decl;

      
   function Grammar_Decl_F_Rules
     (Node : Bare_Grammar_Decl) return Bare_Full_Decl_List
   is
      

   begin
         
         return Node.Grammar_Decl_F_Rules;
      
   end;


   







--# property-start GrammarDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1211
pragma Warnings (Off, "is not referenced");
 function Grammar_Decl_P_Decl_Type_Name
  
  (Node : Bare_Grammar_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Grammar_Decl  := Bare_Grammar_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_101 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_101 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_101;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 373 '<String at parser.py:1211>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1211
Str := Create_String ("grammar declaration"); 
--# expr-done 373

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_101;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_101;




            raise;
      end;



   return Property_Result;
end Grammar_Decl_P_Decl_Type_Name;
--# end

   







--# property-start GrammarDecl.lexer /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1213
pragma Warnings (Off, "is not referenced");
 function Grammar_Decl_P_Lexer
  
  (Node : Bare_Grammar_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Decl
is
   Self : Bare_Grammar_Decl  := Bare_Grammar_Decl (Node);
     Ent : Internal_Entity_Grammar_Decl :=
       Internal_Entity_Grammar_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Decl;

      

      Fld : Internal_Entity_Full_Decl;
Sym : Symbol_Type;
Fld_1 : Bare_Decl_Annotation;
Fld_2 : Bare_Decl_Annotation_Params;
Fld_3 : Bare_Param_List;
Get_Result : Bare_Param;
Fld_4 : Bare_Expr;
As_Entity : Internal_Entity_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Expr;
Fld_5 : Internal_Entity_Decl;
Let_Result : Internal_Entity_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 383 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 382 '<FieldAccess for check_referenced_decl at parser.py:1219>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1219



--# expr-start 381 '<as_entity at parser.py:1219>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1219



--# expr-start 380 '<FieldAccess for value at parser.py:1219>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1219



--# expr-start 379 '<at at parser.py:1219>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1219

--# expr-start 378 '<FieldAccess for params at parser.py:1219>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1219



--# expr-start 377 '<FieldAccess for params at parser.py:1219>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1219



--# expr-start 376 '<FieldAccess for get_annotation at parser.py:1218>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1218



--# expr-start 374 '<FieldAccess for full_decl at parser.py:1218>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1218






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.full_decl
Fld := Liblktlang.Implementation.Decl_P_Full_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 374


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 375 '<Symbol with_lexer at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_With_Lexer); 
--# expr-done 375
--# property-call-start FullDecl.get_annotation
Fld_1 := Liblktlang.Implementation.Full_Decl_P_Get_Annotation (Node => Fld.Node, Name => Sym);
--# end
--# expr-done 376


   

      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Fld_1.Decl_Annotation_F_Params;
--# expr-done 377


   

      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Fld_2.Decl_Annotation_Params_F_Params;
--# expr-done 378


Get_Result := Get (Self, Fld_3, 0, True); 
--# expr-done 379


   

      if Get_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Get_Result.Param_F_Value;
--# expr-done 380
Is_Null := Fld_4 = null; 
if Is_Null then
   
   If_Result := No_Entity_Expr;
else
   

As_Entity := (Info => E_Info, Node => Fld_4); 
   If_Result := As_Entity;
end if;



--# expr-done 381


   

      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_referenced_decl
Fld_5 := Liblktlang.Implementation.Expr_P_Check_Referenced_Decl (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 382
Let_Result := Fld_5; 
--# end
--# expr-done 383

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Grammar_Decl_P_Lexer;
--# end

   







--# property-start GrammarDecl.check_correctness_pre /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1226
pragma Warnings (Off, "is not referenced");
 function Grammar_Decl_P_Check_Correctness_Pre
  
  (Node : Bare_Grammar_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Grammar_Decl  := Bare_Grammar_Decl (Node);
     Ent : Internal_Entity_Grammar_Decl :=
       Internal_Entity_Grammar_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_103 with Inline_Always;
            procedure Finalizer_Scope_358 with Inline_Always;
            procedure Finalizer_Scope_359 with Inline_Always;
            procedure Finalizer_Scope_360 with Inline_Always;
            procedure Finalizer_Scope_363 with Inline_Always;
            procedure Finalizer_Scope_364 with Inline_Always;
            procedure Finalizer_Scope_365 with Inline_Always;
            procedure Finalizer_Scope_366 with Inline_Always;
            procedure Finalizer_Scope_367 with Inline_Always;

      Block_Var : Bare_Decl_Annotation;
Block_Var_1 : Bare_Param_List;
Block_Var_2 : Bare_Param;
Block_Var_3 : Internal_Entity_Lexer_Decl;
Var_Expr_8 : Bare_Decl_Annotation;
Var_Expr_9 : Bare_Decl_Annotation_Params;
Var_Expr_10 : Bare_Param;
Var_Expr_11 : Bare_Ref_Id;
Var_Expr : Boolean;
Var_Expr_1 : Internal_Semantic_Result_Array_Access;
Var_Expr_2 : Boolean;
Var_Expr_3 : Bare_Param_List;
Var_Expr_12 : Bare_Param;
Var_Expr_13 : Bare_Param;
Var_Expr_14 : Bare_Decl_Annotation;
Var_Expr_4 : Boolean;
Var_Expr_15 : Bare_Param;
Fld : Internal_Entity_Full_Decl;
Sym : Symbol_Type;
Fld_1 : Bare_Decl_Annotation;
Fld_2 : Bare_Decl_Annotation_Params;
Fld_3 : Bare_Param_List;
Scope_Result : Bare_Param_List;
Result_Var : Bare_Param_List;
Scope_Result_1 : Bare_Param_List;
Result_Var_1 : Bare_Param_List;
Get_Result : Bare_Param;
Fld_4 : Bare_Expr;
Cast_Result : Bare_Ref_Id;
As_Entity : Internal_Entity_Ref_Id;
Is_Null : Boolean;
If_Result : Internal_Entity_Ref_Id;
Fld_5 : Internal_Semantic_Result := No_Semantic_Result;
Fld_6 : Internal_Entity_Decl;
Cast_Result_1 : Internal_Entity_Lexer_Decl;
Scope_Result_2 : Internal_Entity_Lexer_Decl;
Result_Var_2 : Internal_Entity_Lexer_Decl;
Scope_Result_3 : Internal_Entity_Lexer_Decl;
Result_Var_3 : Internal_Entity_Lexer_Decl;
Is_Null_1 : Boolean;
Str : String_Type;
Fld_7 : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
Scope_Result_4 : Internal_Semantic_Result_Array_Access;
Result_Var_4 : Internal_Semantic_Result_Array_Access;
Node_Children : Bare_Lkt_Node_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Not_Val : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Fld_8 : Bare_Ref_Id;
Is_Null_2 : Boolean;
Scope_Result_6 : Boolean;
Result_Var_6 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Fld_9 : Bare_Expr;
Is_Null_3 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_7 : Boolean;
If_Result_2 : Boolean;
Str_1 : String_Type;
Fld_10 : Internal_Semantic_Result := No_Semantic_Result;
Singleton_1 : Internal_Semantic_Result_Array_Access;
Scope_Result_8 : Internal_Semantic_Result_Array_Access;
Result_Var_8 : Internal_Semantic_Result_Array_Access;
Scope_Result_9 : Internal_Semantic_Result_Array_Access;
Result_Var_9 : Internal_Semantic_Result_Array_Access;
Scope_Result_10 : Internal_Semantic_Result_Array_Access;
Is_Null_4 : Boolean;
Str_2 : String_Type;
Fld_11 : Internal_Semantic_Result := No_Semantic_Result;
Singleton_2 : Internal_Semantic_Result_Array_Access;
Scope_Result_11 : Internal_Semantic_Result_Array_Access;
Result_Var_10 : Internal_Semantic_Result_Array_Access;
Scope_Result_12 : Internal_Semantic_Result_Array_Access;
Result_Var_11 : Internal_Semantic_Result_Array_Access;
Result_Var_12 : Internal_Semantic_Result_Array_Access;
Concat_Result : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_103 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Result_Var_9);
                     Dec_Ref (Scope_Result_10);
                     Dec_Ref (Scope_Result_12);
                     Dec_Ref (Result_Var_11);
                     Dec_Ref (Result_Var_12);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_103;
            procedure Finalizer_Scope_358 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_358;
            procedure Finalizer_Scope_359 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_359;
            procedure Finalizer_Scope_360 is
            begin
                     Dec_Ref (Node_Children);
            end Finalizer_Scope_360;
            procedure Finalizer_Scope_363 is
            begin
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Result_Var_8);
            end Finalizer_Scope_363;
            procedure Finalizer_Scope_364 is
            begin
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_364;
            procedure Finalizer_Scope_365 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_365;
            procedure Finalizer_Scope_366 is
            begin
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (Result_Var_10);
            end Finalizer_Scope_366;
            procedure Finalizer_Scope_367 is
            begin
                     Dec_Ref (Str_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Singleton_2);
            end Finalizer_Scope_367;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 413 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 386 '<FieldAccess for get_annotation at parser.py:1231>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1231



--# expr-start 384 '<FieldAccess for full_decl at parser.py:1231>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1231






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.full_decl
Fld := Liblktlang.Implementation.Decl_P_Full_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 384


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 385 '<Symbol with_lexer at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_With_Lexer); 
--# expr-done 385
--# property-call-start FullDecl.get_annotation
Fld_1 := Liblktlang.Implementation.Full_Decl_P_Get_Annotation (Node => Fld.Node, Name => Sym);
--# end
--# expr-done 386
Block_Var := Fld_1; 
--# bind lexer_annot Block_Var




Var_Expr_8 := Block_Var;





if Var_Expr_8 /= No_Bare_Lkt_Node then
   



   --# scope-start


   




--# expr-start 387 '<FieldAccess for params at parser.py:1232>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1232






   

      if Var_Expr_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Var_Expr_8.Decl_Annotation_F_Params;
--# expr-done 387
Var_Expr_9 := Fld_2;





if Var_Expr_9 /= No_Bare_Lkt_Node then
   



   --# scope-start


   

--# expr-start 388 '<FieldAccess for params at parser.py:1232>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1232






   

      if Var_Expr_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Var_Expr_9.Decl_Annotation_Params_F_Params;
--# expr-done 388
Scope_Result := Fld_3;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Bare_Lkt_Node;
end if;



Scope_Result_1 := Result_Var;


   --# end


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Bare_Lkt_Node;
end if;



Block_Var_1 := Result_Var_1; 
--# bind lexer_annot_params Block_Var_1
--# expr-start 389 '<at at parser.py:1233>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1233




Get_Result := Get (Self, Block_Var_1, 0, True); 
--# expr-done 389
Block_Var_2 := Get_Result; 
--# bind lexer_arg Block_Var_2




Var_Expr_10 := Block_Var_2;





if Var_Expr_10 /= No_Bare_Lkt_Node then
   



   --# scope-start


   




--# expr-start 391 '<Cast to RefId at parser.py:1236>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1236





--# expr-start 390 '<FieldAccess for value at parser.py:1236>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1236






   

      if Var_Expr_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Var_Expr_10.Param_F_Value;
--# expr-done 390



   if Fld_4 = null
      or else Fld_4.Kind in Lkt_Ref_Id_Range
   then
      
      Cast_Result := Fld_4;

   else
         Cast_Result := No_Bare_Lkt_Node;
   end if;


--# expr-done 391
Var_Expr_11 := Cast_Result;





if Var_Expr_11 /= No_Bare_Lkt_Node then
   



   --# scope-start


   

--# expr-start 395 '<Cast to LexerDecl at parser.py:1237>' Cast_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1237





--# expr-start 394 '<FieldAccess for result_decl at parser.py:1237>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1237



--# expr-start 393 '<FieldAccess for referenced_decl at parser.py:1237>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1237



--# expr-start 392 '<as_entity at parser.py:1236>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1236




Is_Null := Var_Expr_11 = null; 
if Is_Null then
   
   If_Result := No_Entity_Ref_Id;
else
   

As_Entity := (Info => E_Info, Node => Var_Expr_11); 
   If_Result := As_Entity;
end if;



--# expr-done 392


   

      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld_5 := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 393


   




Fld_6 := Fld_5.Result_Decl;
--# expr-done 394



   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Lkt_Lexer_Decl_Range
   then
      
      Cast_Result_1 := Create_Internal_Entity_Lexer_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);

   else
         Cast_Result_1 := No_Entity_Lexer_Decl;
   end if;


--# expr-done 395
Scope_Result_2 := Cast_Result_1;


   --# end
      Finalizer_Scope_358;


   Result_Var_2 := Scope_Result_2;
else
   
   Result_Var_2 := No_Entity_Lexer_Decl;
end if;



Scope_Result_3 := Result_Var_2;


   --# end


   Result_Var_3 := Scope_Result_3;
else
   
   Result_Var_3 := No_Entity_Lexer_Decl;
end if;



Block_Var_3 := Result_Var_3; 
--# bind lexer_decl Block_Var_3



--# expr-start 396 '<IsNull at parser.py:1242>' Is_Null_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1242

Is_Null_1 := Block_Var = null; 
--# expr-done 396
Var_Expr := Is_Null_1;





if Var_Expr /= False then
   



   --# scope-start


   --# bind _ Var_Expr

--# expr-start 398 '<FieldAccess for error at parser.py:1244>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1244






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 397 '<String at parser.py:1244>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1244
Str := Create_String ("missing mandatory ""@with_lexer"" annotation"); 
--# expr-done 397
--# property-call-start LktNode.error
Fld_7 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Str);
--# end
--# expr-done 398
                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_7;
                Inc_Ref (Singleton.Items (1));
            
Scope_Result_4 := Singleton;
   Inc_Ref (Scope_Result_4);


   --# end
      Finalizer_Scope_359;


   Result_Var_4 := Scope_Result_4;
else
   
   Result_Var_4 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_4);


















Var_Expr_3 := Block_Var_1;





if Var_Expr_3 /= No_Bare_Lkt_Node then
   



   --# scope-start


   --# bind p Var_Expr_3

--# expr-start 402 '<Not at parser.py:1250>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1250
--# expr-start 401 '<Eq at parser.py:1250>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1250
--# expr-start 400 '<length at parser.py:1250>' Len /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1250
--# expr-start 399 '<children at parser.py:1250>' Node_Children /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1250






   

      if Var_Expr_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Node_Children := Children (Var_Expr_3); 
--# expr-done 399
Len := Length (Node_Children); 
--# expr-done 400

Is_Equal := Len = 1; 
--# expr-done 401
Not_Val := not (Is_Equal); 
--# expr-done 402
Scope_Result_5 := Not_Val;


   --# end
      Finalizer_Scope_360;


   Result_Var_5 := Scope_Result_5;
else
   
   Result_Var_5 := False;
end if;



if Result_Var_5 then
   
   If_Result_1 := True;
else
   --# expr-start 405 '<Not at parser.py:1251>' Not_Val_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1251




Var_Expr_12 := Block_Var_2;





if Var_Expr_12 /= No_Bare_Lkt_Node then
   



   --# scope-start


   

--# expr-start 404 '<IsNull at parser.py:1251>' Is_Null_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1251
--# expr-start 403 '<FieldAccess for name at parser.py:1251>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1251






   

      if Var_Expr_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_8 := Var_Expr_12.Param_F_Name;
--# expr-done 403
Is_Null_2 := Fld_8 = null; 
--# expr-done 404
Scope_Result_6 := Is_Null_2;


   --# end


   Result_Var_6 := Scope_Result_6;
else
   
   Result_Var_6 := False;
end if;



Not_Val_1 := not (Result_Var_6); 
--# expr-done 405
   If_Result_1 := Not_Val_1;
end if;



if If_Result_1 then
   
   If_Result_2 := True;
else
   



Var_Expr_13 := Block_Var_2;





if Var_Expr_13 /= No_Bare_Lkt_Node then
   



   --# scope-start


   

--# expr-start 407 '<IsNull at parser.py:1252>' Is_Null_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1252
--# expr-start 406 '<FieldAccess for value at parser.py:1252>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1252






   

      if Var_Expr_13 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_9 := Var_Expr_13.Param_F_Value;
--# expr-done 406
Is_Null_3 := Fld_9 = null; 
--# expr-done 407
Scope_Result_7 := Is_Null_3;


   --# end


   Result_Var_7 := Scope_Result_7;
else
   
   Result_Var_7 := False;
end if;



   If_Result_2 := Result_Var_7;
end if;



Var_Expr_2 := If_Result_2;





if Var_Expr_2 /= False then
   



   --# scope-start


   --# bind _ Var_Expr_2





Var_Expr_14 := Block_Var;





if Var_Expr_14 /= No_Bare_Lkt_Node then
   



   --# scope-start


   

--# expr-start 409 '<FieldAccess for error at parser.py:1257>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1257






   

      if Var_Expr_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 408 '<String at parser.py:1257>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1257
Str_1 := Create_String ("exactly 1 positional argument expected"); 
--# expr-done 408
--# property-call-start LktNode.error
Fld_10 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Var_Expr_14, Message => Str_1);
--# end
--# expr-done 409
                Singleton_1 := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton_1.Items (1) := Fld_10;
                Inc_Ref (Singleton_1.Items (1));
            
Scope_Result_8 := Singleton_1;
   Inc_Ref (Scope_Result_8);


   --# end
      Finalizer_Scope_364;


   Result_Var_8 := Scope_Result_8;
else
   
   Result_Var_8 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_8);


Scope_Result_9 := Result_Var_8;
   Inc_Ref (Scope_Result_9);


   --# end
      Finalizer_Scope_363;


   Result_Var_9 := Scope_Result_9;
else
   
   Result_Var_9 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_9);


Var_Expr_1 := Result_Var_9;

      Inc_Ref (Var_Expr_1);




if Var_Expr_1 /= No_Internal_Semantic_Result_Array_Type then
   



   --# scope-start


   --# bind e Var_Expr_1


Scope_Result_10 := Var_Expr_1;
   Inc_Ref (Scope_Result_10);


   --# end
      Finalizer_Scope_365;


   Result_Var_12 := Scope_Result_10;
else
   


--# expr-start 410 '<IsNull at parser.py:1261>' Is_Null_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1261

Is_Null_4 := Block_Var_3.Node = null; 
--# expr-done 410
Var_Expr_4 := Is_Null_4;





if Var_Expr_4 /= False then
   



   --# scope-start


   --# bind _ Var_Expr_4





Var_Expr_15 := Block_Var_2;





if Var_Expr_15 /= No_Bare_Lkt_Node then
   



   --# scope-start


   

--# expr-start 412 '<FieldAccess for error at parser.py:1263>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1263






   

      if Var_Expr_15 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 411 '<String at parser.py:1263>' Str_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1263
Str_2 := Create_String ("reference to a lexer expected"); 
--# expr-done 411
--# property-call-start LktNode.error
Fld_11 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Var_Expr_15, Message => Str_2);
--# end
--# expr-done 412
                Singleton_2 := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton_2.Items (1) := Fld_11;
                Inc_Ref (Singleton_2.Items (1));
            
Scope_Result_11 := Singleton_2;
   Inc_Ref (Scope_Result_11);


   --# end
      Finalizer_Scope_367;


   Result_Var_10 := Scope_Result_11;
else
   
   Result_Var_10 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_10);


Scope_Result_12 := Result_Var_10;
   Inc_Ref (Scope_Result_12);


   --# end
      Finalizer_Scope_366;


   Result_Var_11 := Scope_Result_12;
else
   
   Result_Var_11 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_11);


   Result_Var_12 := Result_Var_11;
end if;

      Inc_Ref (Result_Var_12);


Concat_Result := Concat (Result_Var_4, Result_Var_12); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 413

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_103;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_103;
                     Finalizer_Scope_358;
                     Finalizer_Scope_359;
                     Finalizer_Scope_360;
                     Finalizer_Scope_363;
                     Finalizer_Scope_364;
                     Finalizer_Scope_365;
                     Finalizer_Scope_366;
                     Finalizer_Scope_367;




            raise;
      end;



   return Property_Result;
end Grammar_Decl_P_Check_Correctness_Pre;
--# end

   







--# property-start GrammarDecl.check_legality /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1269
pragma Warnings (Off, "is not referenced");
 function Grammar_Decl_P_Check_Legality
  
  (Node : Bare_Grammar_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Grammar_Decl  := Bare_Grammar_Decl (Node);
     Ent : Internal_Entity_Grammar_Decl :=
       Internal_Entity_Grammar_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_104 with Inline_Always;
            procedure Finalizer_Scope_370 with Inline_Always;
            procedure Finalizer_Scope_371 with Inline_Always;
            procedure Finalizer_Scope_372 with Inline_Always;

      Block_Var : Internal_Entity_Full_Decl_Array_Access;
Var_Expr : Boolean;
Fld : Internal_Entity_Full_Decl_List;
Fld_1 : Bare_Full_Decl_List;
Fld_2 : Internal_Entity_Info;
Item_19 : Internal_Entity_Full_Decl;
Bare_Item_19 : Bare_Full_Decl;
As_Entity : Internal_Entity_Full_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Full_Decl;
Fld_3 : Internal_Entity_Decl;
Is_A : Boolean;
Sym : Symbol_Type;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Full_Decl_Array_Access;
Fld_5 : Internal_Entity_Full_Decl_List;
Fld_6 : Bare_Full_Decl_List;
Fld_7 : Internal_Entity_Info;
Item_20 : Internal_Entity_Full_Decl;
Bare_Item_20 : Bare_Full_Decl;
Fld_8 : Internal_Entity_Decl;
Fld_9 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Fld_10 : String_Type;
Concat_Result_1 : String_Type;
Fld_11 : Internal_Semantic_Result := No_Semantic_Result;
As_Entity_1 : Internal_Entity_Full_Decl;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Full_Decl;
Fld_12 : Internal_Entity_Decl;
Is_A_1 : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Semantic_Result_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Str_1 : String_Type;
Fld_13 : Internal_Semantic_Result := No_Semantic_Result;
Array_Lit : Internal_Semantic_Result_Array_Access;
Scope_Result : Internal_Semantic_Result_Array_Access;
Result_Var : Internal_Semantic_Result_Array_Access;
Concat_Result_2 : Internal_Semantic_Result_Array_Access;
Index_22 : Integer;
Str_2 : String_Type;
Fld_14 : Internal_Semantic_Result := No_Semantic_Result;
Comp_Result : Boolean;
Map_Result_2 : Internal_Semantic_Result_Array_Access;
Concat_Result_3 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_104 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Concat_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_104;
            procedure Finalizer_Scope_370 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_370;
            procedure Finalizer_Scope_371 is
            begin
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_371;
            procedure Finalizer_Scope_372 is
            begin
                     Dec_Ref (Str_2);
                     Dec_Ref (Fld_14);
            end Finalizer_Scope_372;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 438 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 419 '<Filter at parser.py:1272>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1272








   

   --# expr-start 414 '<FieldAccess for rules at parser.py:1272>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1272






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Full_Decl_List (Node => Ent.Node.Grammar_Decl_F_Rules, Info => Ent.Info);
--# expr-done 414






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Internal_Entity_Full_Decl_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Internal_Entity_Full_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Full_Decl_List := Fld_1;
      begin
         for Untyped_Item_19 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_19 := Untyped_Item_19; 
                  



Is_Null := Bare_Item_19 = null; 
if Is_Null then
   
   If_Result := No_Entity_Full_Decl;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_19); 
   If_Result := As_Entity;
end if;



                  Item_19 := If_Result; 

            
   --# scope-start


               --# bind d Item_19

            
         


--# expr-start 416 '<IsA GrammarRuleDecl at parser.py:1273>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1273
--# expr-start 415 '<FieldAccess for decl at parser.py:1273>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1273






   

      if Item_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Decl (Node => Item_19.Node.Full_Decl_F_Decl, Info => Item_19.Info);
--# expr-done 415
Is_A := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Lkt_Grammar_Rule_Decl_Range; 
--# expr-done 416
if Is_A then
   --# expr-start 418 '<FieldAccess for has_annotation at parser.py:1273>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1273






   

      if Item_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 417 '<Symbol main_rule at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Main_Rule); 
--# expr-done 417
--# property-call-start FullDecl.has_annotation
Fld_4 := Liblktlang.Implementation.Full_Decl_P_Has_Annotation (Node => Item_19.Node, Name => Sym);
--# end
--# expr-done 418
   If_Result_1 := Fld_4;
else
   
   If_Result_1 := False;
end if;



         if If_Result_1 then
            
         
      
      

         declare
            Item_To_Append : constant Internal_Entity_Full_Decl := Item_19;
         begin
            Internal_Entity_Full_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Entity_Full_Decl_Array
           (Items_Count => Natural (Internal_Entity_Full_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Full_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Full_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Full_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 419
Block_Var := Map_Result; Inc_Ref (Block_Var);
--# bind main_rules Block_Var
--# expr-start 429 '<FilterMap at parser.py:1278>' Map_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1278








   

   --# expr-start 420 '<FieldAccess for rules at parser.py:1278>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1278






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Create_Internal_Entity_Full_Decl_List (Node => Ent.Node.Grammar_Decl_F_Rules, Info => Ent.Info);
--# expr-done 420






   




Fld_6 := Fld_5.Node;

   
      declare
         Map_Result_1_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      if Fld_6 = null then
         
      Map_Result_1 := Create_Internal_Semantic_Result_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Full_Decl_List := Fld_6;
      begin
         for Untyped_Item_20 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_20 := Untyped_Item_20; 
                  



Is_Null_1 := Bare_Item_20 = null; 
if Is_Null_1 then
   
   If_Result_2 := No_Entity_Full_Decl;
else
   





   




Fld_7 := Fld_5.Info;

As_Entity_1 := (Info => Fld_7, Node => Bare_Item_20); 
   If_Result_2 := As_Entity_1;
end if;



                  Item_20 := If_Result_2; 

            
   --# scope-start


               --# bind d Item_20

            
         --# expr-start 423 '<Not at parser.py:1283>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1283
--# expr-start 422 '<IsA GrammarRuleDecl at parser.py:1283>' Is_A_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1283
--# expr-start 421 '<FieldAccess for decl at parser.py:1283>' Fld_12 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1283






   

      if Item_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_12 := Create_Internal_Entity_Decl (Node => Item_20.Node.Full_Decl_F_Decl, Info => Item_20.Info);
--# expr-done 421
Is_A_1 := Fld_12.Node /= null 
and then Fld_12.Node.Kind in Lkt_Grammar_Rule_Decl_Range; 
--# expr-done 422
Not_Val := not (Is_A_1); 
--# expr-done 423
         if Not_Val then
            
         
      --# expr-start 428 '<FieldAccess for error at parser.py:1279>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1279






   

      if Item_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 425 '<FieldAccess for decl_type_name at parser.py:1280>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1280



--# expr-start 424 '<FieldAccess for decl at parser.py:1280>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1280






   

      if Item_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_8 := Create_Internal_Entity_Decl (Node => Item_20.Node.Full_Decl_F_Decl, Info => Item_20.Info);
--# expr-done 424


   

      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.decl_type_name'
Fld_9 := Liblktlang.Implementation.Dispatcher_Decl_P_Decl_Type_Name (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 425
--# expr-start 426 '<String at parser.py:1280>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1280
Str := Create_String (" forbidden in "); 
--# expr-done 426
Concat_Result := Concat_String (Fld_9, Str); 
--# expr-start 427 '<FieldAccess for decl_type_name at parser.py:1281>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1281






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.decl_type_name'
Fld_10 := Liblktlang.Implementation.Dispatcher_Decl_P_Decl_Type_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 427
Concat_Result_1 := Concat_String (Concat_Result, Fld_10); 
--# property-call-start LktNode.error
Fld_11 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Item_20.Node, Message => Concat_Result_1);
--# end
--# expr-done 428
      

         declare
            Item_To_Append : constant Internal_Semantic_Result := Fld_11;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end
      Finalizer_Scope_370;



         end loop;
      end;
   
      end if;

   

         Map_Result_1 := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_1_Vec);

      end;
   



--# expr-done 429



--# expr-start 431 '<Eq at parser.py:1287>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1287
--# expr-start 430 '<length at parser.py:1287>' Len /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1287

Len := Length (Block_Var); 
--# expr-done 430

Is_Equal := Len = 0; 
--# expr-done 431
Var_Expr := Is_Equal;





if Var_Expr /= False then
   



   --# scope-start


   --# bind _ Var_Expr

--# expr-start 434 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 433 '<FieldAccess for error at parser.py:1289>' Fld_13 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1289






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 432 '<String at parser.py:1289>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1289
Str_1 := Create_String ("main rule missing (@main_rule annotation)"); 
--# expr-done 432
--# property-call-start LktNode.error
Fld_13 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Str_1);
--# end
--# expr-done 433
Array_Lit := Create_Internal_Semantic_Result_Array (Internal_Internal_Semantic_Result_Array'(1 => Fld_13)); 
--# expr-done 434
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_371;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var);


Concat_Result_2 := Concat (Map_Result_1, Result_Var); 
--# expr-start 437 '<FilterMap at parser.py:1293>' Map_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1293








   

   

   
      declare
         Map_Result_2_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      
         Index_22 := 0;

      declare
         
         Collection : constant Internal_Entity_Full_Decl_Array_Access := Block_Var;
      begin
         for Item_21 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind d Item_21
               --# bind i Index_22

            
         

Comp_Result := Index_22 > 0; 
         if Comp_Result then
            
         
      --# expr-start 436 '<FieldAccess for error at parser.py:1294>' Fld_14 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1294






   

      if Item_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 435 '<String at parser.py:1294>' Str_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1294
Str_2 := Create_String ("only one main rule allowed"); 
--# expr-done 435
--# property-call-start LktNode.error
Fld_14 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Item_21.Node, Message => Str_2);
--# end
--# expr-done 436
      

         declare
            Item_To_Append : constant Internal_Semantic_Result := Fld_14;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end
      Finalizer_Scope_372;



               Index_22 := Index_22 + 1;
         end loop;
      end;
   

   

         Map_Result_2 := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_2_Vec);

      end;
   



--# expr-done 437
Concat_Result_3 := Concat (Concat_Result_2, Map_Result_2); 
Let_Result := Concat_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 438

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_104;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_104;
                     Finalizer_Scope_370;
                     Finalizer_Scope_371;
                     Finalizer_Scope_372;




            raise;
      end;



   return Property_Result;
end Grammar_Decl_P_Check_Legality;
--# end

   







--# property-start GrammarDecl._env_mappings_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1222
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_6
  
  (Node : Bare_Grammar_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc
is
   Self : Bare_Grammar_Decl  := Bare_Grammar_Decl (Node);
     Ent : Internal_Entity_Grammar_Decl :=
       Internal_Entity_Grammar_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc;

      
            procedure Finalizer_Scope_285 with Inline_Always;

      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Grammar_Decl;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;

            procedure Finalizer_Scope_285 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_285;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 441 '<New EnvAssoc at parser.py:1222>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1222
--# expr-start 439 '<New DesignatedEnv at parser.py:1222>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1222



Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 439
--# expr-start 440 '<FieldAccess for name at parser.py:1222>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1222






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Ent.Node);
--# end
--# expr-done 440







Cast_Expr := Self; 



   
      Cast_Result := Cast_Expr;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 441

         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_285;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_285;




            raise;
      end;



   return Property_Result;
end Env_Mappings_6;
--# end

   







--# property-start GrammarDecl._env_trans_parent_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1223
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_7
  
  (Node : Bare_Grammar_Decl
  )

   return Boolean
is
   Self : Bare_Grammar_Decl  := Bare_Grammar_Decl (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Trans_Parent_7;
--# end



   


      

   --
   --  Primitives for Bare_Lexer_Decl
   --

   



   

   

   

   

   

   

   


      procedure Lexer_Decl_Pre_Env_Actions
        (Self            : Bare_Lexer_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mapping : Internal_Env_Assoc :=
            Liblktlang.Implementation.Env_Mappings_3 (Node => 
Self);
      begin

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:1140");
         Dec_Ref (Mapping.Dest_Env);

      end;
   
   
            


   
      if Add_To_Env_Only then
         return;
      end if;

      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Liblktlang.Implementation.Env_Trans_Parent_4 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
   
   
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mappings : Internal_Env_Assoc_Array_Access :=
            Liblktlang.Implementation.Env_Mappings_5 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:1142");

         end loop;
         Dec_Ref (Mappings);
      end;
   
   
      end;



      
      procedure Initialize_Fields_For_Lexer_Decl
        (Self : Bare_Lexer_Decl
         ; Lexer_Decl_F_Syn_Name : Bare_Def_Id
         ; Lexer_Decl_F_Rules : Bare_Lkt_Node_List
        ) is
      begin

            Self.Lexer_Decl_F_Syn_Name := Lexer_Decl_F_Syn_Name;
            Self.Lexer_Decl_F_Rules := Lexer_Decl_F_Rules;
         

      end Initialize_Fields_For_Lexer_Decl;

      
   function Lexer_Decl_F_Rules
     (Node : Bare_Lexer_Decl) return Bare_Lkt_Node_List
   is
      

   begin
         
         return Node.Lexer_Decl_F_Rules;
      
   end;


   







--# property-start LexerDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1126
pragma Warnings (Off, "is not referenced");
 function Lexer_Decl_P_Decl_Type_Name
  
  (Node : Bare_Lexer_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Lexer_Decl  := Bare_Lexer_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_96 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_96 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_96;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 442 '<String at parser.py:1126>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1126
Str := Create_String ("lexer declaration"); 
--# expr-done 442

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_96;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_96;




            raise;
      end;



   return Property_Result;
end Lexer_Decl_P_Decl_Type_Name;
--# end

   







--# property-start LexerDecl.builtin_decls /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1128
pragma Warnings (Off, "is not referenced");
 function Lexer_Decl_P_Builtin_Decls
  
  (Node : Bare_Lexer_Decl
  )

   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Lexer_Decl  := Bare_Lexer_Decl (Node);
      --# bind self Self

   


   Property_Result : Internal_Env_Assoc_Array_Access;

      
            procedure Finalizer_Scope_98 with Inline_Always;
            procedure Finalizer_Scope_373 with Inline_Always;

      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
New_Node : Bare_Synthetic_Lexer_Decl;
Fld : Lexical_Env := Empty_Env;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;

            procedure Finalizer_Scope_98 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_98;
            procedure Finalizer_Scope_373 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_373;

      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Lexer_Decl_Lexer_Decl_P_Builtin_Decls,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Lexer_Decl,
                                As_Bare_Lexer_Decl => Self);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 452 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 451 '<Map at parser.py:1130>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1130








   

   --# expr-start 446 '<ArrayLiteral at parser.py:1130>' Array_Lit /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1130
--# expr-start 443 '<Symbol newline at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Newline); 
--# expr-done 443
--# expr-start 444 '<Symbol indent at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Indent); 
--# expr-done 444
--# expr-start 445 '<Symbol dedent at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Dedent); 
--# expr-done 445
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1, 3 => Sym_2)); 
--# expr-done 446

   
      declare
         Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Symbol_Type_Array_Access := Array_Lit;
      begin
         for Item_23 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind sym Item_23

            
         
         
      --# expr-start 450 '<New EnvAssoc at parser.py:1132>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1132
--# expr-start 448 '<New DesignatedEnv at parser.py:1135>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1135
--# expr-start 447 '<FieldAccess for children_env at parser.py:1135>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1135






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Children_Env (Node => Self);
--# expr-done 447


Inc_Ref (Fld);
New_Struct := (Direct_Env => Fld, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 448







--# expr-start 449 '<New SyntheticLexerDecl at parser.py:1134>' New_Node /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1134



if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;


New_Node := new Root_Node_Record
  (Lkt_Synthetic_Lexer_Decl);
Initialize
  (Self => New_Node,
   Kind => Lkt_Synthetic_Lexer_Decl,
   Unit => Self.Unit,

   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,

   Parent => Self,

   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);


      New_Node.Synthetic_Lexer_Decl_Sym :=
         Item_23;

--# expr-done 449



   
      Cast_Result := New_Node;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Item_23, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 450
      

         declare
            Item_To_Append : constant Internal_Env_Assoc := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_373;



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Env_Assoc_Array
           (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 451
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 452

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_98;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_98;
                     Finalizer_Scope_373;


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;

         end if;
      end if;



   return Property_Result;
end Lexer_Decl_P_Builtin_Decls;
--# end

   







--# property-start LexerDecl._env_mappings_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1140
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_3
  
  (Node : Bare_Lexer_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc
is
   Self : Bare_Lexer_Decl  := Bare_Lexer_Decl (Node);
     Ent : Internal_Entity_Lexer_Decl :=
       Internal_Entity_Lexer_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc;

      
            procedure Finalizer_Scope_281 with Inline_Always;

      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Lexer_Decl;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;

            procedure Finalizer_Scope_281 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_281;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 455 '<New EnvAssoc at parser.py:1140>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1140
--# expr-start 453 '<New DesignatedEnv at parser.py:1140>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1140



Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 453
--# expr-start 454 '<FieldAccess for name at parser.py:1140>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1140






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Ent.Node);
--# end
--# expr-done 454







Cast_Expr := Self; 



   
      Cast_Result := Cast_Expr;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 455

         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_281;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_281;




            raise;
      end;



   return Property_Result;
end Env_Mappings_3;
--# end

   







--# property-start LexerDecl._env_trans_parent_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1141
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_4
  
  (Node : Bare_Lexer_Decl
  )

   return Boolean
is
   Self : Bare_Lexer_Decl  := Bare_Lexer_Decl (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Trans_Parent_4;
--# end

   







--# property-start LexerDecl._env_mappings_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1142
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_5
  
  (Node : Bare_Lexer_Decl
  )

   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Lexer_Decl  := Bare_Lexer_Decl (Node);
      --# bind self Self

   


   Property_Result : Internal_Env_Assoc_Array_Access;

      
            procedure Finalizer_Scope_284 with Inline_Always;

      Fld : Internal_Env_Assoc_Array_Access;

            procedure Finalizer_Scope_284 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_284;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 456 '<FieldAccess for builtin_decls at parser.py:1142>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1142






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LexerDecl.builtin_decls
Fld := Liblktlang.Implementation.Lexer_Decl_P_Builtin_Decls (Node => Self);
--# end
--# expr-done 456

         Property_Result := Fld;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_284;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_284;




            raise;
      end;



   return Property_Result;
end Env_Mappings_5;
--# end



   


      

   --
   --  Primitives for Bare_Lexer_Family_Decl
   --

   



      
      procedure Initialize_Fields_For_Lexer_Family_Decl
        (Self : Bare_Lexer_Family_Decl
         ; Lexer_Family_Decl_F_Syn_Name : Bare_Def_Id
         ; Lexer_Family_Decl_F_Rules : Bare_Full_Decl_List
        ) is
      begin

            Self.Lexer_Family_Decl_F_Syn_Name := Lexer_Family_Decl_F_Syn_Name;
            Self.Lexer_Family_Decl_F_Rules := Lexer_Family_Decl_F_Rules;
         

      end Initialize_Fields_For_Lexer_Family_Decl;

      
   function Lexer_Family_Decl_F_Rules
     (Node : Bare_Lexer_Family_Decl) return Bare_Full_Decl_List
   is
      

   begin
         
         return Node.Lexer_Family_Decl_F_Rules;
      
   end;


   







--# property-start LexerFamilyDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1152
pragma Warnings (Off, "is not referenced");
 function Lexer_Family_Decl_P_Decl_Type_Name
  
  (Node : Bare_Lexer_Family_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Lexer_Family_Decl  := Bare_Lexer_Family_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_99 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_99 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_99;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 457 '<String at parser.py:1152>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1152
Str := Create_String ("lexer family declaration"); 
--# expr-done 457

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_99;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_99;




            raise;
      end;



   return Property_Result;
end Lexer_Family_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Type_Decl
   --

   




      
   function Type_Decl_F_Traits
     (Node : Bare_Type_Decl) return Bare_Type_Ref_List
   is
      

         Kind : constant Lkt_Type_Decl := Node.Kind;
   begin
         case Kind is
               when Lkt_Function_Type =>
                     return No_Bare_Lkt_Node;
               when Lkt_Generic_Formal_Type_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Instantiated_Generic_Type =>
                     return No_Bare_Lkt_Node;
               when Lkt_Trait_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Enum_Type_Decl =>
                     
         return Node.Enum_Type_Decl_F_Traits;
      
               when Lkt_Struct_Decl =>
                     
         return Node.Struct_Decl_F_Traits;
      
               when Lkt_Class_Decl | Lkt_Enum_Class_Decl =>
                     
         return Node.Basic_Class_Decl_F_Traits;
      
               when Lkt_Enum_Class_Alt_Decl =>
                     return No_Bare_Lkt_Node;
         end case;
   end;

      
   function Type_Decl_F_Syn_Base_Type
     (Node : Bare_Type_Decl) return Bare_Type_Ref
   is
      

         Kind : constant Lkt_Type_Decl := Node.Kind;
   begin
         case Kind is
               when Lkt_Function_Type =>
                     return No_Bare_Lkt_Node;
               when Lkt_Generic_Formal_Type_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Instantiated_Generic_Type =>
                     return No_Bare_Lkt_Node;
               when Lkt_Trait_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Enum_Type_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Struct_Decl =>
                     return No_Bare_Lkt_Node;
               when Lkt_Class_Decl | Lkt_Enum_Class_Decl =>
                     
         return Node.Basic_Class_Decl_F_Syn_Base_Type;
      
               when Lkt_Enum_Class_Alt_Decl =>
                     return No_Bare_Lkt_Node;
         end case;
   end;


   







--# property-start TypeDecl.concrete_matches /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1732
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Concrete_Matches
  
  (Node : Bare_Type_Decl
      ; Other : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind other Other

   Property_Result : Boolean;

      

      Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld : Boolean;
If_Result : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 460 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 458 '<Eq at parser.py:1737>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1737






Cast_Expr := Ent; 



   
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);








Cast_Expr_1 := Other; 



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);


Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 458
if Is_Equiv then
   
   If_Result := True;
else
   --# expr-start 459 '<FieldAccess for is_subtype at parser.py:1737>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1737






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start '[dispatcher]TypeDecl.is_subtype'
Fld := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Is_Subtype (Node => Ent.Node, Potential_Base => Other, E_Info => Ent.Info);
--# end
--# expr-done 459
   If_Result := Fld;
end if;



Let_Result := If_Result; 
--# end
--# expr-done 460

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Concrete_Matches;
--# end

   







--# property-start TypeDecl.matches /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1739
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Matches
  
  (Node : Bare_Type_Decl
      ; Other : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind other Other

   Property_Result : Boolean;

      
            procedure Finalizer_Scope_375 with Inline_Always;
            procedure Finalizer_Scope_376 with Inline_Always;

      Var_Expr : Internal_Entity_Instantiated_Generic_Type;
Var_Expr_1 : Internal_Entity_Instantiated_Generic_Type;
Formal : Internal_Entity_Full_Decl;
Actual_2 : Bare_Type_Decl;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Instantiated_Generic_Type;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Instantiated_Generic_Type;
Fld : Bare_Type_Decl;
As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld_1 : Bare_Type_Decl;
As_Entity_1 : Internal_Entity_Type_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Bare_Type_Decl_Array_Access;
Index_25 : Integer;
Fld_4 : Internal_Entity_Generic_Decl;
Fld_5 : Internal_Entity_Generic_Formal_Decl_List;
Fld_6 : Bare_Generic_Formal_Decl_List;
Fld_7 : Internal_Entity_Info;
Get_Result : Bare_Full_Decl;
As_Entity_2 : Internal_Entity_Full_Decl;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Full_Decl;
Fld_8 : Bare_Type_Decl_Array_Access;
Get_Result_1 : Bare_Type_Decl;
As_Entity_3 : Internal_Entity_Type_Decl;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Type_Decl;
As_Entity_4 : Internal_Entity_Type_Decl;
Is_Null_4 : Boolean;
If_Result_4 : Internal_Entity_Type_Decl;
Fld_9 : Boolean;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_2 : Bare_Type_Decl;
Is_Equal : Boolean;
Sym : Symbol_Type;
Fld_10 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
Quantifier_Result : Boolean;
If_Result_6 : Boolean;
Scope_Result : Boolean;
Fld_11 : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Fld_12 : Boolean;
Result_Var_1 : Boolean;
Let_Result_1 : Boolean;

            procedure Finalizer_Scope_375 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_375;
            procedure Finalizer_Scope_376 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_376;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 484 '<Block at ???>' Let_Result_1 None
--# scope-start



--# expr-start 461 '<Cast to InstantiatedGenericType at parser.py:1744>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1744






Cast_Expr := Ent; 



   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Lkt_Instantiated_Generic_Type_Range
   then
      
      Cast_Result := Create_Internal_Entity_Instantiated_Generic_Type
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);

   else
         Cast_Result := No_Entity_Instantiated_Generic_Type;
   end if;


--# expr-done 461
Var_Expr := Cast_Result;





if Var_Expr /= No_Entity_Instantiated_Generic_Type then
   



   --# scope-start


   --# bind e Var_Expr




--# expr-start 462 '<Cast to InstantiatedGenericType at parser.py:1745>' Cast_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1745






Cast_Expr_1 := Other; 



   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Lkt_Instantiated_Generic_Type_Range
   then
      
      Cast_Result_1 := Create_Internal_Entity_Instantiated_Generic_Type
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);

   else
         Cast_Result_1 := No_Entity_Instantiated_Generic_Type;
   end if;


--# expr-done 462
Var_Expr_1 := Cast_Result_1;





if Var_Expr_1 /= No_Entity_Instantiated_Generic_Type then
   



   --# scope-start


   --# bind o Var_Expr_1




--# expr-start 467 '<FieldAccess for matches at parser.py:1753>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1753



--# expr-start 464 '<as_bare_entity at parser.py:1753>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1753



--# expr-start 463 '<FieldAccess for inner_type_decl at parser.py:1753>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1753






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Var_Expr.Node.Instantiated_Generic_Type_Inner_Type_Decl;
--# expr-done 463
Is_Null := Fld = null; 
if Is_Null then
   
   If_Result := No_Entity_Type_Decl;
else
   

As_Entity := (Info => No_Entity_Info, Node => Fld); 
   If_Result := As_Entity;
end if;



--# expr-done 464


   

      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 466 '<as_bare_entity at parser.py:1754>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1754



--# expr-start 465 '<FieldAccess for inner_type_decl at parser.py:1754>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1754






   

      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Var_Expr_1.Node.Instantiated_Generic_Type_Inner_Type_Decl;
--# expr-done 465
Is_Null_1 := Fld_1 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Type_Decl;
else
   

As_Entity_1 := (Info => No_Entity_Info, Node => Fld_1); 
   If_Result_1 := As_Entity_1;
end if;



--# expr-done 466
--# property-call-start TypeDecl.matches
Fld_2 := Liblktlang.Implementation.Type_Decl_P_Matches (Node => If_Result.Node, Other => If_Result_1, E_Info => If_Result.Info);
--# end
--# expr-done 467
if Fld_2 then
   --# expr-start 481 '<AllQuantifier at parser.py:1758>' Quantifier_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1758








   

   --# expr-start 468 '<FieldAccess for actuals at parser.py:1758>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1758






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Var_Expr.Node.Instantiated_Generic_Type_Actuals;
Inc_Ref (Fld_3);
--# expr-done 468

   
      Quantifier_Result := True;
   

   

      
         Index_25 := 0;

      declare
         
         Collection : constant Bare_Type_Decl_Array_Access := Fld_3;
      begin
         for Item_24 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind actual_1 Item_24
               --# bind i Index_25

            
      --# expr-start 480 '<Let at parser.py:1758>' Let_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1758
--# scope-start
--# expr-start 470 '<FieldAccess for generic_formal_decls at parser.py:1760>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1760



--# expr-start 469 '<FieldAccess for generic_decl at parser.py:1760>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1760






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start InstantiatedGenericType.generic_decl
Fld_4 := Liblktlang.Implementation.Instantiated_Generic_Type_P_Generic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 469


   

      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Create_Internal_Entity_Generic_Formal_Decl_List (Node => Fld_4.Node.Generic_Decl_F_Generic_Formal_Decls, Info => Fld_4.Info);
--# expr-done 470










   




Fld_6 := Fld_5.Node;


Get_Result := Get (Self, Fld_6, Index_25, True); 
Is_Null_2 := Get_Result = null; 
if Is_Null_2 then
   
   If_Result_2 := No_Entity_Full_Decl;
else
   





   




Fld_7 := Fld_5.Info;

As_Entity_2 := (Info => Fld_7, Node => Get_Result); 
   If_Result_2 := As_Entity_2;
end if;



Formal := If_Result_2; 
--# bind formal Formal
--# expr-start 472 '<at at parser.py:1761>' Get_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1761

--# expr-start 471 '<FieldAccess for actuals at parser.py:1761>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1761






   

      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_8 := Var_Expr_1.Node.Instantiated_Generic_Type_Actuals;
Inc_Ref (Fld_8);
--# expr-done 471


Get_Result_1 := Get (Self, Fld_8, Index_25, True); 
--# expr-done 472
Actual_2 := Get_Result_1; 
--# bind actual_2 Actual_2
--# expr-start 479 '<If at parser.py:1762>' If_Result_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1762



--# expr-start 474 '<FieldAccess for has_annotation at parser.py:1763>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1763






   

      if Formal.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 473 '<Symbol out at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Out); 
--# expr-done 473
--# property-call-start FullDecl.has_annotation
Fld_10 := Liblktlang.Implementation.Full_Decl_P_Has_Annotation (Node => Formal.Node, Name => Sym);
--# end
--# expr-done 474
if Fld_10 then
   --# expr-start 477 '<FieldAccess for matches at parser.py:1768>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1768



--# expr-start 475 '<as_bare_entity at parser.py:1768>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1768




Is_Null_3 := Item_24 = null; 
if Is_Null_3 then
   
   If_Result_3 := No_Entity_Type_Decl;
else
   

As_Entity_3 := (Info => No_Entity_Info, Node => Item_24); 
   If_Result_3 := As_Entity_3;
end if;



--# expr-done 475


   

      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 476 '<as_bare_entity at parser.py:1769>' If_Result_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1769




Is_Null_4 := Actual_2 = null; 
if Is_Null_4 then
   
   If_Result_4 := No_Entity_Type_Decl;
else
   

As_Entity_4 := (Info => No_Entity_Info, Node => Actual_2); 
   If_Result_4 := As_Entity_4;
end if;



--# expr-done 476
--# property-call-start TypeDecl.matches
Fld_9 := Liblktlang.Implementation.Type_Decl_P_Matches (Node => If_Result_3.Node, Other => If_Result_4, E_Info => If_Result_3.Info);
--# end
--# expr-done 477
   If_Result_5 := Fld_9;
else
   --# expr-start 478 '<Eq at parser.py:1774>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1774






Cast_Expr_2 := Item_24; 



   
      Cast_Result_2 := Cast_Expr_2;



Is_Equal := Cast_Result_2 = Actual_2; 
--# expr-done 478
   If_Result_5 := Is_Equal;
end if;



--# expr-done 479
Let_Result := If_Result_5; 
--# end
--# expr-done 480
      Quantifier_Result := Let_Result;
   

            
   --# end
      Finalizer_Scope_376;


               exit when not Quantifier_Result;

               Index_25 := Index_25 + 1;
         end loop;
      end;
   

   
   



--# expr-done 481
   If_Result_6 := Quantifier_Result;
else
   
   If_Result_6 := False;
end if;



Scope_Result := If_Result_6;


   --# end
      Finalizer_Scope_375;


   Result_Var := Scope_Result;
else
   --# expr-start 482 '<FieldAccess for concrete_matches at parser.py:1778>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1778






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start TypeDecl.concrete_matches
Fld_11 := Liblktlang.Implementation.Type_Decl_P_Concrete_Matches (Node => Ent.Node, Other => Other, E_Info => Ent.Info);
--# end
--# expr-done 482
   Result_Var := Fld_11;
end if;



Scope_Result_1 := Result_Var;


   --# end


   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 483 '<FieldAccess for concrete_matches at parser.py:1779>' Fld_12 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1779






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start TypeDecl.concrete_matches
Fld_12 := Liblktlang.Implementation.Type_Decl_P_Concrete_Matches (Node => Ent.Node, Other => Other, E_Info => Ent.Info);
--# end
--# expr-done 483
   Result_Var_1 := Fld_12;
end if;



Let_Result_1 := Result_Var_1; 
--# end
--# expr-done 484

         Property_Result := Let_Result_1;
         
   --# end


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_375;
                     Finalizer_Scope_376;




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Matches;
--# end

   







--# property-start TypeDecl.self_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1782
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Self_Decl
  
  (Node : Bare_Type_Decl
  )

   return Bare_Self_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self

   


   Property_Result : Bare_Self_Decl;

      

      New_Node : Bare_Self_Decl;
Let_Result : Bare_Self_Decl;


      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Self_Decl,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Bare_Self_Decl;

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 486 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 485 '<New SelfDecl at parser.py:1784>' New_Node /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1784



if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;


New_Node := new Root_Node_Record
  (Lkt_Self_Decl);
Initialize
  (Self => New_Node,
   Kind => Lkt_Self_Decl,
   Unit => Self.Unit,

   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,

   Parent => Self,

   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);



--# expr-done 485
Let_Result := New_Node; 
--# end
--# expr-done 486

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Bare_Self_Decl,
                        As_Bare_Self_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);

         end if;
      end if;



   return Property_Result;
end Type_Decl_P_Self_Decl;
--# end

   







--# property-start TypeDecl.node_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1786
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Node_Decl
  
  (Node : Bare_Type_Decl
  )

   return Bare_Node_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self

   


   Property_Result : Bare_Node_Decl;

      

      New_Node : Bare_Node_Decl;
Let_Result : Bare_Node_Decl;


      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Node_Decl,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Bare_Node_Decl;

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 488 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 487 '<New NodeDecl at parser.py:1788>' New_Node /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1788



if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;


New_Node := new Root_Node_Record
  (Lkt_Node_Decl);
Initialize
  (Self => New_Node,
   Kind => Lkt_Node_Decl,
   Unit => Self.Unit,

   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,

   Parent => Self,

   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);



--# expr-done 487
Let_Result := New_Node; 
--# end
--# expr-done 488

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Bare_Node_Decl,
                        As_Bare_Node_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);

         end if;
      end if;



   return Property_Result;
end Type_Decl_P_Node_Decl;
--# end

   







--# property-start '[dispatcher]TypeDecl.type_scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Decl_P_Type_Scope
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self

   


   Property_Result : Lexical_Env;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Type_Decl (Self.Kind) is
               when Lkt_Enum_Class_Alt_Decl =>
                  --# property-call-start EnumClassAltDecl.type_scope
                  Property_Result := Enum_Class_Alt_Decl_P_Type_Scope
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Function_Type =>
                  --# property-call-start FunctionType.type_scope
                  Property_Result := Function_Type_P_Type_Scope
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormalTypeDecl.type_scope
                  Property_Result := Generic_Formal_Type_Decl_P_Type_Scope
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Instantiated_Generic_Type =>
                  --# property-call-start InstantiatedGenericType.type_scope
                  Property_Result := Instantiated_Generic_Type_P_Type_Scope
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Class_Decl .. Lkt_Trait_Decl =>
                  --# property-call-start NamedTypeDecl.type_scope
                  Property_Result := Named_Type_Decl_P_Type_Scope
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Type_Decl_P_Type_Scope;
--# end

   







--# property-start TypeDecl.get_fun /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1797
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Get_Fun
  
  (Node : Bare_Type_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Fun_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind name Name

   Property_Result : Internal_Entity_Fun_Decl;

      
            procedure Finalizer_Scope_126 with Inline_Always;

      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Fun_Decl;
Let_Result : Internal_Entity_Fun_Decl;

            procedure Finalizer_Scope_126 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_126;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 492 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 491 '<Cast to FunDecl at parser.py:1802>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1802





--# expr-start 490 '<EnvGet at parser.py:1802>' Env_Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1802
--# expr-start 489 '<FieldAccess for type_scope at parser.py:1802>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1802






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.type_scope'
Fld := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Type_Scope (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 489



Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Thin (Name), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Nocat => True)); 
--# expr-done 490



   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Lkt_Fun_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Fun_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 491
Let_Result := Cast_Result; 
--# end
--# expr-done 492

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_126;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_126;




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Get_Fun;
--# end

   







--# property-start '[dispatcher]TypeDecl.fields' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Decl_P_Fields
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Component_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Component_Decl_Array_Access;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Type_Decl (Self.Kind) is
               when Lkt_Enum_Class_Alt_Decl .. Lkt_Generic_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.fields
                  Property_Result := Type_Decl_P_Fields
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Instantiated_Generic_Type =>
                  --# property-call-start InstantiatedGenericType.fields
                  Property_Result := Instantiated_Generic_Type_P_Fields
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Class_Decl .. Lkt_Trait_Decl =>
                  --# property-call-start NamedTypeDecl.fields
                  Property_Result := Named_Type_Decl_P_Fields
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Type_Decl_P_Fields;
--# end

   







--# property-start TypeDecl.ultimate_instantiated_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1811
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Ultimate_Instantiated_Type
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Match_0 : Internal_Entity_Instantiated_Generic_Type;
Match_1 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Instantiated_Generic_Type;
Let_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Decl;
Let_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Match_Result : Internal_Entity_Type_Decl;
Let_Result_2 : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 496 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 495 '<Match at parser.py:1820>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1820









   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Ent; 

case Lkt_Type_Decl (Match_Prefix.Node.Kind) is
         when Lkt_Instantiated_Generic_Type =>
            



   --# scope-start



--# scope-start









   
      Cast_Result := Create_Internal_Entity_Instantiated_Generic_Type
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result; 
--# bind igt Match_0
--# expr-start 494 '<FieldAccess for ultimate_instantiated_type at parser.py:1822>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1822



--# expr-start 493 '<FieldAccess for get_instantiated_type at parser.py:1822>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1822






   

      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start InstantiatedGenericType.get_instantiated_type
Fld := Liblktlang.Implementation.Instantiated_Generic_Type_P_Get_Instantiated_Type (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 493


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.ultimate_instantiated_type
Fld_1 := Liblktlang.Implementation.Type_Decl_P_Ultimate_Instantiated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 494
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;


   --# end


            Match_Result := Scope_Result; 
         when Lkt_Enum_Class_Alt_Decl .. Lkt_Generic_Formal_Type_Decl | Lkt_Class_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_1; 
--# bind _ Match_1

Let_Result_1 := Ent; 
--# end
Scope_Result_1 := Let_Result_1;


   --# end


            Match_Result := Scope_Result_1; 
end case;

--# expr-done 495
Let_Result_2 := Match_Result; 
--# end
--# expr-done 496

         Property_Result := Let_Result_2;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Ultimate_Instantiated_Type;
--# end

   







--# property-start TypeDecl.is_class /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1826
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Class
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Boolean;

      

      Match_0 : Internal_Entity_Class_Decl;
Match_1 : Internal_Entity_Generic_Formal_Type_Decl;
Match_2 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Class_Qualifier;
Fld_2 : Boolean;
Match_Prefix : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Class_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Generic_Formal_Type_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Let_Result_3 : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 501 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 500 '<Match at parser.py:1831>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1831






--# expr-start 497 '<FieldAccess for ultimate_instantiated_type at parser.py:1831>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1831






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.ultimate_instantiated_type
Fld := Liblktlang.Implementation.Type_Decl_P_Ultimate_Instantiated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 497


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Fld; 

case Lkt_Type_Decl (Match_Prefix.Node.Kind) is
         when Lkt_Class_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result := Create_Internal_Entity_Class_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result; 
--# bind _ Match_0

Let_Result := True; 
--# end
Scope_Result := Let_Result;


   --# end


            Match_Result := Scope_Result; 
         when Lkt_Generic_Formal_Type_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_1 := Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_1; 
--# bind gft Match_1
--# expr-start 499 '<FieldAccess for as_bool at parser.py:1833>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1833



--# expr-start 498 '<FieldAccess for has_class at parser.py:1833>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1833






   

      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Class_Qualifier (Node => Match_1.Node.Generic_Formal_Type_Decl_F_Has_Class, Info => Match_1.Info);
--# expr-done 498


   

      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]ClassQualifier.as_bool'
Fld_2 := Liblktlang.Implementation.Dispatcher_Class_Qualifier_P_As_Bool (Node => Fld_1.Node);
--# end
--# expr-done 499
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;


   --# end


            Match_Result := Scope_Result_1; 
         when Lkt_Enum_Class_Alt_Decl .. Lkt_Function_Type | Lkt_Instantiated_Generic_Type | Lkt_Enum_Class_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_2 := Cast_Result_2; 
--# bind _ Match_2

Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;


   --# end


            Match_Result := Scope_Result_2; 
end case;

--# expr-done 500
Let_Result_3 := Match_Result; 
--# end
--# expr-done 501

         Property_Result := Let_Result_3;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Is_Class;
--# end

   







--# property-start TypeDecl.implemented_traits /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1837
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Implemented_Traits
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl_Array_Access;

      
            procedure Finalizer_Scope_130 with Inline_Always;
            procedure Finalizer_Scope_383 with Inline_Always;

      Fld : Internal_Entity_Type_Ref_List;
Fld_1 : Bare_Type_Ref_List;
Fld_2 : Internal_Entity_Info;
Item_26 : Internal_Entity_Type_Ref;
Bare_Item_26 : Bare_Type_Ref;
Fld_3 : Internal_Entity_Type_Decl;
As_Entity : Internal_Entity_Type_Ref;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Ref;
Map_Result : Internal_Entity_Type_Decl_Array_Access;
Fld_4 : Internal_Entity_Type_Decl_Array_Access;
Fld_5 : Internal_Entity_Type_Decl_Array_Access;
Map_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Type_Decl_Array_Access;

            procedure Finalizer_Scope_130 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_130;
            procedure Finalizer_Scope_383 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_383;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 508 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 504 '<Map at parser.py:1842>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1842








   

   --# expr-start 502 '<FieldAccess for traits at parser.py:1842>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1842






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref_List (Node => Implementation.Type_Decl_F_Traits (Ent.Node), Info => Ent.Info);
--# expr-done 502






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Internal_Entity_Type_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Type_Ref_List := Fld_1;
      begin
         for Untyped_Item_26 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_26 := Untyped_Item_26; 
                  



Is_Null := Bare_Item_26 = null; 
if Is_Null then
   
   If_Result := No_Entity_Type_Ref;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_26); 
   If_Result := As_Entity;
end if;



                  Item_26 := If_Result; 

            
   --# scope-start


               --# bind t Item_26

            
         
         
      --# expr-start 503 '<FieldAccess for designated_type at parser.py:1842>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1842






   

      if Item_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_3 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Item_26.Node, E_Info => Item_26.Info);
--# end
--# expr-done 503
      

         declare
            Item_To_Append : constant Internal_Entity_Type_Decl := Fld_3;
         begin
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Entity_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 504
--# expr-start 507 '<Mapcat at parser.py:1843>' Map_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1843








   

   --# expr-start 505 '<FieldAccess for base_types at parser.py:1843>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1843






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.base_types
Fld_4 := Liblktlang.Implementation.Type_Decl_P_Base_Types (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 505

   
      declare
         Map_Result_1_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Fld_4;
      begin
         for Item_27 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind bt Item_27

            
         
         
      --# expr-start 506 '<FieldAccess for implemented_traits at parser.py:1844>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1844






   

      if Item_27.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.implemented_traits
Fld_5 := Liblktlang.Implementation.Type_Decl_P_Implemented_Traits (Node => Item_27.Node, E_Info => Item_27.Info);
--# end
--# expr-done 506
      

         for Item_To_Append of
               Fld_5.Items
         loop
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;

   
   
   

            
   --# end
      Finalizer_Scope_383;



         end loop;
      end;
   

   

         Map_Result_1 := Create_Internal_Entity_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);

      end;
   



--# expr-done 507
Concat_Result := Concat (Map_Result, Map_Result_1); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 508

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_130;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_130;
                     Finalizer_Scope_383;




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Implemented_Traits;
--# end

   







--# property-start '[dispatcher]TypeDecl.base_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Decl_P_Base_Type
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Ref
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Ref;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Type_Decl (Self.Kind) is
               when Lkt_Enum_Class_Alt_Decl .. Lkt_Generic_Formal_Type_Decl | Lkt_Class_Decl .. Lkt_Trait_Decl =>
                  --# property-call-start TypeDecl.base_type
                  Property_Result := Type_Decl_P_Base_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Instantiated_Generic_Type =>
                  --# property-call-start InstantiatedGenericType.base_type
                  Property_Result := Instantiated_Generic_Type_P_Base_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Type_Decl_P_Base_Type;
--# end

   







--# property-start TypeDecl.base_types /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1857
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Types
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl_Array_Access;

      
            procedure Finalizer_Scope_132 with Inline_Always;
            procedure Finalizer_Scope_384 with Inline_Always;
            procedure Finalizer_Scope_385 with Inline_Always;

      Var_Expr : Internal_Entity_Type_Decl;
Var_Expr_16 : Internal_Entity_Type_Ref;
Fld : Internal_Entity_Type_Ref;
Fld_1 : Internal_Entity_Type_Decl;
Singleton : Internal_Entity_Type_Decl_Array_Access;
Fld_2 : Internal_Entity_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Type_Decl_Array_Access;

            procedure Finalizer_Scope_132 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_132;
            procedure Finalizer_Scope_384 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_384;
            procedure Finalizer_Scope_385 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_385;

      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Base_Types,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Type_Decl_Array_Access;
                     Inc_Ref (Property_Result);

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 512 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 509 '<FieldAccess for base_type at parser.py:1863>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1863






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.base_type'
Fld := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Base_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 509
Var_Expr_16 := Fld;





if Var_Expr_16 /= No_Entity_Type_Ref then
   



   --# scope-start


   




--# expr-start 510 '<FieldAccess for designated_type at parser.py:1863>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1863






   

      if Var_Expr_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Var_Expr_16.Node, E_Info => Var_Expr_16.Info);
--# end
--# expr-done 510
Var_Expr := Fld_1;





if Var_Expr /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind b Var_Expr


                Singleton := Create_Internal_Entity_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
                
            
--# expr-start 511 '<FieldAccess for base_types at parser.py:1864>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1864






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.base_types
Fld_2 := Liblktlang.Implementation.Type_Decl_P_Base_Types (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 511
Concat_Result := Concat (Singleton, Fld_2); 
Scope_Result := Concat_Result;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_385;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Internal_Entity_Type_Decl_Array_Type;
end if;

      Inc_Ref (Result_Var);


Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_384;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Internal_Entity_Type_Decl_Array_Type;
end if;

      Inc_Ref (Result_Var_1);


Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 512

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_132;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_132;
                     Finalizer_Scope_384;
                     Finalizer_Scope_385;


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Internal_Entity_Type_Decl_Array_Access,
                        As_Internal_Entity_Type_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;

         end if;
      end if;



   return Property_Result;
end Type_Decl_P_Base_Types;
--# end

   







--# property-start TypeDecl.root_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1867
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Root_Type
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Var_Expr : Internal_Entity_Type_Ref;
Var_Expr_17 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Ref;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Result_Var_1 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;


      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Root_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Type_Decl;

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 516 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 513 '<FieldAccess for base_type at parser.py:1873>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1873






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.base_type'
Fld := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Base_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 513
Var_Expr := Fld;





if Var_Expr /= No_Entity_Type_Ref then
   



   --# scope-start


   --# bind bd Var_Expr




--# expr-start 514 '<FieldAccess for designated_type at parser.py:1874>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1874






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 514
Var_Expr_17 := Fld_1;





if Var_Expr_17 /= No_Entity_Type_Decl then
   



   --# scope-start


   

--# expr-start 515 '<FieldAccess for root_type at parser.py:1874>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1874






   

      if Var_Expr_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.root_type
Fld_2 := Liblktlang.Implementation.Type_Decl_P_Root_Type (Node => Var_Expr_17.Node, E_Info => Var_Expr_17.Info);
--# end
--# expr-done 515
Scope_Result := Fld_2;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Type_Decl;
end if;



Scope_Result_1 := Result_Var;


   --# end


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := Ent;
end if;



Let_Result := Result_Var_1; 
--# end
--# expr-done 516

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Internal_Entity_Type_Decl,
                        As_Internal_Entity_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);

         end if;
      end if;



   return Property_Result;
end Type_Decl_P_Root_Type;
--# end

   







--# property-start '[dispatcher]TypeDecl.is_subtype' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Decl_P_Is_Subtype
  
  (Node : Bare_Type_Decl
      ; Potential_Base : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self

   

   --# bind potential_base Potential_Base

   Property_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Type_Decl (Self.Kind) is
               when Lkt_Enum_Class_Alt_Decl .. Lkt_Instantiated_Generic_Type | Lkt_Enum_Class_Decl .. Lkt_Trait_Decl =>
                  --# property-call-start TypeDecl.is_subtype
                  Property_Result := Type_Decl_P_Is_Subtype
                    (Self
                        , Potential_Base
                        , E_Info
                    );
                  --# end
               when Lkt_Class_Decl =>
                  --# property-call-start ClassDecl.is_subtype
                  Property_Result := Class_Decl_P_Is_Subtype
                    (Self
                        , Potential_Base
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Type_Decl_P_Is_Subtype;
--# end

   







--# property-start TypeDecl.type_base_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1886
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Type_Base_Scope
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_135 with Inline_Always;
            procedure Finalizer_Scope_388 with Inline_Always;
            procedure Finalizer_Scope_389 with Inline_Always;

      Var_Expr_18 : Internal_Entity_Type_Ref;
Fld : Internal_Entity_Type_Decl_Array_Access;
Fld_1 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Fld_2 : Internal_Entity_Type_Ref;
Fld_3 : Internal_Entity_Type_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Try_Result : Lexical_Env := Empty_Env;
Singleton : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_135 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Try_Result);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_135;
            procedure Finalizer_Scope_388 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_388;
            procedure Finalizer_Scope_389 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_389;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 525 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 524 '<env_group at parser.py:1904>' Group_Env /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1904
--# expr-start 519 '<Map at parser.py:1894>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1894








   

   --# expr-start 517 '<FieldAccess for implemented_traits at parser.py:1894>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1894






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.implemented_traits
Fld := Liblktlang.Implementation.Type_Decl_P_Implemented_Traits (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 517

   
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Fld;
      begin
         for Item_28 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind td Item_28

            
         
         
      --# expr-start 518 '<FieldAccess for type_scope at parser.py:1895>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1895






   

      if Item_28.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.type_scope'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Type_Scope (Node => Item_28.Node, E_Info => Item_28.Info);
--# end
--# expr-done 518
      

         declare
            Item_To_Append : constant Lexical_Env := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_388;



         end loop;
      end;
   

   

         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 519
--# expr-start 523 '<Try at parser.py:69>' Try_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:69



begin
   


--# expr-start 520 '<FieldAccess for base_type at parser.py:1900>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1900






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.base_type'
Fld_2 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Base_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 520
Var_Expr_18 := Fld_2;





if Var_Expr_18 /= No_Entity_Type_Ref then
   



   --# scope-start


   

--# expr-start 522 '<FieldAccess for type_scope at parser.py:1900>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1900



--# expr-start 521 '<FieldAccess for designated_type at parser.py:1900>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1900






   

      if Var_Expr_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_3 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Var_Expr_18.Node, E_Info => Var_Expr_18.Info);
--# end
--# expr-done 521


   

      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.type_scope'
Fld_4 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Type_Scope (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 522
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_389;


   Result_Var := Scope_Result;
else
   
   Result_Var := Empty_Env;
end if;

      Inc_Ref (Result_Var);


   Try_Result := Result_Var;
exception
   when Property_Error =>
      
      Try_Result := Empty_Env;
end;

      Inc_Ref (Try_Result);


--# expr-done 523
                Singleton := Create_Lexical_Env_Array (Items_Count => 1);
                Singleton.Items (1) := Try_Result;
                Inc_Ref (Singleton.Items (1));
            
Concat_Result := Concat (Map_Result, Singleton); 

Group_Env := Group (Concat_Result, No_Metadata); 
--# expr-done 524
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 525

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_135;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_135;
                     Finalizer_Scope_388;
                     Finalizer_Scope_389;




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Type_Base_Scope;
--# end

   







--# property-start TypeDecl.call_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1907
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Call_Scope
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_136 with Inline_Always;

      Fld : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_136 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_136;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 526 '<FieldAccess for type_scope at parser.py:1907>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1907






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.type_scope'
Fld := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Type_Scope (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 526

         Property_Result := Fld;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_136;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_136;




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Call_Scope;
--# end

   







--# property-start TypeDecl.is_generic /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1909
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Generic
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Boolean;

      

      Is_A : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 527 '<IsA InstantiatedGenericType at parser.py:1910>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1910

Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Lkt_Instantiated_Generic_Type_Range; 
--# expr-done 527

         Property_Result := Is_A;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Is_Generic;
--# end

   







--# property-start TypeDecl.array_element_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1914
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Array_Element_Type
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_390 with Inline_Always;

      Var_Expr : Internal_Entity_Instantiated_Generic_Type;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Instantiated_Generic_Type;
Fld : Internal_Entity_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Type_Decl;
Fld_1 : Bare_Type_Decl;
Fld_2 : Internal_Entity_Named_Type_Decl;
Fld_3 : Bare_Decl;
Cast_Result_1 : Bare_Decl;
Is_Equal : Boolean;
If_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_390 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_390;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 536 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 528 '<Cast to InstantiatedGenericType at parser.py:1922>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1922






Cast_Expr := Ent; 



   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Lkt_Instantiated_Generic_Type_Range
   then
      
      Cast_Result := Create_Internal_Entity_Instantiated_Generic_Type
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);

   else
         Cast_Result := No_Entity_Instantiated_Generic_Type;
   end if;


--# expr-done 528
Var_Expr := Cast_Result;





if Var_Expr /= No_Entity_Instantiated_Generic_Type then
   



   --# scope-start


   --# bind array Var_Expr

--# expr-start 535 '<If at parser.py:1923>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1923



--# expr-start 532 '<Eq at parser.py:1924>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1924





--# expr-start 529 '<FieldAccess for get_inner_type at parser.py:1924>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1924






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start InstantiatedGenericType.get_inner_type
Fld_1 := Liblktlang.Implementation.Instantiated_Generic_Type_P_Get_Inner_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 529



   
      Cast_Result_1 := Fld_1;


--# expr-start 531 '<FieldAccess for assert_bare at parser.py:1924>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1924



--# expr-start 530 '<FieldAccess for array_type at parser.py:1924>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1924






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.array_type
Fld_2 := Liblktlang.Implementation.Lkt_Node_P_Array_Type (Node => Ent.Node);
--# end
--# expr-done 530


   

      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.assert_bare
Fld_3 := Liblktlang.Implementation.Decl_P_Assert_Bare (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 531
Is_Equal := Cast_Result_1 = Fld_3; 
--# expr-done 532
if Is_Equal then
   --# expr-start 534 '<at at parser.py:1925>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1925

--# expr-start 533 '<FieldAccess for get_actuals at parser.py:1925>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1925






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start InstantiatedGenericType.get_actuals
Fld := Liblktlang.Implementation.Instantiated_Generic_Type_P_Get_Actuals (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 533


Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 534
   If_Result := Get_Result;
else
   
   If_Result := No_Entity_Type_Decl;
end if;



--# expr-done 535
Scope_Result := If_Result;


   --# end
      Finalizer_Scope_390;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Type_Decl;
end if;



Let_Result := Result_Var; 
--# end
--# expr-done 536

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_390;




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Array_Element_Type;
--# end

   







--# property-start TypeDecl.fields /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1804
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Fields
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Component_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Component_Decl_Array_Access;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := No_Internal_Entity_Component_Decl_Array_Type;
            Inc_Ref (Property_Result);
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Fields;
--# end

   







--# property-start TypeDecl.base_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1850
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Type
  
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Ref
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Ref;

      

      Fld : Internal_Entity_Type_Ref;
Let_Result : Internal_Entity_Type_Ref;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 538 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 537 '<FieldAccess for syn_base_type at parser.py:1855>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1855






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref (Node => Implementation.Type_Decl_F_Syn_Base_Type (Ent.Node), Info => Ent.Info);
--# expr-done 537
Let_Result := Fld; 
--# end
--# expr-done 538

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Base_Type;
--# end

   







--# property-start TypeDecl.is_subtype /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1878
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Subtype
  
  (Node : Bare_Type_Decl
      ; Potential_Base : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self

   

   --# bind potential_base Potential_Base

   Property_Result : Boolean;

      

      Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 539 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := False; 
--# end
--# expr-done 539

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Type_Decl_P_Is_Subtype;
--# end


   
function Type_Decl_P_Type_Base_Scope
  (E : Entity
  ) return Lexical_Env
is
   

      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
      
begin
   
      if E.Node /= null
         and then E.Node.Kind not in Lkt_Type_Decl
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;

   declare
      Result : constant Lexical_Env :=
         Type_Decl_P_Type_Base_Scope (E.Node, E_Info);
   begin
         return Result;
   end;
end;


   


      

   --
   --  Primitives for Bare_Enum_Class_Alt_Decl
   --

   



      
      procedure Initialize_Fields_For_Enum_Class_Alt_Decl
        (Self : Bare_Enum_Class_Alt_Decl
         ; Enum_Class_Alt_Decl_F_Syn_Name : Bare_Def_Id
        ) is
      begin

            Self.Enum_Class_Alt_Decl_F_Syn_Name := Enum_Class_Alt_Decl_F_Syn_Name;
         

      end Initialize_Fields_For_Enum_Class_Alt_Decl;


   







--# property-start EnumClassAltDecl.parent_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2371
pragma Warnings (Off, "is not referenced");
 function Enum_Class_Alt_Decl_P_Parent_Type
  
  (Node : Bare_Enum_Class_Alt_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Enum_Class_Alt_Decl  := Bare_Enum_Class_Alt_Decl (Node);
     Ent : Internal_Entity_Enum_Class_Alt_Decl :=
       Internal_Entity_Enum_Class_Alt_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 542 '<Cast to TypeDecl at parser.py:2371>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2371





--# expr-start 541 '<parent at parser.py:2371>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2371



--# expr-start 540 '<parent at parser.py:2371>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2371






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 540


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Liblktlang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 541



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 542

         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Enum_Class_Alt_Decl_P_Parent_Type;
--# end

   







--# property-start EnumClassAltDecl.type_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2373
pragma Warnings (Off, "is not referenced");
 function Enum_Class_Alt_Decl_P_Type_Scope
  
  (Node : Bare_Enum_Class_Alt_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Enum_Class_Alt_Decl  := Bare_Enum_Class_Alt_Decl (Node);
     Ent : Internal_Entity_Enum_Class_Alt_Decl :=
       Internal_Entity_Enum_Class_Alt_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_182 with Inline_Always;

      Fld : Internal_Entity_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_182 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_182;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 544 '<FieldAccess for type_scope at parser.py:2373>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2373



--# expr-start 543 '<FieldAccess for parent_type at parser.py:2373>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2373






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start EnumClassAltDecl.parent_type
Fld := Liblktlang.Implementation.Enum_Class_Alt_Decl_P_Parent_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 543


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.type_scope'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Type_Scope (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 544

         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_182;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_182;




            raise;
      end;



   return Property_Result;
end Enum_Class_Alt_Decl_P_Type_Scope;
--# end

   







--# property-start EnumClassAltDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2378
pragma Warnings (Off, "is not referenced");
 function Enum_Class_Alt_Decl_P_Decl_Type_Name
  
  (Node : Bare_Enum_Class_Alt_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Enum_Class_Alt_Decl  := Bare_Enum_Class_Alt_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_183 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_183 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_183;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 545 '<String at parser.py:2378>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2378
Str := Create_String ("enum class alt declaration"); 
--# expr-done 545

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_183;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_183;




            raise;
      end;



   return Property_Result;
end Enum_Class_Alt_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Function_Type
   --

   



      
      procedure Initialize_Fields_For_Function_Type
        (Self : Bare_Function_Type
        ) is
      begin

         
      Self.Function_Type_Args := No_Bare_Type_Decl_Array_Type;
      Self.Function_Type_Return_Type := No_Bare_Lkt_Node;

      end Initialize_Fields_For_Function_Type;


   







--# property-start FunctionType.full_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1942
pragma Warnings (Off, "is not referenced");
 function Function_Type_P_Full_Name
  
  (Node : Bare_Function_Type
  )

   return String_Type
is
   Self : Bare_Function_Type  := Bare_Function_Type (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_139 with Inline_Always;
            procedure Finalizer_Scope_393 with Inline_Always;

      Str : String_Type;
Str_1 : String_Type;
Fld : Bare_Type_Decl_Array_Access;
Fld_1 : String_Type;
Map_Result : String_Type_Array_Access;
Join_Result : String_Type;
Concat_Result : String_Type;
Str_2 : String_Type;
Concat_Result_1 : String_Type;
Fld_2 : Bare_Type_Decl;
Fld_3 : String_Type;
Concat_Result_2 : String_Type;

            procedure Finalizer_Scope_139 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Join_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_2);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Concat_Result_2);
            end Finalizer_Scope_139;
            procedure Finalizer_Scope_393 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_393;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 546 '<String at parser.py:1943>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1943
Str := Create_String ("("); 
--# expr-done 546
--# expr-start 551 '<Join at parser.py:1944>' Join_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1944
--# expr-start 547 '<String at parser.py:1944>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1944
Str_1 := Create_String (", "); 
--# expr-done 547
--# expr-start 550 '<Map at parser.py:1944>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1944








   

   --# expr-start 548 '<FieldAccess for args at parser.py:1944>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1944






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Function_Type_Args;
Inc_Ref (Fld);
--# expr-done 548

   
      declare
         Map_Result_Vec : String_Type_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Bare_Type_Decl_Array_Access := Fld;
      begin
         for Item_29 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind t Item_29

            
         
         
      --# expr-start 549 '<FieldAccess for full_name at parser.py:1944>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1944






   

      if Item_29 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_1 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Item_29);
--# end
--# expr-done 549
      

         declare
            Item_To_Append : constant String_Type := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_393;



         end loop;
      end;
   

   

         Map_Result := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := String_Type_Vectors.Get
              (Map_Result_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 550
Join_Result := Join_Strings (Str_1, Map_Result); 
--# expr-done 551
Concat_Result := Concat_String (Str, Join_Result); 
--# expr-start 552 '<String at parser.py:1945>' Str_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1945
Str_2 := Create_String (") -> "); 
--# expr-done 552
Concat_Result_1 := Concat_String (Concat_Result, Str_2); 
--# expr-start 554 '<FieldAccess for full_name at parser.py:1945>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1945



--# expr-start 553 '<FieldAccess for return_type at parser.py:1945>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1945






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Self.Function_Type_Return_Type;
--# expr-done 553


   

      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_3 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Fld_2);
--# end
--# expr-done 554
Concat_Result_2 := Concat_String (Concat_Result_1, Fld_3); 

         Property_Result := Concat_Result_2;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_139;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_139;
                     Finalizer_Scope_393;




            raise;
      end;



   return Property_Result;
end Function_Type_P_Full_Name;
--# end

   







--# property-start FunctionType.type_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1948
pragma Warnings (Off, "is not referenced");
 function Function_Type_P_Type_Scope
  
  (Node : Bare_Function_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Function_Type  := Bare_Function_Type (Node);
      --# bind self Self

   


   Property_Result : Lexical_Env;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := Empty_Env;
            Inc_Ref (Property_Result);
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Function_Type_P_Type_Scope;
--# end

   







--# property-start FunctionType.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1950
pragma Warnings (Off, "is not referenced");
 function Function_Type_P_Decl_Type_Name
  
  (Node : Bare_Function_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Function_Type  := Bare_Function_Type (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_141 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_141 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_141;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 555 '<String at parser.py:1950>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1950
Str := Create_String ("function type"); 
--# expr-done 555

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_141;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_141;




            raise;
      end;



   return Property_Result;
end Function_Type_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Generic_Formal_Type_Decl
   --

   



      
      procedure Initialize_Fields_For_Generic_Formal_Type_Decl
        (Self : Bare_Generic_Formal_Type_Decl
         ; Generic_Formal_Type_Decl_F_Has_Class : Bare_Class_Qualifier
         ; Generic_Formal_Type_Decl_F_Syn_Name : Bare_Def_Id
        ) is
      begin

            Self.Generic_Formal_Type_Decl_F_Has_Class := Generic_Formal_Type_Decl_F_Has_Class;
            Self.Generic_Formal_Type_Decl_F_Syn_Name := Generic_Formal_Type_Decl_F_Syn_Name;
         

      end Initialize_Fields_For_Generic_Formal_Type_Decl;

      
   function Generic_Formal_Type_Decl_F_Has_Class
     (Node : Bare_Generic_Formal_Type_Decl) return Bare_Class_Qualifier
   is
      

   begin
         
         return Node.Generic_Formal_Type_Decl_F_Has_Class;
      
   end;


   







--# property-start GenericFormalTypeDecl.type_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1969
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Type_Decl_P_Type_Scope
  
  (Node : Bare_Generic_Formal_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Generic_Formal_Type_Decl  := Bare_Generic_Formal_Type_Decl (Node);
      --# bind self Self

   


   Property_Result : Lexical_Env;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := Empty_Env;
            Inc_Ref (Property_Result);
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Generic_Formal_Type_Decl_P_Type_Scope;
--# end

   







--# property-start GenericFormalTypeDecl.generic_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1971
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Type_Decl_P_Generic_Decl
  
  (Node : Bare_Generic_Formal_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Formal_Type_Decl  := Bare_Generic_Formal_Type_Decl (Node);
     Ent : Internal_Entity_Generic_Formal_Type_Decl :=
       Internal_Entity_Generic_Formal_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Generic_Decl;

      

      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 559 '<Cast to GenericDecl at parser.py:1972>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1972





--# expr-start 558 '<parent at parser.py:1972>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1972



--# expr-start 557 '<parent at parser.py:1972>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1972



--# expr-start 556 '<parent at parser.py:1972>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1972






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 556


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Liblktlang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 557


   

      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Liblktlang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 558



   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Lkt_Generic_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 559

         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Generic_Formal_Type_Decl_P_Generic_Decl;
--# end

   







--# property-start GenericFormalTypeDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1975
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Type_Decl_P_Decl_Type_Name
  
  (Node : Bare_Generic_Formal_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Generic_Formal_Type_Decl  := Bare_Generic_Formal_Type_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_147 with Inline_Always;

      Str : String_Type;
Str_1 : String_Type;
Fld : Bare_Class_Qualifier;
Fld_1 : Boolean;
If_Result : String_Type;
Let_Result : String_Type;

            procedure Finalizer_Scope_147 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Str_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_147;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 565 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 564 '<If at parser.py:1977>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1977



--# expr-start 561 '<FieldAccess for as_bool at parser.py:1978>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1978



--# expr-start 560 '<FieldAccess for has_class at parser.py:1978>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1978






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Generic_Formal_Type_Decl_F_Has_Class;
--# expr-done 560


   

      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]ClassQualifier.as_bool'
Fld_1 := Liblktlang.Implementation.Dispatcher_Class_Qualifier_P_As_Bool (Node => Fld);
--# end
--# expr-done 561
if Fld_1 then
   --# expr-start 562 '<String at parser.py:1979>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1979
Str := Create_String ("generic formal class type declaration"); 
--# expr-done 562
   If_Result := Str;
else
   --# expr-start 563 '<String at parser.py:1980>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1980
Str_1 := Create_String ("generic formal type declaration"); 
--# expr-done 563
   If_Result := Str_1;
end if;

      Inc_Ref (If_Result);


--# expr-done 564
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 565

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_147;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_147;




            raise;
      end;



   return Property_Result;
end Generic_Formal_Type_Decl_P_Decl_Type_Name;
--# end



   


      

   --
   --  Primitives for Bare_Instantiated_Generic_Type
   --

   



      
      procedure Initialize_Fields_For_Instantiated_Generic_Type
        (Self : Bare_Instantiated_Generic_Type
        ) is
      begin

         
      Self.Instantiated_Generic_Type_Inner_Type_Decl := No_Bare_Lkt_Node;
      Self.Instantiated_Generic_Type_Actuals := No_Bare_Type_Decl_Array_Type;

      end Initialize_Fields_For_Instantiated_Generic_Type;


   







--# property-start InstantiatedGenericType.name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2158
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Name
  
  (Node : Bare_Instantiated_Generic_Type
  )

   return Symbol_Type
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
      --# bind self Self

   


   Property_Result : Symbol_Type;

      

      Fld : Bare_Type_Decl;
Fld_1 : Symbol_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 567 '<FieldAccess for name at parser.py:2158>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2158



--# expr-start 566 '<FieldAccess for inner_type_decl at parser.py:2158>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2158






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Instantiated_Generic_Type_Inner_Type_Decl;
--# expr-done 566


   

      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld_1 := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Fld);
--# end
--# expr-done 567

         Property_Result := Fld_1;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Instantiated_Generic_Type_P_Name;
--# end

   







--# property-start InstantiatedGenericType.full_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2160
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Full_Name
  
  (Node : Bare_Instantiated_Generic_Type
  )

   return String_Type
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_161 with Inline_Always;
            procedure Finalizer_Scope_394 with Inline_Always;

      Fld : Symbol_Type;
Fld_1 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Fld_2 : Bare_Type_Decl_Array_Access;
Fld_3 : String_Type;
Map_Result : String_Type_Array_Access;
Join_Result : String_Type;
Concat_Result_1 : String_Type;
Str_2 : String_Type;
Concat_Result_2 : String_Type;

            procedure Finalizer_Scope_161 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Join_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Str_2);
                     Dec_Ref (Concat_Result_2);
            end Finalizer_Scope_161;
            procedure Finalizer_Scope_394 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_394;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 569 '<FieldAccess for image at parser.py:2161>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2161



--# expr-start 568 '<FieldAccess for name at parser.py:2161>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2161






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Self);
--# end
--# expr-done 568


   




Fld_1 := Liblktlang.Implementation.Image (Self => Fld);
--# expr-done 569
--# expr-start 570 '<String at parser.py:2161>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2161
Str := Create_String ("["); 
--# expr-done 570
Concat_Result := Concat_String (Fld_1, Str); 
--# expr-start 575 '<Join at parser.py:2162>' Join_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2162
--# expr-start 571 '<String at parser.py:2162>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2162
Str_1 := Create_String (", "); 
--# expr-done 571
--# expr-start 574 '<Map at parser.py:2162>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2162








   

   --# expr-start 572 '<FieldAccess for actuals at parser.py:2162>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2162






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Self.Instantiated_Generic_Type_Actuals;
Inc_Ref (Fld_2);
--# expr-done 572

   
      declare
         Map_Result_Vec : String_Type_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Bare_Type_Decl_Array_Access := Fld_2;
      begin
         for Item_30 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind t Item_30

            
         
         
      --# expr-start 573 '<FieldAccess for full_name at parser.py:2162>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2162






   

      if Item_30 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_3 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Item_30);
--# end
--# expr-done 573
      

         declare
            Item_To_Append : constant String_Type := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_394;



         end loop;
      end;
   

   

         Map_Result := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := String_Type_Vectors.Get
              (Map_Result_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 574
Join_Result := Join_Strings (Str_1, Map_Result); 
--# expr-done 575
Concat_Result_1 := Concat_String (Concat_Result, Join_Result); 
--# expr-start 576 '<String at parser.py:2163>' Str_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2163
Str_2 := Create_String ("]"); 
--# expr-done 576
Concat_Result_2 := Concat_String (Concat_Result_1, Str_2); 

         Property_Result := Concat_Result_2;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_161;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_161;
                     Finalizer_Scope_394;




            raise;
      end;



   return Property_Result;
end Instantiated_Generic_Type_P_Full_Name;
--# end

   







--# property-start InstantiatedGenericType.generic_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2166
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Generic_Decl
  
  (Node : Bare_Instantiated_Generic_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Generic_Decl
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
     Ent : Internal_Entity_Instantiated_Generic_Type :=
       Internal_Entity_Instantiated_Generic_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Generic_Decl;

      

      Fld : Bare_Type_Decl;
Fld_1 : Bare_Lkt_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 580 '<Cast to GenericDecl at parser.py:2167>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2167





--# expr-start 579 '<as_entity at parser.py:2167>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2167



--# expr-start 578 '<parent at parser.py:2167>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2167



--# expr-start 577 '<FieldAccess for inner_type_decl at parser.py:2167>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2167






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Ent.Node.Instantiated_Generic_Type_Inner_Type_Decl;
--# expr-done 577


   

      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Fld.Parent; 
--# expr-done 578
Is_Null := Fld_1 = null; 
if Is_Null then
   
   If_Result := No_Entity;
else
   

As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;



--# expr-done 579



   if If_Result.Node = null
      or else If_Result.Node.Kind in Lkt_Generic_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 580

         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Instantiated_Generic_Type_P_Generic_Decl;
--# end

   







--# property-start InstantiatedGenericType.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2170
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Decl_Type_Name
  
  (Node : Bare_Instantiated_Generic_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_163 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_163 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_163;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 581 '<String at parser.py:2170>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2170
Str := Create_String ("instantiated generic type"); 
--# expr-done 581

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_163;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_163;




            raise;
      end;



   return Property_Result;
end Instantiated_Generic_Type_P_Decl_Type_Name;
--# end

   







--# property-start InstantiatedGenericType.base_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2172
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Base_Type
  
  (Node : Bare_Instantiated_Generic_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Ref
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
     Ent : Internal_Entity_Instantiated_Generic_Type :=
       Internal_Entity_Instantiated_Generic_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Ref;

      

      Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Type_Ref;
Let_Result : Internal_Entity_Type_Ref;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 584 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 583 '<FieldAccess for base_type at parser.py:2174>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2174



--# expr-start 582 '<FieldAccess for get_instantiated_type at parser.py:2174>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2174






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start InstantiatedGenericType.get_instantiated_type
Fld := Liblktlang.Implementation.Instantiated_Generic_Type_P_Get_Instantiated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 582


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.base_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Base_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 583
Let_Result := Fld_1; 
--# end
--# expr-done 584

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Instantiated_Generic_Type_P_Base_Type;
--# end

   







--# property-start InstantiatedGenericType.get_inner_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2176
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Get_Inner_Type
  
  (Node : Bare_Instantiated_Generic_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Bare_Type_Decl
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
     Ent : Internal_Entity_Instantiated_Generic_Type :=
       Internal_Entity_Instantiated_Generic_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Bare_Type_Decl;

      

      Fld : Bare_Type_Decl;
Let_Result : Bare_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 586 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 585 '<FieldAccess for inner_type_decl at parser.py:2181>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2181






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Ent.Node.Instantiated_Generic_Type_Inner_Type_Decl;
--# expr-done 585
Let_Result := Fld; 
--# end
--# expr-done 586

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Instantiated_Generic_Type_P_Get_Inner_Type;
--# end

   







--# property-start InstantiatedGenericType.get_actuals /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2183
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Get_Actuals
  
  (Node : Bare_Instantiated_Generic_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
     Ent : Internal_Entity_Instantiated_Generic_Type :=
       Internal_Entity_Instantiated_Generic_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl_Array_Access;

      
            procedure Finalizer_Scope_166 with Inline_Always;

      Fld : Bare_Type_Decl_Array_Access;
As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Map_Result : Internal_Entity_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Type_Decl_Array_Access;

            procedure Finalizer_Scope_166 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_166;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 590 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 589 '<Map at parser.py:2189>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2189








   

   --# expr-start 587 '<FieldAccess for actuals at parser.py:2189>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2189






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Ent.Node.Instantiated_Generic_Type_Actuals;
Inc_Ref (Fld);
--# expr-done 587

   
      declare
         Map_Result_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Bare_Type_Decl_Array_Access := Fld;
      begin
         for Item_31 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind a Item_31

            
         
         
      --# expr-start 588 '<as_entity at parser.py:2189>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2189




Is_Null := Item_31 = null; 
if Is_Null then
   
   If_Result := No_Entity_Type_Decl;
else
   

As_Entity := (Info => E_Info, Node => Item_31); 
   If_Result := As_Entity;
end if;



--# expr-done 588
      

         declare
            Item_To_Append : constant Internal_Entity_Type_Decl := If_Result;
         begin
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Entity_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 589
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 590

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_166;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_166;




            raise;
      end;



   return Property_Result;
end Instantiated_Generic_Type_P_Get_Actuals;
--# end

   







--# property-start InstantiatedGenericType.get_instantiated_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2191
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Get_Instantiated_Type
  
  (Node : Bare_Instantiated_Generic_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
     Ent : Internal_Entity_Instantiated_Generic_Type :=
       Internal_Entity_Instantiated_Generic_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_167 with Inline_Always;

      Fld : Bare_Type_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Bare_Type_Decl;
Fld_4 : Bare_Lkt_Node;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Generic_Decl;
Fld_7 : Bare_Type_Decl_Array_Access;
Fld_8 : Lexical_Env := Empty_Env;
Rebinding : Env_Rebindings;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_167 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_167;

      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Instantiated_Generic_Type_Instantiated_Generic_Type_P_Get_Instantiated_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Instantiated_Generic_Type,
                                As_Bare_Instantiated_Generic_Type => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Type_Decl;

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 603 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 602 '<New TypeDecl.entity at ???>' New_Struct_1 None
--# expr-start 600 '<New EntityInfo at parser.py:2199>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2199


--# expr-start 599 '<append_rebinding at parser.py:2202>' Rebinding /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2202
--# expr-start 592 '<FieldAccess for rebindings at parser.py:2202>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2202



--# expr-start 591 '<FieldAccess for info at parser.py:2202>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2202






   




Fld_1 := Ent.Info;
--# expr-done 591


   




Fld_2 := Fld_1.Rebindings;
--# expr-done 592
--# expr-start 595 '<FieldAccess for children_env at parser.py:2203>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2203



--# expr-start 594 '<parent at parser.py:2203>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2203



--# expr-start 593 '<FieldAccess for inner_type_decl at parser.py:2203>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2203






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Self.Instantiated_Generic_Type_Inner_Type_Decl;
--# expr-done 593


   

      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Fld_3.Parent; 
--# expr-done 594


   

      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Liblktlang.Implementation.Children_Env (Node => Fld_4);
--# expr-done 595
--# expr-start 598 '<FieldAccess for get_inst_env at parser.py:2204>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2204



--# expr-start 596 '<FieldAccess for generic_decl at parser.py:2204>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2204






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start InstantiatedGenericType.generic_decl
Fld_6 := Liblktlang.Implementation.Instantiated_Generic_Type_P_Generic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 596


   

      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 597 '<FieldAccess for actuals at parser.py:2204>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2204






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_7 := Self.Instantiated_Generic_Type_Actuals;
Inc_Ref (Fld_7);
--# expr-done 597
--# property-call-start GenericDecl.get_inst_env
Fld_8 := Liblktlang.Implementation.Generic_Decl_P_Get_Inst_Env (Node => Fld_6.Node, Actuals => Fld_7, E_Info => Fld_6.Info);
--# end
--# expr-done 598
Rebinding := AST_Envs.Append_Rebinding (Fld_2, Fld_5, Fld_8); 
--# expr-done 599
New_Struct := (From_Rebound => False, Md => No_Metadata, Rebindings => Rebinding); 
--# expr-done 600
--# expr-start 601 '<FieldAccess for inner_type_decl at parser.py:2198>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2198






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Instantiated_Generic_Type_Inner_Type_Decl;
--# expr-done 601
New_Struct_1 := (Info => New_Struct, Node => Fld); 
--# expr-done 602
Let_Result := New_Struct_1; 
--# end
--# expr-done 603

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_167;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_167;


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Internal_Entity_Type_Decl,
                        As_Internal_Entity_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);

         end if;
      end if;



   return Property_Result;
end Instantiated_Generic_Type_P_Get_Instantiated_Type;
--# end

   







--# property-start InstantiatedGenericType.type_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2209
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Type_Scope
  
  (Node : Bare_Instantiated_Generic_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
     Ent : Internal_Entity_Instantiated_Generic_Type :=
       Internal_Entity_Instantiated_Generic_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_168 with Inline_Always;

      Fld : Internal_Entity_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_168 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_168;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 605 '<FieldAccess for type_scope at parser.py:2209>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2209



--# expr-start 604 '<FieldAccess for get_instantiated_type at parser.py:2209>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2209






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start InstantiatedGenericType.get_instantiated_type
Fld := Liblktlang.Implementation.Instantiated_Generic_Type_P_Get_Instantiated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 604


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.type_scope'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Type_Scope (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 605

         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_168;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_168;




            raise;
      end;



   return Property_Result;
end Instantiated_Generic_Type_P_Type_Scope;
--# end

   







--# property-start InstantiatedGenericType.fields /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2211
pragma Warnings (Off, "is not referenced");
 function Instantiated_Generic_Type_P_Fields
  
  (Node : Bare_Instantiated_Generic_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Component_Decl_Array_Access
is
   Self : Bare_Instantiated_Generic_Type  := Bare_Instantiated_Generic_Type (Node);
     Ent : Internal_Entity_Instantiated_Generic_Type :=
       Internal_Entity_Instantiated_Generic_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Component_Decl_Array_Access;

      
            procedure Finalizer_Scope_169 with Inline_Always;

      Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Component_Decl_Array_Access;

            procedure Finalizer_Scope_169 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_169;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 607 '<FieldAccess for fields at parser.py:2211>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2211



--# expr-start 606 '<FieldAccess for get_instantiated_type at parser.py:2211>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2211






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start InstantiatedGenericType.get_instantiated_type
Fld := Liblktlang.Implementation.Instantiated_Generic_Type_P_Get_Instantiated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 606


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.fields'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Fields (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 607

         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_169;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_169;




            raise;
      end;



   return Property_Result;
end Instantiated_Generic_Type_P_Fields;
--# end



   


      

   --
   --  Primitives for Bare_Named_Type_Decl
   --

   




      
   function Named_Type_Decl_F_Decls
     (Node : Bare_Named_Type_Decl) return Bare_Decl_Block
   is
      

         Kind : constant Lkt_Named_Type_Decl := Node.Kind;
   begin
         case Kind is
               when Lkt_Trait_Decl =>
                     
         return Node.Trait_Decl_F_Decls;
      
               when Lkt_Enum_Type_Decl =>
                     
         return Node.Enum_Type_Decl_F_Decls;
      
               when Lkt_Struct_Decl =>
                     
         return Node.Struct_Decl_F_Decls;
      
               when Lkt_Class_Decl =>
                     
         return Node.Class_Decl_F_Decls;
      
               when Lkt_Enum_Class_Decl =>
                     
         return Node.Enum_Class_Decl_F_Decls;
      
         end case;
   end;


   







--# property-start NamedTypeDecl.type_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1991
pragma Warnings (Off, "is not referenced");
 function Named_Type_Decl_P_Type_Scope
  
  (Node : Bare_Named_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Named_Type_Decl  := Bare_Named_Type_Decl (Node);
     Ent : Internal_Entity_Named_Type_Decl :=
       Internal_Entity_Named_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_148 with Inline_Always;
            procedure Finalizer_Scope_395 with Inline_Always;

      Var_Expr_19 : Internal_Entity_Named_Type_Decl;
Fld : Internal_Entity_Decl_Block;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_148 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_148;
            procedure Finalizer_Scope_395 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_395;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         



Var_Expr_19 := Ent;





if Var_Expr_19 /= No_Entity_Named_Type_Decl then
   



   --# scope-start


   

--# expr-start 609 '<FieldAccess for children_env at parser.py:1991>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1991



--# expr-start 608 '<FieldAccess for decls at parser.py:1991>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1991






   

      if Var_Expr_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Decl_Block (Node => Implementation.Named_Type_Decl_F_Decls (Var_Expr_19.Node), Info => Var_Expr_19.Info);
--# expr-done 608


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Liblktlang.Implementation.Children_Env (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 609
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_395;


   Result_Var := Scope_Result;
else
   
   Result_Var := Empty_Env;
end if;

      Inc_Ref (Result_Var);



         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_148;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_148;
                     Finalizer_Scope_395;




            raise;
      end;



   return Property_Result;
end Named_Type_Decl_P_Type_Scope;
--# end

   







--# property-start NamedTypeDecl.fields /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1993
pragma Warnings (Off, "is not referenced");
 function Named_Type_Decl_P_Fields
  
  (Node : Bare_Named_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Component_Decl_Array_Access
is
   Self : Bare_Named_Type_Decl  := Bare_Named_Type_Decl (Node);
     Ent : Internal_Entity_Named_Type_Decl :=
       Internal_Entity_Named_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Component_Decl_Array_Access;

      
            procedure Finalizer_Scope_149 with Inline_Always;

      Fld : Internal_Entity_Decl_Block;
Fld_1 : Bare_Decl_Block;
Fld_2 : Internal_Entity_Info;
Item_32 : Internal_Entity_Full_Decl;
Bare_Item_32 : Bare_Full_Decl;
Fld_3 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Component_Decl;
As_Entity : Internal_Entity_Full_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Full_Decl;
Fld_4 : Internal_Entity_Decl;
Is_A : Boolean;
Map_Result : Internal_Entity_Component_Decl_Array_Access;

            procedure Finalizer_Scope_149 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_149;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 615 '<FilterMap at parser.py:1993>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1993








   

   --# expr-start 610 '<FieldAccess for decls at parser.py:1993>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1993






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Decl_Block (Node => Implementation.Named_Type_Decl_F_Decls (Ent.Node), Info => Ent.Info);
--# expr-done 610






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Internal_Entity_Component_Decl_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Internal_Entity_Component_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Decl_Block := Fld_1;
      begin
         for Untyped_Item_32 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_32 := Untyped_Item_32; 
                  



Is_Null := Bare_Item_32 = null; 
if Is_Null then
   
   If_Result := No_Entity_Full_Decl;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_32); 
   If_Result := As_Entity;
end if;



                  Item_32 := If_Result; 

            
   --# scope-start


               --# bind d Item_32

            
         --# expr-start 612 '<IsA ComponentDecl at parser.py:1995>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1995
--# expr-start 611 '<FieldAccess for decl at parser.py:1995>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1995






   

      if Item_32.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Create_Internal_Entity_Decl (Node => Item_32.Node.Full_Decl_F_Decl, Info => Item_32.Info);
--# expr-done 611
Is_A := Fld_4.Node /= null 
and then Fld_4.Node.Kind in Lkt_Component_Decl; 
--# expr-done 612
         if Is_A then
            
         
      --# expr-start 614 '<Cast to ComponentDecl at parser.py:1994>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1994





--# expr-start 613 '<FieldAccess for decl at parser.py:1994>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1994






   

      if Item_32.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Decl (Node => Item_32.Node.Full_Decl_F_Decl, Info => Item_32.Info);
--# expr-done 613



   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Lkt_Component_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Component_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 614
      

         declare
            Item_To_Append : constant Internal_Entity_Component_Decl := Cast_Result;
         begin
            Internal_Entity_Component_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Entity_Component_Decl_Array
           (Items_Count => Natural (Internal_Entity_Component_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Component_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Component_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Component_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 615

         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_149;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_149;




            raise;
      end;



   return Property_Result;
end Named_Type_Decl_P_Fields;
--# end

   







--# property-start '[dispatcher]NamedTypeDecl.is_authorized_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Named_Type_Decl_P_Is_Authorized_Decl
  
  (Node : Bare_Named_Type_Decl
      ; Decl : Internal_Entity_Decl
  )

   return Boolean
is
   Self : Bare_Named_Type_Decl  := Bare_Named_Type_Decl (Node);
      --# bind self Self

   

   --# bind decl Decl

   Property_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Named_Type_Decl (Self.Kind) is
               when Lkt_Class_Decl =>
                  --# property-call-start ClassDecl.is_authorized_decl
                  Property_Result := Class_Decl_P_Is_Authorized_Decl
                    (Self
                        , Decl
                    );
                  --# end
               when Lkt_Enum_Class_Decl =>
                  --# property-call-start EnumClassDecl.is_authorized_decl
                  Property_Result := Enum_Class_Decl_P_Is_Authorized_Decl
                    (Self
                        , Decl
                    );
                  --# end
               when Lkt_Enum_Type_Decl =>
                  --# property-call-start EnumTypeDecl.is_authorized_decl
                  Property_Result := Enum_Type_Decl_P_Is_Authorized_Decl
                    (Self
                        , Decl
                    );
                  --# end
               when Lkt_Struct_Decl =>
                  --# property-call-start StructDecl.is_authorized_decl
                  Property_Result := Struct_Decl_P_Is_Authorized_Decl
                    (Self
                        , Decl
                    );
                  --# end
               when Lkt_Trait_Decl =>
                  --# property-call-start TraitDecl.is_authorized_decl
                  Property_Result := Trait_Decl_P_Is_Authorized_Decl
                    (Self
                        , Decl
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Named_Type_Decl_P_Is_Authorized_Decl;
--# end

   







--# property-start NamedTypeDecl.check_legality /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2007
pragma Warnings (Off, "is not referenced");
 function Named_Type_Decl_P_Check_Legality
  
  (Node : Bare_Named_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Named_Type_Decl  := Bare_Named_Type_Decl (Node);
     Ent : Internal_Entity_Named_Type_Decl :=
       Internal_Entity_Named_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_151 with Inline_Always;
            procedure Finalizer_Scope_397 with Inline_Always;

      Fld : Internal_Entity_Decl_Block;
Fld_1 : Bare_Decl_Block;
Fld_2 : Internal_Entity_Info;
Item_33 : Internal_Entity_Full_Decl;
Bare_Item_33 : Bare_Full_Decl;
Fld_3 : Internal_Entity_Decl;
Fld_4 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Fld_5 : String_Type;
Concat_Result_1 : String_Type;
Fld_6 : Internal_Semantic_Result := No_Semantic_Result;
As_Entity : Internal_Entity_Full_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Full_Decl;
Fld_7 : Internal_Entity_Decl;
Fld_8 : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_151 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_151;
            procedure Finalizer_Scope_397 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_397;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 626 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 625 '<FilterMap at parser.py:2019>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2019








   

   --# expr-start 616 '<FieldAccess for decls at parser.py:2019>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2019






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Decl_Block (Node => Implementation.Named_Type_Decl_F_Decls (Ent.Node), Info => Ent.Info);
--# expr-done 616






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Internal_Semantic_Result_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Decl_Block := Fld_1;
      begin
         for Untyped_Item_33 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_33 := Untyped_Item_33; 
                  



Is_Null := Bare_Item_33 = null; 
if Is_Null then
   
   If_Result := No_Entity_Full_Decl;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_33); 
   If_Result := As_Entity;
end if;



                  Item_33 := If_Result; 

            
   --# scope-start


               --# bind d Item_33

            
         --# expr-start 619 '<Not at parser.py:2025>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2025
--# expr-start 618 '<FieldAccess for is_authorized_decl at parser.py:2025>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2025






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 617 '<FieldAccess for decl at parser.py:2025>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2025






   

      if Item_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_7 := Create_Internal_Entity_Decl (Node => Item_33.Node.Full_Decl_F_Decl, Info => Item_33.Info);
--# expr-done 617
--# property-call-start '[dispatcher]NamedTypeDecl.is_authorized_decl'
Fld_8 := Liblktlang.Implementation.Dispatcher_Named_Type_Decl_P_Is_Authorized_Decl (Node => Ent.Node, Decl => Fld_7);
--# end
--# expr-done 618
Not_Val := not (Fld_8); 
--# expr-done 619
         if Not_Val then
            
         
      --# expr-start 624 '<FieldAccess for error at parser.py:2020>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2020






   

      if Item_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 621 '<FieldAccess for decl_type_name at parser.py:2021>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2021



--# expr-start 620 '<FieldAccess for decl at parser.py:2021>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2021






   

      if Item_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Decl (Node => Item_33.Node.Full_Decl_F_Decl, Info => Item_33.Info);
--# expr-done 620


   

      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.decl_type_name'
Fld_4 := Liblktlang.Implementation.Dispatcher_Decl_P_Decl_Type_Name (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 621
--# expr-start 622 '<String at parser.py:2022>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2022
Str := Create_String (" forbidden in "); 
--# expr-done 622
Concat_Result := Concat_String (Fld_4, Str); 
--# expr-start 623 '<FieldAccess for decl_type_name at parser.py:2023>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2023






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.decl_type_name'
Fld_5 := Liblktlang.Implementation.Dispatcher_Decl_P_Decl_Type_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 623
Concat_Result_1 := Concat_String (Concat_Result, Fld_5); 
--# property-call-start LktNode.error
Fld_6 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Item_33.Node, Message => Concat_Result_1);
--# end
--# expr-done 624
      

         declare
            Item_To_Append : constant Internal_Semantic_Result := Fld_6;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end
      Finalizer_Scope_397;



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 625
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 626

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_151;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_151;
                     Finalizer_Scope_397;




            raise;
      end;



   return Property_Result;
end Named_Type_Decl_P_Check_Legality;
--# end



   


      

   --
   --  Primitives for Bare_Basic_Class_Decl
   --

   



      
      procedure Initialize_Fields_For_Basic_Class_Decl
        (Self : Bare_Basic_Class_Decl
         ; Basic_Class_Decl_F_Syn_Name : Bare_Def_Id
         ; Basic_Class_Decl_F_Syn_Base_Type : Bare_Type_Ref
         ; Basic_Class_Decl_F_Traits : Bare_Type_Ref_List
        ) is
      begin

            Self.Basic_Class_Decl_F_Syn_Name := Basic_Class_Decl_F_Syn_Name;
            Self.Basic_Class_Decl_F_Syn_Base_Type := Basic_Class_Decl_F_Syn_Base_Type;
            Self.Basic_Class_Decl_F_Traits := Basic_Class_Decl_F_Traits;
         

      end Initialize_Fields_For_Basic_Class_Decl;


   







--# property-start BasicClassDecl.check_legality /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2310
pragma Warnings (Off, "is not referenced");
 function Basic_Class_Decl_P_Check_Legality
  
  (Node : Bare_Basic_Class_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Basic_Class_Decl  := Bare_Basic_Class_Decl (Node);
     Ent : Internal_Entity_Basic_Class_Decl :=
       Internal_Entity_Basic_Class_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_176 with Inline_Always;
            procedure Finalizer_Scope_399 with Inline_Always;

      Block_Var : Internal_Semantic_Result_Array_Access;
Block_Var_1 : Internal_Entity_Type_Decl;
Var_Expr_20 : Internal_Entity_Type_Ref;
Var_Expr : Internal_Entity_Type_Decl;
Fld : Internal_Semantic_Result_Array_Access;
Fld_1 : Internal_Entity_Type_Ref;
Fld_2 : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Fld_3 : Bare_Type_Ref;
Str : String_Type;
Fld_4 : Internal_Entity_Type_Decl;
Fld_5 : String_Type;
Concat_Result : String_Type;
Fld_6 : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
Fld_7 : Boolean;
Not_Val : Boolean;
If_Result : Internal_Semantic_Result_Array_Access;
Scope_Result_1 : Internal_Semantic_Result_Array_Access;
Result_Var_1 : Internal_Semantic_Result_Array_Access;
Concat_Result_1 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_176 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_176;
            procedure Finalizer_Scope_399 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_399;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 638 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 627 '<Super at parser.py:2312>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2312






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start NamedTypeDecl.check_legality
Fld := Liblktlang.Implementation.Named_Type_Decl_P_Check_Legality (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 627
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind base_result Block_Var



--# expr-start 628 '<FieldAccess for syn_base_type at parser.py:2315>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2315






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Type_Ref (Node => Ent.Node.Basic_Class_Decl_F_Syn_Base_Type, Info => Ent.Info);
--# expr-done 628
Var_Expr_20 := Fld_1;





if Var_Expr_20 /= No_Entity_Type_Ref then
   



   --# scope-start


   

--# expr-start 629 '<FieldAccess for designated_type at parser.py:2315>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2315






   

      if Var_Expr_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_2 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Var_Expr_20.Node, E_Info => Var_Expr_20.Info);
--# end
--# expr-done 629
Scope_Result := Fld_2;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Type_Decl;
end if;



Block_Var_1 := Result_Var; 
--# bind base Block_Var_1





Var_Expr := Block_Var_1;





if Var_Expr /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind b Var_Expr

--# expr-start 637 '<If at parser.py:2318>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2318



--# expr-start 631 '<Not at parser.py:2319>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2319
--# expr-start 630 '<FieldAccess for is_class at parser.py:2319>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2319






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.is_class
Fld_7 := Liblktlang.Implementation.Type_Decl_P_Is_Class (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 630
Not_Val := not (Fld_7); 
--# expr-done 631
if Not_Val then
   --# expr-start 636 '<FieldAccess for error at parser.py:2320>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2320



--# expr-start 632 '<FieldAccess for syn_base_type at parser.py:2320>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2320






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Self.Basic_Class_Decl_F_Syn_Base_Type;
--# expr-done 632


   

      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 633 '<String at parser.py:2321>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2321
Str := Create_String ("Classes can derive from classes only, but got "); 
--# expr-done 633
--# expr-start 635 '<FieldAccess for decl_type_name at parser.py:2322>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2322



--# expr-start 634 '<FieldAccess for ultimate_instantiated_type at parser.py:2322>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2322






   

      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.ultimate_instantiated_type
Fld_4 := Liblktlang.Implementation.Type_Decl_P_Ultimate_Instantiated_Type (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 634


   

      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.decl_type_name'
Fld_5 := Liblktlang.Implementation.Dispatcher_Decl_P_Decl_Type_Name (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 635
Concat_Result := Concat_String (Str, Fld_5); 
--# property-call-start LktNode.error
Fld_6 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Fld_3, Message => Concat_Result);
--# end
--# expr-done 636
                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_6;
                Inc_Ref (Singleton.Items (1));
            
   If_Result := Singleton;
else
   
   If_Result := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (If_Result);


--# expr-done 637
Scope_Result_1 := If_Result;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_399;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_1);


Concat_Result_1 := Concat (Block_Var, Result_Var_1); 
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 638

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_176;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_176;
                     Finalizer_Scope_399;




            raise;
      end;



   return Property_Result;
end Basic_Class_Decl_P_Check_Legality;
--# end



   


      

   --
   --  Primitives for Bare_Class_Decl
   --

   



      
      procedure Initialize_Fields_For_Class_Decl
        (Self : Bare_Class_Decl
         ; Basic_Class_Decl_F_Syn_Name : Bare_Def_Id
         ; Basic_Class_Decl_F_Syn_Base_Type : Bare_Type_Ref
         ; Basic_Class_Decl_F_Traits : Bare_Type_Ref_List
         ; Class_Decl_F_Decls : Bare_Decl_Block
        ) is
      begin
            Initialize_Fields_For_Basic_Class_Decl
              (Self, Basic_Class_Decl_F_Syn_Name, Basic_Class_Decl_F_Syn_Base_Type, Basic_Class_Decl_F_Traits);

            Self.Class_Decl_F_Decls := Class_Decl_F_Decls;
         

      end Initialize_Fields_For_Class_Decl;


   







--# property-start ClassDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2336
pragma Warnings (Off, "is not referenced");
 function Class_Decl_P_Decl_Type_Name
  
  (Node : Bare_Class_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Class_Decl  := Bare_Class_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_177 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_177 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_177;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 639 '<String at parser.py:2336>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2336
Str := Create_String ("class declaration"); 
--# expr-done 639

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_177;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_177;




            raise;
      end;



   return Property_Result;
end Class_Decl_P_Decl_Type_Name;
--# end

   







--# property-start ClassDecl.is_subtype /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2338
pragma Warnings (Off, "is not referenced");
 function Class_Decl_P_Is_Subtype
  
  (Node : Bare_Class_Decl
      ; Potential_Base : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Class_Decl  := Bare_Class_Decl (Node);
     Ent : Internal_Entity_Class_Decl :=
       Internal_Entity_Class_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind potential_base Potential_Base

   Property_Result : Boolean;

      
            procedure Finalizer_Scope_178 with Inline_Always;

      Fld : Internal_Entity_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Find_Result : Internal_Entity_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;

            procedure Finalizer_Scope_178 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_178;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 645 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 644 '<Not at parser.py:2340>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2340
--# expr-start 643 '<IsNull at parser.py:2341>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2341
--# expr-start 642 '<Find at parser.py:2341>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2341








   

   --# expr-start 640 '<FieldAccess for base_types at parser.py:2341>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2341






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.base_types
Fld := Liblktlang.Implementation.Type_Decl_P_Base_Types (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 640

   
      Find_Result := No_Entity_Type_Decl;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Fld;
      begin
         for Item_34 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind bt Item_34

            
      --# expr-start 641 '<Eq at parser.py:2341>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2341






Cast_Expr := Item_34; 



   
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);








Cast_Expr_1 := Potential_Base; 



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);


Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 641
      if Is_Equiv then
         Find_Result := Item_34; 
      end if;
   

            
   --# end


               exit when Is_Equiv;

         end loop;
      end;
   

   
   



--# expr-done 642
Is_Null := Find_Result.Node = null; 
--# expr-done 643
Not_Val := not (Is_Null); 
--# expr-done 644
Let_Result := Not_Val; 
--# end
--# expr-done 645

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_178;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_178;




            raise;
      end;



   return Property_Result;
end Class_Decl_P_Is_Subtype;
--# end

   







--# property-start ClassDecl.is_authorized_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2344
pragma Warnings (Off, "is not referenced");
 function Class_Decl_P_Is_Authorized_Decl
  
  (Node : Bare_Class_Decl
      ; Decl : Internal_Entity_Decl
  )

   return Boolean
is
   Self : Bare_Class_Decl  := Bare_Class_Decl (Node);
      --# bind self Self

   

   --# bind decl Decl

   Property_Result : Boolean;

      

      Is_A : Boolean;
Fld : Boolean;
If_Result : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 648 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 646 '<IsA FunDecl, FieldDecl at parser.py:2347>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2347

Is_A := Decl.Node /= null 
and then Decl.Node.Kind in Lkt_Fun_Decl_Range | Lkt_Field_Decl_Range; 
--# expr-done 646
if Is_A then
   
   If_Result := True;
else
   --# expr-start 647 '<FieldAccess for is_generic_fun_decl at parser.py:2347>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2347






   

      if Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.is_generic_fun_decl
Fld := Liblktlang.Implementation.Decl_P_Is_Generic_Fun_Decl (Node => Decl.Node, E_Info => Decl.Info);
--# end
--# expr-done 647
   If_Result := Fld;
end if;



Let_Result := If_Result; 
--# end
--# expr-done 648

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Class_Decl_P_Is_Authorized_Decl;
--# end



   


      

   --
   --  Primitives for Bare_Enum_Class_Decl
   --

   



   

   

   

   

   

   

   


      procedure Enum_Class_Decl_Pre_Env_Actions
        (Self            : Bare_Enum_Class_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mapping : Internal_Env_Assoc :=
            Liblktlang.Implementation.Env_Mappings_15 (Node => 
Self);
      begin

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:2403");
         Dec_Ref (Mapping.Dest_Env);

      end;
   
   
      end;

      procedure Enum_Class_Decl_Post_Env_Actions
        (Self : Bare_Enum_Class_Decl; State : in out PLE_Node_State) is
      begin
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mappings : Internal_Env_Assoc_Array_Access :=
            Liblktlang.Implementation.Env_Mappings_16 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:2405");

         end loop;
         Dec_Ref (Mappings);
      end;
   
   
      end;


      
      procedure Initialize_Fields_For_Enum_Class_Decl
        (Self : Bare_Enum_Class_Decl
         ; Basic_Class_Decl_F_Syn_Name : Bare_Def_Id
         ; Basic_Class_Decl_F_Syn_Base_Type : Bare_Type_Ref
         ; Basic_Class_Decl_F_Traits : Bare_Type_Ref_List
         ; Enum_Class_Decl_F_Branches : Bare_Enum_Class_Case_List
         ; Enum_Class_Decl_F_Decls : Bare_Decl_Block
        ) is
      begin
            Initialize_Fields_For_Basic_Class_Decl
              (Self, Basic_Class_Decl_F_Syn_Name, Basic_Class_Decl_F_Syn_Base_Type, Basic_Class_Decl_F_Traits);

            Self.Enum_Class_Decl_F_Branches := Enum_Class_Decl_F_Branches;
            Self.Enum_Class_Decl_F_Decls := Enum_Class_Decl_F_Decls;
         

      end Initialize_Fields_For_Enum_Class_Decl;

      
   function Enum_Class_Decl_F_Branches
     (Node : Bare_Enum_Class_Decl) return Bare_Enum_Class_Case_List
   is
      

   begin
         
         return Node.Enum_Class_Decl_F_Branches;
      
   end;


   







--# property-start EnumClassDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2396
pragma Warnings (Off, "is not referenced");
 function Enum_Class_Decl_P_Decl_Type_Name
  
  (Node : Bare_Enum_Class_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Enum_Class_Decl  := Bare_Enum_Class_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_184 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_184 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_184;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 649 '<String at parser.py:2396>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2396
Str := Create_String ("enum class declaration"); 
--# expr-done 649

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_184;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_184;




            raise;
      end;



   return Property_Result;
end Enum_Class_Decl_P_Decl_Type_Name;
--# end

   







--# property-start EnumClassDecl.alts /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2398
pragma Warnings (Off, "is not referenced");
 function Enum_Class_Decl_P_Alts
  
  (Node : Bare_Enum_Class_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Enum_Class_Alt_Decl_Array_Access
is
   Self : Bare_Enum_Class_Decl  := Bare_Enum_Class_Decl (Node);
     Ent : Internal_Entity_Enum_Class_Decl :=
       Internal_Entity_Enum_Class_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Enum_Class_Alt_Decl_Array_Access;

      
            procedure Finalizer_Scope_185 with Inline_Always;
            procedure Finalizer_Scope_401 with Inline_Always;

      Fld : Internal_Entity_Enum_Class_Case_List;
Fld_1 : Bare_Enum_Class_Case_List;
Fld_2 : Internal_Entity_Info;
Item_35 : Internal_Entity_Enum_Class_Case;
Bare_Item_35 : Bare_Enum_Class_Case;
Fld_3 : Internal_Entity_Enum_Class_Alt_Decl_List;
Fld_4 : Bare_Enum_Class_Alt_Decl_List;
Fld_5 : Internal_Entity_Info;
Item_36 : Internal_Entity_Enum_Class_Alt_Decl;
Bare_Item_36 : Bare_Enum_Class_Alt_Decl;
As_Entity : Internal_Entity_Enum_Class_Alt_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Class_Alt_Decl;
Map_Result : Internal_Entity_Enum_Class_Alt_Decl_Array_Access;
As_Entity_1 : Internal_Entity_Enum_Class_Case;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Enum_Class_Case;
Map_Result_1 : Internal_Entity_Enum_Class_Alt_Decl_Array_Access;

            procedure Finalizer_Scope_185 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_185;
            procedure Finalizer_Scope_401 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_401;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 653 '<Mapcat at parser.py:2398>' Map_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2398








   

   --# expr-start 650 '<FieldAccess for branches at parser.py:2398>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2398






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Enum_Class_Case_List (Node => Ent.Node.Enum_Class_Decl_F_Branches, Info => Ent.Info);
--# expr-done 650






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_1_Vec : Internal_Entity_Enum_Class_Alt_Decl_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result_1 := Create_Internal_Entity_Enum_Class_Alt_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Enum_Class_Case_List := Fld_1;
      begin
         for Untyped_Item_35 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_35 := Untyped_Item_35; 
                  



Is_Null_1 := Bare_Item_35 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Enum_Class_Case;
else
   





   




Fld_2 := Fld.Info;

As_Entity_1 := (Info => Fld_2, Node => Bare_Item_35); 
   If_Result_1 := As_Entity_1;
end if;



                  Item_35 := If_Result_1; 

            
   --# scope-start


               --# bind branch Item_35

            
         
         
      --# expr-start 652 '<Map at parser.py:2399>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2399








   

   --# expr-start 651 '<FieldAccess for decls at parser.py:2399>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2399






   

      if Item_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Enum_Class_Alt_Decl_List (Node => Item_35.Node.Enum_Class_Case_F_Decls, Info => Item_35.Info);
--# expr-done 651






   




Fld_4 := Fld_3.Node;

   
      declare
         Map_Result_Vec : Internal_Entity_Enum_Class_Alt_Decl_Vectors.Vector;
      begin
   

   

      if Fld_4 = null then
         
      Map_Result := Create_Internal_Entity_Enum_Class_Alt_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Enum_Class_Alt_Decl_List := Fld_4;
      begin
         for Untyped_Item_36 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_36 := Untyped_Item_36; 
                  



Is_Null := Bare_Item_36 = null; 
if Is_Null then
   
   If_Result := No_Entity_Enum_Class_Alt_Decl;
else
   





   




Fld_5 := Fld_3.Info;

As_Entity := (Info => Fld_5, Node => Bare_Item_36); 
   If_Result := As_Entity;
end if;



                  Item_36 := If_Result; 

            
   --# scope-start


               --# bind d Item_36

            
         
         
      
      

         declare
            Item_To_Append : constant Internal_Entity_Enum_Class_Alt_Decl := Item_36;
         begin
            Internal_Entity_Enum_Class_Alt_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Entity_Enum_Class_Alt_Decl_Array
           (Items_Count => Natural (Internal_Entity_Enum_Class_Alt_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Enum_Class_Alt_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Enum_Class_Alt_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Enum_Class_Alt_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 652
      

         for Item_To_Append of
               Map_Result.Items
         loop
            Internal_Entity_Enum_Class_Alt_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;

   
   
   

            
   --# end
      Finalizer_Scope_401;



         end loop;
      end;
   
      end if;

   

         Map_Result_1 := Create_Internal_Entity_Enum_Class_Alt_Decl_Array
           (Items_Count => Natural (Internal_Entity_Enum_Class_Alt_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Enum_Class_Alt_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Enum_Class_Alt_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Enum_Class_Alt_Decl_Vectors.Destroy (Map_Result_1_Vec);

      end;
   



--# expr-done 653

         Property_Result := Map_Result_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_185;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_185;
                     Finalizer_Scope_401;




            raise;
      end;



   return Property_Result;
end Enum_Class_Decl_P_Alts;
--# end

   







--# property-start EnumClassDecl.is_authorized_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2412
pragma Warnings (Off, "is not referenced");
 function Enum_Class_Decl_P_Is_Authorized_Decl
  
  (Node : Bare_Enum_Class_Decl
      ; Decl : Internal_Entity_Decl
  )

   return Boolean
is
   Self : Bare_Enum_Class_Decl  := Bare_Enum_Class_Decl (Node);
      --# bind self Self

   

   --# bind decl Decl

   Property_Result : Boolean;

      

      Is_A : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 655 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 654 '<IsA FunDecl at parser.py:2414>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2414

Is_A := Decl.Node /= null 
and then Decl.Node.Kind in Lkt_Fun_Decl_Range; 
--# expr-done 654
Let_Result := Is_A; 
--# end
--# expr-done 655

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Enum_Class_Decl_P_Is_Authorized_Decl;
--# end

   







--# property-start EnumClassDecl._env_mappings_15 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2403
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_15
  
  (Node : Bare_Enum_Class_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc
is
   Self : Bare_Enum_Class_Decl  := Bare_Enum_Class_Decl (Node);
     Ent : Internal_Entity_Enum_Class_Decl :=
       Internal_Entity_Enum_Class_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc;

      
            procedure Finalizer_Scope_295 with Inline_Always;

      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Enum_Class_Decl;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;

            procedure Finalizer_Scope_295 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_295;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 658 '<New EnvAssoc at parser.py:2403>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2403
--# expr-start 656 '<New DesignatedEnv at parser.py:2403>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2403



Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 656
--# expr-start 657 '<FieldAccess for name at parser.py:2403>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2403






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Ent.Node);
--# end
--# expr-done 657







Cast_Expr := Self; 



   
      Cast_Result := Cast_Expr;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 658

         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_295;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_295;




            raise;
      end;



   return Property_Result;
end Env_Mappings_15;
--# end

   







--# property-start EnumClassDecl._env_mappings_16 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2405
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_16
  
  (Node : Bare_Enum_Class_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Enum_Class_Decl  := Bare_Enum_Class_Decl (Node);
     Ent : Internal_Entity_Enum_Class_Decl :=
       Internal_Entity_Enum_Class_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc_Array_Access;

      
            procedure Finalizer_Scope_296 with Inline_Always;
            procedure Finalizer_Scope_404 with Inline_Always;

      Fld : Internal_Entity_Enum_Class_Alt_Decl_Array_Access;
Fld_1 : Symbol_Type;
Fld_2 : Bare_Enum_Class_Alt_Decl;
Fld_3 : Bare_Decl_Block;
Fld_4 : Lexical_Env := Empty_Env;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;

            procedure Finalizer_Scope_296 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_296;
            procedure Finalizer_Scope_404 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_404;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 666 '<Map at parser.py:2405>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2405








   

   --# expr-start 659 '<FieldAccess for alts at parser.py:2405>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2405






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start EnumClassDecl.alts
Fld := Liblktlang.Implementation.Enum_Class_Decl_P_Alts (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 659

   
      declare
         Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Entity_Enum_Class_Alt_Decl_Array_Access := Fld;
      begin
         for Item_37 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind alt Item_37

            
         
         
      --# expr-start 665 '<New EnvAssoc at parser.py:2405>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2405
--# expr-start 662 '<New DesignatedEnv at parser.py:2408>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2408
--# expr-start 661 '<FieldAccess for children_env at parser.py:2408>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2408



--# expr-start 660 '<FieldAccess for decls at parser.py:2408>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2408






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Self.Enum_Class_Decl_F_Decls;
--# expr-done 660


   

      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Liblktlang.Implementation.Children_Env (Node => Fld_3);
--# expr-done 661


Inc_Ref (Fld_4);
New_Struct := (Direct_Env => Fld_4, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 662
--# expr-start 663 '<FieldAccess for name at parser.py:2406>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2406






   

      if Item_37.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld_1 := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Item_37.Node);
--# end
--# expr-done 663






--# expr-start 664 '<FieldAccess for node at parser.py:2407>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2407






   




Fld_2 := Item_37.Node;
--# expr-done 664



   
      Cast_Result := Fld_2;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_1, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 665
      

         declare
            Item_To_Append : constant Internal_Env_Assoc := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_404;



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Env_Assoc_Array
           (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 666

         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_296;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_296;
                     Finalizer_Scope_404;




            raise;
      end;



   return Property_Result;
end Env_Mappings_16;
--# end



   


      

   --
   --  Primitives for Bare_Enum_Type_Decl
   --

   



   

   

   

   

   

   

   


      procedure Enum_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Enum_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mapping : Internal_Env_Assoc :=
            Liblktlang.Implementation.Env_Mappings_12 (Node => 
Self);
      begin

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:2256");
         Dec_Ref (Mapping.Dest_Env);

      end;
   
   
      end;

      procedure Enum_Type_Decl_Post_Env_Actions
        (Self : Bare_Enum_Type_Decl; State : in out PLE_Node_State) is
      begin
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mappings : Internal_Env_Assoc_Array_Access :=
            Liblktlang.Implementation.Env_Mappings_13 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:2259");

         end loop;
         Dec_Ref (Mappings);
      end;
   
   
            


   

      declare
         Resolver : constant Entity_Resolver :=
            null;

         

         Mappings : Internal_Env_Assoc_Array_Access :=
            Liblktlang.Implementation.Env_Mappings_14 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop

         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "parser.py:2267");

         end loop;
         Dec_Ref (Mappings);
      end;
   
   
      end;


      
      procedure Initialize_Fields_For_Enum_Type_Decl
        (Self : Bare_Enum_Type_Decl
         ; Enum_Type_Decl_F_Syn_Name : Bare_Def_Id
         ; Enum_Type_Decl_F_Traits : Bare_Type_Ref_List
         ; Enum_Type_Decl_F_Literals : Bare_Enum_Lit_Decl_List
         ; Enum_Type_Decl_F_Decls : Bare_Decl_Block
        ) is
      begin

            Self.Enum_Type_Decl_F_Syn_Name := Enum_Type_Decl_F_Syn_Name;
            Self.Enum_Type_Decl_F_Traits := Enum_Type_Decl_F_Traits;
            Self.Enum_Type_Decl_F_Literals := Enum_Type_Decl_F_Literals;
            Self.Enum_Type_Decl_F_Decls := Enum_Type_Decl_F_Decls;
         

      end Initialize_Fields_For_Enum_Type_Decl;

      
   function Enum_Type_Decl_F_Literals
     (Node : Bare_Enum_Type_Decl) return Bare_Enum_Lit_Decl_List
   is
      

   begin
         
         return Node.Enum_Type_Decl_F_Literals;
      
   end;


   







--# property-start EnumTypeDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2253
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Decl_P_Decl_Type_Name
  
  (Node : Bare_Enum_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Enum_Type_Decl  := Bare_Enum_Type_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_172 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_172 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_172;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 667 '<String at parser.py:2253>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2253
Str := Create_String ("enum declaration"); 
--# expr-done 667

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_172;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_172;




            raise;
      end;



   return Property_Result;
end Enum_Type_Decl_P_Decl_Type_Name;
--# end

   







--# property-start EnumTypeDecl.is_authorized_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2278
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Decl_P_Is_Authorized_Decl
  
  (Node : Bare_Enum_Type_Decl
      ; Decl : Internal_Entity_Decl
  )

   return Boolean
is
   Self : Bare_Enum_Type_Decl  := Bare_Enum_Type_Decl (Node);
      --# bind self Self

   

   --# bind decl Decl

   Property_Result : Boolean;

      

      Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 668 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := False; 
--# end
--# expr-done 668

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Enum_Type_Decl_P_Is_Authorized_Decl;
--# end

   







--# property-start EnumTypeDecl._env_mappings_12 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2256
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_12
  
  (Node : Bare_Enum_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc
is
   Self : Bare_Enum_Type_Decl  := Bare_Enum_Type_Decl (Node);
     Ent : Internal_Entity_Enum_Type_Decl :=
       Internal_Entity_Enum_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc;

      
            procedure Finalizer_Scope_292 with Inline_Always;

      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Enum_Type_Decl;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;

            procedure Finalizer_Scope_292 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_292;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 671 '<New EnvAssoc at parser.py:2256>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2256
--# expr-start 669 '<New DesignatedEnv at parser.py:2256>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2256



Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 669
--# expr-start 670 '<FieldAccess for name at parser.py:2256>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2256






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Ent.Node);
--# end
--# expr-done 670







Cast_Expr := Self; 



   
      Cast_Result := Cast_Expr;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 671

         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_292;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_292;




            raise;
      end;



   return Property_Result;
end Env_Mappings_12;
--# end

   







--# property-start EnumTypeDecl._env_mappings_13 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2259
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_13
  
  (Node : Bare_Enum_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Enum_Type_Decl  := Bare_Enum_Type_Decl (Node);
     Ent : Internal_Entity_Enum_Type_Decl :=
       Internal_Entity_Enum_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc_Array_Access;

      
            procedure Finalizer_Scope_293 with Inline_Always;
            procedure Finalizer_Scope_405 with Inline_Always;

      Fld : Internal_Entity_Enum_Lit_Decl_List;
Fld_1 : Bare_Enum_Lit_Decl_List;
Fld_2 : Internal_Entity_Info;
Item_38 : Internal_Entity_Enum_Lit_Decl;
Bare_Item_38 : Bare_Enum_Lit_Decl;
Fld_3 : Symbol_Type;
Fld_4 : Bare_Enum_Lit_Decl;
Fld_5 : Bare_Decl_Block;
Fld_6 : Lexical_Env := Empty_Env;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
As_Entity : Internal_Entity_Enum_Lit_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Lit_Decl;
Map_Result : Internal_Env_Assoc_Array_Access;

            procedure Finalizer_Scope_293 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_293;
            procedure Finalizer_Scope_405 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_405;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 679 '<Map at parser.py:2259>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2259








   

   --# expr-start 672 '<FieldAccess for literals at parser.py:2259>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2259






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Enum_Lit_Decl_List (Node => Ent.Node.Enum_Type_Decl_F_Literals, Info => Ent.Info);
--# expr-done 672






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Internal_Env_Assoc_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Enum_Lit_Decl_List := Fld_1;
      begin
         for Untyped_Item_38 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_38 := Untyped_Item_38; 
                  



Is_Null := Bare_Item_38 = null; 
if Is_Null then
   
   If_Result := No_Entity_Enum_Lit_Decl;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_38); 
   If_Result := As_Entity;
end if;



                  Item_38 := If_Result; 

            
   --# scope-start


               --# bind lit Item_38

            
         
         
      --# expr-start 678 '<New EnvAssoc at parser.py:2259>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2259
--# expr-start 675 '<New DesignatedEnv at parser.py:2262>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2262
--# expr-start 674 '<FieldAccess for children_env at parser.py:2262>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2262



--# expr-start 673 '<FieldAccess for decls at parser.py:2262>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2262






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Self.Enum_Type_Decl_F_Decls;
--# expr-done 673


   

      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_6 := Liblktlang.Implementation.Children_Env (Node => Fld_5);
--# expr-done 674


Inc_Ref (Fld_6);
New_Struct := (Direct_Env => Fld_6, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 675
--# expr-start 676 '<FieldAccess for name at parser.py:2260>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2260






   

      if Item_38.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld_3 := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Item_38.Node);
--# end
--# expr-done 676






--# expr-start 677 '<FieldAccess for node at parser.py:2261>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2261






   




Fld_4 := Item_38.Node;
--# expr-done 677



   
      Cast_Result := Fld_4;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_3, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 678
      

         declare
            Item_To_Append : constant Internal_Env_Assoc := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_405;



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Env_Assoc_Array
           (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 679

         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_293;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_293;
                     Finalizer_Scope_405;




            raise;
      end;



   return Property_Result;
end Env_Mappings_13;
--# end

   







--# property-start EnumTypeDecl._env_mappings_14 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2267
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_14
  
  (Node : Bare_Enum_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Enum_Type_Decl  := Bare_Enum_Type_Decl (Node);
     Ent : Internal_Entity_Enum_Type_Decl :=
       Internal_Entity_Enum_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Env_Assoc_Array_Access;

      
            procedure Finalizer_Scope_294 with Inline_Always;
            procedure Finalizer_Scope_406 with Inline_Always;

      Fld : Internal_Entity_Enum_Lit_Decl_List;
Fld_1 : Bare_Enum_Lit_Decl_List;
Fld_2 : Internal_Entity_Info;
Item_39 : Internal_Entity_Enum_Lit_Decl;
Bare_Item_39 : Bare_Enum_Lit_Decl;
Fld_3 : Symbol_Type;
Fld_4 : Bare_Enum_Lit_Decl;
Fld_5 : Lexical_Env := Empty_Env;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result : Bare_Lkt_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
As_Entity : Internal_Entity_Enum_Lit_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Lit_Decl;
Map_Result : Internal_Env_Assoc_Array_Access;
Fld_6 : Internal_Entity_Full_Decl;
Sym : Symbol_Type;
Fld_7 : Boolean;
If_Result_1 : Internal_Env_Assoc_Array_Access;

            procedure Finalizer_Scope_294 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_294;
            procedure Finalizer_Scope_406 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_406;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 690 '<If at parser.py:2267>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2267



--# expr-start 682 '<FieldAccess for has_annotation at parser.py:2268>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2268



--# expr-start 680 '<FieldAccess for full_decl at parser.py:2268>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2268






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.full_decl
Fld_6 := Liblktlang.Implementation.Decl_P_Full_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 680


   

      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 681 '<Symbol open at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Open); 
--# expr-done 681
--# property-call-start FullDecl.has_annotation
Fld_7 := Liblktlang.Implementation.Full_Decl_P_Has_Annotation (Node => Fld_6.Node, Name => Sym);
--# end
--# expr-done 682
if Fld_7 then
   --# expr-start 689 '<Map at parser.py:2269>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2269








   

   --# expr-start 683 '<FieldAccess for literals at parser.py:2269>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2269






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Enum_Lit_Decl_List (Node => Ent.Node.Enum_Type_Decl_F_Literals, Info => Ent.Info);
--# expr-done 683






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Internal_Env_Assoc_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Enum_Lit_Decl_List := Fld_1;
      begin
         for Untyped_Item_39 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_39 := Untyped_Item_39; 
                  



Is_Null := Bare_Item_39 = null; 
if Is_Null then
   
   If_Result := No_Entity_Enum_Lit_Decl;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_39); 
   If_Result := As_Entity;
end if;



                  Item_39 := If_Result; 

            
   --# scope-start


               --# bind lit Item_39

            
         
         
      --# expr-start 688 '<New EnvAssoc at parser.py:2269>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2269
--# expr-start 685 '<New DesignatedEnv at parser.py:2272>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2272
--# expr-start 684 '<FieldAccess for node_env at parser.py:2272>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2272






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Liblktlang.Implementation.Node_Env (Node => Self);
--# expr-done 684


Inc_Ref (Fld_5);
New_Struct := (Direct_Env => Fld_5, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 685
--# expr-start 686 '<FieldAccess for name at parser.py:2270>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2270






   

      if Item_39.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.name'
Fld_3 := Liblktlang.Implementation.Dispatcher_Decl_P_Name (Node => Item_39.Node);
--# end
--# expr-done 686






--# expr-start 687 '<FieldAccess for node at parser.py:2271>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2271






   




Fld_4 := Item_39.Node;
--# expr-done 687



   
      Cast_Result := Fld_4;


Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_3, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 688
      

         declare
            Item_To_Append : constant Internal_Env_Assoc := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_406;



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Env_Assoc_Array
           (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 689
   If_Result_1 := Map_Result;
else
   
   If_Result_1 := No_Internal_Env_Assoc_Array_Type;
end if;

      Inc_Ref (If_Result_1);


--# expr-done 690

         Property_Result := If_Result_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_294;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_294;
                     Finalizer_Scope_406;




            raise;
      end;



   return Property_Result;
end Env_Mappings_14;
--# end



   


      

   --
   --  Primitives for Bare_Struct_Decl
   --

   



      
      procedure Initialize_Fields_For_Struct_Decl
        (Self : Bare_Struct_Decl
         ; Struct_Decl_F_Syn_Name : Bare_Def_Id
         ; Struct_Decl_F_Traits : Bare_Type_Ref_List
         ; Struct_Decl_F_Decls : Bare_Decl_Block
        ) is
      begin

            Self.Struct_Decl_F_Syn_Name := Struct_Decl_F_Syn_Name;
            Self.Struct_Decl_F_Traits := Struct_Decl_F_Traits;
            Self.Struct_Decl_F_Decls := Struct_Decl_F_Decls;
         

      end Initialize_Fields_For_Struct_Decl;


   







--# property-start StructDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2294
pragma Warnings (Off, "is not referenced");
 function Struct_Decl_P_Decl_Type_Name
  
  (Node : Bare_Struct_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Struct_Decl  := Bare_Struct_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_174 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_174 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_174;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 691 '<String at parser.py:2294>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2294
Str := Create_String ("struct declaration"); 
--# expr-done 691

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_174;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_174;




            raise;
      end;



   return Property_Result;
end Struct_Decl_P_Decl_Type_Name;
--# end

   







--# property-start StructDecl.is_authorized_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2296
pragma Warnings (Off, "is not referenced");
 function Struct_Decl_P_Is_Authorized_Decl
  
  (Node : Bare_Struct_Decl
      ; Decl : Internal_Entity_Decl
  )

   return Boolean
is
   Self : Bare_Struct_Decl  := Bare_Struct_Decl (Node);
      --# bind self Self

   

   --# bind decl Decl

   Property_Result : Boolean;

      

      Is_A : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 693 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 692 '<IsA FunDecl, FieldDecl at parser.py:2298>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2298

Is_A := Decl.Node /= null 
and then Decl.Node.Kind in Lkt_Fun_Decl_Range | Lkt_Field_Decl_Range; 
--# expr-done 692
Let_Result := Is_A; 
--# end
--# expr-done 693

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Struct_Decl_P_Is_Authorized_Decl;
--# end



   


      

   --
   --  Primitives for Bare_Trait_Decl
   --

   



      
      procedure Initialize_Fields_For_Trait_Decl
        (Self : Bare_Trait_Decl
         ; Trait_Decl_F_Syn_Name : Bare_Def_Id
         ; Trait_Decl_F_Decls : Bare_Decl_Block
        ) is
      begin

            Self.Trait_Decl_F_Syn_Name := Trait_Decl_F_Syn_Name;
            Self.Trait_Decl_F_Decls := Trait_Decl_F_Decls;
         

      end Initialize_Fields_For_Trait_Decl;


   







--# property-start TraitDecl.decl_type_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2234
pragma Warnings (Off, "is not referenced");
 function Trait_Decl_P_Decl_Type_Name
  
  (Node : Bare_Trait_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return String_Type
is
   Self : Bare_Trait_Decl  := Bare_Trait_Decl (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_170 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_170 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_170;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 694 '<String at parser.py:2234>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2234
Str := Create_String ("trait declaration"); 
--# expr-done 694

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_170;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_170;




            raise;
      end;



   return Property_Result;
end Trait_Decl_P_Decl_Type_Name;
--# end

   







--# property-start TraitDecl.is_authorized_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2236
pragma Warnings (Off, "is not referenced");
 function Trait_Decl_P_Is_Authorized_Decl
  
  (Node : Bare_Trait_Decl
      ; Decl : Internal_Entity_Decl
  )

   return Boolean
is
   Self : Bare_Trait_Decl  := Bare_Trait_Decl (Node);
      --# bind self Self

   

   --# bind decl Decl

   Property_Result : Boolean;

      

      Is_A : Boolean;
Fld : Boolean;
If_Result : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 697 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 695 '<IsA FunDecl at parser.py:2239>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2239

Is_A := Decl.Node /= null 
and then Decl.Node.Kind in Lkt_Fun_Decl_Range; 
--# expr-done 695
if Is_A then
   
   If_Result := True;
else
   --# expr-start 696 '<FieldAccess for is_generic_fun_decl at parser.py:2239>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2239






   

      if Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.is_generic_fun_decl
Fld := Liblktlang.Implementation.Decl_P_Is_Generic_Fun_Decl (Node => Decl.Node, E_Info => Decl.Info);
--# end
--# expr-done 696
   If_Result := Fld;
end if;



Let_Result := If_Result; 
--# end
--# expr-done 697

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Trait_Decl_P_Is_Authorized_Decl;
--# end



   


      

   --
   --  Primitives for Bare_Decl_Annotation
   --

   



      
      procedure Initialize_Fields_For_Decl_Annotation
        (Self : Bare_Decl_Annotation
         ; Decl_Annotation_F_Name : Bare_Id
         ; Decl_Annotation_F_Params : Bare_Decl_Annotation_Params
        ) is
      begin

            Self.Decl_Annotation_F_Name := Decl_Annotation_F_Name;
            Self.Decl_Annotation_F_Params := Decl_Annotation_F_Params;
         

      end Initialize_Fields_For_Decl_Annotation;

      
   function Decl_Annotation_F_Name
     (Node : Bare_Decl_Annotation) return Bare_Id
   is
      

   begin
         
         return Node.Decl_Annotation_F_Name;
      
   end;

      
   function Decl_Annotation_F_Params
     (Node : Bare_Decl_Annotation) return Bare_Decl_Annotation_Params
   is
      

   begin
         
         return Node.Decl_Annotation_F_Params;
      
   end;




   


      

   --
   --  Primitives for Bare_Decl_Annotation_Params
   --

   



      
      procedure Initialize_Fields_For_Decl_Annotation_Params
        (Self : Bare_Decl_Annotation_Params
         ; Decl_Annotation_Params_F_Params : Bare_Param_List
        ) is
      begin

            Self.Decl_Annotation_Params_F_Params := Decl_Annotation_Params_F_Params;
         

      end Initialize_Fields_For_Decl_Annotation_Params;

      
   function Decl_Annotation_Params_F_Params
     (Node : Bare_Decl_Annotation_Params) return Bare_Param_List
   is
      

   begin
         
         return Node.Decl_Annotation_Params_F_Params;
      
   end;




   


      

   --
   --  Primitives for Bare_Elsif_Branch
   --

   



      
      procedure Initialize_Fields_For_Elsif_Branch
        (Self : Bare_Elsif_Branch
         ; Elsif_Branch_F_Cond_Expr : Bare_Expr
         ; Elsif_Branch_F_Then_Expr : Bare_Expr
        ) is
      begin

            Self.Elsif_Branch_F_Cond_Expr := Elsif_Branch_F_Cond_Expr;
            Self.Elsif_Branch_F_Then_Expr := Elsif_Branch_F_Then_Expr;
         

      end Initialize_Fields_For_Elsif_Branch;

      
   function Elsif_Branch_F_Cond_Expr
     (Node : Bare_Elsif_Branch) return Bare_Expr
   is
      

   begin
         
         return Node.Elsif_Branch_F_Cond_Expr;
      
   end;

      
   function Elsif_Branch_F_Then_Expr
     (Node : Bare_Elsif_Branch) return Bare_Expr
   is
      

   begin
         
         return Node.Elsif_Branch_F_Then_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Enum_Class_Case
   --

   



      
      procedure Initialize_Fields_For_Enum_Class_Case
        (Self : Bare_Enum_Class_Case
         ; Enum_Class_Case_F_Decls : Bare_Enum_Class_Alt_Decl_List
        ) is
      begin

            Self.Enum_Class_Case_F_Decls := Enum_Class_Case_F_Decls;
         

      end Initialize_Fields_For_Enum_Class_Case;

      
   function Enum_Class_Case_F_Decls
     (Node : Bare_Enum_Class_Case) return Bare_Enum_Class_Alt_Decl_List
   is
      

   begin
         
         return Node.Enum_Class_Case_F_Decls;
      
   end;




   


      

   --
   --  Primitives for Bare_Excludes_Null
   --

   





   







--# property-start '[dispatcher]ExcludesNull.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Excludes_Null_P_As_Bool
  
  (Node : Bare_Excludes_Null
  )

   return Boolean
is
   Self : Bare_Excludes_Null  := Bare_Excludes_Null (Node);
      --# bind self Self

   


   Property_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Excludes_Null (Self.Kind) is
               when Lkt_Excludes_Null_Absent =>
                  --# property-call-start ExcludesNull.Absent.as_bool
                  Property_Result := Excludes_Null_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Lkt_Excludes_Null_Present =>
                  --# property-call-start ExcludesNull.Present.as_bool
                  Property_Result := Excludes_Null_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Excludes_Null_P_As_Bool;
--# end



   


      

   --
   --  Primitives for Bare_Excludes_Null_Absent
   --

   





   







--# property-start ExcludesNull.Absent.as_bool /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2852
pragma Warnings (Off, "is not referenced");
 function Excludes_Null_Absent_P_As_Bool
  
  (Node : Bare_Excludes_Null_Absent
  )

   return Boolean
is
   Self : Bare_Excludes_Null_Absent  := Bare_Excludes_Null_Absent (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Excludes_Null_Absent_P_As_Bool;
--# end



   


      

   --
   --  Primitives for Bare_Excludes_Null_Present
   --

   





   







--# property-start ExcludesNull.Present.as_bool /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2852
pragma Warnings (Off, "is not referenced");
 function Excludes_Null_Present_P_As_Bool
  
  (Node : Bare_Excludes_Null_Present
  )

   return Boolean
is
   Self : Bare_Excludes_Null_Present  := Bare_Excludes_Null_Present (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := True;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Excludes_Null_Present_P_As_Bool;
--# end



   


      

   --
   --  Primitives for Bare_Expr
   --

   





   







--# property-start Expr.in_type_ref /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:800
pragma Warnings (Off, "is not referenced");
 function Expr_P_In_Type_Ref
  
  (Node : Bare_Expr
  )

   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      
            procedure Finalizer_Scope_81 with Inline_Always;

      Node_Parents : Bare_Lkt_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Lkt_Node;
Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;

            procedure Finalizer_Scope_81 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_81;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 703 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 702 '<Not at parser.py:805>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:805
--# expr-start 701 '<IsNull at parser.py:805>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:805
--# expr-start 700 '<Find at parser.py:805>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:805








   

   --# expr-start 698 '<FieldAccess for parents at parser.py:805>' Node_Parents /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:805






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Node_Parents := Parents (Self, True); 
--# expr-done 698

   
      Find_Result := No_Bare_Lkt_Node;
   

   

      

      declare
         
         Collection : constant Bare_Lkt_Node_Array_Access := Node_Parents;
      begin
         for Item_40 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind p Item_40

            
      --# expr-start 699 '<IsA TypeRef at parser.py:805>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:805

Is_A := Item_40 /= null 
and then Item_40.Kind in Lkt_Type_Ref; 
--# expr-done 699
      if Is_A then
         Find_Result := Item_40; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 700
Is_Null := Find_Result = null; 
--# expr-done 701
Not_Val := not (Is_Null); 
--# expr-done 702
Let_Result := Not_Val; 
--# end
--# expr-done 703

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_81;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_81;




            raise;
      end;



   return Property_Result;
end Expr_P_In_Type_Ref;
--# end

   







--# property-start Expr.in_decl_annotation /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:807
pragma Warnings (Off, "is not referenced");
 function Expr_P_In_Decl_Annotation
  
  (Node : Bare_Expr
  )

   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      
            procedure Finalizer_Scope_82 with Inline_Always;

      Node_Parents : Bare_Lkt_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Lkt_Node;
Is_Null : Boolean;
Not_Val : Boolean;

            procedure Finalizer_Scope_82 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_82;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 708 '<Not at parser.py:808>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:808
--# expr-start 707 '<IsNull at parser.py:809>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:809
--# expr-start 706 '<Find at parser.py:809>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:809








   

   --# expr-start 704 '<FieldAccess for parents at parser.py:808>' Node_Parents /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:808






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Node_Parents := Parents (Self, True); 
--# expr-done 704

   
      Find_Result := No_Bare_Lkt_Node;
   

   

      

      declare
         
         Collection : constant Bare_Lkt_Node_Array_Access := Node_Parents;
      begin
         for Item_41 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind p Item_41

            
      --# expr-start 705 '<IsA DeclAnnotation at parser.py:809>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:809

Is_A := Item_41 /= null 
and then Item_41.Kind in Lkt_Decl_Annotation_Range; 
--# expr-done 705
      if Is_A then
         Find_Result := Item_41; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 706
Is_Null := Find_Result = null; 
--# expr-done 707
Not_Val := not (Is_Null); 
--# expr-done 708

         Property_Result := Not_Val;
         
   --# end
      Finalizer_Scope_82;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_82;




            raise;
      end;



   return Property_Result;
end Expr_P_In_Decl_Annotation;
--# end

   







--# property-start Expr.in_grammar_rule /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:812
pragma Warnings (Off, "is not referenced");
 function Expr_P_In_Grammar_Rule
  
  (Node : Bare_Expr
  )

   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      
            procedure Finalizer_Scope_83 with Inline_Always;

      Node_Parents : Bare_Lkt_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Lkt_Node;
Is_Null : Boolean;
Not_Val : Boolean;

            procedure Finalizer_Scope_83 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_83;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 713 '<Not at parser.py:813>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:813
--# expr-start 712 '<IsNull at parser.py:814>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:814
--# expr-start 711 '<Find at parser.py:814>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:814








   

   --# expr-start 709 '<FieldAccess for parents at parser.py:813>' Node_Parents /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:813






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Node_Parents := Parents (Self, True); 
--# expr-done 709

   
      Find_Result := No_Bare_Lkt_Node;
   

   

      

      declare
         
         Collection : constant Bare_Lkt_Node_Array_Access := Node_Parents;
      begin
         for Item_42 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind p Item_42

            
      --# expr-start 710 '<IsA GrammarRuleDecl at parser.py:814>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:814

Is_A := Item_42 /= null 
and then Item_42.Kind in Lkt_Grammar_Rule_Decl_Range; 
--# expr-done 710
      if Is_A then
         Find_Result := Item_42; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 711
Is_Null := Find_Result = null; 
--# expr-done 712
Not_Val := not (Is_Null); 
--# expr-done 713

         Property_Result := Not_Val;
         
   --# end
      Finalizer_Scope_83;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_83;




            raise;
      end;



   return Property_Result;
end Expr_P_In_Grammar_Rule;
--# end

   







--# property-start Expr.designated_scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:817
pragma Warnings (Off, "is not referenced");
 function Expr_P_Designated_Scope
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_84 with Inline_Always;
            procedure Finalizer_Scope_410 with Inline_Always;
            procedure Finalizer_Scope_411 with Inline_Always;

      Block_Var : Internal_Semantic_Result := No_Semantic_Result;
Var_Expr : Internal_Entity_Type_Decl;
Var_Expr_21 : Internal_Entity_Type_Decl;
Fld : Internal_Semantic_Result := No_Semantic_Result;
Fld_1 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_3 : Internal_Semantic_Result := No_Semantic_Result;
Fld_4 : Internal_Entity_Type_Decl;
Fld_5 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Fld_6 : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_84 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_84;
            procedure Finalizer_Scope_410 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_410;
            procedure Finalizer_Scope_411 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_411;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 723 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 714 '<FieldAccess for referenced_decl at parser.py:822>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:822






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 714
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind decl Block_Var
--# expr-start 722 '<If at parser.py:823>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:823



--# expr-start 715 '<FieldAccess for has_error at parser.py:824>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:824






   




Fld_6 := Block_Var.Has_Error;
--# expr-done 715
if Fld_6 then
   
   If_Result := Empty_Env;
else
   


--# expr-start 717 '<Cast to TypeDecl at parser.py:830>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:830





--# expr-start 716 '<FieldAccess for result_decl at parser.py:830>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:830






   




Fld_1 := Block_Var.Result_Decl;
--# expr-done 716



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Cast_Result := No_Entity_Type_Decl;
   end if;


--# expr-done 717
Var_Expr := Cast_Result;





if Var_Expr /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind td Var_Expr

--# expr-start 718 '<FieldAccess for type_scope at parser.py:831>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:831






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.type_scope'
Fld_2 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Type_Scope (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 718
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_410;


   Result_Var_1 := Scope_Result;
else
   


--# expr-start 720 '<FieldAccess for result_type at parser.py:833>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:833



--# expr-start 719 '<FieldAccess for expr_type at parser.py:833>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:833






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expr_type
Fld_3 := Liblktlang.Implementation.Expr_P_Expr_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 719


   




Fld_4 := Fld_3.Result_Type;
--# expr-done 720
Var_Expr_21 := Fld_4;





if Var_Expr_21 /= No_Entity_Type_Decl then
   



   --# scope-start


   

--# expr-start 721 '<FieldAccess for type_scope at parser.py:833>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:833






   

      if Var_Expr_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.type_scope'
Fld_5 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Type_Scope (Node => Var_Expr_21.Node, E_Info => Var_Expr_21.Info);
--# end
--# expr-done 721
Scope_Result_1 := Fld_5;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_411;


   Result_Var := Scope_Result_1;
else
   
   Result_Var := Empty_Env;
end if;

      Inc_Ref (Result_Var);


   Result_Var_1 := Result_Var;
end if;

      Inc_Ref (Result_Var_1);


   If_Result := Result_Var_1;
end if;

      Inc_Ref (If_Result);


--# expr-done 722
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 723

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_84;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_84;
                     Finalizer_Scope_410;
                     Finalizer_Scope_411;




            raise;
      end;



   return Property_Result;
end Expr_P_Designated_Scope;
--# end

   







--# property-start Expr.get_array_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:837
pragma Warnings (Off, "is not referenced");
 function Expr_P_Get_Array_Type
  
  (Node : Bare_Expr
      ; Elt_Type : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Instantiated_Generic_Type
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   

   --# bind elt_type Elt_Type

   Property_Result : Internal_Entity_Instantiated_Generic_Type;

      
            procedure Finalizer_Scope_85 with Inline_Always;

      Fld : Internal_Entity_Generic_Decl;
Array_Lit : Bare_Type_Decl_Array_Access;
Fld_1 : Bare_Instantiated_Generic_Type;
As_Entity : Internal_Entity_Instantiated_Generic_Type;
Is_Null : Boolean;
If_Result : Internal_Entity_Instantiated_Generic_Type;
Let_Result : Internal_Entity_Instantiated_Generic_Type;

            procedure Finalizer_Scope_85 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_85;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 728 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 727 '<as_entity at parser.py:844>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:844



--# expr-start 726 '<FieldAccess for get_instantiated_type at parser.py:842>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:842



--# expr-start 724 '<FieldAccess for array_gen_type at parser.py:842>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:842






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.array_gen_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Array_Gen_Type (Node => Self);
--# end
--# expr-done 724


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 725 '<ArrayLiteral at ???>' Array_Lit None

Array_Lit := Create_Bare_Type_Decl_Array (Internal_Bare_Type_Decl_Array'(1 => Elt_Type)); 
--# expr-done 725
--# property-call-start GenericDecl.get_instantiated_type
Fld_1 := Liblktlang.Implementation.Generic_Decl_P_Get_Instantiated_Type (Node => Fld.Node, Actuals => Array_Lit);
--# end
--# expr-done 726
Is_Null := Fld_1 = null; 
if Is_Null then
   
   If_Result := No_Entity_Instantiated_Generic_Type;
else
   

As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;



--# expr-done 727
Let_Result := If_Result; 
--# end
--# expr-done 728

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_85;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_85;




            raise;
      end;



   return Property_Result;
end Expr_P_Get_Array_Type;
--# end

   







--# property-start Expr.expected_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:846
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expected_Type
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_416 with Inline_Always;
            procedure Finalizer_Scope_422 with Inline_Always;

      Match_0 : Internal_Entity_Fun_Decl;
Match_1 : Internal_Entity_Lambda_Expr;
Match_2 : Internal_Entity_Fun_Arg_Decl;
Match_3 : Internal_Entity_Match_Branch;
Match_4 : Internal_Entity_Bin_Op;
Match_5 : Internal_Entity_Elsif_Branch;
Match_6 : Internal_Entity_If_Expr;
Match_7 : Internal_Entity_Error_On_Null;
Match_8 : Internal_Entity_Paren_Expr;
Match_9 : Internal_Entity_Param;
Match_10 : Internal_Entity_Base_Val_Decl;
Match_11 : Internal_Entity_Expr_List;
Match_12 : Internal_Entity_Block_Expr;
Match_13 : Internal_Entity;
Var_Expr_22 : Internal_Entity_Type_Decl;
Var_Expr : Internal_Param_Match;
Match_0_1 : Internal_Entity_Array_Literal;
Match_1_1 : Internal_Entity;
Var_Expr_1 : Internal_Entity_Type_Decl;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Type_Ref;
Fld_2 : Internal_Entity_Type_Decl;
Fld_3 : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Function_Type;
Fld_4 : Bare_Type_Decl;
As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld_5 : Internal_Entity_Type_Decl;
Fld_6 : Internal_Entity;
Fld_7 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Expr;
Fld_8 : Internal_Entity_Type_Decl;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_1 : Internal_Entity_Expr;
Fld_12 : Internal_Semantic_Result := No_Semantic_Result;
Fld_13 : Internal_Entity_Type_Decl;
Fld_14 : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Fld_15 : Internal_Entity_Type_Decl;
Fld_16 : Internal_Entity_Op;
Is_A : Boolean;
If_Result_2 : Internal_Entity_Type_Decl;
Fld_17 : Internal_Entity_Named_Type_Decl;
Fld_18 : Internal_Entity;
Fld_19 : Internal_Entity;
Cast_Result_4 : Internal_Entity_If_Expr;
Fld_20 : Internal_Entity_Type_Decl;
Cast_Result_5 : Internal_Entity_Type_Decl;
Fld_21 : Internal_Entity_Expr;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_6 : Internal_Entity;
Cast_Result_7 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result_3 : Internal_Entity_Type_Decl;
Fld_22 : Internal_Entity_Named_Type_Decl;
Fld_23 : Internal_Entity_Type_Decl;
Cast_Result_8 : Internal_Entity_Type_Decl;
Fld_24 : Internal_Entity_Expr;
Cast_Expr_2 : Internal_Entity_Expr;
Cast_Result_9 : Internal_Entity;
Cast_Result_10 : Internal_Entity;
Is_Equiv_2 : Boolean;
If_Result_4 : Internal_Entity_Type_Decl;
Fld_25 : Internal_Entity_Type_Decl;
Fld_26 : Internal_Entity_Type_Decl;
Fld_27 : Internal_Entity_Call_Expr;
Fld_28 : Internal_Param_Match_Array_Access;
Fld_29 : Internal_Entity_Param;
Cast_Result_11 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Param;
Cast_Result_12 : Internal_Entity;
Is_Equiv_3 : Boolean;
Find_Result : Internal_Param_Match;
Fld_30 : Internal_Formal_Param;
Fld_31 : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Result_Var_1 : Internal_Entity_Type_Decl;
Fld_32 : Internal_Entity_Type_Decl;
Fld_33 : Internal_Entity;
Fld_34 : Internal_Entity_Type_Decl;
Scope_Result_2 : Internal_Entity_Type_Decl;
Fld_35 : Internal_Entity_Type_Decl;
Fld_36 : Internal_Entity_Type_Decl;
Result_Var_2 : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_13 : Internal_Entity_Array_Literal;
Let_Result : Internal_Entity_Type_Decl;
Scope_Result_3 : Internal_Entity_Type_Decl;
Cast_Result_14 : Internal_Entity;
Let_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_4 : Internal_Entity_Type_Decl;
Match_Result : Internal_Entity_Type_Decl;
Fld_37 : Internal_Entity_Type_Decl;
Match_Prefix_1 : Internal_Entity;
Cast_Result_15 : Internal_Entity_Fun_Decl;
Let_Result_2 : Internal_Entity_Type_Decl;
Scope_Result_5 : Internal_Entity_Type_Decl;
Cast_Result_16 : Internal_Entity_Lambda_Expr;
Let_Result_3 : Internal_Entity_Type_Decl;
Scope_Result_6 : Internal_Entity_Type_Decl;
Cast_Result_17 : Internal_Entity_Fun_Arg_Decl;
Let_Result_4 : Internal_Entity_Type_Decl;
Scope_Result_7 : Internal_Entity_Type_Decl;
Cast_Result_18 : Internal_Entity_Match_Branch;
Let_Result_5 : Internal_Entity_Type_Decl;
Scope_Result_8 : Internal_Entity_Type_Decl;
Cast_Result_19 : Internal_Entity_Bin_Op;
Let_Result_6 : Internal_Entity_Type_Decl;
Scope_Result_9 : Internal_Entity_Type_Decl;
Cast_Result_20 : Internal_Entity_Elsif_Branch;
Let_Result_7 : Internal_Entity_Type_Decl;
Scope_Result_10 : Internal_Entity_Type_Decl;
Cast_Result_21 : Internal_Entity_If_Expr;
Let_Result_8 : Internal_Entity_Type_Decl;
Scope_Result_11 : Internal_Entity_Type_Decl;
Cast_Result_22 : Internal_Entity_Error_On_Null;
Let_Result_9 : Internal_Entity_Type_Decl;
Scope_Result_12 : Internal_Entity_Type_Decl;
Cast_Result_23 : Internal_Entity_Paren_Expr;
Let_Result_10 : Internal_Entity_Type_Decl;
Scope_Result_13 : Internal_Entity_Type_Decl;
Cast_Result_24 : Internal_Entity_Param;
Let_Result_11 : Internal_Entity_Type_Decl;
Scope_Result_14 : Internal_Entity_Type_Decl;
Cast_Result_25 : Internal_Entity_Base_Val_Decl;
Let_Result_12 : Internal_Entity_Type_Decl;
Scope_Result_15 : Internal_Entity_Type_Decl;
Cast_Result_26 : Internal_Entity_Expr_List;
Let_Result_13 : Internal_Entity_Type_Decl;
Scope_Result_16 : Internal_Entity_Type_Decl;
Cast_Result_27 : Internal_Entity_Block_Expr;
Let_Result_14 : Internal_Entity_Type_Decl;
Scope_Result_17 : Internal_Entity_Type_Decl;
Cast_Result_28 : Internal_Entity;
Let_Result_15 : Internal_Entity_Type_Decl;
Scope_Result_18 : Internal_Entity_Type_Decl;
Match_Result_1 : Internal_Entity_Type_Decl;
Let_Result_16 : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_416 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_416;
            procedure Finalizer_Scope_422 is
            begin
                     Dec_Ref (Fld_28);
            end Finalizer_Scope_422;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 783 '<Block at ???>' Let_Result_16 None
--# scope-start
--# expr-start 782 '<Match at parser.py:848>' Match_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:848






--# expr-start 729 '<parent at parser.py:848>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:848






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 729


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix_1 := Fld; 

case Lkt_Lkt_Node (Match_Prefix_1.Node.Kind) is
         when Lkt_Fun_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_15 := Create_Internal_Entity_Fun_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_0 := Cast_Result_15; 
--# bind fun_decl Match_0
--# expr-start 731 '<FieldAccess for designated_type at parser.py:849>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:849



--# expr-start 730 '<FieldAccess for return_type at parser.py:849>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:849






   

      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Type_Ref (Node => Match_0.Node.Fun_Decl_F_Return_Type, Info => Match_0.Info);
--# expr-done 730


   

      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_2 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 731
Let_Result_2 := Fld_2; 
--# end
Scope_Result_5 := Let_Result_2;


   --# end


            Match_Result_1 := Scope_Result_5; 
         when Lkt_Lambda_Expr =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_16 := Create_Internal_Entity_Lambda_Expr
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_1 := Cast_Result_16; 
--# bind lbd Match_1
--# expr-start 735 '<as_bare_entity at parser.py:852>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:852



--# expr-start 734 '<FieldAccess for return_type at parser.py:852>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:852



--# expr-start 733 '<Cast to FunctionType at parser.py:851>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:851





--# expr-start 732 '<FieldAccess for expected_type at parser.py:851>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:851






   

      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_3 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 732



   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Lkt_Function_Type_Range
   then
      
      Cast_Result := Create_Internal_Entity_Function_Type
        (Node => Fld_3.Node,
         Info => Fld_3.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 733


   

      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Cast_Result.Node.Function_Type_Return_Type;
--# expr-done 734
Is_Null := Fld_4 = null; 
if Is_Null then
   
   If_Result := No_Entity_Type_Decl;
else
   

As_Entity := (Info => No_Entity_Info, Node => Fld_4); 
   If_Result := As_Entity;
end if;



--# expr-done 735
Let_Result_3 := If_Result; 
--# end
Scope_Result_6 := Let_Result_3;


   --# end


            Match_Result_1 := Scope_Result_6; 
         when Lkt_Fun_Arg_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_17 := Create_Internal_Entity_Fun_Arg_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_2 := Cast_Result_17; 
--# bind fun_arg_decl Match_2
--# expr-start 736 '<FieldAccess for get_type at parser.py:854>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:854






   

      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]BaseValDecl.get_type'
Fld_5 := Liblktlang.Implementation.Dispatcher_Base_Val_Decl_P_Get_Type (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 736
Let_Result_4 := Fld_5; 
--# end
Scope_Result_7 := Let_Result_4;


   --# end


            Match_Result_1 := Scope_Result_7; 
         when Lkt_Match_Branch =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_18 := Create_Internal_Entity_Match_Branch
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_3 := Cast_Result_18; 
--# bind match_branch Match_3
--# expr-start 740 '<FieldAccess for check_expr_type at parser.py:859>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:859



--# expr-start 739 '<Cast to Expr at parser.py:859>' Cast_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:859





--# expr-start 738 '<parent at parser.py:859>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:859



--# expr-start 737 '<parent at parser.py:859>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:859






   

      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_6 := Liblktlang.Implementation.Parent (Node => Match_3.Node, E_Info => Match_3.Info);
--# expr-done 737


   

      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_7 := Liblktlang.Implementation.Parent (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# expr-done 738



   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Lkt_Expr
   then
      
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Fld_7.Node,
         Info => Fld_7.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 739


   

      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_expr_type
Fld_8 := Liblktlang.Implementation.Expr_P_Check_Expr_Type (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 740
Let_Result_5 := Fld_8; 
--# end
Scope_Result_8 := Let_Result_5;


   --# end


            Match_Result_1 := Scope_Result_8; 
         when Lkt_Bin_Op =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_19 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_4 := Cast_Result_19; 
--# bind bin_op Match_4
--# expr-start 752 '<If at parser.py:861>' If_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:861



--# expr-start 742 '<IsA Op.Lte, Op.Gte, Op.Gt, Op.Lt, Op.Eq at parser.py:869>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:869
--# expr-start 741 '<FieldAccess for op at parser.py:869>' Fld_16 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:869






   

      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_16 := Create_Internal_Entity_Op (Node => Match_4.Node.Bin_Op_F_Op, Info => Match_4.Info);
--# expr-done 741
Is_A := Fld_16.Node /= null 
and then Fld_16.Node.Kind in Lkt_Op_Lte_Range | Lkt_Op_Gte_Range | Lkt_Op_Gt_Range | Lkt_Op_Lt_Range | Lkt_Op_Eq_Range; 
--# expr-done 742
if Is_A then
   


--# expr-start 749 '<FieldAccess for result_type at parser.py:874>' Fld_13 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:874



--# expr-start 748 '<FieldAccess for expr_type_impl at parser.py:873>' Fld_12 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:873



--# expr-start 747 '<If at parser.py:872>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:872



--# expr-start 744 '<Eq at parser.py:872>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:872






Cast_Expr := Ent; 



   
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);







--# expr-start 743 '<FieldAccess for left at parser.py:872>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:872






   

      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_11 := Create_Internal_Entity_Expr (Node => Match_4.Node.Bin_Op_F_Left, Info => Match_4.Info);
--# expr-done 743



   
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_11.Node,
         Info => Fld_11.Info);


Is_Equiv := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 744
if Is_Equiv then
   --# expr-start 745 '<FieldAccess for right at parser.py:872>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:872






   

      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_9 := Create_Internal_Entity_Expr (Node => Match_4.Node.Bin_Op_F_Right, Info => Match_4.Info);
--# expr-done 745
   If_Result_1 := Fld_9;
else
   --# expr-start 746 '<FieldAccess for left at parser.py:872>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:872






   

      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_10 := Create_Internal_Entity_Expr (Node => Match_4.Node.Bin_Op_F_Left, Info => Match_4.Info);
--# expr-done 746
   If_Result_1 := Fld_10;
end if;



--# expr-done 747


   

      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start Expr.expr_type_impl
Fld_12 := Liblktlang.Implementation.Expr_P_Expr_Type_Impl (Node => If_Result_1.Node, Expected_Type => No_Entity_Type_Decl, E_Info => If_Result_1.Info);
--# end
--# expr-done 748


   




Fld_13 := Fld_12.Result_Type;
--# expr-done 749
Var_Expr_22 := Fld_13;





if Var_Expr_22 /= No_Entity_Type_Decl then
   



   --# scope-start


   

--# expr-start 750 '<FieldAccess for root_type at parser.py:874>' Fld_14 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:874






   

      if Var_Expr_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.root_type
Fld_14 := Liblktlang.Implementation.Type_Decl_P_Root_Type (Node => Var_Expr_22.Node, E_Info => Var_Expr_22.Info);
--# end
--# expr-done 750
Scope_Result := Fld_14;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Type_Decl;
end if;



   If_Result_2 := Result_Var;
else
   --# expr-start 751 '<FieldAccess for expected_type at parser.py:878>' Fld_15 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:878






   

      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_15 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 751
   If_Result_2 := Fld_15;
end if;



--# expr-done 752
Let_Result_6 := If_Result_2; 
--# end
Scope_Result_9 := Let_Result_6;


   --# end
      Finalizer_Scope_416;


            Match_Result_1 := Scope_Result_9; 
         when Lkt_Elsif_Branch =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_20 := Create_Internal_Entity_Elsif_Branch
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_5 := Cast_Result_20; 
--# bind elsif_branch Match_5
--# expr-start 760 '<If at parser.py:882>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:882



--# expr-start 754 '<Eq at parser.py:885>' Is_Equiv_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:885






Cast_Expr_1 := Ent; 



   
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);







--# expr-start 753 '<FieldAccess for cond_expr at parser.py:885>' Fld_21 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:885






   

      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_21 := Create_Internal_Entity_Expr (Node => Match_5.Node.Elsif_Branch_F_Cond_Expr, Info => Match_5.Info);
--# expr-done 753



   
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_21.Node,
         Info => Fld_21.Info);


Is_Equiv_1 := Equivalent (Cast_Result_6, Cast_Result_7); 
--# expr-done 754
if Is_Equiv_1 then
   




--# expr-start 755 '<FieldAccess for bool_type at parser.py:886>' Fld_17 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:886






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.bool_type
Fld_17 := Liblktlang.Implementation.Lkt_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 755



   
      Cast_Result_5 := Create_Internal_Entity_Type_Decl
        (Node => Fld_17.Node,
         Info => Fld_17.Info);


   If_Result_3 := Cast_Result_5;
else
   --# expr-start 759 '<FieldAccess for if_expected_type at parser.py:891>' Fld_20 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:891



--# expr-start 758 '<Cast to IfExpr at parser.py:890>' Cast_Result_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:890





--# expr-start 757 '<parent at parser.py:890>' Fld_19 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:890



--# expr-start 756 '<parent at parser.py:890>' Fld_18 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:890






   

      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_18 := Liblktlang.Implementation.Parent (Node => Match_5.Node, E_Info => Match_5.Info);
--# expr-done 756


   

      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_19 := Liblktlang.Implementation.Parent (Node => Fld_18.Node, E_Info => Fld_18.Info);
--# expr-done 757



   if Fld_19.Node = null
      or else Fld_19.Node.Kind in Lkt_If_Expr_Range
   then
      
      Cast_Result_4 := Create_Internal_Entity_If_Expr
        (Node => Fld_19.Node,
         Info => Fld_19.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 758


   

      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start IfExpr.if_expected_type
Fld_20 := Liblktlang.Implementation.If_Expr_P_If_Expected_Type (Node => Cast_Result_4.Node, E_Info => Cast_Result_4.Info);
--# end
--# expr-done 759
   If_Result_3 := Fld_20;
end if;



--# expr-done 760
Let_Result_7 := If_Result_3; 
--# end
Scope_Result_10 := Let_Result_7;


   --# end


            Match_Result_1 := Scope_Result_10; 
         when Lkt_If_Expr =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_21 := Create_Internal_Entity_If_Expr
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_6 := Cast_Result_21; 
--# bind if_expr Match_6
--# expr-start 765 '<If at parser.py:894>' If_Result_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:894



--# expr-start 762 '<Eq at parser.py:897>' Is_Equiv_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:897






Cast_Expr_2 := Ent; 



   
      Cast_Result_9 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);







--# expr-start 761 '<FieldAccess for cond_expr at parser.py:897>' Fld_24 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:897






   

      if Match_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_24 := Create_Internal_Entity_Expr (Node => Match_6.Node.If_Expr_F_Cond_Expr, Info => Match_6.Info);
--# expr-done 761



   
      Cast_Result_10 := Create_Internal_Entity
        (Node => Fld_24.Node,
         Info => Fld_24.Info);


Is_Equiv_2 := Equivalent (Cast_Result_9, Cast_Result_10); 
--# expr-done 762
if Is_Equiv_2 then
   




--# expr-start 763 '<FieldAccess for bool_type at parser.py:898>' Fld_22 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:898






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.bool_type
Fld_22 := Liblktlang.Implementation.Lkt_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 763



   
      Cast_Result_8 := Create_Internal_Entity_Type_Decl
        (Node => Fld_22.Node,
         Info => Fld_22.Info);


   If_Result_4 := Cast_Result_8;
else
   --# expr-start 764 '<FieldAccess for if_expected_type at parser.py:902>' Fld_23 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:902






   

      if Match_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start IfExpr.if_expected_type
Fld_23 := Liblktlang.Implementation.If_Expr_P_If_Expected_Type (Node => Match_6.Node, E_Info => Match_6.Info);
--# end
--# expr-done 764
   If_Result_4 := Fld_23;
end if;



--# expr-done 765
Let_Result_8 := If_Result_4; 
--# end
Scope_Result_11 := Let_Result_8;


   --# end


            Match_Result_1 := Scope_Result_11; 
         when Lkt_Error_On_Null =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_22 := Create_Internal_Entity_Error_On_Null
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_7 := Cast_Result_22; 
--# bind err_on_null Match_7
--# expr-start 766 '<FieldAccess for expected_type at parser.py:905>' Fld_25 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:905






   

      if Match_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_25 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Match_7.Node, E_Info => Match_7.Info);
--# end
--# expr-done 766
Let_Result_9 := Fld_25; 
--# end
Scope_Result_12 := Let_Result_9;


   --# end


            Match_Result_1 := Scope_Result_12; 
         when Lkt_Paren_Expr =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_23 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_8 := Cast_Result_23; 
--# bind paren_expr Match_8
--# expr-start 767 '<FieldAccess for expected_type at parser.py:906>' Fld_26 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:906






   

      if Match_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_26 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Match_8.Node, E_Info => Match_8.Info);
--# end
--# expr-done 767
Let_Result_10 := Fld_26; 
--# end
Scope_Result_13 := Let_Result_10;


   --# end


            Match_Result_1 := Scope_Result_13; 
         when Lkt_Param =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_24 := Create_Internal_Entity_Param
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_9 := Cast_Result_24; 
--# bind p Match_9



--# expr-start 772 '<Find at parser.py:908>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:908








   

   --# expr-start 769 '<FieldAccess for match_params at parser.py:908>' Fld_28 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:908



--# expr-start 768 '<FieldAccess for call_expr at parser.py:908>' Fld_27 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:908






   

      if Match_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Param.call_expr
Fld_27 := Liblktlang.Implementation.Param_P_Call_Expr (Node => Match_9.Node, E_Info => Match_9.Info);
--# end
--# expr-done 768


   

      if Fld_27.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.match_params
Fld_28 := Liblktlang.Implementation.Call_Expr_P_Match_Params (Node => Fld_27.Node, E_Info => Fld_27.Info);
--# end
--# expr-done 769

   
      Find_Result := No_Param_Match;
   

   

      

      declare
         
         Collection : constant Internal_Param_Match_Array_Access := Fld_28;
      begin
         for Item_43 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind pm Item_43

            
      --# expr-start 771 '<Eq at parser.py:909>' Is_Equiv_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:909





--# expr-start 770 '<FieldAccess for actual at parser.py:909>' Fld_29 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:909






   




Fld_29 := Item_43.Actual;
--# expr-done 770



   
      Cast_Result_11 := Create_Internal_Entity
        (Node => Fld_29.Node,
         Info => Fld_29.Info);








Cast_Expr_3 := Match_9; 



   
      Cast_Result_12 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);


Is_Equiv_3 := Equivalent (Cast_Result_11, Cast_Result_12); 
--# expr-done 771
      if Is_Equiv_3 then
         Find_Result := Item_43; 
      end if;
   

            
   --# end


               exit when Is_Equiv_3;

         end loop;
      end;
   

   
   



--# expr-done 772
Var_Expr := Find_Result;





if Var_Expr /= No_Param_Match then
   



   --# scope-start


   --# bind pm Var_Expr

--# expr-start 774 '<FieldAccess for formal_type at parser.py:910>' Fld_31 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:910



--# expr-start 773 '<FieldAccess for formal at parser.py:910>' Fld_30 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:910






   




Fld_30 := Var_Expr.Formal;
--# expr-done 773


   




Fld_31 := Fld_30.Formal_Type;
--# expr-done 774
Scope_Result_1 := Fld_31;


   --# end


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Entity_Type_Decl;
end if;



Let_Result_11 := Result_Var_1; 
--# end
Scope_Result_14 := Let_Result_11;


   --# end
      Finalizer_Scope_422;


            Match_Result_1 := Scope_Result_14; 
         when Lkt_Node_Decl .. Lkt_Field_Decl | Lkt_Lambda_Arg_Decl .. Lkt_Val_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_25 := Create_Internal_Entity_Base_Val_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_10 := Cast_Result_25; 
--# bind val_decl Match_10
--# expr-start 775 '<FieldAccess for get_type at parser.py:912>' Fld_32 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:912






   

      if Match_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start '[dispatcher]BaseValDecl.get_type'
Fld_32 := Liblktlang.Implementation.Dispatcher_Base_Val_Decl_P_Get_Type (Node => Match_10.Node, No_Inference => True, E_Info => Match_10.Info);
--# end
--# expr-done 775
Let_Result_12 := Fld_32; 
--# end
Scope_Result_15 := Let_Result_12;


   --# end


            Match_Result_1 := Scope_Result_15; 
         when Lkt_Expr_List .. Lkt_Any_Of_List =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_26 := Create_Internal_Entity_Expr_List
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_11 := Cast_Result_26; 
--# bind expr_list Match_11
--# expr-start 780 '<Match at parser.py:916>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:916






--# expr-start 776 '<parent at parser.py:916>' Fld_33 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:916






   

      if Match_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_33 := Liblktlang.Implementation.Parent (Node => Match_11.Node, E_Info => Match_11.Info);
--# expr-done 776


   

      if Fld_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Fld_33; 

case Lkt_Lkt_Node (Match_Prefix.Node.Kind) is
         when Lkt_Array_Literal =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_13 := Create_Internal_Entity_Array_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0_1 := Cast_Result_13; 
--# bind array Match_0_1



--# expr-start 777 '<FieldAccess for first_elt_context_free_type at parser.py:918>' Fld_34 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:918






   

      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start ArrayLiteral.first_elt_context_free_type
Fld_34 := Liblktlang.Implementation.Array_Literal_P_First_Elt_Context_Free_Type (Node => Match_0_1.Node, E_Info => Match_0_1.Info);
--# end
--# expr-done 777
Var_Expr_1 := Fld_34;





if Var_Expr_1 /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind t Var_Expr_1


Scope_Result_2 := Var_Expr_1;


   --# end


   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 779 '<FieldAccess for array_element_type at parser.py:925>' Fld_36 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:925



--# expr-start 778 '<FieldAccess for expected_type at parser.py:925>' Fld_35 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:925






   

      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_35 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Match_0_1.Node, E_Info => Match_0_1.Info);
--# end
--# expr-done 778


   

      if Fld_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.array_element_type
Fld_36 := Liblktlang.Implementation.Type_Decl_P_Array_Element_Type (Node => Fld_35.Node, E_Info => Fld_35.Info);
--# end
--# expr-done 779
   Result_Var_2 := Fld_36;
end if;



Let_Result := Result_Var_2; 
--# end
Scope_Result_3 := Let_Result;


   --# end


            Match_Result := Scope_Result_3; 
         when Lkt_Lexer_Case_Rule_Cond_Alt .. Lkt_Any_Of | Lkt_Dot_Expr .. Lkt_Var_Bind =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_14 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1_1 := Cast_Result_14; 
--# bind _ Match_1_1

Let_Result_1 := No_Entity_Type_Decl; 
--# end
Scope_Result_4 := Let_Result_1;


   --# end


            Match_Result := Scope_Result_4; 
end case;

--# expr-done 780
Let_Result_13 := Match_Result; 
--# end
Scope_Result_16 := Let_Result_13;


   --# end


            Match_Result_1 := Scope_Result_16; 
         when Lkt_Block_Expr =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_27 := Create_Internal_Entity_Block_Expr
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_12 := Cast_Result_27; 
--# bind block_expr Match_12
--# expr-start 781 '<FieldAccess for expected_type at parser.py:930>' Fld_37 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:930






   

      if Match_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_37 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Match_12.Node, E_Info => Match_12.Info);
--# end
--# expr-done 781
Let_Result_14 := Fld_37; 
--# end
Scope_Result_17 := Let_Result_14;


   --# end


            Match_Result_1 := Scope_Result_17; 
         when Lkt_Lexer_Case_Rule_Cond_Alt .. Lkt_Synthetic_Lexer_Decl | Lkt_Env_Spec_Decl .. Lkt_Decl_Annotation_Params | Lkt_Enum_Class_Case .. Lkt_Null_Cond_Dotted_Name | Lkt_Call_Expr .. Lkt_Cast_Expr | Lkt_Generic_Instantiation .. Lkt_Ref_Id | Lkt_Isa .. Lkt_Keep_Expr | Lkt_Big_Num_Lit .. Lkt_Not_Expr | Lkt_Raise_Expr .. Lkt_Enum_Lit_Decl_List | Lkt_Full_Decl_List .. Lkt_Isa_List | Lkt_Op_Amp .. Lkt_Op_Plus | Lkt_Default_List_Type_Ref .. Lkt_Var_Bind =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_28 := Create_Internal_Entity
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_13 := Cast_Result_28; 
--# bind _ Match_13

Let_Result_15 := No_Entity_Type_Decl; 
--# end
Scope_Result_18 := Let_Result_15;


   --# end


            Match_Result_1 := Scope_Result_18; 
end case;

--# expr-done 782
Let_Result_16 := Match_Result_1; 
--# end
--# expr-done 783

         Property_Result := Let_Result_16;
         
   --# end


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_416;
                     Finalizer_Scope_422;




            raise;
      end;



   return Property_Result;
end Expr_P_Expected_Type;
--# end

   







--# property-start Expr.is_regular_expr /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:935
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Regular_Expr
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Boolean;

      
            procedure Finalizer_Scope_87 with Inline_Always;

      Fld : Boolean;
Is_A : Boolean;
If_Result : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
Fld_2 : Boolean;
If_Result_2 : Boolean;
Is_A_1 : Boolean;
If_Result_3 : Boolean;
Fld_3 : Internal_Semantic_Result := No_Semantic_Result;
Fld_4 : Internal_Entity_Decl;
Is_A_2 : Boolean;
If_Result_4 : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;

            procedure Finalizer_Scope_87 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_87;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 793 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 792 '<Not at parser.py:948>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:948















--# expr-start 784 '<FieldAccess for in_type_ref at parser.py:949>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:949






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.in_type_ref
Fld := Liblktlang.Implementation.Expr_P_In_Type_Ref (Node => Ent.Node);
--# end
--# expr-done 784
if Fld then
   
   If_Result := True;
else
   --# expr-start 785 '<IsA DefId at parser.py:950>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:950

Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Lkt_Def_Id_Range; 
--# expr-done 785
   If_Result := Is_A;
end if;



if If_Result then
   
   If_Result_1 := True;
else
   --# expr-start 786 '<FieldAccess for in_decl_annotation at parser.py:951>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:951






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.in_decl_annotation
Fld_1 := Liblktlang.Implementation.Expr_P_In_Decl_Annotation (Node => Ent.Node);
--# end
--# expr-done 786
   If_Result_1 := Fld_1;
end if;



if If_Result_1 then
   
   If_Result_2 := True;
else
   --# expr-start 787 '<FieldAccess for in_grammar_rule at parser.py:952>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:952






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.in_grammar_rule
Fld_2 := Liblktlang.Implementation.Expr_P_In_Grammar_Rule (Node => Ent.Node);
--# end
--# expr-done 787
   If_Result_2 := Fld_2;
end if;



if If_Result_2 then
   
   If_Result_3 := True;
else
   --# expr-start 788 '<IsA ModuleRefId at parser.py:955>' Is_A_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:955

Is_A_1 := Ent.Node /= null 
and then Ent.Node.Kind in Lkt_Module_Ref_Id_Range; 
--# expr-done 788
   If_Result_3 := Is_A_1;
end if;



if If_Result_3 then
   
   If_Result_4 := True;
else
   --# expr-start 791 '<IsA TypeDecl, GenericDecl at parser.py:958>' Is_A_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:958
--# expr-start 790 '<FieldAccess for result_decl at parser.py:958>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:958



--# expr-start 789 '<FieldAccess for referenced_decl at parser.py:958>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:958






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld_3 := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 789


   




Fld_4 := Fld_3.Result_Decl;
--# expr-done 790
Is_A_2 := Fld_4.Node /= null 
and then Fld_4.Node.Kind in Lkt_Type_Decl | Lkt_Generic_Decl_Range; 
--# expr-done 791
   If_Result_4 := Is_A_2;
end if;



Not_Val := not (If_Result_4); 
--# expr-done 792
Let_Result := Not_Val; 
--# end
--# expr-done 793

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_87;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_87;




            raise;
      end;



   return Property_Result;
end Expr_P_Is_Regular_Expr;
--# end

   







--# property-start Expr.expr_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:961
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expr_Type
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_88 with Inline_Always;

      Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;
Fld_2 : Boolean;
If_Result : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_88 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_88;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 798 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 797 '<If at parser.py:967>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:967



--# expr-start 794 '<FieldAccess for is_regular_expr at parser.py:968>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:968






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.is_regular_expr
Fld_2 := Liblktlang.Implementation.Expr_P_Is_Regular_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 794
if Fld_2 then
   --# expr-start 796 '<FieldAccess for expr_type_impl at parser.py:969>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:969






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 795 '<FieldAccess for expected_type at parser.py:969>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:969






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 795
--# property-call-start Expr.expr_type_impl
Fld_1 := Liblktlang.Implementation.Expr_P_Expr_Type_Impl (Node => Ent.Node, Expected_Type => Fld, E_Info => Ent.Info);
--# end
--# expr-done 796
   If_Result := Fld_1;
else
   
   If_Result := No_Semantic_Result;
end if;

      Inc_Ref (If_Result);


--# expr-done 797
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 798

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_88;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_88;




            raise;
      end;



   return Property_Result;
end Expr_P_Expr_Type;
--# end

   







--# property-start Expr.check_expr_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:973
pragma Warnings (Off, "is not referenced");
 function Expr_P_Check_Expr_Type
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_89 with Inline_Always;

      Var_Expr : Internal_Entity_Type_Decl;
Fld : Internal_Semantic_Result := No_Semantic_Result;
Fld_1 : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Fld_2 : Boolean;
If_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_89 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_89;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 803 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 802 '<If at parser.py:980>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:980



--# expr-start 799 '<FieldAccess for is_regular_expr at parser.py:981>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:981






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.is_regular_expr
Fld_2 := Liblktlang.Implementation.Expr_P_Is_Regular_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 799
if Fld_2 then
   


--# expr-start 801 '<FieldAccess for result_type at parser.py:982>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:982



--# expr-start 800 '<FieldAccess for expr_type at parser.py:982>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:982






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expr_type
Fld := Liblktlang.Implementation.Expr_P_Expr_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 800


   




Fld_1 := Fld.Result_Type;
--# expr-done 801
Var_Expr := Fld_1;





if Var_Expr /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind rt Var_Expr


Scope_Result := Var_Expr;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := raise Property_Error with "Typing error";
end if;



   If_Result := Result_Var;
else
   
   If_Result := raise Property_Error with "non regular expression";
end if;



--# expr-done 802
Let_Result := If_Result; 
--# end
--# expr-done 803

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_89;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_89;




            raise;
      end;



   return Property_Result;
end Expr_P_Check_Expr_Type;
--# end

   







--# property-start '[dispatcher]Expr.expr_context_free_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Expr_Context_Free_Type
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Expr (Self.Kind) is
               when Lkt_Any_Of | Lkt_Null_Cond_Dotted_Name | Lkt_Block_Expr | Lkt_Error_On_Null | Lkt_Grammar_Cut .. Lkt_Module_Ref_Id | Lkt_Isa .. Lkt_Keep_Expr | Lkt_Char_Lit .. Lkt_Null_Lit | Lkt_Block_String_Lit | Lkt_Logic_Expr .. Lkt_Not_Expr | Lkt_Raise_Expr .. Lkt_Un_Op =>
                  --# property-call-start Expr.expr_context_free_type
                  Property_Result := Expr_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Array_Literal =>
                  --# property-call-start ArrayLiteral.expr_context_free_type
                  Property_Result := Array_Literal_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Dot_Expr =>
                  --# property-call-start DotExpr.expr_context_free_type
                  Property_Result := Dot_Expr_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Bin_Op =>
                  --# property-call-start BinOp.expr_context_free_type
                  Property_Result := Bin_Op_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Call_Expr =>
                  --# property-call-start CallExpr.expr_context_free_type
                  Property_Result := Call_Expr_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Cast_Expr =>
                  --# property-call-start CastExpr.expr_context_free_type
                  Property_Result := Cast_Expr_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Generic_Instantiation =>
                  --# property-call-start GenericInstantiation.expr_context_free_type
                  Property_Result := Generic_Instantiation_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Ref_Id =>
                  --# property-call-start RefId.expr_context_free_type
                  Property_Result := Ref_Id_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_If_Expr =>
                  --# property-call-start IfExpr.expr_context_free_type
                  Property_Result := If_Expr_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Lambda_Expr =>
                  --# property-call-start LambdaExpr.expr_context_free_type
                  Property_Result := Lambda_Expr_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Big_Num_Lit =>
                  --# property-call-start BigNumLit.expr_context_free_type
                  Property_Result := Big_Num_Lit_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Num_Lit =>
                  --# property-call-start NumLit.expr_context_free_type
                  Property_Result := Num_Lit_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Single_Line_String_Lit .. Lkt_Pattern_Single_Line_String_Lit =>
                  --# property-call-start SingleLineStringLit.expr_context_free_type
                  Property_Result := Single_Line_String_Lit_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Paren_Expr =>
                  --# property-call-start ParenExpr.expr_context_free_type
                  Property_Result := Paren_Expr_P_Expr_Context_Free_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Expr_P_Expr_Context_Free_Type;
--# end

   







--# property-start '[dispatcher]Expr.expected_type_predicate' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Expected_Type_Predicate
  
  (Node : Bare_Expr
      ; Expected_Type : Internal_Entity_Type_Decl
  )

   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   

   --# bind expected_type Expected_Type

   Property_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Expr (Self.Kind) is
               when Lkt_Any_Of | Lkt_Dot_Expr .. Lkt_Keep_Expr | Lkt_Big_Num_Lit | Lkt_Null_Lit .. Lkt_Block_String_Lit | Lkt_Logic_Expr .. Lkt_Un_Op =>
                  --# property-call-start Expr.expected_type_predicate
                  Property_Result := Expr_P_Expected_Type_Predicate
                    (Self
                        , Expected_Type
                    );
                  --# end
               when Lkt_Array_Literal =>
                  --# property-call-start ArrayLiteral.expected_type_predicate
                  Property_Result := Array_Literal_P_Expected_Type_Predicate
                    (Self
                        , Expected_Type
                    );
                  --# end
               when Lkt_Lambda_Expr =>
                  --# property-call-start LambdaExpr.expected_type_predicate
                  Property_Result := Lambda_Expr_P_Expected_Type_Predicate
                    (Self
                        , Expected_Type
                    );
                  --# end
               when Lkt_Char_Lit =>
                  --# property-call-start CharLit.expected_type_predicate
                  Property_Result := Char_Lit_P_Expected_Type_Predicate
                    (Self
                        , Expected_Type
                    );
                  --# end
               when Lkt_Single_Line_String_Lit .. Lkt_Pattern_Single_Line_String_Lit =>
                  --# property-call-start SingleLineStringLit.expected_type_predicate
                  Property_Result := Single_Line_String_Lit_P_Expected_Type_Predicate
                    (Self
                        , Expected_Type
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Expr_P_Expected_Type_Predicate;
--# end

   







--# property-start '[dispatcher]Expr.invalid_expected_type_error_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Invalid_Expected_Type_Error_Name
  
  (Node : Bare_Expr
  )

   return String_Type
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   


   Property_Result : String_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Expr (Self.Kind) is
               when Lkt_Any_Of | Lkt_Dot_Expr .. Lkt_Keep_Expr | Lkt_Null_Lit | Lkt_Block_String_Lit | Lkt_Logic_Expr .. Lkt_Un_Op =>
                  --# property-call-start Expr.invalid_expected_type_error_name
                  Property_Result := Expr_P_Invalid_Expected_Type_Error_Name
                    (Self
                    );
                  --# end
               when Lkt_Array_Literal =>
                  --# property-call-start ArrayLiteral.invalid_expected_type_error_name
                  Property_Result := Array_Literal_P_Invalid_Expected_Type_Error_Name
                    (Self
                    );
                  --# end
               when Lkt_Lambda_Expr =>
                  --# property-call-start LambdaExpr.invalid_expected_type_error_name
                  Property_Result := Lambda_Expr_P_Invalid_Expected_Type_Error_Name
                    (Self
                    );
                  --# end
               when Lkt_Big_Num_Lit =>
                  --# property-call-start BigNumLit.invalid_expected_type_error_name
                  Property_Result := Big_Num_Lit_P_Invalid_Expected_Type_Error_Name
                    (Self
                    );
                  --# end
               when Lkt_Char_Lit =>
                  --# property-call-start CharLit.invalid_expected_type_error_name
                  Property_Result := Char_Lit_P_Invalid_Expected_Type_Error_Name
                    (Self
                    );
                  --# end
               when Lkt_Num_Lit =>
                  --# property-call-start NumLit.invalid_expected_type_error_name
                  Property_Result := Num_Lit_P_Invalid_Expected_Type_Error_Name
                    (Self
                    );
                  --# end
               when Lkt_Single_Line_String_Lit .. Lkt_Pattern_Single_Line_String_Lit =>
                  --# property-call-start SingleLineStringLit.invalid_expected_type_error_name
                  Property_Result := Single_Line_String_Lit_P_Invalid_Expected_Type_Error_Name
                    (Self
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Expr_P_Invalid_Expected_Type_Error_Name;
--# end

   







--# property-start Expr.expr_type_impl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1020
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expr_Type_Impl
  
  (Node : Bare_Expr
      ; Expected_Type : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind expected_type Expected_Type

   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_93 with Inline_Always;

      Block_Var : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Cast_Expr : Bare_Expr;
Cast_Result : Bare_Lkt_Node;
New_Struct : Internal_Semantic_Result := No_Semantic_Result;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;
Fld_2 : Boolean;
If_Result_1 : Internal_Semantic_Result := No_Semantic_Result;
Is_Null_2 : Boolean;
Not_Val_2 : Boolean;
Cast_Expr_1 : Bare_Expr;
Cast_Result_1 : Bare_Lkt_Node;
New_Struct_1 : Internal_Semantic_Result := No_Semantic_Result;
Fld_3 : String_Type;
Fld_4 : Internal_Semantic_Result := No_Semantic_Result;
Fld_5 : Boolean;
If_Result_2 : Internal_Semantic_Result := No_Semantic_Result;
Is_Null_3 : Boolean;
Not_Val_3 : Boolean;
Cast_Expr_2 : Bare_Expr;
Cast_Result_2 : Bare_Lkt_Node;
New_Struct_2 : Internal_Semantic_Result := No_Semantic_Result;
Str : String_Type;
Fld_6 : Internal_Semantic_Result := No_Semantic_Result;
If_Result_3 : Internal_Semantic_Result := No_Semantic_Result;
If_Result_4 : Internal_Semantic_Result := No_Semantic_Result;
If_Result_5 : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_93 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_93;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 826 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 804 '<FieldAccess for expr_context_free_type at parser.py:1065>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1065






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 804
Block_Var := Fld; 
--# bind cf_type Block_Var
--# expr-start 825 '<Cond at parser.py:1067>' If_Result_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1067






--# expr-start 806 '<Not at parser.py:1070>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1070
--# expr-start 805 '<IsNull at parser.py:1070>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1070

Is_Null := Expected_Type.Node = null; 
--# expr-done 805
Not_Val := not (Is_Null); 
--# expr-done 806
if Not_Val then
   --# expr-start 808 '<Not at parser.py:1070>' Not_Val_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1070
--# expr-start 807 '<IsNull at parser.py:1070>' Is_Null_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1070

Is_Null_1 := Block_Var.Node = null; 
--# expr-done 807
Not_Val_1 := not (Is_Null_1); 
--# expr-done 808
   If_Result := Not_Val_1;
else
   
   If_Result := False;
end if;



if If_Result then
   --# expr-start 812 '<If at parser.py:1071>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1071



--# expr-start 809 '<FieldAccess for matches at parser.py:1072>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1072






   

      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start TypeDecl.matches
Fld_2 := Liblktlang.Implementation.Type_Decl_P_Matches (Node => Block_Var.Node, Other => Expected_Type, E_Info => Block_Var.Info);
--# end
--# expr-done 809
if Fld_2 then
   --# expr-start 810 '<New SemanticResult at parser.py:1073>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1073









Cast_Expr := Self; 



   
      Cast_Result := Cast_Expr;




Inc_Ref (Empty_String);
New_Struct := (Error_Message => Empty_String, Exempt_Analysis => False, Has_Error => False, Node => Cast_Result, Result_Decl => No_Entity_Decl, Result_Type => Expected_Type); 
--# expr-done 810
   If_Result_1 := New_Struct;
else
   --# expr-start 811 '<FieldAccess for type_mismatch_error at parser.py:1076>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1076






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;





--# property-call-start LktNode.type_mismatch_error
Fld_1 := Liblktlang.Implementation.Lkt_Node_P_Type_Mismatch_Error (Node => Self, Expected => Expected_Type, Got => Block_Var);
--# end
--# expr-done 811
   If_Result_1 := Fld_1;
end if;

      Inc_Ref (If_Result_1);


--# expr-done 812
   If_Result_5 := If_Result_1;
else
   


--# expr-start 814 '<Not at parser.py:1081>' Not_Val_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1081
--# expr-start 813 '<IsNull at parser.py:1081>' Is_Null_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1081

Is_Null_2 := Expected_Type.Node = null; 
--# expr-done 813
Not_Val_2 := not (Is_Null_2); 
--# expr-done 814
if Not_Val_2 then
   --# expr-start 819 '<If at parser.py:1082>' If_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1082



--# expr-start 815 '<FieldAccess for expected_type_predicate at parser.py:1083>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1083






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start '[dispatcher]Expr.expected_type_predicate'
Fld_5 := Liblktlang.Implementation.Dispatcher_Expr_P_Expected_Type_Predicate (Node => Ent.Node, Expected_Type => Expected_Type);
--# end
--# expr-done 815
if Fld_5 then
   --# expr-start 816 '<New SemanticResult at parser.py:1084>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1084









Cast_Expr_1 := Self; 



   
      Cast_Result_1 := Cast_Expr_1;




Inc_Ref (Empty_String);
New_Struct_1 := (Error_Message => Empty_String, Exempt_Analysis => False, Has_Error => False, Node => Cast_Result_1, Result_Decl => No_Entity_Decl, Result_Type => Expected_Type); 
--# expr-done 816
   If_Result_2 := New_Struct_1;
else
   --# expr-start 818 '<FieldAccess for expected_type_error at parser.py:1087>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1087






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# expr-start 817 '<FieldAccess for invalid_expected_type_error_name at parser.py:1089>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1089






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.invalid_expected_type_error_name'
Fld_3 := Liblktlang.Implementation.Dispatcher_Expr_P_Invalid_Expected_Type_Error_Name (Node => Self);
--# end
--# expr-done 817
--# property-call-start LktNode.expected_type_error
Fld_4 := Liblktlang.Implementation.Lkt_Node_P_Expected_Type_Error (Node => Self, Expected => Expected_Type, Got => Fld_3);
--# end
--# expr-done 818
   If_Result_2 := Fld_4;
end if;

      Inc_Ref (If_Result_2);


--# expr-done 819
   If_Result_4 := If_Result_2;
else
   


--# expr-start 821 '<Not at parser.py:1095>' Not_Val_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1095
--# expr-start 820 '<IsNull at parser.py:1095>' Is_Null_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1095

Is_Null_3 := Block_Var.Node = null; 
--# expr-done 820
Not_Val_3 := not (Is_Null_3); 
--# expr-done 821
if Not_Val_3 then
   --# expr-start 822 '<New SemanticResult at parser.py:1096>' New_Struct_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1096









Cast_Expr_2 := Self; 



   
      Cast_Result_2 := Cast_Expr_2;




Inc_Ref (Empty_String);
New_Struct_2 := (Error_Message => Empty_String, Exempt_Analysis => False, Has_Error => False, Node => Cast_Result_2, Result_Decl => No_Entity_Decl, Result_Type => Block_Var); 
--# expr-done 822
   If_Result_3 := New_Struct_2;
else
   --# expr-start 824 '<FieldAccess for error at parser.py:1099>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1099






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 823 '<String at parser.py:1099>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1099
Str := Create_String ("ambiguous type for expression"); 
--# expr-done 823
--# property-call-start LktNode.error
Fld_6 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Ent.Node, Message => Str);
--# end
--# expr-done 824
   If_Result_3 := Fld_6;
end if;

      Inc_Ref (If_Result_3);


   If_Result_4 := If_Result_3;
end if;

      Inc_Ref (If_Result_4);


   If_Result_5 := If_Result_4;
end if;

      Inc_Ref (If_Result_5);


--# expr-done 825
Let_Result := If_Result_5; Inc_Ref (Let_Result);
--# end
--# expr-done 826

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_93;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_93;




            raise;
      end;



   return Property_Result;
end Expr_P_Expr_Type_Impl;
--# end

   







--# property-start '[dispatcher]Expr.referenced_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Referenced_Decl
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Expr (Self.Kind) is
               when Lkt_Any_Of .. Lkt_Array_Literal | Lkt_Null_Cond_Dotted_Name .. Lkt_Error_On_Null | Lkt_Grammar_Cut .. Lkt_Module_Ref_Id | Lkt_If_Expr .. Lkt_Un_Op =>
                  --# property-call-start Expr.referenced_decl
                  Property_Result := Expr_P_Referenced_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Dot_Expr =>
                  --# property-call-start DotExpr.referenced_decl
                  Property_Result := Dot_Expr_P_Referenced_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Generic_Instantiation =>
                  --# property-call-start GenericInstantiation.referenced_decl
                  Property_Result := Generic_Instantiation_P_Referenced_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Ref_Id =>
                  --# property-call-start RefId.referenced_decl
                  Property_Result := Ref_Id_P_Referenced_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Expr_P_Referenced_Decl;
--# end

   







--# property-start Expr.check_referenced_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1110
pragma Warnings (Off, "is not referenced");
 function Expr_P_Check_Referenced_Decl
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Decl;

      
            procedure Finalizer_Scope_95 with Inline_Always;
            procedure Finalizer_Scope_445 with Inline_Always;

      Var_Expr : Internal_Entity_Decl;
Var_Expr_1 : Internal_Entity_Decl;
Var_Expr_23 : Internal_Semantic_Result := No_Semantic_Result;
Fld : Internal_Semantic_Result := No_Semantic_Result;
Fld_1 : Internal_Entity_Decl;
Scope_Result : Internal_Entity_Decl;
Result_Var : Internal_Entity_Decl;
Scope_Result_1 : Internal_Entity_Decl;
Result_Var_1 : Internal_Entity_Decl;
Scope_Result_2 : Internal_Entity_Decl;
Result_Var_2 : Internal_Entity_Decl;
Let_Result : Internal_Entity_Decl;

            procedure Finalizer_Scope_95 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_95;
            procedure Finalizer_Scope_445 is
            begin
                     Dec_Ref (Var_Expr_23);
            end Finalizer_Scope_445;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 829 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 827 '<FieldAccess for referenced_decl at parser.py:1115>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1115






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 827
Var_Expr_23 := Fld;

      Inc_Ref (Var_Expr_23);




if Var_Expr_23 /= No_Semantic_Result then
   



   --# scope-start


   







--# expr-start 828 '<FieldAccess for result_decl at parser.py:1115>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1115






   




Fld_1 := Var_Expr_23.Result_Decl;
--# expr-done 828
Var_Expr_1 := Fld_1;





if Var_Expr_1 /= No_Entity_Decl then
   



   --# scope-start


   --# bind rr Var_Expr_1


Scope_Result := Var_Expr_1;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Decl;
end if;



Var_Expr := Result_Var;





if Var_Expr /= No_Entity_Decl then
   



   --# scope-start


   --# bind e Var_Expr


Scope_Result_1 := Var_Expr;


   --# end


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := raise Property_Error with "No referenced decl error";
end if;



Scope_Result_2 := Result_Var_1;


   --# end
      Finalizer_Scope_445;


   Result_Var_2 := Scope_Result_2;
else
   
   Result_Var_2 := No_Entity_Decl;
end if;



Let_Result := Result_Var_2; 
--# end
--# expr-done 829

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_95;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_95;
                     Finalizer_Scope_445;




            raise;
      end;



   return Property_Result;
end Expr_P_Check_Referenced_Decl;
--# end

   







--# property-start Expr.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:989
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expr_Context_Free_Type
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 830 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := No_Entity_Type_Decl; 
--# end
--# expr-done 830

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Expr_P_Expr_Context_Free_Type;
--# end

   







--# property-start Expr.expected_type_predicate /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:998
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expected_Type_Predicate
  
  (Node : Bare_Expr
      ; Expected_Type : Internal_Entity_Type_Decl
  )

   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   

   --# bind expected_type Expected_Type

   Property_Result : Boolean;

      

      Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 831 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := True; 
--# end
--# expr-done 831

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Expr_P_Expected_Type_Predicate;
--# end

   







--# property-start Expr.invalid_expected_type_error_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1011
pragma Warnings (Off, "is not referenced");
 function Expr_P_Invalid_Expected_Type_Error_Name
  
  (Node : Bare_Expr
  )

   return String_Type
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_92 with Inline_Always;

      Str : String_Type;
Let_Result : String_Type;

            procedure Finalizer_Scope_92 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_92;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 833 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 832 '<String at parser.py:1018>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1018
Str := Create_String ("<not implemented>"); 
--# expr-done 832
Let_Result := Str; Inc_Ref (Let_Result);
--# end
--# expr-done 833

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_92;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_92;




            raise;
      end;



   return Property_Result;
end Expr_P_Invalid_Expected_Type_Error_Name;
--# end

   







--# property-start Expr.referenced_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1102
pragma Warnings (Off, "is not referenced");
 function Expr_P_Referenced_Decl
  
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_94 with Inline_Always;

      Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_94 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_94;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 834 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := No_Semantic_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 834

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_94;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_94;




            raise;
      end;



   return Property_Result;
end Expr_P_Referenced_Decl;
--# end



   


      

   --
   --  Primitives for Bare_Any_Of
   --

   



      
      procedure Initialize_Fields_For_Any_Of
        (Self : Bare_Any_Of
         ; Any_Of_F_Expr : Bare_Expr
         ; Any_Of_F_Values : Bare_Any_Of_List
        ) is
      begin

            Self.Any_Of_F_Expr := Any_Of_F_Expr;
            Self.Any_Of_F_Values := Any_Of_F_Values;
         

      end Initialize_Fields_For_Any_Of;

      
   function Any_Of_F_Expr
     (Node : Bare_Any_Of) return Bare_Expr
   is
      

   begin
         
         return Node.Any_Of_F_Expr;
      
   end;

      
   function Any_Of_F_Values
     (Node : Bare_Any_Of) return Bare_Any_Of_List
   is
      

   begin
         
         return Node.Any_Of_F_Values;
      
   end;




   


      

   --
   --  Primitives for Bare_Array_Literal
   --

   



      
      procedure Initialize_Fields_For_Array_Literal
        (Self : Bare_Array_Literal
         ; Array_Literal_F_Exprs : Bare_Expr_List
         ; Array_Literal_F_Element_Type : Bare_Type_Ref
        ) is
      begin

            Self.Array_Literal_F_Exprs := Array_Literal_F_Exprs;
            Self.Array_Literal_F_Element_Type := Array_Literal_F_Element_Type;
         

      end Initialize_Fields_For_Array_Literal;

      
   function Array_Literal_F_Exprs
     (Node : Bare_Array_Literal) return Bare_Expr_List
   is
      

   begin
         
         return Node.Array_Literal_F_Exprs;
      
   end;

      
   function Array_Literal_F_Element_Type
     (Node : Bare_Array_Literal) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Array_Literal_F_Element_Type;
      
   end;


   







--# property-start ArrayLiteral.expected_type_predicate /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2789
pragma Warnings (Off, "is not referenced");
 function Array_Literal_P_Expected_Type_Predicate
  
  (Node : Bare_Array_Literal
      ; Expected_Type : Internal_Entity_Type_Decl
  )

   return Boolean
is
   Self : Bare_Array_Literal  := Bare_Array_Literal (Node);
      --# bind self Self

   

   --# bind expected_type Expected_Type

   Property_Result : Boolean;

      

      Var_Expr : Internal_Entity_Type_Decl;
Var_Expr_1 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 837 '<Block at ???>' Let_Result None
--# scope-start




Var_Expr := Expected_Type;





if Var_Expr /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind t Var_Expr




--# expr-start 835 '<FieldAccess for array_element_type at parser.py:2795>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2795






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.array_element_type
Fld := Liblktlang.Implementation.Type_Decl_P_Array_Element_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 835
Var_Expr_1 := Fld;





if Var_Expr_1 /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind elt_type Var_Expr_1

--# expr-start 836 '<FieldAccess for all_elements_match at parser.py:2796>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2796






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start ArrayLiteral.all_elements_match
Fld_1 := Liblktlang.Implementation.Array_Literal_P_All_Elements_Match (Node => Self, Elt_Type => Var_Expr_1);
--# end
--# expr-done 836
Scope_Result := Fld_1;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := False;
end if;



Scope_Result_1 := Result_Var;


   --# end


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := False;
end if;



Let_Result := Result_Var_1; 
--# end
--# expr-done 837

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Array_Literal_P_Expected_Type_Predicate;
--# end

   







--# property-start ArrayLiteral.invalid_expected_type_error_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2800
pragma Warnings (Off, "is not referenced");
 function Array_Literal_P_Invalid_Expected_Type_Error_Name
  
  (Node : Bare_Array_Literal
  )

   return String_Type
is
   Self : Bare_Array_Literal  := Bare_Array_Literal (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_218 with Inline_Always;

      Str : String_Type;
Let_Result : String_Type;

            procedure Finalizer_Scope_218 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_218;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 839 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 838 '<String at parser.py:2802>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2802
Str := Create_String ("an array"); 
--# expr-done 838
Let_Result := Str; Inc_Ref (Let_Result);
--# end
--# expr-done 839

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_218;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_218;




            raise;
      end;



   return Property_Result;
end Array_Literal_P_Invalid_Expected_Type_Error_Name;
--# end

   







--# property-start ArrayLiteral.all_elements_match /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2804
pragma Warnings (Off, "is not referenced");
 function Array_Literal_P_All_Elements_Match
  
  (Node : Bare_Array_Literal
      ; Elt_Type : Internal_Entity_Type_Decl
  )

   return Boolean
is
   Self : Bare_Array_Literal  := Bare_Array_Literal (Node);
      --# bind self Self

   

   --# bind elt_type Elt_Type

   Property_Result : Boolean;

      

      Fld : Bare_Expr_List;
Item_44 : Bare_Expr;
Fld_1 : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 843 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 842 '<AllQuantifier at parser.py:2810>' Quantifier_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2810








   

   --# expr-start 840 '<FieldAccess for exprs at parser.py:2810>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2810






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Array_Literal_F_Exprs;
--# expr-done 840

   
      Quantifier_Result := True;
   

   

      if Fld = null then
         
      null;
   
      else
         

      declare
         
         Collection : constant Bare_Expr_List := Fld;
      begin
         for Untyped_Item_44 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Item_44 := Untyped_Item_44; 

            
   --# scope-start


               --# bind e Item_44

            
      --# expr-start 841 '<FieldAccess for expected_type_predicate at parser.py:2810>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2810






   

      if Item_44 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start '[dispatcher]Expr.expected_type_predicate'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Expected_Type_Predicate (Node => Item_44, Expected_Type => Elt_Type);
--# end
--# expr-done 841
      Quantifier_Result := Fld_1;
   

            
   --# end


               exit when not Quantifier_Result;

         end loop;
      end;
   
      end if;

   
   



--# expr-done 842
Let_Result := Quantifier_Result; 
--# end
--# expr-done 843

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Array_Literal_P_All_Elements_Match;
--# end

   







--# property-start ArrayLiteral.first_elt_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2812
pragma Warnings (Off, "is not referenced");
 function Array_Literal_P_First_Elt_Context_Free_Type
  
  (Node : Bare_Array_Literal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Array_Literal  := Bare_Array_Literal (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_220 with Inline_Always;
            procedure Finalizer_Scope_439 with Inline_Always;

      Var_Expr : Internal_Entity_Type_Decl_Array_Access;
Fld : Bare_Expr_List;
Item_45 : Bare_Expr;
As_Entity : Internal_Entity_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Decl;
As_Entity_1 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Expr;
Fld_2 : Internal_Entity_Type_Decl;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_220 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_220;
            procedure Finalizer_Scope_439 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_439;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 853 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 851 '<FilterMap at parser.py:2819>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2819








   

   --# expr-start 844 '<FieldAccess for exprs at parser.py:2819>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2819






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Array_Literal_F_Exprs;
--# expr-done 844

   
      declare
         Map_Result_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
      begin
   

   

      if Fld = null then
         
      Map_Result := Create_Internal_Entity_Type_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Expr_List := Fld;
      begin
         for Untyped_Item_45 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Item_45 := Untyped_Item_45; 

            
   --# scope-start


               --# bind e Item_45

            
         --# expr-start 848 '<Not at parser.py:2821>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2821
--# expr-start 847 '<IsNull at parser.py:2821>' Is_Null_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2821
--# expr-start 846 '<FieldAccess for expr_context_free_type at parser.py:2821>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2821



--# expr-start 845 '<as_entity at parser.py:2821>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2821




Is_Null_1 := Item_45 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Expr;
else
   

As_Entity_1 := (Info => E_Info, Node => Item_45); 
   If_Result_1 := As_Entity_1;
end if;



--# expr-done 845


   

      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld_2 := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 846
Is_Null_2 := Fld_2.Node = null; 
--# expr-done 847
Not_Val := not (Is_Null_2); 
--# expr-done 848
         if Not_Val then
            
         
      --# expr-start 850 '<FieldAccess for expr_context_free_type at parser.py:2820>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2820



--# expr-start 849 '<as_entity at parser.py:2820>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2820




Is_Null := Item_45 = null; 
if Is_Null then
   
   If_Result := No_Entity_Expr;
else
   

As_Entity := (Info => E_Info, Node => Item_45); 
   If_Result := As_Entity;
end if;



--# expr-done 849


   

      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 850
      

         declare
            Item_To_Append : constant Internal_Entity_Type_Decl := Fld_1;
         begin
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Entity_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 851
Var_Expr := Map_Result;

      Inc_Ref (Var_Expr);




if Var_Expr /= No_Internal_Entity_Type_Decl_Array_Type then
   



   --# scope-start


   --# bind l Var_Expr

--# expr-start 852 '<at at parser.py:2824>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2824




Get_Result := Get (Self, Var_Expr, 0, True); 
--# expr-done 852
Scope_Result := Get_Result;


   --# end
      Finalizer_Scope_439;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Type_Decl;
end if;



Let_Result := Result_Var; 
--# end
--# expr-done 853

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_220;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_220;
                     Finalizer_Scope_439;




            raise;
      end;



   return Property_Result;
end Array_Literal_P_First_Elt_Context_Free_Type;
--# end

   







--# property-start ArrayLiteral.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2827
pragma Warnings (Off, "is not referenced");
 function Array_Literal_P_Expr_Context_Free_Type
  
  (Node : Bare_Array_Literal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Array_Literal  := Bare_Array_Literal (Node);
     Ent : Internal_Entity_Array_Literal :=
       Internal_Entity_Array_Literal'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Block_Var : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Bare_Decl;
Cast_Result : Bare_Type_Decl;
Fld_2 : Internal_Entity_Instantiated_Generic_Type;
Cast_Result_1 : Internal_Entity_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 862 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 854 '<FieldAccess for first_elt_context_free_type at parser.py:2829>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2829






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start ArrayLiteral.first_elt_context_free_type
Fld := Liblktlang.Implementation.Array_Literal_P_First_Elt_Context_Free_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 854
Block_Var := Fld; 
--# bind elt_type Block_Var
--# expr-start 861 '<If at parser.py:2833>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2833






--# expr-start 856 '<Not at parser.py:2835>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2835
--# expr-start 855 '<IsNull at parser.py:2835>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2835

Is_Null := Block_Var.Node = null; 
--# expr-done 855
Not_Val := not (Is_Null); 
--# expr-done 856
if Not_Val then
   --# expr-start 857 '<FieldAccess for all_elements_match at parser.py:2836>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2836






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start ArrayLiteral.all_elements_match
Fld_3 := Liblktlang.Implementation.Array_Literal_P_All_Elements_Match (Node => Self, Elt_Type => Block_Var);
--# end
--# expr-done 857
   If_Result := Fld_3;
else
   
   If_Result := False;
end if;



if If_Result then
   




--# expr-start 860 '<FieldAccess for get_array_type at parser.py:2839>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2839






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 859 '<Cast to TypeDecl at parser.py:2839>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2839





--# expr-start 858 '<FieldAccess for assert_bare at parser.py:2839>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2839






   

      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.assert_bare
Fld_1 := Liblktlang.Implementation.Decl_P_Assert_Bare (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 858



   if Fld_1 = null
      or else Fld_1.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Fld_1;

   else
         Cast_Result := No_Bare_Lkt_Node;
   end if;


--# expr-done 859
--# property-call-start Expr.get_array_type
Fld_2 := Liblktlang.Implementation.Expr_P_Get_Array_Type (Node => Ent.Node, Elt_Type => Cast_Result, E_Info => Ent.Info);
--# end
--# expr-done 860



   
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);


   If_Result_1 := Cast_Result_1;
else
   
   If_Result_1 := No_Entity_Type_Decl;
end if;



--# expr-done 861
Let_Result := If_Result_1; 
--# end
--# expr-done 862

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Array_Literal_P_Expr_Context_Free_Type;
--# end



   


      

   --
   --  Primitives for Bare_Base_Dot_Expr
   --

   



      
      procedure Initialize_Fields_For_Base_Dot_Expr
        (Self : Bare_Base_Dot_Expr
         ; Base_Dot_Expr_F_Prefix : Bare_Expr
         ; Base_Dot_Expr_F_Suffix : Bare_Ref_Id
        ) is
      begin

            Self.Base_Dot_Expr_F_Prefix := Base_Dot_Expr_F_Prefix;
            Self.Base_Dot_Expr_F_Suffix := Base_Dot_Expr_F_Suffix;
         

      end Initialize_Fields_For_Base_Dot_Expr;

      
   function Base_Dot_Expr_F_Prefix
     (Node : Bare_Base_Dot_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Base_Dot_Expr_F_Prefix;
      
   end;

      
   function Base_Dot_Expr_F_Suffix
     (Node : Bare_Base_Dot_Expr) return Bare_Ref_Id
   is
      

   begin
         
         return Node.Base_Dot_Expr_F_Suffix;
      
   end;




   


      

   --
   --  Primitives for Bare_Dot_Expr
   --

   



      
      procedure Initialize_Fields_For_Dot_Expr
        (Self : Bare_Dot_Expr
         ; Base_Dot_Expr_F_Prefix : Bare_Expr
         ; Base_Dot_Expr_F_Suffix : Bare_Ref_Id
        ) is
      begin
            Initialize_Fields_For_Base_Dot_Expr
              (Self, Base_Dot_Expr_F_Prefix, Base_Dot_Expr_F_Suffix);

         

      end Initialize_Fields_For_Dot_Expr;


   







--# property-start DotExpr.referenced_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1371
pragma Warnings (Off, "is not referenced");
 function Dot_Expr_P_Referenced_Decl
  
  (Node : Bare_Dot_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result
is
   Self : Bare_Dot_Expr  := Bare_Dot_Expr (Node);
     Ent : Internal_Entity_Dot_Expr :=
       Internal_Entity_Dot_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_108 with Inline_Always;

      Fld : Internal_Entity_Ref_Id;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_108 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_108;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 864 '<FieldAccess for referenced_decl at parser.py:1371>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1371



--# expr-start 863 '<FieldAccess for suffix at parser.py:1371>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1371






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Ref_Id (Node => Ent.Node.Base_Dot_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 863


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 864

         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_108;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_108;




            raise;
      end;



   return Property_Result;
end Dot_Expr_P_Referenced_Decl;
--# end

   







--# property-start DotExpr.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1372
pragma Warnings (Off, "is not referenced");
 function Dot_Expr_P_Expr_Context_Free_Type
  
  (Node : Bare_Dot_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Dot_Expr  := Bare_Dot_Expr (Node);
     Ent : Internal_Entity_Dot_Expr :=
       Internal_Entity_Dot_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity_Ref_Id;
Fld_1 : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 866 '<FieldAccess for expr_context_free_type at parser.py:1372>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1372



--# expr-start 865 '<FieldAccess for suffix at parser.py:1372>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1372






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Ref_Id (Node => Ent.Node.Base_Dot_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 865


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 866

         Property_Result := Fld_1;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Dot_Expr_P_Expr_Context_Free_Type;
--# end



   


      

   --
   --  Primitives for Bare_Null_Cond_Dotted_Name
   --

   



      
      procedure Initialize_Fields_For_Null_Cond_Dotted_Name
        (Self : Bare_Null_Cond_Dotted_Name
         ; Base_Dot_Expr_F_Prefix : Bare_Expr
         ; Base_Dot_Expr_F_Suffix : Bare_Ref_Id
        ) is
      begin
            Initialize_Fields_For_Base_Dot_Expr
              (Self, Base_Dot_Expr_F_Prefix, Base_Dot_Expr_F_Suffix);

         

      end Initialize_Fields_For_Null_Cond_Dotted_Name;




   


      

   --
   --  Primitives for Bare_Bin_Op
   --

   



      
      procedure Initialize_Fields_For_Bin_Op
        (Self : Bare_Bin_Op
         ; Bin_Op_F_Left : Bare_Expr
         ; Bin_Op_F_Op : Bare_Op
         ; Bin_Op_F_Right : Bare_Expr
        ) is
      begin

            Self.Bin_Op_F_Left := Bin_Op_F_Left;
            Self.Bin_Op_F_Op := Bin_Op_F_Op;
            Self.Bin_Op_F_Right := Bin_Op_F_Right;
         

      end Initialize_Fields_For_Bin_Op;

      
   function Bin_Op_F_Left
     (Node : Bare_Bin_Op) return Bare_Expr
   is
      

   begin
         
         return Node.Bin_Op_F_Left;
      
   end;

      
   function Bin_Op_F_Op
     (Node : Bare_Bin_Op) return Bare_Op
   is
      

   begin
         
         return Node.Bin_Op_F_Op;
      
   end;

      
   function Bin_Op_F_Right
     (Node : Bare_Bin_Op) return Bare_Expr
   is
      

   begin
         
         return Node.Bin_Op_F_Right;
      
   end;


   







--# property-start BinOp.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3697
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Expr_Context_Free_Type
  
  (Node : Bare_Bin_Op
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Block_Var : Internal_Entity_Type_Decl;
Block_Var_1 : Internal_Entity_Type_Decl;
Var_Expr_24 : Internal_Entity_Type_Decl;
Fld : Bare_Expr;
As_Entity : Internal_Entity_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Bare_Expr;
As_Entity_1 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Type_Decl;
Fld_4 : Bare_Op;
Is_A : Boolean;
Fld_5 : Internal_Entity_Named_Type_Decl;
Fld_6 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_2 : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
If_Result_3 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 878 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 869 '<FieldAccess for expr_context_free_type at parser.py:3699>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3699



--# expr-start 868 '<as_entity at parser.py:3699>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3699



--# expr-start 867 '<FieldAccess for left at parser.py:3699>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3699






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Bin_Op_F_Left;
--# expr-done 867
Is_Null := Fld = null; 
if Is_Null then
   
   If_Result := No_Entity_Expr;
else
   

As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;



--# expr-done 868


   

      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 869
Block_Var := Fld_1; 
--# bind left_cft Block_Var
--# expr-start 872 '<FieldAccess for expr_context_free_type at parser.py:3700>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3700



--# expr-start 871 '<as_entity at parser.py:3700>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3700



--# expr-start 870 '<FieldAccess for right at parser.py:3700>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3700






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Self.Bin_Op_F_Right;
--# expr-done 870
Is_Null_1 := Fld_2 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Expr;
else
   

As_Entity_1 := (Info => E_Info, Node => Fld_2); 
   If_Result_1 := As_Entity_1;
end if;



--# expr-done 871


   

      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld_3 := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 872
Block_Var_1 := Fld_3; 
--# bind right_cft Block_Var_1
--# expr-start 877 '<Cond at parser.py:3702>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3702



--# expr-start 874 '<IsA Op.And, Op.Or, Op.Lt, Op.Gt, Op.Lte, Op.Gte, Op.Eq at parser.py:3704>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3704
--# expr-start 873 '<FieldAccess for op at parser.py:3704>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3704






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Self.Bin_Op_F_Op;
--# expr-done 873
Is_A := Fld_4 /= null 
and then Fld_4.Kind in Lkt_Op_And_Range | Lkt_Op_Or_Range | Lkt_Op_Lt_Range | Lkt_Op_Gt_Range | Lkt_Op_Lte_Range | Lkt_Op_Gte_Range | Lkt_Op_Eq_Range; 
--# expr-done 874
if Is_A then
   




--# expr-start 875 '<FieldAccess for bool_type at parser.py:3707>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3707






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.bool_type
Fld_5 := Liblktlang.Implementation.Lkt_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 875



   
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);


   If_Result_3 := Cast_Result;
else
   






Var_Expr_24 := Block_Var;





if Var_Expr_24 /= No_Entity_Type_Decl then
   



   --# scope-start


   

--# expr-start 876 '<FieldAccess for matches at parser.py:3710>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3710






   

      if Var_Expr_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start TypeDecl.matches
Fld_6 := Liblktlang.Implementation.Type_Decl_P_Matches (Node => Var_Expr_24.Node, Other => Block_Var_1, E_Info => Var_Expr_24.Info);
--# end
--# expr-done 876
Scope_Result := Fld_6;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := False;
end if;



if Result_Var then
   
   If_Result_2 := Block_Var;
else
   
   If_Result_2 := No_Entity_Type_Decl;
end if;



   If_Result_3 := If_Result_2;
end if;



--# expr-done 877
Let_Result := If_Result_3; 
--# end
--# expr-done 878

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Bin_Op_P_Expr_Context_Free_Type;
--# end



   


      

   --
   --  Primitives for Bare_Block_Expr
   --

   



   

   

   

   

   

   

   


      procedure Block_Expr_Pre_Env_Actions
        (Self            : Bare_Block_Expr;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   
      if Add_To_Env_Only then
         return;
      end if;

      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Liblktlang.Implementation.Env_Trans_Parent_21 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
   
   
      end;



      
      procedure Initialize_Fields_For_Block_Expr
        (Self : Bare_Block_Expr
         ; Block_Expr_F_Val_Defs : Bare_Block_Decl_List
         ; Block_Expr_F_Expr : Bare_Expr
        ) is
      begin

            Self.Block_Expr_F_Val_Defs := Block_Expr_F_Val_Defs;
            Self.Block_Expr_F_Expr := Block_Expr_F_Expr;
         

      end Initialize_Fields_For_Block_Expr;

      
   function Block_Expr_F_Val_Defs
     (Node : Bare_Block_Expr) return Bare_Block_Decl_List
   is
      

   begin
         
         return Node.Block_Expr_F_Val_Defs;
      
   end;

      
   function Block_Expr_F_Expr
     (Node : Bare_Block_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Block_Expr_F_Expr;
      
   end;


   







--# property-start BlockExpr._env_trans_parent_21 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3657
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_21
  
  (Node : Bare_Block_Expr
  )

   return Boolean
is
   Self : Bare_Block_Expr  := Bare_Block_Expr (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Trans_Parent_21;
--# end



   


      

   --
   --  Primitives for Bare_Call_Expr
   --

   



      
      procedure Initialize_Fields_For_Call_Expr
        (Self : Bare_Call_Expr
         ; Call_Expr_F_Name : Bare_Expr
         ; Call_Expr_F_Args : Bare_Param_List
        ) is
      begin

            Self.Call_Expr_F_Name := Call_Expr_F_Name;
            Self.Call_Expr_F_Args := Call_Expr_F_Args;
         

      end Initialize_Fields_For_Call_Expr;

      
   function Call_Expr_F_Name
     (Node : Bare_Call_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Call_Expr_F_Name;
      
   end;

      
   function Call_Expr_F_Args
     (Node : Bare_Call_Expr) return Bare_Param_List
   is
      

   begin
         
         return Node.Call_Expr_F_Args;
      
   end;


   







--# property-start CallExpr.check_correctness_pre /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2985
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_Correctness_Pre
  
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_229 with Inline_Always;
            procedure Finalizer_Scope_452 with Inline_Always;
            procedure Finalizer_Scope_453 with Inline_Always;
            procedure Finalizer_Scope_454 with Inline_Always;
            procedure Finalizer_Scope_455 with Inline_Always;
            procedure Finalizer_Scope_475 with Inline_Always;
            procedure Finalizer_Scope_477 with Inline_Always;
            procedure Finalizer_Scope_478 with Inline_Always;

      Block_Var : Internal_Semantic_Result := No_Semantic_Result;
Block_Var_1 : Internal_Semantic_Result := No_Semantic_Result;
Var_Expr : Internal_Semantic_Result_Array_Access;
Var_Expr_1 : Internal_Semantic_Result_Array_Access;
Var_Expr_2 : Internal_Entity_Generic_Decl;
Var_Expr_3 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Var_Expr_4 : Internal_Semantic_Result := No_Semantic_Result;
Var_Expr_25 : Internal_Entity_Ref_Id;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;
Fld_2 : Bare_Expr;
Fld_3 : Internal_Semantic_Result := No_Semantic_Result;
Try_Result : Internal_Semantic_Result := No_Semantic_Result;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Internal_Semantic_Result := No_Semantic_Result;
Fld_6 : Bare_Expr;
Fld_7 : Internal_Semantic_Result := No_Semantic_Result;
Try_Result_1 : Internal_Semantic_Result := No_Semantic_Result;
Fld_8 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Generic_Decl;
Fld_9 : Internal_Entity_Type_Decl;
Fld_10 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld_11 : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
Scope_Result : Internal_Semantic_Result_Array_Access;
Result_Var : Internal_Semantic_Result_Array_Access;
Scope_Result_1 : Internal_Semantic_Result_Array_Access;
Result_Var_1 : Internal_Semantic_Result_Array_Access;
Scope_Result_2 : Internal_Semantic_Result_Array_Access;
Result_Var_2 : Internal_Semantic_Result_Array_Access;
Scope_Result_3 : Internal_Semantic_Result_Array_Access;
Fld_12 : Internal_Entity_Expr;
Str : String_Type;
Fld_13 : Internal_Entity_Type_Decl;
Fld_14 : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;
Fld_15 : Internal_Semantic_Result := No_Semantic_Result;
Singleton_1 : Internal_Semantic_Result_Array_Access;
Fld_16 : Internal_Entity_Decl;
Is_Null : Boolean;
Fld_17 : Internal_Entity_Type_Decl;
Is_Null_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Semantic_Result_Array_Access;
Result_Var_3 : Internal_Semantic_Result_Array_Access;
Scope_Result_4 : Internal_Semantic_Result_Array_Access;
Fld_18 : Internal_Param_Match_Array_Access;
Fld_19 : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_20 : Internal_Entity_Param;
Str_2 : String_Type;
Str_3 : String_Type;
Fld_21 : Internal_Entity_Decl;
Fld_22 : String_Type;
Concat_Result_2 : String_Type;
Str_4 : String_Type;
Concat_Result_3 : String_Type;
Is_Equal : Boolean;
If_Result_2 : String_Type;
Fld_23 : Internal_Semantic_Result := No_Semantic_Result;
Fld_24 : Internal_Entity_Param;
Fld_25 : Internal_Entity_Ref_Id;
Fld_26 : Internal_Semantic_Result := No_Semantic_Result;
Scope_Result_5 : Internal_Semantic_Result := No_Semantic_Result;
Result_Var_4 : Internal_Semantic_Result := No_Semantic_Result;
Array_Lit : Internal_Semantic_Result_Array_Access;
Str_5 : String_Type;
Str_6 : String_Type;
Fld_27 : Internal_Formal_Param;
Fld_28 : Symbol_Type;
Fld_29 : String_Type;
Concat_Result_4 : String_Type;
Str_7 : String_Type;
Concat_Result_5 : String_Type;
Fld_30 : Internal_Entity_Decl;
Fld_31 : String_Type;
Concat_Result_6 : String_Type;
Str_8 : String_Type;
Concat_Result_7 : String_Type;
Is_Equal_1 : Boolean;
If_Result_3 : String_Type;
Fld_32 : Internal_Semantic_Result := No_Semantic_Result;
Singleton_2 : Internal_Semantic_Result_Array_Access;
Fld_33 : Internal_Formal_Param;
Is_Equal_2 : Boolean;
If_Result_4 : Internal_Semantic_Result_Array_Access;
Map_Result_1 : Internal_Semantic_Result_Array_Access;
Result_Var_5 : Internal_Semantic_Result_Array_Access;
Fld_34 : Boolean;
Fld_35 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_229 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Try_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Try_Result_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_229;
            procedure Finalizer_Scope_452 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_452;
            procedure Finalizer_Scope_453 is
            begin
                     Dec_Ref (Var_Expr_3);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_453;
            procedure Finalizer_Scope_454 is
            begin
                     Dec_Ref (Var_Expr_4);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_454;
            procedure Finalizer_Scope_455 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_455;
            procedure Finalizer_Scope_475 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_475;
            procedure Finalizer_Scope_477 is
            begin
                     Dec_Ref (Str_2);
                     Dec_Ref (Str_3);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Str_4);
                     Dec_Ref (Concat_Result_3);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Str_5);
                     Dec_Ref (Str_6);
                     Dec_Ref (Fld_29);
                     Dec_Ref (Concat_Result_4);
                     Dec_Ref (Str_7);
                     Dec_Ref (Concat_Result_5);
                     Dec_Ref (Fld_31);
                     Dec_Ref (Concat_Result_6);
                     Dec_Ref (Str_8);
                     Dec_Ref (Concat_Result_7);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Fld_32);
                     Dec_Ref (Singleton_2);
                     Dec_Ref (If_Result_4);
            end Finalizer_Scope_477;
            procedure Finalizer_Scope_478 is
            begin
                     Dec_Ref (Fld_26);
            end Finalizer_Scope_478;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 941 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 883 '<Try at parser.py:69>' Try_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:69



begin
   --# expr-start 880 '<FieldAccess for referenced_decl at parser.py:2988>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2988



--# expr-start 879 '<FieldAccess for name at parser.py:2988>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2988






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 879


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 880
   Try_Result := Fld_1;
exception
   when Property_Error =>
      --# expr-start 882 '<FieldAccess for internal_error at parser.py:2989>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2989



--# expr-start 881 '<FieldAccess for name at parser.py:2989>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2989






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Self.Call_Expr_F_Name;
--# expr-done 881


   

      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.internal_error
Fld_3 := Liblktlang.Implementation.Lkt_Node_P_Internal_Error (Node => Fld_2);
--# end
--# expr-done 882
      Try_Result := Fld_3;
end;

      Inc_Ref (Try_Result);


--# expr-done 883
Block_Var := Try_Result; Inc_Ref (Block_Var);
--# bind rd Block_Var
--# expr-start 888 '<Try at parser.py:69>' Try_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:69



begin
   --# expr-start 885 '<FieldAccess for expr_type at parser.py:2991>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2991



--# expr-start 884 '<FieldAccess for name at parser.py:2991>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2991






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 884


   

      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expr_type
Fld_5 := Liblktlang.Implementation.Expr_P_Expr_Type (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 885
   Try_Result_1 := Fld_5;
exception
   when Property_Error =>
      --# expr-start 887 '<FieldAccess for internal_error at parser.py:2991>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2991



--# expr-start 886 '<FieldAccess for name at parser.py:2991>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2991






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_6 := Self.Call_Expr_F_Name;
--# expr-done 886


   

      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.internal_error
Fld_7 := Liblktlang.Implementation.Lkt_Node_P_Internal_Error (Node => Fld_6);
--# end
--# expr-done 887
      Try_Result_1 := Fld_7;
end;

      Inc_Ref (Try_Result_1);


--# expr-done 888
Block_Var_1 := Try_Result_1; Inc_Ref (Block_Var_1);
--# bind et Block_Var_1
--# expr-start 940 '<If at parser.py:2993>' If_Result_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2993






--# expr-start 889 '<FieldAccess for has_error at parser.py:2994>' Fld_34 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2994






   




Fld_34 := Block_Var.Has_Error;
--# expr-done 889
if Fld_34 then
   
   If_Result_5 := True;
else
   --# expr-start 890 '<FieldAccess for has_error at parser.py:2994>' Fld_35 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2994






   




Fld_35 := Block_Var_1.Has_Error;
--# expr-done 890
   If_Result_5 := Fld_35;
end if;



if If_Result_5 then
   
   If_Result_6 := No_Internal_Semantic_Result_Array_Type;
else
   








--# expr-start 892 '<Cast to GenericDecl at parser.py:3000>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3000





--# expr-start 891 '<FieldAccess for result_decl at parser.py:3000>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3000






   




Fld_8 := Block_Var.Result_Decl;
--# expr-done 891



   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Lkt_Generic_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Fld_8.Node,
         Info => Fld_8.Info);

   else
         Cast_Result := No_Entity_Generic_Decl;
   end if;


--# expr-done 892
Var_Expr_2 := Cast_Result;





if Var_Expr_2 /= No_Entity_Generic_Decl then
   



   --# scope-start


   --# bind generic_decl Var_Expr_2




--# expr-start 894 '<FieldAccess for infer_actuals at parser.py:3001>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3001






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# expr-start 893 '<FieldAccess for expected_type at parser.py:3003>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3003






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_9 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 893
--# property-call-start CallExpr.infer_actuals
Fld_10 := Liblktlang.Implementation.Call_Expr_P_Infer_Actuals (Node => Ent.Node, Generic_Decl => Var_Expr_2, Expected_Type => Fld_9, E_Info => Ent.Info);
--# end
--# expr-done 894
Var_Expr_3 := Fld_10;

      Inc_Ref (Var_Expr_3);




if Var_Expr_3 /= No_Infer_Instantiation then
   



   --# scope-start


   --# bind iir Var_Expr_3




--# expr-start 895 '<FieldAccess for error at parser.py:3004>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3004






   




Fld_11 := Var_Expr_3.Error;
Inc_Ref (Fld_11);
--# expr-done 895
Var_Expr_4 := Fld_11;

      Inc_Ref (Var_Expr_4);




if Var_Expr_4 /= No_Semantic_Result then
   



   --# scope-start


   --# bind e Var_Expr_4


                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr_4;
                Inc_Ref (Singleton.Items (1));
            
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_454;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var);


Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_453;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_1);


Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);


   --# end
      Finalizer_Scope_452;


   Result_Var_2 := Scope_Result_2;
else
   
   Result_Var_2 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_2);


Var_Expr_1 := Result_Var_2;

      Inc_Ref (Var_Expr_1);




if Var_Expr_1 /= No_Internal_Semantic_Result_Array_Type then
   



   --# scope-start


   --# bind e Var_Expr_1


Scope_Result_3 := Var_Expr_1;
   Inc_Ref (Scope_Result_3);


   --# end
      Finalizer_Scope_455;


   Result_Var_3 := Scope_Result_3;
else
   --# expr-start 906 '<If at parser.py:3012>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3012






--# expr-start 897 '<IsNull at parser.py:3013>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3013
--# expr-start 896 '<FieldAccess for called_decl at parser.py:3013>' Fld_16 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3013






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.called_decl
Fld_16 := Liblktlang.Implementation.Call_Expr_P_Called_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 896
Is_Null := Fld_16.Node = null; 
--# expr-done 897
if Is_Null then
   --# expr-start 899 '<IsNull at parser.py:3014>' Is_Null_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3014
--# expr-start 898 '<FieldAccess for called_object_type at parser.py:3014>' Fld_17 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3014






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.called_object_type
Fld_17 := Liblktlang.Implementation.Call_Expr_P_Called_Object_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 898
Is_Null_1 := Fld_17.Node = null; 
--# expr-done 899
   If_Result := Is_Null_1;
else
   
   If_Result := False;
end if;



if If_Result then
   --# expr-start 905 '<FieldAccess for error at parser.py:3016>' Fld_15 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3016



--# expr-start 900 '<FieldAccess for name at parser.py:3016>' Fld_12 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3016






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_12 := Create_Internal_Entity_Expr (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 900


   

      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 901 '<String at parser.py:3017>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3017
Str := Create_String ("Object of type `"); 
--# expr-done 901
--# expr-start 903 '<FieldAccess for full_name at parser.py:3018>' Fld_14 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3018



--# expr-start 902 '<FieldAccess for result_type at parser.py:3018>' Fld_13 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3018






   




Fld_13 := Block_Var_1.Result_Type;
--# expr-done 902


   

      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_14 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Fld_13.Node);
--# end
--# expr-done 903
Concat_Result := Concat_String (Str, Fld_14); 
--# expr-start 904 '<String at parser.py:3019>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3019
Str_1 := Create_String ("` is not callable"); 
--# expr-done 904
Concat_Result_1 := Concat_String (Concat_Result, Str_1); 
--# property-call-start LktNode.error
Fld_15 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Fld_12.Node, Message => Concat_Result_1);
--# end
--# expr-done 905
                Singleton_1 := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton_1.Items (1) := Fld_15;
                Inc_Ref (Singleton_1.Items (1));
            
   If_Result_1 := Singleton_1;
else
   
   If_Result_1 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (If_Result_1);


--# expr-done 906
   Result_Var_3 := If_Result_1;
end if;

      Inc_Ref (Result_Var_3);


Var_Expr := Result_Var_3;

      Inc_Ref (Var_Expr);




if Var_Expr /= No_Internal_Semantic_Result_Array_Type then
   



   --# scope-start


   --# bind e Var_Expr


Scope_Result_4 := Var_Expr;
   Inc_Ref (Scope_Result_4);


   --# end
      Finalizer_Scope_475;


   Result_Var_5 := Scope_Result_4;
else
   --# expr-start 939 '<Mapcat at parser.py:3028>' Map_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3028








   

   --# expr-start 910 '<Filter at parser.py:3027>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3027








   

   --# expr-start 907 '<FieldAccess for match_params at parser.py:3027>' Fld_18 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3027






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.match_params
Fld_18 := Liblktlang.Implementation.Call_Expr_P_Match_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 907

   
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Param_Match_Array_Access := Fld_18;
      begin
         for Item_47 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind pm Item_47

            
         --# expr-start 909 '<Not at parser.py:3027>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3027
--# expr-start 908 '<FieldAccess for has_matched at parser.py:3027>' Fld_19 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3027






   




Fld_19 := Item_47.Has_Matched;
--# expr-done 908
Not_Val := not (Fld_19); 
--# expr-done 909
         if Not_Val then
            
         
      
      

         declare
            Item_To_Append : constant Internal_Param_Match := Item_47;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 910

   
      declare
         Map_Result_1_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Param_Match_Array_Access := Map_Result;
      begin
         for Item_46 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind pm Item_46

            
         
         
      --# expr-start 938 '<If at parser.py:3029>' If_Result_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3029



--# expr-start 912 '<IsNull at parser.py:3030>' Is_Equal_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3030
--# expr-start 911 '<FieldAccess for formal at parser.py:3030>' Fld_33 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3030






   




Fld_33 := Item_46.Formal;
--# expr-done 911

Is_Equal_2 := Fld_33 = No_Formal_Param; 
--# expr-done 912
if Is_Equal_2 then
   --# expr-start 925 '<ArrayLiteral at parser.py:3032>' Array_Lit /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3032
--# expr-start 921 '<FieldAccess for error at parser.py:3033>' Fld_23 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3033



--# expr-start 913 '<FieldAccess for actual at parser.py:3033>' Fld_20 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3033






   




Fld_20 := Item_46.Actual;
--# expr-done 913


   

      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 920 '<If at parser.py:3033>' If_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3033



--# expr-start 914 '<IsNull at parser.py:3034>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3034


Is_Equal := Equivalent (Block_Var, No_Semantic_Result); 
--# expr-done 914
if Is_Equal then
   --# expr-start 915 '<String at parser.py:3036>' Str_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3036
Str_2 := Create_String ("Unmatched actual"); 
--# expr-done 915
   If_Result_2 := Str_2;
else
   --# expr-start 916 '<String at parser.py:3038>' Str_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3038
Str_3 := Create_String ("Unmatched actual in call to `"); 
--# expr-done 916
--# expr-start 918 '<FieldAccess for full_name at parser.py:3039>' Fld_22 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3039



--# expr-start 917 '<FieldAccess for called_decl at parser.py:3039>' Fld_21 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3039






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.called_decl
Fld_21 := Liblktlang.Implementation.Call_Expr_P_Called_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 917


   

      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_22 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Fld_21.Node);
--# end
--# expr-done 918
Concat_Result_2 := Concat_String (Str_3, Fld_22); 
--# expr-start 919 '<String at parser.py:3040>' Str_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3040
Str_4 := Create_String ("`"); 
--# expr-done 919
Concat_Result_3 := Concat_String (Concat_Result_2, Str_4); 
   If_Result_2 := Concat_Result_3;
end if;

      Inc_Ref (If_Result_2);


--# expr-done 920
--# property-call-start LktNode.error
Fld_23 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Fld_20.Node, Message => If_Result_2);
--# end
--# expr-done 921



--# expr-start 923 '<FieldAccess for name at parser.py:3046>' Fld_25 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3046



--# expr-start 922 '<FieldAccess for actual at parser.py:3046>' Fld_24 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3046






   




Fld_24 := Item_46.Actual;
--# expr-done 922


   

      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_25 := Create_Internal_Entity_Ref_Id (Node => Fld_24.Node.Param_F_Name, Info => Fld_24.Info);
--# expr-done 923
Var_Expr_25 := Fld_25;





if Var_Expr_25 /= No_Entity_Ref_Id then
   



   --# scope-start


   

--# expr-start 924 '<FieldAccess for exempt at parser.py:3046>' Fld_26 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3046






   

      if Var_Expr_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.exempt
Fld_26 := Liblktlang.Implementation.Lkt_Node_P_Exempt (Node => Var_Expr_25.Node);
--# end
--# expr-done 924
Scope_Result_5 := Fld_26;
   Inc_Ref (Scope_Result_5);


   --# end
      Finalizer_Scope_478;


   Result_Var_4 := Scope_Result_5;
else
   
   Result_Var_4 := No_Semantic_Result;
end if;

      Inc_Ref (Result_Var_4);


Array_Lit := Create_Internal_Semantic_Result_Array (Internal_Internal_Semantic_Result_Array'(1 => Fld_23, 2 => Result_Var_4)); 
--# expr-done 925
   If_Result_4 := Array_Lit;
else
   --# expr-start 937 '<FieldAccess for error at parser.py:3048>' Fld_32 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3048






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 936 '<If at parser.py:3049>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3049



--# expr-start 926 '<IsNull at parser.py:3050>' Is_Equal_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3050


Is_Equal_1 := Equivalent (Block_Var, No_Semantic_Result); 
--# expr-done 926
if Is_Equal_1 then
   --# expr-start 927 '<String at parser.py:3052>' Str_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3052
Str_5 := Create_String ("No value for parameter"); 
--# expr-done 927
   If_Result_3 := Str_5;
else
   --# expr-start 928 '<String at parser.py:3054>' Str_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3054
Str_6 := Create_String ("No value for parameter `"); 
--# expr-done 928
--# expr-start 931 '<FieldAccess for image at parser.py:3055>' Fld_29 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3055



--# expr-start 930 '<FieldAccess for formal_name at parser.py:3055>' Fld_28 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3055



--# expr-start 929 '<FieldAccess for formal at parser.py:3055>' Fld_27 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3055






   




Fld_27 := Item_46.Formal;
--# expr-done 929


   




Fld_28 := Fld_27.Formal_Name;
--# expr-done 930


   




Fld_29 := Liblktlang.Implementation.Image (Self => Fld_28);
--# expr-done 931
Concat_Result_4 := Concat_String (Str_6, Fld_29); 
--# expr-start 932 '<String at parser.py:3056>' Str_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3056
Str_7 := Create_String ("` in call to `"); 
--# expr-done 932
Concat_Result_5 := Concat_String (Concat_Result_4, Str_7); 
--# expr-start 934 '<FieldAccess for full_name at parser.py:3057>' Fld_31 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3057



--# expr-start 933 '<FieldAccess for called_decl at parser.py:3057>' Fld_30 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3057






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.called_decl
Fld_30 := Liblktlang.Implementation.Call_Expr_P_Called_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 933


   

      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_31 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Fld_30.Node);
--# end
--# expr-done 934
Concat_Result_6 := Concat_String (Concat_Result_5, Fld_31); 
--# expr-start 935 '<String at parser.py:3058>' Str_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3058
Str_8 := Create_String ("`"); 
--# expr-done 935
Concat_Result_7 := Concat_String (Concat_Result_6, Str_8); 
   If_Result_3 := Concat_Result_7;
end if;

      Inc_Ref (If_Result_3);


--# expr-done 936
--# property-call-start LktNode.error
Fld_32 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => If_Result_3);
--# end
--# expr-done 937
                Singleton_2 := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton_2.Items (1) := Fld_32;
                Inc_Ref (Singleton_2.Items (1));
            
   If_Result_4 := Singleton_2;
end if;

      Inc_Ref (If_Result_4);


--# expr-done 938
      

         for Item_To_Append of
               If_Result_4.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;

   
   
   

            
   --# end
      Finalizer_Scope_477;



         end loop;
      end;
   

   

         Map_Result_1 := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_1_Vec);

      end;
   



--# expr-done 939
   Result_Var_5 := Map_Result_1;
end if;

      Inc_Ref (Result_Var_5);


   If_Result_6 := Result_Var_5;
end if;

      Inc_Ref (If_Result_6);


--# expr-done 940
Let_Result := If_Result_6; Inc_Ref (Let_Result);
--# end
--# expr-done 941

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_229;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_229;
                     Finalizer_Scope_452;
                     Finalizer_Scope_453;
                     Finalizer_Scope_454;
                     Finalizer_Scope_455;
                     Finalizer_Scope_475;
                     Finalizer_Scope_477;
                     Finalizer_Scope_478;




            raise;
      end;



   return Property_Result;
end Call_Expr_P_Check_Correctness_Pre;
--# end

   







--# property-start CallExpr.infer_actuals_impl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3067
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Infer_Actuals_Impl
  
  (Node : Bare_Call_Expr
      ; Generic_Decl : Internal_Entity_Generic_Decl
      ; Gen_Type : Internal_Entity_Type_Decl
      ; Expected_Type : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Infer_Instantiation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind generic_decl Generic_Decl
   --# bind gen_type Gen_Type
   --# bind expected_type Expected_Type

   Property_Result : Internal_Infer_Instantiation;

      
            procedure Finalizer_Scope_230 with Inline_Always;
            procedure Finalizer_Scope_479 with Inline_Always;
            procedure Finalizer_Scope_480 with Inline_Always;
            procedure Finalizer_Scope_481 with Inline_Always;
            procedure Finalizer_Scope_482 with Inline_Always;
            procedure Finalizer_Scope_484 with Inline_Always;
            procedure Finalizer_Scope_485 with Inline_Always;
            procedure Finalizer_Scope_486 with Inline_Always;
            procedure Finalizer_Scope_487 with Inline_Always;

      Match_0 : Internal_Entity_Generic_Formal_Type_Decl;
Match_1 : Internal_Entity_Instantiated_Generic_Type;
Match_2 : Internal_Entity_Type_Decl;
Var_Expr : Internal_Entity_Instantiated_Generic_Type;
Var_Expr_1 : Internal_Infer_Instantiation_Array_Access;
Var_Expr_2 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld : Bare_Generic_Formal_Type_Decl;
Fld_1 : Bare_Type_Decl;
New_Struct : Internal_Generic_Param_Assoc;
Singleton : Internal_Generic_Param_Assoc_Array_Access;
New_Struct_1 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld_2 : Internal_Entity_Generic_Decl;
Fld_3 : Bare_Generic_Decl;
Fld_4 : Bare_Generic_Decl;
Cast_Result : Bare_Generic_Decl;
Is_Equal : Boolean;
If_Result : Internal_Infer_Instantiation := No_Infer_Instantiation;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Instantiated_Generic_Type;
Fld_5 : Bare_Type_Decl;
Fld_6 : Bare_Type_Decl;
Cast_Result_2 : Bare_Type_Decl;
Is_Equal_1 : Boolean;
Not_Val : Boolean;
Fld_7 : Internal_Semantic_Result := No_Semantic_Result;
New_Struct_2 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld_8 : Bare_Type_Decl_Array_Access;
Index_49 : Integer;
As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Type_Decl;
Fld_9 : Bare_Type_Decl_Array_Access;
Get_Result : Bare_Type_Decl;
As_Entity_1 : Internal_Entity_Type_Decl;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Type_Decl;
Fld_10 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Map_Result : Internal_Infer_Instantiation_Array_Access;
Fld_11 : Internal_Semantic_Result := No_Semantic_Result;
Is_Equal_2 : Boolean;
Not_Val_1 : Boolean;
Find_Result : Internal_Infer_Instantiation := No_Infer_Instantiation;
Scope_Result : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld_12 : Internal_Generic_Param_Assoc_Array_Access;
Map_Result_1 : Internal_Generic_Param_Assoc_Array_Access;
New_Struct_3 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Result_Var : Internal_Infer_Instantiation := No_Infer_Instantiation;
Scope_Result_1 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Result_Var_1 : Internal_Infer_Instantiation := No_Infer_Instantiation;
If_Result_3 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Scope_Result_2 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld_13 : Internal_Semantic_Result := No_Semantic_Result;
New_Struct_4 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Result_Var_2 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Match_Prefix : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Generic_Formal_Type_Decl;
Let_Result : Internal_Infer_Instantiation := No_Infer_Instantiation;
Scope_Result_3 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Cast_Result_4 : Internal_Entity_Instantiated_Generic_Type;
Let_Result_1 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Scope_Result_4 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Cast_Result_5 : Internal_Entity_Type_Decl;
Let_Result_2 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Scope_Result_5 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Match_Result : Internal_Infer_Instantiation := No_Infer_Instantiation;
Let_Result_3 : Internal_Infer_Instantiation := No_Infer_Instantiation;

            procedure Finalizer_Scope_230 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_230;
            procedure Finalizer_Scope_479 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_479;
            procedure Finalizer_Scope_480 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Fld_13);
                     Dec_Ref (New_Struct_4);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_480;
            procedure Finalizer_Scope_481 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_481;
            procedure Finalizer_Scope_482 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_482;
            procedure Finalizer_Scope_484 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Find_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (New_Struct_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_484;
            procedure Finalizer_Scope_485 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_485;
            procedure Finalizer_Scope_486 is
            begin
                     Dec_Ref (Var_Expr_2);
            end Finalizer_Scope_486;
            procedure Finalizer_Scope_487 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_487;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 976 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 975 '<Match at parser.py:3084>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3084









   

      if Gen_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Gen_Type; 

case Lkt_Type_Decl (Match_Prefix.Node.Kind) is
         when Lkt_Generic_Formal_Type_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_3 := Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result_3; 
--# bind gen_form Match_0
--# expr-start 950 '<If at parser.py:3087>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3087



--# expr-start 945 '<Eq at parser.py:3088>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3088





--# expr-start 943 '<FieldAccess for node at parser.py:3088>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3088



--# expr-start 942 '<FieldAccess for generic_decl at parser.py:3088>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3088






   

      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start GenericFormalTypeDecl.generic_decl
Fld_2 := Liblktlang.Implementation.Generic_Formal_Type_Decl_P_Generic_Decl (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 942


   




Fld_3 := Fld_2.Node;
--# expr-done 943



   
      Cast_Result := Fld_3;


--# expr-start 944 '<FieldAccess for node at parser.py:3088>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3088






   




Fld_4 := Generic_Decl.Node;
--# expr-done 944
Is_Equal := Cast_Result = Fld_4; 
--# expr-done 945
if Is_Equal then
   --# expr-start 949 '<New InferInstantiation at parser.py:3089>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3089

--# expr-start 948 '<New GenericParamAssoc at parser.py:3090>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3090
--# expr-start 946 '<FieldAccess for node at parser.py:3092>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3092






   




Fld_1 := Expected_Type.Node;
--# expr-done 946
--# expr-start 947 '<FieldAccess for node at parser.py:3091>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3091






   




Fld := Match_0.Node;
--# expr-done 947
New_Struct := (Actual => Fld_1, Formal => Fld); 
--# expr-done 948
                Singleton := Create_Internal_Generic_Param_Assoc_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
                
            
Inc_Ref (No_Semantic_Result);
Inc_Ref (Singleton);
New_Struct_1 := (Error => No_Semantic_Result, Result => Singleton); 
--# expr-done 949
   If_Result := New_Struct_1;
else
   
   If_Result := raise Property_Error with "should not happen";
end if;

      Inc_Ref (If_Result);


--# expr-done 950
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
Scope_Result_3 := Let_Result;
   Inc_Ref (Scope_Result_3);


   --# end
      Finalizer_Scope_479;


            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Lkt_Instantiated_Generic_Type =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_4 := Create_Internal_Entity_Instantiated_Generic_Type
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_4; 
--# bind inst Match_1



--# expr-start 951 '<Cast to InstantiatedGenericType at parser.py:3105>' Cast_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3105






Cast_Expr := Expected_Type; 



   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Lkt_Instantiated_Generic_Type_Range
   then
      
      Cast_Result_1 := Create_Internal_Entity_Instantiated_Generic_Type
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);

   else
         Cast_Result_1 := No_Entity_Instantiated_Generic_Type;
   end if;


--# expr-done 951
Var_Expr := Cast_Result_1;





if Var_Expr /= No_Entity_Instantiated_Generic_Type then
   



   --# scope-start


   --# bind etype Var_Expr

--# expr-start 972 '<Cond at parser.py:3106>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3106



--# expr-start 955 '<Not at parser.py:3109>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3109
--# expr-start 954 '<Eq at parser.py:3109>' Is_Equal_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3109





--# expr-start 952 '<FieldAccess for inner_type_decl at parser.py:3109>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3109






   

      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Match_1.Node.Instantiated_Generic_Type_Inner_Type_Decl;
--# expr-done 952



   
      Cast_Result_2 := Fld_5;


--# expr-start 953 '<FieldAccess for inner_type_decl at parser.py:3109>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3109






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_6 := Var_Expr.Node.Instantiated_Generic_Type_Inner_Type_Decl;
--# expr-done 953
Is_Equal_1 := Cast_Result_2 = Fld_6; 
--# expr-done 954
Not_Val := not (Is_Equal_1); 
--# expr-done 955
if Not_Val then
   --# expr-start 957 '<New InferInstantiation at parser.py:3110>' New_Struct_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3110
--# expr-start 956 '<FieldAccess for internal_error at parser.py:3110>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3110






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.internal_error
Fld_7 := Liblktlang.Implementation.Lkt_Node_P_Internal_Error (Node => Self);
--# end
--# expr-done 956

Inc_Ref (Fld_7);
Inc_Ref (No_Internal_Generic_Param_Assoc_Array_Type);
New_Struct_2 := (Error => Fld_7, Result => No_Internal_Generic_Param_Assoc_Array_Type); 
--# expr-done 957
   If_Result_3 := New_Struct_2;
else
   


--# expr-start 964 '<Map at parser.py:3115>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3115








   

   --# expr-start 958 '<FieldAccess for actuals at parser.py:3115>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3115






   

      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_8 := Match_1.Node.Instantiated_Generic_Type_Actuals;
Inc_Ref (Fld_8);
--# expr-done 958

   
      declare
         Map_Result_Vec : Internal_Infer_Instantiation_Vectors.Vector;
      begin
   

   

      
         Index_49 := 0;

      declare
         
         Collection : constant Bare_Type_Decl_Array_Access := Fld_8;
      begin
         for Item_48 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind actual Item_48
               --# bind i Index_49

            
         
         
      --# expr-start 963 '<FieldAccess for infer_actuals_impl at parser.py:3116>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3116






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# expr-start 959 '<as_bare_entity at parser.py:3118>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3118




Is_Null := Item_48 = null; 
if Is_Null then
   
   If_Result_1 := No_Entity_Type_Decl;
else
   

As_Entity := (Info => No_Entity_Info, Node => Item_48); 
   If_Result_1 := As_Entity;
end if;



--# expr-done 959
--# expr-start 962 '<as_bare_entity at parser.py:3119>' If_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3119



--# expr-start 961 '<at at parser.py:3119>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3119

--# expr-start 960 '<FieldAccess for actuals at parser.py:3119>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3119






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_9 := Var_Expr.Node.Instantiated_Generic_Type_Actuals;
Inc_Ref (Fld_9);
--# expr-done 960


Get_Result := Get (Self, Fld_9, Index_49, True); 
--# expr-done 961
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   
   If_Result_2 := No_Entity_Type_Decl;
else
   

As_Entity_1 := (Info => No_Entity_Info, Node => Get_Result); 
   If_Result_2 := As_Entity_1;
end if;



--# expr-done 962
--# property-call-start CallExpr.infer_actuals_impl
Fld_10 := Liblktlang.Implementation.Call_Expr_P_Infer_Actuals_Impl (Node => Ent.Node, Generic_Decl => Generic_Decl, Gen_Type => If_Result_1, Expected_Type => If_Result_2, E_Info => Ent.Info);
--# end
--# expr-done 963
      

         declare
            Item_To_Append : constant Internal_Infer_Instantiation := Fld_10;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Infer_Instantiation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_482;



               Index_49 := Index_49 + 1;
         end loop;
      end;
   

   

         Map_Result := Create_Internal_Infer_Instantiation_Array
           (Items_Count => Natural (Internal_Infer_Instantiation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Infer_Instantiation_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Infer_Instantiation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Infer_Instantiation_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 964
Var_Expr_1 := Map_Result;

      Inc_Ref (Var_Expr_1);




if Var_Expr_1 /= No_Internal_Infer_Instantiation_Array_Type then
   



   --# scope-start


   --# bind subresults Var_Expr_1




--# expr-start 968 '<Find at parser.py:3124>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3124








   

   

   
      Find_Result := No_Infer_Instantiation;
   

   

      

      declare
         
         Collection : constant Internal_Infer_Instantiation_Array_Access := Var_Expr_1;
      begin
         for Item_50 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind sr Item_50

            
      --# expr-start 967 '<Not at parser.py:3124>' Not_Val_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3124
--# expr-start 966 '<IsNull at parser.py:3124>' Is_Equal_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3124
--# expr-start 965 '<FieldAccess for error at parser.py:3124>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3124






   




Fld_11 := Item_50.Error;
Inc_Ref (Fld_11);
--# expr-done 965

Is_Equal_2 := Equivalent (Fld_11, No_Semantic_Result); 
--# expr-done 966
Not_Val_1 := not (Is_Equal_2); 
--# expr-done 967
      if Not_Val_1 then
         Find_Result := Item_50; Inc_Ref (Find_Result);
      end if;
   

            
   --# end
      Finalizer_Scope_485;


               exit when Not_Val_1;

         end loop;
      end;
   

   
   



--# expr-done 968
Var_Expr_2 := Find_Result;

      Inc_Ref (Var_Expr_2);




if Var_Expr_2 /= No_Infer_Instantiation then
   



   --# scope-start


   --# bind sr Var_Expr_2


Scope_Result := Var_Expr_2;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_486;


   Result_Var := Scope_Result;
else
   --# expr-start 971 '<New InferInstantiation at parser.py:3127>' New_Struct_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3127

--# expr-start 970 '<Mapcat at parser.py:3128>' Map_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3128








   

   

   
      declare
         Map_Result_1_Vec : Internal_Generic_Param_Assoc_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Infer_Instantiation_Array_Access := Var_Expr_1;
      begin
         for Item_51 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind sr Item_51

            
         
         
      --# expr-start 969 '<FieldAccess for result at parser.py:3128>' Fld_12 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3128






   




Fld_12 := Item_51.Result;
Inc_Ref (Fld_12);
--# expr-done 969
      

         for Item_To_Append of
               Fld_12.Items
         loop
            Internal_Generic_Param_Assoc_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;

   
   
   

            
   --# end
      Finalizer_Scope_487;



         end loop;
      end;
   

   

         Map_Result_1 := Create_Internal_Generic_Param_Assoc_Array
           (Items_Count => Natural (Internal_Generic_Param_Assoc_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Generic_Param_Assoc_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Generic_Param_Assoc_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Generic_Param_Assoc_Vectors.Destroy (Map_Result_1_Vec);

      end;
   



--# expr-done 970
Inc_Ref (No_Semantic_Result);
Inc_Ref (Map_Result_1);
New_Struct_3 := (Error => No_Semantic_Result, Result => Map_Result_1); 
--# expr-done 971
   Result_Var := New_Struct_3;
end if;

      Inc_Ref (Result_Var);


Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_484;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Infer_Instantiation;
end if;

      Inc_Ref (Result_Var_1);


   If_Result_3 := Result_Var_1;
end if;

      Inc_Ref (If_Result_3);


--# expr-done 972
Scope_Result_2 := If_Result_3;
   Inc_Ref (Scope_Result_2);


   --# end
      Finalizer_Scope_481;


   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 974 '<New InferInstantiation at parser.py:3135>' New_Struct_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3135
--# expr-start 973 '<FieldAccess for internal_error at parser.py:3135>' Fld_13 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3135






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.internal_error
Fld_13 := Liblktlang.Implementation.Lkt_Node_P_Internal_Error (Node => Self);
--# end
--# expr-done 973

Inc_Ref (Fld_13);
Inc_Ref (No_Internal_Generic_Param_Assoc_Array_Type);
New_Struct_4 := (Error => Fld_13, Result => No_Internal_Generic_Param_Assoc_Array_Type); 
--# expr-done 974
   Result_Var_2 := New_Struct_4;
end if;

      Inc_Ref (Result_Var_2);


Let_Result_1 := Result_Var_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_4 := Let_Result_1;
   Inc_Ref (Scope_Result_4);


   --# end
      Finalizer_Scope_480;


            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
         when Lkt_Enum_Class_Alt_Decl .. Lkt_Function_Type | Lkt_Class_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_5 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_2 := Cast_Result_5; 
--# bind _ Match_2

Let_Result_2 := No_Infer_Instantiation; Inc_Ref (Let_Result_2);
--# end
Scope_Result_5 := Let_Result_2;
   Inc_Ref (Scope_Result_5);


   --# end


            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
end case;

--# expr-done 975
Let_Result_3 := Match_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 976

         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_230;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_230;
                     Finalizer_Scope_479;
                     Finalizer_Scope_480;
                     Finalizer_Scope_481;
                     Finalizer_Scope_482;
                     Finalizer_Scope_484;
                     Finalizer_Scope_485;
                     Finalizer_Scope_486;
                     Finalizer_Scope_487;




            raise;
      end;



   return Property_Result;
end Call_Expr_P_Infer_Actuals_Impl;
--# end

   







--# property-start CallExpr.infer_actuals /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3144
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Infer_Actuals
  
  (Node : Bare_Call_Expr
      ; Generic_Decl : Internal_Entity_Generic_Decl
      ; Expected_Type : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Infer_Instantiation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind generic_decl Generic_Decl
   --# bind expected_type Expected_Type

   Property_Result : Internal_Infer_Instantiation;

      
            procedure Finalizer_Scope_231 with Inline_Always;
            procedure Finalizer_Scope_489 with Inline_Always;
            procedure Finalizer_Scope_491 with Inline_Always;
            procedure Finalizer_Scope_492 with Inline_Always;

      Block_Var : Internal_Infer_Instantiation := No_Infer_Instantiation;
Var_Expr : Internal_Infer_Instantiation := No_Infer_Instantiation;
Var_Expr_1 : Internal_Generic_Param_Assoc;
Fld : Internal_Entity_Decl;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld_3 : Internal_Semantic_Result := No_Semantic_Result;
Is_Equal : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Semantic_Result := No_Semantic_Result;
Fld_5 : String_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Str : String_Type;
Fld_6 : Internal_Entity_Decl;
Fld_7 : Internal_Entity_Type_Decl;
Fld_8 : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;
Fld_9 : String_Type;
Concat_Result_2 : String_Type;
Str_2 : String_Type;
Concat_Result_3 : String_Type;
Fld_10 : Internal_Semantic_Result := No_Semantic_Result;
New_Struct : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld_11 : Internal_Generic_Param_Assoc_Array_Access;
Len : Integer;
Fld_12 : Internal_Entity_Generic_Formal_Type_Decl_Array_Access;
Len_1 : Integer;
Comp_Result : Boolean;
Str_3 : String_Type;
Fld_13 : Internal_Semantic_Result := No_Semantic_Result;
New_Struct_1 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld_14 : Internal_Generic_Param_Assoc_Array_Access;
Fld_15 : Internal_Generic_Param_Assoc_Array_Access;
Fld_16 : Bare_Generic_Formal_Type_Decl;
Fld_17 : Bare_Generic_Formal_Type_Decl;
Cast_Result : Bare_Generic_Formal_Type_Decl;
Is_Equal_2 : Boolean;
Fld_18 : Bare_Type_Decl;
Fld_19 : Bare_Type_Decl;
Cast_Result_1 : Bare_Type_Decl;
Is_Equal_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Find_Result : Internal_Generic_Param_Assoc;
Str_4 : String_Type;
Fld_20 : Bare_Generic_Formal_Type_Decl;
Fld_21 : String_Type;
Concat_Result_4 : String_Type;
Str_5 : String_Type;
Concat_Result_5 : String_Type;
Fld_22 : Bare_Type_Decl;
Fld_23 : String_Type;
Concat_Result_6 : String_Type;
Str_6 : String_Type;
Concat_Result_7 : String_Type;
Fld_24 : Bare_Type_Decl;
Fld_25 : String_Type;
Concat_Result_8 : String_Type;
Fld_26 : Internal_Semantic_Result := No_Semantic_Result;
New_Struct_2 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Scope_Result : Internal_Infer_Instantiation := No_Infer_Instantiation;
Result_Var : Internal_Infer_Instantiation := No_Infer_Instantiation;
Map_Result : Internal_Infer_Instantiation_Array_Access;
Get_Result : Internal_Infer_Instantiation := No_Infer_Instantiation;
Scope_Result_1 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Result_Var_1 : Internal_Infer_Instantiation := No_Infer_Instantiation;
If_Result_2 : Internal_Infer_Instantiation := No_Infer_Instantiation;
If_Result_3 : Internal_Infer_Instantiation := No_Infer_Instantiation;
Let_Result : Internal_Infer_Instantiation := No_Infer_Instantiation;

            procedure Finalizer_Scope_231 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Str_2);
                     Dec_Ref (Concat_Result_3);
                     Dec_Ref (Fld_10);
                     Dec_Ref (New_Struct);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Str_3);
                     Dec_Ref (Fld_13);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_231;
            procedure Finalizer_Scope_489 is
            begin
                     Dec_Ref (Fld_15);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_489;
            procedure Finalizer_Scope_491 is
            begin
                     Dec_Ref (Str_4);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Concat_Result_4);
                     Dec_Ref (Str_5);
                     Dec_Ref (Concat_Result_5);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Concat_Result_6);
                     Dec_Ref (Str_6);
                     Dec_Ref (Concat_Result_7);
                     Dec_Ref (Fld_25);
                     Dec_Ref (Concat_Result_8);
                     Dec_Ref (Fld_26);
                     Dec_Ref (New_Struct_2);
            end Finalizer_Scope_491;
            procedure Finalizer_Scope_492 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_492;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1026 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 979 '<FieldAccess for infer_actuals_impl at parser.py:3153>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3153






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# expr-start 978 '<FieldAccess for called_decl_type at parser.py:3155>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3155



--# expr-start 977 '<FieldAccess for decl at parser.py:3155>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3155






   

      if Generic_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Decl (Node => Generic_Decl.Node.Generic_Decl_F_Decl, Info => Generic_Decl.Info);
--# expr-done 977


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.called_decl_type
Fld_1 := Liblktlang.Implementation.Decl_P_Called_Decl_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 978

--# property-call-start CallExpr.infer_actuals_impl
Fld_2 := Liblktlang.Implementation.Call_Expr_P_Infer_Actuals_Impl (Node => Ent.Node, Generic_Decl => Generic_Decl, Gen_Type => Fld_1, Expected_Type => Expected_Type, E_Info => Ent.Info);
--# end
--# expr-done 979
Block_Var := Fld_2; Inc_Ref (Block_Var);
--# bind unsorted_actuals Block_Var
--# expr-start 1025 '<Cond at parser.py:3158>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3158






--# expr-start 982 '<Not at parser.py:3161>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3161
--# expr-start 981 '<IsNull at parser.py:3161>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3161
--# expr-start 980 '<FieldAccess for error at parser.py:3161>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3161






   




Fld_3 := Block_Var.Error;
Inc_Ref (Fld_3);
--# expr-done 980

Is_Equal := Equivalent (Fld_3, No_Semantic_Result); 
--# expr-done 981
Not_Val := not (Is_Equal); 
--# expr-done 982
if Not_Val then
   --# expr-start 985 '<IsNull at parser.py:3162>' Is_Equal_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3162
--# expr-start 984 '<FieldAccess for error_message at parser.py:3162>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3162



--# expr-start 983 '<FieldAccess for error at parser.py:3162>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3162






   




Fld_4 := Block_Var.Error;
Inc_Ref (Fld_4);
--# expr-done 983


   




Fld_5 := Fld_4.Error_Message;
Inc_Ref (Fld_5);
--# expr-done 984

Is_Equal_1 := Equivalent (Fld_5, Empty_String); 
--# expr-done 985
   If_Result := Is_Equal_1;
else
   
   If_Result := False;
end if;



if If_Result then
   --# expr-start 994 '<New InferInstantiation at parser.py:3163>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3163
--# expr-start 993 '<FieldAccess for error at parser.py:3163>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3163






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 986 '<String at parser.py:3164>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3164
Str := Create_String ("Cannot instantiate callee: cannot unify `"); 
--# expr-done 986
--# expr-start 989 '<FieldAccess for full_name at parser.py:3165>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3165



--# expr-start 988 '<FieldAccess for called_decl_type at parser.py:3165>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3165



--# expr-start 987 '<FieldAccess for decl at parser.py:3165>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3165






   

      if Generic_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_6 := Create_Internal_Entity_Decl (Node => Generic_Decl.Node.Generic_Decl_F_Decl, Info => Generic_Decl.Info);
--# expr-done 987


   

      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.called_decl_type
Fld_7 := Liblktlang.Implementation.Decl_P_Called_Decl_Type (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 988


   

      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_8 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Fld_7.Node);
--# end
--# expr-done 989
Concat_Result := Concat_String (Str, Fld_8); 
--# expr-start 990 '<String at parser.py:3166>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3166
Str_1 := Create_String ("` with `"); 
--# expr-done 990
Concat_Result_1 := Concat_String (Concat_Result, Str_1); 
--# expr-start 991 '<FieldAccess for full_name at parser.py:3166>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3166






   

      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_9 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Expected_Type.Node);
--# end
--# expr-done 991
Concat_Result_2 := Concat_String (Concat_Result_1, Fld_9); 
--# expr-start 992 '<String at parser.py:3167>' Str_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3167
Str_2 := Create_String ("`"); 
--# expr-done 992
Concat_Result_3 := Concat_String (Concat_Result_2, Str_2); 
--# property-call-start LktNode.error
Fld_10 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Concat_Result_3);
--# end
--# expr-done 993

Inc_Ref (Fld_10);
Inc_Ref (No_Internal_Generic_Param_Assoc_Array_Type);
New_Struct := (Error => Fld_10, Result => No_Internal_Generic_Param_Assoc_Array_Type); 
--# expr-done 994
   If_Result_3 := New_Struct;
else
   


--# expr-start 996 '<length at parser.py:3171>' Len /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3171
--# expr-start 995 '<FieldAccess for result at parser.py:3171>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3171






   




Fld_11 := Block_Var.Result;
Inc_Ref (Fld_11);
--# expr-done 995
Len := Length (Fld_11); 
--# expr-done 996
--# expr-start 998 '<length at parser.py:3172>' Len_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3172
--# expr-start 997 '<FieldAccess for generic_formals at parser.py:3172>' Fld_12 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3172






   

      if Generic_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start GenericDecl.generic_formals
Fld_12 := Liblktlang.Implementation.Generic_Decl_P_Generic_Formals (Node => Generic_Decl.Node, E_Info => Generic_Decl.Info);
--# end
--# expr-done 997
Len_1 := Length (Fld_12); 
--# expr-done 998
Comp_Result := Len < Len_1; 
if Comp_Result then
   --# expr-start 1001 '<New InferInstantiation at parser.py:3173>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3173
--# expr-start 1000 '<FieldAccess for error at parser.py:3174>' Fld_13 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3174






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 999 '<String at parser.py:3175>' Str_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3175
Str_3 := Create_String ("Not enough information to infer instantiation of generic callee"); 
--# expr-done 999
--# property-call-start LktNode.error
Fld_13 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Str_3);
--# end
--# expr-done 1000

Inc_Ref (Fld_13);
Inc_Ref (No_Internal_Generic_Param_Assoc_Array_Type);
New_Struct_1 := (Error => Fld_13, Result => No_Internal_Generic_Param_Assoc_Array_Type); 
--# expr-done 1001
   If_Result_2 := New_Struct_1;
else
   


--# expr-start 1024 '<at at parser.py:3194>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3194

--# expr-start 1023 '<Map at parser.py:3181>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3181








   

   --# expr-start 1002 '<FieldAccess for result at parser.py:3181>' Fld_14 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3181






   




Fld_14 := Block_Var.Result;
Inc_Ref (Fld_14);
--# expr-done 1002

   
      declare
         Map_Result_Vec : Internal_Infer_Instantiation_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Generic_Param_Assoc_Array_Access := Fld_14;
      begin
         for Item_52 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind a Item_52

            
         
         
      


--# expr-start 1011 '<Find at parser.py:3182>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3182








   

   --# expr-start 1003 '<FieldAccess for result at parser.py:3182>' Fld_15 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3182






   




Fld_15 := Block_Var.Result;
Inc_Ref (Fld_15);
--# expr-done 1003

   
      Find_Result := No_Generic_Param_Assoc;
   

   

      

      declare
         
         Collection : constant Internal_Generic_Param_Assoc_Array_Access := Fld_15;
      begin
         for Item_53 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind a2 Item_53

            
      


--# expr-start 1006 '<Eq at parser.py:3184>' Is_Equal_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3184





--# expr-start 1004 '<FieldAccess for formal at parser.py:3184>' Fld_16 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3184






   




Fld_16 := Item_52.Formal;
--# expr-done 1004



   
      Cast_Result := Fld_16;


--# expr-start 1005 '<FieldAccess for formal at parser.py:3184>' Fld_17 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3184






   




Fld_17 := Item_53.Formal;
--# expr-done 1005
Is_Equal_2 := Cast_Result = Fld_17; 
--# expr-done 1006
if Is_Equal_2 then
   --# expr-start 1010 '<Not at parser.py:3184>' Not_Val_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3184
--# expr-start 1009 '<Eq at parser.py:3184>' Is_Equal_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3184





--# expr-start 1007 '<FieldAccess for actual at parser.py:3184>' Fld_18 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3184






   




Fld_18 := Item_52.Actual;
--# expr-done 1007



   
      Cast_Result_1 := Fld_18;


--# expr-start 1008 '<FieldAccess for actual at parser.py:3184>' Fld_19 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3184






   




Fld_19 := Item_53.Actual;
--# expr-done 1008
Is_Equal_3 := Cast_Result_1 = Fld_19; 
--# expr-done 1009
Not_Val_1 := not (Is_Equal_3); 
--# expr-done 1010
   If_Result_1 := Not_Val_1;
else
   
   If_Result_1 := False;
end if;



      if If_Result_1 then
         Find_Result := Item_53; 
      end if;
   

            
   --# end


               exit when If_Result_1;

         end loop;
      end;
   

   
   



--# expr-done 1011
Var_Expr_1 := Find_Result;





if Var_Expr_1 /= No_Generic_Param_Assoc then
   



   --# scope-start


   --# bind a2 Var_Expr_1

--# expr-start 1022 '<New InferInstantiation at parser.py:3185>' New_Struct_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3185
--# expr-start 1021 '<FieldAccess for error at parser.py:3186>' Fld_26 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3186






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1012 '<String at parser.py:3187>' Str_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3187
Str_4 := Create_String ("Instantiation error: conflicting actual types for formal "); 
--# expr-done 1012
--# expr-start 1014 '<FieldAccess for quoted_name at parser.py:3189>' Fld_21 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3189



--# expr-start 1013 '<FieldAccess for formal at parser.py:3189>' Fld_20 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3189






   




Fld_20 := Item_52.Formal;
--# expr-done 1013


   

      if Fld_20 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.quoted_name
Fld_21 := Liblktlang.Implementation.Decl_P_Quoted_Name (Node => Fld_20);
--# end
--# expr-done 1014
Concat_Result_4 := Concat_String (Str_4, Fld_21); 
--# expr-start 1015 '<String at parser.py:3189>' Str_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3189
Str_5 := Create_String (": "); 
--# expr-done 1015
Concat_Result_5 := Concat_String (Concat_Result_4, Str_5); 
--# expr-start 1017 '<FieldAccess for quoted_name at parser.py:3190>' Fld_23 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3190



--# expr-start 1016 '<FieldAccess for actual at parser.py:3190>' Fld_22 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3190






   




Fld_22 := Item_52.Actual;
--# expr-done 1016


   

      if Fld_22 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.quoted_name
Fld_23 := Liblktlang.Implementation.Decl_P_Quoted_Name (Node => Fld_22);
--# end
--# expr-done 1017
Concat_Result_6 := Concat_String (Concat_Result_5, Fld_23); 
--# expr-start 1018 '<String at parser.py:3190>' Str_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3190
Str_6 := Create_String (" and "); 
--# expr-done 1018
Concat_Result_7 := Concat_String (Concat_Result_6, Str_6); 
--# expr-start 1020 '<FieldAccess for quoted_name at parser.py:3191>' Fld_25 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3191



--# expr-start 1019 '<FieldAccess for actual at parser.py:3191>' Fld_24 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3191






   




Fld_24 := Var_Expr_1.Actual;
--# expr-done 1019


   

      if Fld_24 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.quoted_name
Fld_25 := Liblktlang.Implementation.Decl_P_Quoted_Name (Node => Fld_24);
--# end
--# expr-done 1020
Concat_Result_8 := Concat_String (Concat_Result_7, Fld_25); 
--# property-call-start LktNode.error
Fld_26 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Concat_Result_8);
--# end
--# expr-done 1021

Inc_Ref (Fld_26);
Inc_Ref (No_Internal_Generic_Param_Assoc_Array_Type);
New_Struct_2 := (Error => Fld_26, Result => No_Internal_Generic_Param_Assoc_Array_Type); 
--# expr-done 1022
Scope_Result := New_Struct_2;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_491;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Infer_Instantiation;
end if;

      Inc_Ref (Result_Var);


      

         declare
            Item_To_Append : constant Internal_Infer_Instantiation := Result_Var;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Infer_Instantiation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_489;



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Infer_Instantiation_Array
           (Items_Count => Natural (Internal_Infer_Instantiation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Infer_Instantiation_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Infer_Instantiation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Infer_Instantiation_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1023


Get_Result := Get (Self, Map_Result, 0, True); 
--# expr-done 1024
Var_Expr := Get_Result;

      Inc_Ref (Var_Expr);




if Var_Expr /= No_Infer_Instantiation then
   



   --# scope-start


   --# bind e Var_Expr


Scope_Result_1 := Var_Expr;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_492;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := Block_Var;
end if;

      Inc_Ref (Result_Var_1);


   If_Result_2 := Result_Var_1;
end if;

      Inc_Ref (If_Result_2);


   If_Result_3 := If_Result_2;
end if;

      Inc_Ref (If_Result_3);


--# expr-done 1025
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 1026

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_231;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_231;
                     Finalizer_Scope_489;
                     Finalizer_Scope_491;
                     Finalizer_Scope_492;




            raise;
      end;



   return Property_Result;
end Call_Expr_P_Infer_Actuals;
--# end

   







--# property-start CallExpr.check_infer_actuals /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3202
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_Infer_Actuals
  
  (Node : Bare_Call_Expr
      ; Generic_Decl : Internal_Entity_Generic_Decl
      ; Expected_Type : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Bare_Type_Decl_Array_Access
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   

   --# bind generic_decl Generic_Decl
   --# bind expected_type Expected_Type

   Property_Result : Bare_Type_Decl_Array_Access;

      
            procedure Finalizer_Scope_232 with Inline_Always;
            procedure Finalizer_Scope_493 with Inline_Always;

      Block_Var : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld : Internal_Infer_Instantiation := No_Infer_Instantiation;
Fld_1 : Internal_Entity_Generic_Formal_Type_Decl_Array_Access;
Fld_2 : Internal_Generic_Param_Assoc_Array_Access;
Fld_3 : Bare_Generic_Formal_Type_Decl;
Fld_4 : Bare_Generic_Formal_Type_Decl;
Cast_Result : Bare_Generic_Formal_Type_Decl;
Is_Equal : Boolean;
Find_Result : Internal_Generic_Param_Assoc;
Fld_5 : Bare_Type_Decl;
Map_Result : Bare_Type_Decl_Array_Access;
Fld_6 : Internal_Semantic_Result := No_Semantic_Result;
Is_Equal_1 : Boolean;
If_Result : Bare_Type_Decl_Array_Access;
Let_Result : Bare_Type_Decl_Array_Access;

            procedure Finalizer_Scope_232 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_232;
            procedure Finalizer_Scope_493 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_493;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1039 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1027 '<FieldAccess for infer_actuals at parser.py:3213>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3213






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;





--# property-call-start CallExpr.infer_actuals
Fld := Liblktlang.Implementation.Call_Expr_P_Infer_Actuals (Node => Ent.Node, Generic_Decl => Generic_Decl, Expected_Type => Expected_Type, E_Info => Ent.Info);
--# end
--# expr-done 1027
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind unsorted_actuals Block_Var
--# expr-start 1038 '<If at parser.py:3218>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3218



--# expr-start 1029 '<IsNull at parser.py:3219>' Is_Equal_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3219
--# expr-start 1028 '<FieldAccess for error at parser.py:3219>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3219






   




Fld_6 := Block_Var.Error;
Inc_Ref (Fld_6);
--# expr-done 1028

Is_Equal_1 := Equivalent (Fld_6, No_Semantic_Result); 
--# expr-done 1029
if Is_Equal_1 then
   --# expr-start 1037 '<Map at parser.py:3220>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3220








   

   --# expr-start 1030 '<FieldAccess for generic_formals at parser.py:3220>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3220






   

      if Generic_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start GenericDecl.generic_formals
Fld_1 := Liblktlang.Implementation.Generic_Decl_P_Generic_Formals (Node => Generic_Decl.Node, E_Info => Generic_Decl.Info);
--# end
--# expr-done 1030

   
      declare
         Map_Result_Vec : Bare_Type_Decl_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Entity_Generic_Formal_Type_Decl_Array_Access := Fld_1;
      begin
         for Item_54 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind gen_form Item_54

            
         
         
      --# expr-start 1036 '<FieldAccess for actual at parser.py:3223>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3223



--# expr-start 1035 '<Find at parser.py:3221>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3221








   

   --# expr-start 1031 '<FieldAccess for result at parser.py:3221>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3221






   




Fld_2 := Block_Var.Result;
Inc_Ref (Fld_2);
--# expr-done 1031

   
      Find_Result := No_Generic_Param_Assoc;
   

   

      

      declare
         
         Collection : constant Internal_Generic_Param_Assoc_Array_Access := Fld_2;
      begin
         for Item_55 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind a Item_55

            
      --# expr-start 1034 '<Eq at parser.py:3222>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3222





--# expr-start 1032 '<FieldAccess for formal at parser.py:3222>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3222






   




Fld_3 := Item_55.Formal;
--# expr-done 1032



   
      Cast_Result := Fld_3;


--# expr-start 1033 '<FieldAccess for node at parser.py:3222>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3222






   




Fld_4 := Item_54.Node;
--# expr-done 1033
Is_Equal := Cast_Result = Fld_4; 
--# expr-done 1034
      if Is_Equal then
         Find_Result := Item_55; 
      end if;
   

            
   --# end


               exit when Is_Equal;

         end loop;
      end;
   

   
   



--# expr-done 1035


   




Fld_5 := Find_Result.Actual;
--# expr-done 1036
      

         declare
            Item_To_Append : constant Bare_Type_Decl := Fld_5;
         begin
            Bare_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_493;



         end loop;
      end;
   

   

         Map_Result := Create_Bare_Type_Decl_Array
           (Items_Count => Natural (Bare_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1037
   If_Result := Map_Result;
else
   
   If_Result := raise Property_Error with "Generic inference error";
end if;

      Inc_Ref (If_Result);


--# expr-done 1038
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1039

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_232;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_232;
                     Finalizer_Scope_493;




            raise;
      end;



   return Property_Result;
end Call_Expr_P_Check_Infer_Actuals;
--# end

   







--# property-start CallExpr.called_object_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3228
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Called_Object_Type
  
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Block_Var : Internal_Entity_Type_Decl;
Match_0 : Internal_Entity_Function_Type;
Match_1 : Internal_Entity_Type_Decl;
Var_Expr_26 : Internal_Entity_Fun_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Decl;
Sym : Symbol_Type;
Fld_2 : Internal_Entity_Fun_Decl;
Fld_3 : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Function_Type;
Cast_Expr : Internal_Entity_Function_Type;
Cast_Result_1 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity_Type_Decl;
Let_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_2 : Internal_Entity_Type_Decl;
Match_Result : Internal_Entity_Type_Decl;
Let_Result_2 : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 1046 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 1041 '<FieldAccess for check_expr_type at parser.py:3233>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3233



--# expr-start 1040 '<FieldAccess for name at parser.py:3233>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3233






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 1040


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_expr_type
Fld_1 := Liblktlang.Implementation.Expr_P_Check_Expr_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1041
Block_Var := Fld_1; 
--# bind exp_type Block_Var
--# expr-start 1045 '<Match at parser.py:3234>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3234









   

      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Block_Var; 

case Lkt_Type_Decl (Match_Prefix.Node.Kind) is
         when Lkt_Function_Type =>
            



   --# scope-start



--# scope-start









   
      Cast_Result := Create_Internal_Entity_Function_Type
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result; 
--# bind fun_type Match_0






Cast_Expr := Match_0; 



   
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);


Let_Result := Cast_Result_1; 
--# end
Scope_Result_1 := Let_Result;


   --# end


            Match_Result := Scope_Result_1; 
         when Lkt_Enum_Class_Alt_Decl | Lkt_Generic_Formal_Type_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_2; 
--# bind t Match_1



--# expr-start 1043 '<FieldAccess for get_fun at parser.py:3236>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3236






   

      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1042 '<Symbol __call__ at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Call); 
--# expr-done 1042
--# property-call-start TypeDecl.get_fun
Fld_2 := Liblktlang.Implementation.Type_Decl_P_Get_Fun (Node => Match_1.Node, Name => Sym, E_Info => Match_1.Info);
--# end
--# expr-done 1043
Var_Expr_26 := Fld_2;





if Var_Expr_26 /= No_Entity_Fun_Decl then
   



   --# scope-start


   

--# expr-start 1044 '<FieldAccess for get_type at parser.py:3236>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3236






   

      if Var_Expr_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]BaseValDecl.get_type'
Fld_3 := Liblktlang.Implementation.Dispatcher_Base_Val_Decl_P_Get_Type (Node => Var_Expr_26.Node, E_Info => Var_Expr_26.Info);
--# end
--# expr-done 1044
Scope_Result := Fld_3;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Type_Decl;
end if;



Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;


   --# end


            Match_Result := Scope_Result_2; 
end case;

--# expr-done 1045
Let_Result_2 := Match_Result; 
--# end
--# expr-done 1046

         Property_Result := Let_Result_2;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Call_Expr_P_Called_Object_Type;
--# end

   







--# property-start CallExpr.called_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3239
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Called_Decl
  
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Decl
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Decl;

      
            procedure Finalizer_Scope_234 with Inline_Always;
            procedure Finalizer_Scope_456 with Inline_Always;
            procedure Finalizer_Scope_466 with Inline_Always;
            procedure Finalizer_Scope_467 with Inline_Always;

      Block_Var : Internal_Entity_Decl;
Block_Var_1 : Internal_Entity_Decl;
Var_Expr : Internal_Entity_Decl;
Var_Expr_1 : Internal_Semantic_Result := No_Semantic_Result;
Var_Expr_27 : Internal_Entity_Fun_Decl;
Var_Expr_2 : Internal_Entity_Decl;
Match_0 : Internal_Entity_Fun_Decl;
Match_1 : Internal_Entity_Base_Val_Decl;
Match_2 : Internal_Entity_Decl;
Var_Expr_3 : Internal_Entity_Fun_Decl;
Var_Expr_28 : Internal_Entity_Base_Val_Decl;
Var_Expr_29 : Internal_Entity_Type_Decl;
Match_0_1 : Internal_Entity_Type_Decl;
Match_1_1 : Internal_Entity_Fun_Decl;
Match_2_1 : Internal_Entity_Decl;
Var_Expr_4 : Internal_Entity_Generic_Decl;
Var_Expr_5 : Bare_Type_Decl_Array_Access;
Var_Expr_30 : Internal_Entity_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;
Fld_2 : Internal_Entity_Decl;
Scope_Result : Internal_Entity_Decl;
Result_Var : Internal_Entity_Decl;
Cast_Expr : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Fun_Decl;
Fld_3 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Internal_Entity_Decl;
Scope_Result_2 : Internal_Entity_Decl;
Result_Var_2 : Internal_Entity_Decl;
Fld_4 : Internal_Entity_Expr;
Is_A : Boolean;
Not_Val : Boolean;
If_Result_1 : Internal_Entity_Decl;
Fld_5 : Internal_Entity_Type_Decl;
Sym : Symbol_Type;
Fld_6 : Internal_Entity_Fun_Decl;
Scope_Result_3 : Internal_Entity_Fun_Decl;
Cast_Result_1 : Internal_Entity_Decl;
Result_Var_3 : Internal_Entity_Decl;
Scope_Result_4 : Internal_Entity_Decl;
Result_Var_4 : Internal_Entity_Decl;
Scope_Result_5 : Internal_Entity_Decl;
Result_Var_5 : Internal_Entity_Decl;
Match_Prefix : Internal_Entity_Decl;
Cast_Result_2 : Internal_Entity_Fun_Decl;
Let_Result : Internal_Entity_Decl;
Scope_Result_6 : Internal_Entity_Decl;
Cast_Result_3 : Internal_Entity_Base_Val_Decl;
Let_Result_1 : Internal_Entity_Decl;
Scope_Result_7 : Internal_Entity_Decl;
Cast_Result_4 : Internal_Entity_Decl;
Let_Result_2 : Internal_Entity_Decl;
Scope_Result_8 : Internal_Entity_Decl;
Match_Result : Internal_Entity_Decl;
Scope_Result_9 : Internal_Entity_Decl;
Result_Var_6 : Internal_Entity_Decl;
Cast_Expr_1 : Internal_Entity_Decl;
Cast_Result_5 : Internal_Entity_Generic_Decl;
Fld_7 : Internal_Entity_Type_Decl;
Fld_8 : Bare_Type_Decl_Array_Access;
Fld_9 : Internal_Entity_Decl;
Scope_Result_10 : Internal_Entity_Decl;
Result_Var_7 : Internal_Entity_Decl;
Scope_Result_11 : Internal_Entity_Decl;
Result_Var_8 : Internal_Entity_Decl;
Match_Prefix_1 : Internal_Entity_Decl;
Cast_Result_6 : Internal_Entity_Type_Decl;
Cast_Expr_2 : Internal_Entity_Type_Decl;
Cast_Result_7 : Internal_Entity_Decl;
Let_Result_3 : Internal_Entity_Decl;
Scope_Result_12 : Internal_Entity_Decl;
Cast_Result_8 : Internal_Entity_Fun_Decl;
Cast_Expr_3 : Internal_Entity_Fun_Decl;
Cast_Result_9 : Internal_Entity_Decl;
Let_Result_4 : Internal_Entity_Decl;
Scope_Result_13 : Internal_Entity_Decl;
Cast_Result_10 : Internal_Entity_Decl;
Let_Result_5 : Internal_Entity_Decl;
Scope_Result_14 : Internal_Entity_Decl;
Match_Result_1 : Internal_Entity_Decl;
Scope_Result_15 : Internal_Entity_Decl;
Result_Var_9 : Internal_Entity_Decl;
Let_Result_6 : Internal_Entity_Decl;

            procedure Finalizer_Scope_234 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_234;
            procedure Finalizer_Scope_456 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_456;
            procedure Finalizer_Scope_466 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_466;
            procedure Finalizer_Scope_467 is
            begin
                     Dec_Ref (Var_Expr_5);
            end Finalizer_Scope_467;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 1066 '<Block at ???>' Let_Result_6 None
--# scope-start
--# expr-start 1056 '<If at parser.py:3256>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3256



--# expr-start 1049 '<Not at parser.py:3261>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3261
--# expr-start 1048 '<IsA CallExpr at parser.py:3261>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3261
--# expr-start 1047 '<FieldAccess for name at parser.py:3261>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3261






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 1047
Is_A := Fld_4.Node /= null 
and then Fld_4.Node.Kind in Lkt_Call_Expr_Range; 
--# expr-done 1048
Not_Val := not (Is_A); 
--# expr-done 1049
if Not_Val then
   





--# expr-start 1051 '<FieldAccess for referenced_decl at parser.py:3263>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3263



--# expr-start 1050 '<FieldAccess for name at parser.py:3263>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3263






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 1050


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1051
Var_Expr_1 := Fld_1;

      Inc_Ref (Var_Expr_1);




if Var_Expr_1 /= No_Semantic_Result then
   



   --# scope-start


   --# bind sr Var_Expr_1

--# expr-start 1052 '<FieldAccess for result_decl at parser.py:3264>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3264






   




Fld_2 := Var_Expr_1.Result_Decl;
--# expr-done 1052
Scope_Result := Fld_2;


   --# end
      Finalizer_Scope_456;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Decl;
end if;



Var_Expr := Result_Var;





if Var_Expr /= No_Entity_Decl then
   



   --# scope-start


   --# bind decl Var_Expr

--# expr-start 1055 '<If at parser.py:3265>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3265






--# expr-start 1053 '<Cast to FunDecl at parser.py:3266>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3266






Cast_Expr := Var_Expr; 



   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Lkt_Fun_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Fun_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);

   else
         Cast_Result := No_Entity_Fun_Decl;
   end if;


--# expr-done 1053
Var_Expr_27 := Cast_Result;





if Var_Expr_27 /= No_Entity_Fun_Decl then
   



   --# scope-start


   

--# expr-start 1054 '<FieldAccess for is_property at parser.py:3266>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3266






   

      if Var_Expr_27.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start FunDecl.is_property
Fld_3 := Liblktlang.Implementation.Fun_Decl_P_Is_Property (Node => Var_Expr_27.Node, E_Info => Var_Expr_27.Info);
--# end
--# expr-done 1054
Scope_Result_1 := Fld_3;


   --# end


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := False;
end if;



if Result_Var_1 then
   
   If_Result := No_Entity_Decl;
else
   
   If_Result := Var_Expr;
end if;



--# expr-done 1055
Scope_Result_2 := If_Result;


   --# end


   Result_Var_2 := Scope_Result_2;
else
   
   Result_Var_2 := No_Entity_Decl;
end if;



   If_Result_1 := Result_Var_2;
else
   
   If_Result_1 := No_Entity_Decl;
end if;



--# expr-done 1056
Block_Var := If_Result_1; 
--# bind refd_decl Block_Var




Var_Expr_2 := Block_Var;





if Var_Expr_2 /= No_Entity_Decl then
   



   --# scope-start


   --# bind refd_decl Var_Expr_2

--# expr-start 1060 '<Match at parser.py:3277>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3277









   

      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Var_Expr_2; 

case Lkt_Decl (Match_Prefix.Node.Kind) is
         when Lkt_Fun_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_2 := Create_Internal_Entity_Fun_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result_2; 
--# bind _ Match_0

Let_Result := Var_Expr_2; 
--# end
Scope_Result_6 := Let_Result;


   --# end


            Match_Result := Scope_Result_6; 
         when Lkt_Node_Decl .. Lkt_Val_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_3 := Create_Internal_Entity_Base_Val_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_3; 
--# bind v Match_1




Var_Expr_28 := Match_1;





if Var_Expr_28 /= No_Entity_Base_Val_Decl then
   



   --# scope-start


   




--# expr-start 1057 '<FieldAccess for get_type at parser.py:3283>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3283






   

      if Var_Expr_28.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]BaseValDecl.get_type'
Fld_5 := Liblktlang.Implementation.Dispatcher_Base_Val_Decl_P_Get_Type (Node => Var_Expr_28.Node, E_Info => Var_Expr_28.Info);
--# end
--# expr-done 1057
Var_Expr_29 := Fld_5;





if Var_Expr_29 /= No_Entity_Type_Decl then
   



   --# scope-start


   




--# expr-start 1059 '<FieldAccess for get_fun at parser.py:3283>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3283






   

      if Var_Expr_29.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1058 '<Symbol __call__ at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Call); 
--# expr-done 1058
--# property-call-start TypeDecl.get_fun
Fld_6 := Liblktlang.Implementation.Type_Decl_P_Get_Fun (Node => Var_Expr_29.Node, Name => Sym, E_Info => Var_Expr_29.Info);
--# end
--# expr-done 1059
Var_Expr_3 := Fld_6;





if Var_Expr_3 /= No_Entity_Fun_Decl then
   








   --# scope-start


   --# bind a Var_Expr_3


Scope_Result_3 := Var_Expr_3;


   --# end





   
      Cast_Result_1 := Create_Internal_Entity_Decl
        (Node => Scope_Result_3.Node,
         Info => Scope_Result_3.Info);


   Result_Var_3 := Cast_Result_1;
else
   
   Result_Var_3 := Var_Expr_2;
end if;



Scope_Result_4 := Result_Var_3;


   --# end


   Result_Var_4 := Scope_Result_4;
else
   
   Result_Var_4 := No_Entity_Decl;
end if;



Scope_Result_5 := Result_Var_4;


   --# end


   Result_Var_5 := Scope_Result_5;
else
   
   Result_Var_5 := No_Entity_Decl;
end if;



Let_Result_1 := Result_Var_5; 
--# end
Scope_Result_7 := Let_Result_1;


   --# end


            Match_Result := Scope_Result_7; 
         when Lkt_Grammar_Rule_Decl .. Lkt_Synthetic_Lexer_Decl | Lkt_Env_Spec_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_4 := Create_Internal_Entity_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_2 := Cast_Result_4; 
--# bind _ Match_2

Let_Result_2 := Var_Expr_2; 
--# end
Scope_Result_8 := Let_Result_2;


   --# end


            Match_Result := Scope_Result_8; 
end case;

--# expr-done 1060
Scope_Result_9 := Match_Result;


   --# end


   Result_Var_6 := Scope_Result_9;
else
   
   Result_Var_6 := No_Entity_Decl;
end if;



Block_Var_1 := Result_Var_6; 
--# bind called_decl Block_Var_1






--# expr-start 1061 '<Cast to GenericDecl at parser.py:3290>' Cast_Result_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3290






Cast_Expr_1 := Block_Var_1; 



   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Lkt_Generic_Decl_Range
   then
      
      Cast_Result_5 := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);

   else
         Cast_Result_5 := No_Entity_Generic_Decl;
   end if;


--# expr-done 1061
Var_Expr_4 := Cast_Result_5;





if Var_Expr_4 /= No_Entity_Generic_Decl then
   



   --# scope-start


   --# bind gd Var_Expr_4




--# expr-start 1063 '<FieldAccess for check_infer_actuals at parser.py:3293>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3293






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# expr-start 1062 '<FieldAccess for expected_type at parser.py:3293>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3293






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_7 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1062
--# property-call-start CallExpr.check_infer_actuals
Fld_8 := Liblktlang.Implementation.Call_Expr_P_Check_Infer_Actuals (Node => Ent.Node, Generic_Decl => Var_Expr_4, Expected_Type => Fld_7, E_Info => Ent.Info);
--# end
--# expr-done 1063
Var_Expr_5 := Fld_8;

      Inc_Ref (Var_Expr_5);




if Var_Expr_5 /= No_Bare_Type_Decl_Array_Type then
   



   --# scope-start


   --# bind actuals Var_Expr_5

--# expr-start 1064 '<FieldAccess for instantiate at parser.py:3297>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3297






   

      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start GenericDecl.instantiate
Fld_9 := Liblktlang.Implementation.Generic_Decl_P_Instantiate (Node => Var_Expr_4.Node, Actuals => Var_Expr_5, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 1064
Scope_Result_10 := Fld_9;


   --# end
      Finalizer_Scope_467;


   Result_Var_7 := Scope_Result_10;
else
   
   Result_Var_7 := No_Entity_Decl;
end if;



Scope_Result_11 := Result_Var_7;


   --# end
      Finalizer_Scope_466;


   Result_Var_8 := Scope_Result_11;
else
   
   Result_Var_8 := Block_Var_1;
end if;



Var_Expr_30 := Result_Var_8;





if Var_Expr_30 /= No_Entity_Decl then
   



   --# scope-start


   

--# expr-start 1065 '<Match at parser.py:3301>' Match_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3301









   

      if Var_Expr_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix_1 := Var_Expr_30; 

case Lkt_Decl (Match_Prefix_1.Node.Kind) is
         when Lkt_Enum_Class_Alt_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_6 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_0_1 := Cast_Result_6; 
--# bind t Match_0_1






Cast_Expr_2 := Match_0_1; 



   
      Cast_Result_7 := Create_Internal_Entity_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);


Let_Result_3 := Cast_Result_7; 
--# end
Scope_Result_12 := Let_Result_3;


   --# end


            Match_Result_1 := Scope_Result_12; 
         when Lkt_Fun_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_8 := Create_Internal_Entity_Fun_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_1_1 := Cast_Result_8; 
--# bind f Match_1_1






Cast_Expr_3 := Match_1_1; 



   
      Cast_Result_9 := Create_Internal_Entity_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);


Let_Result_4 := Cast_Result_9; 
--# end
Scope_Result_13 := Let_Result_4;


   --# end


            Match_Result_1 := Scope_Result_13; 
         when Lkt_Grammar_Rule_Decl .. Lkt_Val_Decl | Lkt_Env_Spec_Decl .. Lkt_Lexer_Family_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_10 := Create_Internal_Entity_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_2_1 := Cast_Result_10; 
--# bind _ Match_2_1

Let_Result_5 := No_Entity_Decl; 
--# end
Scope_Result_14 := Let_Result_5;


   --# end


            Match_Result_1 := Scope_Result_14; 
end case;

--# expr-done 1065
Scope_Result_15 := Match_Result_1;


   --# end


   Result_Var_9 := Scope_Result_15;
else
   
   Result_Var_9 := No_Entity_Decl;
end if;



Let_Result_6 := Result_Var_9; 
--# end
--# expr-done 1066

         Property_Result := Let_Result_6;
         
   --# end
      Finalizer_Scope_234;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_234;
                     Finalizer_Scope_456;
                     Finalizer_Scope_466;
                     Finalizer_Scope_467;




            raise;
      end;



   return Property_Result;
end Call_Expr_P_Called_Decl;
--# end

   







--# property-start CallExpr.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3320
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Expr_Context_Free_Type
  
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Var_Expr : Internal_Entity_Type_Decl;
Var_Expr_1 : Internal_Entity_Decl;
Match_0 : Internal_Entity_Fun_Decl;
Match_1 : Internal_Entity_Type_Decl;
Match_2 : Internal_Entity_Decl;
Var_Expr_2 : Internal_Entity_Type_Decl;
Match_0_1 : Internal_Entity_Function_Type;
Match_1_1 : Internal_Entity_Type_Decl;
Var_Expr_31 : Internal_Entity_Fun_Decl;
Fld : Internal_Entity_Decl;
Fld_1 : Internal_Entity_Type_Ref;
Fld_2 : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Fun_Decl;
Let_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Decl;
Let_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity_Decl;
Let_Result_2 : Internal_Entity_Type_Decl;
Scope_Result_2 : Internal_Entity_Type_Decl;
Match_Result : Internal_Entity_Type_Decl;
Scope_Result_3 : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Scope_Result_4 : Internal_Entity_Type_Decl;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Type_Decl;
Fld_5 : Bare_Type_Decl;
As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Sym : Symbol_Type;
Fld_6 : Internal_Entity_Fun_Decl;
Fld_7 : Internal_Entity_Type_Ref;
Fld_8 : Internal_Entity_Type_Decl;
Scope_Result_5 : Internal_Entity_Type_Decl;
Result_Var_1 : Internal_Entity_Type_Decl;
Match_Prefix_1 : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Function_Type;
Let_Result_3 : Internal_Entity_Type_Decl;
Scope_Result_6 : Internal_Entity_Type_Decl;
Cast_Result_4 : Internal_Entity_Type_Decl;
Let_Result_4 : Internal_Entity_Type_Decl;
Scope_Result_7 : Internal_Entity_Type_Decl;
Match_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_8 : Internal_Entity_Type_Decl;
Result_Var_2 : Internal_Entity_Type_Decl;
Result_Var_3 : Internal_Entity_Type_Decl;
Let_Result_5 : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1080 '<Block at ???>' Let_Result_5 None
--# scope-start






--# expr-start 1067 '<FieldAccess for called_decl at parser.py:3322>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3322






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.called_decl
Fld := Liblktlang.Implementation.Call_Expr_P_Called_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1067
Var_Expr_1 := Fld;





if Var_Expr_1 /= No_Entity_Decl then
   



   --# scope-start


   --# bind rd Var_Expr_1

--# expr-start 1070 '<Match at parser.py:3327>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3327









   

      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Var_Expr_1; 

case Lkt_Decl (Match_Prefix.Node.Kind) is
         when Lkt_Fun_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result := Create_Internal_Entity_Fun_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result; 
--# bind fd Match_0
--# expr-start 1069 '<FieldAccess for designated_type at parser.py:3328>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3328



--# expr-start 1068 '<FieldAccess for return_type at parser.py:3328>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3328






   

      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Type_Ref (Node => Match_0.Node.Fun_Decl_F_Return_Type, Info => Match_0.Info);
--# expr-done 1068


   

      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_2 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 1069
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;


   --# end


            Match_Result := Scope_Result; 
         when Lkt_Enum_Class_Alt_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_1; 
--# bind td Match_1

Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;


   --# end


            Match_Result := Scope_Result_1; 
         when Lkt_Grammar_Rule_Decl .. Lkt_Val_Decl | Lkt_Env_Spec_Decl .. Lkt_Lexer_Family_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_2 := Create_Internal_Entity_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_2 := Cast_Result_2; 
--# bind _ Match_2

Let_Result_2 := raise Property_Error with "should not happen"; 
--# end
Scope_Result_2 := Let_Result_2;


   --# end


            Match_Result := Scope_Result_2; 
end case;

--# expr-done 1070
Scope_Result_3 := Match_Result;


   --# end


   Result_Var := Scope_Result_3;
else
   
   Result_Var := No_Entity_Type_Decl;
end if;



Var_Expr := Result_Var;





if Var_Expr /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind e Var_Expr


Scope_Result_4 := Var_Expr;


   --# end


   Result_Var_3 := Scope_Result_4;
else
   


--# expr-start 1072 '<FieldAccess for expr_context_free_type at parser.py:3336>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3336



--# expr-start 1071 '<FieldAccess for name at parser.py:3336>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3336






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 1071


   

      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld_4 := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 1072
Var_Expr_2 := Fld_4;





if Var_Expr_2 /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind t Var_Expr_2

--# expr-start 1079 '<Match at parser.py:3337>' Match_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3337









   

      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix_1 := Var_Expr_2; 

case Lkt_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Lkt_Function_Type =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_3 := Create_Internal_Entity_Function_Type
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_0_1 := Cast_Result_3; 
--# bind ft Match_0_1
--# expr-start 1074 '<as_entity at parser.py:3338>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3338



--# expr-start 1073 '<FieldAccess for return_type at parser.py:3338>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3338






   

      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Match_0_1.Node.Function_Type_Return_Type;
--# expr-done 1073
Is_Null := Fld_5 = null; 
if Is_Null then
   
   If_Result := No_Entity_Type_Decl;
else
   

As_Entity := (Info => E_Info, Node => Fld_5); 
   If_Result := As_Entity;
end if;



--# expr-done 1074
Let_Result_3 := If_Result; 
--# end
Scope_Result_6 := Let_Result_3;


   --# end


            Match_Result_1 := Scope_Result_6; 
         when Lkt_Enum_Class_Alt_Decl | Lkt_Generic_Formal_Type_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_4 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_1_1 := Cast_Result_4; 
--# bind t Match_1_1



--# expr-start 1076 '<FieldAccess for get_fun at parser.py:3340>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3340






   

      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1075 '<Symbol __call__ at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Call); 
--# expr-done 1075
--# property-call-start TypeDecl.get_fun
Fld_6 := Liblktlang.Implementation.Type_Decl_P_Get_Fun (Node => Match_1_1.Node, Name => Sym, E_Info => Match_1_1.Info);
--# end
--# expr-done 1076
Var_Expr_31 := Fld_6;





if Var_Expr_31 /= No_Entity_Fun_Decl then
   



   --# scope-start


   

--# expr-start 1078 '<FieldAccess for designated_type at parser.py:3340>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3340



--# expr-start 1077 '<FieldAccess for return_type at parser.py:3340>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3340






   

      if Var_Expr_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_7 := Create_Internal_Entity_Type_Ref (Node => Var_Expr_31.Node.Fun_Decl_F_Return_Type, Info => Var_Expr_31.Info);
--# expr-done 1077


   

      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_8 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 1078
Scope_Result_5 := Fld_8;


   --# end


   Result_Var_1 := Scope_Result_5;
else
   
   Result_Var_1 := No_Entity_Type_Decl;
end if;



Let_Result_4 := Result_Var_1; 
--# end
Scope_Result_7 := Let_Result_4;


   --# end


            Match_Result_1 := Scope_Result_7; 
end case;

--# expr-done 1079
Scope_Result_8 := Match_Result_1;


   --# end


   Result_Var_2 := Scope_Result_8;
else
   
   Result_Var_2 := No_Entity_Type_Decl;
end if;



   Result_Var_3 := Result_Var_2;
end if;



Let_Result_5 := Result_Var_3; 
--# end
--# expr-done 1080

         Property_Result := Let_Result_5;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Call_Expr_P_Expr_Context_Free_Type;
--# end

   







--# property-start CallExpr.formals /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3345
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Formals
  
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Formal_Param_Array_Access
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Formal_Param_Array_Access;

      
            procedure Finalizer_Scope_237 with Inline_Always;
            procedure Finalizer_Scope_504 with Inline_Always;
            procedure Finalizer_Scope_505 with Inline_Always;
            procedure Finalizer_Scope_507 with Inline_Always;
            procedure Finalizer_Scope_510 with Inline_Always;

      Var_Expr : Internal_Entity_Decl;
Match_0 : Internal_Entity_Function_Type;
Match_1 : Internal_Entity_Type_Decl;
Match_0_1 : Internal_Entity_Fun_Decl;
Match_1_1 : Internal_Entity_Type_Decl;
Match_2 : Internal_Entity_Decl;
Fld : Internal_Entity_Decl;
Fld_1 : Internal_Entity_Fun_Arg_Decl_List;
Fld_2 : Bare_Fun_Arg_Decl_List;
Fld_3 : Internal_Entity_Info;
Item_57 : Internal_Entity_Fun_Arg_Decl;
Bare_Item_57 : Bare_Fun_Arg_Decl;
Cast_Expr : Internal_Entity_Fun_Arg_Decl;
Cast_Result : Internal_Entity_Component_Decl;
Fld_4 : Internal_Formal_Param;
As_Entity : Internal_Entity_Fun_Arg_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Fun_Arg_Decl;
Map_Result : Internal_Formal_Param_Array_Access;
Fld_5 : Internal_Entity_Component_Decl_Array_Access;
Fld_6 : Internal_Formal_Param;
Map_Result_1 : Internal_Formal_Param_Array_Access;
Match_Prefix : Internal_Entity_Decl;
Cast_Result_1 : Internal_Entity_Fun_Decl;
Let_Result : Internal_Formal_Param_Array_Access;
Scope_Result : Internal_Formal_Param_Array_Access;
Cast_Result_2 : Internal_Entity_Type_Decl;
Let_Result_1 : Internal_Formal_Param_Array_Access;
Scope_Result_1 : Internal_Formal_Param_Array_Access;
Cast_Result_3 : Internal_Entity_Decl;
Let_Result_2 : Internal_Formal_Param_Array_Access;
Scope_Result_2 : Internal_Formal_Param_Array_Access;
Match_Result : Internal_Formal_Param_Array_Access;
Scope_Result_3 : Internal_Formal_Param_Array_Access;
Fld_7 : Internal_Entity_Type_Decl;
Fld_8 : Bare_Type_Decl_Array_Access;
As_Entity_1 : Internal_Entity_Type_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Type_Decl;
New_Struct : Internal_Formal_Param;
Map_Result_2 : Internal_Formal_Param_Array_Access;
Match_Prefix_1 : Internal_Entity_Type_Decl;
Cast_Result_4 : Internal_Entity_Function_Type;
Let_Result_3 : Internal_Formal_Param_Array_Access;
Scope_Result_4 : Internal_Formal_Param_Array_Access;
Cast_Result_5 : Internal_Entity_Type_Decl;
Let_Result_4 : Internal_Formal_Param_Array_Access;
Scope_Result_5 : Internal_Formal_Param_Array_Access;
Match_Result_1 : Internal_Formal_Param_Array_Access;
Result_Var : Internal_Formal_Param_Array_Access;
Let_Result_5 : Internal_Formal_Param_Array_Access;

            procedure Finalizer_Scope_237 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Match_Result_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_237;
            procedure Finalizer_Scope_504 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_504;
            procedure Finalizer_Scope_505 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_505;
            procedure Finalizer_Scope_507 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_507;
            procedure Finalizer_Scope_510 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_510;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1096 '<Block at ???>' Let_Result_5 None
--# scope-start



--# expr-start 1081 '<FieldAccess for called_decl at parser.py:3351>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3351






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.called_decl
Fld := Liblktlang.Implementation.Call_Expr_P_Called_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1081
Var_Expr := Fld;





if Var_Expr /= No_Entity_Decl then
   



   --# scope-start


   --# bind cd Var_Expr

--# expr-start 1089 '<Match at parser.py:3353>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3353









   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Var_Expr; 

case Lkt_Decl (Match_Prefix.Node.Kind) is
         when Lkt_Fun_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_1 := Create_Internal_Entity_Fun_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0_1 := Cast_Result_1; 
--# bind fd Match_0_1
--# expr-start 1085 '<Map at parser.py:3354>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3354








   

   --# expr-start 1082 '<FieldAccess for args at parser.py:3354>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3354






   

      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Fun_Arg_Decl_List (Node => Match_0_1.Node.Fun_Decl_F_Args, Info => Match_0_1.Info);
--# expr-done 1082






   




Fld_2 := Fld_1.Node;

   
      declare
         Map_Result_Vec : Internal_Formal_Param_Vectors.Vector;
      begin
   

   

      if Fld_2 = null then
         
      Map_Result := Create_Internal_Formal_Param_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Fun_Arg_Decl_List := Fld_2;
      begin
         for Untyped_Item_57 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_57 := Untyped_Item_57; 
                  



Is_Null := Bare_Item_57 = null; 
if Is_Null then
   
   If_Result := No_Entity_Fun_Arg_Decl;
else
   





   




Fld_3 := Fld_1.Info;

As_Entity := (Info => Fld_3, Node => Bare_Item_57); 
   If_Result := As_Entity;
end if;



                  Item_57 := If_Result; 

            
   --# scope-start


               --# bind p Item_57

            
         
         
      --# expr-start 1084 '<FieldAccess for to_formal_param at parser.py:3355>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3355



--# expr-start 1083 '<Cast to ComponentDecl at parser.py:3355>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3355






Cast_Expr := Item_57; 



   
      Cast_Result := Create_Internal_Entity_Component_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);


--# expr-done 1083


   

      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start ComponentDecl.to_formal_param
Fld_4 := Liblktlang.Implementation.Component_Decl_P_To_Formal_Param (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1084
      

         declare
            Item_To_Append : constant Internal_Formal_Param := Fld_4;
         begin
            Internal_Formal_Param_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Formal_Param_Array
           (Items_Count => Natural (Internal_Formal_Param_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Formal_Param_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Formal_Param_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Formal_Param_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1085
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_505;


            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Lkt_Enum_Class_Alt_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1_1 := Cast_Result_2; 
--# bind td Match_1_1
--# expr-start 1088 '<Map at parser.py:3358>' Map_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3358








   

   --# expr-start 1086 '<FieldAccess for fields at parser.py:3358>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3358






   

      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeDecl.fields'
Fld_5 := Liblktlang.Implementation.Dispatcher_Type_Decl_P_Fields (Node => Match_1_1.Node, E_Info => Match_1_1.Info);
--# end
--# expr-done 1086

   
      declare
         Map_Result_1_Vec : Internal_Formal_Param_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Entity_Component_Decl_Array_Access := Fld_5;
      begin
         for Item_58 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind c Item_58

            
         
         
      --# expr-start 1087 '<FieldAccess for to_formal_param at parser.py:3358>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3358






   

      if Item_58.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start ComponentDecl.to_formal_param
Fld_6 := Liblktlang.Implementation.Component_Decl_P_To_Formal_Param (Node => Item_58.Node, E_Info => Item_58.Info);
--# end
--# expr-done 1087
      

         declare
            Item_To_Append : constant Internal_Formal_Param := Fld_6;
         begin
            Internal_Formal_Param_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result_1 := Create_Internal_Formal_Param_Array
           (Items_Count => Natural (Internal_Formal_Param_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Formal_Param_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Formal_Param_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Formal_Param_Vectors.Destroy (Map_Result_1_Vec);

      end;
   



--# expr-done 1088
Let_Result_1 := Map_Result_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_507;


            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Lkt_Grammar_Rule_Decl .. Lkt_Val_Decl | Lkt_Env_Spec_Decl .. Lkt_Lexer_Family_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_3 := Create_Internal_Entity_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_2 := Cast_Result_3; 
--# bind _ Match_2

Let_Result_2 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);


   --# end


            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;

--# expr-done 1089
Scope_Result_3 := Match_Result;
   Inc_Ref (Scope_Result_3);


   --# end
      Finalizer_Scope_504;


   Result_Var := Scope_Result_3;
else
   --# expr-start 1095 '<Match at parser.py:3365>' Match_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3365






--# expr-start 1090 '<FieldAccess for called_object_type at parser.py:3365>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3365






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.called_object_type
Fld_7 := Liblktlang.Implementation.Call_Expr_P_Called_Object_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1090


   

      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix_1 := Fld_7; 

case Lkt_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Lkt_Function_Type =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_4 := Create_Internal_Entity_Function_Type
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_0 := Cast_Result_4; 
--# bind ft Match_0
--# expr-start 1094 '<Map at parser.py:3366>' Map_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3366








   

   --# expr-start 1091 '<FieldAccess for args at parser.py:3366>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3366






   

      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_8 := Match_0.Node.Function_Type_Args;
Inc_Ref (Fld_8);
--# expr-done 1091

   
      declare
         Map_Result_2_Vec : Internal_Formal_Param_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Bare_Type_Decl_Array_Access := Fld_8;
      begin
         for Item_56 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind a Item_56

            
         
         
      --# expr-start 1093 '<New FormalParam at parser.py:3367>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3367


--# expr-start 1092 '<as_entity at parser.py:3369>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3369




Is_Null_1 := Item_56 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Type_Decl;
else
   

As_Entity_1 := (Info => E_Info, Node => Item_56); 
   If_Result_1 := As_Entity_1;
end if;



--# expr-done 1092
New_Struct := (Default_Value => No_Entity_Expr, Formal_Name => No_Symbol, Formal_Type => If_Result_1); 
--# expr-done 1093
      

         declare
            Item_To_Append : constant Internal_Formal_Param := New_Struct;
         begin
            Internal_Formal_Param_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result_2 := Create_Internal_Formal_Param_Array
           (Items_Count => Natural (Internal_Formal_Param_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Formal_Param_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Formal_Param_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Formal_Param_Vectors.Destroy (Map_Result_2_Vec);

      end;
   



--# expr-done 1094
Let_Result_3 := Map_Result_2; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);


   --# end
      Finalizer_Scope_510;


            Match_Result_1 := Scope_Result_4; Inc_Ref (Match_Result_1);
         when Lkt_Enum_Class_Alt_Decl | Lkt_Generic_Formal_Type_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_5 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);


Match_1 := Cast_Result_5; 
--# bind _ Match_1

Let_Result_4 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_4);
--# end
Scope_Result_5 := Let_Result_4;
   Inc_Ref (Scope_Result_5);


   --# end


            Match_Result_1 := Scope_Result_5; Inc_Ref (Match_Result_1);
end case;

--# expr-done 1095
   Result_Var := Match_Result_1;
end if;

      Inc_Ref (Result_Var);


Let_Result_5 := Result_Var; Inc_Ref (Let_Result_5);
--# end
--# expr-done 1096

         Property_Result := Let_Result_5;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_237;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_237;
                     Finalizer_Scope_504;
                     Finalizer_Scope_505;
                     Finalizer_Scope_507;
                     Finalizer_Scope_510;




            raise;
      end;



   return Property_Result;
end Call_Expr_P_Formals;
--# end

   







--# property-start CallExpr.static_match_params /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3378
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Static_Match_Params
  
  (Node : Bare_Call_Expr
      ; Formals : Internal_Formal_Param_Array_Access
      ; Actuals : Internal_Entity_Param_Array_Access
  )

   return Internal_Param_Match_Array_Access
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
      --# bind self Self

   

   --# bind formals Formals
   --# bind actuals Actuals

   Property_Result : Internal_Param_Match_Array_Access;

      
            procedure Finalizer_Scope_238 with Inline_Always;

      Block_Var : Internal_Param_Match_Array_Access;
Block_Var_1 : Internal_Param_Match_Array_Access;
Var_Expr : Internal_Formal_Param;
Var_Expr_1 : Internal_Formal_Param;
Index_60 : Integer;
Get_Result : Internal_Formal_Param;
New_Struct : Internal_Param_Match;
Scope_Result : Internal_Param_Match;
New_Struct_1 : Internal_Param_Match;
Result_Var : Internal_Param_Match;
Fld : Symbol_Type;
Fld_1 : Internal_Entity_Ref_Id;
Fld_2 : Bare_Ref_Id;
Sym : Symbol_Type;
Is_Equal : Boolean;
Find_Result : Internal_Formal_Param;
New_Struct_2 : Internal_Param_Match;
Scope_Result_1 : Internal_Param_Match;
New_Struct_3 : Internal_Param_Match;
Result_Var_1 : Internal_Param_Match;
Fld_3 : Internal_Entity_Ref_Id;
Is_Null : Boolean;
If_Result : Internal_Param_Match;
Map_Result : Internal_Param_Match_Array_Access;
Fld_4 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Fld_5 : Internal_Formal_Param;
Is_Equal_1 : Boolean;
Find_Result_1 : Internal_Param_Match;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Map_Result_1 : Internal_Formal_Param_Array_Access;
New_Struct_4 : Internal_Param_Match;
Map_Result_2 : Internal_Param_Match_Array_Access;
Concat_Result : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;

            procedure Finalizer_Scope_238 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_238;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1120 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1110 '<Map at parser.py:3388>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3388








   

   

   
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
   

   

      
         Index_60 := 0;

      declare
         
         Collection : constant Internal_Entity_Param_Array_Access := Actuals;
      begin
         for Item_59 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind a Item_59
               --# bind i Index_60

            
         
         
      --# expr-start 1109 '<If at parser.py:3388>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3388



--# expr-start 1098 '<IsNull at parser.py:3389>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3389
--# expr-start 1097 '<FieldAccess for name at parser.py:3389>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3389






   

      if Item_59.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Ref_Id (Node => Item_59.Node.Param_F_Name, Info => Item_59.Info);
--# expr-done 1097
Is_Null := Fld_3.Node = null; 
--# expr-done 1098
if Is_Null then
   


--# expr-start 1099 '<at at parser.py:3390>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3390




Get_Result := Get (Self, Formals, Index_60, True); 
--# expr-done 1099
Var_Expr := Get_Result;





if Var_Expr /= No_Formal_Param then
   



   --# scope-start


   --# bind f Var_Expr

--# expr-start 1100 '<New ParamMatch at parser.py:3391>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3391



New_Struct := (Actual => Item_59, Formal => Var_Expr, Has_Matched => True); 
--# expr-done 1100
Scope_Result := New_Struct;


   --# end


   Result_Var := Scope_Result;
else
   --# expr-start 1101 '<New ParamMatch at parser.py:3392>' New_Struct_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3392



New_Struct_1 := (Actual => Item_59, Formal => No_Formal_Param, Has_Matched => False); 
--# expr-done 1101
   Result_Var := New_Struct_1;
end if;



   If_Result := Result_Var;
else
   


--# expr-start 1106 '<Find at parser.py:3394>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3394








   

   

   
      Find_Result := No_Formal_Param;
   

   

      

      declare
         
         Collection : constant Internal_Formal_Param_Array_Access := Formals;
      begin
         for Item_61 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind f Item_61

            
      --# expr-start 1105 '<Eq at parser.py:3394>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3394
--# expr-start 1102 '<FieldAccess for formal_name at parser.py:3394>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3394






   




Fld := Item_61.Formal_Name;
--# expr-done 1102
--# expr-start 1104 '<GetSymbol at parser.py:3394>' Sym /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3394



--# expr-start 1103 '<FieldAccess for name at parser.py:3394>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3394






   

      if Item_59.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Ref_Id (Node => Item_59.Node.Param_F_Name, Info => Item_59.Info);
--# expr-done 1103


   




Fld_2 := Fld_1.Node; 
Sym := Get_Symbol (Fld_2); 
--# expr-done 1104
Is_Equal := Fld = Sym; 
--# expr-done 1105
      if Is_Equal then
         Find_Result := Item_61; 
      end if;
   

            
   --# end


               exit when Is_Equal;

         end loop;
      end;
   

   
   



--# expr-done 1106
Var_Expr_1 := Find_Result;





if Var_Expr_1 /= No_Formal_Param then
   



   --# scope-start


   --# bind f Var_Expr_1

--# expr-start 1107 '<New ParamMatch at parser.py:3395>' New_Struct_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3395



New_Struct_2 := (Actual => Item_59, Formal => Var_Expr_1, Has_Matched => True); 
--# expr-done 1107
Scope_Result_1 := New_Struct_2;


   --# end


   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 1108 '<New ParamMatch at parser.py:3396>' New_Struct_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3396



New_Struct_3 := (Actual => Item_59, Formal => No_Formal_Param, Has_Matched => False); 
--# expr-done 1108
   Result_Var_1 := New_Struct_3;
end if;



   If_Result := Result_Var_1;
end if;



--# expr-done 1109
      

         declare
            Item_To_Append : constant Internal_Param_Match := If_Result;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



               Index_60 := Index_60 + 1;
         end loop;
      end;
   

   

         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1110
Block_Var := Map_Result; Inc_Ref (Block_Var);
--# bind actual_matches Block_Var
--# expr-start 1119 '<Map at parser.py:3407>' Map_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3407








   

   --# expr-start 1117 '<Filter at parser.py:3401>' Map_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3401








   

   

   
      declare
         Map_Result_1_Vec : Internal_Formal_Param_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Formal_Param_Array_Access := Formals;
      begin
         for Item_63 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind formal Item_63

            
         


--# expr-start 1112 '<IsNull at parser.py:3403>' Is_Null_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3403
--# expr-start 1111 '<FieldAccess for default_value at parser.py:3403>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3403






   




Fld_4 := Item_63.Default_Value;
--# expr-done 1111
Is_Null_1 := Fld_4.Node = null; 
--# expr-done 1112
if Is_Null_1 then
   --# expr-start 1116 '<IsNull at parser.py:3406>' Is_Equal_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3406
--# expr-start 1115 '<Find at parser.py:3404>' Find_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3404








   

   

   
      Find_Result_1 := No_Param_Match;
   

   

      

      declare
         
         Collection : constant Internal_Param_Match_Array_Access := Block_Var;
      begin
         for Item_64 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind pm Item_64

            
      --# expr-start 1114 '<Eq at parser.py:3405>' Is_Equal_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3405
--# expr-start 1113 '<FieldAccess for formal at parser.py:3405>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3405






   




Fld_5 := Item_64.Formal;
--# expr-done 1113

Is_Equal_1 := Fld_5 = Item_63; 
--# expr-done 1114
      if Is_Equal_1 then
         Find_Result_1 := Item_64; 
      end if;
   

            
   --# end


               exit when Is_Equal_1;

         end loop;
      end;
   

   
   



--# expr-done 1115

Is_Equal_2 := Find_Result_1 = No_Param_Match; 
--# expr-done 1116
   If_Result_1 := Is_Equal_2;
else
   
   If_Result_1 := False;
end if;



         if If_Result_1 then
            
         
      
      

         declare
            Item_To_Append : constant Internal_Formal_Param := Item_63;
         begin
            Internal_Formal_Param_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result_1 := Create_Internal_Formal_Param_Array
           (Items_Count => Natural (Internal_Formal_Param_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Formal_Param_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Formal_Param_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Formal_Param_Vectors.Destroy (Map_Result_1_Vec);

      end;
   



--# expr-done 1117

   
      declare
         Map_Result_2_Vec : Internal_Param_Match_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Formal_Param_Array_Access := Map_Result_1;
      begin
         for Item_62 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind formal Item_62

            
         
         
      --# expr-start 1118 '<New ParamMatch at parser.py:3407>' New_Struct_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3407



New_Struct_4 := (Actual => No_Entity_Param, Formal => Item_62, Has_Matched => False); 
--# expr-done 1118
      

         declare
            Item_To_Append : constant Internal_Param_Match := New_Struct_4;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result_2 := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_2_Vec);

      end;
   



--# expr-done 1119
Block_Var_1 := Map_Result_2; Inc_Ref (Block_Var_1);
--# bind formal_misses Block_Var_1


Concat_Result := Concat (Block_Var, Block_Var_1); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1120

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_238;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_238;




            raise;
      end;



   return Property_Result;
end Call_Expr_P_Static_Match_Params;
--# end

   







--# property-start CallExpr.match_params /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3411
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Match_Params
  
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Param_Match_Array_Access
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Param_Match_Array_Access;

      
            procedure Finalizer_Scope_239 with Inline_Always;

      Fld : Internal_Formal_Param_Array_Access;
Fld_1 : Internal_Entity_Param_List;
Fld_2 : Bare_Param_List;
Fld_3 : Internal_Entity_Info;
Item_89 : Internal_Entity_Param;
Bare_Item_89 : Bare_Param;
As_Entity : Internal_Entity_Param;
Is_Null : Boolean;
If_Result : Internal_Entity_Param;
Map_Result : Internal_Entity_Param_Array_Access;
Fld_4 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;

            procedure Finalizer_Scope_239 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_239;

      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Call_Expr_Call_Expr_P_Match_Params,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Call_Expr,
                                As_Bare_Call_Expr => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Internal_Param_Match_Array_Access;
                     Inc_Ref (Property_Result);

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 1125 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1124 '<FieldAccess for static_match_params at parser.py:3418>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3418






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1121 '<FieldAccess for formals at parser.py:3418>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3418






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.formals
Fld := Liblktlang.Implementation.Call_Expr_P_Formals (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1121
--# expr-start 1123 '<TakeWhile at ???>' Map_Result None








   

   --# expr-start 1122 '<FieldAccess for args at parser.py:3418>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3418






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Param_List (Node => Ent.Node.Call_Expr_F_Args, Info => Ent.Info);
--# expr-done 1122






   




Fld_2 := Fld_1.Node;

   
      declare
         Map_Result_Vec : Internal_Entity_Param_Vectors.Vector;
      begin
   

   

      if Fld_2 = null then
         
      Map_Result := Create_Internal_Entity_Param_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Param_List := Fld_2;
      begin
         for Untyped_Item_89 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_89 := Untyped_Item_89; 
                  



Is_Null := Bare_Item_89 = null; 
if Is_Null then
   
   If_Result := No_Entity_Param;
else
   





   




Fld_3 := Fld_1.Info;

As_Entity := (Info => Fld_3, Node => Bare_Item_89); 
   If_Result := As_Entity;
end if;



                  Item_89 := If_Result; 

            
   --# scope-start



            
         
         
      
      

         declare
            Item_To_Append : constant Internal_Entity_Param := Item_89;
         begin
            Internal_Entity_Param_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Entity_Param_Array
           (Items_Count => Natural (Internal_Entity_Param_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Param_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Param_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Param_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1123
--# property-call-start CallExpr.static_match_params
Fld_4 := Liblktlang.Implementation.Call_Expr_P_Static_Match_Params (Node => Self, Formals => Fld, Actuals => Map_Result);
--# end
--# expr-done 1124
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
--# expr-done 1125

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_239;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_239;


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Internal_Param_Match_Array_Access,
                        As_Internal_Param_Match_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;

         end if;
      end if;



   return Property_Result;
end Call_Expr_P_Match_Params;
--# end



   


      

   --
   --  Primitives for Bare_Cast_Expr
   --

   



      
      procedure Initialize_Fields_For_Cast_Expr
        (Self : Bare_Cast_Expr
         ; Cast_Expr_F_Expr : Bare_Expr
         ; Cast_Expr_F_Excludes_Null : Bare_Excludes_Null
         ; Cast_Expr_F_Dest_Type : Bare_Type_Ref
        ) is
      begin

            Self.Cast_Expr_F_Expr := Cast_Expr_F_Expr;
            Self.Cast_Expr_F_Excludes_Null := Cast_Expr_F_Excludes_Null;
            Self.Cast_Expr_F_Dest_Type := Cast_Expr_F_Dest_Type;
         

      end Initialize_Fields_For_Cast_Expr;

      
   function Cast_Expr_F_Expr
     (Node : Bare_Cast_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Cast_Expr_F_Expr;
      
   end;

      
   function Cast_Expr_F_Excludes_Null
     (Node : Bare_Cast_Expr) return Bare_Excludes_Null
   is
      

   begin
         
         return Node.Cast_Expr_F_Excludes_Null;
      
   end;

      
   function Cast_Expr_F_Dest_Type
     (Node : Bare_Cast_Expr) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Cast_Expr_F_Dest_Type;
      
   end;


   







--# property-start CastExpr.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2877
pragma Warnings (Off, "is not referenced");
 function Cast_Expr_P_Expr_Context_Free_Type
  
  (Node : Bare_Cast_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Cast_Expr  := Bare_Cast_Expr (Node);
     Ent : Internal_Entity_Cast_Expr :=
       Internal_Entity_Cast_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity_Type_Ref;
Fld_1 : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1127 '<FieldAccess for designated_type at parser.py:2877>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2877



--# expr-start 1126 '<FieldAccess for dest_type at parser.py:2877>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2877






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref (Node => Ent.Node.Cast_Expr_F_Dest_Type, Info => Ent.Info);
--# expr-done 1126


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1127

         Property_Result := Fld_1;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Cast_Expr_P_Expr_Context_Free_Type;
--# end

   







--# property-start CastExpr.check_legality /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2879
pragma Warnings (Off, "is not referenced");
 function Cast_Expr_P_Check_Legality
  
  (Node : Bare_Cast_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Cast_Expr  := Bare_Cast_Expr (Node);
     Ent : Internal_Entity_Cast_Expr :=
       Internal_Entity_Cast_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_226 with Inline_Always;

      Block_Var : Internal_Entity_Type_Decl;
Block_Var_1 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Internal_Entity_Type_Decl;
Str : String_Type;
Fld_3 : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
Fld_4 : Internal_Entity_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Fld_5 : Internal_Entity_Type_Decl_Array_Access;
Cast_Expr_2 : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Quantifier_Result_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_226 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_226;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1140 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1129 '<FieldAccess for check_expr_type at parser.py:2881>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2881



--# expr-start 1128 '<FieldAccess for expr at parser.py:2881>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2881






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Cast_Expr_F_Expr, Info => Ent.Info);
--# expr-done 1128


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_expr_type
Fld_1 := Liblktlang.Implementation.Expr_P_Check_Expr_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1129
Block_Var := Fld_1; 
--# bind expr_type Block_Var
--# expr-start 1130 '<FieldAccess for expr_context_free_type at parser.py:2882>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2882






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld_2 := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1130
Block_Var_1 := Fld_2; 
--# bind dest_type Block_Var_1
--# expr-start 1139 '<If at parser.py:2883>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2883






--# expr-start 1133 '<Contains at parser.py:2884>' Quantifier_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2884








   

   --# expr-start 1131 '<FieldAccess for base_types at parser.py:2884>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2884






   

      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.base_types
Fld_4 := Liblktlang.Implementation.Type_Decl_P_Base_Types (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 1131

   
      Quantifier_Result := False;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Fld_4;
      begin
         for Item_65 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind item Item_65

            
      --# expr-start 1132 '<Eq at ???>' Is_Equiv None






Cast_Expr := Item_65; 



   
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);








Cast_Expr_1 := Block_Var_1; 



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);


Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1132
      Quantifier_Result := Is_Equiv;
   

            
   --# end


               exit when Quantifier_Result;

         end loop;
      end;
   

   
   



--# expr-done 1133
if Quantifier_Result then
   
   If_Result := True;
else
   --# expr-start 1136 '<Contains at parser.py:2885>' Quantifier_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2885








   

   --# expr-start 1134 '<FieldAccess for base_types at parser.py:2885>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2885






   

      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start TypeDecl.base_types
Fld_5 := Liblktlang.Implementation.Type_Decl_P_Base_Types (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 1134

   
      Quantifier_Result_1 := False;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Fld_5;
      begin
         for Item_66 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind item Item_66

            
      --# expr-start 1135 '<Eq at ???>' Is_Equiv_1 None






Cast_Expr_2 := Item_66; 



   
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);








Cast_Expr_3 := Block_Var; 



   
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);


Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 1135
      Quantifier_Result_1 := Is_Equiv_1;
   

            
   --# end


               exit when Quantifier_Result_1;

         end loop;
      end;
   

   
   



--# expr-done 1136
   If_Result := Quantifier_Result_1;
end if;



if If_Result then
   
   If_Result_1 := No_Internal_Semantic_Result_Array_Type;
else
   --# expr-start 1138 '<FieldAccess for error at parser.py:2889>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2889






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1137 '<String at parser.py:2890>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2890
Str := Create_String ("Invalid cast: only up/down casting allowed"); 
--# expr-done 1137
--# property-call-start LktNode.error
Fld_3 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Ent.Node, Message => Str);
--# end
--# expr-done 1138
                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_3;
                Inc_Ref (Singleton.Items (1));
            
   If_Result_1 := Singleton;
end if;

      Inc_Ref (If_Result_1);


--# expr-done 1139
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1140

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_226;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_226;




            raise;
      end;



   return Property_Result;
end Cast_Expr_P_Check_Legality;
--# end



   


      

   --
   --  Primitives for Bare_Error_On_Null
   --

   



      
      procedure Initialize_Fields_For_Error_On_Null
        (Self : Bare_Error_On_Null
         ; Error_On_Null_F_Expr : Bare_Expr
        ) is
      begin

            Self.Error_On_Null_F_Expr := Error_On_Null_F_Expr;
         

      end Initialize_Fields_For_Error_On_Null;

      
   function Error_On_Null_F_Expr
     (Node : Bare_Error_On_Null) return Bare_Expr
   is
      

   begin
         
         return Node.Error_On_Null_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Generic_Instantiation
   --

   



      
      procedure Initialize_Fields_For_Generic_Instantiation
        (Self : Bare_Generic_Instantiation
         ; Generic_Instantiation_F_Name : Bare_Expr
         ; Generic_Instantiation_F_Args : Bare_Type_Ref_List
        ) is
      begin

            Self.Generic_Instantiation_F_Name := Generic_Instantiation_F_Name;
            Self.Generic_Instantiation_F_Args := Generic_Instantiation_F_Args;
         

      end Initialize_Fields_For_Generic_Instantiation;

      
   function Generic_Instantiation_F_Name
     (Node : Bare_Generic_Instantiation) return Bare_Expr
   is
      

   begin
         
         return Node.Generic_Instantiation_F_Name;
      
   end;

      
   function Generic_Instantiation_F_Args
     (Node : Bare_Generic_Instantiation) return Bare_Type_Ref_List
   is
      

   begin
         
         return Node.Generic_Instantiation_F_Args;
      
   end;


   







--# property-start GenericInstantiation.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3443
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Expr_Context_Free_Type
  
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Fun_Decl;
Match_2 : Internal_Entity_Decl;
Fld : Internal_Entity_Decl;
Fld_1 : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Fun_Decl;
Let_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity_Decl;
Let_Result_2 : Internal_Entity_Type_Decl;
Scope_Result_2 : Internal_Entity_Type_Decl;
Match_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1143 '<Match at parser.py:3444>' Match_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3444






--# expr-start 1141 '<FieldAccess for instantiated_decl at parser.py:3444>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3444






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start GenericInstantiation.instantiated_decl
Fld := Liblktlang.Implementation.Generic_Instantiation_P_Instantiated_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1141


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Match_Prefix := Fld; 

case Lkt_Decl (Match_Prefix.Node.Kind) is
         when Lkt_Enum_Class_Alt_Decl .. Lkt_Trait_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_0 := Cast_Result; 
--# bind t Match_0

Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;


   --# end


            Match_Result := Scope_Result; 
         when Lkt_Fun_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_1 := Create_Internal_Entity_Fun_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_1 := Cast_Result_1; 
--# bind f Match_1
--# expr-start 1142 '<FieldAccess for get_type at parser.py:3446>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3446






   

      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]BaseValDecl.get_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Base_Val_Decl_P_Get_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 1142
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;


   --# end


            Match_Result := Scope_Result_1; 
         when Lkt_Grammar_Rule_Decl .. Lkt_Val_Decl | Lkt_Env_Spec_Decl .. Lkt_Lexer_Family_Decl =>
            



   --# scope-start



--# scope-start









   
      Cast_Result_2 := Create_Internal_Entity_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);


Match_2 := Cast_Result_2; 
--# bind _ Match_2

Let_Result_2 := raise Property_Error; 
--# end
Scope_Result_2 := Let_Result_2;


   --# end


            Match_Result := Scope_Result_2; 
end case;

--# expr-done 1143

         Property_Result := Match_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Generic_Instantiation_P_Expr_Context_Free_Type;
--# end

   







--# property-start GenericInstantiation.instantiated_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3451
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Instantiated_Decl
  
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Decl
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Decl;

      
            procedure Finalizer_Scope_241 with Inline_Always;

      Block_Var : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Generic_Decl;
Fld_2 : Internal_Entity_Type_Ref_List;
Fld_3 : Bare_Type_Ref_List;
Fld_4 : Internal_Entity_Info;
Item_67 : Internal_Entity_Type_Ref;
Bare_Item_67 : Bare_Type_Ref;
Fld_5 : Internal_Entity_Type_Decl;
Fld_6 : Bare_Decl;
Cast_Result_1 : Bare_Type_Decl;
As_Entity : Internal_Entity_Type_Ref;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Ref;
Map_Result : Bare_Type_Decl_Array_Access;
Fld_7 : Internal_Entity_Decl;
Let_Result : Internal_Entity_Decl;

            procedure Finalizer_Scope_241 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_241;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 1153 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1146 '<Cast to GenericDecl at parser.py:3458>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3458





--# expr-start 1145 '<FieldAccess for check_referenced_decl at parser.py:3458>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3458



--# expr-start 1144 '<FieldAccess for name at parser.py:3458>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3458






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Generic_Instantiation_F_Name, Info => Ent.Info);
--# expr-done 1144


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_referenced_decl
Fld_1 := Liblktlang.Implementation.Expr_P_Check_Referenced_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1145



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Generic_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 1146
Block_Var := Cast_Result; 
--# bind generic_decl Block_Var
--# expr-start 1152 '<FieldAccess for instantiate at parser.py:3462>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3462






   

      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1151 '<Map at parser.py:3463>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3463








   

   --# expr-start 1147 '<FieldAccess for args at parser.py:3463>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3463






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Create_Internal_Entity_Type_Ref_List (Node => Ent.Node.Generic_Instantiation_F_Args, Info => Ent.Info);
--# expr-done 1147






   




Fld_3 := Fld_2.Node;

   
      declare
         Map_Result_Vec : Bare_Type_Decl_Vectors.Vector;
      begin
   

   

      if Fld_3 = null then
         
      Map_Result := Create_Bare_Type_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Type_Ref_List := Fld_3;
      begin
         for Untyped_Item_67 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_67 := Untyped_Item_67; 
                  



Is_Null := Bare_Item_67 = null; 
if Is_Null then
   
   If_Result := No_Entity_Type_Ref;
else
   





   




Fld_4 := Fld_2.Info;

As_Entity := (Info => Fld_4, Node => Bare_Item_67); 
   If_Result := As_Entity;
end if;



                  Item_67 := If_Result; 

            
   --# scope-start


               --# bind p Item_67

            
         
         
      --# expr-start 1150 '<Cast to TypeDecl at parser.py:3465>' Cast_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3465





--# expr-start 1149 '<FieldAccess for assert_bare at parser.py:3464>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3464



--# expr-start 1148 '<FieldAccess for designated_type at parser.py:3464>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3464






   

      if Item_67.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_5 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Item_67.Node, E_Info => Item_67.Info);
--# end
--# expr-done 1148


   

      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.assert_bare
Fld_6 := Liblktlang.Implementation.Decl_P_Assert_Bare (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 1149



   if Fld_6 = null
      or else Fld_6.Kind in Lkt_Type_Decl
   then
      
      Cast_Result_1 := Fld_6;

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 1150
      

         declare
            Item_To_Append : constant Bare_Type_Decl := Cast_Result_1;
         begin
            Bare_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Bare_Type_Decl_Array
           (Items_Count => Natural (Bare_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1151
--# property-call-start GenericDecl.instantiate
Fld_7 := Liblktlang.Implementation.Generic_Decl_P_Instantiate (Node => Block_Var.Node, Actuals => Map_Result, E_Info => Block_Var.Info);
--# end
--# expr-done 1152
Let_Result := Fld_7; 
--# end
--# expr-done 1153

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_241;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_241;




            raise;
      end;



   return Property_Result;
end Generic_Instantiation_P_Instantiated_Decl;
--# end

   







--# property-start GenericInstantiation.check_correctness_pre /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3469
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Check_Correctness_Pre
  
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_242 with Inline_Always;

      Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Ref_List;
Fld_2 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_242 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_242;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1157 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1156 '<FieldAccess for check_generic_inst_correctness_pre at parser.py:3471>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3471






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1154 '<FieldAccess for name at parser.py:3472>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3472






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Generic_Instantiation_F_Name, Info => Ent.Info);
--# expr-done 1154
--# expr-start 1155 '<FieldAccess for args at parser.py:3472>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3472






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Type_Ref_List (Node => Ent.Node.Generic_Instantiation_F_Args, Info => Ent.Info);
--# expr-done 1155
--# property-call-start LktNode.check_generic_inst_correctness_pre
Fld_2 := Liblktlang.Implementation.Lkt_Node_P_Check_Generic_Inst_Correctness_Pre (Node => Ent.Node, Prefix => Fld, Params => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 1156
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1157

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_242;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_242;




            raise;
      end;



   return Property_Result;
end Generic_Instantiation_P_Check_Correctness_Pre;
--# end

   







--# property-start GenericInstantiation.referenced_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3475
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Referenced_Decl
  
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_243 with Inline_Always;

      Fld : Internal_Entity_Decl;
Cast_Expr : Bare_Generic_Instantiation;
Cast_Result : Bare_Lkt_Node;
New_Struct : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_243 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_243;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1159 '<New SemanticResult at parser.py:3476>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3476









Cast_Expr := Self; 



   
      Cast_Result := Cast_Expr;


--# expr-start 1158 '<FieldAccess for instantiated_decl at parser.py:3476>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3476






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start GenericInstantiation.instantiated_decl
Fld := Liblktlang.Implementation.Generic_Instantiation_P_Instantiated_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1158

Inc_Ref (Empty_String);
New_Struct := (Error_Message => Empty_String, Exempt_Analysis => False, Has_Error => False, Node => Cast_Result, Result_Decl => Fld, Result_Type => No_Entity_Type_Decl); 
--# expr-done 1159

         Property_Result := New_Struct;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_243;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_243;




            raise;
      end;



   return Property_Result;
end Generic_Instantiation_P_Referenced_Decl;
--# end



   


      

   --
   --  Primitives for Bare_Grammar_Expr
   --

   







   


      

   --
   --  Primitives for Bare_Grammar_Cut
   --

   







   


      

   --
   --  Primitives for Bare_Grammar_Discard
   --

   



      
      procedure Initialize_Fields_For_Grammar_Discard
        (Self : Bare_Grammar_Discard
         ; Grammar_Discard_F_Expr : Bare_Grammar_Expr
        ) is
      begin

            Self.Grammar_Discard_F_Expr := Grammar_Discard_F_Expr;
         

      end Initialize_Fields_For_Grammar_Discard;

      
   function Grammar_Discard_F_Expr
     (Node : Bare_Grammar_Discard) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Grammar_Discard_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Dont_Skip
   --

   



      
      procedure Initialize_Fields_For_Grammar_Dont_Skip
        (Self : Bare_Grammar_Dont_Skip
         ; Grammar_Dont_Skip_F_Expr : Bare_Grammar_Expr
         ; Grammar_Dont_Skip_F_Dont_Skip : Bare_Grammar_Expr
        ) is
      begin

            Self.Grammar_Dont_Skip_F_Expr := Grammar_Dont_Skip_F_Expr;
            Self.Grammar_Dont_Skip_F_Dont_Skip := Grammar_Dont_Skip_F_Dont_Skip;
         

      end Initialize_Fields_For_Grammar_Dont_Skip;

      
   function Grammar_Dont_Skip_F_Expr
     (Node : Bare_Grammar_Dont_Skip) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Grammar_Dont_Skip_F_Expr;
      
   end;

      
   function Grammar_Dont_Skip_F_Dont_Skip
     (Node : Bare_Grammar_Dont_Skip) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Grammar_Dont_Skip_F_Dont_Skip;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_List
   --

   



      
      procedure Initialize_Fields_For_Grammar_List
        (Self : Bare_Grammar_List
         ; Grammar_List_F_List_Type : Bare_Type_Ref
         ; Grammar_List_F_Kind : Bare_List_Kind
         ; Grammar_List_F_Expr : Bare_Grammar_Expr
         ; Grammar_List_F_Sep : Bare_Grammar_List_Sep
        ) is
      begin

            Self.Grammar_List_F_List_Type := Grammar_List_F_List_Type;
            Self.Grammar_List_F_Kind := Grammar_List_F_Kind;
            Self.Grammar_List_F_Expr := Grammar_List_F_Expr;
            Self.Grammar_List_F_Sep := Grammar_List_F_Sep;
         

      end Initialize_Fields_For_Grammar_List;

      
   function Grammar_List_F_List_Type
     (Node : Bare_Grammar_List) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Grammar_List_F_List_Type;
      
   end;

      
   function Grammar_List_F_Kind
     (Node : Bare_Grammar_List) return Bare_List_Kind
   is
      

   begin
         
         return Node.Grammar_List_F_Kind;
      
   end;

      
   function Grammar_List_F_Expr
     (Node : Bare_Grammar_List) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Grammar_List_F_Expr;
      
   end;

      
   function Grammar_List_F_Sep
     (Node : Bare_Grammar_List) return Bare_Grammar_List_Sep
   is
      

   begin
         
         return Node.Grammar_List_F_Sep;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Null
   --

   



      
      procedure Initialize_Fields_For_Grammar_Null
        (Self : Bare_Grammar_Null
         ; Grammar_Null_F_Name : Bare_Type_Ref
        ) is
      begin

            Self.Grammar_Null_F_Name := Grammar_Null_F_Name;
         

      end Initialize_Fields_For_Grammar_Null;

      
   function Grammar_Null_F_Name
     (Node : Bare_Grammar_Null) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Grammar_Null_F_Name;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Opt
   --

   



      
      procedure Initialize_Fields_For_Grammar_Opt
        (Self : Bare_Grammar_Opt
         ; Grammar_Opt_F_Expr : Bare_Grammar_Expr
        ) is
      begin

            Self.Grammar_Opt_F_Expr := Grammar_Opt_F_Expr;
         

      end Initialize_Fields_For_Grammar_Opt;

      
   function Grammar_Opt_F_Expr
     (Node : Bare_Grammar_Opt) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Grammar_Opt_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Opt_Error
   --

   



      
      procedure Initialize_Fields_For_Grammar_Opt_Error
        (Self : Bare_Grammar_Opt_Error
         ; Grammar_Opt_Error_F_Expr : Bare_Grammar_Expr
        ) is
      begin

            Self.Grammar_Opt_Error_F_Expr := Grammar_Opt_Error_F_Expr;
         

      end Initialize_Fields_For_Grammar_Opt_Error;

      
   function Grammar_Opt_Error_F_Expr
     (Node : Bare_Grammar_Opt_Error) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Grammar_Opt_Error_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Opt_Error_Group
   --

   



      
      procedure Initialize_Fields_For_Grammar_Opt_Error_Group
        (Self : Bare_Grammar_Opt_Error_Group
         ; Grammar_Opt_Error_Group_F_Expr : Bare_Grammar_Expr_List
        ) is
      begin

            Self.Grammar_Opt_Error_Group_F_Expr := Grammar_Opt_Error_Group_F_Expr;
         

      end Initialize_Fields_For_Grammar_Opt_Error_Group;

      
   function Grammar_Opt_Error_Group_F_Expr
     (Node : Bare_Grammar_Opt_Error_Group) return Bare_Grammar_Expr_List
   is
      

   begin
         
         return Node.Grammar_Opt_Error_Group_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Opt_Group
   --

   



      
      procedure Initialize_Fields_For_Grammar_Opt_Group
        (Self : Bare_Grammar_Opt_Group
         ; Grammar_Opt_Group_F_Expr : Bare_Grammar_Expr_List
        ) is
      begin

            Self.Grammar_Opt_Group_F_Expr := Grammar_Opt_Group_F_Expr;
         

      end Initialize_Fields_For_Grammar_Opt_Group;

      
   function Grammar_Opt_Group_F_Expr
     (Node : Bare_Grammar_Opt_Group) return Bare_Grammar_Expr_List
   is
      

   begin
         
         return Node.Grammar_Opt_Group_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Or_Expr
   --

   



      
      procedure Initialize_Fields_For_Grammar_Or_Expr
        (Self : Bare_Grammar_Or_Expr
         ; Grammar_Or_Expr_F_Sub_Exprs : Bare_Grammar_Expr_List_List
        ) is
      begin

            Self.Grammar_Or_Expr_F_Sub_Exprs := Grammar_Or_Expr_F_Sub_Exprs;
         

      end Initialize_Fields_For_Grammar_Or_Expr;

      
   function Grammar_Or_Expr_F_Sub_Exprs
     (Node : Bare_Grammar_Or_Expr) return Bare_Grammar_Expr_List_List
   is
      

   begin
         
         return Node.Grammar_Or_Expr_F_Sub_Exprs;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Pick
   --

   



      
      procedure Initialize_Fields_For_Grammar_Pick
        (Self : Bare_Grammar_Pick
         ; Grammar_Pick_F_Exprs : Bare_Grammar_Expr_List
        ) is
      begin

            Self.Grammar_Pick_F_Exprs := Grammar_Pick_F_Exprs;
         

      end Initialize_Fields_For_Grammar_Pick;

      
   function Grammar_Pick_F_Exprs
     (Node : Bare_Grammar_Pick) return Bare_Grammar_Expr_List
   is
      

   begin
         
         return Node.Grammar_Pick_F_Exprs;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Implicit_Pick
   --

   



      
      procedure Initialize_Fields_For_Grammar_Implicit_Pick
        (Self : Bare_Grammar_Implicit_Pick
         ; Grammar_Pick_F_Exprs : Bare_Grammar_Expr_List
        ) is
      begin
            Initialize_Fields_For_Grammar_Pick
              (Self, Grammar_Pick_F_Exprs);

         

      end Initialize_Fields_For_Grammar_Implicit_Pick;




   


      

   --
   --  Primitives for Bare_Grammar_Predicate
   --

   



      
      procedure Initialize_Fields_For_Grammar_Predicate
        (Self : Bare_Grammar_Predicate
         ; Grammar_Predicate_F_Expr : Bare_Grammar_Expr
         ; Grammar_Predicate_F_Prop_Ref : Bare_Expr
        ) is
      begin

            Self.Grammar_Predicate_F_Expr := Grammar_Predicate_F_Expr;
            Self.Grammar_Predicate_F_Prop_Ref := Grammar_Predicate_F_Prop_Ref;
         

      end Initialize_Fields_For_Grammar_Predicate;

      
   function Grammar_Predicate_F_Expr
     (Node : Bare_Grammar_Predicate) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Grammar_Predicate_F_Expr;
      
   end;

      
   function Grammar_Predicate_F_Prop_Ref
     (Node : Bare_Grammar_Predicate) return Bare_Expr
   is
      

   begin
         
         return Node.Grammar_Predicate_F_Prop_Ref;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Rule_Ref
   --

   



      
      procedure Initialize_Fields_For_Grammar_Rule_Ref
        (Self : Bare_Grammar_Rule_Ref
         ; Grammar_Rule_Ref_F_Node_Name : Bare_Ref_Id
        ) is
      begin

            Self.Grammar_Rule_Ref_F_Node_Name := Grammar_Rule_Ref_F_Node_Name;
         

      end Initialize_Fields_For_Grammar_Rule_Ref;

      
   function Grammar_Rule_Ref_F_Node_Name
     (Node : Bare_Grammar_Rule_Ref) return Bare_Ref_Id
   is
      

   begin
         
         return Node.Grammar_Rule_Ref_F_Node_Name;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Skip
   --

   



      
      procedure Initialize_Fields_For_Grammar_Skip
        (Self : Bare_Grammar_Skip
         ; Grammar_Skip_F_Name : Bare_Type_Ref
        ) is
      begin

            Self.Grammar_Skip_F_Name := Grammar_Skip_F_Name;
         

      end Initialize_Fields_For_Grammar_Skip;

      
   function Grammar_Skip_F_Name
     (Node : Bare_Grammar_Skip) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Grammar_Skip_F_Name;
      
   end;




   


      

   --
   --  Primitives for Bare_Grammar_Stop_Cut
   --

   



      
      procedure Initialize_Fields_For_Grammar_Stop_Cut
        (Self : Bare_Grammar_Stop_Cut
         ; Grammar_Stop_Cut_F_Expr : Bare_Grammar_Expr
        ) is
      begin

            Self.Grammar_Stop_Cut_F_Expr := Grammar_Stop_Cut_F_Expr;
         

      end Initialize_Fields_For_Grammar_Stop_Cut;

      
   function Grammar_Stop_Cut_F_Expr
     (Node : Bare_Grammar_Stop_Cut) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Grammar_Stop_Cut_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Parse_Node_Expr
   --

   



      
      procedure Initialize_Fields_For_Parse_Node_Expr
        (Self : Bare_Parse_Node_Expr
         ; Parse_Node_Expr_F_Node_Name : Bare_Type_Ref
         ; Parse_Node_Expr_F_Sub_Exprs : Bare_Grammar_Expr_List
        ) is
      begin

            Self.Parse_Node_Expr_F_Node_Name := Parse_Node_Expr_F_Node_Name;
            Self.Parse_Node_Expr_F_Sub_Exprs := Parse_Node_Expr_F_Sub_Exprs;
         

      end Initialize_Fields_For_Parse_Node_Expr;

      
   function Parse_Node_Expr_F_Node_Name
     (Node : Bare_Parse_Node_Expr) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Parse_Node_Expr_F_Node_Name;
      
   end;

      
   function Parse_Node_Expr_F_Sub_Exprs
     (Node : Bare_Parse_Node_Expr) return Bare_Grammar_Expr_List
   is
      

   begin
         
         return Node.Parse_Node_Expr_F_Sub_Exprs;
      
   end;




   


      

   --
   --  Primitives for Bare_Token_Lit
   --

   







   


      

   --
   --  Primitives for Bare_Token_No_Case_Lit
   --

   



      
      procedure Initialize_Fields_For_Token_No_Case_Lit
        (Self : Bare_Token_No_Case_Lit
         ; Token_No_Case_Lit_F_Lit : Bare_Token_Lit
        ) is
      begin

            Self.Token_No_Case_Lit_F_Lit := Token_No_Case_Lit_F_Lit;
         

      end Initialize_Fields_For_Token_No_Case_Lit;

      
   function Token_No_Case_Lit_F_Lit
     (Node : Bare_Token_No_Case_Lit) return Bare_Token_Lit
   is
      

   begin
         
         return Node.Token_No_Case_Lit_F_Lit;
      
   end;




   


      

   --
   --  Primitives for Bare_Token_Pattern_Lit
   --

   







   


      

   --
   --  Primitives for Bare_Token_Ref
   --

   



      
      procedure Initialize_Fields_For_Token_Ref
        (Self : Bare_Token_Ref
         ; Token_Ref_F_Token_Name : Bare_Ref_Id
         ; Token_Ref_F_Expr : Bare_Token_Lit
        ) is
      begin

            Self.Token_Ref_F_Token_Name := Token_Ref_F_Token_Name;
            Self.Token_Ref_F_Expr := Token_Ref_F_Expr;
         

      end Initialize_Fields_For_Token_Ref;

      
   function Token_Ref_F_Token_Name
     (Node : Bare_Token_Ref) return Bare_Ref_Id
   is
      

   begin
         
         return Node.Token_Ref_F_Token_Name;
      
   end;

      
   function Token_Ref_F_Expr
     (Node : Bare_Token_Ref) return Bare_Token_Lit
   is
      

   begin
         
         return Node.Token_Ref_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Id
   --

   







   


      

   --
   --  Primitives for Bare_Def_Id
   --

   







   


      

   --
   --  Primitives for Bare_Module_Ref_Id
   --

   







   


      

   --
   --  Primitives for Bare_Ref_Id
   --

   





   







--# property-start RefId.referenced_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1414
pragma Warnings (Off, "is not referenced");
 function Ref_Id_P_Referenced_Decl
  
  (Node : Bare_Ref_Id
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result
is
   Self : Bare_Ref_Id  := Bare_Ref_Id (Node);
     Ent : Internal_Entity_Ref_Id :=
       Internal_Entity_Ref_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_111 with Inline_Always;
            procedure Finalizer_Scope_526 with Inline_Always;

      Block_Var : Internal_Entity_Param;
Block_Var_1 : Boolean;
Scope : Lexical_Env := Empty_Env;
Var_Expr : Internal_Entity_Decl;
Fld : Internal_Entity_Param;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_2 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Fld_3 : Internal_Entity_Dot_Expr;
Is_Null_2 : Boolean;
If_Result_1 : Lookup_Kind;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Decl;
Cast_Expr : Bare_Ref_Id;
Cast_Result_1 : Bare_Lkt_Node;
New_Struct : Internal_Semantic_Result := No_Semantic_Result;
Scope_Result : Internal_Semantic_Result := No_Semantic_Result;
Fld_4 : Internal_Semantic_Result := No_Semantic_Result;
Result_Var : Internal_Semantic_Result := No_Semantic_Result;
Is_Equal : Boolean;
If_Result_2 : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;
If_Result_3 : Internal_Semantic_Result := No_Semantic_Result;
Let_Result_1 : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_111 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_111;
            procedure Finalizer_Scope_526 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_526;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1181 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 1160 '<FieldAccess for param_if_param_name at parser.py:1417>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1417






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start RefId.param_if_param_name
Fld := Liblktlang.Implementation.Ref_Id_P_Param_If_Param_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1160
Block_Var := Fld; 
--# bind param_if_name Block_Var



--# expr-start 1162 '<Not at parser.py:1419>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1419
--# expr-start 1161 '<IsNull at parser.py:1419>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1419

Is_Null := Block_Var.Node = null; 
--# expr-done 1161
Not_Val := not (Is_Null); 
--# expr-done 1162
if Not_Val then
   --# expr-start 1167 '<Not at parser.py:1420>' Not_Val_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1420
--# expr-start 1166 '<IsNull at parser.py:1421>' Is_Null_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1421
--# expr-start 1165 '<Find at parser.py:1421>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1421








   

   --# expr-start 1163 '<FieldAccess for parents at parser.py:1420>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1420






   

      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Fld_1 := Liblktlang.Implementation.Parents (Node => Block_Var.Node, With_Self => True, E_Info => Block_Var.Info);
--# expr-done 1163

   
      Find_Result := No_Entity;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_68 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind n Item_68

            
      --# expr-start 1164 '<IsA DeclAnnotation at parser.py:1421>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1421

Is_A := Item_68.Node /= null 
and then Item_68.Node.Kind in Lkt_Decl_Annotation_Range; 
--# expr-done 1164
      if Is_A then
         Find_Result := Item_68; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 1165
Is_Null_1 := Find_Result.Node = null; 
--# expr-done 1166
Not_Val_1 := not (Is_Null_1); 
--# expr-done 1167
   If_Result := Not_Val_1;
else
   
   If_Result := False;
end if;



Block_Var_1 := If_Result; 
--# bind is_annot_param Block_Var_1
--# expr-start 1180 '<If at parser.py:1424>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1424




if Block_Var_1 then
   
   If_Result_3 := No_Semantic_Result;
else
   --# expr-start 1179 '<Let at parser.py:1431>' Let_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1431
--# scope-start
--# expr-start 1168 '<FieldAccess for scope at parser.py:1432>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1432






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start RefId.scope
Fld_2 := Liblktlang.Implementation.Ref_Id_P_Scope (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1168
Scope := Fld_2; Inc_Ref (Scope);
--# bind scope Scope
--# expr-start 1178 '<If at parser.py:1433>' If_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1433



--# expr-start 1169 '<Eq at parser.py:1434>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1434


Is_Equal := Equivalent (Scope, Empty_Env); 
--# expr-done 1169
if Is_Equal then
   
   If_Result_2 := No_Semantic_Result;
else
   


--# expr-start 1175 '<Cast to Decl at parser.py:1442>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1442





--# expr-start 1174 '<EnvGet at parser.py:1436>' Env_Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1436

--# expr-start 1170 '<GetSymbol at parser.py:1437>' Sym /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1437

Sym := Get_Symbol (Self); 
--# expr-done 1170
--# expr-start 1173 '<If at parser.py:1437>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1437



--# expr-start 1172 '<IsNull at parser.py:1438>' Is_Null_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1438
--# expr-start 1171 '<FieldAccess for dot_expr_if_suffix at parser.py:1438>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1438






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start RefId.dot_expr_if_suffix
Fld_3 := Liblktlang.Implementation.Ref_Id_P_Dot_Expr_If_Suffix (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1171
Is_Null_2 := Fld_3.Node = null; 
--# expr-done 1172
if Is_Null_2 then
   
   If_Result_1 := Recursive;
else
   
   If_Result_1 := Flat;
end if;



--# expr-done 1173

Env_Get_Result := AST_Envs.Get_First (Self => Scope, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (If_Result_1), Categories => (Nocat => True)); 
--# expr-done 1174



   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Lkt_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);

   else
         Cast_Result := No_Entity_Decl;
   end if;


--# expr-done 1175
Var_Expr := Cast_Result;





if Var_Expr /= No_Entity_Decl then
   



   --# scope-start


   --# bind d Var_Expr

--# expr-start 1176 '<New SemanticResult at parser.py:1443>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1443









Cast_Expr := Self; 



   
      Cast_Result_1 := Cast_Expr;




Inc_Ref (Empty_String);
New_Struct := (Error_Message => Empty_String, Exempt_Analysis => False, Has_Error => False, Node => Cast_Result_1, Result_Decl => Var_Expr, Result_Type => No_Entity_Type_Decl); 
--# expr-done 1176
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_526;


   Result_Var := Scope_Result;
else
   --# expr-start 1177 '<FieldAccess for ref_not_found_error at parser.py:1444>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1444






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start RefId.ref_not_found_error
Fld_4 := Liblktlang.Implementation.Ref_Id_P_Ref_Not_Found_Error (Node => Self);
--# end
--# expr-done 1177
   Result_Var := Fld_4;
end if;

      Inc_Ref (Result_Var);


   If_Result_2 := Result_Var;
end if;

      Inc_Ref (If_Result_2);


--# expr-done 1178
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1179
   If_Result_3 := Let_Result;
end if;

      Inc_Ref (If_Result_3);


--# expr-done 1180
Let_Result_1 := If_Result_3; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1181

         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_111;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_111;
                     Finalizer_Scope_526;




            raise;
      end;



   return Property_Result;
end Ref_Id_P_Referenced_Decl;
--# end

   







--# property-start RefId.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1450
pragma Warnings (Off, "is not referenced");
 function Ref_Id_P_Expr_Context_Free_Type
  
  (Node : Bare_Ref_Id
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Ref_Id  := Bare_Ref_Id (Node);
     Ent : Internal_Entity_Ref_Id :=
       Internal_Entity_Ref_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Base_Val_Decl;
Fld_1 : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1184 '<FieldAccess for get_type at parser.py:1451>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1451



--# expr-start 1183 '<Cast to BaseValDecl at parser.py:1451>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1451





--# expr-start 1182 '<FieldAccess for check_referenced_decl at parser.py:1451>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1451






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_referenced_decl
Fld := Liblktlang.Implementation.Expr_P_Check_Referenced_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1182



   if Fld.Node = null
      or else Fld.Node.Kind in Lkt_Base_Val_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Base_Val_Decl
        (Node => Fld.Node,
         Info => Fld.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 1183


   

      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]BaseValDecl.get_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Base_Val_Decl_P_Get_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1184

         Property_Result := Fld_1;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Ref_Id_P_Expr_Context_Free_Type;
--# end

   







--# property-start RefId.dot_expr_if_suffix /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1454
pragma Warnings (Off, "is not referenced");
 function Ref_Id_P_Dot_Expr_If_Suffix
  
  (Node : Bare_Ref_Id
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Dot_Expr
is
   Self : Bare_Ref_Id  := Bare_Ref_Id (Node);
     Ent : Internal_Entity_Ref_Id :=
       Internal_Entity_Ref_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Dot_Expr;

      

      Var_Expr : Internal_Entity_Dot_Expr;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Dot_Expr;
Fld_1 : Internal_Entity_Ref_Id;
Cast_Expr : Internal_Entity_Ref_Id;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Dot_Expr;
Scope_Result : Internal_Entity_Dot_Expr;
Result_Var : Internal_Entity_Dot_Expr;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         


--# expr-start 1186 '<Cast to DotExpr at parser.py:1455>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1455





--# expr-start 1185 '<parent at parser.py:1455>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1455






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1185



   if Fld.Node = null
      or else Fld.Node.Kind in Lkt_Dot_Expr_Range
   then
      
      Cast_Result := Create_Internal_Entity_Dot_Expr
        (Node => Fld.Node,
         Info => Fld.Info);

   else
         Cast_Result := No_Entity_Dot_Expr;
   end if;


--# expr-done 1186
Var_Expr := Cast_Result;





if Var_Expr /= No_Entity_Dot_Expr then
   



   --# scope-start


   --# bind de Var_Expr

--# expr-start 1189 '<If at parser.py:1456>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1456



--# expr-start 1188 '<Eq at parser.py:1456>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1456






Cast_Expr := Ent; 



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);







--# expr-start 1187 '<FieldAccess for suffix at parser.py:1456>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1456






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Ref_Id (Node => Var_Expr.Node.Base_Dot_Expr_F_Suffix, Info => Var_Expr.Info);
--# expr-done 1187



   
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);


Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1188
if Is_Equiv then
   
   If_Result := Var_Expr;
else
   
   If_Result := No_Entity_Dot_Expr;
end if;



--# expr-done 1189
Scope_Result := If_Result;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Dot_Expr;
end if;




         Property_Result := Result_Var;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Ref_Id_P_Dot_Expr_If_Suffix;
--# end

   







--# property-start RefId.param_if_param_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1459
pragma Warnings (Off, "is not referenced");
 function Ref_Id_P_Param_If_Param_Name
  
  (Node : Bare_Ref_Id
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Param
is
   Self : Bare_Ref_Id  := Bare_Ref_Id (Node);
     Ent : Internal_Entity_Ref_Id :=
       Internal_Entity_Ref_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Param;

      

      Var_Expr : Internal_Entity_Param;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Param;
Fld_1 : Internal_Entity_Ref_Id;
Cast_Expr : Internal_Entity_Ref_Id;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Param;
Scope_Result : Internal_Entity_Param;
Result_Var : Internal_Entity_Param;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         


--# expr-start 1191 '<Cast to Param at parser.py:1460>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1460





--# expr-start 1190 '<parent at parser.py:1460>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1460






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1190



   if Fld.Node = null
      or else Fld.Node.Kind in Lkt_Param_Range
   then
      
      Cast_Result := Create_Internal_Entity_Param
        (Node => Fld.Node,
         Info => Fld.Info);

   else
         Cast_Result := No_Entity_Param;
   end if;


--# expr-done 1191
Var_Expr := Cast_Result;





if Var_Expr /= No_Entity_Param then
   



   --# scope-start


   --# bind p Var_Expr

--# expr-start 1194 '<If at parser.py:1461>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1461



--# expr-start 1193 '<Eq at parser.py:1461>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1461






Cast_Expr := Ent; 



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);







--# expr-start 1192 '<FieldAccess for name at parser.py:1461>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1461






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Ref_Id (Node => Var_Expr.Node.Param_F_Name, Info => Var_Expr.Info);
--# expr-done 1192



   
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);


Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1193
if Is_Equiv then
   
   If_Result := Var_Expr;
else
   
   If_Result := No_Entity_Param;
end if;



--# expr-done 1194
Scope_Result := If_Result;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Entity_Param;
end if;




         Property_Result := Result_Var;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Ref_Id_P_Param_If_Param_Name;
--# end

   







--# property-start RefId.scope /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1465
pragma Warnings (Off, "is not referenced");
 function Ref_Id_P_Scope
  
  (Node : Bare_Ref_Id
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Lexical_Env
is
   Self : Bare_Ref_Id  := Bare_Ref_Id (Node);
     Ent : Internal_Entity_Ref_Id :=
       Internal_Entity_Ref_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Lexical_Env;

      
            procedure Finalizer_Scope_115 with Inline_Always;
            procedure Finalizer_Scope_522 with Inline_Always;
            procedure Finalizer_Scope_523 with Inline_Always;
            procedure Finalizer_Scope_524 with Inline_Always;

      Var_Expr : Internal_Entity_Dot_Expr;
Var_Expr_1 : Internal_Entity_Param;
Var_Expr_2 : Bare_Token_Ref;
Fld : Internal_Entity_Dot_Expr;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Param;
Fld_4 : Internal_Entity_Call_Expr;
Fld_5 : Internal_Entity_Decl;
Fld_6 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Fld_7 : Bare_Lkt_Node;
Cast_Result : Bare_Token_Ref;
Fld_8 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Grammar_Decl;
Fld_9 : Internal_Entity_Decl;
Fld_10 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Fld_11 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;

            procedure Finalizer_Scope_115 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_115;
            procedure Finalizer_Scope_522 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_522;
            procedure Finalizer_Scope_523 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_523;
            procedure Finalizer_Scope_524 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_524;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1211 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 1195 '<FieldAccess for dot_expr_if_suffix at parser.py:1468>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1468






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start RefId.dot_expr_if_suffix
Fld := Liblktlang.Implementation.Ref_Id_P_Dot_Expr_If_Suffix (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1195
Var_Expr := Fld;





if Var_Expr /= No_Entity_Dot_Expr then
   



   --# scope-start


   --# bind de Var_Expr

--# expr-start 1197 '<FieldAccess for designated_scope at parser.py:1470>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1470



--# expr-start 1196 '<FieldAccess for prefix at parser.py:1470>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1470






   

      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Base_Dot_Expr_F_Prefix, Info => Var_Expr.Info);
--# expr-done 1196


   

      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.designated_scope
Fld_2 := Liblktlang.Implementation.Expr_P_Designated_Scope (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 1197
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_522;


   Result_Var_2 := Scope_Result;
else
   


--# expr-start 1198 '<FieldAccess for param_if_param_name at parser.py:1472>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1472






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start RefId.param_if_param_name
Fld_3 := Liblktlang.Implementation.Ref_Id_P_Param_If_Param_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1198
Var_Expr_1 := Fld_3;





if Var_Expr_1 /= No_Entity_Param then
   



   --# scope-start


   --# bind p Var_Expr_1

--# expr-start 1201 '<FieldAccess for call_scope at parser.py:1475>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1475



--# expr-start 1200 '<FieldAccess for called_decl at parser.py:1475>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1475



--# expr-start 1199 '<FieldAccess for call_expr at parser.py:1475>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1475






   

      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Param.call_expr
Fld_4 := Liblktlang.Implementation.Param_P_Call_Expr (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1199


   

      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.called_decl
Fld_5 := Liblktlang.Implementation.Call_Expr_P_Called_Decl (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 1200


   

      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.call_scope'
Fld_6 := Liblktlang.Implementation.Dispatcher_Decl_P_Call_Scope (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 1201
Scope_Result_1 := Fld_6;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_523;


   Result_Var_1 := Scope_Result_1;
else
   


--# expr-start 1203 '<Cast to TokenRef at parser.py:1477>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1477





--# expr-start 1202 '<parent at parser.py:1477>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1477






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_7 := Self.Parent; 
--# expr-done 1202



   if Fld_7 = null
      or else Fld_7.Kind in Lkt_Token_Ref_Range
   then
      
      Cast_Result := Fld_7;

   else
         Cast_Result := No_Bare_Lkt_Node;
   end if;


--# expr-done 1203
Var_Expr_2 := Cast_Result;





if Var_Expr_2 /= No_Bare_Lkt_Node then
   



   --# scope-start


   --# bind _ Var_Expr_2

--# expr-start 1209 '<FieldAccess for children_env at parser.py:1482>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1482



--# expr-start 1208 '<FieldAccess for lexer at parser.py:1482>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1482



--# expr-start 1207 '<Cast to GrammarDecl at parser.py:1482>' Cast_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1482





--# expr-start 1206 '<Find at parser.py:1481>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1481








   

   --# expr-start 1204 '<FieldAccess for parents at parser.py:1481>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1481






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Fld_8 := Liblktlang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 1204

   
      Find_Result := No_Entity;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Array_Access := Fld_8;
      begin
         for Item_69 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind n Item_69

            
      --# expr-start 1205 '<IsA GrammarDecl at parser.py:1481>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1481

Is_A := Item_69.Node /= null 
and then Item_69.Node.Kind in Lkt_Grammar_Decl_Range; 
--# expr-done 1205
      if Is_A then
         Find_Result := Item_69; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 1206



   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Lkt_Grammar_Decl_Range
   then
      
      Cast_Result_1 := Create_Internal_Entity_Grammar_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);

   else
         Cast_Result_1 := No_Entity_Grammar_Decl;
   end if;


--# expr-done 1207


   

      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start GrammarDecl.lexer
Fld_9 := Liblktlang.Implementation.Grammar_Decl_P_Lexer (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 1208


   

      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_10 := Liblktlang.Implementation.Children_Env (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# expr-done 1209
Scope_Result_2 := Fld_10;
   Inc_Ref (Scope_Result_2);


   --# end
      Finalizer_Scope_524;


   Result_Var := Scope_Result_2;
else
   --# expr-start 1210 '<FieldAccess for children_env at parser.py:1485>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1485






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_11 := Liblktlang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1210
   Result_Var := Fld_11;
end if;

      Inc_Ref (Result_Var);


   Result_Var_1 := Result_Var;
end if;

      Inc_Ref (Result_Var_1);


   Result_Var_2 := Result_Var_1;
end if;

      Inc_Ref (Result_Var_2);


Let_Result := Result_Var_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1211

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_115;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_115;
                     Finalizer_Scope_522;
                     Finalizer_Scope_523;
                     Finalizer_Scope_524;




            raise;
      end;



   return Property_Result;
end Ref_Id_P_Scope;
--# end

   







--# property-start RefId.ref_not_found_error /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1490
pragma Warnings (Off, "is not referenced");
 function Ref_Id_P_Ref_Not_Found_Error
  
  (Node : Bare_Ref_Id
  )

   return Internal_Semantic_Result
is
   Self : Bare_Ref_Id  := Bare_Ref_Id (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_116 with Inline_Always;

      Str : String_Type;
Fld : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_116 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_116;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1216 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1215 '<FieldAccess for error at parser.py:1492>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1492






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1212 '<String at parser.py:1493>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1493
Str := Create_String ("Cannot find entity `"); 
--# expr-done 1212
--# expr-start 1213 '<FieldAccess for text at parser.py:1494>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1494






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Text (Node => Self);
--# expr-done 1213
Concat_Result := Concat_String (Str, Fld); 
--# expr-start 1214 '<String at parser.py:1494>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1494
Str_1 := Create_String ("` in this scope"); 
--# expr-done 1214
Concat_Result_1 := Concat_String (Concat_Result, Str_1); 
--# property-call-start LktNode.error
Fld_1 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Concat_Result_1);
--# end
--# expr-done 1215
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1216

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_116;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_116;




            raise;
      end;



   return Property_Result;
end Ref_Id_P_Ref_Not_Found_Error;
--# end

   







--# property-start RefId.check_legality /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1497
pragma Warnings (Off, "is not referenced");
 function Ref_Id_P_Check_Legality
  
  (Node : Bare_Ref_Id
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Ref_Id  := Bare_Ref_Id (Node);
     Ent : Internal_Entity_Ref_Id :=
       Internal_Entity_Ref_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_117 with Inline_Always;

      Block_Var : Internal_Entity_Named_Type_Decl;
Fld : Internal_Entity_Named_Type_Decl;
Str : String_Type;
Fld_1 : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;
Fld_2 : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
Fld_3 : Internal_Semantic_Result := No_Semantic_Result;
Fld_4 : Internal_Entity_Decl;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Named_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_5 : Internal_Entity_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_117 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_117;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1230 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1217 '<FieldAccess for property_error_type at parser.py:1499>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1499






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.property_error_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Property_Error_Type (Node => Ent.Node);
--# end
--# expr-done 1217
Block_Var := Fld; 
--# bind error_type Block_Var
--# expr-start 1229 '<If at parser.py:1501>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1501






--# expr-start 1220 '<Eq at parser.py:1509>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1509





--# expr-start 1219 '<FieldAccess for result_decl at parser.py:1509>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1509



--# expr-start 1218 '<FieldAccess for referenced_decl at parser.py:1509>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1509






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld_3 := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1218


   




Fld_4 := Fld_3.Result_Decl;
--# expr-done 1219



   
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);








Cast_Expr := Block_Var; 



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);


Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1220
if Is_Equiv then
   --# expr-start 1224 '<Not at parser.py:1510>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1510
--# expr-start 1223 '<AnyQuantifier at parser.py:1510>' Quantifier_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1510








   

   --# expr-start 1221 '<FieldAccess for parents at parser.py:1510>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1510






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Fld_5 := Liblktlang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 1221

   
      Quantifier_Result := False;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Array_Access := Fld_5;
      begin
         for Item_70 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind r Item_70

            
      --# expr-start 1222 '<IsA RaiseExpr at parser.py:1510>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1510

Is_A := Item_70.Node /= null 
and then Item_70.Node.Kind in Lkt_Raise_Expr_Range; 
--# expr-done 1222
      Quantifier_Result := Is_A;
   

            
   --# end


               exit when Quantifier_Result;

         end loop;
      end;
   

   
   



--# expr-done 1223
Not_Val := not (Quantifier_Result); 
--# expr-done 1224
   If_Result := Not_Val;
else
   
   If_Result := False;
end if;



if If_Result then
   --# expr-start 1228 '<FieldAccess for error at parser.py:1512>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1512






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1225 '<String at parser.py:1512>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1512
Str := Create_String ("Cannot reference `"); 
--# expr-done 1225
--# expr-start 1226 '<FieldAccess for full_name at parser.py:1512>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1512






   

      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_1 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Block_Var.Node);
--# end
--# expr-done 1226
Concat_Result := Concat_String (Str, Fld_1); 
--# expr-start 1227 '<String at parser.py:1513>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1513
Str_1 := Create_String ("` outside of a raise expression"); 
--# expr-done 1227
Concat_Result_1 := Concat_String (Concat_Result, Str_1); 
--# property-call-start LktNode.error
Fld_2 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Concat_Result_1);
--# end
--# expr-done 1228
                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_2;
                Inc_Ref (Singleton.Items (1));
            
   If_Result_1 := Singleton;
else
   
   If_Result_1 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (If_Result_1);


--# expr-done 1229
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1230

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_117;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_117;




            raise;
      end;



   return Property_Result;
end Ref_Id_P_Check_Legality;
--# end



   


      

   --
   --  Primitives for Bare_If_Expr
   --

   



      
      procedure Initialize_Fields_For_If_Expr
        (Self : Bare_If_Expr
         ; If_Expr_F_Cond_Expr : Bare_Expr
         ; If_Expr_F_Then_Expr : Bare_Expr
         ; If_Expr_F_Alternatives : Bare_Elsif_Branch_List
         ; If_Expr_F_Else_Expr : Bare_Expr
        ) is
      begin

            Self.If_Expr_F_Cond_Expr := If_Expr_F_Cond_Expr;
            Self.If_Expr_F_Then_Expr := If_Expr_F_Then_Expr;
            Self.If_Expr_F_Alternatives := If_Expr_F_Alternatives;
            Self.If_Expr_F_Else_Expr := If_Expr_F_Else_Expr;
         

      end Initialize_Fields_For_If_Expr;

      
   function If_Expr_F_Cond_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.If_Expr_F_Cond_Expr;
      
   end;

      
   function If_Expr_F_Then_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.If_Expr_F_Then_Expr;
      
   end;

      
   function If_Expr_F_Alternatives
     (Node : Bare_If_Expr) return Bare_Elsif_Branch_List
   is
      

   begin
         
         return Node.If_Expr_F_Alternatives;
      
   end;

      
   function If_Expr_F_Else_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.If_Expr_F_Else_Expr;
      
   end;


   







--# property-start IfExpr.result_exprs /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3599
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Result_Exprs
  
  (Node : Bare_If_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Expr_Array_Access;

      
            procedure Finalizer_Scope_250 with Inline_Always;

      Fld : Internal_Entity_Expr;
Singleton : Internal_Entity_Expr_Array_Access;
Fld_1 : Internal_Entity_Elsif_Branch_List;
Fld_2 : Bare_Elsif_Branch_List;
Fld_3 : Internal_Entity_Info;
Item_71 : Internal_Entity_Elsif_Branch;
Bare_Item_71 : Bare_Elsif_Branch;
Fld_4 : Internal_Entity_Expr;
As_Entity : Internal_Entity_Elsif_Branch;
Is_Null : Boolean;
If_Result : Internal_Entity_Elsif_Branch;
Map_Result : Internal_Entity_Expr_Array_Access;
Concat_Result : Internal_Entity_Expr_Array_Access;
Fld_5 : Internal_Entity_Expr;
Singleton_1 : Internal_Entity_Expr_Array_Access;
Concat_Result_1 : Internal_Entity_Expr_Array_Access;
Let_Result : Internal_Entity_Expr_Array_Access;

            procedure Finalizer_Scope_250 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_250;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1236 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1231 '<FieldAccess for then_expr at parser.py:3605>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3605






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 1231
                Singleton := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
                
            
--# expr-start 1234 '<Map at parser.py:3606>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3606








   

   --# expr-start 1232 '<FieldAccess for alternatives at parser.py:3606>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3606






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Elsif_Branch_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 1232






   




Fld_2 := Fld_1.Node;

   
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
   

   

      if Fld_2 = null then
         
      Map_Result := Create_Internal_Entity_Expr_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Elsif_Branch_List := Fld_2;
      begin
         for Untyped_Item_71 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_71 := Untyped_Item_71; 
                  



Is_Null := Bare_Item_71 = null; 
if Is_Null then
   
   If_Result := No_Entity_Elsif_Branch;
else
   





   




Fld_3 := Fld_1.Info;

As_Entity := (Info => Fld_3, Node => Bare_Item_71); 
   If_Result := As_Entity;
end if;



                  Item_71 := If_Result; 

            
   --# scope-start


               --# bind alt Item_71

            
         
         
      --# expr-start 1233 '<FieldAccess for then_expr at parser.py:3606>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3606






   

      if Item_71.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Create_Internal_Entity_Expr (Node => Item_71.Node.Elsif_Branch_F_Then_Expr, Info => Item_71.Info);
--# expr-done 1233
      

         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_4;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1234
Concat_Result := Concat (Singleton, Map_Result); 
--# expr-start 1235 '<FieldAccess for else_expr at parser.py:3607>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3607






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 1235
                Singleton_1 := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton_1.Items (1) := Fld_5;
                
            
Concat_Result_1 := Concat (Concat_Result, Singleton_1); 
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1236

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_250;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_250;




            raise;
      end;



   return Property_Result;
end If_Expr_P_Result_Exprs;
--# end

   







--# property-start IfExpr.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3610
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Expr_Context_Free_Type
  
  (Node : Bare_If_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_251 with Inline_Always;

      Fld : Internal_Entity_Expr_Array_Access;
Fld_1 : Internal_Entity_Type_Decl;
Map_Result : Internal_Entity_Type_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Find_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_251 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_251;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1243 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1242 '<Find at parser.py:3620>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3620








   

   --# expr-start 1239 '<Map at parser.py:3619>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3619








   

   --# expr-start 1237 '<FieldAccess for result_exprs at parser.py:3618>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3618






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start IfExpr.result_exprs
Fld := Liblktlang.Implementation.If_Expr_P_Result_Exprs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1237

   
      declare
         Map_Result_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Entity_Expr_Array_Access := Fld;
      begin
         for Item_73 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind expr Item_73

            
         
         
      --# expr-start 1238 '<FieldAccess for expr_context_free_type at parser.py:3619>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3619






   

      if Item_73.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => Item_73.Node, E_Info => Item_73.Info);
--# end
--# expr-done 1238
      

         declare
            Item_To_Append : constant Internal_Entity_Type_Decl := Fld_1;
         begin
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Entity_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1239

   
      Find_Result := No_Entity_Type_Decl;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Map_Result;
      begin
         for Item_72 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind t Item_72

            
      --# expr-start 1241 '<Not at parser.py:3620>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3620
--# expr-start 1240 '<IsNull at parser.py:3620>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3620

Is_Null := Item_72.Node = null; 
--# expr-done 1240
Not_Val := not (Is_Null); 
--# expr-done 1241
      if Not_Val then
         Find_Result := Item_72; 
      end if;
   

            
   --# end


               exit when Not_Val;

         end loop;
      end;
   

   
   



--# expr-done 1242
Let_Result := Find_Result; 
--# end
--# expr-done 1243

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_251;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_251;




            raise;
      end;



   return Property_Result;
end If_Expr_P_Expr_Context_Free_Type;
--# end

   







--# property-start IfExpr.if_expected_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3623
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_If_Expected_Type
  
  (Node : Bare_If_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Var_Expr : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1246 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 1244 '<FieldAccess for expr_context_free_type at parser.py:3630>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3630






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1244
Var_Expr := Fld;





if Var_Expr /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind e Var_Expr


Scope_Result := Var_Expr;


   --# end


   Result_Var := Scope_Result;
else
   --# expr-start 1245 '<FieldAccess for expected_type at parser.py:3630>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3630






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.expected_type
Fld_1 := Liblktlang.Implementation.Expr_P_Expected_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1245
   Result_Var := Fld_1;
end if;



Let_Result := Result_Var; 
--# end
--# expr-done 1246

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end If_Expr_P_If_Expected_Type;
--# end



   


      

   --
   --  Primitives for Bare_Isa
   --

   



      
      procedure Initialize_Fields_For_Isa
        (Self : Bare_Isa
         ; Isa_F_Expr : Bare_Expr
         ; Isa_F_Dest_Type : Bare_Isa_List
        ) is
      begin

            Self.Isa_F_Expr := Isa_F_Expr;
            Self.Isa_F_Dest_Type := Isa_F_Dest_Type;
         

      end Initialize_Fields_For_Isa;

      
   function Isa_F_Expr
     (Node : Bare_Isa) return Bare_Expr
   is
      

   begin
         
         return Node.Isa_F_Expr;
      
   end;

      
   function Isa_F_Dest_Type
     (Node : Bare_Isa) return Bare_Isa_List
   is
      

   begin
         
         return Node.Isa_F_Dest_Type;
      
   end;




   


      

   --
   --  Primitives for Bare_Keep_Expr
   --

   



      
      procedure Initialize_Fields_For_Keep_Expr
        (Self : Bare_Keep_Expr
         ; Keep_Expr_F_Expr : Bare_Expr
         ; Keep_Expr_F_Keep_Type : Bare_Type_Ref
        ) is
      begin

            Self.Keep_Expr_F_Expr := Keep_Expr_F_Expr;
            Self.Keep_Expr_F_Keep_Type := Keep_Expr_F_Keep_Type;
         

      end Initialize_Fields_For_Keep_Expr;

      
   function Keep_Expr_F_Expr
     (Node : Bare_Keep_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Keep_Expr_F_Expr;
      
   end;

      
   function Keep_Expr_F_Keep_Type
     (Node : Bare_Keep_Expr) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Keep_Expr_F_Keep_Type;
      
   end;




   


      

   --
   --  Primitives for Bare_Lambda_Expr
   --

   



   

   

   

   

   

   

   


      procedure Lambda_Expr_Pre_Env_Actions
        (Self            : Bare_Lambda_Expr;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   
      if Add_To_Env_Only then
         return;
      end if;

      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Liblktlang.Implementation.Env_Trans_Parent_20 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
   
   
      end;



      
      procedure Initialize_Fields_For_Lambda_Expr
        (Self : Bare_Lambda_Expr
         ; Lambda_Expr_F_Params : Bare_Lambda_Arg_Decl_List
         ; Lambda_Expr_F_Return_Type : Bare_Type_Ref
         ; Lambda_Expr_F_Body : Bare_Expr
        ) is
      begin

            Self.Lambda_Expr_F_Params := Lambda_Expr_F_Params;
            Self.Lambda_Expr_F_Return_Type := Lambda_Expr_F_Return_Type;
            Self.Lambda_Expr_F_Body := Lambda_Expr_F_Body;
         

      end Initialize_Fields_For_Lambda_Expr;

      
   function Lambda_Expr_F_Params
     (Node : Bare_Lambda_Expr) return Bare_Lambda_Arg_Decl_List
   is
      

   begin
         
         return Node.Lambda_Expr_F_Params;
      
   end;

      
   function Lambda_Expr_F_Return_Type
     (Node : Bare_Lambda_Expr) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Lambda_Expr_F_Return_Type;
      
   end;

      
   function Lambda_Expr_F_Body
     (Node : Bare_Lambda_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Lambda_Expr_F_Body;
      
   end;


   







--# property-start LambdaExpr.expected_type_predicate /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3495
pragma Warnings (Off, "is not referenced");
 function Lambda_Expr_P_Expected_Type_Predicate
  
  (Node : Bare_Lambda_Expr
      ; Expected_Type : Internal_Entity_Type_Decl
  )

   return Boolean
is
   Self : Bare_Lambda_Expr  := Bare_Lambda_Expr (Node);
      --# bind self Self

   

   --# bind expected_type Expected_Type

   Property_Result : Boolean;

      

      Is_A : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1248 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1247 '<IsA FunctionType at parser.py:3497>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3497

Is_A := Expected_Type.Node /= null 
and then Expected_Type.Node.Kind in Lkt_Function_Type_Range; 
--# expr-done 1247
Let_Result := Is_A; 
--# end
--# expr-done 1248

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lambda_Expr_P_Expected_Type_Predicate;
--# end

   







--# property-start LambdaExpr.invalid_expected_type_error_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3499
pragma Warnings (Off, "is not referenced");
 function Lambda_Expr_P_Invalid_Expected_Type_Error_Name
  
  (Node : Bare_Lambda_Expr
  )

   return String_Type
is
   Self : Bare_Lambda_Expr  := Bare_Lambda_Expr (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_245 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_245 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_245;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1249 '<String at parser.py:3499>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3499
Str := Create_String ("a function literal"); 
--# expr-done 1249

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_245;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_245;




            raise;
      end;



   return Property_Result;
end Lambda_Expr_P_Invalid_Expected_Type_Error_Name;
--# end

   







--# property-start LambdaExpr.is_annotated /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3501
pragma Warnings (Off, "is not referenced");
 function Lambda_Expr_P_Is_Annotated
  
  (Node : Bare_Lambda_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Boolean
is
   Self : Bare_Lambda_Expr  := Bare_Lambda_Expr (Node);
     Ent : Internal_Entity_Lambda_Expr :=
       Internal_Entity_Lambda_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Boolean;

      

      Fld : Internal_Entity_Type_Ref;
Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1253 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1252 '<Not at parser.py:3508>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3508
--# expr-start 1251 '<IsNull at parser.py:3508>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3508
--# expr-start 1250 '<FieldAccess for return_type at parser.py:3508>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3508






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref (Node => Ent.Node.Lambda_Expr_F_Return_Type, Info => Ent.Info);
--# expr-done 1250
Is_Null := Fld.Node = null; 
--# expr-done 1251
Not_Val := not (Is_Null); 
--# expr-done 1252
Let_Result := Not_Val; 
--# end
--# expr-done 1253

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Lambda_Expr_P_Is_Annotated;
--# end

   







--# property-start LambdaExpr.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3510
pragma Warnings (Off, "is not referenced");
 function Lambda_Expr_P_Expr_Context_Free_Type
  
  (Node : Bare_Lambda_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Lambda_Expr  := Bare_Lambda_Expr (Node);
     Ent : Internal_Entity_Lambda_Expr :=
       Internal_Entity_Lambda_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_247 with Inline_Always;

      Fld : Internal_Entity_Lambda_Arg_Decl_List;
Fld_1 : Bare_Lambda_Arg_Decl_List;
Fld_2 : Internal_Entity_Info;
Item_74 : Internal_Entity_Lambda_Arg_Decl;
Bare_Item_74 : Bare_Lambda_Arg_Decl;
Fld_3 : Internal_Entity_Type_Ref;
Fld_4 : Internal_Entity_Type_Decl;
Fld_5 : Bare_Decl;
Cast_Result : Bare_Type_Decl;
As_Entity : Internal_Entity_Lambda_Arg_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Lambda_Arg_Decl;
Map_Result : Bare_Type_Decl_Array_Access;
Fld_6 : Internal_Entity_Type_Ref;
Fld_7 : Internal_Entity_Type_Decl;
Fld_8 : Bare_Decl;
Cast_Result_1 : Bare_Type_Decl;
Fld_9 : Bare_Function_Type;
As_Entity_1 : Internal_Entity_Function_Type;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Function_Type;
Cast_Result_2 : Internal_Entity_Type_Decl;
Fld_10 : Boolean;
If_Result_2 : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_247 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_247;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1267 '<If at parser.py:3510>' If_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3510



--# expr-start 1254 '<FieldAccess for is_annotated at parser.py:3512>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3512






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LambdaExpr.is_annotated
Fld_10 := Liblktlang.Implementation.Lambda_Expr_P_Is_Annotated (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1254
if Fld_10 then
   




--# expr-start 1266 '<as_bare_entity at parser.py:3519>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3519



--# expr-start 1265 '<FieldAccess for function_type at parser.py:3513>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3513






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1260 '<Map at parser.py:3514>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3514








   

   --# expr-start 1255 '<FieldAccess for params at parser.py:3514>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3514






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Lambda_Arg_Decl_List (Node => Ent.Node.Lambda_Expr_F_Params, Info => Ent.Info);
--# expr-done 1255






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Bare_Type_Decl_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Bare_Type_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Lambda_Arg_Decl_List := Fld_1;
      begin
         for Untyped_Item_74 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_74 := Untyped_Item_74; 
                  



Is_Null := Bare_Item_74 = null; 
if Is_Null then
   
   If_Result := No_Entity_Lambda_Arg_Decl;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_74); 
   If_Result := As_Entity;
end if;



                  Item_74 := If_Result; 

            
   --# scope-start


               --# bind p Item_74

            
         
         
      --# expr-start 1259 '<Cast to TypeDecl at parser.py:3516>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3516





--# expr-start 1258 '<FieldAccess for assert_bare at parser.py:3516>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3516



--# expr-start 1257 '<FieldAccess for designated_type at parser.py:3516>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3516



--# expr-start 1256 '<FieldAccess for decl_type at parser.py:3516>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3516






   

      if Item_74.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Type_Ref (Node => Item_74.Node.Lambda_Arg_Decl_F_Decl_Type, Info => Item_74.Info);
--# expr-done 1256


   

      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_4 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 1257


   

      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.assert_bare
Fld_5 := Liblktlang.Implementation.Decl_P_Assert_Bare (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 1258



   if Fld_5 = null
      or else Fld_5.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Fld_5;

   else
         Cast_Result := No_Bare_Lkt_Node;
   end if;


--# expr-done 1259
      

         declare
            Item_To_Append : constant Bare_Type_Decl := Cast_Result;
         begin
            Bare_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Bare_Type_Decl_Array
           (Items_Count => Natural (Bare_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1260
--# expr-start 1264 '<Cast to TypeDecl at parser.py:3518>' Cast_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3518





--# expr-start 1263 '<FieldAccess for assert_bare at parser.py:3518>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3518



--# expr-start 1262 '<FieldAccess for designated_type at parser.py:3518>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3518



--# expr-start 1261 '<FieldAccess for return_type at parser.py:3518>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3518






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_6 := Create_Internal_Entity_Type_Ref (Node => Ent.Node.Lambda_Expr_F_Return_Type, Info => Ent.Info);
--# expr-done 1261


   

      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_7 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1262


   

      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Decl.assert_bare
Fld_8 := Liblktlang.Implementation.Decl_P_Assert_Bare (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 1263



   if Fld_8 = null
      or else Fld_8.Kind in Lkt_Type_Decl
   then
      
      Cast_Result_1 := Fld_8;

   else
         Cast_Result_1 := No_Bare_Lkt_Node;
   end if;


--# expr-done 1264
--# property-call-start LktNode.function_type
Fld_9 := Liblktlang.Implementation.Lkt_Node_P_Function_Type (Node => Self, Args_Types => Map_Result, Return_Type => Cast_Result_1);
--# end
--# expr-done 1265
Is_Null_1 := Fld_9 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Function_Type;
else
   

As_Entity_1 := (Info => No_Entity_Info, Node => Fld_9); 
   If_Result_1 := As_Entity_1;
end if;



--# expr-done 1266



   
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);


   If_Result_2 := Cast_Result_2;
else
   
   If_Result_2 := No_Entity_Type_Decl;
end if;



--# expr-done 1267

         Property_Result := If_Result_2;
         
   --# end
      Finalizer_Scope_247;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_247;




            raise;
      end;



   return Property_Result;
end Lambda_Expr_P_Expr_Context_Free_Type;
--# end

   







--# property-start LambdaExpr.check_correctness_pre /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3524
pragma Warnings (Off, "is not referenced");
 function Lambda_Expr_P_Check_Correctness_Pre
  
  (Node : Bare_Lambda_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Lambda_Expr  := Bare_Lambda_Expr (Node);
     Ent : Internal_Entity_Lambda_Expr :=
       Internal_Entity_Lambda_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_248 with Inline_Always;

      Block_Var : Boolean_Array_Access;
Fld : Internal_Entity_Lambda_Arg_Decl_List;
Fld_1 : Bare_Lambda_Arg_Decl_List;
Fld_2 : Internal_Entity_Info;
Item_75 : Internal_Entity_Lambda_Arg_Decl;
Bare_Item_75 : Bare_Lambda_Arg_Decl;
Fld_3 : Internal_Entity_Type_Ref;
Is_Null : Boolean;
Not_Val : Boolean;
As_Entity : Internal_Entity_Lambda_Arg_Decl;
Is_Null_1 : Boolean;
If_Result : Internal_Entity_Lambda_Arg_Decl;
Map_Result : Boolean_Array_Access;
Quantifier_Result : Boolean;
Not_Val_1 : Boolean;
Quantifier_Result_1 : Boolean;
If_Result_1 : Boolean;
Not_Val_2 : Boolean;
Fld_4 : Bare_Lambda_Arg_Decl_List;
Str : String_Type;
Fld_5 : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
If_Result_2 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_248 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_248;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1281 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1272 '<Map at parser.py:3528>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3528








   

   --# expr-start 1268 '<FieldAccess for params at parser.py:3528>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3528






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Lambda_Arg_Decl_List (Node => Ent.Node.Lambda_Expr_F_Params, Info => Ent.Info);
--# expr-done 1268






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Boolean_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Boolean_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Lambda_Arg_Decl_List := Fld_1;
      begin
         for Untyped_Item_75 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_75 := Untyped_Item_75; 
                  



Is_Null_1 := Bare_Item_75 = null; 
if Is_Null_1 then
   
   If_Result := No_Entity_Lambda_Arg_Decl;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_75); 
   If_Result := As_Entity;
end if;



                  Item_75 := If_Result; 

            
   --# scope-start


               --# bind p Item_75

            
         
         
      --# expr-start 1271 '<Not at parser.py:3529>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3529
--# expr-start 1270 '<IsNull at parser.py:3529>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3529
--# expr-start 1269 '<FieldAccess for decl_type at parser.py:3529>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3529






   

      if Item_75.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_3 := Create_Internal_Entity_Type_Ref (Node => Item_75.Node.Lambda_Arg_Decl_F_Decl_Type, Info => Item_75.Info);
--# expr-done 1269
Is_Null := Fld_3.Node = null; 
--# expr-done 1270
Not_Val := not (Is_Null); 
--# expr-done 1271
      

         declare
            Item_To_Append : constant Boolean := Not_Val;
         begin
            Boolean_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Boolean_Array
           (Items_Count => Natural (Boolean_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Boolean_Vectors.Get
              (Map_Result_Vec,
               I + Boolean_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Boolean_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1272
Block_Var := Map_Result; Inc_Ref (Block_Var);
--# bind typs Block_Var
--# expr-start 1280 '<Cond at parser.py:3532>' If_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3532



--# expr-start 1276 '<Not at parser.py:3534>' Not_Val_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3534



--# expr-start 1274 '<Not at parser.py:3536>' Not_Val_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3536
--# expr-start 1273 '<AnyQuantifier at parser.py:3536>' Quantifier_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3536








   

   

   
      Quantifier_Result := False;
   

   

      

      declare
         
         Collection : constant Boolean_Array_Access := Block_Var;
      begin
         for Item_76 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind t Item_76

            
      
      Quantifier_Result := Item_76;
   

            
   --# end


               exit when Quantifier_Result;

         end loop;
      end;
   

   
   



--# expr-done 1273
Not_Val_1 := not (Quantifier_Result); 
--# expr-done 1274
if Not_Val_1 then
   
   If_Result_1 := True;
else
   --# expr-start 1275 '<AllQuantifier at parser.py:3538>' Quantifier_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3538








   

   

   
      Quantifier_Result_1 := True;
   

   

      

      declare
         
         Collection : constant Boolean_Array_Access := Block_Var;
      begin
         for Item_77 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind t Item_77

            
      
      Quantifier_Result_1 := Item_77;
   

            
   --# end


               exit when not Quantifier_Result_1;

         end loop;
      end;
   

   
   



--# expr-done 1275
   If_Result_1 := Quantifier_Result_1;
end if;



Not_Val_2 := not (If_Result_1); 
--# expr-done 1276
if Not_Val_2 then
   --# expr-start 1279 '<FieldAccess for error at parser.py:3541>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3541



--# expr-start 1277 '<FieldAccess for params at parser.py:3541>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3541






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Self.Lambda_Expr_F_Params;
--# expr-done 1277


   

      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1278 '<String at parser.py:3542>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3542
Str := Create_String ("Invalid type annotations for lambda parameters." & Character_Type'Val (10)
 & "Annotate either all or no params"); 
--# expr-done 1278
--# property-call-start LktNode.error
Fld_5 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Fld_4, Message => Str);
--# end
--# expr-done 1279
                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_5;
                Inc_Ref (Singleton.Items (1));
            
   If_Result_2 := Singleton;
else
   
   If_Result_2 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (If_Result_2);


--# expr-done 1280
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1281

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_248;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_248;




            raise;
      end;



   return Property_Result;
end Lambda_Expr_P_Check_Correctness_Pre;
--# end

   







--# property-start LambdaExpr._env_trans_parent_20 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3549
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_20
  
  (Node : Bare_Lambda_Expr
  )

   return Boolean
is
   Self : Bare_Lambda_Expr  := Bare_Lambda_Expr (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Trans_Parent_20;
--# end



   


      

   --
   --  Primitives for Bare_Lit
   --

   







   


      

   --
   --  Primitives for Bare_Big_Num_Lit
   --

   





   







--# property-start BigNumLit.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3989
pragma Warnings (Off, "is not referenced");
 function Big_Num_Lit_P_Expr_Context_Free_Type
  
  (Node : Bare_Big_Num_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Big_Num_Lit  := Bare_Big_Num_Lit (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity_Named_Type_Decl;
Let_Result : Internal_Entity_Named_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         




--# expr-start 1283 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1282 '<FieldAccess for bigint_type at parser.py:3991>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3991






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.bigint_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Bigint_Type (Node => Self);
--# end
--# expr-done 1282
Let_Result := Fld; 
--# end
--# expr-done 1283



   
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Let_Result.Node,
         Info => Let_Result.Info);



         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Big_Num_Lit_P_Expr_Context_Free_Type;
--# end

   







--# property-start BigNumLit.invalid_expected_type_error_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3993
pragma Warnings (Off, "is not referenced");
 function Big_Num_Lit_P_Invalid_Expected_Type_Error_Name
  
  (Node : Bare_Big_Num_Lit
  )

   return String_Type
is
   Self : Bare_Big_Num_Lit  := Bare_Big_Num_Lit (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_277 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_277 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_277;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1284 '<String at parser.py:3993>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3993
Str := Create_String ("a big number literal"); 
--# expr-done 1284

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_277;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_277;




            raise;
      end;



   return Property_Result;
end Big_Num_Lit_P_Invalid_Expected_Type_Error_Name;
--# end



   


      

   --
   --  Primitives for Bare_Char_Lit
   --

   





   







--# property-start CharLit.expected_type_predicate /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3963
pragma Warnings (Off, "is not referenced");
 function Char_Lit_P_Expected_Type_Predicate
  
  (Node : Bare_Char_Lit
      ; Expected_Type : Internal_Entity_Type_Decl
  )

   return Boolean
is
   Self : Bare_Char_Lit  := Bare_Char_Lit (Node);
      --# bind self Self

   

   --# bind expected_type Expected_Type

   Property_Result : Boolean;

      

      Fld : Internal_Entity_Named_Type_Decl;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1287 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1286 '<Eq at parser.py:3965>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3965






Cast_Expr := Expected_Type; 



   
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);







--# expr-start 1285 '<FieldAccess for char_type at parser.py:3965>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3965






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.char_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Char_Type (Node => Self);
--# end
--# expr-done 1285



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);


Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1286
Let_Result := Is_Equiv; 
--# end
--# expr-done 1287

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Char_Lit_P_Expected_Type_Predicate;
--# end

   







--# property-start CharLit.invalid_expected_type_error_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3967
pragma Warnings (Off, "is not referenced");
 function Char_Lit_P_Invalid_Expected_Type_Error_Name
  
  (Node : Bare_Char_Lit
  )

   return String_Type
is
   Self : Bare_Char_Lit  := Bare_Char_Lit (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_273 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_273 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_273;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1288 '<String at parser.py:3967>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3967
Str := Create_String ("a character literal"); 
--# expr-done 1288

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_273;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_273;




            raise;
      end;



   return Property_Result;
end Char_Lit_P_Invalid_Expected_Type_Error_Name;
--# end



   


      

   --
   --  Primitives for Bare_Null_Lit
   --

   



      
      procedure Initialize_Fields_For_Null_Lit
        (Self : Bare_Null_Lit
         ; Null_Lit_F_Dest_Type : Bare_Type_Ref
        ) is
      begin

            Self.Null_Lit_F_Dest_Type := Null_Lit_F_Dest_Type;
         

      end Initialize_Fields_For_Null_Lit;

      
   function Null_Lit_F_Dest_Type
     (Node : Bare_Null_Lit) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Null_Lit_F_Dest_Type;
      
   end;




   


      

   --
   --  Primitives for Bare_Num_Lit
   --

   





   







--# property-start NumLit.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3976
pragma Warnings (Off, "is not referenced");
 function Num_Lit_P_Expr_Context_Free_Type
  
  (Node : Bare_Num_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Num_Lit  := Bare_Num_Lit (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity_Named_Type_Decl;
Let_Result : Internal_Entity_Named_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         




--# expr-start 1290 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1289 '<FieldAccess for int_type at parser.py:3978>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3978






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.int_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 1289
Let_Result := Fld; 
--# end
--# expr-done 1290



   
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Let_Result.Node,
         Info => Let_Result.Info);



         Property_Result := Cast_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Num_Lit_P_Expr_Context_Free_Type;
--# end

   







--# property-start NumLit.invalid_expected_type_error_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3980
pragma Warnings (Off, "is not referenced");
 function Num_Lit_P_Invalid_Expected_Type_Error_Name
  
  (Node : Bare_Num_Lit
  )

   return String_Type
is
   Self : Bare_Num_Lit  := Bare_Num_Lit (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_275 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_275 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_275;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1291 '<String at parser.py:3980>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3980
Str := Create_String ("a number literal"); 
--# expr-done 1291

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_275;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_275;




            raise;
      end;



   return Property_Result;
end Num_Lit_P_Invalid_Expected_Type_Error_Name;
--# end



   


      

   --
   --  Primitives for Bare_String_Lit
   --

   





   







--# property-start '[dispatcher]StringLit.denoted_value' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_String_Lit_P_Denoted_Value
  
  (Node : Bare_String_Lit
  )

   return Internal_Decoded_String_Value
is
   Self : Bare_String_Lit  := Bare_String_Lit (Node);
      --# bind self Self

   


   Property_Result : Internal_Decoded_String_Value;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_String_Lit (Self.Kind) is
               when Lkt_Block_String_Lit =>
                  --# property-call-start BlockStringLit.denoted_value
                  Property_Result := Block_String_Lit_P_Denoted_Value
                    (Self
                    );
                  --# end
               when Lkt_Single_Line_String_Lit .. Lkt_Pattern_Single_Line_String_Lit =>
                  --# property-call-start SingleLineStringLit.denoted_value
                  Property_Result := Single_Line_String_Lit_P_Denoted_Value
                    (Self
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_String_Lit_P_Denoted_Value;
--# end

   







--# property-start '[dispatcher]StringLit.is_prefixed_string' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_String_Lit_P_Is_Prefixed_String
  
  (Node : Bare_String_Lit
  )

   return Boolean
is
   Self : Bare_String_Lit  := Bare_String_Lit (Node);
      --# bind self Self

   


   Property_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_String_Lit (Self.Kind) is
               when Lkt_Block_String_Lit =>
                  --# property-call-start BlockStringLit.is_prefixed_string
                  Property_Result := Block_String_Lit_P_Is_Prefixed_String
                    (Self
                    );
                  --# end
               when Lkt_Single_Line_String_Lit .. Lkt_Pattern_Single_Line_String_Lit =>
                  --# property-call-start SingleLineStringLit.is_prefixed_string
                  Property_Result := Single_Line_String_Lit_P_Is_Prefixed_String
                    (Self
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_String_Lit_P_Is_Prefixed_String;
--# end

   







--# property-start '[dispatcher]StringLit.prefix' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_String_Lit_P_Prefix
  
  (Node : Bare_String_Lit
  )

   return Character_Type
is
   Self : Bare_String_Lit  := Bare_String_Lit (Node);
      --# bind self Self

   


   Property_Result : Character_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_String_Lit (Self.Kind) is
               when Lkt_Block_String_Lit =>
                  --# property-call-start BlockStringLit.prefix
                  Property_Result := Block_String_Lit_P_Prefix
                    (Self
                    );
                  --# end
               when Lkt_Single_Line_String_Lit .. Lkt_Pattern_Single_Line_String_Lit =>
                  --# property-call-start SingleLineStringLit.prefix
                  Property_Result := Single_Line_String_Lit_P_Prefix
                    (Self
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_String_Lit_P_Prefix;
--# end

   







--# property-start StringLit.is_regexp_literal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3837
pragma Warnings (Off, "is not referenced");
 function String_Lit_P_Is_Regexp_Literal
  
  (Node : Bare_String_Lit
  )

   return Boolean
is
   Self : Bare_String_Lit  := Bare_String_Lit (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      Fld : Character_Type;
Is_Equal : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1294 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1293 '<Eq at parser.py:3843>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3843
--# expr-start 1292 '<FieldAccess for prefix at parser.py:3843>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3843






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]StringLit.prefix'
Fld := Liblktlang.Implementation.Dispatcher_String_Lit_P_Prefix (Node => Self);
--# end
--# expr-done 1292

Is_Equal := Fld = Character_Type'Val (112); 
--# expr-done 1293
Let_Result := Is_Equal; 
--# end
--# expr-done 1294

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end String_Lit_P_Is_Regexp_Literal;
--# end



   


      

   --
   --  Primitives for Bare_Block_String_Lit
   --

   



      
      procedure Initialize_Fields_For_Block_String_Lit
        (Self : Bare_Block_String_Lit
         ; Block_String_Lit_F_Lines : Bare_Block_String_Line_List
        ) is
      begin

            Self.Block_String_Lit_F_Lines := Block_String_Lit_F_Lines;
         

      end Initialize_Fields_For_Block_String_Lit;

      
   function Block_String_Lit_F_Lines
     (Node : Bare_Block_String_Lit) return Bare_Block_String_Line_List
   is
      

   begin
         
         return Node.Block_String_Lit_F_Lines;
      
   end;


   







--# property-start BlockStringLit.is_prefixed_string /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3873
pragma Warnings (Off, "is not referenced");
 function Block_String_Lit_P_Is_Prefixed_String
  
  (Node : Bare_Block_String_Lit
  )

   return Boolean
is
   Self : Bare_Block_String_Lit  := Bare_Block_String_Lit (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1295 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := False; 
--# end
--# expr-done 1295

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Block_String_Lit_P_Is_Prefixed_String;
--# end

   







--# property-start BlockStringLit.prefix /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3877
pragma Warnings (Off, "is not referenced");
 function Block_String_Lit_P_Prefix
  
  (Node : Bare_Block_String_Lit
  )

   return Character_Type
is
   Self : Bare_Block_String_Lit  := Bare_Block_String_Lit (Node);
      --# bind self Self

   


   Property_Result : Character_Type;

      

      Let_Result : Character_Type;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1296 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := Character_Type'Val (0); 
--# end
--# expr-done 1296

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Block_String_Lit_P_Prefix;
--# end



   


      

   --
   --  Primitives for Bare_Single_Line_String_Lit
   --

   





   







--# property-start SingleLineStringLit.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3905
pragma Warnings (Off, "is not referenced");
 function Single_Line_String_Lit_P_Expr_Context_Free_Type
  
  (Node : Bare_Single_Line_String_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Single_Line_String_Lit  := Bare_Single_Line_String_Lit (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Boolean;
Fld_1 : Internal_Entity_Named_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
If_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1300 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1299 '<Cond at parser.py:3907>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3907



--# expr-start 1297 '<FieldAccess for is_regexp_literal at parser.py:3908>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3908






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start StringLit.is_regexp_literal
Fld := Liblktlang.Implementation.String_Lit_P_Is_Regexp_Literal (Node => Self);
--# end
--# expr-done 1297
if Fld then
   




--# expr-start 1298 '<FieldAccess for regexp_type at parser.py:3909>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3909






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.regexp_type
Fld_1 := Liblktlang.Implementation.Lkt_Node_P_Regexp_Type (Node => Self);
--# end
--# expr-done 1298



   
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);


   If_Result := Cast_Result;
else
   
   If_Result := No_Entity_Type_Decl;
end if;



--# expr-done 1299
Let_Result := If_Result; 
--# end
--# expr-done 1300

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Single_Line_String_Lit_P_Expr_Context_Free_Type;
--# end

   







--# property-start SingleLineStringLit.expected_type_predicate /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3914
pragma Warnings (Off, "is not referenced");
 function Single_Line_String_Lit_P_Expected_Type_Predicate
  
  (Node : Bare_Single_Line_String_Lit
      ; Expected_Type : Internal_Entity_Type_Decl
  )

   return Boolean
is
   Self : Bare_Single_Line_String_Lit  := Bare_Single_Line_String_Lit (Node);
      --# bind self Self

   

   --# bind expected_type Expected_Type

   Property_Result : Boolean;

      

      Fld : Internal_Entity_Named_Type_Decl;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_1 : Internal_Entity_Named_Type_Decl;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1305 '<Block at ???>' Let_Result None
--# scope-start



--# expr-start 1302 '<Eq at parser.py:3919>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3919






Cast_Expr := Expected_Type; 



   
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);







--# expr-start 1301 '<FieldAccess for string_type at parser.py:3919>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3919






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.string_type
Fld := Liblktlang.Implementation.Lkt_Node_P_String_Type (Node => Self);
--# end
--# expr-done 1301



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);


Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1302
if Is_Equiv then
   
   If_Result := True;
else
   --# expr-start 1304 '<Eq at parser.py:3920>' Is_Equiv_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3920






Cast_Expr_1 := Expected_Type; 



   
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);







--# expr-start 1303 '<FieldAccess for symbol_type at parser.py:3920>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3920






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.symbol_type
Fld_1 := Liblktlang.Implementation.Lkt_Node_P_Symbol_Type (Node => Self);
--# end
--# expr-done 1303



   
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);


Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 1304
   If_Result := Is_Equiv_1;
end if;



Let_Result := If_Result; 
--# end
--# expr-done 1305

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Single_Line_String_Lit_P_Expected_Type_Predicate;
--# end

   







--# property-start SingleLineStringLit.invalid_expected_type_error_name /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3922
pragma Warnings (Off, "is not referenced");
 function Single_Line_String_Lit_P_Invalid_Expected_Type_Error_Name
  
  (Node : Bare_Single_Line_String_Lit
  )

   return String_Type
is
   Self : Bare_Single_Line_String_Lit  := Bare_Single_Line_String_Lit (Node);
      --# bind self Self

   


   Property_Result : String_Type;

      
            procedure Finalizer_Scope_270 with Inline_Always;

      Str : String_Type;

            procedure Finalizer_Scope_270 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_270;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1306 '<String at parser.py:3922>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3922
Str := Create_String ("a string literal"); 
--# expr-done 1306

         Property_Result := Str;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_270;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_270;




            raise;
      end;



   return Property_Result;
end Single_Line_String_Lit_P_Invalid_Expected_Type_Error_Name;
--# end



   


      

   --
   --  Primitives for Bare_Pattern_Single_Line_String_Lit
   --

   







   


      

   --
   --  Primitives for Bare_Logic_Expr
   --

   



      
      procedure Initialize_Fields_For_Logic_Expr
        (Self : Bare_Logic_Expr
         ; Logic_Expr_F_Expr : Bare_Expr
        ) is
      begin

            Self.Logic_Expr_F_Expr := Logic_Expr_F_Expr;
         

      end Initialize_Fields_For_Logic_Expr;

      
   function Logic_Expr_F_Expr
     (Node : Bare_Logic_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Logic_Expr_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Match_Expr
   --

   



      
      procedure Initialize_Fields_For_Match_Expr
        (Self : Bare_Match_Expr
         ; Match_Expr_F_Match_Expr : Bare_Expr
         ; Match_Expr_F_Branches : Bare_Match_Branch_List
        ) is
      begin

            Self.Match_Expr_F_Match_Expr := Match_Expr_F_Match_Expr;
            Self.Match_Expr_F_Branches := Match_Expr_F_Branches;
         

      end Initialize_Fields_For_Match_Expr;

      
   function Match_Expr_F_Match_Expr
     (Node : Bare_Match_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Match_Expr_F_Match_Expr;
      
   end;

      
   function Match_Expr_F_Branches
     (Node : Bare_Match_Expr) return Bare_Match_Branch_List
   is
      

   begin
         
         return Node.Match_Expr_F_Branches;
      
   end;




   


      

   --
   --  Primitives for Bare_Not_Expr
   --

   



      
      procedure Initialize_Fields_For_Not_Expr
        (Self : Bare_Not_Expr
         ; Not_Expr_F_Expr : Bare_Expr
        ) is
      begin

            Self.Not_Expr_F_Expr := Not_Expr_F_Expr;
         

      end Initialize_Fields_For_Not_Expr;

      
   function Not_Expr_F_Expr
     (Node : Bare_Not_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Not_Expr_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Paren_Expr
   --

   



      
      procedure Initialize_Fields_For_Paren_Expr
        (Self : Bare_Paren_Expr
         ; Paren_Expr_F_Expr : Bare_Expr
        ) is
      begin

            Self.Paren_Expr_F_Expr := Paren_Expr_F_Expr;
         

      end Initialize_Fields_For_Paren_Expr;

      
   function Paren_Expr_F_Expr
     (Node : Bare_Paren_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Paren_Expr_F_Expr;
      
   end;


   







--# property-start ParenExpr.expr_context_free_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2965
pragma Warnings (Off, "is not referenced");
 function Paren_Expr_P_Expr_Context_Free_Type
  
  (Node : Bare_Paren_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Paren_Expr  := Bare_Paren_Expr (Node);
     Ent : Internal_Entity_Paren_Expr :=
       Internal_Entity_Paren_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1308 '<FieldAccess for expr_context_free_type at parser.py:2965>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2965



--# expr-start 1307 '<FieldAccess for expr at parser.py:2965>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2965






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Paren_Expr_F_Expr, Info => Ent.Info);
--# expr-done 1307


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.expr_context_free_type'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Expr_Context_Free_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1308

         Property_Result := Fld_1;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Paren_Expr_P_Expr_Context_Free_Type;
--# end



   


      

   --
   --  Primitives for Bare_Raise_Expr
   --

   



      
      procedure Initialize_Fields_For_Raise_Expr
        (Self : Bare_Raise_Expr
         ; Raise_Expr_F_Dest_Type : Bare_Type_Ref
         ; Raise_Expr_F_Except_Expr : Bare_Expr
        ) is
      begin

            Self.Raise_Expr_F_Dest_Type := Raise_Expr_F_Dest_Type;
            Self.Raise_Expr_F_Except_Expr := Raise_Expr_F_Except_Expr;
         

      end Initialize_Fields_For_Raise_Expr;

      
   function Raise_Expr_F_Dest_Type
     (Node : Bare_Raise_Expr) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Raise_Expr_F_Dest_Type;
      
   end;

      
   function Raise_Expr_F_Except_Expr
     (Node : Bare_Raise_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Raise_Expr_F_Except_Expr;
      
   end;


   







--# property-start RaiseExpr.check_legality /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3567
pragma Warnings (Off, "is not referenced");
 function Raise_Expr_P_Check_Legality
  
  (Node : Bare_Raise_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Raise_Expr  := Bare_Raise_Expr (Node);
     Ent : Internal_Entity_Raise_Expr :=
       Internal_Entity_Raise_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_249 with Inline_Always;
            procedure Finalizer_Scope_537 with Inline_Always;

      Block_Var : Internal_Entity_Named_Type_Decl;
Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Call_Expr;
Var_Expr_2 : Internal_Param_Match;
Fld : Internal_Entity_Named_Type_Decl;
Fld_1 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Call_Expr;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Named_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Fld_4 : Internal_Param_Match_Array_Access;
Get_Result : Internal_Param_Match;
Fld_5 : Internal_Entity_Param;
Fld_6 : Internal_Entity_Expr;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Internal_Semantic_Result_Array_Access;
Str : String_Type;
Fld_7 : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;
Fld_8 : Internal_Semantic_Result := No_Semantic_Result;
Singleton : Internal_Semantic_Result_Array_Access;
Result_Var_2 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_249 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Singleton);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_249;
            procedure Finalizer_Scope_537 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_537;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1324 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1309 '<FieldAccess for property_error_type at parser.py:3571>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3571






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.property_error_type
Fld := Liblktlang.Implementation.Lkt_Node_P_Property_Error_Type (Node => Ent.Node);
--# end
--# expr-done 1309
Block_Var := Fld; 
--# bind error_type Block_Var






--# expr-start 1311 '<Cast to CallExpr at parser.py:3573>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3573





--# expr-start 1310 '<FieldAccess for except_expr at parser.py:3573>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3573






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Expr (Node => Ent.Node.Raise_Expr_F_Except_Expr, Info => Ent.Info);
--# expr-done 1310



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Call_Expr_Range
   then
      
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Cast_Result := No_Entity_Call_Expr;
   end if;


--# expr-done 1311
Var_Expr_1 := Cast_Result;





if Var_Expr_1 /= No_Entity_Call_Expr then
   



   --# scope-start


   --# bind ce Var_Expr_1




--# expr-start 1314 '<Eq at parser.py:3575>' Is_Equiv /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3575





--# expr-start 1313 '<FieldAccess for check_referenced_decl at parser.py:3575>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3575



--# expr-start 1312 '<FieldAccess for name at parser.py:3575>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3575






   

      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Create_Internal_Entity_Expr (Node => Var_Expr_1.Node.Call_Expr_F_Name, Info => Var_Expr_1.Info);
--# expr-done 1312


   

      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_referenced_decl
Fld_3 := Liblktlang.Implementation.Expr_P_Check_Referenced_Decl (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 1313



   
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);








Cast_Expr := Block_Var; 



   
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);


Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1314
if Is_Equiv then
   


--# expr-start 1316 '<at at parser.py:3576>' Get_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3576

--# expr-start 1315 '<FieldAccess for match_params at parser.py:3576>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3576






   

      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start CallExpr.match_params
Fld_4 := Liblktlang.Implementation.Call_Expr_P_Match_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1315


Get_Result := Get (Self, Fld_4, 0, True); 
--# expr-done 1316
Var_Expr_2 := Get_Result;





if Var_Expr_2 /= No_Param_Match then
   



   --# scope-start


   --# bind p Var_Expr_2

--# expr-start 1319 '<IsA StringLit at parser.py:3579>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3579
--# expr-start 1318 '<FieldAccess for value at parser.py:3579>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3579



--# expr-start 1317 '<FieldAccess for actual at parser.py:3579>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3579






   




Fld_5 := Var_Expr_2.Actual;
--# expr-done 1317


   

      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_6 := Create_Internal_Entity_Expr (Node => Fld_5.Node.Param_F_Value, Info => Fld_5.Info);
--# expr-done 1318
Is_A := Fld_6.Node /= null 
and then Fld_6.Node.Kind in Lkt_String_Lit; 
--# expr-done 1319
Scope_Result := Is_A;


   --# end


   Result_Var := Scope_Result;
else
   
   Result_Var := True;
end if;



   If_Result := Result_Var;
else
   
   If_Result := False;
end if;



Scope_Result_1 := If_Result;


   --# end
      Finalizer_Scope_537;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := False;
end if;



Var_Expr := Result_Var_1;





if Var_Expr /= False then
   



   --# scope-start


   --# bind _ Var_Expr


Scope_Result_2 := No_Internal_Semantic_Result_Array_Type;
   Inc_Ref (Scope_Result_2);


   --# end


   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 1323 '<FieldAccess for error at parser.py:3583>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3583






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1320 '<String at parser.py:3584>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3584
Str := Create_String ("raised expression needs to be a `"); 
--# expr-done 1320
--# expr-start 1321 '<FieldAccess for full_name at parser.py:3585>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3585






   

      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Decl.full_name'
Fld_7 := Liblktlang.Implementation.Dispatcher_Decl_P_Full_Name (Node => Block_Var.Node);
--# end
--# expr-done 1321
Concat_Result := Concat_String (Str, Fld_7); 
--# expr-start 1322 '<String at parser.py:3585>' Str_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3585
Str_1 := Create_String ("` constructor call"); 
--# expr-done 1322
Concat_Result_1 := Concat_String (Concat_Result, Str_1); 
--# property-call-start LktNode.error
Fld_8 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Self, Message => Concat_Result_1);
--# end
--# expr-done 1323
                Singleton := Create_Internal_Semantic_Result_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_8;
                Inc_Ref (Singleton.Items (1));
            
   Result_Var_2 := Singleton;
end if;

      Inc_Ref (Result_Var_2);


Let_Result := Result_Var_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1324

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_249;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_249;
                     Finalizer_Scope_537;




            raise;
      end;



   return Property_Result;
end Raise_Expr_P_Check_Legality;
--# end



   


      

   --
   --  Primitives for Bare_Subscript_Expr
   --

   



      
      procedure Initialize_Fields_For_Subscript_Expr
        (Self : Bare_Subscript_Expr
         ; Subscript_Expr_F_Prefix : Bare_Expr
         ; Subscript_Expr_F_Index : Bare_Expr
        ) is
      begin

            Self.Subscript_Expr_F_Prefix := Subscript_Expr_F_Prefix;
            Self.Subscript_Expr_F_Index := Subscript_Expr_F_Index;
         

      end Initialize_Fields_For_Subscript_Expr;

      
   function Subscript_Expr_F_Prefix
     (Node : Bare_Subscript_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Subscript_Expr_F_Prefix;
      
   end;

      
   function Subscript_Expr_F_Index
     (Node : Bare_Subscript_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Subscript_Expr_F_Index;
      
   end;




   


      

   --
   --  Primitives for Bare_Null_Cond_Subscript_Expr
   --

   



      
      procedure Initialize_Fields_For_Null_Cond_Subscript_Expr
        (Self : Bare_Null_Cond_Subscript_Expr
         ; Subscript_Expr_F_Prefix : Bare_Expr
         ; Subscript_Expr_F_Index : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Subscript_Expr
              (Self, Subscript_Expr_F_Prefix, Subscript_Expr_F_Index);

         

      end Initialize_Fields_For_Null_Cond_Subscript_Expr;




   


      

   --
   --  Primitives for Bare_Try_Expr
   --

   



      
      procedure Initialize_Fields_For_Try_Expr
        (Self : Bare_Try_Expr
         ; Try_Expr_F_Try_Expr : Bare_Expr
         ; Try_Expr_F_Or_Expr : Bare_Expr
        ) is
      begin

            Self.Try_Expr_F_Try_Expr := Try_Expr_F_Try_Expr;
            Self.Try_Expr_F_Or_Expr := Try_Expr_F_Or_Expr;
         

      end Initialize_Fields_For_Try_Expr;

      
   function Try_Expr_F_Try_Expr
     (Node : Bare_Try_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Try_Expr_F_Try_Expr;
      
   end;

      
   function Try_Expr_F_Or_Expr
     (Node : Bare_Try_Expr) return Bare_Expr
   is
      

   begin
         
         return Node.Try_Expr_F_Or_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Un_Op
   --

   



      
      procedure Initialize_Fields_For_Un_Op
        (Self : Bare_Un_Op
         ; Un_Op_F_Op : Bare_Op
         ; Un_Op_F_Expr : Bare_Expr
        ) is
      begin

            Self.Un_Op_F_Op := Un_Op_F_Op;
            Self.Un_Op_F_Expr := Un_Op_F_Expr;
         

      end Initialize_Fields_For_Un_Op;

      
   function Un_Op_F_Op
     (Node : Bare_Un_Op) return Bare_Op
   is
      

   begin
         
         return Node.Un_Op_F_Op;
      
   end;

      
   function Un_Op_F_Expr
     (Node : Bare_Un_Op) return Bare_Expr
   is
      

   begin
         
         return Node.Un_Op_F_Expr;
      
   end;




   


      

   --
   --  Primitives for Bare_Full_Decl
   --

   



      
      procedure Initialize_Fields_For_Full_Decl
        (Self : Bare_Full_Decl
         ; Full_Decl_F_Doc : Bare_String_Lit
         ; Full_Decl_F_Decl_Annotations : Bare_Decl_Annotation_List
         ; Full_Decl_F_Decl : Bare_Decl
        ) is
      begin

            Self.Full_Decl_F_Doc := Full_Decl_F_Doc;
            Self.Full_Decl_F_Decl_Annotations := Full_Decl_F_Decl_Annotations;
            Self.Full_Decl_F_Decl := Full_Decl_F_Decl;
         

      end Initialize_Fields_For_Full_Decl;

      
   function Full_Decl_F_Doc
     (Node : Bare_Full_Decl) return Bare_String_Lit
   is
      

   begin
         
         return Node.Full_Decl_F_Doc;
      
   end;

      
   function Full_Decl_F_Decl_Annotations
     (Node : Bare_Full_Decl) return Bare_Decl_Annotation_List
   is
      

   begin
         
         return Node.Full_Decl_F_Decl_Annotations;
      
   end;

      
   function Full_Decl_F_Decl
     (Node : Bare_Full_Decl) return Bare_Decl
   is
      

   begin
         
         return Node.Full_Decl_F_Decl;
      
   end;


   







--# property-start FullDecl.has_annotation /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:656
pragma Warnings (Off, "is not referenced");
 function Full_Decl_P_Has_Annotation
  
  (Node : Bare_Full_Decl
      ; Name : Symbol_Type
  )

   return Boolean
is
   Self : Bare_Full_Decl  := Bare_Full_Decl (Node);
      --# bind self Self

   

   --# bind name Name

   Property_Result : Boolean;

      

      Fld : Bare_Decl_Annotation;
Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1328 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1327 '<Not at parser.py:661>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:661
--# expr-start 1326 '<IsNull at parser.py:661>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:661
--# expr-start 1325 '<FieldAccess for get_annotation at parser.py:661>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:661






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




--# property-call-start FullDecl.get_annotation
Fld := Liblktlang.Implementation.Full_Decl_P_Get_Annotation (Node => Self, Name => Name);
--# end
--# expr-done 1325
Is_Null := Fld = null; 
--# expr-done 1326
Not_Val := not (Is_Null); 
--# expr-done 1327
Let_Result := Not_Val; 
--# end
--# expr-done 1328

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Full_Decl_P_Has_Annotation;
--# end

   







--# property-start FullDecl.get_annotation /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:663
pragma Warnings (Off, "is not referenced");
 function Full_Decl_P_Get_Annotation
  
  (Node : Bare_Full_Decl
      ; Name : Symbol_Type
  )

   return Bare_Decl_Annotation
is
   Self : Bare_Full_Decl  := Bare_Full_Decl (Node);
      --# bind self Self

   

   --# bind name Name

   Property_Result : Bare_Decl_Annotation;

      

      Fld : Bare_Decl_Annotation_List;
Item_78 : Bare_Decl_Annotation;
Fld_1 : Bare_Id;
Sym : Symbol_Type;
Is_Equal : Boolean;
Find_Result : Bare_Decl_Annotation;
Let_Result : Bare_Decl_Annotation;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1334 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1333 '<Find at parser.py:668>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:668








   

   --# expr-start 1329 '<FieldAccess for decl_annotations at parser.py:668>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:668






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Full_Decl_F_Decl_Annotations;
--# expr-done 1329

   
      Find_Result := No_Bare_Lkt_Node;
   

   

      if Fld = null then
         
      Find_Result := No_Bare_Lkt_Node;
   
      else
         

      declare
         
         Collection : constant Bare_Decl_Annotation_List := Fld;
      begin
         for Untyped_Item_78 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Item_78 := Untyped_Item_78; 

            
   --# scope-start


               --# bind ann Item_78

            
      --# expr-start 1332 '<Eq at parser.py:669>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:669
--# expr-start 1331 '<GetSymbol at parser.py:669>' Sym /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:669
--# expr-start 1330 '<FieldAccess for name at parser.py:669>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:669






   

      if Item_78 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Item_78.Decl_Annotation_F_Name;
--# expr-done 1330
Sym := Get_Symbol (Fld_1); 
--# expr-done 1331

Is_Equal := Sym = Name; 
--# expr-done 1332
      if Is_Equal then
         Find_Result := Item_78; 
      end if;
   

            
   --# end


               exit when Is_Equal;

         end loop;
      end;
   
      end if;

   
   



--# expr-done 1333
Let_Result := Find_Result; 
--# end
--# expr-done 1334

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Full_Decl_P_Get_Annotation;
--# end

   







--# property-start FullDecl.check_decl /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:672
pragma Warnings (Off, "is not referenced");
 function Full_Decl_P_Check_Decl
  
  (Node : Bare_Full_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Tree_Semantic_Result
is
   Self : Bare_Full_Decl  := Bare_Full_Decl (Node);
     Ent : Internal_Entity_Full_Decl :=
       Internal_Entity_Full_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Tree_Semantic_Result;

      
            procedure Finalizer_Scope_70 with Inline_Always;

      Block_Var : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Fld : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Fld_1 : Internal_Semantic_Result_Array_Access;
Fld_2 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Array_Lit : Internal_Tree_Semantic_Result_Array_Access;
Fld_3 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Fld_4 : Boolean;
If_Result : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Let_Result : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;

            procedure Finalizer_Scope_70 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_70;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 1342 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1335 '<FieldAccess for check_sem_recursive at parser.py:679>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:679






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.check_sem_recursive
Fld := Liblktlang.Implementation.Lkt_Node_P_Check_Sem_Recursive (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1335
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind res Block_Var
--# expr-start 1341 '<If at parser.py:681>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:681



--# expr-start 1336 '<FieldAccess for has_error at parser.py:683>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:683






   




Fld_4 := Block_Var.Has_Error;
--# expr-done 1336
if Fld_4 then
   
   If_Result := Block_Var;
else
   --# expr-start 1340 '<FieldAccess for concat_tree_semantic_results at parser.py:688>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:688






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1339 '<ArrayLiteral at ???>' Array_Lit None

--# expr-start 1338 '<FieldAccess for new_tree_semantic_result at parser.py:690>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:690






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1337 '<FieldAccess for check_legality_visitor at parser.py:691>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:691






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start LktNode.check_legality_visitor
Fld_1 := Liblktlang.Implementation.Lkt_Node_P_Check_Legality_Visitor (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1337
--# property-call-start LktNode.new_tree_semantic_result
Fld_2 := Liblktlang.Implementation.Lkt_Node_P_New_Tree_Semantic_Result (Node => Ent.Node, Elements => Fld_1);
--# end
--# expr-done 1338
Array_Lit := Create_Internal_Tree_Semantic_Result_Array (Internal_Internal_Tree_Semantic_Result_Array'(1 => Block_Var, 2 => Fld_2)); 
--# expr-done 1339
--# property-call-start LktNode.concat_tree_semantic_results
Fld_3 := Liblktlang.Implementation.Lkt_Node_P_Concat_Tree_Semantic_Results (Node => Ent.Node, Res => Array_Lit);
--# end
--# expr-done 1340
   If_Result := Fld_3;
end if;

      Inc_Ref (If_Result);


--# expr-done 1341
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1342

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_70;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_70;




            raise;
      end;



   return Property_Result;
end Full_Decl_P_Check_Decl;
--# end



   


      

   --
   --  Primitives for Bare_Grammar_List_Sep
   --

   



      
      procedure Initialize_Fields_For_Grammar_List_Sep
        (Self : Bare_Grammar_List_Sep
         ; Grammar_List_Sep_F_Token : Bare_Grammar_Expr
         ; Grammar_List_Sep_F_Extra : Bare_Id
        ) is
      begin

            Self.Grammar_List_Sep_F_Token := Grammar_List_Sep_F_Token;
            Self.Grammar_List_Sep_F_Extra := Grammar_List_Sep_F_Extra;
         

      end Initialize_Fields_For_Grammar_List_Sep;

      
   function Grammar_List_Sep_F_Token
     (Node : Bare_Grammar_List_Sep) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Grammar_List_Sep_F_Token;
      
   end;

      
   function Grammar_List_Sep_F_Extra
     (Node : Bare_Grammar_List_Sep) return Bare_Id
   is
      

   begin
         
         return Node.Grammar_List_Sep_F_Extra;
      
   end;




   


      

   --
   --  Primitives for Bare_Import
   --

   



   

   

   

   

   

   

   


      procedure Import_Pre_Env_Actions
        (Self            : Bare_Import;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   
      declare
         Dummy : Internal_Unit;
      begin
         Dummy := Liblktlang.Implementation.Env_Do_1 (Node => 
Self);
      end;
   
   
      end;



      
      procedure Initialize_Fields_For_Import
        (Self : Bare_Import
         ; Import_F_Name : Bare_Module_Ref_Id
        ) is
      begin

            Self.Import_F_Name := Import_F_Name;
         

      end Initialize_Fields_For_Import;

      
   function Import_F_Name
     (Node : Bare_Import) return Bare_Module_Ref_Id
   is
      

   begin
         
         return Node.Import_F_Name;
      
   end;


   







--# property-start Import.referenced_unit /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:626
pragma Warnings (Off, "is not referenced");
 function Import_P_Referenced_Unit
  
  (Node : Bare_Import
  )

   return Internal_Unit
is
   Self : Bare_Import  := Bare_Import (Node);
      --# bind self Self

   


   Property_Result : Internal_Unit;

      
            procedure Finalizer_Scope_66 with Inline_Always;

      Fld : Bare_Module_Ref_Id;
Fld_1 : String_Type;
Fld_2 : Internal_Unit;
Let_Result : Internal_Unit;

            procedure Finalizer_Scope_66 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_66;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 1346 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1345 '<FieldAccess for internal_fetch_referenced_unit at parser.py:632>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:632






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1344 '<FieldAccess for text at parser.py:632>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:632



--# expr-start 1343 '<FieldAccess for name at parser.py:632>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:632






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Import_F_Name;
--# expr-done 1343


   

      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Liblktlang.Implementation.Text (Node => Fld);
--# expr-done 1344
Fld_2 := Liblktlang.Implementation.Extensions.Lkt_Node_P_Internal_Fetch_Referenced_Unit (Node => Self, Name => Fld_1);
--# expr-done 1345
Let_Result := Fld_2; 
--# end
--# expr-done 1346

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_66;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_66;




            raise;
      end;



   return Property_Result;
end Import_P_Referenced_Unit;
--# end

   







--# property-start Import.check_import /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:634
pragma Warnings (Off, "is not referenced");
 function Import_P_Check_Import
  
  (Node : Bare_Import
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result
is
   Self : Bare_Import  := Bare_Import (Node);
     Ent : Internal_Entity_Import :=
       Internal_Entity_Import'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result;

      
            procedure Finalizer_Scope_67 with Inline_Always;

      Str : String_Type;
Fld : Bare_Module_Ref_Id;
Fld_1 : String_Type;
Concat_Result : String_Type;
Fld_2 : Internal_Semantic_Result := No_Semantic_Result;
Fld_3 : Internal_Unit;
Fld_4 : Bare_Lkt_Node;
Is_Null : Boolean;
If_Result : Internal_Semantic_Result := No_Semantic_Result;
Let_Result : Internal_Semantic_Result := No_Semantic_Result;

            procedure Finalizer_Scope_67 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_67;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1355 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1354 '<If at parser.py:636>' If_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:636



--# expr-start 1349 '<IsNull at parser.py:637>' Is_Null /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:637
--# expr-start 1348 '<FieldAccess for root at parser.py:637>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:637



--# expr-start 1347 '<FieldAccess for referenced_unit at parser.py:637>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:637






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Import.referenced_unit
Fld_3 := Liblktlang.Implementation.Import_P_Referenced_Unit (Node => Self);
--# end
--# expr-done 1347


   

      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Fld_3.Ast_Root;
--# expr-done 1348
Is_Null := Fld_4 = null; 
--# expr-done 1349
if Is_Null then
   --# expr-start 1353 '<FieldAccess for error at parser.py:638>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:638






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1350 '<String at parser.py:638>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:638
Str := Create_String ("cannot find "); 
--# expr-done 1350
--# expr-start 1352 '<FieldAccess for text at parser.py:638>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:638



--# expr-start 1351 '<FieldAccess for name at parser.py:638>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:638






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Self.Import_F_Name;
--# expr-done 1351


   

      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Liblktlang.Implementation.Text (Node => Fld);
--# expr-done 1352
Concat_Result := Concat_String (Str, Fld_1); 
--# property-call-start LktNode.error
Fld_2 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Ent.Node, Message => Concat_Result);
--# end
--# expr-done 1353
   If_Result := Fld_2;
else
   
   If_Result := No_Semantic_Result;
end if;

      Inc_Ref (If_Result);


--# expr-done 1354
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1355

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_67;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_67;




            raise;
      end;



   return Property_Result;
end Import_P_Check_Import;
--# end

   







--# property-start Import._env_do_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:643
pragma Warnings (Off, "is not referenced");
 function Env_Do_1
  
  (Node : Bare_Import
  )

   return Internal_Unit
is
   Self : Bare_Import  := Bare_Import (Node);
      --# bind self Self

   


   Property_Result : Internal_Unit;

      

      Fld : Internal_Unit;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1356 '<FieldAccess for referenced_unit at parser.py:643>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:643






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Import.referenced_unit
Fld := Liblktlang.Implementation.Import_P_Referenced_Unit (Node => Self);
--# end
--# expr-done 1356

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Do_1;
--# end



   


      

   --
   --  Primitives for Bare_Langkit_Root
   --

   



   

   

   

   

   

   

   


      procedure Langkit_Root_Pre_Env_Actions
        (Self            : Bare_Langkit_Root;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   
      declare
         Dummy : Boolean;
      begin
         Dummy := Liblktlang.Implementation.Env_Do_0 (Node => 
Self);
      end;
   
   
      end;



      
      procedure Initialize_Fields_For_Langkit_Root
        (Self : Bare_Langkit_Root
         ; Langkit_Root_F_Imports : Bare_Import_List
         ; Langkit_Root_F_Decls : Bare_Full_Decl_List
        ) is
      begin

            Self.Langkit_Root_F_Imports := Langkit_Root_F_Imports;
            Self.Langkit_Root_F_Decls := Langkit_Root_F_Decls;
         

      end Initialize_Fields_For_Langkit_Root;

      
   function Langkit_Root_F_Imports
     (Node : Bare_Langkit_Root) return Bare_Import_List
   is
      

   begin
         
         return Node.Langkit_Root_F_Imports;
      
   end;

      
   function Langkit_Root_F_Decls
     (Node : Bare_Langkit_Root) return Bare_Full_Decl_List
   is
      

   begin
         
         return Node.Langkit_Root_F_Decls;
      
   end;


   







--# property-start LangkitRoot.function_type_helper /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:562
pragma Warnings (Off, "is not referenced");
 function Langkit_Root_P_Function_Type_Helper
  
  (Node : Bare_Langkit_Root
      ; Args_Types : Bare_Type_Decl_Array_Access
      ; Return_Type : Bare_Type_Decl
  )

   return Bare_Function_Type
is
   Self : Bare_Langkit_Root  := Bare_Langkit_Root (Node);
      --# bind self Self

   

   --# bind args_types Args_Types
   --# bind return_type Return_Type

   Property_Result : Bare_Function_Type;

      

      New_Node : Bare_Function_Type;
Let_Result : Bare_Function_Type;


      
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;

      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.

      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it

      --------------------
      -- Create_Mmz_Key --
      --------------------

      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Langkit_Root_Langkit_Root_P_Function_Type_Helper,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Langkit_Root,
                                As_Bare_Langkit_Root => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Type_Decl_Array_Access,
                                      As_Bare_Type_Decl_Array_Access => Args_Types);
                  Inc_Ref (Mmz_K.Items (2).As_Bare_Type_Decl_Array_Access);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Type_Decl,
                                      As_Bare_Type_Decl => Return_Type);
         end return;
      end Create_Mmz_Key;

begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;

      if Self /= null then

         if not Self.Unit.Context.In_Populate_Lexical_Env then

            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup

               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");

               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);

               else
                  Property_Result := Mmz_Val.As_Bare_Function_Type;

                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;

         end if;
      end if;

      begin
         
   --# scope-start

         --# expr-start 1358 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1357 '<New FunctionType at parser.py:565>' New_Node /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:565


Inc_Ref (Args_Types);


if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;


New_Node := new Root_Node_Record
  (Lkt_Function_Type);
Initialize
  (Self => New_Node,
   Kind => Lkt_Function_Type,
   Unit => Self.Unit,

   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,

   Parent => Self,

   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);


      New_Node.Function_Type_Args :=
         Args_Types;
      New_Node.Function_Type_Return_Type :=
         Return_Type;

--# expr-done 1357
Let_Result := New_Node; 
--# end
--# expr-done 1358

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>


               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;


            raise;
      end;

      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then

            Mmz_Val := (Kind => Mmz_Bare_Function_Type,
                        As_Bare_Function_Type => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);

         end if;
      end if;



   return Property_Result;
end Langkit_Root_P_Function_Type_Helper;
--# end

   







--# property-start LangkitRoot.check_semantic /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:569
pragma Warnings (Off, "is not referenced");
 function Langkit_Root_P_Check_Semantic
  
  (Node : Bare_Langkit_Root
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Tree_Semantic_Result
is
   Self : Bare_Langkit_Root  := Bare_Langkit_Root (Node);
     Ent : Internal_Entity_Langkit_Root :=
       Internal_Entity_Langkit_Root'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Tree_Semantic_Result;

      
            procedure Finalizer_Scope_65 with Inline_Always;
            procedure Finalizer_Scope_540 with Inline_Always;
            procedure Finalizer_Scope_541 with Inline_Always;
            procedure Finalizer_Scope_543 with Inline_Always;
            procedure Finalizer_Scope_544 with Inline_Always;

      Block_Var : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Block_Var_1 : Internal_Semantic_Result_Array_Access;
Block_Var_2 : Internal_Semantic_Result_Array_Access;
Block_Var_3 : Boolean;
Fld : Internal_Entity_Full_Decl_List;
Fld_1 : Bare_Full_Decl_List;
Fld_2 : Internal_Entity_Info;
Item_79 : Internal_Entity_Full_Decl;
Bare_Item_79 : Bare_Full_Decl;
Fld_3 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
As_Entity : Internal_Entity_Full_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Full_Decl;
Map_Result : Internal_Tree_Semantic_Result_Array_Access;
Fld_4 : Internal_Entity_Import_List;
Fld_5 : Bare_Import_List;
Fld_6 : Internal_Entity_Info;
Item_80 : Internal_Entity_Import;
Bare_Item_80 : Bare_Import;
Fld_7 : Internal_Semantic_Result := No_Semantic_Result;
As_Entity_1 : Internal_Entity_Import;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Import;
Map_Result_1 : Internal_Semantic_Result_Array_Access;
Fld_8 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Array_Lit : Internal_Tree_Semantic_Result_Array_Access;
Concat_Result : Internal_Tree_Semantic_Result_Array_Access;
Fld_9 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Fld_10 : Internal_Semantic_Result_Array_Access;
Fld_11 : Boolean;
Map_Result_2 : Internal_Semantic_Result_Array_Access;
Fld_12 : String_Type;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result_3 : Internal_Semantic_Result_Array_Access;
Fld_13 : String_Type;
Is_Equal_1 : Boolean;
Quantifier_Result : Boolean;
Len : Integer;
Is_Equal_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;
Let_Result : Internal_Tree_Semantic_Result := No_Tree_Semantic_Result;

            procedure Finalizer_Scope_65 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_65;
            procedure Finalizer_Scope_540 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_540;
            procedure Finalizer_Scope_541 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_541;
            procedure Finalizer_Scope_543 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_543;
            procedure Finalizer_Scope_544 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_544;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      begin
         
   --# scope-start

         --# expr-start 1381 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1367 '<FieldAccess for concat_tree_semantic_results at parser.py:585>' Fld_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:585






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1361 '<Map at parser.py:586>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:586








   

   --# expr-start 1359 '<FieldAccess for decls at parser.py:586>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:586






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Full_Decl_List (Node => Ent.Node.Langkit_Root_F_Decls, Info => Ent.Info);
--# expr-done 1359






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Internal_Tree_Semantic_Result_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Internal_Tree_Semantic_Result_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Full_Decl_List := Fld_1;
      begin
         for Untyped_Item_79 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_79 := Untyped_Item_79; 
                  



Is_Null := Bare_Item_79 = null; 
if Is_Null then
   
   If_Result := No_Entity_Full_Decl;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_79); 
   If_Result := As_Entity;
end if;



                  Item_79 := If_Result; 

            
   --# scope-start


               --# bind d Item_79

            
         
         
      --# expr-start 1360 '<FieldAccess for check_decl at parser.py:586>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:586






   

      if Item_79.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start FullDecl.check_decl
Fld_3 := Liblktlang.Implementation.Full_Decl_P_Check_Decl (Node => Item_79.Node, E_Info => Item_79.Info);
--# end
--# expr-done 1360
      

         declare
            Item_To_Append : constant Internal_Tree_Semantic_Result := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Tree_Semantic_Result_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_540;



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Internal_Tree_Semantic_Result_Array
           (Items_Count => Natural (Internal_Tree_Semantic_Result_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Tree_Semantic_Result_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Tree_Semantic_Result_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Tree_Semantic_Result_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1361
--# expr-start 1366 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 1365 '<FieldAccess for new_tree_semantic_result at parser.py:587>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:587






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1364 '<Map at parser.py:588>' Map_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:588








   

   --# expr-start 1362 '<FieldAccess for imports at parser.py:588>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:588






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_4 := Create_Internal_Entity_Import_List (Node => Ent.Node.Langkit_Root_F_Imports, Info => Ent.Info);
--# expr-done 1362






   




Fld_5 := Fld_4.Node;

   
      declare
         Map_Result_1_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      if Fld_5 = null then
         
      Map_Result_1 := Create_Internal_Semantic_Result_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Import_List := Fld_5;
      begin
         for Untyped_Item_80 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_80 := Untyped_Item_80; 
                  



Is_Null_1 := Bare_Item_80 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Import;
else
   





   




Fld_6 := Fld_4.Info;

As_Entity_1 := (Info => Fld_6, Node => Bare_Item_80); 
   If_Result_1 := As_Entity_1;
end if;



                  Item_80 := If_Result_1; 

            
   --# scope-start


               --# bind i Item_80

            
         
         
      --# expr-start 1363 '<FieldAccess for check_import at parser.py:588>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:588






   

      if Item_80.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Import.check_import
Fld_7 := Liblktlang.Implementation.Import_P_Check_Import (Node => Item_80.Node, E_Info => Item_80.Info);
--# end
--# expr-done 1363
      

         declare
            Item_To_Append : constant Internal_Semantic_Result := Fld_7;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_541;



         end loop;
      end;
   
      end if;

   

         Map_Result_1 := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_1_Vec);

      end;
   



--# expr-done 1364
--# property-call-start LktNode.new_tree_semantic_result
Fld_8 := Liblktlang.Implementation.Lkt_Node_P_New_Tree_Semantic_Result (Node => Ent.Node, Elements => Map_Result_1);
--# end
--# expr-done 1365
Array_Lit := Create_Internal_Tree_Semantic_Result_Array (Internal_Internal_Tree_Semantic_Result_Array'(1 => Fld_8)); 
--# expr-done 1366
Concat_Result := Concat (Map_Result, Array_Lit); 
--# property-call-start LktNode.concat_tree_semantic_results
Fld_9 := Liblktlang.Implementation.Lkt_Node_P_Concat_Tree_Semantic_Results (Node => Ent.Node, Res => Concat_Result);
--# end
--# expr-done 1367
Block_Var := Fld_9; Inc_Ref (Block_Var);
--# bind res Block_Var
--# expr-start 1370 '<Filter at parser.py:603>' Map_Result_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:603








   

   --# expr-start 1368 '<FieldAccess for results at parser.py:603>' Fld_10 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:603






   




Fld_10 := Block_Var.Results;
Inc_Ref (Fld_10);
--# expr-done 1368

   
      declare
         Map_Result_2_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Semantic_Result_Array_Access := Fld_10;
      begin
         for Item_81 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind r Item_81

            
         --# expr-start 1369 '<FieldAccess for has_error at parser.py:603>' Fld_11 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:603






   




Fld_11 := Item_81.Has_Error;
--# expr-done 1369
         if Fld_11 then
            
         
      
      

         declare
            Item_To_Append : constant Internal_Semantic_Result := Item_81;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end



         end loop;
      end;
   

   

         Map_Result_2 := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_2_Vec);

      end;
   



--# expr-done 1370
Block_Var_1 := Map_Result_2; Inc_Ref (Block_Var_1);
--# bind errs Block_Var_1
--# expr-start 1374 '<Filter at parser.py:604>' Map_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:604








   

   

   
      declare
         Map_Result_3_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Internal_Semantic_Result_Array_Access := Block_Var_1;
      begin
         for Item_82 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind r Item_82

            
         --# expr-start 1373 '<Not at parser.py:604>' Not_Val /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:604
--# expr-start 1372 '<IsNull at parser.py:604>' Is_Equal /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:604
--# expr-start 1371 '<FieldAccess for error_message at parser.py:604>' Fld_12 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:604






   




Fld_12 := Item_82.Error_Message;
Inc_Ref (Fld_12);
--# expr-done 1371

Is_Equal := Equivalent (Fld_12, Empty_String); 
--# expr-done 1372
Not_Val := not (Is_Equal); 
--# expr-done 1373
         if Not_Val then
            
         
      
      

         declare
            Item_To_Append : constant Internal_Semantic_Result := Item_82;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   
   
         end if;
   

            
   --# end
      Finalizer_Scope_543;



         end loop;
      end;
   

   

         Map_Result_3 := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_3_Vec);

      end;
   



--# expr-done 1374
Block_Var_2 := Map_Result_3; Inc_Ref (Block_Var_2);
--# bind diags Block_Var_2
--# expr-start 1377 '<AnyQuantifier at parser.py:606>' Quantifier_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:606








   

   

   
      Quantifier_Result := False;
   

   

      

      declare
         
         Collection : constant Internal_Semantic_Result_Array_Access := Block_Var_1;
      begin
         for Item_83 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind r Item_83

            
      --# expr-start 1376 '<IsNull at parser.py:606>' Is_Equal_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:606
--# expr-start 1375 '<FieldAccess for error_message at parser.py:606>' Fld_13 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:606






   




Fld_13 := Item_83.Error_Message;
Inc_Ref (Fld_13);
--# expr-done 1375

Is_Equal_1 := Equivalent (Fld_13, Empty_String); 
--# expr-done 1376
      Quantifier_Result := Is_Equal_1;
   

            
   --# end
      Finalizer_Scope_544;


               exit when Quantifier_Result;

         end loop;
      end;
   

   
   



--# expr-done 1377
Block_Var_3 := Quantifier_Result; 
--# bind has_internal_errors Block_Var_3
--# expr-start 1380 '<If at parser.py:608>' If_Result_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:608







if Block_Var_3 then
   --# expr-start 1379 '<Eq at parser.py:612>' Is_Equal_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:612
--# expr-start 1378 '<length at parser.py:612>' Len /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:612

Len := Length (Block_Var_2); 
--# expr-done 1378

Is_Equal_2 := Len = 0; 
--# expr-done 1379
   If_Result_2 := Is_Equal_2;
else
   
   If_Result_2 := False;
end if;



if If_Result_2 then
   
   If_Result_3 := raise Property_Error with "ERROR: internal errors without diagnostics";
else
   
   If_Result_3 := Block_Var;
end if;

      Inc_Ref (If_Result_3);


--# expr-done 1380
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 1381

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_65;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_65;
                     Finalizer_Scope_540;
                     Finalizer_Scope_541;
                     Finalizer_Scope_543;
                     Finalizer_Scope_544;




            raise;
      end;



   return Property_Result;
end Langkit_Root_P_Check_Semantic;
--# end

   







--# property-start LangkitRoot._env_do_0 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:567
pragma Warnings (Off, "is not referenced");
 function Env_Do_0
  
  (Node : Bare_Langkit_Root
  )

   return Boolean
is
   Self : Bare_Langkit_Root  := Bare_Langkit_Root (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      Fld : Boolean;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1382 '<FieldAccess for fetch_prelude at parser.py:567>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:567






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Liblktlang.Implementation.Extensions.Langkit_Root_P_Fetch_Prelude (Node => Self);
--# expr-done 1382

         Property_Result := Fld;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Do_0;
--# end



   


      

   --
   --  Primitives for Bare_Lexer_Case_Rule
   --

   



      
      procedure Initialize_Fields_For_Lexer_Case_Rule
        (Self : Bare_Lexer_Case_Rule
         ; Lexer_Case_Rule_F_Expr : Bare_Grammar_Expr
         ; Lexer_Case_Rule_F_Alts : Bare_Base_Lexer_Case_Rule_Alt_List
        ) is
      begin

            Self.Lexer_Case_Rule_F_Expr := Lexer_Case_Rule_F_Expr;
            Self.Lexer_Case_Rule_F_Alts := Lexer_Case_Rule_F_Alts;
         

      end Initialize_Fields_For_Lexer_Case_Rule;

      
   function Lexer_Case_Rule_F_Expr
     (Node : Bare_Lexer_Case_Rule) return Bare_Grammar_Expr
   is
      

   begin
         
         return Node.Lexer_Case_Rule_F_Expr;
      
   end;

      
   function Lexer_Case_Rule_F_Alts
     (Node : Bare_Lexer_Case_Rule) return Bare_Base_Lexer_Case_Rule_Alt_List
   is
      

   begin
         
         return Node.Lexer_Case_Rule_F_Alts;
      
   end;


   







--# property-start LexerCaseRule.check_correctness_pre /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1165
pragma Warnings (Off, "is not referenced");
 function Lexer_Case_Rule_P_Check_Correctness_Pre
  
  (Node : Bare_Lexer_Case_Rule
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Lexer_Case_Rule  := Bare_Lexer_Case_Rule (Node);
      --# bind self Self

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_100 with Inline_Always;
            procedure Finalizer_Scope_545 with Inline_Always;

      Node_Children : Bare_Lkt_Node_Array_Access;
New_Struct : Internal_Semantic_Result := No_Semantic_Result;
Map_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_100 is
            begin
                     Dec_Ref (Node_Children);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_100;
            procedure Finalizer_Scope_545 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_545;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1385 '<Map at parser.py:1165>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1165








   

   --# expr-start 1383 '<children at parser.py:1165>' Node_Children /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1165






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Node_Children := Children (Self); 
--# expr-done 1383

   
      declare
         Map_Result_Vec : Internal_Semantic_Result_Vectors.Vector;
      begin
   

   

      

      declare
         
         Collection : constant Bare_Lkt_Node_Array_Access := Node_Children;
      begin
         for Item_84 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind c Item_84

            
         
         
      --# expr-start 1384 '<New SemanticResult at parser.py:1166>' New_Struct /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1166






Inc_Ref (Empty_String);
New_Struct := (Error_Message => Empty_String, Exempt_Analysis => True, Has_Error => False, Node => Item_84, Result_Decl => No_Entity_Decl, Result_Type => No_Entity_Type_Decl); 
--# expr-done 1384
      

         declare
            Item_To_Append : constant Internal_Semantic_Result := New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Semantic_Result_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end
      Finalizer_Scope_545;



         end loop;
      end;
   

   

         Map_Result := Create_Internal_Semantic_Result_Array
           (Items_Count => Natural (Internal_Semantic_Result_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Semantic_Result_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Semantic_Result_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Semantic_Result_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1385

         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_100;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_100;
                     Finalizer_Scope_545;




            raise;
      end;



   return Property_Result;
end Lexer_Case_Rule_P_Check_Correctness_Pre;
--# end



   


      

   --
   --  Primitives for Bare_Lexer_Case_Rule_Send
   --

   



      
      procedure Initialize_Fields_For_Lexer_Case_Rule_Send
        (Self : Bare_Lexer_Case_Rule_Send
         ; Lexer_Case_Rule_Send_F_Sent : Bare_Ref_Id
         ; Lexer_Case_Rule_Send_F_Match_Size : Bare_Num_Lit
        ) is
      begin

            Self.Lexer_Case_Rule_Send_F_Sent := Lexer_Case_Rule_Send_F_Sent;
            Self.Lexer_Case_Rule_Send_F_Match_Size := Lexer_Case_Rule_Send_F_Match_Size;
         

      end Initialize_Fields_For_Lexer_Case_Rule_Send;

      
   function Lexer_Case_Rule_Send_F_Sent
     (Node : Bare_Lexer_Case_Rule_Send) return Bare_Ref_Id
   is
      

   begin
         
         return Node.Lexer_Case_Rule_Send_F_Sent;
      
   end;

      
   function Lexer_Case_Rule_Send_F_Match_Size
     (Node : Bare_Lexer_Case_Rule_Send) return Bare_Num_Lit
   is
      

   begin
         
         return Node.Lexer_Case_Rule_Send_F_Match_Size;
      
   end;




   


      

   --
   --  Primitives for Bare_List_Kind
   --

   







   


      

   --
   --  Primitives for Bare_List_Kind_One
   --

   







   


      

   --
   --  Primitives for Bare_List_Kind_Zero
   --

   







   


      

   --
   --  Primitives for Bare_Lkt_Node_Base_List
   --

   







   


      

   --
   --  Primitives for Bare_Base_Lexer_Case_Rule_Alt_List
   --

   







   


      

   --
   --  Primitives for Bare_Block_String_Line_List
   --

   







   


      

   --
   --  Primitives for Bare_Call_Expr_List
   --

   







   


      

   --
   --  Primitives for Bare_Decl_Annotation_List
   --

   







   


      

   --
   --  Primitives for Bare_Elsif_Branch_List
   --

   







   


      

   --
   --  Primitives for Bare_Enum_Class_Alt_Decl_List
   --

   







   


      

   --
   --  Primitives for Bare_Enum_Class_Case_List
   --

   







   


      

   --
   --  Primitives for Bare_Enum_Lit_Decl_List
   --

   







   


      

   --
   --  Primitives for Bare_Expr_List
   --

   







   


      

   --
   --  Primitives for Bare_Any_Of_List
   --

   







   


      

   --
   --  Primitives for Bare_Full_Decl_List
   --

   







   


      

   --
   --  Primitives for Bare_Decl_Block
   --

   



   

   

   

   

   

   

   


      procedure Decl_Block_Pre_Env_Actions
        (Self            : Bare_Decl_Block;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   
      if Add_To_Env_Only then
         return;
      end if;

      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Liblktlang.Implementation.Env_Trans_Parent_8 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
   
   
            


   

         declare
            Ref_Env_Nodes : Bare_Lkt_Node_Array_Access :=
               Liblktlang.Implementation.Ref_Env_Nodes_9 (Node => 
Self);

            Env : Lexical_Env :=
              Self.Self_Env;
         begin

            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Type_Decl_P_Type_Base_Scope'Access,
               Transitive,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;

   
   
      end;





   







--# property-start DeclBlock.owning_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1716
pragma Warnings (Off, "is not referenced");
 function Decl_Block_P_Owning_Type
  
  (Node : Bare_Decl_Block
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Decl_Block  := Bare_Decl_Block (Node);
     Ent : Internal_Entity_Decl_Block :=
       Internal_Entity_Decl_Block'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_120 with Inline_Always;

      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_120 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_120;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1390 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1389 '<Cast to TypeDecl at parser.py:1723>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1723





--# expr-start 1388 '<Find at parser.py:1721>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1721








   

   --# expr-start 1386 '<FieldAccess for parents at parser.py:1721>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1721






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Fld := Liblktlang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 1386

   
      Find_Result := No_Entity;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_85 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind p Item_85

            
      --# expr-start 1387 '<IsA TypeDecl at parser.py:1722>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1722

Is_A := Item_85.Node /= null 
and then Item_85.Node.Kind in Lkt_Type_Decl; 
--# expr-done 1387
      if Is_A then
         Find_Result := Item_85; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 1388



   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);

   else
         Cast_Result := No_Entity_Type_Decl;
   end if;


--# expr-done 1389
Let_Result := Cast_Result; 
--# end
--# expr-done 1390

         Property_Result := Let_Result;
         
   --# end
      Finalizer_Scope_120;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_120;




            raise;
      end;



   return Property_Result;
end Decl_Block_P_Owning_Type;
--# end

   







--# property-start DeclBlock._env_trans_parent_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1705
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_8
  
  (Node : Bare_Decl_Block
  )

   return Boolean
is
   Self : Bare_Decl_Block  := Bare_Decl_Block (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Trans_Parent_8;
--# end

   







--# property-start DeclBlock._ref_env_nodes_9 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1710
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_9
  
  (Node : Bare_Decl_Block
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Bare_Lkt_Node_Array_Access
is
   Self : Bare_Decl_Block  := Bare_Decl_Block (Node);
     Ent : Internal_Entity_Decl_Block :=
       Internal_Entity_Decl_Block'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Bare_Lkt_Node_Array_Access;

      
            procedure Finalizer_Scope_289 with Inline_Always;
            procedure Finalizer_Scope_547 with Inline_Always;

      Var_Expr_32 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Bare_Type_Decl;
Cast_Result : Bare_Lkt_Node;
Singleton : Bare_Lkt_Node_Array_Access;
Scope_Result : Bare_Lkt_Node_Array_Access;
Result_Var : Bare_Lkt_Node_Array_Access;

            procedure Finalizer_Scope_289 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_289;
            procedure Finalizer_Scope_547 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_547;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         


--# expr-start 1391 '<FieldAccess for owning_type at parser.py:1710>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1710






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start DeclBlock.owning_type
Fld := Liblktlang.Implementation.Decl_Block_P_Owning_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1391
Var_Expr_32 := Fld;





if Var_Expr_32 /= No_Entity_Type_Decl then
   



   --# scope-start


   

--# expr-start 1393 '<Cast to LktNode at parser.py:1710>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1710





--# expr-start 1392 '<FieldAccess for node at parser.py:1710>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:1710






   




Fld_1 := Var_Expr_32.Node;
--# expr-done 1392



   
      Cast_Result := Fld_1;


--# expr-done 1393
                Singleton := Create_Bare_Lkt_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
                
            
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);


   --# end
      Finalizer_Scope_547;


   Result_Var := Scope_Result;
else
   
   Result_Var := No_Bare_Lkt_Node_Array_Type;
end if;

      Inc_Ref (Result_Var);



         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_289;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_289;
                     Finalizer_Scope_547;




            raise;
      end;



   return Property_Result;
end Ref_Env_Nodes_9;
--# end



   


      

   --
   --  Primitives for Bare_Generic_Formal_Decl_List
   --

   







   


      

   --
   --  Primitives for Bare_Fun_Arg_Decl_List
   --

   







   


      

   --
   --  Primitives for Bare_Grammar_Expr_List
   --

   







   


      

   --
   --  Primitives for Bare_Grammar_Expr_List_List
   --

   







   


      

   --
   --  Primitives for Bare_Import_List
   --

   







   


      

   --
   --  Primitives for Bare_Lambda_Arg_Decl_List
   --

   







   


      

   --
   --  Primitives for Bare_Lkt_Node_List
   --

   







   


      

   --
   --  Primitives for Bare_Block_Decl_List
   --

   







   


      

   --
   --  Primitives for Bare_Match_Branch_List
   --

   







   


      

   --
   --  Primitives for Bare_Param_List
   --

   







   


      

   --
   --  Primitives for Bare_Ref_Id_List
   --

   







   


      

   --
   --  Primitives for Bare_Type_Ref_List
   --

   







   


      

   --
   --  Primitives for Bare_Isa_List
   --

   







   


      

   --
   --  Primitives for Bare_Match_Branch
   --

   



   

   

   

   

   

   

   


      procedure Match_Branch_Pre_Env_Actions
        (Self            : Bare_Match_Branch;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
            


   
      if Add_To_Env_Only then
         return;
      end if;

      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Liblktlang.Implementation.Env_Trans_Parent_22 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
   
   
      end;



      
      procedure Initialize_Fields_For_Match_Branch
        (Self : Bare_Match_Branch
         ; Match_Branch_F_Decl : Bare_Match_Val_Decl
         ; Match_Branch_F_Expr : Bare_Expr
        ) is
      begin

            Self.Match_Branch_F_Decl := Match_Branch_F_Decl;
            Self.Match_Branch_F_Expr := Match_Branch_F_Expr;
         

      end Initialize_Fields_For_Match_Branch;

      
   function Match_Branch_F_Decl
     (Node : Bare_Match_Branch) return Bare_Match_Val_Decl
   is
      

   begin
         
         return Node.Match_Branch_F_Decl;
      
   end;

      
   function Match_Branch_F_Expr
     (Node : Bare_Match_Branch) return Bare_Expr
   is
      

   begin
         
         return Node.Match_Branch_F_Expr;
      
   end;


   







--# property-start MatchBranch._env_trans_parent_22 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:3676
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_22
  
  (Node : Bare_Match_Branch
  )

   return Boolean
is
   Self : Bare_Match_Branch  := Bare_Match_Branch (Node);
      --# bind self Self

   


   Property_Result : Boolean;

      

      



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         

         Property_Result := False;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Env_Trans_Parent_22;
--# end



   


      

   --
   --  Primitives for Bare_Op
   --

   







   


      

   --
   --  Primitives for Bare_Op_Amp
   --

   







   


      

   --
   --  Primitives for Bare_Op_And
   --

   







   


      

   --
   --  Primitives for Bare_Op_Div
   --

   







   


      

   --
   --  Primitives for Bare_Op_Eq
   --

   







   


      

   --
   --  Primitives for Bare_Op_Gt
   --

   







   


      

   --
   --  Primitives for Bare_Op_Gte
   --

   







   


      

   --
   --  Primitives for Bare_Op_Lt
   --

   







   


      

   --
   --  Primitives for Bare_Op_Lte
   --

   







   


      

   --
   --  Primitives for Bare_Op_Minus
   --

   







   


      

   --
   --  Primitives for Bare_Op_Mult
   --

   







   


      

   --
   --  Primitives for Bare_Op_Ne
   --

   







   


      

   --
   --  Primitives for Bare_Op_Or
   --

   







   


      

   --
   --  Primitives for Bare_Op_Or_Int
   --

   







   


      

   --
   --  Primitives for Bare_Op_Plus
   --

   







   


      

   --
   --  Primitives for Bare_Param
   --

   



      
      procedure Initialize_Fields_For_Param
        (Self : Bare_Param
         ; Param_F_Name : Bare_Ref_Id
         ; Param_F_Value : Bare_Expr
        ) is
      begin

            Self.Param_F_Name := Param_F_Name;
            Self.Param_F_Value := Param_F_Value;
         

      end Initialize_Fields_For_Param;

      
   function Param_F_Name
     (Node : Bare_Param) return Bare_Ref_Id
   is
      

   begin
         
         return Node.Param_F_Name;
      
   end;

      
   function Param_F_Value
     (Node : Bare_Param) return Bare_Expr
   is
      

   begin
         
         return Node.Param_F_Value;
      
   end;


   







--# property-start Param.call_expr /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2953
pragma Warnings (Off, "is not referenced");
 function Param_P_Call_Expr
  
  (Node : Bare_Param
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Call_Expr
is
   Self : Bare_Param  := Bare_Param (Node);
     Ent : Internal_Entity_Param :=
       Internal_Entity_Param'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Call_Expr;

      
            procedure Finalizer_Scope_227 with Inline_Always;

      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Call_Expr;

            procedure Finalizer_Scope_227 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_227;


begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1397 '<Cast to CallExpr at parser.py:2954>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2954





--# expr-start 1396 '<Find at parser.py:2954>' Find_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2954








   

   --# expr-start 1394 '<FieldAccess for parents at parser.py:2954>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2954






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;




Fld := Liblktlang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 1394

   
      Find_Result := No_Entity;
   

   

      

      declare
         
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_86 of
               Collection.Items
         loop

            
   --# scope-start


               --# bind p Item_86

            
      --# expr-start 1395 '<IsA CallExpr at parser.py:2954>' Is_A /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2954

Is_A := Item_86.Node /= null 
and then Item_86.Node.Kind in Lkt_Call_Expr_Range; 
--# expr-done 1395
      if Is_A then
         Find_Result := Item_86; 
      end if;
   

            
   --# end


               exit when Is_A;

         end loop;
      end;
   

   
   



--# expr-done 1396



   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Lkt_Call_Expr_Range
   then
      
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Find_Result.Node,
         Info => Find_Result.Info);

   else
         Cast_Result := No_Entity_Call_Expr;
   end if;


--# expr-done 1397

         Property_Result := Cast_Result;
         
   --# end
      Finalizer_Scope_227;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_227;




            raise;
      end;



   return Property_Result;
end Param_P_Call_Expr;
--# end



   


      

   --
   --  Primitives for Bare_Type_Ref
   --

   





   







--# property-start '[dispatcher]TypeRef.designated_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Ref_P_Designated_Type
  
  (Node : Bare_Type_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Type_Ref  := Bare_Type_Ref (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

            Populate_Lexical_Env_For_Unit (Self);
      end if;


      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;

      case Lkt_Type_Ref (Self.Kind) is
               when Lkt_Default_List_Type_Ref =>
                  --# property-call-start DefaultListTypeRef.designated_type
                  Property_Result := Default_List_Type_Ref_P_Designated_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Function_Type_Ref =>
                  --# property-call-start FunctionTypeRef.designated_type
                  Property_Result := Function_Type_Ref_P_Designated_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Generic_Type_Ref =>
                  --# property-call-start GenericTypeRef.designated_type
                  Property_Result := Generic_Type_Ref_P_Designated_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Lkt_Simple_Type_Ref =>
                  --# property-call-start SimpleTypeRef.designated_type
                  Property_Result := Simple_Type_Ref_P_Designated_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;




   return Property_Result;
end Dispatcher_Type_Ref_P_Designated_Type;
--# end



   


      

   --
   --  Primitives for Bare_Default_List_Type_Ref
   --

   





   







--# property-start DefaultListTypeRef.designated_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2734
pragma Warnings (Off, "is not referenced");
 function Default_List_Type_Ref_P_Designated_Type
  
  (Node : Bare_Default_List_Type_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Default_List_Type_Ref  := Bare_Default_List_Type_Ref (Node);
      --# bind self Self

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);





      begin
         
   --# scope-start

         --# expr-start 1398 '<Block at ???>' Let_Result None
--# scope-start

Let_Result := raise Property_Error; 
--# end
--# expr-done 1398

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Default_List_Type_Ref_P_Designated_Type;
--# end



   


      

   --
   --  Primitives for Bare_Function_Type_Ref
   --

   



      
      procedure Initialize_Fields_For_Function_Type_Ref
        (Self : Bare_Function_Type_Ref
         ; Function_Type_Ref_F_Args_Types : Bare_Type_Ref_List
         ; Function_Type_Ref_F_Return_Type : Bare_Type_Ref
        ) is
      begin

            Self.Function_Type_Ref_F_Args_Types := Function_Type_Ref_F_Args_Types;
            Self.Function_Type_Ref_F_Return_Type := Function_Type_Ref_F_Return_Type;
         

      end Initialize_Fields_For_Function_Type_Ref;

      
   function Function_Type_Ref_F_Args_Types
     (Node : Bare_Function_Type_Ref) return Bare_Type_Ref_List
   is
      

   begin
         
         return Node.Function_Type_Ref_F_Args_Types;
      
   end;

      
   function Function_Type_Ref_F_Return_Type
     (Node : Bare_Function_Type_Ref) return Bare_Type_Ref
   is
      

   begin
         
         return Node.Function_Type_Ref_F_Return_Type;
      
   end;


   







--# property-start FunctionTypeRef.designated_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2771
pragma Warnings (Off, "is not referenced");
 function Function_Type_Ref_P_Designated_Type
  
  (Node : Bare_Function_Type_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Function_Type_Ref  := Bare_Function_Type_Ref (Node);
     Ent : Internal_Entity_Function_Type_Ref :=
       Internal_Entity_Function_Type_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_216 with Inline_Always;

      Fld : Internal_Entity_Type_Ref_List;
Fld_1 : Bare_Type_Ref_List;
Fld_2 : Internal_Entity_Info;
Item_87 : Internal_Entity_Type_Ref;
Bare_Item_87 : Bare_Type_Ref;
Fld_3 : Internal_Entity_Type_Decl;
Fld_4 : Bare_Type_Decl;
As_Entity : Internal_Entity_Type_Ref;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Ref;
Map_Result : Bare_Type_Decl_Array_Access;
Fld_5 : Internal_Entity_Type_Ref;
Fld_6 : Internal_Entity_Type_Decl;
Fld_7 : Bare_Type_Decl;
Fld_8 : Bare_Function_Type;
As_Entity_1 : Internal_Entity_Function_Type;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Function_Type;
Let_Result : Internal_Entity_Function_Type;
Cast_Result : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_216 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_216;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         




--# expr-start 1408 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1407 '<as_bare_entity at parser.py:2779>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2779



--# expr-start 1406 '<FieldAccess for function_type at parser.py:2773>' Fld_8 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2773






   

      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1402 '<Map at parser.py:2774>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2774








   

   --# expr-start 1399 '<FieldAccess for args_types at parser.py:2774>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2774






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Type_Ref_List (Node => Ent.Node.Function_Type_Ref_F_Args_Types, Info => Ent.Info);
--# expr-done 1399






   




Fld_1 := Fld.Node;

   
      declare
         Map_Result_Vec : Bare_Type_Decl_Vectors.Vector;
      begin
   

   

      if Fld_1 = null then
         
      Map_Result := Create_Bare_Type_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Type_Ref_List := Fld_1;
      begin
         for Untyped_Item_87 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_87 := Untyped_Item_87; 
                  



Is_Null := Bare_Item_87 = null; 
if Is_Null then
   
   If_Result := No_Entity_Type_Ref;
else
   





   




Fld_2 := Fld.Info;

As_Entity := (Info => Fld_2, Node => Bare_Item_87); 
   If_Result := As_Entity;
end if;



                  Item_87 := If_Result; 

            
   --# scope-start


               --# bind a Item_87

            
         
         
      --# expr-start 1401 '<FieldAccess for node at parser.py:2776>' Fld_4 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2776



--# expr-start 1400 '<FieldAccess for designated_type at parser.py:2776>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2776






   

      if Item_87.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_3 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Item_87.Node, E_Info => Item_87.Info);
--# end
--# expr-done 1400


   




Fld_4 := Fld_3.Node;
--# expr-done 1401
      

         declare
            Item_To_Append : constant Bare_Type_Decl := Fld_4;
         begin
            Bare_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Bare_Type_Decl_Array
           (Items_Count => Natural (Bare_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1402
--# expr-start 1405 '<FieldAccess for node at parser.py:2778>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2778



--# expr-start 1404 '<FieldAccess for designated_type at parser.py:2778>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2778



--# expr-start 1403 '<FieldAccess for return_type at parser.py:2778>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2778






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_5 := Create_Internal_Entity_Type_Ref (Node => Ent.Node.Function_Type_Ref_F_Return_Type, Info => Ent.Info);
--# expr-done 1403


   

      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_6 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 1404


   




Fld_7 := Fld_6.Node;
--# expr-done 1405
--# property-call-start LktNode.function_type
Fld_8 := Liblktlang.Implementation.Lkt_Node_P_Function_Type (Node => Self, Args_Types => Map_Result, Return_Type => Fld_7);
--# end
--# expr-done 1406
Is_Null_1 := Fld_8 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Function_Type;
else
   

As_Entity_1 := (Info => No_Entity_Info, Node => Fld_8); 
   If_Result_1 := As_Entity_1;
end if;



--# expr-done 1407
Let_Result := If_Result_1; 
--# end
--# expr-done 1408



   
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Let_Result.Node,
         Info => Let_Result.Info);



         Property_Result := Cast_Result;
         
   --# end
      Finalizer_Scope_216;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_216;




            raise;
      end;



   return Property_Result;
end Function_Type_Ref_P_Designated_Type;
--# end



   


      

   --
   --  Primitives for Bare_Generic_Type_Ref
   --

   



      
      procedure Initialize_Fields_For_Generic_Type_Ref
        (Self : Bare_Generic_Type_Ref
         ; Generic_Type_Ref_F_Type_Name : Bare_Expr
         ; Generic_Type_Ref_F_Params : Bare_Type_Ref_List
        ) is
      begin

            Self.Generic_Type_Ref_F_Type_Name := Generic_Type_Ref_F_Type_Name;
            Self.Generic_Type_Ref_F_Params := Generic_Type_Ref_F_Params;
         

      end Initialize_Fields_For_Generic_Type_Ref;

      
   function Generic_Type_Ref_F_Type_Name
     (Node : Bare_Generic_Type_Ref) return Bare_Expr
   is
      

   begin
         
         return Node.Generic_Type_Ref_F_Type_Name;
      
   end;

      
   function Generic_Type_Ref_F_Params
     (Node : Bare_Generic_Type_Ref) return Bare_Type_Ref_List
   is
      

   begin
         
         return Node.Generic_Type_Ref_F_Params;
      
   end;


   







--# property-start GenericTypeRef.designated_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2746
pragma Warnings (Off, "is not referenced");
 function Generic_Type_Ref_P_Designated_Type
  
  (Node : Bare_Generic_Type_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Generic_Type_Ref  := Bare_Generic_Type_Ref (Node);
     Ent : Internal_Entity_Generic_Type_Ref :=
       Internal_Entity_Generic_Type_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      
            procedure Finalizer_Scope_214 with Inline_Always;

      Block_Var : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Generic_Decl;
Fld_2 : Internal_Entity_Type_Ref_List;
Fld_3 : Bare_Type_Ref_List;
Fld_4 : Internal_Entity_Info;
Item_88 : Internal_Entity_Type_Ref;
Bare_Item_88 : Bare_Type_Ref;
Fld_5 : Internal_Entity_Type_Decl;
Fld_6 : Bare_Type_Decl;
As_Entity : Internal_Entity_Type_Ref;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Ref;
Map_Result : Bare_Type_Decl_Array_Access;
Fld_7 : Bare_Instantiated_Generic_Type;
As_Entity_1 : Internal_Entity_Instantiated_Generic_Type;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Instantiated_Generic_Type;
Let_Result : Internal_Entity_Instantiated_Generic_Type;
Cast_Result_1 : Internal_Entity_Type_Decl;

            procedure Finalizer_Scope_214 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_214;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         




--# expr-start 1418 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1411 '<Cast to GenericDecl at parser.py:2749>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2749





--# expr-start 1410 '<FieldAccess for check_referenced_decl at parser.py:2749>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2749



--# expr-start 1409 '<FieldAccess for type_name at parser.py:2749>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2749






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Generic_Type_Ref_F_Type_Name, Info => Ent.Info);
--# expr-done 1409


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_referenced_decl
Fld_1 := Liblktlang.Implementation.Expr_P_Check_Referenced_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1410



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Generic_Decl_Range
   then
      
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 1411
Block_Var := Cast_Result; 
--# bind generic_decl Block_Var
--# expr-start 1417 '<as_bare_entity at parser.py:2755>' If_Result_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2755



--# expr-start 1416 '<FieldAccess for get_instantiated_type at parser.py:2751>' Fld_7 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2751






   

      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1415 '<Map at parser.py:2752>' Map_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2752








   

   --# expr-start 1412 '<FieldAccess for params at parser.py:2752>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2752






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_2 := Create_Internal_Entity_Type_Ref_List (Node => Ent.Node.Generic_Type_Ref_F_Params, Info => Ent.Info);
--# expr-done 1412






   




Fld_3 := Fld_2.Node;

   
      declare
         Map_Result_Vec : Bare_Type_Decl_Vectors.Vector;
      begin
   

   

      if Fld_3 = null then
         
      Map_Result := Create_Bare_Type_Decl_Array (0);
   
      else
         

      declare
         
         Collection : constant Bare_Type_Ref_List := Fld_3;
      begin
         for Untyped_Item_88 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  
                  Bare_Item_88 := Untyped_Item_88; 
                  



Is_Null := Bare_Item_88 = null; 
if Is_Null then
   
   If_Result := No_Entity_Type_Ref;
else
   





   




Fld_4 := Fld_2.Info;

As_Entity := (Info => Fld_4, Node => Bare_Item_88); 
   If_Result := As_Entity;
end if;



                  Item_88 := If_Result; 

            
   --# scope-start


               --# bind p Item_88

            
         
         
      --# expr-start 1414 '<FieldAccess for node at parser.py:2753>' Fld_6 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2753



--# expr-start 1413 '<FieldAccess for designated_type at parser.py:2753>' Fld_5 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2753






   

      if Item_88.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]TypeRef.designated_type'
Fld_5 := Liblktlang.Implementation.Dispatcher_Type_Ref_P_Designated_Type (Node => Item_88.Node, E_Info => Item_88.Info);
--# end
--# expr-done 1413


   




Fld_6 := Fld_5.Node;
--# expr-done 1414
      

         declare
            Item_To_Append : constant Bare_Type_Decl := Fld_6;
         begin
            Bare_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   
   
   

            
   --# end



         end loop;
      end;
   
      end if;

   

         Map_Result := Create_Bare_Type_Decl_Array
           (Items_Count => Natural (Bare_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Type_Decl_Vectors.Destroy (Map_Result_Vec);

      end;
   



--# expr-done 1415
--# property-call-start GenericDecl.get_instantiated_type
Fld_7 := Liblktlang.Implementation.Generic_Decl_P_Get_Instantiated_Type (Node => Block_Var.Node, Actuals => Map_Result);
--# end
--# expr-done 1416
Is_Null_1 := Fld_7 = null; 
if Is_Null_1 then
   
   If_Result_1 := No_Entity_Instantiated_Generic_Type;
else
   

As_Entity_1 := (Info => No_Entity_Info, Node => Fld_7); 
   If_Result_1 := As_Entity_1;
end if;



--# expr-done 1417
Let_Result := If_Result_1; 
--# end
--# expr-done 1418



   
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Let_Result.Node,
         Info => Let_Result.Info);



         Property_Result := Cast_Result_1;
         
   --# end
      Finalizer_Scope_214;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_214;




            raise;
      end;



   return Property_Result;
end Generic_Type_Ref_P_Designated_Type;
--# end

   







--# property-start GenericTypeRef.check_correctness_pre /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2757
pragma Warnings (Off, "is not referenced");
 function Generic_Type_Ref_P_Check_Correctness_Pre
  
  (Node : Bare_Generic_Type_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Generic_Type_Ref  := Bare_Generic_Type_Ref (Node);
     Ent : Internal_Entity_Generic_Type_Ref :=
       Internal_Entity_Generic_Type_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_215 with Inline_Always;

      Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Ref_List;
Fld_2 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_215 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_215;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1422 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1421 '<FieldAccess for check_generic_inst_correctness_pre at parser.py:2759>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2759






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1419 '<FieldAccess for type_name at parser.py:2760>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2760






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Generic_Type_Ref_F_Type_Name, Info => Ent.Info);
--# expr-done 1419
--# expr-start 1420 '<FieldAccess for params at parser.py:2760>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2760






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld_1 := Create_Internal_Entity_Type_Ref_List (Node => Ent.Node.Generic_Type_Ref_F_Params, Info => Ent.Info);
--# expr-done 1420
--# property-call-start LktNode.check_generic_inst_correctness_pre
Fld_2 := Liblktlang.Implementation.Lkt_Node_P_Check_Generic_Inst_Correctness_Pre (Node => Ent.Node, Prefix => Fld, Params => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 1421
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1422

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_215;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_215;




            raise;
      end;



   return Property_Result;
end Generic_Type_Ref_P_Check_Correctness_Pre;
--# end



   


      

   --
   --  Primitives for Bare_Simple_Type_Ref
   --

   



      
      procedure Initialize_Fields_For_Simple_Type_Ref
        (Self : Bare_Simple_Type_Ref
         ; Simple_Type_Ref_F_Type_Name : Bare_Expr
        ) is
      begin

            Self.Simple_Type_Ref_F_Type_Name := Simple_Type_Ref_F_Type_Name;
         

      end Initialize_Fields_For_Simple_Type_Ref;

      
   function Simple_Type_Ref_F_Type_Name
     (Node : Bare_Simple_Type_Ref) return Bare_Expr
   is
      

   begin
         
         return Node.Simple_Type_Ref_F_Type_Name;
      
   end;


   







--# property-start SimpleTypeRef.designated_type /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2712
pragma Warnings (Off, "is not referenced");
 function Simple_Type_Ref_P_Designated_Type
  
  (Node : Bare_Simple_Type_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Entity_Type_Decl
is
   Self : Bare_Simple_Type_Ref  := Bare_Simple_Type_Ref (Node);
     Ent : Internal_Entity_Simple_Type_Ref :=
       Internal_Entity_Simple_Type_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Entity_Type_Decl;

      

      Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;



begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1426 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1425 '<Cast to TypeDecl at parser.py:2714>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2714





--# expr-start 1424 '<FieldAccess for check_referenced_decl at parser.py:2714>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2714



--# expr-start 1423 '<FieldAccess for type_name at parser.py:2714>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2714






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Simple_Type_Ref_F_Type_Name, Info => Ent.Info);
--# expr-done 1423


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start Expr.check_referenced_decl
Fld_1 := Liblktlang.Implementation.Expr_P_Check_Referenced_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1424



   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);

   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;


--# expr-done 1425
Let_Result := Cast_Result; 
--# end
--# expr-done 1426

         Property_Result := Let_Result;
         
   --# end


      exception
         when Exc : Property_Error =>




            raise;
      end;



   return Property_Result;
end Simple_Type_Ref_P_Designated_Type;
--# end

   







--# property-start SimpleTypeRef.check_correctness_pre /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2716
pragma Warnings (Off, "is not referenced");
 function Simple_Type_Ref_P_Check_Correctness_Pre
  
  (Node : Bare_Simple_Type_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )

   return Internal_Semantic_Result_Array_Access
is
   Self : Bare_Simple_Type_Ref  := Bare_Simple_Type_Ref (Node);
     Ent : Internal_Entity_Simple_Type_Ref :=
       Internal_Entity_Simple_Type_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent

   


   Property_Result : Internal_Semantic_Result_Array_Access;

      
            procedure Finalizer_Scope_212 with Inline_Always;
            procedure Finalizer_Scope_550 with Inline_Always;

      Block_Var : Internal_Semantic_Result := No_Semantic_Result;
Var_Expr : Internal_Entity_Decl;
Var_Expr_1 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Semantic_Result := No_Semantic_Result;
Fld_2 : Internal_Entity_Decl;
Cast_Expr : Internal_Entity_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Semantic_Result_Array_Access;
Str : String_Type;
Fld_3 : Internal_Semantic_Result := No_Semantic_Result;
Array_Lit : Internal_Semantic_Result_Array_Access;
Result_Var : Internal_Semantic_Result_Array_Access;
Scope_Result_1 : Internal_Semantic_Result_Array_Access;
Result_Var_1 : Internal_Semantic_Result_Array_Access;
Let_Result : Internal_Semantic_Result_Array_Access;

            procedure Finalizer_Scope_212 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_212;
            procedure Finalizer_Scope_550 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_550;


begin
   --# property-body-start

   pragma Assert (Self = Node);



      if Self /= null then
         Reset_Caches (Self.Unit);

      end if;


      begin
         
   --# scope-start

         --# expr-start 1434 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1428 '<FieldAccess for referenced_decl at parser.py:2718>' Fld_1 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2718



--# expr-start 1427 '<FieldAccess for type_name at parser.py:2718>' Fld /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2718






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Simple_Type_Ref_F_Type_Name, Info => Ent.Info);
--# expr-done 1427


   

      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# property-call-start '[dispatcher]Expr.referenced_decl'
Fld_1 := Liblktlang.Implementation.Dispatcher_Expr_P_Referenced_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1428
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind d Block_Var



--# expr-start 1429 '<FieldAccess for result_decl at parser.py:2719>' Fld_2 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2719






   




Fld_2 := Block_Var.Result_Decl;
--# expr-done 1429
Var_Expr := Fld_2;





if Var_Expr /= No_Entity_Decl then
   



   --# scope-start


   --# bind d Var_Expr




--# expr-start 1430 '<Cast to TypeDecl at parser.py:2720>' Cast_Result /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2720






Cast_Expr := Var_Expr; 



   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Lkt_Type_Decl
   then
      
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);

   else
         Cast_Result := No_Entity_Type_Decl;
   end if;


--# expr-done 1430
Var_Expr_1 := Cast_Result;





if Var_Expr_1 /= No_Entity_Type_Decl then
   



   --# scope-start


   --# bind _ Var_Expr_1


Scope_Result := No_Internal_Semantic_Result_Array_Type;
   Inc_Ref (Scope_Result);


   --# end


   Result_Var := Scope_Result;
else
   --# expr-start 1433 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 1432 '<FieldAccess for error at parser.py:2722>' Fld_3 /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2722






   

      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;



--# expr-start 1431 '<String at parser.py:2722>' Str /builddir/build/BUILD/liblktlang-25.0.0-build/langkit-25.0.0/contrib/lkt/language/parser.py:2722
Str := Create_String ("Invalid type reference"); 
--# expr-done 1431
--# property-call-start LktNode.error
Fld_3 := Liblktlang.Implementation.Lkt_Node_P_Error (Node => Ent.Node, Message => Str);
--# end
--# expr-done 1432
Array_Lit := Create_Internal_Semantic_Result_Array (Internal_Internal_Semantic_Result_Array'(1 => Fld_3)); 
--# expr-done 1433
   Result_Var := Array_Lit;
end if;

      Inc_Ref (Result_Var);


Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);


   --# end
      Finalizer_Scope_550;


   Result_Var_1 := Scope_Result_1;
else
   
   Result_Var_1 := No_Internal_Semantic_Result_Array_Type;
end if;

      Inc_Ref (Result_Var_1);


Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1434

         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
         
   --# end
      Finalizer_Scope_212;


      exception
         when Exc : Property_Error =>
                     Finalizer_Scope_212;
                     Finalizer_Scope_550;




            raise;
      end;



   return Property_Result;
end Simple_Type_Ref_P_Check_Correctness_Pre;
--# end



   


      

   --
   --  Primitives for Bare_Var_Bind
   --

   



      
      procedure Initialize_Fields_For_Var_Bind
        (Self : Bare_Var_Bind
         ; Var_Bind_F_Name : Bare_Ref_Id
         ; Var_Bind_F_Expr : Bare_Expr
        ) is
      begin

            Self.Var_Bind_F_Name := Var_Bind_F_Name;
            Self.Var_Bind_F_Expr := Var_Bind_F_Expr;
         

      end Initialize_Fields_For_Var_Bind;

      
   function Var_Bind_F_Name
     (Node : Bare_Var_Bind) return Bare_Ref_Id
   is
      

   begin
         
         return Node.Var_Bind_F_Name;
      
   end;

      
   function Var_Bind_F_Expr
     (Node : Bare_Var_Bind) return Bare_Expr
   is
      

   begin
         
         return Node.Var_Bind_F_Expr;
      
   end;




   



   ----------------------------
   -- Destroy_Synthetic_Node --
   ----------------------------

   procedure Destroy_Synthetic_Node (Node : in out Bare_Lkt_Node) is
      procedure Free is new Ada.Unchecked_Deallocation
        (Root_Node_Record, Bare_Lkt_Node);
   begin
      --  Don't call Node.Destroy, as Node's children may be gone already: they
      --  have their own destructor and there is no specified order for the
      --  call of these destructors.
      Free_User_Fields (Node);
      Free (Node);
   end Destroy_Synthetic_Node;

   -----------
   -- Image --
   -----------

   function Image (Value : Boolean) return String
   is (if Value then "True" else "False");

      -----------------
      -- Trace_Image --
      -----------------

      function Trace_Image
        (Node       : Bare_Lkt_Node;
         Decoration : Boolean := True) return String is
      begin
         if Node = null then
            return "None";
         else
            declare
               Result : constant String :=
                 (Kind_Name (Node) & " "
                  & Basename (Node.Unit) & ":"
                  & Image (Sloc_Range (Node)));
            begin
               return (if Decoration then "<" & Result & ">" else Result);
            end;
         end if;
      end Trace_Image;

   Kind_Names : array (Lkt_Node_Kind_Type) of Unbounded_String :=
     (Lkt_Lexer_Case_Rule_Cond_Alt => To_Unbounded_String ("LexerCaseRuleCondAlt"), 
Lkt_Lexer_Case_Rule_Default_Alt => To_Unbounded_String ("LexerCaseRuleDefaultAlt"), 
Lkt_Block_String_Line => To_Unbounded_String ("BlockStringLine"), 
Lkt_Class_Qualifier_Absent => To_Unbounded_String ("ClassQualifierAbsent"), 
Lkt_Class_Qualifier_Present => To_Unbounded_String ("ClassQualifierPresent"), 
Lkt_Grammar_Rule_Decl => To_Unbounded_String ("GrammarRuleDecl"), 
Lkt_Synthetic_Lexer_Decl => To_Unbounded_String ("SyntheticLexerDecl"), 
Lkt_Node_Decl => To_Unbounded_String ("NodeDecl"), 
Lkt_Self_Decl => To_Unbounded_String ("SelfDecl"), 
Lkt_Enum_Lit_Decl => To_Unbounded_String ("EnumLitDecl"), 
Lkt_Field_Decl => To_Unbounded_String ("FieldDecl"), 
Lkt_Fun_Arg_Decl => To_Unbounded_String ("FunArgDecl"), 
Lkt_Lambda_Arg_Decl => To_Unbounded_String ("LambdaArgDecl"), 
Lkt_Dyn_Var_Decl => To_Unbounded_String ("DynVarDecl"), 
Lkt_Match_Val_Decl => To_Unbounded_String ("MatchValDecl"), 
Lkt_Val_Decl => To_Unbounded_String ("ValDecl"), 
Lkt_Fun_Decl => To_Unbounded_String ("FunDecl"), 
Lkt_Env_Spec_Decl => To_Unbounded_String ("EnvSpecDecl"), 
Lkt_Generic_Decl => To_Unbounded_String ("GenericDecl"), 
Lkt_Grammar_Decl => To_Unbounded_String ("GrammarDecl"), 
Lkt_Lexer_Decl => To_Unbounded_String ("LexerDecl"), 
Lkt_Lexer_Family_Decl => To_Unbounded_String ("LexerFamilyDecl"), 
Lkt_Enum_Class_Alt_Decl => To_Unbounded_String ("EnumClassAltDecl"), 
Lkt_Function_Type => To_Unbounded_String ("FunctionType"), 
Lkt_Generic_Formal_Type_Decl => To_Unbounded_String ("GenericFormalTypeDecl"), 
Lkt_Instantiated_Generic_Type => To_Unbounded_String ("InstantiatedGenericType"), 
Lkt_Class_Decl => To_Unbounded_String ("ClassDecl"), 
Lkt_Enum_Class_Decl => To_Unbounded_String ("EnumClassDecl"), 
Lkt_Enum_Type_Decl => To_Unbounded_String ("EnumTypeDecl"), 
Lkt_Struct_Decl => To_Unbounded_String ("StructDecl"), 
Lkt_Trait_Decl => To_Unbounded_String ("TraitDecl"), 
Lkt_Decl_Annotation => To_Unbounded_String ("DeclAnnotation"), 
Lkt_Decl_Annotation_Params => To_Unbounded_String ("DeclAnnotationParams"), 
Lkt_Elsif_Branch => To_Unbounded_String ("ElsifBranch"), 
Lkt_Enum_Class_Case => To_Unbounded_String ("EnumClassCase"), 
Lkt_Excludes_Null_Absent => To_Unbounded_String ("ExcludesNullAbsent"), 
Lkt_Excludes_Null_Present => To_Unbounded_String ("ExcludesNullPresent"), 
Lkt_Any_Of => To_Unbounded_String ("AnyOf"), 
Lkt_Array_Literal => To_Unbounded_String ("ArrayLiteral"), 
Lkt_Dot_Expr => To_Unbounded_String ("DotExpr"), 
Lkt_Null_Cond_Dotted_Name => To_Unbounded_String ("NullCondDottedName"), 
Lkt_Bin_Op => To_Unbounded_String ("BinOp"), 
Lkt_Block_Expr => To_Unbounded_String ("BlockExpr"), 
Lkt_Call_Expr => To_Unbounded_String ("CallExpr"), 
Lkt_Cast_Expr => To_Unbounded_String ("CastExpr"), 
Lkt_Error_On_Null => To_Unbounded_String ("ErrorOnNull"), 
Lkt_Generic_Instantiation => To_Unbounded_String ("GenericInstantiation"), 
Lkt_Grammar_Cut => To_Unbounded_String ("GrammarCut"), 
Lkt_Grammar_Discard => To_Unbounded_String ("GrammarDiscard"), 
Lkt_Grammar_Dont_Skip => To_Unbounded_String ("GrammarDontSkip"), 
Lkt_Grammar_List => To_Unbounded_String ("GrammarList"), 
Lkt_Grammar_Null => To_Unbounded_String ("GrammarNull"), 
Lkt_Grammar_Opt => To_Unbounded_String ("GrammarOpt"), 
Lkt_Grammar_Opt_Error => To_Unbounded_String ("GrammarOptError"), 
Lkt_Grammar_Opt_Error_Group => To_Unbounded_String ("GrammarOptErrorGroup"), 
Lkt_Grammar_Opt_Group => To_Unbounded_String ("GrammarOptGroup"), 
Lkt_Grammar_Or_Expr => To_Unbounded_String ("GrammarOrExpr"), 
Lkt_Grammar_Pick => To_Unbounded_String ("GrammarPick"), 
Lkt_Grammar_Implicit_Pick => To_Unbounded_String ("GrammarImplicitPick"), 
Lkt_Grammar_Predicate => To_Unbounded_String ("GrammarPredicate"), 
Lkt_Grammar_Rule_Ref => To_Unbounded_String ("GrammarRuleRef"), 
Lkt_Grammar_Skip => To_Unbounded_String ("GrammarSkip"), 
Lkt_Grammar_Stop_Cut => To_Unbounded_String ("GrammarStopCut"), 
Lkt_Parse_Node_Expr => To_Unbounded_String ("ParseNodeExpr"), 
Lkt_Token_Lit => To_Unbounded_String ("TokenLit"), 
Lkt_Token_No_Case_Lit => To_Unbounded_String ("TokenNoCaseLit"), 
Lkt_Token_Pattern_Lit => To_Unbounded_String ("TokenPatternLit"), 
Lkt_Token_Ref => To_Unbounded_String ("TokenRef"), 
Lkt_Id => To_Unbounded_String ("Id"), 
Lkt_Def_Id => To_Unbounded_String ("DefId"), 
Lkt_Module_Ref_Id => To_Unbounded_String ("ModuleRefId"), 
Lkt_Ref_Id => To_Unbounded_String ("RefId"), 
Lkt_If_Expr => To_Unbounded_String ("IfExpr"), 
Lkt_Isa => To_Unbounded_String ("Isa"), 
Lkt_Keep_Expr => To_Unbounded_String ("KeepExpr"), 
Lkt_Lambda_Expr => To_Unbounded_String ("LambdaExpr"), 
Lkt_Big_Num_Lit => To_Unbounded_String ("BigNumLit"), 
Lkt_Char_Lit => To_Unbounded_String ("CharLit"), 
Lkt_Null_Lit => To_Unbounded_String ("NullLit"), 
Lkt_Num_Lit => To_Unbounded_String ("NumLit"), 
Lkt_Block_String_Lit => To_Unbounded_String ("BlockStringLit"), 
Lkt_Single_Line_String_Lit => To_Unbounded_String ("SingleLineStringLit"), 
Lkt_Pattern_Single_Line_String_Lit => To_Unbounded_String ("PatternSingleLineStringLit"), 
Lkt_Logic_Expr => To_Unbounded_String ("LogicExpr"), 
Lkt_Match_Expr => To_Unbounded_String ("MatchExpr"), 
Lkt_Not_Expr => To_Unbounded_String ("NotExpr"), 
Lkt_Paren_Expr => To_Unbounded_String ("ParenExpr"), 
Lkt_Raise_Expr => To_Unbounded_String ("RaiseExpr"), 
Lkt_Subscript_Expr => To_Unbounded_String ("SubscriptExpr"), 
Lkt_Null_Cond_Subscript_Expr => To_Unbounded_String ("NullCondSubscriptExpr"), 
Lkt_Try_Expr => To_Unbounded_String ("TryExpr"), 
Lkt_Un_Op => To_Unbounded_String ("UnOp"), 
Lkt_Full_Decl => To_Unbounded_String ("FullDecl"), 
Lkt_Grammar_List_Sep => To_Unbounded_String ("GrammarListSep"), 
Lkt_Import => To_Unbounded_String ("Import"), 
Lkt_Langkit_Root => To_Unbounded_String ("LangkitRoot"), 
Lkt_Lexer_Case_Rule => To_Unbounded_String ("LexerCaseRule"), 
Lkt_Lexer_Case_Rule_Send => To_Unbounded_String ("LexerCaseRuleSend"), 
Lkt_List_Kind_One => To_Unbounded_String ("ListKindOne"), 
Lkt_List_Kind_Zero => To_Unbounded_String ("ListKindZero"), 
Lkt_Base_Lexer_Case_Rule_Alt_List => To_Unbounded_String ("BaseLexerCaseRuleAltList"), 
Lkt_Block_String_Line_List => To_Unbounded_String ("BlockStringLineList"), 
Lkt_Call_Expr_List => To_Unbounded_String ("CallExprList"), 
Lkt_Decl_Annotation_List => To_Unbounded_String ("DeclAnnotationList"), 
Lkt_Elsif_Branch_List => To_Unbounded_String ("ElsifBranchList"), 
Lkt_Enum_Class_Alt_Decl_List => To_Unbounded_String ("EnumClassAltDeclList"), 
Lkt_Enum_Class_Case_List => To_Unbounded_String ("EnumClassCaseList"), 
Lkt_Enum_Lit_Decl_List => To_Unbounded_String ("EnumLitDeclList"), 
Lkt_Expr_List => To_Unbounded_String ("ExprList"), 
Lkt_Any_Of_List => To_Unbounded_String ("AnyOfList"), 
Lkt_Full_Decl_List => To_Unbounded_String ("FullDeclList"), 
Lkt_Decl_Block => To_Unbounded_String ("DeclBlock"), 
Lkt_Generic_Formal_Decl_List => To_Unbounded_String ("GenericFormalDeclList"), 
Lkt_Fun_Arg_Decl_List => To_Unbounded_String ("FunArgDeclList"), 
Lkt_Grammar_Expr_List => To_Unbounded_String ("GrammarExprList"), 
Lkt_Grammar_Expr_List_List => To_Unbounded_String ("GrammarExprListList"), 
Lkt_Import_List => To_Unbounded_String ("ImportList"), 
Lkt_Lambda_Arg_Decl_List => To_Unbounded_String ("LambdaArgDeclList"), 
Lkt_Lkt_Node_List => To_Unbounded_String ("LktNodeList"), 
Lkt_Block_Decl_List => To_Unbounded_String ("BlockDeclList"), 
Lkt_Match_Branch_List => To_Unbounded_String ("MatchBranchList"), 
Lkt_Param_List => To_Unbounded_String ("ParamList"), 
Lkt_Ref_Id_List => To_Unbounded_String ("RefIdList"), 
Lkt_Type_Ref_List => To_Unbounded_String ("TypeRefList"), 
Lkt_Isa_List => To_Unbounded_String ("IsaList"), 
Lkt_Match_Branch => To_Unbounded_String ("MatchBranch"), 
Lkt_Op_Amp => To_Unbounded_String ("OpAmp"), 
Lkt_Op_And => To_Unbounded_String ("OpAnd"), 
Lkt_Op_Div => To_Unbounded_String ("OpDiv"), 
Lkt_Op_Eq => To_Unbounded_String ("OpEq"), 
Lkt_Op_Gt => To_Unbounded_String ("OpGt"), 
Lkt_Op_Gte => To_Unbounded_String ("OpGte"), 
Lkt_Op_Lt => To_Unbounded_String ("OpLt"), 
Lkt_Op_Lte => To_Unbounded_String ("OpLte"), 
Lkt_Op_Minus => To_Unbounded_String ("OpMinus"), 
Lkt_Op_Mult => To_Unbounded_String ("OpMult"), 
Lkt_Op_Ne => To_Unbounded_String ("OpNe"), 
Lkt_Op_Or => To_Unbounded_String ("OpOr"), 
Lkt_Op_Or_Int => To_Unbounded_String ("OpOrInt"), 
Lkt_Op_Plus => To_Unbounded_String ("OpPlus"), 
Lkt_Param => To_Unbounded_String ("Param"), 
Lkt_Default_List_Type_Ref => To_Unbounded_String ("DefaultListTypeRef"), 
Lkt_Function_Type_Ref => To_Unbounded_String ("FunctionTypeRef"), 
Lkt_Generic_Type_Ref => To_Unbounded_String ("GenericTypeRef"), 
Lkt_Simple_Type_Ref => To_Unbounded_String ("SimpleTypeRef"), 
Lkt_Var_Bind => To_Unbounded_String ("VarBind"));

   ---------------
   -- Kind_Name --
   ---------------

   function Kind_Name (Node : Bare_Lkt_Node) return String is
   begin
      return To_String (Kind_Names (Node.Kind));
   end Kind_Name;

   --------------------
   -- Children_Count --
   --------------------

   function Children_Count (Node : Bare_Lkt_Node) return Natural is
      C : Integer := Kind_To_Node_Children_Count (Node.Kind);
   begin
      if C = -1 then
         return Node.Count;
      else
         return C;
      end if;
   end Children_Count;

   ----------------------
   -- Free_User_Fields --
   ----------------------

   procedure Free_User_Fields (Node : Bare_Lkt_Node) is

      procedure Reset_Logic_Var (LV : in out Logic_Var_Record);
      --  Reset the LV logic variable, clearing the value it stores

      ---------------------
      -- Reset_Logic_Var --
      ---------------------

      procedure Reset_Logic_Var (LV : in out Logic_Var_Record) is
      begin
         LV.Value := No_Entity;
         Entity_Vars.Reset (LV'Unrestricted_Access);
         Entity_Vars.Destroy (LV);
      end Reset_Logic_Var;

      K : constant Lkt_Node_Kind_Type := Node.Kind;

   begin
      
      case Lkt_Lkt_Node (K) is
when Lkt_Function_Type_Range =>
declare
N_Bare_Function_Type : constant Bare_Function_Type := Node;
begin
Dec_Ref (N_Bare_Function_Type.Function_Type_Args);
end;
when Lkt_Instantiated_Generic_Type_Range =>
declare
N_Bare_Instantiated_Generic_Type : constant Bare_Instantiated_Generic_Type := Node;
begin
Dec_Ref (N_Bare_Instantiated_Generic_Type.Instantiated_Generic_Type_Actuals);
end;
when others => null;
end case;
   end Free_User_Fields;

   ----------------
   -- Token_Data --
   ----------------

   function Token_Data (Unit : Internal_Unit) return Token_Data_Handler_Access
   is (Unit.TDH'Access);

   -------------------
   -- Lookup_Symbol --
   -------------------

   function Lookup_Symbol
     (Context : Internal_Context; Symbol : Text_Type) return Symbol_Type
   is
      Canon_Symbol : constant Symbolization_Result :=
            Create_Symbol (Symbol)
      ;
   begin
      if Canon_Symbol.Success then
         return To_Symbol
           (Context.Symbols, Find (Context.Symbols, Canon_Symbol.Symbol));
      else
         raise Invalid_Symbol_Error with Image (Canon_Symbol.Error_Message);
      end if;
   end Lookup_Symbol;

   -------------------------
   -- Create_Special_Unit --
   -------------------------

   function Create_Special_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      Unit : Internal_Unit := new Analysis_Unit_Type'
        (Context                      => Context,
         Is_Internal                  => False,
         Ast_Root                     => null,
         Filename                     => Normalized_Filename,
         Charset                      => To_Unbounded_String (Charset),
         TDH                          => <>,
         Diagnostics                  => <>,
         Rule                         => Rule,
         Ast_Mem_Pool                 => No_Pool,
         Destroyables                 => Destroyable_Vectors.Empty_Vector,
         Referenced_Units             => <>,
         Exiled_Entries               => Exiled_Entry_Vectors.Empty_Vector,
         Foreign_Nodes                =>
            Foreign_Node_Entry_Vectors.Empty_Vector,
         Exiled_Entries_In_NED        =>
            Exiled_Entry_In_NED_Vectors.Empty_Vector,
         Exiled_Envs                  => Exiled_Env_Vectors.Empty_Vector,
         Named_Envs                   => Named_Env_Vectors.Empty_Vector,
         Nodes_With_Foreign_Env       => <>,
         Rebindings                   => Env_Rebindings_Vectors.Empty_Vector,
         Cache_Version                => <>,
         Unit_Version                 => <>,
         Memoization_Map            => <>,
         others => <>
      );
   begin
      Initialize
        (Unit.TDH, Context.Symbols, Unit.all'Address, Context.Tab_Stop);
      return Unit;
   end Create_Special_Unit;

   --------------------
   -- Templates_Unit --
   --------------------

   function Templates_Unit (Context : Internal_Context) return Internal_Unit is
   begin
      if Context.Templates_Unit = No_Analysis_Unit then
         Context.Templates_Unit := Create_Special_Unit
           (Context             => Context,
            Normalized_Filename => No_File,
            Charset             => Default_Charset,
            Rule                => Main_Rule_Rule);
      end if;
      return Context.Templates_Unit;
   end Templates_Unit;

   --------------
   -- Set_Rule --
   --------------

   procedure Set_Rule (Unit : Internal_Unit; Rule : Grammar_Rule) is
   begin
      Unit.Rule := Rule;
   end Set_Rule;

   ------------------------------
   -- Normalized_Unit_Filename --
   ------------------------------

   function Normalized_Unit_Filename
     (Context : Internal_Context; Filename : String) return Virtual_File
   is
      use Virtual_File_Maps;
      Key : constant Unbounded_String := To_Unbounded_String (Filename);
      Cur : Cursor := Context.Filenames.Find (Key);
   begin
      if Cur = No_Element then
         declare
            F : constant Virtual_File := Create
              (Create_From_Base (+Filename).Full_Name,
               Normalize => True);
         begin
            Context.Filenames.Insert (Key, F);
            return F;
         end;
      else
         return Element (Cur);
      end if;
   end Normalized_Unit_Filename;

   --------------------------
   -- Register_Destroyable --
   --------------------------

   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure)
   is
   begin
      Destroyable_Vectors.Append (Unit.Destroyables, (Object, Destroy));
   end Register_Destroyable_Helper;

   --------------------------
   -- Register_Destroyable --
   --------------------------

   procedure Register_Destroyable
     (Unit : Internal_Unit; Node : Bare_Lkt_Node)
   is
      procedure Helper is new Register_Destroyable_Gen
        (Root_Node_Record,
         Bare_Lkt_Node,
         Destroy_Synthetic_Node);
   begin
      Helper (Unit, Node);
   end Register_Destroyable;

   --------------------------
   -- Register_Destroyable --
   --------------------------

   procedure Register_Destroyable
     (Unit : Internal_Unit; Env : AST_Envs.Lexical_Env_Access)
   is
      procedure Helper is new Register_Destroyable_Gen
        (AST_Envs.Lexical_Env_Record, AST_Envs.Lexical_Env_Access, Destroy);
   begin
      Helper (Unit, Env);
   end Register_Destroyable;

   -----------------------
   -- Invalidate_Caches --
   -----------------------

   procedure Invalidate_Caches
     (Context : Internal_Context; Invalidate_Envs : Boolean) is
   begin
      --  Increase Context's version number. If we are about to overflow, reset
      --  all version numbers from analysis units.
      if Context.Cache_Version = Version_Number'Last then
         Context.Cache_Version := 1;
         for Unit of Context.Units loop
            Unit.Cache_Version := 0;
         end loop;
      else
         Context.Cache_Version := Context.Cache_Version + 1;
      end if;

      if Invalidate_Envs then
         Context.Reparse_Cache_Version := Context.Cache_Version;
      end if;
   end Invalidate_Caches;

   ------------------
   --  Reset_Envs  --
   ------------------

   procedure Reset_Envs (Unit : Internal_Unit) is

      procedure Deactivate_Refd_Envs (Node : Bare_Lkt_Node);
      procedure Recompute_Refd_Envs (Node : Bare_Lkt_Node);

      --------------------------
      -- Deactivate_Refd_Envs --
      --------------------------

      procedure Deactivate_Refd_Envs (Node : Bare_Lkt_Node) is
      begin
         if Node = null then
            return;
         end if;

         Deactivate_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Deactivate_Refd_Envs (Child (Node, I));
         end loop;
      end Deactivate_Refd_Envs;

      -------------------------
      -- Recompute_Refd_Envs --
      -------------------------

      procedure Recompute_Refd_Envs (Node : Bare_Lkt_Node) is
      begin
         if Node = null then
            return;
         end if;
         Recompute_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Recompute_Refd_Envs (Child (Node, I));
         end loop;
      end Recompute_Refd_Envs;

   begin
      --  First pass will deactivate every referenced envs that Unit possesses
      Deactivate_Refd_Envs (Unit.Ast_Root);

      --  Second pass will recompute the env they are pointing to
      Recompute_Refd_Envs (Unit.Ast_Root);
   end Reset_Envs;

   -------------
   -- Destroy --
   -------------

   procedure Destroy (Reparsed : in out Reparsed_Unit) is
   begin
      Free (Reparsed.TDH);
      Reparsed.Diagnostics := Diagnostics_Vectors.Empty_Vector;
      Free (Reparsed.Ast_Mem_Pool);
      Reparsed.Ast_Root := null;
   end Destroy;

   --------------
   -- Basename --
   --------------

   function Basename (Filename : String) return String is
   begin
      return +Create (+Filename).Base_Name;
   end Basename;

   --------------
   -- Basename --
   --------------

   function Basename (Unit : Internal_Unit) return String is
   begin
      return +Unit.Filename.Base_Name;
   end Basename;

   ------------------
   -- Reset_Caches --
   ------------------

   procedure Reset_Caches (Unit : Internal_Unit) is
      Cache_Version : constant Version_Number := Unit.Cache_Version;
   begin
      if Cache_Version < Unit.Context.Reparse_Cache_Version then
         Unit.Cache_Version := Unit.Context.Reparse_Cache_Version;
         Reset_Envs (Unit);
      end if;

      if Cache_Version < Unit.Context.Cache_Version then
         Unit.Cache_Version := Unit.Context.Cache_Version;
            Destroy (Unit.Memoization_Map);
      end if;
   end Reset_Caches;

   --------------------
   -- Reference_Unit --
   --------------------

   procedure Reference_Unit (From, Referenced : Internal_Unit) is
      Dummy : Boolean;
   begin
      Dummy := Analysis_Unit_Sets.Add (From.Referenced_Units, Referenced);
   end Reference_Unit;

   ------------------------
   -- Is_Referenced_From --
   ------------------------

   function Is_Referenced_From
     (Self, Unit : Internal_Unit) return Boolean is
   begin
      if Unit = null or else Self = null then
         return False;
      elsif Unit = Self then
         return True;
      else
         return Analysis_Unit_Sets.Has (Unit.Referenced_Units, Self);
      end if;
   end Is_Referenced_From;

   ----------------
   -- Do_Parsing --
   ----------------

   procedure Do_Parsing
     (Unit   : Internal_Unit;
      Input  : Internal_Lexer_Input;
      Result : out Reparsed_Unit)
   is
      Context  : constant Internal_Context := Unit.Context;
      Unit_TDH : constant Token_Data_Handler_Access := Token_Data (Unit);

      Saved_TDH : Token_Data_Handler;
      --  Holder to save tokens data in Unit.
      --
      --  By design, parsing is required to bind the nodes it creates to an
      --  analysis unit. However, this procedure is supposed to preserve the
      --  Unit itself and return its parsing result in Result.
      --
      --  In order to implement this, we first move "old" token data in this
      --  variable, then we do parsing. Only then, we can move "new" token data
      --  from the unit to Result, and restore the "old" token data to Unit.
      --  This last step is what Rotate_TDH (see below) is above.

      procedure Rotate_TDH;
      --  Move token data from Unit to Result and restore data in Saved_TDH to
      --  Unit.

      ----------------
      -- Rotate_TDH --
      ----------------

      procedure Rotate_TDH is
      begin
         Move (Result.TDH, Unit_TDH.all);
         Move (Unit_TDH.all, Saved_TDH);
      end Rotate_TDH;

   begin
      GNATCOLL.Traces.Trace (Main_Trace, "Parsing unit " & Basename (Unit));

      Result.Ast_Root := null;

      Move (Saved_TDH, Unit_TDH.all);
      Initialize (Unit_TDH.all,
                  Saved_TDH.Symbols,
                  Unit.all'Address,
                  Unit.Context.Tab_Stop);

      --  This is where lexing occurs, so this is where we get most "setup"
      --  issues: missing input file, bad charset, etc. If we have such an
      --  error, catch it, turn it into diagnostics and abort parsing.
      --
      --  As it is quite common, first check if the file is readable: if not,
      --  don't bother opening it and directly emit a diagnostic. This avoid
      --  pointless exceptions which harm debugging. Note that this
      --  optimization is valid only when there is no file reader, which can
      --  work even when there is no real source file.

      if Context.File_Reader = null
         and then Input.Kind = File
         and then (Input.Filename.Is_Directory
                   or else (not Input.Filename.Is_Readable))
      then
         declare
            Name : constant String := Basename (Unit);
         begin
            GNATCOLL.Traces.Trace
              (Main_Trace, "WARNING: File is not readable: " & Name);
            Append
              (Result.Diagnostics,
               No_Source_Location_Range,
               "Cannot read " & To_Text (Name));
            Rotate_TDH;
            return;
         end;
      end if;

      --  Initialize the parser, which fetches the source buffer and extract
      --  all tokens.

      Init_Parser
        (Input, Context.With_Trivia, Unit, Unit_TDH, Unit.Context.Parser);

      --  If we could run the lexer, run the parser and get the root node

      if Unit_TDH.Source_Buffer /= null then
         Result.Ast_Mem_Pool := Create;
         Unit.Context.Parser.Mem_Pool := Result.Ast_Mem_Pool;
         Result.Ast_Root := Bare_Lkt_Node
           (Parse (Unit.Context.Parser, Rule => Unit.Rule));
      end if;

      --  Forward token data and diagnostics to the returned unit

      Rotate_TDH;
      Result.Diagnostics.Append_Vector (Unit.Context.Parser.Diagnostics);
   end Do_Parsing;

   --------------------------
   -- Update_After_Reparse --
   --------------------------

   procedure Update_After_Reparse
     (Unit : Internal_Unit; Reparsed : in out Reparsed_Unit) is
   begin
      --  Remove the `symbol -> AST node` associations for Unit's nodes in
      --  foreign lexical environments.
      Remove_Exiled_Entries (Unit);

      --  Remove the named envs that Unit created
      declare
         Named_Envs_Needing_Update : NED_Maps.Map;
      begin
         Remove_Named_Envs (Unit, Named_Envs_Needing_Update);
         Update_Named_Envs (Unit.Context, Named_Envs_Needing_Update);
      end;

      --  Explicitly clear the env caches of this unit while it is still fully
      --  alive to make sure that ``Lexical_Env_Cache_Updated`` accesses valid
      --  data. Otherwise the env caches end up being cleared during the call
      --  to ``Destroy_Unit_Destroyables`` where the unit is already partially
      --  destroyed.
      Reset_Envs_Caches (Unit);

      --  At this point, envs and nodes that don't belong to this unit no
      --  longer reference this unit's envs and nodes. It is thus now safe to
      --  deallocate this unit's obsolete data.

      --  Replace Unit's diagnostics by Reparsed's
      Unit.Diagnostics := Reparsed.Diagnostics;
      Reparsed.Diagnostics.Clear;

      --  As (re-)loading a unit can change how any AST node property in the
      --  whole analysis context behaves, we have to invalidate caches. This
      --  is likely overkill, but kill all caches here as it's easy to do.
      --
      --  As an optimization, invalidate referenced envs cache only if this is
      --  not the first time we parse Unit.
      Invalidate_Caches
        (Unit.Context, Invalidate_Envs => Unit.Ast_Root /= null);

      --  Likewise for token data
      Free (Unit.TDH);
      Move (Unit.TDH, Reparsed.TDH);

      --  Reparsing will invalidate all lexical environments related to this
      --  unit, so destroy all related rebindings as well. This browses AST
      --  nodes, so we have to do this before destroying the old AST nodes
      --  pool.
      Destroy_Rebindings (Unit.Rebindings'Access);

      --  Clear the set of units referenced from that one, as it may no longer
      --  hold in the reparsed unit.
      Analysis_Unit_Sets.Destroy (Unit.Referenced_Units);

      --  Destroy the old AST node and replace it by the new one
      if Unit.Ast_Root /= null then
         Destroy (Unit.Ast_Root);
      end if;
      Unit.Ast_Root := Reparsed.Ast_Root;

      --  Likewise for memory pools
      Free (Unit.Ast_Mem_Pool);
      Unit.Ast_Mem_Pool := Reparsed.Ast_Mem_Pool;
      Reparsed.Ast_Mem_Pool := No_Pool;

      --  Increment unit version number to invalidate caches and stale node
      --  reference. Also propagate it to the TDH.
      Unit.Unit_Version := Unit.Unit_Version + 1;
      Unit.TDH.Version := Unit.Unit_Version;

      --  Compute the PLE_Roots_Starting_Token table

      Unit.PLE_Roots_Starting_Token.Clear;

      --  Update all the lexical envs entries affected by the reparse

      declare
         Unit_Name     : constant String := +Unit.Filename.Base_Name;
         Context       : constant Internal_Context := Unit.Context;
         Foreign_Nodes : Bare_Lkt_Node_Vectors.Vector :=
           Bare_Lkt_Node_Vectors.Empty_Vector;

         Saved_In_Populate_Lexical_Env : constant Boolean :=
           Context.In_Populate_Lexical_Env;
         Saved_Env_Populated_Roots     : constant Boolean_Vectors.Vector :=
           Unit.Env_Populated_Roots;
      begin
         Context.In_Populate_Lexical_Env := True;
         if Main_Trace.Active then
            Main_Trace.Trace
              ("Updating lexical envs for " & Unit_Name & " after reparse");
            Main_Trace.Increase_Indent;
         end if;

         --  Collect all nodes that are foreign in this Unit's lexical envs.
         --  Exclude them from the corresponding lists of exiled entries.
         Extract_Foreign_Nodes (Unit, Foreign_Nodes);

         --  Temporarily reset Env_Populated_Roots so that Populate_Lexical_Env
         --  accepts to do its work on reparsed trees.

         Unit.Env_Populated_Roots := Boolean_Vectors.Empty_Vector;

         --  Now that Unit has been reparsed, we can destroy all its
         --  destroyables, which refer to the old tree (i.e. dangling
         --  pointers).
         Destroy_Unit_Destroyables (Unit);

         for FN of Foreign_Nodes loop
            declare
               Node_Image : constant String := Image (Short_Text_Image (FN));
               Unit_Name  : constant String := +FN.Unit.Filename.Base_Name;
            begin
               GNATCOLL.Traces.Trace
                 (Main_Trace, "Rerooting: " & Node_Image
                              & " (from " & Unit_Name & ")");
            end;
            Reroot_Foreign_Node (FN);
         end loop;
         Foreign_Nodes.Destroy;

         --  Re-populate all PLE roots that were requested so far for this
         --  unit. In the case where the unit has no PLE root, run PLE on the
         --  whole unit iff it was requested on at least one PLE root.

         declare
            function At_Least_One_Root_Populated return Boolean
            is (for some B of Saved_Env_Populated_Roots => B);
         begin
               if At_Least_One_Root_Populated then
                  Populate_Lexical_Env (Unit);
               end if;
         end;

         --  Restore the unit's original Env_Populated_Roots flags

         Unit.Env_Populated_Roots.Destroy;
         Unit.Env_Populated_Roots := Saved_Env_Populated_Roots;

         Context.In_Populate_Lexical_Env := Saved_In_Populate_Lexical_Env;
         if Main_Trace.Is_Active then
            Main_Trace.Decrease_Indent;
         end if;
      end;
   end Update_After_Reparse;

   -------------------------------
   -- Destroy_Unit_Destroyables --
   -------------------------------

   procedure Destroy_Unit_Destroyables (Unit : Internal_Unit) is
   begin
      for D of Unit.Destroyables loop
         D.Destroy (D.Object);
      end loop;
      Destroyable_Vectors.Clear (Unit.Destroyables);
   end Destroy_Unit_Destroyables;

   ---------------------------
   -- Remove_Exiled_Entries --
   ---------------------------

   procedure Remove_Exiled_Entries (Unit : Internal_Unit) is
   begin
      for EE of Unit.Exiled_Entries loop
         AST_Envs.Remove (EE.Env, Thin (EE.Key), EE.Node);

         --  Also strip foreign nodes information from "outer" units so that it
         --  does not contain stale information (i.e. dangling pointers to
         --  nodes that belong to the units in the queue).
         if EE.Env.Owner /= No_Generic_Unit then
            declare
               Foreign_Nodes : Foreign_Node_Entry_Vectors.Vector renames
                  Convert_Unit (EE.Env.Owner).Foreign_Nodes;
               Current       : Positive := Foreign_Nodes.First_Index;
            begin
               while Current <= Foreign_Nodes.Last_Index loop
                  if Foreign_Nodes.Get (Current).Node = EE.Node then
                     Foreign_Nodes.Pop (Current);
                  else
                     Current := Current + 1;
                  end if;
               end loop;
            end;
         end if;
      end loop;

      Unit.Exiled_Entries.Clear;
   end Remove_Exiled_Entries;

   -----------------------
   -- Remove_Named_Envs --
   -----------------------

   procedure Remove_Named_Envs
     (Unit                      : Internal_Unit;
      Named_Envs_Needing_Update : in out NED_Maps.Map) is
   begin
      --  Remove nodes in this unit from the Named_Env_Descriptor.Foreign_Nodes
      --  components in which they are registered and from the foreign
      --  environments themselves.
      for EE of Unit.Exiled_Entries_In_NED loop
         Remove (EE.Named_Env.Foreign_Nodes, EE.Key, EE.Node);
         Remove (EE.Named_Env.Env_With_Precedence, Thin (EE.Key), EE.Node);
      end loop;
      Unit.Exiled_Entries_In_NED.Clear;

      --  Remove nodes in this unit from the
      --  Named_Env_Descriptor.Nodes_With_Foreign_Env components in which they
      --  are registered.
      for Cur in Unit.Nodes_With_Foreign_Env.Iterate loop
         declare
            use Node_To_Named_Env_Maps;
            Node : constant Bare_Lkt_Node := Key (Cur);
            NE   : constant Named_Env_Descriptor_Access := Element (Cur);
         begin
            NE.Nodes_With_Foreign_Env.Delete (Node);
         end;
      end loop;
      Unit.Nodes_With_Foreign_Env.Clear;

      --  Remove ends in this unit from the Named_Env_Descriptor.Foreign_Envs
      --  components in which they are registered.
      for EE of Unit.Exiled_Envs loop
         EE.Named_Env.Foreign_Envs.Delete (Env_Node (EE.Env));
      end loop;
      Unit.Exiled_Envs.Clear;

      --  Remove named envs that this unit created
      for NE of Unit.Named_Envs loop
         declare
            NED_Access : constant Named_Env_Descriptor_Access :=
               Unit.Context.Named_Envs.Element (NE.Name);
            NED        : Named_Env_Descriptor renames NED_Access.all;
         begin
            NED.Envs.Delete (Env_Node (NE.Env));

            --  If this named environment had precedence, we must schedule an
            --  update for this name environment entry.
            if NE.Env = NED.Env_With_Precedence then
               Named_Envs_Needing_Update.Include (NE.Name, NED_Access);
               NED.Env_With_Precedence := Empty_Env;
            end if;
         end;
      end loop;
      Unit.Named_Envs.Clear;
   end Remove_Named_Envs;

   ---------------------------
   -- Extract_Foreign_Nodes --
   ---------------------------

   procedure Extract_Foreign_Nodes
     (Unit          : Internal_Unit;
      Foreign_Nodes : in out Bare_Lkt_Node_Vectors.Vector) is
   begin
      --  Go through all foreign nodes registered in Unit's lexical
      --  environments.
      for FN of Unit.Foreign_Nodes loop
         --  Collect them
         Foreign_Nodes.Append (FN.Node);

         --  For each foreign node, remove the corresponding exiled entry in
         --  that foreign unit (each foreign node in unit A has a corresponding
         --  exiled entry in unit B).
         declare
            Exiled_Entries : Exiled_Entry_Vectors.Vector renames
               FN.Unit.Exiled_Entries;
            Current        : Positive := Exiled_Entries.First_Index;
         begin
            while Current <= Exiled_Entries.Last_Index loop
               if Exiled_Entries.Get (Current).Node = FN.Node then
                  Exiled_Entries.Pop (Current);
               else
                  Current := Current + 1;
               end if;
            end loop;
         end;
      end loop;
      Unit.Foreign_Nodes.Clear;
   end Extract_Foreign_Nodes;

   --------------------------
   -- Reroot_Foreign_Nodes --
   --------------------------

   procedure Reroot_Foreign_Node (Node : Bare_Lkt_Node) is
      Unit : constant Internal_Unit := Node.Unit;
   begin
      --  First, filter the exiled entries in foreign units so that they don't
      --  contain references to this unit's lexical environments.  We need to
      --  do that before running the partial Populate_Lexical_Env pass so that
      --  we don't remove exiled entries that this pass will produce.
      declare
         Exiled_Entries : Exiled_Entry_Vectors.Vector renames
            Unit.Exiled_Entries;
         Current        : Positive := Exiled_Entries.First_Index;
      begin
         while Current <= Exiled_Entries.Last_Index loop
            if Exiled_Entries.Get (Current).Node = Node then
               Exiled_Entries.Pop (Current);
            else
               Current := Current + 1;
            end if;
         end loop;
      end;

      --  Re-do a partial Populate_Lexical_Env pass for each foreign node that
      --  this unit contains so that they are relocated in our new lexical
      --  environments.
      declare
         Unit_State : aliased PLE_Unit_State :=
           (Named_Envs_Needing_Update => <>);
         State      : PLE_Node_State :=
           (Unit_State  => Unit_State'Unchecked_Access,
            Current_Env => Node.Self_Env,
            Current_NED => null);
      begin
         Pre_Env_Actions (Node, State, Add_To_Env_Only => True);
         Post_Env_Actions (Node, State);
      end;
   end Reroot_Foreign_Node;

   ----------
   -- Text --
   ----------

   function Text (Node : Bare_Lkt_Node) return String_Type is
   begin
      return Create_String (Text (Node));
   end Text;

   ------------------------
   -- Destroy_Rebindings --
   ------------------------

   procedure Destroy_Rebindings
     (Rebindings : access Env_Rebindings_Vectors.Vector)
   is
      procedure Recurse (R : in out Env_Rebindings);
      --  Destroy R's children and then destroy R. It is up to the caller to
      --  remove R from its parent's Children vector.

      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector);
      --  Remove R from Rebindings

      -------------
      -- Recurse --
      -------------

      procedure Recurse (R : in out Env_Rebindings) is
      begin
         for C of R.Children loop
            declare
               C_Var : Env_Rebindings := C;
            begin
               Recurse (C_Var);
            end;
         end loop;
         R.Children.Destroy;

         Unregister (R, Convert_Unit (R.Old_Env.Owner).Rebindings);
         Unregister (R, Convert_Unit (R.New_Env.Owner).Rebindings);

         Release_Rebinding (R);
      end Recurse;

      ----------------
      -- Unregister --
      ----------------

      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector) is
      begin
         for I in 1 .. Rebindings.Length loop
            if Rebindings.Get (I) = R then
               Rebindings.Pop (I);
               return;
            end if;
         end loop;

         --  We are always supposed to find R in Rebindings, so this should be
         --  unreachable.
         raise Program_Error;
      end Unregister;

   begin
      while Rebindings.Length > 0 loop
         declare
            R : Env_Rebindings := Rebindings.Get (1);
         begin
            --  Here, we basically undo what has been done in AST_Envs.Append

            --  If this rebinding has no parent, then during its creation we
            --  registered it in its Old_Env. Otherwise, it is registered
            --  in its Parent's Children list.
            if R.Parent = null then
               Unwrap (R.Old_Env).Rebindings_Pool.Delete (R.New_Env);
            else
               Unregister (R, R.Parent.Children);
            end if;

            --  In all cases it's registered in Old_Env's and New_Env's units
            Recurse (R);
         end;
      end loop;
   end Destroy_Rebindings;

   --------------------------
   -- Get_Rewriting_Handle --
   --------------------------

   function Get_Rewriting_Handle
     (Context : Internal_Context) return Rewriting_Handle_Pointer is
   begin
      return Context.Rewriting_Handle;
   end Get_Rewriting_Handle;

   --------------------------
   -- Set_Rewriting_Handle --
   --------------------------

   procedure Set_Rewriting_Handle
     (Context : Internal_Context; Handle : Rewriting_Handle_Pointer) is
   begin
      Context.Rewriting_Handle := Handle;
   end Set_Rewriting_Handle;

   -----------------------
   -- Create_Safety_Net --
   -----------------------

   function Create_Safety_Net
     (Context : Internal_Context) return Iterator_Safety_Net
   is
   begin
      return (Context         => Context,
              Context_Serial  => Context.Serial_Number,
              Context_Version => Context.Cache_Version);
   end Create_Safety_Net;

   ----------------------
   -- Check_Safety_Net --
   ----------------------

   procedure Check_Safety_Net (Self : Iterator_Safety_Net) is
   begin
      if Self.Context = null then
         return;
      end if;

      --  Check that the context is still the same (not released nor reused)
      if Self.Context.Serial_Number /= Self.Context_Serial
         or else Self.Context.Cache_Version /= Self.Context_Version
      then
         raise Stale_Reference_Error;
      end if;
   end Check_Safety_Net;

   ----------------------
   -- String_To_Symbol --
   ----------------------

   function String_To_Symbol
     (Self    : Bare_Lkt_Node;
      Context : Internal_Context;
      S       : String_Type) return Symbol_Type is
   begin
      return (if S.Length > 0
              then Lookup_Symbol (Context, S.Content)
              else No_Symbol);
   exception
      when Exc : Invalid_Symbol_Error =>
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            Ada.Exceptions.Exception_Message (Exc));
   end String_To_Symbol;

   -------------
   -- Inc_Ref --
   -------------

   procedure Inc_Ref (Self : String_Type) is
   begin
      if Self.Ref_Count >= 0 then
         Self.Ref_Count := Self.Ref_Count + 1;
      end if;
   end Inc_Ref;

   -------------
   -- Dec_Ref --
   -------------

   procedure Dec_Ref (Self : in out String_Type) is
   begin
      if Self = null or else Self.Ref_Count < 0 then
         return;
      end if;

      if Self.Ref_Count = 1 then
         Free (Self);
      else
         Self.Ref_Count := Self.Ref_Count - 1;
         Self := null;
      end if;
   end Dec_Ref;

   -------------------
   -- Create_String --
   -------------------

   function Create_String (Content : Text_Type) return String_Type is
   begin
      return Result : constant String_Type := new String_Record'
        (Length    => Content'Length,
         Ref_Count => 1,
         Content   => Content);
   end Create_String;

   -------------------
   -- Create_String --
   -------------------

   function Create_String (Content : Unbounded_Text_Type) return String_Type is
      S : Big_Wide_Wide_String_Access;
      L : Natural;
   begin
      Get_Wide_Wide_String (Content, S, L);
      return Create_String (S.all (1 .. L));
   end Create_String;

   -------------------
   -- Concat_String --
   -------------------

   function Concat_String (Left, Right : String_Type) return String_Type is
   begin
      return Result : constant String_Type :=
        new String_Record (Length => Left.Length + Right.Length)
      do
         Result.Ref_Count := 1;
         Result.Content (1 .. Left.Length) := Left.Content;
         Result.Content (Left.Length + 1 .. Result.Length) := Right.Content;
      end return;
   end Concat_String;

   ----------------
   -- Equivalent --
   ----------------

   function Equivalent (Left, Right : String_Type) return Boolean is
   begin
      return Left.Content = Right.Content;
   end Equivalent;

begin
   No_Big_Integer.Value.Set (0);
end Liblktlang.Implementation;
